/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/*
 * Initial version
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 */

/*
 * Routines for generic manipulation of the interrupts found on the 
 * INCAIP boards.
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/kernel_stat.h>

#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#include <asm/traps.h>
#if 0
#include <asm/gdb-stub.h>
#endif

#include <asm/incaip/inca-ip.h>
#include <linux/tqueue.h>

extern unsigned int incaip_get_fpi_hz(void);
extern asmlinkage void mipsIRQ(void);
extern asmlinkage unsigned int do_IRQ(int irq, struct pt_regs *regs);

#if 0
#define DEBUG_INT(x...) printk(x)
#else
#define DEBUG_INT(x...)
#endif

#ifdef CONFIG_KGDB
extern void breakpoint(void);
extern void set_debug_traps(void);
extern int remote_debug;
#endif

void disable_incaip_irq(unsigned int irq_nr)
{
	unsigned int flags=0;
	
	save_and_cli(flags);
	DEBUG_INT("disable_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*INCA_IP_ICU_IM0_IER &= ~INCA_IP_ICU_IM0_IER_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*INCA_IP_ICU_IM1_IER &= ~INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*INCA_IP_ICU_IM2_IER &= ~INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
	}
	restore_flags(flags);
}

void mask_and_ack_incaip_irq(unsigned int irq_nr)
{
	unsigned int flags=0;
	
	save_and_cli(flags);
	DEBUG_INT("mask_and_ack_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		/* mask */
		*INCA_IP_ICU_IM0_IER &= ~INCA_IP_ICU_IM0_IER_IR(irq_nr);
		/* ack */
		*INCA_IP_ICU_IM0_ISR = INCA_IP_ICU_IM0_ISR_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		/* mask */
		*INCA_IP_ICU_IM1_IER &= ~INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
		/* ack */
		*INCA_IP_ICU_IM1_ISR = INCA_IP_ICU_IM1_ISR_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		/* mask */
		*INCA_IP_ICU_IM2_IER &= ~INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
		/* ack */
		*INCA_IP_ICU_IM2_ISR = INCA_IP_ICU_IM2_ISR_IR(irq_nr - 64);
	}
	restore_flags(flags);
}

void enable_incaip_irq(unsigned int irq_nr)
{
	unsigned int flags=0;
	
	save_and_cli(flags);
	DEBUG_INT("enable_incaip_irq: %d ", irq_nr);
	/* have to access the correct register here */
	if (irq_nr <= INT_NUM_IM0_IRL31 && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
		*INCA_IP_ICU_IM0_IER |= INCA_IP_ICU_IM0_IER_IR(irq_nr);
	}
	if (irq_nr <= INT_NUM_IM1_IRL31 && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
		*INCA_IP_ICU_IM1_IER |= INCA_IP_ICU_IM1_IER_IR(irq_nr - 32);
	}
	if (irq_nr <= INT_NUM_IM2_IRL31 && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
		*INCA_IP_ICU_IM2_IER |= INCA_IP_ICU_IM2_IER_IR(irq_nr - 64);
	}
	restore_flags(flags);
}

static unsigned int startup_incaip_irq(unsigned int irq)
{
	enable_incaip_irq(irq);
	return 0; /* never anything pending */
}

#define shutdown_incaip_irq	disable_incaip_irq

static void end_incaip_irq(unsigned int irq)
{
	if (!(irq_desc[irq].status & (IRQ_DISABLED|IRQ_INPROGRESS)))
		enable_incaip_irq(irq);
}

static struct hw_interrupt_type incaip_irq_type = {
	"INCAIP",
	startup_incaip_irq,
	shutdown_incaip_irq,
	enable_incaip_irq,
	disable_incaip_irq,
	mask_and_ack_incaip_irq,
	end_incaip_irq,
	NULL
};

/* find the least signifcant bit set in x */
static inline int ls1bit32(unsigned int x)
{
	int b = 31, s;

	s = 16; if (x << 16 == 0) s = 0; b -= s; x <<= s;
	s =  8; if (x <<  8 == 0) s = 0; b -= s; x <<= s;
	s =  4; if (x <<  4 == 0) s = 0; b -= s; x <<= s;
	s =  2; if (x <<  2 == 0) s = 0; b -= s; x <<= s;
	s =  1; if (x <<  1 == 0) s = 0; b -= s;

	return b;
}

/* Cascaded interrupts from IM0 */
void incaip_hw0_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;
	unsigned int flags=0;

	int_status = *INCA_IP_ICU_IM0_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0) {
		return;
	}

	irq = ls1bit32(int_status);

	DEBUG_INT("incaip_hw0_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw0 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw0 ");
	do_IRQ(irq, regs);
#endif

	return;
}

/* Cascaded interrupts from IM1 */
void incaip_hw1_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;
	unsigned int flags=0;

	int_status = *INCA_IP_ICU_IM1_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0) {
		return;
	}

	irq = ls1bit32(int_status) + 32;

	DEBUG_INT("incaip_hw1_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw1 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw1 ");
	do_IRQ(irq, regs);
#endif

	return;
}

/* Cascaded interrupts from IM2 */
void incaip_hw2_irqdispatch(struct pt_regs *regs)
{
	struct irqaction *action;
	unsigned long int_status;
	int irq;
	unsigned int flags=0;

	int_status = *INCA_IP_ICU_IM2_IOSR;

	/* if int_status == 0, then the interrupt has already been cleared */
	if (int_status == 0) {
		return;
	}
	
	irq = ls1bit32(int_status) + 64;

	DEBUG_INT("incaip_hw2_irqdispatch: irq=%d\n", irq);

#if 0
	action = irq_desc[irq].action;
	/* if action == NULL, then we don't have a handler for the irq */
	if (action == NULL) {
		printk("No handler for hw2 irq: %i\n", irq);
		atomic_inc(&irq_err_count);
		return;
	}

	irq_enter();
	kstat_cpu(0).irqs[irq]++;
	action->handler(irq, action->dev_id, regs);
	irq_exit();
#else
	DEBUG_INT("hw2 ");
	do_IRQ(irq, regs);
#endif

	return;
}

int inca_be_handler(struct pt_regs *regs, int is_fixup)
{
	volatile u32 reg1,reg2,reg3,reg4,reg5;

	reg1 = *(INCA_IP_BCU_BCU_ECON);       
	reg2 = *(INCA_IP_BCU_BCU_EADD);       
	reg3 = *(INCA_IP_BCU_BCU_EDAT);       
	reg4 = *(INCA_IP_SDRAM_MC_ERRADDR);       
	reg5 = *(INCA_IP_SDRAM_MC_ERRCAUSE);       
	
	if (reg2 == 0x00000000) {
		if (reg4 == 0x00000000) return MIPS_BE_DISCARD;
	}
	
	printk(KERN_ALERT "BCU_ECON=0x%08lx ",reg1); 
	printk("BCU_EADD=0x%08lx ",reg2); 
	printk("BCU_EDAT=0x%08lx\n",reg3);
	printk(KERN_ALERT "SDRAM_ECAUSE=0x%08lx ",reg5);
	printk("SDRAM_EADDR=0x%08lx\n",reg4);

	return MIPS_BE_FATAL;
}


/* Function for careful CP0 interrupt mask access */

void __init init_IRQ(void)
{
	int i;

	DEBUG_INT("init_IRQ ");

	board_be_handler = &inca_be_handler;
	
	init_generic_irq();

	/* mask all interrupt sources */
	*INCA_IP_ICU_IM0_IER = 0;
	*INCA_IP_ICU_IM1_IER = 0;
	*INCA_IP_ICU_IM2_IER = 0;

	/* Now safe to set the exception vector. */
	set_except_vector(0, mipsIRQ);

	for (i = 0; i <= INT_NUM_IM2_IRL31; i++) {
		irq_desc[i].status	= IRQ_DISABLED;
		irq_desc[i].action	= 0;
		irq_desc[i].depth	= 1;
		irq_desc[i].handler	= &incaip_irq_type;
	}
	/* VERY important! */
#ifdef COUNTER_WORKS
	set_c0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5);
#else
	clear_c0_status(IE_IRQ5);
	set_c0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4);
#endif
#ifdef CONFIG_KGDB
	if (remote_debug) {
		set_debug_traps();
		breakpoint();
	}
#endif
}

u32 inca_wdt_getpw(void)
{
	u32 password;

	/***************************************************************************
	0 	Current state of the ENDINIT bit, WDT_CON0.ENDINIT
	[15:8] 	Current value of user-definable password field, WDT_CON0.WDTPW
	[31:16] Current value of user-definable reload value, WDT_CON0.WDTREL

	In our case this amounts to the entire contents of WDT_CON0 with the WDTLCK
	bit cleared.
	*****************************************************************************/

    	/* val &= ~2; */
	password = *(INCA_IP_WDT_WDT_CON0) & (~INCA_IP_WDT_WDT_CON0_WDTLCK);

	/***************************************************************************
		bits [7:4] 	Fixed; must be written to 1111B
	*****************************************************************************/
 	password |= 0xf0;

	/***************************************************************************
		Now get bits from WDT_CON1
		Current state of the WDT disable request bit, WDT_CON1.WDTDR
		Current state of the input frequency request bit, WDT_CON1.WDTIR
	*****************************************************************************/
 	password |= *(INCA_IP_WDT_WDT_CON1) & (INCA_IP_WDT_WDT_CON1_WDTDR | INCA_IP_WDT_WDT_CON1_WDTIR );
	return( password );
	
}

unsigned int inca_wdt_access_enable(unsigned int oldvalue)
{
	oldvalue &= ~0xff;
	oldvalue |= 0xf2;

	return( oldvalue );
}




unsigned int inca_wdt_access_disable(unsigned int oldvalue)
{
	oldvalue &= ~0xff;
	oldvalue |= 0xf3;

	return( oldvalue );
}



void inca_wdt_disable(void)
{
	u32 wdCon0Password;


  /* calculate password from current INCA_IP_WDT_WDT_CON0 value */
 	wdCon0Password = inca_wdt_getpw();
 	*INCA_IP_WDT_WDT_CON0 = wdCon0Password;

 /* enable write access INCA_IP_WDT_WDT_CON0 value */

 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_access_enable( wdCon0Password );   /* enabling ENDINIT bit */

 	*INCA_IP_WDT_WDT_CON1 |= INCA_IP_WDT_WDT_CON1_WDTDR;  /*disable the watchdog */


 /* re- calculate password from current INCA_IP_WDT_WDT_CON0 value */

 	wdCon0Password = inca_wdt_getpw();
 	*INCA_IP_WDT_WDT_CON0 = wdCon0Password;
 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_access_disable( wdCon0Password );  /* disabling ENDINIT bit */
}

void inca_wdt_enable(void)
{
	u32 wdCon0Password;


  /* calculate password from current INCA_IP_WDT_WDT_CON0 value */
 	wdCon0Password = inca_wdt_getpw();
 	*INCA_IP_WDT_WDT_CON0 = wdCon0Password;

 /* enable write access INCA_IP_WDT_WDT_CON0 value */

 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_access_enable( wdCon0Password );   /* enabling ENDINIT bit */

 	*INCA_IP_WDT_WDT_CON1 &= ~INCA_IP_WDT_WDT_CON1_WDTDR;  /*enable the watchdog */


 /* re- calculate password from current INCA_IP_WDT_WDT_CON0 value */

 	wdCon0Password = inca_wdt_getpw();
 	*INCA_IP_WDT_WDT_CON0 = wdCon0Password;
 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_access_disable( wdCon0Password );  /* disabling ENDINIT bit */
}

void inca_wdt_kick_dog(void)
{
	unsigned int flags;
	
	u32 wdCon0Value = *INCA_IP_WDT_WDT_CON0;

	save_and_cli(flags);
	/* write password */
 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_getpw();

	/* now do a modify keeping the current values */
	wdCon0Value &= ~(INCA_IP_WDT_WDT_CON1_WDTDR | INCA_IP_WDT_WDT_CON1_WDTIR );
	wdCon0Value |= (0xf0 | INCA_IP_WDT_WDT_CON0_WDTLCK | INCA_IP_WDT_WDT_CON0_ENDINIT);

	*INCA_IP_WDT_WDT_CON0	= wdCon0Value;
	restore_flags(flags);
}

int inca_wdt_set_reload_value(u32 Delay)
{

	u32 wdClockTickRate;
	u32 wdReloadValue;
	u32 wdCon0Value;

	/* calculate watchdog tickrate per second */
	if( *INCA_IP_WDT_WDT_SR & INCA_IP_WDT_WDT_SR_WDTIS )
	{
		wdClockTickRate = incaip_get_fpi_hz()/256;
	}
	else
	{
		wdClockTickRate = incaip_get_fpi_hz()/16384;
	}

	if( (wdClockTickRate*Delay) > 0xffff )
		return( -1 );

	wdReloadValue = 0xFFFF - ((wdClockTickRate*Delay)&0xffff);

	wdCon0Value = *INCA_IP_WDT_WDT_CON0 & 0xffff;	/* clear old reload value */

	wdCon0Value |= (wdReloadValue << 16);		/* add our new one */

	wdCon0Value &= ~(INCA_IP_WDT_WDT_CON1_WDTDR | INCA_IP_WDT_WDT_CON1_WDTIR );
	wdCon0Value |= (0xf0 | INCA_IP_WDT_WDT_CON0_WDTLCK | INCA_IP_WDT_WDT_CON0_ENDINIT);


 	*INCA_IP_WDT_WDT_CON0 = inca_wdt_getpw();
 	*INCA_IP_WDT_WDT_CON0 = wdCon0Value;
	return( 0 );
}

struct tq_struct inca_wdt_timer;
long inca_wdt_timer_count=0;
#ifdef CONFIG_INCAIP_WATCHDOG_TIMEOUT
u32 inca_wdt_timeout = CONFIG_INCAIP_WATCHDOG_TIMEOUT;
#else
u32 inca_wdt_timeout = 4;
#endif

void inca_wdt_timer_task(void* data)
{
	inca_wdt_timer_count++;
	if (inca_wdt_timer_count > ((inca_wdt_timeout >> 2)*HZ)) {
		inca_wdt_timer_count=0;
		inca_wdt_kick_dog();
#ifdef CONFIG_INCAIP_WATCHDOG_HEARTBEAT
		if ((*INCA_IP_TSF_LED_REG & 0x01) == 0) {
			*INCA_IP_TSF_LED_REG |= 5;
		}
		else {
			*INCA_IP_TSF_LED_REG &= ~5;
		}
#endif
			
	}
	queue_task(&inca_wdt_timer,&tq_timer);
}

void inca_wdt_start(void)
{
	u32 flags=0;
	
	printk("Enabling watchdog with timeout of %d second(s)\n",inca_wdt_timeout);
	if (inca_wdt_set_reload_value(inca_wdt_timeout) != 0) {
		printk("Error! Could not set reload value!\n");
	}
	else {
		inca_wdt_timer.routine = inca_wdt_timer_task;
		inca_wdt_timer.data = NULL;
		queue_task(&inca_wdt_timer,&tq_timer);
		inca_wdt_enable();

#ifdef CONFIG_INCAIP_WATCHDOG_HEARTBEAT
		/* enable the clock of the LED module in the Power Management Unit */
		save_flags(flags); cli();
		*INCA_IP_PMU_PM_GEN |= INCA_IP_PMU_PM_GEN_EN9;   
		*INCA_IP_TSF_TSF_CONF |= INCA_IP_TSF_TSF_CONF_LEDEN;
		restore_flags(flags);
		*INCA_IP_TSF_LEDMUX_CONF = 0;
#endif
	}
}
