/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* 
 * Initial version
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 */

/*
 * INCAIP specific setup.
 */
 
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel_stat.h>

#include <linux/interrupt.h>
#include <asm/time.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#include <asm/debug.h>

#ifdef CONFIG_ILATENCY
#include <linux/ilatency.h>
#define ILATENCY_RET_FROM_EXCEPTION() intr_ret_from_exception()
#else
#define ILATENCY_RET_FROM_EXCEPTION()
#endif

extern void incaip_reboot_setup(void);

#define COUNTER_WORKS
#define MIPS_CPU_TIMER_IRQ 7
#define ALLINTS (IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5)

static unsigned int r4k_offset; /* Amount to increment compare reg each time */
static unsigned int r4k_cur;    /* What counter should be at next timer irq */

void __init bus_error_init(void) { /* nothing */ }

#ifdef CONFIG_KGDB
extern void rs_kgdb_hook(int);
int remote_debug = 0;
//static struct async_struct kdb_port_info = {0};
#endif

/* modify inca-ip register with a small delay after writing to the register */
/* this delay is located in the same cache line as the write access, to prevent */
/* erroneous updates of the caches (seen with CGU_DIVCR register !) */
/* Input: reg = address of register */
/*        andmask = bits to clear in the register */
/*        ormask  = bits to set in the register */
static inline unsigned long modify_inca_reg(volatile unsigned int *reg, 
                                            unsigned long andmask, 
                                            unsigned long ormask)
{
    int count=15000;
    unsigned long regValue = 0;

    /* equivalent C-Code: (*reg) = ((*reg) & (~andmask)) | (ormask); */
    asm volatile (" .set noreorder\n"
                  "\tlw\t%2,(%3)\n"
                  "\tand\t%2,%2,%5\n"
                  "\tor\t%2,%2,%6\n"
                  "\t.align\t4\n" /* align on cache line boundary */
                  "\tsw\t%2,(%3)\n"
                  "\t0:\n"
                  "\tbnez\t%4,0b\n"
                  "\taddiu\t%4,%4,-1\n" /* branch delay */
                  "\t.set\treorder\n"
                  :
                  "=r" (regValue), /* %0 - output */
                  "=r" (count)  /* %1 - count is modified */
                  :
                  "0" (regValue),    /* %2 - same register as %0 */
                  "r" (reg),    /* %3 - address to modify */
                  "1" (count),  /* %4 - number of delay cycles */
                  "r" (~andmask), /* %5 - value to 'and' with on-chip register */
                  "r" (ormask)  /* %6 - value to 'or' with on-chip register */
/*                    : */
                  /* nothing clobbered */
        );

    return regValue;
}

/* the CPU clock rate - lifted from u-boot */
unsigned int
incaip_get_cpu_hz(void)
{
	/*----------------------------------------------------------------*/
	/* CPU Clock Input Multiplexer (MUX I)                            */
	/* Multiplexer MUX I selects the maximum input clock to the CPU.  */
	/*----------------------------------------------------------------*/
	if (*INCA_IP_CGU_CGU_MUXCR & INCA_IP_CGU_CGU_MUXCR_MUXI)
	{
		/* MUX I set to 150 MHz clock */
		return 150000000;
	}
	else
	{
		/* MUX I set to 100/133 MHz clock */
		if (*INCA_IP_CGU_CGU_DIVCR & 0x40) 
		{
			 /* Division value is 1/3, maximum CPU operating */
			 /* frequency is 133.3 MHz                       */
			 return 133333333;
		}
		else
		{
			 /* Division value is 1/4, maximum CPU operating */
			 /* frequency is 100 MHz                         */
			 return 100000000;
		}
	}
}

/* the FPI clock rate - lifted from u-boot */
unsigned int
incaip_get_fpi_hz(void)
{
	unsigned int  clkCPU;

	clkCPU = incaip_get_cpu_hz();

	switch (*INCA_IP_CGU_CGU_DIVCR & 0xC)
	{
		case 0x4:
			return clkCPU >> 1; /* divided by 2 */
		break;
		case 0x8:
			return clkCPU >> 2; /* divided by 4 */
		break;
		default:
			return clkCPU;
		break;
	}
}

/* get the CPU version number  - based on sysLib.c from VxWorks sources */
/* this doesn't really belong here, but it's a convenient location */
unsigned int
incaip_get_cpu_ver(void)
{
	static unsigned int cpu_ver = 0;

	if (cpu_ver == 0)
		cpu_ver = *INCA_IP_WDT_CHIPID & 0xFFFFF000;
	return cpu_ver;
}
EXPORT_SYMBOL(incaip_get_cpu_hz);
EXPORT_SYMBOL(incaip_get_fpi_hz);
EXPORT_SYMBOL(incaip_get_cpu_ver);
static void prom_printf(const char * fmt, ...);

void incaip_time_init(void)
{
	unsigned int ver;

	ver = incaip_get_cpu_ver();
	printk("INCA-IP CPU Ver. 1.%d ",(INCA_IP_WDT_CHIPID_VERSION_GET(ver) & 0x07));
	
	if (incaip_get_cpu_hz() == 100000000)
	{
		printk("100MHz\n");
		/* for CPU frequency is 100 MHz */
		r4k_offset = 100000000/HZ;
	}
	else if (incaip_get_cpu_hz() == 150000000)
	{
		printk("150MHz\n");
		/* for CPU frequency is 150 MHz */
		r4k_offset = 150000000/HZ;
	}
	else
	{
		printk("133MHz\n");
		/* for CPU frequency of 133 MHz */
		r4k_offset = 133000000/HZ;
	}

	if ((read_c0_prid() & 0xffff00) !=
		(PRID_COMP_MIPS | PRID_IMP_20KC))
		r4k_offset = r4k_offset >> 1;

	mips_hpt_frequency = r4k_offset*HZ;
}

/*
 * THe CPU timer doesn't work (yet) on INCAIP, so we have to
 * use TIMER6 in its place. Takenb from sysLib.c. Timer is set
 * to 100 Hz.
 * Also misuse this routine to print out the CPU type and clock.
 */
void incaip_timer_setup(struct irqaction *irq)
{
	volatile unsigned int *pPM_CR = INCA_IP_PMU_PM_CR;
	volatile unsigned int *pPM_GEN = INCA_IP_PMU_PM_GEN;
	volatile unsigned int *pCGU_DIVCR = INCA_IP_CGU_CGU_DIVCR;
	int retval;

	irq->handler = no_action;
	retval = setup_irq(MIPS_CPU_TIMER_IRQ, irq);
	if (retval)
		printk("setup_irq failed\n");

     write_c0_count(0);
     /* to generate the first timer interrupt */
	r4k_cur = (read_c0_count() + r4k_offset);
	write_c0_compare(r4k_cur);
	set_c0_status(ALLINTS);
}

#define MIPS_CPU_TIMER_IRQ 7
/* this gets called straight from incaipIRQ.S */
asmlinkage void incaip_timer_interrupt(struct pt_regs *regs)
{
	int cpu = smp_processor_id();
     int irq = MIPS_CPU_TIMER_IRQ;
    
#if defined(CONFIG_PREEMPT)
	preempt_disable();
#endif
	irq_enter(cpu, irq);
	kstat.irqs[cpu][irq]++;

	/* we keep interrupt disabled all the time */
	timer_interrupt(irq, NULL, regs);

	irq_exit(cpu, irq);

	if (softirq_pending(cpu))
		do_softirq();

#if defined(CONFIG_PREEMPT)
	for(;;) {
		preempt_enable_no_resched();
		if (preempt_is_disabled() || !need_resched())
			break;

		db_assert(intr_off());
		db_assert(!in_interrupt());

		preempt_disable();
		__sti();
		preempt_schedule();
		__cli();
	}
#endif
	ILATENCY_RET_FROM_EXCEPTION();
}

#ifdef CONFIG_KGDB
int putDebugChar(char c)
{
	while (((*INCA_IP_ASC_FSTAT) >> 8) != 0x00) ; 
	*INCA_IP_ASC_ASC_TBUF=c;
	return 1;
}

char getDebugChar(void)
{
	while (((*INCA_IP_ASC_FSTAT) & 0x3f) == 0x00) ; 
	return *INCA_IP_ASC_ASC_RBUF;
}
#endif

static char buf[1024];

static void prom_printf(const char * fmt, ...)
{
	va_list args;
	int l;
	char *p, *buf_end;

#ifdef CONFIG_KGDB
	/* Do not print if we are already debugging */
	if (remote_debug == 1) return;
#endif
	
	/* Low level, brute force, not SMP safe... */
	va_start(args, fmt);
	l = vsprintf(buf, fmt, args); /* hopefully i < sizeof(buf) */
	va_end(args);
	buf_end = buf + l;
	
	for (p = buf; p < buf_end; p++) {
		/* Wait for FIFO to empty */
		while (((*INCA_IP_ASC_FSTAT) >> 8) != 0x00) ; 
		/* Crude cr/nl handling is better than none */
		if(*p == '\n') *INCA_IP_ASC_ASC_TBUF=('\r');
		*INCA_IP_ASC_ASC_TBUF=(*p);
	}

}

#define INCAIP_COMPACT		0x00C1
#define INCAIP_STANDARD		0x00C0
#define INCAIP_PC_PORT_PHY	5
#define INCAIP_LAN_PORT_PHY	6
extern void inca_mux_init(void);
extern u16 mdio_read(u16 phy_id, u16 reg_num);
extern u16 mdio_write(u16 phy_id, u16 reg_num, u16 val);

void __init incaip_setup(void)
{
	u32 chipid = 0;
	u32 part_no = 0;
	u16 phyregsave = 0;
#ifdef CONFIG_KGDB
	char *argptr;
#endif
	
	prom_printf("Checking for Inca-IP type.\n");
	chipid = *(INCA_IP_WDT_CHIPID);
	part_no = INCA_IP_WDT_CHIPID_PART_NUMBER_GET(chipid);
#ifdef CONFIG_INCAIP_COMPACT
	if (part_no != INCAIP_COMPACT) {
		prom_printf("FATAL ERROR.\n");
		prom_printf("Inca-IP Compact kernel running on Inca-IP\n");
		prom_printf("Died.\n");
		panic("Help!");
	}
#else
	if (part_no == INCAIP_COMPACT) {
		prom_printf("FATAL ERROR.\n");
		prom_printf("Inca-IP kernel running on Inca-IP Compact");
		prom_printf("Died.\n");
		panic("Help!");
	}
#endif		

	/* 
	 * disable the ethernet switch to avoid problems 
	 * with ethernet traffic during startup 
	 */
	*INCA_IP_PMU_PM_GEN &= ~( INCA_IP_PMU_PM_GEN_EN5 | INCA_IP_PMU_PM_GEN_EN6 );    

	/* Changing amplitude setting for Ethernet PHYs */
	phyregsave=mdio_read(INCAIP_PC_PORT_PHY,0);
	mdio_write(INCAIP_PC_PORT_PHY,0,0x2100);
	mdio_write(INCAIP_PC_PORT_PHY,20,0x0000);
	mdio_write(INCAIP_PC_PORT_PHY,20,0x0400);
	mdio_write(INCAIP_PC_PORT_PHY,20,0x0000);
	mdio_write(INCAIP_PC_PORT_PHY,20,0x0400);
	mdio_write(INCAIP_PC_PORT_PHY,23,0x81b8);
	mdio_write(INCAIP_PC_PORT_PHY,20,0x4416);
	mdio_write(INCAIP_PC_PORT_PHY,0,phyregsave);
	
	phyregsave=mdio_read(INCAIP_LAN_PORT_PHY,0);
	mdio_write(INCAIP_LAN_PORT_PHY,0,0x2100);
	mdio_write(INCAIP_LAN_PORT_PHY,20,0x0000);
	mdio_write(INCAIP_LAN_PORT_PHY,20,0x0400);
	mdio_write(INCAIP_LAN_PORT_PHY,20,0x0000);
	mdio_write(INCAIP_LAN_PORT_PHY,20,0x0400);
	mdio_write(INCAIP_LAN_PORT_PHY,23,0x81b8);
	mdio_write(INCAIP_LAN_PORT_PHY,20,0x4416);
	mdio_write(INCAIP_LAN_PORT_PHY,0,phyregsave);
	
	inca_mux_init();
	
	incaip_reboot_setup();
	board_time_init = incaip_time_init;
	board_timer_setup = incaip_timer_setup;

#ifdef CONFIG_KGDB

#if 0
		struct serial_state *ser = &rs_table[line];

		kdb_port_info.state = ser;
		kdb_port_info.magic = SERIAL_MAGIC;
		kdb_port_info.port = ser->port;
		kdb_port_info.flags = ser->flags;

		kdb_port_info.MCR = 0;		/* no interrupts, please */
#endif
		/*
		 * We could set the serial speed here, but we leave it
		 * at the speed set by the bootloader.
		 */


		prom_printf("KGDB: Using serial line /dev/ttyS0 for session, "
			    "please connect your debugger\n");

		remote_debug = 1;
		/* Breakpoints are in init_IRQ() */

#endif
}
