/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/*
 * i2c-algo-inca.h
 *
 * Inca-IP I2C algorithm driver header file
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#ifndef I2C_ALGO_INCA_H
#define I2C_ALGO_INCA_H

struct i2c_algo_inca_data {
	void *data;		/* private data for lowlevel routines */
	int (*writebyte) (int, char);
	char (*readbyte) (int);
};

#define IIC_DEFAULT_BAUDRATE 100000
//#define IIC_DEFAULT_SLAVE_ADDRESS 0x55
#define IIC_DEFAULT_SLAVE_ADDRESS 0x66
#define IIC_10BIT_PREFIX	0xF0

#define I2C_IRQD_BIT	(1 << 22)
#define I2C_IRQD_ENABLE()  *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) |= I2C_IRQD_BIT;
#define I2C_IRQD_DISABLE() *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) &= ~I2C_IRQD_BIT;
#define I2C_IRQD_CLEAR()   *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_ISR) = I2C_IRQD_BIT;

#define I2C_IRQP_BIT	(1 << 23)
#define I2C_IRQP_ENABLE()  *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) |= I2C_IRQP_BIT;
#define I2C_IRQP_DISABLE() *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) &= ~I2C_IRQP_BIT;
#define I2C_IRQP_CLEAR()   *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_ISR) = I2C_IRQP_BIT;

#define I2C_IRQE_BIT	(1 << 24)
#define I2C_IRQE_ENABLE()  *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) |= I2C_IRQE_BIT;
#define I2C_IRQE_DISABLE() *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_IER) &= ~I2C_IRQE_BIT;
#define I2C_IRQE_CLEAR()   *((volatile unsigned long*) \
				INCA_IP_ICU_IM2_ISR) = I2C_IRQE_BIT;




/*----------------------------------------------------------------------------*/
/* Default register values                                                    */
/*----------------------------------------------------------------------------*/
#define INIT_IIP_CLC        0x00000100
#define INIT_IIP_PISEL      0x00000000
#define INIT_IIP_SYSCON     0x051A0000
#define INIT_IIP_BUSCON     0x00AA0011
#define INIT_IIP_RTB        0x00000000
#define INIT_IIP_WHBSYSCON  0x00000000

//#define INIT_IIP_SYSCON     0x01880000
#define INIT_IIP_BUSCON_100     0xC0AA8311
#define INIT_IIP_BUSCON_400     0xA0AA4211

/*----------------------------------------------------------------------------*/
/* bit definition for the INIT_IIP_SYSCON register                            */
/*----------------------------------------------------------------------------*/
#define ADR               (1 << 0) 
#define AL                (1 << 1) 
#define SLA               (1 << 2) 
#define LRB               (1 << 3) 
#define BB                (1 << 4) 
#define IRQD              (1 << 5) 
#define IRQP              (1 << 6) 
#define IRQE              (1 << 7) 
#define RMEN              (1 << 15)
                                   
#define M10               (1 << 16)
#define RSC               (1 << 17)
#define DISMODE           (0 << 18)
#define SLMODE            (1 << 18)
#define MSMODE            (2 << 18)
#define MMMODE            (3 << 18)
#define BUM               (1 << 20)
#define ACKDIS            (1 << 21)
#define INTR              (1 << 22)
#define TRX               (1 << 23)
#define IGE               (1 << 24)
#define STP               (1 << 25)
#define T1BYTE            (0 << 26)
#define T2BYTE            (1 << 26)
#define T3BYTE            (2 << 26)
#define T4BYTE            (3 << 26)
#define WMEN              (1 << 31)

#define I2C_ALGO_INCA I2C_ALGO_EXP
#define SYNC asm volatile ("sync;")
#define NOPS asm volatile ("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop")
#define I2C_WRITE_REG(reg, value)   *((volatile u32*)(reg)) = (u32)value;SYNC;NOPS
#define I2C_READ_REG(reg, value)    value = (u32)*((volatile u32*)(reg));SYNC;NOPS
//#define I2C_WRITE_REG(reg, value)   *((volatile u32*)(reg)) = (u32)value;SYNC;NOPS
//#define I2C_READ_REG(reg, value)    value = (u32)*((volatile u32*)(reg));SYNC;NOPS

int i2c_inca_add_bus(struct i2c_adapter *adap);
int i2c_inca_del_bus(struct i2c_adapter *adap);

#endif /* I2C_ALGO_INCA_H */ 
