/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/*
 * i2c-at24c512.c
 *
 * Inca-IP I2C driver for Atmel AT24C02
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/string.h>  /* for 2.0 kernels to get NULL   */
#include <asm/errno.h>     /* for 2.0 kernels to get ENODEV */
#include <asm/io.h>

#include <linux/i2c.h>

#define I2C_ALGO_INCA I2C_ALGO_EXP
#define I2C_HW_INCA 0x00
#define AT24C02_ADDR	0x50
#define I2C_DRIVERID_AT24C02 I2C_DRIVERID_EXP0+1

static int at24c02_major = 0;

static unsigned short normal_i2c[] = { AT24C02_ADDR, I2C_CLIENT_END };
static unsigned short normal_i2c_range[] = { AT24C02_ADDR, AT24C02_ADDR, I2C_CLIENT_END };
I2C_CLIENT_INSMOD;

struct i2c_client* at24c02_client;

static int at24c02_attach_client(struct i2c_adapter* adapter, int address, unsigned short flags,
		int kind);

int at24c02_attach_adapter(struct i2c_adapter *adap)
{
	if (adap->id == (I2C_ALGO_INCA | I2C_HW_INCA))
		return i2c_probe(adap, &addr_data, &at24c02_attach_client);
	return 0;
}

int at24c02_detach_client(struct i2c_client *adap)
{
	return 0;
}

int at24c02_command(struct i2c_client *client,unsigned int cmd, void *arg)
{
	return 0;
}

void at24c02_inc_use(struct i2c_client *client)
{
	return;
}

void at24c02_dec_use(struct i2c_client *client)
{
	return;
}

static struct i2c_driver at24c02_driver={
	"at24c02",
	I2C_DRIVERID_AT24C02,
	I2C_DF_NOTIFY,
	at24c02_attach_adapter,
	at24c02_detach_client,
	at24c02_command,
	at24c02_inc_use,
	at24c02_dec_use
};

static int at24c02_attach_client(struct i2c_adapter* adapter, int address, unsigned short flags,
		int kind) {
	int err = 0;
	struct i2c_client* new_client;
//	struct maven_data* data;

	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_WRITE_WORD_DATA |
					      I2C_FUNC_SMBUS_BYTE_DATA |
					      I2C_FUNC_PROTOCOL_MANGLING))
		goto ERROR0;

	if (!(new_client = (struct i2c_client*)kmalloc(sizeof(struct i2c_client),
			GFP_KERNEL))) {
		err = -ENOMEM;
		goto ERROR0;
	}

//	if (!(new_client = (struct i2c_client*)kmalloc(sizeof(struct i2c_client) + sizeof(struct maven_data),
//			GFP_KERNEL))) {
//		err = -ENOMEM;
//		goto ERROR0;
//	}
//	data = (struct maven_data*)(new_client + 1);
//	new_client->data = data;
	new_client->data = NULL;
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &at24c02_driver;
	new_client->flags = 0;
	strcpy(new_client->name, "AT24C02 Client");
	if ((err = i2c_attach_client(new_client)))
		goto ERROR3;
	at24c02_client = new_client;
	return 0;
ERROR3:;
	kfree(new_client);
ERROR0:;
	return err;
}

static int at24c02_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static int at24c02_release(struct inode *inode, struct file *filp)
{
	return 0;
}

static ssize_t at24c02_read(struct file * file, char * buf,
			size_t count, loff_t *f_pos)
{
	char* p;
	char temp_buf[2];
	int t;
	struct i2c_msg msgs[] = {{ at24c02_client->addr, 0, 0, 0 },
				 { at24c02_client->addr, I2C_M_RD, 0, 0 }};

	if (*f_pos > 0xffff) return -EINVAL;
	
//	p = kmalloc(10,GFP_KERNEL);
	p = kmalloc(count,GFP_KERNEL);
	if (p == NULL) return -ENOMEM;

	msgs[0].len=1;
	msgs[0].buf=temp_buf;
	
	msgs[1].len=1;
	msgs[1].buf=p;

	for (t=0;t<count;t++) {	
//		temp_buf[0]=((((*f_pos)+t) & 0x0000ff00) >> 8);
		temp_buf[0]=(((*f_pos)+t) & 0x000000ff);
		i2c_transfer(at24c02_client->adapter, msgs, 2);
		msgs[1].buf++;

		/* wait 20 ms */
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(20*HZ/1000);
	
	}
	
	if (copy_to_user(buf, p, count)) {
		kfree(p);
		return -EFAULT;
	}
	*f_pos +=  count;
	kfree(p);
	return count;
}

static ssize_t at24c02_write(struct file * file, const char * buf, 
			 size_t count, loff_t *f_pos)
{
	char *p,*temp_p;
	char temp_buf[3];
	int t;
	struct i2c_msg msgs[1] = {{ at24c02_client->addr, 0, 0, 0 }};
	
	if (*f_pos > 0xffff) return -EINVAL;
	
	p = kmalloc(count,GFP_KERNEL);
	if (p == NULL) return -ENOMEM;
	temp_p = p;
	
	msgs[0].len=2;
	msgs[0].buf=temp_buf;
	
	if (copy_from_user(p, buf, count)) {
		kfree(p);
		return -EFAULT;
	}

	for (t=0;t<count;t++) {
//		temp_buf[0]=((((*f_pos)+t) & 0x0000ff00) >> 8);
		temp_buf[0]=(((*f_pos)+t) & 0x000000ff);
		temp_buf[1]= *temp_p;
		i2c_transfer(at24c02_client->adapter, msgs, 1);
		temp_p++;

		/* wait 20 ms */
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(20*HZ/1000);
		
	}

	*f_pos +=  count;
	kfree(p);
	return count;
}

static loff_t at24c02_lseek(struct file * file, loff_t offset, int orig)
{
//	printk("at24c: seeking\n");
	switch (orig) {
		case 0:
			file->f_pos = offset;
			return file->f_pos;
		case 1:
			file->f_pos += offset;
			return file->f_pos;
		default:
			return -EINVAL;
	}
}



static struct file_operations at24c02_fops = {
	llseek  : at24c02_lseek,
	open    : at24c02_open,
	release : at24c02_release,
	read	   :	at24c02_read,
	write   :	at24c02_write
};

int __init  at24c02_init(void)
{
	int err;

	printk("i2c-at24c02.o: AT24C02 driver module\n");

	err = i2c_add_driver(&at24c02_driver);
	if (err) {
		printk(KERN_ERR "AT24C02: Driver failed to register (%d).\n", err);
		return err;
	}
	
   	/* register EEPROM device */
   	err = register_chrdev(0,"at24c02", &at24c02_fops);
	if (err > 0) at24c02_major = err;
	else {
		printk("at24c02: Error! Could not register at24c02 device. #%d\n",err);
		return err;
	}

	return 0;
}

int at24c02_exit(void)
{
	printk("i2c-at24c02.o: AT24C02 driver module removed\n");

	i2c_detach_client(at24c02_client);
	i2c_del_driver(&at24c02_driver);
	
	unregister_chrdev(at24c02_major, "at24c02");

	return 0;
}


module_init(at24c02_init);

#ifdef MODULE
MODULE_AUTHOR("Andre Messerschmidt");
MODULE_DESCRIPTION("AT24C02 driver routines for INCA-IP");
MODULE_LICENSE("GPL");

void cleanup_module(void) 
{
	at24c02_exit();
}

#endif
