/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/*
 * i2c-at24c512.c
 *
 * Inca-IP I2C driver for INCA-IP as slave device
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <asm/uaccess.h>
#include <linux/string.h>  /* for 2.0 kernels to get NULL   */
#include <asm/errno.h>     /* for 2.0 kernels to get ENODEV */
#include <asm/io.h>

#include <linux/i2c.h>

#define I2C_ALGO_INCA I2C_ALGO_EXP
#define I2C_HW_INCA 0x00
#define INCA_SLAVE_ADDR	0x66
//#define INCA_SLAVE_ADDR	0x55
#define I2C_DRIVERID_INCA_SLAVE I2C_DRIVERID_EXP0

static int inca_sl_major = 0;

static unsigned short normal_i2c[] = { INCA_SLAVE_ADDR, I2C_CLIENT_END };
static unsigned short normal_i2c_range[] = { INCA_SLAVE_ADDR, INCA_SLAVE_ADDR, I2C_CLIENT_END };
I2C_CLIENT_INSMOD;

struct i2c_client* inca_sl_client;

static int inca_sl_attach_client(struct i2c_adapter* adapter, int address, unsigned short flags,
		int kind);

int inca_sl_attach_adapter(struct i2c_adapter *adap)
{
	if (adap->id == (I2C_ALGO_INCA | I2C_HW_INCA))
		return i2c_probe(adap, &addr_data, &inca_sl_attach_client);
	return 0;
}

int inca_sl_detach_client(struct i2c_client *adap)
{
	return 0;
}

int inca_sl_command(struct i2c_client *client,unsigned int cmd, void *arg)
{
	return 0;
}

void inca_sl_inc_use(struct i2c_client *client)
{
	return;
}

void inca_sl_dec_use(struct i2c_client *client)
{
	return;
}

static int inca_sl_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static int inca_sl_release(struct inode *inode, struct file *filp)
{
	return 0;
}

static ssize_t inca_sl_read(struct file * file, char * buf,
			size_t count, loff_t *f_pos)
{
	char* p;
	char temp_buf[2];
	int t;
//	struct i2c_msg msgs[] = {{ inca_sl_client->addr, 0, 0, 0 },
//				 { inca_sl_client->addr, I2C_M_RD, 0, 0 }};
	struct i2c_msg msgs[] = { inca_sl_client->addr, I2C_M_RD, 0, 0 };

	if (*f_pos > 0xffff) return -EINVAL;
	
//	p = kmalloc(10,GFP_KERNEL);
	p = kmalloc(count,GFP_KERNEL);
	if (p == NULL) return -ENOMEM;

//	msgs[0].len=2;
//	msgs[0].buf=temp_buf;
	
//	msgs[1].len=1;
//	msgs[1].buf=p;
	msgs[0].len=1;
	msgs[0].buf=p;

	for (t=0;t<count;t++) {	
//		temp_buf[0]=((((*f_pos)+t) & 0x0000ff00) >> 8);
//		temp_buf[1]=(((*f_pos)+t) & 0x000000ff);
		i2c_transfer(inca_sl_client->adapter, msgs, 1);
//		printk("at24c512: p[t]=%x p[t+1]=%x\n",*(msgs[1].buf),*(msgs[1].buf+1));
		msgs[0].buf++;

		/* wait 20 ms */
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(20*HZ/1000);
	
	}
	
	if (copy_to_user(buf, p, count)) {
		kfree(p);
		return -EFAULT;
	}
	*f_pos +=  count;
	kfree(p);
	return count;
}

static ssize_t inca_sl_write(struct file * file, const char * buf, 
			 size_t count, loff_t *f_pos)
{
	char *p,*temp_p;
	char temp_buf[3];
	int t;
	struct i2c_msg msgs[1] = {{ inca_sl_client->addr, 0, 0, 0 }};
	
	if (*f_pos > 0xffff) return -EINVAL;
	
	p = kmalloc(count,GFP_KERNEL);
	if (p == NULL) return -ENOMEM;
	temp_p = p;
	
	msgs[0].len=1;
	msgs[0].buf=temp_buf;
	
	if (copy_from_user(p, buf, count)) {
		kfree(p);
		return -EFAULT;
	}

	for (t=0;t<count;t++) {
//		temp_buf[0]=((((*f_pos)+t) & 0x0000ff00) >> 8);
//		temp_buf[1]=(((*f_pos)+t) & 0x000000ff);
//		temp_buf[2]= *temp_p;
		temp_buf[0]= *temp_p;

//		printk("at24c: write #%d: 0x%02x 0x%02x 0x%02x \n",t,temp_buf[0]&0xff,temp_buf[1]&0xff,temp_buf[2]&0xff);
		i2c_transfer(inca_sl_client->adapter, msgs, 1);
		temp_p++;

		/* wait 20 ms */
		current->state = TASK_INTERRUPTIBLE;
		schedule_timeout(20*HZ/1000);
		
	}

	*f_pos +=  count;
	kfree(p);
	return count;
}

static loff_t inca_sl_lseek(struct file * file, loff_t offset, int orig)
{
//	printk("inca_sl: seeking\n");
	switch (orig) {
		case 0:
			file->f_pos = offset;
			return file->f_pos;
		case 1:
			file->f_pos += offset;
			return file->f_pos;
		default:
			return -EINVAL;
	}
}



static struct file_operations inca_sl_fops = {
	llseek  : inca_sl_lseek,
	open    : inca_sl_open,
	release : inca_sl_release,
	read	   :	inca_sl_read,
	write   :	inca_sl_write
};


static struct i2c_driver inca_sl_driver={
	"inca_slave",
	I2C_DRIVERID_INCA_SLAVE,
	I2C_DF_NOTIFY,
	inca_sl_attach_adapter,
	inca_sl_detach_client,
	inca_sl_command,
	inca_sl_inc_use,
	inca_sl_dec_use
};

static int inca_sl_attach_client(struct i2c_adapter* adapter, int address, 
	unsigned short flags, int kind) 
{
	int err = 0;
	struct i2c_client* new_client;
//	struct maven_data* data;

	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_WRITE_WORD_DATA |
					      I2C_FUNC_SMBUS_BYTE_DATA |
					      I2C_FUNC_PROTOCOL_MANGLING))
		goto ERROR0;

	if (!(new_client = (struct i2c_client*)kmalloc(sizeof(struct i2c_client),
			GFP_KERNEL))) {
		err = -ENOMEM;
		goto ERROR0;
	}

//	if (!(new_client = (struct i2c_client*)kmalloc(sizeof(struct i2c_client) + sizeof(struct maven_data),
//			GFP_KERNEL))) {
//		err = -ENOMEM;
//		goto ERROR0;
//	}
//	data = (struct maven_data*)(new_client + 1);
//	new_client->data = data;
	new_client->data = NULL;
	new_client->addr = address;
	new_client->adapter = adapter;
	new_client->driver = &inca_sl_driver;
	new_client->flags = 0;
	strcpy(new_client->name, "inca_sl Client");
	if ((err = i2c_attach_client(new_client)))
		goto ERROR3;
	inca_sl_client = new_client;
	return 0;
ERROR3:;
	kfree(new_client);
ERROR0:;
	return err;
}

int __init  inca_sl_init(void)
{
	int err;

	printk("i2c-inca_sl: inca_sl driver module\n");

	err = i2c_add_driver(&inca_sl_driver);
	if (err) {
		printk(KERN_ERR "inca_sl: Driver failed to register (%d).\n", err);
		return err;
	}
	
   	/* register Inca-I2C Slave device */
   	err = register_chrdev(0,"inca_i2c_slave", &inca_sl_fops);
	if (err > 0) inca_sl_major = err;
	else {
		printk("inca_sl: Error! Could not register incaq_i2c_slave device. #%d\n",err);
		return err;
	}

	return 0;
}

int inca_sl_exit(void)
{
	printk("i2c-inca_sl: inca_sl driver module removed\n");

	i2c_detach_client(inca_sl_client);
	i2c_del_driver(&inca_sl_driver);
	unregister_chrdev(inca_sl_major, "inca_i2c_slave");
	
	return 0;
}


module_init(inca_sl_init);

#ifdef MODULE
MODULE_AUTHOR("Andre Messerschmidt");
MODULE_DESCRIPTION("inca_slave driver routines for INCA-IP");
MODULE_LICENSE("GPL");


void cleanup_module(void) 
{
	inca_sl_exit();
}

#endif
