/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/*
 * i2c-inca.c
 *
 * Inca-IP I2C adapter driver
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/string.h>  /* for 2.0 kernels to get NULL   */
#include <asm/errno.h>     /* for 2.0 kernels to get ENODEV */
#include <asm/io.h>

#include <linux/i2c.h>
#include "i2c-algo-inca.h"

#define I2C_HW_INCA 0x00

static int inca_writebyte(int addr, char data)
{
	return 0;
}

static char inca_readbyte(int addr)
{
	return 1;
}

static int inca_init(void)
{
	return 0;
}

static void __exit inca_exit(void)
{	
	;
}


static int inca_reg(struct i2c_client *client)
{
	return 0;
}

static int inca_unreg(struct i2c_client *client)
{
	return 0;
}

static void inca_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void inca_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_inca_data inca_data = {
	NULL,
	inca_writebyte,
	inca_readbyte,
};

static struct i2c_adapter inca_ops = {
	"INCA-IP",
	I2C_HW_INCA,
	NULL,
	&inca_data,
	inca_inc_use,
	inca_dec_use,
	inca_reg,
	inca_unreg,
};

int __init  i2c_inca_init(void)
{
	printk("i2c-inca.o: i2c INCA-IP adapter module\n");
	if (inca_init()==0) {
		if(i2c_inca_add_bus(&inca_ops) < 0)
			return -ENODEV;
	} else {
		return -ENODEV;
	}
	return 0;
}

module_init(i2c_inca_init);

#ifdef MODULE
MODULE_AUTHOR("Andre Messerschmidt");
MODULE_DESCRIPTION("I2C-Bus adapter routines for INCA-IP");
MODULE_LICENSE("GPL");

/*
int init_module(void) 
{
	return i2c_inca_init();
}
*/

void cleanup_module(void) 
{
	i2c_inca_del_bus(&inca_ops);
	printk("i2c-inca.o: i2c INCA-IP adapter removed\n");
	//inca_exit();
}

#endif
