/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/*
 * mux.c
 *
 * Inca-IP multiplexer
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/addrspace.h>
#include <asm/incaip/mux.h>
#include "mux_defs.h"

int mux_led = 0;
int mux_addr = 0;
int mux_key = 0;
u32 mux_iom2_bits = 0;
u32 mux_asc_bits = 0;
u32 mux_gpio0_bits = 0;
u32 mux_gpio1_bits = 0;
u32 mux_gpio2_bits = 0;
u32 mux_exin_bits = 0;
int mux_debug_mode = 0;


/**
 *     inca_mux_iom2_init - Initialize IOM-2 return value
 *
 *     This function prepares the return value for inca_mux_iom_pins()
 */
void inca_mux_iom2_init(void)
{
#ifdef CONFIG_INCAIP_MUX_FSC 
	mux_iom2_bits |= INCAIP_MUX_IOM2_FSC;
#endif 
#ifdef CONFIG_INCAIP_MUX_DCL 
	mux_iom2_bits |= INCAIP_MUX_IOM2_DCL;
#endif
#ifdef CONFIG_INCAIP_MUX_DU 
	mux_iom2_bits |= INCAIP_MUX_IOM2_DU;
#endif
#ifdef CONFIG_INCAIP_MUX_DD 
	mux_iom2_bits |= INCAIP_MUX_IOM2_DD;
#endif
#ifdef CONFIG_INCAIP_MUX_SDS 
	mux_iom2_bits |= INCAIP_MUX_IOM2_SDS;
#endif
#ifdef CONFIG_INCAIP_MUX_BCL 
	mux_iom2_bits |= INCAIP_MUX_IOM2_BCL;
#endif
}

/**
 *     inca_mux_asc_init - Initialize ASC return value
 *
 *     This function prepares the return value for inca_mux_asc_pins()
 */
void inca_mux_asc_init(void)
{
#ifdef CONFIG_INCAIP_MUX_RXD 
	mux_asc_bits |= INCAIP_MUX_ASC_RXD;
#endif
#ifdef CONFIG_INCAIP_MUX_TXD 
	mux_asc_bits |= INCAIP_MUX_ASC_TXD;
#endif
} 

/**
 *     inca_mux_exin_init - Initialize EXIN return value
 *
 *     This function prepares the return value for inca_mux_exin_pins()
 */
 
void inca_mux_exin_init(void)
{
	mux_exin_bits = INCAIP_MUX_EXIN0|INCAIP_MUX_EXIN1|INCAIP_MUX_EXIN2;
#ifdef CONFIG_INCAIP_MUX_EXIN3 
	mux_exin_bits |= INCAIP_MUX_EXIN3;
#endif
#ifdef CONFIG_INCAIP_MUX_EXIN4 
	mux_exin_bits |= INCAIP_MUX_EXIN4;
#endif
#ifdef CONFIG_INCAIP_MUX_EXIN5 
	mux_exin_bits |= INCAIP_MUX_EXIN5;
#endif
} 

/**
 *     inca_mux_port_init - Initialize GPIO return values
 *
 *     This function prepares the return values for inca_mux_gpio_pins()
 */
 
void inca_mux_gpio_init(void)
{
/* Port #0 Config */
#ifdef CONFIG_INCAIP_MUX_P0_0 
	mux_gpio0_bits |= INCAIP_MUX_P0_0;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_1 
	mux_gpio0_bits |= INCAIP_MUX_P0_1;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_2 
	mux_gpio0_bits |= INCAIP_MUX_P0_2;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_3 
	mux_gpio0_bits |= INCAIP_MUX_P0_3;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_4 
	mux_gpio0_bits |= INCAIP_MUX_P0_4;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_5 
	mux_gpio0_bits |= INCAIP_MUX_P0_5;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_6 
	mux_gpio0_bits |= INCAIP_MUX_P0_6;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_7 
	mux_gpio0_bits |= INCAIP_MUX_P0_7;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_8 
	mux_gpio0_bits |= INCAIP_MUX_P0_8;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_9 
	mux_gpio0_bits |= INCAIP_MUX_P0_9;
#endif
#ifdef CONFIG_INCAIP_MUX_P0_10 
	mux_gpio0_bits |= INCAIP_MUX_P0_10;
#endif

/* Port #1 Config */
#ifdef CONFIG_INCAIP_MUX_P1_0 
	mux_gpio1_bits |= INCAIP_MUX_P1_0;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_1 
	mux_gpio1_bits |= INCAIP_MUX_P1_1;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_2 
	mux_gpio1_bits |= INCAIP_MUX_P1_2;
#endif 
#ifdef CONFIG_INCAIP_MUX_P1_3 
	mux_gpio1_bits |= INCAIP_MUX_P1_3;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_4 
	mux_gpio1_bits |= INCAIP_MUX_P1_4;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_5 
	mux_gpio1_bits |= INCAIP_MUX_P1_5;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_6 
	mux_gpio1_bits |= INCAIP_MUX_P1_6;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_7 
	mux_gpio1_bits |= INCAIP_MUX_P1_7;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_8 
	mux_gpio1_bits |= INCAIP_MUX_P1_8;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_9 
	mux_gpio1_bits |= INCAIP_MUX_P1_9;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_10 
	mux_gpio1_bits |= INCAIP_MUX_P1_10;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_11 
	mux_gpio1_bits |= INCAIP_MUX_P1_11;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_12 
	mux_gpio1_bits |= INCAIP_MUX_P1_12;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_13 
	mux_gpio1_bits |= INCAIP_MUX_P1_13;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_14 
	mux_gpio1_bits |= INCAIP_MUX_P1_14;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_15 
	mux_gpio1_bits |= INCAIP_MUX_P1_15;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_16 
	mux_gpio1_bits |= INCAIP_MUX_P1_16;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_17 
	mux_gpio1_bits |= INCAIP_MUX_P1_17;
#endif 
#ifdef CONFIG_INCAIP_MUX_P1_18 
	mux_gpio1_bits |= INCAIP_MUX_P1_18;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_19 
	mux_gpio1_bits |= INCAIP_MUX_P1_19;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_20 
	mux_gpio1_bits |= INCAIP_MUX_P1_20;
#endif 
#ifdef CONFIG_INCAIP_MUX_P1_21 
	mux_gpio1_bits |= INCAIP_MUX_P1_21;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_22 
	mux_gpio1_bits |= INCAIP_MUX_P1_22;
#endif 
#ifdef CONFIG_INCAIP_MUX_P1_23 
	mux_gpio1_bits |= INCAIP_MUX_P1_23;
#endif

/* Port #2 Config */
#ifdef CONFIG_INCAIP_MUX_P2_0 
	mux_gpio2_bits |= INCAIP_MUX_P2_0;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_1 
	mux_gpio2_bits |= INCAIP_MUX_P2_1;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_2 
	mux_gpio2_bits |= INCAIP_MUX_P2_2;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_3 
	mux_gpio2_bits |= INCAIP_MUX_P2_3;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_4 
	mux_gpio2_bits |= INCAIP_MUX_P2_4;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_5 
	mux_gpio2_bits |= INCAIP_MUX_P2_5;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_6 
	mux_gpio2_bits |= INCAIP_MUX_P2_6;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_7 
	mux_gpio2_bits |= INCAIP_MUX_P2_7;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_8 
	mux_gpio2_bits |= INCAIP_MUX_P2_8;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_9 
	mux_gpio2_bits |= INCAIP_MUX_P2_9;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_10 
	mux_gpio2_bits |= INCAIP_MUX_P2_10;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_11 
	mux_gpio2_bits |= INCAIP_MUX_P2_11;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_12 
	mux_gpio2_bits |= INCAIP_MUX_P2_12;
#endif 
#ifdef CONFIG_INCAIP_MUX_P2_13 
	mux_gpio2_bits |= INCAIP_MUX_P2_13;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_14 
	mux_gpio2_bits |= INCAIP_MUX_P2_14;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_15 
	mux_gpio2_bits |= INCAIP_MUX_P2_15;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_16 
	mux_gpio2_bits |= INCAIP_MUX_P2_16;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_17 
	mux_gpio2_bits |= INCAIP_MUX_P2_17;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_18 
	mux_gpio2_bits |= INCAIP_MUX_P2_18;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_19 
	mux_gpio2_bits |= INCAIP_MUX_P2_19;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_20 
	mux_gpio2_bits |= INCAIP_MUX_P2_20;
#endif
#ifdef CONFIG_INCAIP_MUX_P2_21 
	mux_gpio2_bits |= INCAIP_MUX_P2_21;
#endif 

} 

/**
 *     inca_mux_led_init - Initialize LED return value
 *
 *     This function prepares the return value for inca_mux_led_pins()
 */
 
void inca_mux_led_init(void)
{
#ifdef CONFIG_INCAIP_MUX_LED0 
	mux_led = 1;
#endif
#ifdef CONFIG_INCAIP_MUX_LED1 
	mux_led = 2;
#endif
#ifdef CONFIG_INCAIP_MUX_LED2 
	mux_led = 3;
#endif 
#ifdef CONFIG_INCAIP_MUX_LED3 
	mux_led = 4;
#endif
#ifdef CONFIG_INCAIP_MUX_LED4 
	mux_led = 5;
#endif
#ifdef CONFIG_INCAIP_MUX_LED5 
	mux_led = 6;
#endif
#ifdef CONFIG_INCAIP_MUX_LED6 
	mux_led = 7;
#endif
#ifdef CONFIG_INCAIP_MUX_LED7 
	mux_led = 8;
#endif
#ifdef CONFIG_INCAIP_MUX_LED8 
	mux_led = 9;
#endif
#ifdef CONFIG_INCAIP_MUX_LED9 
	mux_led = 10;
#endif
} 

/**
 *     inca_mux_key_init - Initialize KEY return value
 *
 *     This function prepares the return value for inca_mux_key_pins()
 */
 
void inca_mux_key_init(void)
{
#ifdef CONFIG_INCAIP_MUX_KEY0 
	mux_key = 1;
#endif 
#ifdef CONFIG_INCAIP_MUX_KEY1 
	mux_key = 2;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY2 
	mux_key = 3;
#endif 
#ifdef CONFIG_INCAIP_MUX_KEY3 
	mux_key = 4;
#endif 
#ifdef CONFIG_INCAIP_MUX_KEY4 
	mux_key = 5;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY5 
	mux_key = 6;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY6 
	mux_key = 7;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY7 
	mux_key = 8;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY8 
	mux_key = 9;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY9 
	mux_key = 10;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY10 
	mux_key = 11;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY11 
	mux_key = 12;
#endif 
#ifdef CONFIG_INCAIP_MUX_KEY12 
	mux_key = 13;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY13 
	mux_key = 14;
#endif
} 

/**
 *     inca_mux_addr_init - Initialize ADDR return value
 *
 *     This function prepares the return value for inca_mux_addr_pins()
 */
 
void inca_mux_addr_init(void)
{
	mux_addr=24;
#ifdef CONFIG_INCAIP_MUX_P1_12 
	mux_addr=23;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_11 
	mux_addr=22;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_10 
	mux_addr=21;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_9 
	mux_addr=20;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_8 
	mux_addr=19;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_7 
	mux_addr=18;
#endif 
#ifdef CONFIG_INCAIP_MUX_P1_6 
	mux_addr=17;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_5 
	mux_addr=16;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_4 
	mux_addr=15;
#endif/
#ifdef CONFIG_INCAIP_MUX_P1_3 
	mux_addr=14;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_2 
	mux_addr=13;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_1 
	mux_addr=12;
#endif
#ifdef CONFIG_INCAIP_MUX_P1_0 
	mux_addr=11;
#endif

#ifdef CONFIG_INCAIP_MUX_SDS 
	mux_addr=10;
#endif
#ifdef CONFIG_INCAIP_MUX_BCL 
	mux_addr=9;
#endif
#ifdef CONFIG_INCAIP_MUX_EXIN5 
	mux_addr=8;
#endif
#ifdef CONFIG_INCAIP_MUX_EXIN4 
	mux_addr=7;
#endif 
#ifdef CONFIG_INCAIP_MUX_EXIN3 
	mux_addr=6;
#endif

#ifdef CONFIG_INCAIP_MUX_KEY13 
	mux_addr=5;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY12 
	mux_addr=4;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY11 
	mux_addr=3;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY10 
	mux_addr=2;
#endif
#ifdef CONFIG_INCAIP_MUX_KEY9 
	mux_addr=1;
#endif

#ifdef CONFIG_INCAIP_MUX_LED9 
	mux_addr=0;
#endif
} 



/**
 *     inca_mux_debug_init - Apply Debug Mode
 *     @mcr: Value of register MCD_MCR
 *
 *     This function changes return values for MUX functions to reflect
 *     the detected debug mode
 */
 
void inca_mux_debug_init(u32 mcr)
{
	/* Disable pins that are multiplexed with MUX5 */
	if ((mcr & INCA_IP_Debug_MCD_MCR_MUX5) != 0) {
		mux_exin_bits &= ~(INCAIP_MUX_EXIN1 | INCAIP_MUX_EXIN2);
		if (mux_key > 6) mux_key=6;
		if (mux_led > 8) mux_led=8;
	}

	/* Disable pins that are multiplexed with MUX4 */
	if ((mcr & INCA_IP_Debug_MCD_MCR_MUX4) != 0) {
		if (mux_led > 4) mux_led=4;
	}

	/* Disable pins that are multiplexed with MUX1 */
	if ((mcr & INCA_IP_Debug_MCD_MCR_MUX1) != 0) {
		if (mux_key > 5) mux_key=5;
		mux_led = 0;				
	}
	

} 

/**
 *     inca_mux_addr_pins - Number of address pins
 *
 *     Returns number of continuous available address pins starting by 0
 */
int inca_mux_addr_pins(void)
{
	return mux_addr;
}

/**
 *     inca_mux_led_pins - Number of LED pins
 *
 *     Returns number of available LED pins
 */
int inca_mux_led_pins(void)
{
	return mux_led;
}

/**
 *     inca_mux_key_pins - Number of keypad pins
 *
 *     Returns number of available keypad pins
 */
int inca_mux_key_pins(void)
{
	return mux_key;
}

/**
 *     inca_mux_gpio_pins - Available GPIO pins
 *     @port: Port that is queried (0,1, or 2)
 *
 *     Returns a bitfield describing which GPIO pins are available for this port
 *     bit x = port.x
 *     0x01=available,0x00=n/a
 */
u32 inca_mux_gpio_pins(int port)
{
	switch (port) {
		case 0: return mux_gpio0_bits;
		case 1: return mux_gpio1_bits;
		case 2: return mux_gpio2_bits;
	}
	return 0x00000000;
}

/**
 *     inca_mux_asc_pins - Available ASC pins
 *
 *     Returns a bitfield describing which ASC pins are available
 *     bit 0=RXD, bit 1=TXD
 *     0x01=available,0x00=n/a
 */
u32 inca_mux_asc_pins(void)
{
	return mux_asc_bits;
}

/**
 *     inca_mux_ssc1_pins - Availability of SSC1
 *
 *     Returns 0x01 if SSC1 is available and 0x00 if not.
 */
u32 inca_mux_ssc1_pins(void)
{
#ifdef CONFIG_INCAIP_MUX_SSC1
	return 1;
#else
	return 0;
#endif
}

/**
 *     inca_mux_ssc2_pins - Availability of SSC2
 *
 *     Returns 0x01 if SSC2 is available and 0x00 if not.
 */
u32 inca_mux_ssc2_pins(void)
{
#ifdef CONFIG_INCAIP_MUX_SSC2
	return 1;
#else
	return 0;
#endif
}

/**
 *     inca_mux_iom_pins - Available IOM-2 pins
 *
 *     Returns a bitfield describing the status of the IOM-2 pins
 *     bit 0=FSC, bit 1=DCL,bit 2=DU,bit 3=DD,bit 4=SDS, bit 5=BCL
 *     0x01=available, 0x00=n/a
 */
u32 inca_mux_iom_pins(void)
{
	return mux_iom2_bits;
}

/**
 *     inca_mux_pwm1_pins - Availability of PWM1
 *
 *     Returns 0x01 if PWM1 is available and 0x00 if not.
 */
u32 inca_mux_pwm1_pins(void)
{
#ifdef CONFIG_INCAIP_MUX_PWM1
	return 1;
#else
	return 0;
#endif 
}

/**
 *     inca_mux_pwm2_pins - Availability of PWM2
 *
 *     Returns 0x01 if PWM2 is available and 0x00 if not.
 */
u32 inca_mux_pwm2_pins(void)
{
#ifdef CONFIG_INCAIP_MUX_PWM2
	return 1;
#else
	return 0;
#endif
}

/**
 *     inca_mux_exin_pins - Available external interrupt pins
 *
 *     Returns a bitfield describing the which external interrupts are 
 *     available
 *     bit 0=EXIN0, bit 1=EXIN1,bit 2=EXIN2,bit 3=EXIN3,bit 4=EXIN4, 
 *     bit 5=EXIN5
 *     0x01=available, 0x00=n/a
 */
u32 inca_mux_exin_pins(void)
{
	return mux_exin_bits;
}

/**
 *     inca_mux_init - Initialize multiplex unit
 *
 *     This function checks wether the CPU is in debug mode and calculates 
 *     the correct return values for the other functions.
 */
void inca_mux_init(void)
{
	u32 mcr = 0;
	u32 altsel = 0;
	u32 pudsel = 0;
	u32 puden = 0;
	
	printk("Inca-IP Multiplex Initialization\n");
	inca_mux_iom2_init();
	inca_mux_asc_init();
	inca_mux_exin_init();
	inca_mux_gpio_init();
	inca_mux_key_init();
	inca_mux_led_init();
	inca_mux_addr_init();
	
	MUX_READ_REG(INCA_IP_Debug_MCD_MCR,mcr);
	if ((mcr & INCAIP_MCD_MCR_DEBUG) > 0) {
		printk("Inca-IP Debug Mode selected.\n");
		mux_debug_mode = 1;
		inca_mux_debug_init(mcr);
	}
	
	/* Check IOM-2 bits on port #0 */
	if ( (mux_iom2_bits & INCAIP_MUX_IOM2_SDS) > 0) {
		altsel |= INCAIP_MUX_P0_10;
	}
	if ( (mux_iom2_bits & INCAIP_MUX_IOM2_BCL) > 0) {
		altsel |= INCAIP_MUX_P0_9;
	}

	/* Check external interrupts on port #0 */
	if ( (mux_exin_bits & INCAIP_MUX_EXIN5) > 0) {
		altsel |= INCAIP_MUX_P0_8;
	}
	if ( (mux_exin_bits & INCAIP_MUX_EXIN4) > 0) {
		altsel |= INCAIP_MUX_P0_7;
	}
	if ( (mux_exin_bits & INCAIP_MUX_EXIN3) > 0) {
		altsel |= INCAIP_MUX_P0_6;
	}
	
	/* Check keypad pins on port #0 */
	if (mux_key == 14) {
		altsel |= INCAIP_MUX_P0_5;
	}
	if (mux_key > 13) {
		altsel |= INCAIP_MUX_P0_4;
	}
	if (mux_key > 12) {
		altsel |= INCAIP_MUX_P0_3;
	}
	if (mux_key > 11) {
		altsel |= INCAIP_MUX_P0_2;
	}
	if (mux_key > 10) {
		altsel |= INCAIP_MUX_P0_1;
	}

	/* Check led pins on port #0 */
	if (mux_led == 10) {
		altsel |= INCAIP_MUX_P0_0;
	}

	/* Store value in P0_ALTSEL */
	MUX_WRITE_REG(INCA_IP_Ports_P0_ALTSEL,altsel);

	altsel = 0;
	/* Check gpio pins on port #1 */
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_17) > 0) {
		altsel |= INCAIP_MUX_P1_17;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_16) > 0) {
		altsel |= INCAIP_MUX_P1_16;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_15) > 0) {
		altsel |= INCAIP_MUX_P1_15;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_14) > 0) {
		altsel |= INCAIP_MUX_P1_14;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_13) > 0) {
		altsel |= INCAIP_MUX_P1_13;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_12) > 0) {
		altsel |= INCAIP_MUX_P1_12;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_11) > 0) {
		altsel |= INCAIP_MUX_P1_11;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_10) > 0) {
		altsel |= INCAIP_MUX_P1_10;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_9) > 0) {
		altsel |= INCAIP_MUX_P1_9;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_8) > 0) {
		altsel |= INCAIP_MUX_P1_8;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_7) > 0) {
		altsel |= INCAIP_MUX_P1_7;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_6) > 0) {
		altsel |= INCAIP_MUX_P1_6;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_5) > 0) {
		altsel |= INCAIP_MUX_P1_5;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_4) > 0) {
		altsel |= INCAIP_MUX_P1_4;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_3) > 0) {
		altsel |= INCAIP_MUX_P1_3;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_2) > 0) {
		altsel |= INCAIP_MUX_P1_2;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_1) > 0) {
		altsel |= INCAIP_MUX_P1_1;
	}
	if ( (mux_gpio1_bits & INCAIP_MUX_P1_0) > 0) {
		altsel |= INCAIP_MUX_P1_0;
	}

	/* Store value in P1_ALTSEL */
	MUX_WRITE_REG(INCA_IP_Ports_P1_ALTSEL,altsel);

	altsel = 0;
	/* Read current value in for Pull Up/Pull Down */
	MUX_READ_REG(INCA_IP_Ports_P2_PUDSEL,pudsel);
	MUX_READ_REG(INCA_IP_Ports_P2_PUDEN,puden);
	/* Check gpio pins on port #2 */
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_15) > 0) {
		altsel |= INCAIP_MUX_P2_15;
		pudsel &= ~INCAIP_MUX_P2_15;
		puden &= ~INCAIP_MUX_P2_15;
	}
	else {
		pudsel |= INCAIP_MUX_P2_15;
		puden |= INCAIP_MUX_P2_15;		
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_14) > 0) {
		altsel |= INCAIP_MUX_P2_14;
		pudsel &= ~INCAIP_MUX_P2_14;
		puden &= ~INCAIP_MUX_P2_14;
	}
	else {
		pudsel |= INCAIP_MUX_P2_14;
		puden |= INCAIP_MUX_P2_14;		
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_13) > 0) {
		altsel |= INCAIP_MUX_P2_13;
		pudsel &= ~INCAIP_MUX_P2_13;
		puden &= ~INCAIP_MUX_P2_13;
	}
	else {
		pudsel |= INCAIP_MUX_P2_13;
		puden |= INCAIP_MUX_P2_13;		
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_12) > 0) {
		altsel |= INCAIP_MUX_P2_12;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_11) > 0) {
		altsel |= INCAIP_MUX_P2_11;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_10) > 0) {
		altsel |= INCAIP_MUX_P2_10;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_9) > 0) {
		altsel |= INCAIP_MUX_P2_9;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_8) > 0) {
		altsel |= INCAIP_MUX_P2_8;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_7) > 0) {
		altsel |= INCAIP_MUX_P2_7;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_6) > 0) {
		altsel |= INCAIP_MUX_P2_6;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_5) > 0) {
		altsel |= INCAIP_MUX_P2_5;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_4) > 0) {
		altsel |= INCAIP_MUX_P2_4;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_3) > 0) {
		altsel |= INCAIP_MUX_P2_3;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_2) > 0) {
		altsel |= INCAIP_MUX_P2_2;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_1) > 0) {
		altsel |= INCAIP_MUX_P2_1;
	}
	if ( (mux_gpio2_bits & INCAIP_MUX_P2_0) > 0) {
		altsel |= INCAIP_MUX_P2_0;
	}

	/* Store value in P2_ALTSEL */
	MUX_WRITE_REG(INCA_IP_Ports_P2_ALTSEL,altsel);

	/* Store value in for Pull Up/Pull Down */
	MUX_WRITE_REG(INCA_IP_Ports_P2_PUDSEL,pudsel);
	MUX_WRITE_REG(INCA_IP_Ports_P2_PUDEN,puden);

	printk("LEDs=%d Addr=%d Keys=%d\n",mux_led,mux_addr,mux_key);
#ifdef CONFIG_INCAIP_MUX_SSC1
	printk("SSC1 available\n");
#else
	printk("SSC1 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_SSC2
	printk("SSC2 available\n");
#else
	printk("SSC2 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_PWM1
	printk("PWM1 available\n");
#else
	printk("PWM1 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_PWM2
	printk("PWM2 available\n");
#else
	printk("PWM2 n/a\n");
#endif
	
}

/**
 *	inca_mux_read_procmem - Create proc file output
 *	@buf:      Buffer to write the string to
 *	@start:    not used (Linux internal)
 *	@offset:   not used (Linux internal)
 *	@count:    not used (Linux internal)
 *	@eof:      Set to 1 when all data is stored in buffer
 *	@data:     not used (Linux internal)
 *
 *	This function creates the output for the multiplexer proc file, by 
 *	reading all appropriate registers and variables and displaying the 
 *	content as a table.
 *
 *  	Return Value:	
 *	@len = Lenght of data in buffer
 */
int inca_mux_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
	long	len = 0;
	int 	t=0;
	u32	bit=0;
	u32	altsel0=0;
	u32	altsel1=0;
	u32	altsel2=0;

	len = sprintf(buf,"\nInca-IP Multiplex Settings\n");

	if (mux_debug_mode == 1) {
		len=len+sprintf(buf+len,"Running in debug mode.\n\n");
	}
	
	len=len+sprintf(buf+len,"         3         2         1         0\n");
	len=len+sprintf(buf+len,"        10987654321098765432109876543210\n");
	len=len+sprintf(buf+len,"----------------------------------------\n");
	len=len+sprintf(buf+len,"Port 0: ");
	MUX_READ_REG(INCA_IP_Ports_P0_ALTSEL,altsel0);
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((altsel0 & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}

	len=len+sprintf(buf+len,"\nPort 1: ");
	MUX_READ_REG(INCA_IP_Ports_P1_ALTSEL,altsel1);
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((altsel1 & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}

	len=len+sprintf(buf+len,"\nPort 2: ");
	MUX_READ_REG(INCA_IP_Ports_P2_ALTSEL,altsel2);
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((altsel2 & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}

	len=len+sprintf(buf+len,"\nIOM-2:  ");
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((mux_iom2_bits & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}

	len=len+sprintf(buf+len,"\nASC:    ");
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((mux_asc_bits & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}

	len=len+sprintf(buf+len,"\nEXIN:   ");
	bit = 0x80000000;
	for (t=0;t<32;t++) {
		if ((mux_exin_bits & bit) > 0) len=len+sprintf(buf+len,"X");
		else len=len+sprintf(buf+len," ");
		bit = bit >> 1;
	}
	
	len=len+sprintf(buf+len,"\n");
	len=len+sprintf(buf+len,"LEDs=%d Addr=%d Keys=%d\n",mux_led,mux_addr,mux_key);
#ifdef CONFIG_INCAIP_MUX_SSC1
	len=len+sprintf(buf+len,"SSC1 available\n");
#else
	len=len+sprintf(buf+len,"SSC1 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_SSC2
	len=len+sprintf(buf+len,"SSC2 available\n");
#else
	len=len+sprintf(buf+len,"SSC2 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_PWM1
	len=len+sprintf(buf+len,"PWM1 available\n");
#else
	len=len+sprintf(buf+len,"PWM1 n/a\n");
#endif
#ifdef CONFIG_INCAIP_MUX_PWM2
	len=len+sprintf(buf+len,"PWM2 available\n");
#else
	len=len+sprintf(buf+len,"PWM2 n/a\n");
#endif

	len=len+sprintf(buf+len,"\n");

	*eof = 1;	// No more data available
	return len;
}


/**
 *	inca_mux_init_proc - Initialize mux proc file
 *
 *	This function will create the proc file entry.
 *
 *  	Return Value:	
 *	@0 = OK
 */
int __init inca_mux_init_proc(void)
{
	//printk("------------ Initializing Inca-IP MUX proc file\n");
	create_proc_read_entry("driver/inca_mux", 0, NULL, inca_mux_read_procmem, NULL);
	return 0;
}


module_init(inca_mux_init_proc);

EXPORT_SYMBOL(inca_mux_addr_pins);
EXPORT_SYMBOL(inca_mux_led_pins);
EXPORT_SYMBOL(inca_mux_key_pins);
EXPORT_SYMBOL(inca_mux_gpio_pins);
EXPORT_SYMBOL(inca_mux_asc_pins);
EXPORT_SYMBOL(inca_mux_ssc1_pins);
EXPORT_SYMBOL(inca_mux_ssc2_pins);
EXPORT_SYMBOL(inca_mux_iom_pins);
EXPORT_SYMBOL(inca_mux_pwm1_pins);
EXPORT_SYMBOL(inca_mux_pwm2_pins);
EXPORT_SYMBOL(inca_mux_exin_pins);



