/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   File        : $RCSfile: INCAIPOAKcommand3.c,v $
   Revision    : $Revision: 1.4 $
   Date        : $Date: 2004/02/19 07:24:24 $
   Description : 

******************************************************************************/


#ifdef NEVER
/****************************************************************************
Description :
     This function will write data into AOK memory. There is no difference
     between program or data memory transfer. A normal transfer is only valid
     for one Oak instance. 
     An extended transfer writes data to all Oaks. Thus the confirmation
     handling is different. 
     A single transfer is acknowledged by a single message. 
     An extended transfer by several messages in arbitrary order. The Oak
     instances expected to answer are selected by the channel mask. This 
     32 bit value carries a '1' at the channel numbers position e.g. OAK1
     has the channel number 10h and thus it's mask would be 0x00010000.
     The mask for all Oaks is 0x000f0000.
Arguments   :
Return      :
Remarks     : Each incomming message has a message id derived during runtime. 
     This message id has to be masked out before comparison.
****************************************************************************/
LOCAL STATUS WriteMemory(DWNLD *pDwnLd, OAK_MB_DEV_STR *pMBDev, void **p)
{
   UINT32   i, numwords, addr, len;
   UINT32   Msg[MAX_MSG_LNG_32BIT];
   UINT32   *pData;
   UINT32   *ptr;
   UINT32   incoming[MAX_MSG_LNG_32BIT];
   INT32    err;
   UINT32   ackn;
   UINT32   cnt = 0;

   /*generate global copy of command data*/
   pDwnLd->write_mem.true_ptr   = Read_uI32(p);
   pDwnLd->write_mem.false_ptr  = Read_uI32(p);
   pDwnLd->write_mem.start_addr = Read_uI32(p);
   len                          = Read_uI32(p);
   pDwnLd->write_mem.len        = len;

   ptr = pDwnLd->write_mem.Data;
   for(i = 0; i < len; i += 2)
   {
      *ptr++ = Read_uI32(p);
   }

   /*generate local copy of command data*/
   numwords             = pDwnLd->glob_var.maxlen-3;
   addr                 = (UINT16)(pDwnLd->write_mem.start_addr & 0x0000FFFF);
   pData                = pDwnLd->write_mem.Data;

   ackn = pDwnLd->write_mem.ch_mask;
   for(i=0; i<32; i++)
   {
      cnt += (ackn & 0x1);
      ackn >>= 1;
   }

   if (len == 0)
   err = (OK);    /* No data to send, so return with success */
   else
   err = (ERROR);


   /*send messages*/
   while(len > 0)
   {
      /*calc. actual length of payload data*/
      if(len < numwords)
         numwords = len;

      /*build message array & and send message*/
      Msg[0] = (pDwnLd->write_mem.cmd << 16) | numwords ;
      Msg[1] = addr;
      for(i=3; i<(numwords+3); i++) Msg[i] = *pData++;
      err = Prim_SendMessage(pDwnLd, pMBDev, Msg,(UINT16)(numwords+3));

      if(err != (OK))
      {
         err = TIMEOUT;
         break;
      }

      /*update message stat.*/
      len  -= numwords;
      addr += numwords;

      /*wait for receive confirmation*/
      err = (OK);
      if((pDwnLd->block_hdr.block_type == FLAG_EP) ||
         (pDwnLd->block_hdr.block_type == FLAG_ED))
      {
         /*For the next cnt messages do*/
         ackn = 0;
         for(i=cnt; i>0; i--)
         {
            if (Prim_ReadMessage(pDwnLd, pMBDev, incoming, 128,
               pDwnLd->glob_var.timeout) <= 0)
            {
               /*test for timeout*/
               err = TIMEOUT;
               break;
            }

            /*test for message type*/
            if((*incoming && TYPE_MASK) == (*Msg && TYPE_MASK))
            {
               ackn |= 0x01 << (*incoming & CHANNEL_MASK) ;          
            }                 
         }

         if(err == TIMEOUT)
         break;

         if(ackn != pDwnLd->write_mem.ch_mask)
         {
            err = (ERROR);
            break;
         }
      }
      else
      {
         if (Prim_ReadMessage(pDwnLd, pMBDev, incoming, 128,
            pDwnLd->glob_var.timeout)
            <= 0)
         {
            /*test for timeout*/
            err = TIMEOUT;
            break;
         }

         /*test for correct response command*/        
         *incoming &= ~ID_MASK;
         if(*Msg != *incoming)
         {
            err = (ERROR);
            break;
         }
      }
   }
   return err;
}
#endif /* NEVER */

