/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   File        : $RCSfile: INCAOAK_DeviceDriver.c,v $
   Revision    : $Revision: 1.5 $
   Date        : $Date: 2004/06/24 08:44:39 $
   Description : Generic driver module for the INCA-IP OAK DSP

******************************************************************************/
#include "drv_api.h"
#include "INCAOAK_DeviceDriver.h"
#include "INCAOAK_DeviceDriver_Ext.h"
#include "RegIncaIP.h"

#ifdef VXWORKS
#include <common/src/sys_drv_debug.h>
#endif /* VXWORKS */
   
#ifdef LINUX
#include <common/src/sys_debug.h>
#endif /* LINUX */

#define OAK_INTERN
//#define INCA_GIVE_DSP_FILL_LEVEL_WARNING

/* Declarations for debug interface */
DECLARE_TRACE_GROUP(INCA_OAK_DRV)
DECLARE_LOG_GROUP(INCA_OAK_DRV)

/*----------------------------------------------------------------------------*/
/* Function Declaration which are required by the firmware download feature   */
/*----------------------------------------------------------------------------*/
OAK_INTERN int oak_write_cmd_mailbox(register OAK_MB_DEV_STR *pMBDev, 
   OAK_READWRITE *readWrite);
   
OAK_INTERN int oak_read_fifo(OAK_MB_DEV_STR  *pMBDev, 
   OAK_READWRITE   *pPkg, 
   INT32           timeout);
   
OAK_INTERN int oak_restart_dsp(OAK_GLOBAL_REG *pBaseAddress);
OAK_INTERN int oak_reset_dsp(OAK_GLOBAL_REG *pBaseAddress);
int oak_jump0_dsp(OAK_MB_DEV_STR *pMBDev);
 
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
#include "INCAIPOAKdownload.h"
#include "INCAIPOAKmemaccess.c"
#include "INCAIPOAKprimitives.c"
#include "INCAIPOAKcommand0.c"
#include "INCAIPOAKcommand1.c"
#include "INCAIPOAKcommand3.c"
#include "INCAIPOAKcommand2.c"
#include "INCAIPOAKcommand4.c"
#include "INCAIPOAKdownload.c"

/*----------------------------------------------------------------------------*/
/* Mask bit for the downstream voice package firmware header                  */
/*----------------------------------------------------------------------------*/
#define OAK_VOICE0_AND_MASK   0xFFFEFFFF
#define OAK_VOICE0_OR_MASK    0x80000000
#define OAK_VOICE1_AND_MASK   0xFFFFFFFF
#define OAK_VOICE1_OR_MASK    0x80010000
/*----------------------------------------------------------------------------*/

#define OAK_WRITE_MB_TIMEOUT  10

int debug_oak_int=0;

#ifdef VXWORKS
extern OAK_DEV  *pOakDev;
OAK_DEV *pOAKDev=pOakDev;
#else
extern OAK_DEV inca_oak_dev_struct;
OAK_DEV *pOAKDev=&inca_oak_dev_struct;
#endif

/****************************************************************************
  FIFO support functions
****************************************************************************/

/****************************************************************************
Description : Returns the size in bytes free to write in the fifo  
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
UINT32 oak_fifo_write_space(OAK_FIFO_MB *pFifo)
{
   volatile UINT32 pRead = (UINT32)pFifo->pRead;
   volatile UINT32 pWrite = (UINT32)pFifo->pWrite;

   if (pRead > pWrite)
   {
      return (pRead - pWrite);
   }
   else
   {
      return (pFifo->FifoSize - (pWrite - pRead));
   }
}

/****************************************************************************
Description : 
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
void oak_fifo_read_advance(OAK_FIFO_MB *pFifo, UINT32 count)
{
   volatile UINT32 pRead = (UINT32)pFifo->pRead;
   volatile UINT32 pEnd = (UINT32)pFifo->pEnd;

   if ((pEnd - pRead)/4 > count) 
   {
      pFifo->pRead += count;
   }
   else
   {
      pFifo->pRead = pFifo->pStart + ( count - (pEnd - pRead)/4);
   }
}

/****************************************************************************
Description : 
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
UINT32 oak_fifo_read_check_split(OAK_FIFO_MB *pFifo, UINT32 count)
{
   if ((pFifo->pRead + count) > pFifo->pEnd) 
   {
      return 1;
   }
   else
   {
      return 0;
   }
}

/****************************************************************************
Description : 
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
UINT32 oak_fifo_read_data_available(OAK_FIFO_MB *pFifo)
{
   volatile UINT32 pRead = (UINT32)pFifo->pRead;
   volatile UINT32 pWrite = (UINT32)pFifo->pWrite;

   if (pRead == pWrite) 
   {
      return 0;
   }
   else
   {
      if (pRead < pWrite)
      {
         return (pWrite - pRead);
      }
      else
      {
         return (pFifo->FifoSize - (pRead - pWrite));
      }
   }
}

/****************************************************************************
Description : 
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
void oak_fifo_read_packet(OAK_FIFO_MB *pFifo, UINT32 count, UINT32 *buf)    
{
    UINT32 left = count;
    UINT32 cur;
    volatile UINT32 *pRead = pFifo->pRead;


    if (oak_fifo_read_check_split(pFifo, count))
    { 
       cur = ((UINT32)pFifo->pEnd - (UINT32)pFifo->pRead)/4;
    }
    else
    {
      cur = count; 
    }
    left -= cur;
    for (; cur > 0; cur--)
    {
       *(buf++) = *(pRead++);
    }
    if (left)
    {
       pRead = pFifo->pStart;
       for (; left > 0; left--)
       {
          *(buf++) = *(pRead++);
       }
    }
}


/****************************************************************************
Description : Clear Fifos in case of a reset of the mailbox
Arguments   : Handle to the fifo parameter structure
Return      :
Remarks     :
****************************************************************************/
void oak_fifo_clear(OAK_FIFO_MB *pFifo)
{
   pFifo->pRead  = pFifo->pStart;
   pFifo->pWrite = pFifo->pStart;
}


/****************************************************************************
  Common maintenance routines
****************************************************************************/

/****************************************************************************
Description : Open routine for the device driver
Arguments   :
Return      :
Remarks     :
****************************************************************************/
OAK_INTERN INT32 oak_common_open(OAK_DEV *pDev, 
                           OAK_MB_DEV_STR *pMBDev, 
                           BOOL bcommand)
{
   OAK_GLOBAL_REG *pBaseGlobal;

   /* device is already installed */
   if (pMBDev->Installed == TRUE)
   {
#ifdef VXWORKS
      errno = S_iosLib_OAKMB_DEV_INST ;
#endif /* VXWORKS */
      return (ERROR);
   }

   pMBDev->Installed = TRUE;

   /* enable interrupts */
   if (bcommand == TRUE)
   {
      /* command connection */
      OAK_INTERRUPTS_ENABLE(EXC_INCAIP_DSP_B3DAIR);
   }
   else
   {
      /* voice channel connection */
      OAK_INTERRUPTS_ENABLE(EXC_INCAIP_DSP_B2DAIR | EXC_INCAIP_DSP_SCIR0);
   }

   /* enable the IRQ mask in the DSP mailbox register */
   pBaseGlobal = pDev->pBaseGlobal;
   pBaseGlobal->MBC_MSK   &= ~pMBDev->MBC_Mask;
   pBaseGlobal->MBC_MSK01 &= ~pMBDev->MBC_Mask01;
   pBaseGlobal->MBC_MSK10 &= ~pMBDev->MBC_Mask10;
   
   return (OK);
}


/****************************************************************************
Description : Close routine for the device driver
Arguments   :
Return      :
Remarks     :
****************************************************************************/
INT32 oak_common_close(OAK_MB_DEV_STR *pMBDev)
{
   OAK_DEV        *pOAKDev;
   OAK_GLOBAL_REG *pBaseGlobal;


   pMBDev->Installed = FALSE;

   pBaseGlobal = pMBDev->pBaseGlobal;

   /* Clear the downstream queues for voice fds only*/
   if(pMBDev->devID != command)
   {
      /* voice channel connection */

      /* look if there is still a connection open */
      pOAKDev = pMBDev->pOAK_DEV;
      if ((pOAKDev->VoiceMB[0].Installed == FALSE) &&
         (pOAKDev->VoiceMB[1].Installed == FALSE))
      {
         /* no other connection is open */
         /* disable the interrupts */
         OAK_INTERRUPTS_DISABLE(EXC_INCAIP_DSP_B2DAIR | EXC_INCAIP_DSP_B0EIR |
            EXC_INCAIP_DSP_SCIR0);
         /* disable the IRQ mask in the DSP mailbox register */
         pBaseGlobal->MBC_MSK |= pMBDev->MBC_Mask;
      }

      /* Clear the downstream queue */
      oak_fifo_clear(&pMBDev->dwstrm_fifo);

      pMBDev->VoiceChannelStream = 0;
   }
   else
   {
      /* command channel connection */
      /* disable the interrupt for the command and the events */
      OAK_INTERRUPTS_DISABLE(EXC_INCAIP_DSP_B3DAIR | EXC_INCAIP_DSP_B1EIR
         | EXC_INCAIP_DSP_WDTIR | EXC_INCAIP_DSP_SCIR8);

      /* disable the IRQ mask in the DSP mailbox register */
      pBaseGlobal->MBC_MSK |= pMBDev->MBC_Mask;
  }

   /* Clear the upstream queue */
   oak_fifo_clear(&pMBDev->upstrm_fifo);

  /* disable the IRQ mask in the DSP mailbox register */
   pBaseGlobal->MBC_MSK01 |= pMBDev->MBC_Mask01;
   pBaseGlobal->MBC_MSK10 |= pMBDev->MBC_Mask10;

   return (OK);
}

/****************************************************************************
Description :
   Restart the DSP at Internal Low Address, where the DSP code is located  
Arguments   :
   BaseAddress of the DSP
Return      :
Remarks     :
****************************************************************************/
int oak_jump0_dsp(OAK_MB_DEV_STR *pMBDev)
{
   OAK_GLOBAL_REG *pBaseAddress;
   UINT32   j = 0;
   volatile UINT32 val;
   
   pBaseAddress=pOAKDev->pBaseGlobal;
   
   OAK_READWRITE  rw;
   static UINT32     ui_workaround[] = {0x0600FE02, 0};

   /* reset the DSP again to jump into the code */
   pBaseAddress->MBC_DCTRL = 0;

   rw.pWrite = ui_workaround;
   rw.nWrite = sizeof(ui_workaround);
   rw.nRead    = 0;
   rw.pRead    = NULL;
   rw.nMsgRead = 0;

   /* set PC to 0 */
   if (oak_write_cmd_mailbox(pMBDev, &rw) == (ERROR))
      return (ERROR);
   Wait(40);

   /* reset the mailboxes of the DSP and enable the swap unit */
   pBaseAddress->MBC_CFG = OAK_MBC_CFG_SWAP2 | OAK_MBC_CFG_RES;
 
   /* wait until the mailbox is reset */
   j = 0;
   do
   {
#ifdef VXWORKS
      if (++j < sysClkRateGet())
#elif defined(LINUX)
      if (++j < 100)
#else
#error No OS parameter selected
#endif /* VXWORKS */
      {
         Wait(10);
      } else {
         return (ERROR);
      }
      val = pBaseAddress->MBC_CFG;
   } while (val & OAK_MBC_CFG_RES);

   return (OK);
}


/****************************************************************************
Description :
   Restart the DSP at Internal Low Address, where the DSP code is located  
Arguments   :
   BaseAddress of the DSP
Return      :
Remarks     :
****************************************************************************/
int oak_restart_dsp(OAK_GLOBAL_REG *pBaseAddress)
{
   UINT32   j = 0;
   volatile UINT32 val;
   
   /* reset the DSP again to jump into the code */
   pBaseAddress->MBC_DCTRL = 0;
   pBaseAddress->MBC_DCTRL = OAK_MBC_DCTRL_RES;

   /* wait till the reset is finished */
   do
   {
#ifdef VXWORKS
      if (++j < sysClkRateGet())
#else
      if (++j < 100)
#endif /* VXWORKS */
     {
         Wait(10);
      } else {
         return (ERROR);
      }
      val = pBaseAddress->MBC_DCTRL;
   } while (val & OAK_MBC_DCTRL_RES);

   /* reset the mailboxes of the DSP and enable the swap unit */
   pBaseAddress->MBC_CFG = OAK_MBC_CFG_SWAP2 | OAK_MBC_CFG_RES;
 
   /* wait till the mailbox is reseted */
   j = 0;
   do
   {
#ifdef VXWORKS
      if (++j < sysClkRateGet())
#elif defined(LINUX)
      if (++j < 100)
#else
#error No OS parameter selected
#endif /* VXWORKS */
      {
         Wait(10);
      } else {
         return (ERROR);
      }
      val = pBaseAddress->MBC_CFG;
   } while (val & OAK_MBC_CFG_RES);
   
   return (OK);
}

/****************************************************************************
Description :
   Resets the DSP at Internal Download Address
Arguments   :
   BaseAddress of the DSP
Return      :
Remarks     :
****************************************************************************/
OAK_INTERN int oak_reset_dsp(OAK_GLOBAL_REG *pBaseAddress)
{
   UINT32   j = 0;
   volatile UINT32 val;
   
   /* enable the clock for the DSP module */
#ifdef VXWORKS
   sysIncaHWRegSetBit(INCA_IP_PMU_PM_GEN, INCA_IP_PMU_PM_GEN_EN7);
#else
   /* TODO: Make this atomic... */
   *((volatile UINT32*)INCA_IP_PMU_PM_GEN) |= INCA_IP_PMU_PM_GEN_EN7;
#endif

   /* soft reset the DSP to bring up in download mode */
   pBaseAddress->MBC_DCTRL = OAK_MBC_DCTRL_BA | OAK_MBC_DCTRL_BMOD;
   pBaseAddress->MBC_DCTRL = OAK_MBC_DCTRL_BA | OAK_MBC_DCTRL_BMOD |
      OAK_MBC_DCTRL_RES;

   /* wait till the reset is finished */
   do
   {
#ifdef VXWORKS
      if (++j < sysClkRateGet())
#else
      if (++j < 100)
#endif /* VXWORKS */
     {
         Wait(10);
      } else {
         return (ERROR);
      }
      val = pBaseAddress->MBC_DCTRL;
   } while (val & OAK_MBC_DCTRL_RES);

   /* reset the mailboxes of the DSP and enable the swap unit */
   pBaseAddress->MBC_CFG = OAK_MBC_CFG_SWAP2 | OAK_MBC_CFG_RES;

   /* wait till the mailbox is reseted */
   j = 0;
   do
   {
#ifdef VXWORKS
      if (++j < sysClkRateGet())
#elif LINUX
      if (++j < 100)
#else
#error No OS parameter selected
#endif /* VXWORKS */
      {
         Wait(10);
      } else {
         return (ERROR);
      }
      val = pBaseAddress->MBC_CFG;
   } while (val & OAK_MBC_CFG_RES);
   
   return (OK);
}

/****************************************************************************
Description :
   Initialize and reboot the DSP  
Arguments   :
Return      :
Remarks     :
****************************************************************************/
void OAK_Drv(void)
{
   /* disable the interrupt of the ICU */
   /* disable IM1_IER register for bits IRL20 to IRL26 */
   OAK_INTERRUPTS_DISABLE(0x07F00000);
}

#ifdef LINUX
/****************************************************************************
Description : Release of the device driver structure elements
Arguments   : pDev: Handle of the device driver structure
Return      : Zero is no error, otherwise a negative value
Remarks     :
****************************************************************************/
void oak_release_structures(OAK_DEV *pDev)
{
   int      count;
   /* Initialize the Message queues for the voice packets */
   for( count=0 ; count < NUM_VOICE_CHANNEL ; count++)
   {
      Free_Mem(pDev->VoiceMB[count].upstrm_fifo.pStart);
      Free_Mem(pDev->VoiceMB[count].dwstrm_fifo.pStart);

      Sem_Free(pDev->VoiceMB[count].sem_dev);
      Sem_Free(pDev->VoiceMB[count].sem_read_fifo);

#ifdef OAK_FIFO_BLOCKING_WRITE
      Sem_Free(pDev->VoiceMB[count].sem_write_fifo);
#endif /* OAK_FIFO_BLOCKING_WRITE */
  }

   Free_Mem(pDev->CommandMB.upstrm_fifo.pStart);
   Free_Mem(pDev->CommandMB.dwstrm_fifo.pStart);      

   Sem_Free(pDev->CommandMB.sem_dev);
   Sem_Free(pDev->CommandMB.sem_read_fifo);
}
#endif /* LINUX */

/****************************************************************************
Description : Initialization of the device driver structure elements
Arguments   : pDev: Handle of the device driver structure
Return      : Zero is no error, otherwise a negative value
Remarks     :
****************************************************************************/
OAK_INTERN UINT32 oak_init_structures(OAK_DEV *pDev)
{
   int      count;
   UINT32   *ui32ptr;

   memset((void*)pDev, 0, sizeof(OAK_DEV));

   /* global DMA base pointer for the ISR */
   pDev->pBaseGlobal = (OAK_GLOBAL_REG*) OAK_BASEADDRESS;
   pDev->flags       = 0x00000000;

   /* Initialize the Message queues for the voice packets */
   for( count=0 ; count < NUM_VOICE_CHANNEL ; count++)
   {
      pDev->VoiceMB[count].Installed = FALSE ;

      /* Upstream Voice Buffers*/
      ui32ptr = Alloc_Mem(sizeof(UINT32) * MAX_FIFO_VOICE);
      if (ui32ptr == NULL)
         return (ERROR);

      pDev->VoiceMB[count].upstrm_fifo.pStart    = ui32ptr;
      pDev->VoiceMB[count].upstrm_fifo.pEnd      = ui32ptr + MAX_FIFO_VOICE;
      pDev->VoiceMB[count].upstrm_fifo.pWrite    = ui32ptr;
      pDev->VoiceMB[count].upstrm_fifo.pRead     = ui32ptr;
      pDev->VoiceMB[count].upstrm_fifo.FifoSize  =
         sizeof(UINT32) * MAX_FIFO_VOICE;

      /* Downstream Voice Buffers*/
      ui32ptr = Alloc_Mem(sizeof(UINT32) * MAX_FIFO_VOICE);
      if (ui32ptr == NULL)
         return (ERROR);

      pDev->VoiceMB[count].dwstrm_fifo.pStart    = ui32ptr;
      pDev->VoiceMB[count].dwstrm_fifo.pEnd      = ui32ptr + MAX_FIFO_VOICE;
      pDev->VoiceMB[count].dwstrm_fifo.pWrite    = ui32ptr;
      pDev->VoiceMB[count].dwstrm_fifo.pRead     = ui32ptr;
      pDev->VoiceMB[count].dwstrm_fifo.FifoSize  =
         sizeof(UINT32) * MAX_FIFO_VOICE;

      pDev->VoiceMB[count].pBaseGlobal = (OAK_GLOBAL_REG*) OAK_BASEADDRESS;

      /* set the global pointer reference in the substructure */
      pDev->VoiceMB[count].pOAK_DEV = pDev;

      /* initialize devices with control addresses of the voice mailbox */
      pDev->VoiceMB[count].pBaseWriteMB =
         (OAK_RW_MB*) &(pDev->VoiceMB[count].pBaseGlobal)->Reserved[0];

      pDev->VoiceMB[count].pBaseReadMB =
         (OAK_RW_MB*) &(pDev->VoiceMB[count].pBaseGlobal)->Reserved[2];

      pDev->VoiceMB[count].down_callback = NULL;
      pDev->VoiceMB[count].up_callback = NULL;

      /* initialize the semaphores for multitasking access */
      Sem_BinaryInit(pDev->VoiceMB[count].sem_dev);

      /* initialize the semaphores to read from the fifo */
      Sem_BinaryInit(pDev->VoiceMB[count].sem_read_fifo);
      Sem_Lock(pDev->VoiceMB[count].sem_read_fifo);

#ifdef OAK_FIFO_BLOCKING_WRITE
      Sem_BinaryInit(pDev->VoiceMB[count].sem_write_fifo);
      Sem_Lock(pDev->VoiceMB[count].sem_write_fifo);
#endif /* OAK_FIFO_BLOCKING_WRITE */

      if(pDev->VoiceMB[count].sem_dev == NULL)
         return (ERROR);

      /* select mechanism implemented for each queue */
      Init_WakeupQueue(pDev->VoiceMB[count].oak_wklist);
   }

   /* the packet identifier for the command data */
   pDev->VoiceMB[0].devID = voice0;
   pDev->VoiceMB[1].devID = voice1;
   pDev->CommandMB.devID  = command;

   /* configure the AND- and OR- mask for the downstream voice packages */
   pDev->VoiceMB[0].HeaderANDMask = OAK_VOICE0_AND_MASK;
   pDev->VoiceMB[0].HeaderORMask  = OAK_VOICE0_OR_MASK;
   pDev->VoiceMB[1].HeaderANDMask = OAK_VOICE1_AND_MASK;
   pDev->VoiceMB[1].HeaderORMask  = OAK_VOICE1_OR_MASK;

#ifdef OAK_FIFO_BLOCKING_WRITE
   /* set the default parameter for the transmit FIFO blocking feature */
   pDev->VoiceMB[0].bBlockWriteMB = TRUE;
   pDev->VoiceMB[1].bBlockWriteMB = TRUE;
#endif /* OAK_FIFO_BLOCKING_WRITE */

   /* setup the interrupt mask for the device */
   pDev->VoiceMB[0].MBC_Mask   = OAK_MBC_ISR_B2DA;
   pDev->VoiceMB[0].MBC_Mask01 = OAK_MBC_ISR_DEC1_REQ;
   pDev->VoiceMB[0].MBC_Mask10 = OAK_MBC_ISR_DEC1_REQ;
   pDev->VoiceMB[1].MBC_Mask   = OAK_MBC_ISR_B2DA;
   pDev->VoiceMB[1].MBC_Mask01 = OAK_MBC_ISR_DEC2_REQ;
   pDev->VoiceMB[1].MBC_Mask10 = OAK_MBC_ISR_DEC2_REQ;
   pDev->CommandMB.MBC_Mask    = OAK_MBC_ISR_B3DA;

   /* set the global pointer reference in the substructure */
   pDev->CommandMB.pOAK_DEV = pDev;

   pDev->CommandMB.Installed = FALSE ;

   /* upstream command queue */
   ui32ptr = Alloc_Mem(sizeof(UINT32) * MAX_FIFO_CMD);
   if (ui32ptr == NULL)
      return (ERROR);

   pDev->CommandMB.upstrm_fifo.pStart    = ui32ptr;
   pDev->CommandMB.upstrm_fifo.pEnd      = ui32ptr + MAX_FIFO_CMD;
   pDev->CommandMB.upstrm_fifo.pWrite    = ui32ptr;
   pDev->CommandMB.upstrm_fifo.pRead     = ui32ptr;
   pDev->CommandMB.upstrm_fifo.FifoSize  = sizeof(UINT32) * MAX_FIFO_CMD;
   
   /* downstream command queue. NOTE : Not required */
   ui32ptr = Alloc_Mem(sizeof(UINT32) * MAX_FIFO_CMD);
   if (ui32ptr == NULL)
      return (ERROR);

   pDev->CommandMB.dwstrm_fifo.pStart    = ui32ptr;
   pDev->CommandMB.dwstrm_fifo.pEnd      = ui32ptr + MAX_FIFO_CMD;
   pDev->CommandMB.dwstrm_fifo.pWrite    = ui32ptr;
   pDev->CommandMB.dwstrm_fifo.pRead     = ui32ptr;
   pDev->CommandMB.dwstrm_fifo.FifoSize  = sizeof(UINT32) * MAX_FIFO_CMD;

   /* initialize the basepointer of the mailboxes and the global register */
   pDev->CommandMB.pBaseGlobal = (OAK_GLOBAL_REG*) OAK_BASEADDRESS;

   pDev->CommandMB.down_callback = NULL;
   pDev->CommandMB.up_callback = NULL;

   /* initialize devices with control addresses of the command mailbox */
   pDev->CommandMB.pBaseWriteMB  =
      (OAK_RW_MB*) &(pDev->CommandMB.pBaseGlobal)->Reserved[1];

   pDev->CommandMB.pBaseReadMB =
      (OAK_RW_MB*) &(pDev->CommandMB.pBaseGlobal)->Reserved[3];

   /* initialize the select mechanism */
   Init_WakeupQueue(pDev->CommandMB.oak_wklist);

   /* initialize the semaphores for multitasking access */
   Sem_BinaryInit(pDev->CommandMB.sem_dev);

   if(pDev->CommandMB.sem_dev == NULL)
      return (ERROR) ;

   /* initialize the semaphores to read from the fifo */
   Sem_BinaryInit(pDev->CommandMB.sem_read_fifo);
   Sem_Lock(pDev->CommandMB.sem_read_fifo);
   
   return 0;   
}


/****************************************************************************
  Data routines
****************************************************************************/

/****************************************************************************
Description : Write from Queue to Downstream mailbox
Arguments   :
Return      :
Remarks     :
****************************************************************************/
LOCAL void oak_downstream_voice(OAK_MB_DEV_STR *pMBDev)
{
   UINT32         *pRead;
   UINT32         count8, count16, count32;
   OAK_FIFO_MB    *fifo;
   OAK_GLOBAL_REG *pBaseGlobal;
   OAK_READWRITE  rw;
   UINT32         split = 0;

   fifo = &pMBDev->dwstrm_fifo;

   pRead = fifo->pRead;

#ifdef CONFIG_PROC_FS
   /* increase the counter of interrupts */
   pMBDev->TxnumIRQs++;
#endif /* CONFIG_PROC_FS */

   /* Check if there is no data in the Downstream FIFO to Read from */
   if(!oak_fifo_read_data_available(fifo))
   {
#ifdef CONFIG_PROC_FS
      pMBDev->TxnumMiss++;
#endif
      pMBDev->errnum = S_iosLib_OAKMB_FIFO_DAI;
      pMBDev->VoiceMissingPackage = TRUE;
      goto OAK_WRITEVOICE_ISR_ERROR;
   }

   /* Calculate number of words including the header */
   count16 = (*pRead & 0xff) + 2;
   count32 = (count16 +1) >> 1;
   count8 = count16 << 1;

#ifdef CONFIG_PROC_FS
   pMBDev->TxnumBytes += count8;
#endif

   if ((split = oak_fifo_read_check_split(fifo, count32)) == 1)
   {
      rw.pWrite = Alloc_Mem(count32*4);
      oak_fifo_read_packet(fifo, count32, rw.pWrite);     
   }
   else
   {
      rw.pWrite = pRead;
   }
   rw.nWrite = count8;
   if (oak_write_voice_mailbox(pMBDev, &rw ) == ERROR)
   {
      pMBDev->errnum =  S_iosLib_OAKMB_MB_WRI;
#ifdef CONFIG_PROC_FS
      pMBDev->TxnumBytes -= count8;
#endif
      goto OAK_WRITEVOICE_ISR_MB_ERROR;
   }
   if (split) 
   {
      Free_Mem(rw.pWrite);
   }
   oak_fifo_read_advance(fifo, count32);

#ifdef OAK_FIFO_BLOCKING_WRITE
   /* Notify task that is waiting for FIFO - if any... */
// TODO: Why not unlock the sem every time?
//   if (TRUE == pMBDev->full_write_fifo)
   {
      pMBDev->full_write_fifo = FALSE;
      Sem_Unlock(pMBDev->sem_write_fifo);
   }
#endif /* OAK_FIFO_BLOCKING_WRITE */
   
   Wake_Down_Queue(pMBDev->oak_wklist);
   return;

OAK_WRITEVOICE_ISR_MB_ERROR:
   if (split) 
   {
      Free_Mem(rw.pWrite);
   }
   return;
OAK_WRITEVOICE_ISR_ERROR:
   if (split) 
   {
      Free_Mem(rw.pWrite);
   }
   /* Disable the interrupts, because we can't service now */
   pBaseGlobal = pMBDev->pBaseGlobal;
   pBaseGlobal->MBC_MSK01 |= pMBDev->MBC_Mask01;
   pBaseGlobal->MBC_MSK10 |= pMBDev->MBC_Mask01;
   return;
}

/****************************************************************************
Description : Write from Task to Downstream queue
Arguments   :
Return      :
Remarks     :
****************************************************************************/
OAK_INTERN int oak_write_fifo(OAK_MB_DEV_STR* pMBDev, OAK_READWRITE *pPkg)
{
   UINT16         count8, count16, count32;
   UINT32         *pSource, *pWrite;
   UINT32         maxNumBytes;
   OAK_FIFO_MB    *fifo;
   OAK_GLOBAL_REG *pBaseGlobal;
   Lock_Variable  irqLockNum;

   /* source addr of the packets */
   pSource     = pPkg->pWrite;
   maxNumBytes = pPkg->nWrite;

   fifo = &pMBDev->dwstrm_fifo;
   pWrite      = pMBDev->dwstrm_fifo.pWrite;

   pPkg->nMsgRead = 0 ; /* total number of packets written to the queue */
   pPkg->nWrite = 0 ; /* total number of bytes written to the queue */

   while(pPkg->nWrite < maxNumBytes)
   {
      /* Number of words including the header */
      count16 = (*pSource & 0xff) + 2 ; /* Num of halfwords (16 bit)*/
      count8 = count16 << 1;

      /* Check if maximum bytes to be read are not exceeded */
      if((count8 + pPkg->nWrite ) > maxNumBytes )
      {
         return (ERROR);
      }
#ifdef OAK_FIFO_BLOCKING_WRITE
OAK_WRITE_FIFO_CHECKFIFOSPACE:
#endif /* OAK_FIFO_BLOCKING_WRITE */
      if (count8 >= oak_fifo_write_space(fifo))
#ifdef OAK_FIFO_BLOCKING_WRITE
      {
         if (TRUE == pMBDev->bBlockWriteMB)
         {
            /* wait till somebody read from the fifo */
            pMBDev->full_write_fifo = TRUE;

            /* wait till somebody took something from the fifo */
            Sem_Lock(pMBDev->sem_write_fifo);

            goto OAK_WRITE_FIFO_CHECKFIFOSPACE;
         }
         else
            break;
      }
#else
      {
         return(ERROR);
      }
#endif /*  OAK_FIFO_BLOCKING_WRITE */

      count32 = (count16 + 1) >> 1;      
      while(count32)
      {
         *pWrite++ = *pSource++;

         if(pWrite == fifo->pEnd)
            pWrite = fifo->pStart;

         count32--;
      }

      /* update the write pointer */
      pMBDev->dwstrm_fifo.pWrite = pWrite;

      pPkg->nWrite += count8;

      /* check if the mailbox is waiting for data */
      if (TRUE == pMBDev->VoiceMissingPackage)
      {
         /* disable the interrupts for preparing an exception */
         Lock_Interrupts(irqLockNum);
         pMBDev->VoiceMissingPackage = FALSE;

         /* reset the streaming index of the voice package */
         pMBDev->VoiceChannelStream = 0xFFFFFFFF;
         
         /* enable the specific interrupt again */
         pBaseGlobal = pMBDev->pBaseGlobal;
         pBaseGlobal->MBC_MSK01 &= ~pMBDev->MBC_Mask01;
         pBaseGlobal->MBC_MSK10 &= ~pMBDev->MBC_Mask01;

	    if (pMBDev->down_callback != NULL) {
			    pMBDev->down_callback(voice0);
	    }
	    else {
		    /* new request for a streaming package for voice channel 1 */
		    oak_downstream_voice(pMBDev);
	    }
	
         /* create an interrupt */
//         OAK_INTERRUPTS_SET(EXC_INCAIP_DSP_SCIR0);

         /* enable the interrupts again */
         Unlock_Interrupts(irqLockNum);
      }
   }

   /* The write to FIFO can continue even if the mailBox is full/blocking */
   return (OK);
}


/****************************************************************************
Description : Write to Task from upstream queue
Arguments   :
Return      :
Remarks     :
****************************************************************************/
OAK_INTERN int oak_read_fifo(OAK_MB_DEV_STR  *pMBDev, 
                             OAK_READWRITE   *pPkg, 
                             INT32           timeout)
{
   UINT32         count8, count16, count32;
   UINT32         maxMsgNum, maxRead, lp_count;
   UINT32         *pDest;
   OAK_FIFO_MB    *fifo;
   INT32          ret;

   fifo = &pMBDev->upstrm_fifo;

   pDest     = pPkg->pRead;
   maxMsgNum = pPkg->nMsgRead;
   maxRead   = pPkg->nRead;

   pPkg->nMsgRead = 0 ; /* total number of packets written to the queue */
   pPkg->nRead = 0 ;    /* total number of bytes written to the queue   */

   if ( pMBDev->event )
   {
      pPkg->event = pMBDev->event;
      pMBDev->event = 0;
   }

   for( lp_count = 0 ; lp_count < maxMsgNum ; lp_count++ )
   {
      /* Data not available to be read from the Upstream queue */
      if(fifo->pWrite == fifo->pRead)
      {
         if (0 == timeout)
            break;
#ifdef VXWORKS
         /* wait with timeout for the next message */
         ret = Sleep_On(pMBDev->sem_read_fifo, timeout);  /* wait with timeout */
#else /* #ifdef LINUX */
         /* TODO: Implement Sleep_On compatible to VxWorks version */
         ret = 0;
#endif /* #ifdef VXWORKS */

         /* check if there is data available */
         if((fifo->pWrite == fifo->pRead) || (ret == 0))
            break;
      }

      /* No of words including the header */
      count16 = (*fifo->pRead & 0xff) + 2;
      count8 = count16 << 1;

      if ((count8 + pPkg->nRead) > maxRead)
      {
         return (OK);
      }

      /* Packet read from upstream queue */
      count32 = (count16 + 1) >> 1;
      /* Copy... */
      while(count32--)
      {
         *(pDest++) = *fifo->pRead;

         if( ++fifo->pRead == pMBDev->upstrm_fifo.pEnd )
            fifo->pRead = pMBDev->upstrm_fifo.pStart;
      }
      /* updating the bytes read */
      pPkg->nRead += count8 ;
      /* updating number of packets read in s/w */
      pPkg->nMsgRead++ ;
   }

   if (pMBDev->irqMissingBytes)
   {
      if (oak_fifo_write_space(fifo) > pMBDev->irqMissingBytes)
      {
         pMBDev->irqMissingBytes = 0;
         /* enable the interrupts again */
	    pMBDev->pBaseGlobal->MBC_MSK &= ~(pMBDev->MBC_Mask);
      }
   }
   return (OK);
}


/****************************************************************************
 Description : 
Write to the mailbox . NOTE : only the command mailbox is to be
written via this routine. This routine can be used in conjugation in case the
Command mailbox is being buffered in the downstream direction as the same
sempahore is controlling both routines.
This is a semaphore controlled routine as
different tasks may need to write to the command mailbox directly.Note the same
semaphore (sem_dev) can be used to access the mailbox.

  Arguments   :
  Return      :
  Remarks     :
****************************************************************************/
OAK_INTERN int oak_write_cmd_mailbox(register OAK_MB_DEV_STR *pMBDev , OAK_READWRITE *readWrite )
{
   UINT32     DWordsTotal, DWordsWritten, packetsizeDWords, freespaceDWords;
   register UINT32     *pSource, i;
   register OAK_RW_MB  *pWriteTypereg;
   UINT32      waitLoops = 0;


   pWriteTypereg = pMBDev->pBaseWriteMB;
   pSource       = readWrite->pWrite;
   /* Total number of shorts to write */
   DWordsTotal   = readWrite->nWrite >> 2;

   DWordsWritten = 0;

   Sem_Lock(pMBDev->sem_dev);

   /* only execute if there are still enough bytes for the header */
   while (DWordsTotal)
   {

	 /* Do not send DownloadEND command to DSP. This needs to be done due 
	    to a hardware bug */
      if (*pSource == 0x0600FF00)
      {
	    pSource++;
         DWordsTotal -= 1;
         DWordsWritten += 1;
         Wait(5);
 	    continue;
	 }
	   
	   /* get the length of this message and check if it still fits   */
      /* into the mailbox                                            */
      if (*pSource & 0x80000000)
         packetsizeDWords = 1;
      else
         packetsizeDWords = ((*pSource & 0xff) + 2) >> 1;

      if (packetsizeDWords > DWordsTotal)
      {
         PRINTF("ERROR: Mailbox Message to write is bigger than data "
            "buffer!\n");
         goto OAK_WRITE_2_MAILBOX_WRITE_DONE;
      }

OAK_WRITE_2_MAILBOX_READ_AGAIN:

      /* get the free space in the mailbox (- 4 due to erratum) */
      freespaceDWords = (OAK_REG_READ(pWriteTypereg->MBC_FS) - 4) >> 2;

      /* check if the message still fits into the mailbox */
      if (packetsizeDWords > freespaceDWords)
      {
         if (++waitLoops < OAK_WRITE_MB_TIMEOUT)
         {
            /* wait a bit and try again to write to the mailbox */
            Wait(1);
            goto OAK_WRITE_2_MAILBOX_READ_AGAIN;
         }
         else
         {
            break;
         }
      }

      /* copy the data to the mailbox */
      for (i = 0; i < packetsizeDWords; i++)
      {
         OAK_REG_WRITE(pWriteTypereg->MBC_IOD,*pSource++);
      }

      /* set the ready command to the mailbox */
      OAK_REG_WRITE_NOP(pWriteTypereg->MBC_CR,1);

      DWordsTotal -= packetsizeDWords;
      DWordsWritten += packetsizeDWords;
   }

OAK_WRITE_2_MAILBOX_WRITE_DONE:

   readWrite->nWrite = DWordsWritten << 2;

   Sem_Unlock(pMBDev->sem_dev);

  return (OK);
}

/****************************************************************************
Description :
Arguments   :
Return      :
Remarks     :
****************************************************************************/
int oak_write_voice_mailbox(register OAK_MB_DEV_STR *pMBDev, OAK_READWRITE *readWrite )
{
   register UINT32 count, *data;
    
   /* Only proceed if packet size is smaller than freespace available
      (-4 due to erratum */
   if (readWrite->nWrite > (OAK_REG_READ(pMBDev->pBaseWriteMB->MBC_FS) - 4)) 
   {
      return( ERROR );
   }
   data = readWrite->pWrite;

   /* Set the channel information in the header */
   OAK_REG_WRITE(pMBDev->pBaseWriteMB->MBC_IOD, (*data & pMBDev->HeaderANDMask) | pMBDev->HeaderORMask);
   data++;

   
   /* Copy rest of data to mailbox */
   for (count = ((readWrite->nWrite - 4) & 0xfffffffc); count > 0; count -= 4)
   {
      OAK_REG_WRITE(pMBDev->pBaseWriteMB->MBC_IOD, *data);
      data++;
   }
   if (readWrite->nWrite % 4)
   {
      OAK_REG_WRITE(pMBDev->pBaseWriteMB->MBC_IOD, *data);
   }

   /* set the ready command to the mailbox */
   OAK_REG_WRITE_NOP(pMBDev->pBaseWriteMB->MBC_CR,1);
   return( OK );
}

/****************************************************************************
  OAK interrupt service routines
****************************************************************************/

/****************************************************************************
Description : Write from mailbox to upstream queues. 
Arguments   :
Return      :
Remarks     :
****************************************************************************/
#ifdef LINUX
void oak_upstream_voice_isr(int irq, OAK_DEV *pDev, struct pt_regs *regs)
{
#endif /* LINUX */
#ifdef VXWORKS
LOCAL void oak_upstream_voice_isr(OAK_DEV *pDev)
{
#endif /* VXWORKS */
   OAK_RW_MB      *pReadMB;
   INT32          mbDataByteLen, cmdByteLen;
   BOOL           bVoice1, bVoice2;
   UINT32         cmdword,fifo_size;
   OAK_FIFO_MB    *fifo;

   TRACE(INCA_OAK_DRV, DBG_LEVEL_LOW, ("INCA_OAK_DRV: Voice ISR\n"));

   /* acknowledge the interrupt */
   OAK_INTERRUPTS_CLEAR(EXC_INCAIP_DSP_B2DAIR);
   
   /* address for the read buffer */
   pReadMB = &(pDev->pBaseGlobal->Reserved[2]);

   /* boolean variables to weak up the pending tasks */
   bVoice1 = FALSE;
   bVoice2 = FALSE;

   while (1)
   {
      /* get the data length of available bytes in mailbox */
      mbDataByteLen = OAK_REG_READ(pReadMB->MBC_DA);

      /* quit if the are no more data to be read from the mailbox */
      if (0 >= mbDataByteLen)	{
			if ((bVoice1 == TRUE) || (bVoice2 == TRUE))
			{
				goto OAK_READVOICE_ISR_EXIT;
			}
			return;
	 }

      while (0 < mbDataByteLen)
      {
         /* get the command word of the next message */
         cmdword = OAK_REG_READ(pReadMB->MBC_IOD);

	    if (cmdword == 0)
	    	{
			mbDataByteLen -= 4;
			/* something has gone wrong...flush the read fifo */
			OAK_REG_WRITE(pReadMB->MBC_CR,4);
			printk("<oak> invalid cmd word \n");
			return;
		}
		
         /* get the length of the whole command in bytes */
         cmdByteLen = ((cmdword & 0x000000FF) + 2) << 1;

         if (cmdword & 0x00010000)
         {
            /* voice channel 2 */
            fifo = &pDev->VoiceMB[1].upstrm_fifo;
            bVoice2 = TRUE;

#ifdef CONFIG_PROC_FS
            /* increase the counter of interrupts */
            pDev->VoiceMB[1].RxnumIRQs++;
            pDev->VoiceMB[1].RxnumBytes += cmdByteLen;
#endif /* CONFIG_PROC_FS */
         }
         else
         {
            /* voice channel 1 */
            fifo = &pDev->VoiceMB[0].upstrm_fifo;
            bVoice1 = TRUE;

#ifdef CONFIG_PROC_FS
            /* increase the counter of interrupts */
            pDev->VoiceMB[0].RxnumIRQs++;
            pDev->VoiceMB[0].RxnumBytes += cmdByteLen;
#endif /* CONFIG_PROC_FS */
         }

         /* check if there is enough space in the fifo for this message */
	    fifo_size = oak_fifo_write_space(fifo);

#ifdef INCA_GIVE_DSP_FILL_LEVEL_WARNING
	    /* Check fill level of fifo and give a message if it is over 1000 */
	    if ((MAX_FIFO_VOICE * 4)- fifo_size > 1000) {
			printk("DA: %d\n",(MAX_FIFO_VOICE * 4)- fifo_size);
	    }
#endif

	    if (fifo_size < cmdByteLen) {
			printk("DSP DRIVER FIFO FULL voice 0 %d voice 1 %d\n",
									bVoice1, bVoice2);

		     /* too less space in fifo for the message                 */
			/* disable the interrupt till the space in the fifo is   */
			/* available again                                       */
			pDev->pBaseGlobal->MBC_MSK |= OAK_MBC_ISR_B2DA;

			if (cmdword & 0x00010000) {
				/* voice channel 2 */
				pDev->VoiceMB[1].irqMissingBytes = cmdByteLen;
#ifdef CONFIG_PROC_FS
            	pDev->VoiceMB[1].RxnumBytes -= cmdByteLen;
#endif
			}
			else {
				/* voice channel 1 */
				pDev->VoiceMB[0].irqMissingBytes = cmdByteLen;
#ifdef CONFIG_PROC_FS
               	pDev->VoiceMB[0].RxnumBytes -= cmdByteLen;
#endif
			}

			/* set the abort command to the voice mailbox */
			OAK_REG_WRITE_NOP(pReadMB->MBC_CR,2);

			return;
         }
 
         mbDataByteLen -= cmdByteLen;

         /* copy command header to the fifo */
         *fifo->pWrite++ = cmdword;
   
         if (fifo->pWrite >= fifo->pEnd)
            fifo->pWrite = fifo->pStart;

         cmdByteLen -= 4;

         /* copy data to the fifo */
         while (0 < cmdByteLen)
         {
            *fifo->pWrite++ = OAK_REG_READ(pReadMB->MBC_IOD);

            if (fifo->pWrite >= fifo->pEnd)
               fifo->pWrite = fifo->pStart;

            cmdByteLen -= 4;
         }
         /* set the ready command to the voice mailbox */
         OAK_REG_WRITE_NOP(pReadMB->MBC_CR,1);
      }
   }

OAK_READVOICE_ISR_EXIT:
   /*The same task uses the voice fds to read the queues */
   if(TRUE == bVoice1)
   {
      if(pDev->VoiceMB[0].up_callback != NULL) 
      {
         pDev->VoiceMB[0].up_callback( voice0 );
      } 
      else 
      {
         Wake_Up_Queue(pDev->VoiceMB[0].oak_wklist);
      }
   }
      
   if(TRUE == bVoice2)
   {
      if(pDev->VoiceMB[1].up_callback != NULL) 
      {
         pDev->VoiceMB[1].up_callback( voice1 );
      }
      else
      {
         Wake_Up_Queue(pDev->VoiceMB[1].oak_wklist);
      }
   }

   /* now clear the interrupt in the ICU */
   OAK_INTERRUPTS_CLEAR(EXC_INCAIP_DSP_B2DAIR);
}


/****************************************************************************
Description : ISR to read the mailbox entries of the command mailbox.
   This interrupt also acknowledge the interrupt of the mailbox inside the
   register of the mailbox itself.
Arguments   :
   device handle of the whole device
Return      :
Remarks     :
****************************************************************************/
#ifdef LINUX
void oak_upstream_cmd_isr(int irq, OAK_DEV *pDev, struct pt_regs *regs)
{
#endif /* LINUX */
#ifdef VXWORKS
LOCAL void oak_upstream_cmd_isr(OAK_DEV *pDev)
{
#endif /* VXWORKS */
   OAK_RW_MB   *pReadMB;
   UINT32      mbDataByteLen;
   UINT32      cmdword, cmdByteLen;
   OAK_FIFO_MB *fifo;

   /* address for the read buffer */
   pReadMB = &(pDev->pBaseGlobal->Reserved[3]);
   fifo = &pDev->CommandMB.upstrm_fifo;

#ifdef CONFIG_PROC_FS
   /* increase the counter of interrupts */
   pDev->CommandMB.RxnumIRQs++;
#endif /* CONFIG_PROC_FS */

   TRACE(INCA_OAK_DRV, DBG_LEVEL_LOW, ("INCA_OAK_DRV: CMD ISR\n"));
   while (1)
   {
      /* get the data length of available bytes in mailbox */
      mbDataByteLen = OAK_REG_READ(pReadMB->MBC_DA);

      /* quit if the are no more data to read from the mailbox */
      if (0 == mbDataByteLen)
      {
         /*TRACE(INCA_OAK_DRV, DBG_LEVEL_LOW, ("CMD ISR:no data in mailbox\n"));*/
         goto OAK_READCMD_ISR_EXIT;
      }

      while (mbDataByteLen)
      {
         /* get the command word of the next message */
         cmdword =OAK_REG_READ(pReadMB->MBC_IOD);

         /* get the length of the whole command in bytes */
         cmdByteLen = ((cmdword & 0x000000FF) + 2) << 1;

         /* check if there is enough space in the fifo for this message */
         if (oak_fifo_write_space(fifo) < cmdByteLen)
         {
            /* too less space in fifo for the message                 */
            /* disable the interrupt till the space in the fifo is   */
            /* available again                                       */
            pDev->pBaseGlobal->MBC_MSK |= OAK_MBC_ISR_B3DA;

            pDev->CommandMB.irqMissingBytes = cmdByteLen;

            /* set the abort command to the voice mailbox */
            OAK_REG_WRITE_NOP(pReadMB->MBC_CR,2);

            /*TRACE(INCA_OAK_DRV, DBG_LEVEL_LOW, ("CMD ISR:no space in fifo\n"));*/
            goto OAK_READCMD_ISR_EXIT;
         }

         mbDataByteLen -= cmdByteLen;

         /* copy comannd header to the fifo */
         *fifo->pWrite++ = cmdword;

         if (fifo->pWrite >= fifo->pEnd)
            fifo->pWrite = fifo->pStart;

         cmdByteLen -= 4;

         /* copy data to the fifo */
         while (cmdByteLen)
         {
            *fifo->pWrite++ = OAK_REG_READ(pReadMB->MBC_IOD);

            if (fifo->pWrite >= fifo->pEnd)
               fifo->pWrite = fifo->pStart;

            cmdByteLen -= 4;
         }

         /* set the ready command to the voice mailbox */
         OAK_REG_WRITE_NOP(pReadMB->MBC_CR,1);
      }
   }

OAK_READCMD_ISR_EXIT:
   /* acknowledge the interrupt */
   OAK_INTERRUPTS_CLEAR(EXC_INCAIP_DSP_B3DAIR);

   /*TRACE(INCA_OAK_DRV, DBG_LEVEL_LOW, ("CMD ISR:leave\n"));*/
   if(pDev->CommandMB.up_callback != NULL) 
   {
      pDev->CommandMB.up_callback( command );
   }
   else
   {
      Wake_Up_Queue(pDev->CommandMB.oak_wklist);
   }
}

/****************************************************************************
Description :   Interrupt service routine (general)

Arguments   : VxWorks parameter
Return      : none
Remarks     :
****************************************************************************/
#ifdef LINUX
void oak_downstream_voice_isr(int irq, OAK_DEV *pDev, struct pt_regs *regs)
{
#endif /* LINUX */
#ifdef VXWORKS
LOCAL void oak_downstream_voice_isr(OAK_DEV *pDev)
{
#endif /* VXWORKS */

   volatile OAK_GLOBAL_REG *pBaseGlobal;
   UINT32                  dISRstatus, i;
   INT32                   mask01, mask10;
   static UINT32           oldISRstatus;
   OAK_MB_DEV_STR          *pMBDev;

   pBaseGlobal = pDev->pBaseGlobal;

   /* get the status information */
   dISRstatus = pBaseGlobal->MBC_ISR;

   /* acknowledge enabled OAK interrupts */
   mask01 = pBaseGlobal->MBC_MSK01;
   mask10 = pBaseGlobal->MBC_MSK10;

   pBaseGlobal->MBC_ACK = ~mask01 | ~mask10;

   /* acknowledge the interrupt in the ICU */
   OAK_INTERRUPTS_CLEAR(EXC_INCAIP_DSP_SCIR0);

   /* voice channel 0 */
   pMBDev = &pDev->VoiceMB[0];
   if ((dISRstatus & OAK_MBC_ISR_DEC1_REQ) != pMBDev->VoiceChannelStream)
   {
      pMBDev->VoiceChannelStream = dISRstatus & OAK_MBC_ISR_DEC1_REQ;
      if (pMBDev->down_callback != NULL)
      {
         pMBDev->down_callback(voice0);
      }
      else
      {
         /* new request for a streaming package for voice channel 1 */
         oak_downstream_voice(pMBDev);
      }
   }

   /* voice channel 1 */
   pMBDev = &pDev->VoiceMB[1];
   if ((dISRstatus & OAK_MBC_ISR_DEC2_REQ) != pMBDev->VoiceChannelStream)
   {
      pMBDev->VoiceChannelStream = dISRstatus & OAK_MBC_ISR_DEC2_REQ;
      if (pMBDev->down_callback != NULL)
      {
         pMBDev->down_callback(voice1);
      }
      else
      {
         /* new request for a streaming package for voice channel 2 */
         oak_downstream_voice(pMBDev);
      }
   }

   /* other event */
   if (dISRstatus & (mask01 | mask10) & ~(OAK_MBC_ISR_DEC1_REQ | OAK_MBC_ISR_DEC2_REQ))
   {
      /* check which MB registered for this event */
      for (i = 0; i < 3; i++)
      {
         switch (i)
         {
         case 0:
            pMBDev = &pDev->VoiceMB[0];
            break;
         case 1:
            pMBDev = &pDev->VoiceMB[1];
            break;
         case 2:   
            pMBDev = &pDev->CommandMB;
            break;
         default:
         }
         if (dISRstatus & pMBDev->event_mask)
         {
            /* /add/ all unmasked events - event field will be cleard on read */
            pMBDev->event |= dISRstatus & ~OAK_MBC_ISR_DTMFR_DTC  & ~pMBDev->event_mask;

            pDev->flags |= OAK_STATUS_EVENT;
            if (pMBDev->event_callback != NULL)
            {
               pMBDev->event_callback(pMBDev->event);
               pMBDev->event = 0;
            }
            else
            {
               Wake_Up_Queue(pMBDev->oak_wklist);
            }
         }
      }
   }
 
  oldISRstatus = dISRstatus;
}

