/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   Module      : INCAOAK_DeviceDriver.h
   Date        : $Date: 2004/03/22 09:56:25 $
   Description :
   
   $Log: INCAOAK_DeviceDriver.h,v $
   Revision 1.7  2004/03/22 09:56:25  thomas
   fixed typo

   Revision 1.6  2004/03/04 05:26:15  thomas
   Fixed bit names for MBC_ISR register
   Renamed OAK_DMA to OAK_GLOBAL_REG
   Added vars for event notification to device struct

   Revision 1.5  2004/02/02 14:08:54  thomas
   Added global flags for event signalling

   Revision 1.22  2003/10/21 14:52:52  aschmidt
   introduced OAK_INTERRUPTS_ENABLE/DISABLE macros to be interrupt safe (Note: to be adapted for Linux!)

   Revision 1.21  2003/07/07 07:21:43  rutkowski
   - Remove unused code.
   - Make the boolean variable to synchronize the interrupt and the task level as 'volatile'.

   Revision 1.20  2003/06/27 12:58:35  rutkowski
   - Add a feature to count the number of different interrupts for debugging. 
     This feature can be enabled by a compiler switch.

   Revision 1.19  2003/06/23 17:05:56  rutkowski
   - First version with the merge of the support for Linux

   Revision 1.18  2003/06/23 15:19:29  rutkowski
   Add the module for the INCA-IP VxWorks BSP and change the name of the 
   INCA-IP Linux BSP module name to come to a common name convention for 
   both modules
   
******************************************************************************/
#ifndef _INCAOAK_DEVICEDRIVER_H
#define _INCAOAK_DEVICEDRIVER_H

/*---------------------------------------------------------------------------*/
/* Definition of the Baseaddress of the OAK Device                           */
/*---------------------------------------------------------------------------*/
#define OAK_BASEADDRESS 0xBF103000
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Defination of macros for read write of OAK registers                      */
/*---------------------------------------------------------------------------*/
#define OAK_REG_WRITE(Reg_Address,Reg_Value)  (Reg_Address)=(Reg_Value)
#define OAK_REG_READ(Reg_Address)  (Reg_Address)

#define OAK_REG_WRITE_NOP(Reg_Address,Reg_Value)  \
	 asm(".set noreorder"); \
      OAK_REG_WRITE((Reg_Address),(Reg_Value)); \
	 asm("nop;nop;nop;nop;"); \
	 asm(".set reorder"); 


#ifdef VXWORKS
#define OAK_INTERRUPTS_ENABLE(X) 	sysIncaHWRegSetBit(INCA_IP_ICU_IM1_IER, X);   
#define OAK_INTERRUPTS_DISABLE(X) 	sysIncaHWRegResetBit(INCA_IP_ICU_IM1_IER, X);
#define OAK_INTERRUPTS_CLEAR(X) 	sysIncaHWRegWrite(INCA_IP_ICU_IM1_ISR, X);   
#define OAK_INTERRUPTS_SET(X) 		sysIncaHWRegWrite(INCA_IP_ICU_IM1_IRSR, X); /* ? */  
#else /* LINUX */
#define OAK_INTERRUPTS_ENABLE(X)  *((volatile UINT32*) INCA_IP_ICU_IM1_IER) |= X;
#define OAK_INTERRUPTS_DISABLE(X) *((volatile UINT32*) INCA_IP_ICU_IM1_IER) &= ~X;
#define OAK_INTERRUPTS_CLEAR(X)   *((volatile UINT32*) INCA_IP_ICU_IM1_ISR) = X;
#define OAK_INTERRUPTS_SET(X)     *((volatile UINT32*) INCA_IP_ICU_IM1_IRSR) = X;  /*  |= ? */

#endif

/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Interrupt register values for ICU                                         */
/*---------------------------------------------------------------------------*/
#define  EXC_INCAIP_DSP_B3DAIR  (1 << 20)  /* IM1_IRL20  */
#define  EXC_INCAIP_DSP_B2DAIR  (1 << 21)  /* IM1_IRL21  */
#define  EXC_INCAIP_DSP_B1EIR   (1 << 22)  /* IM1_IRL22  */
#define  EXC_INCAIP_DSP_B0EIR   (1 << 23)  /* IM1_IRL23  */
#define  EXC_INCAIP_DSP_WDTIR   (1 << 24)  /* IM1_IRL24  */
#define  EXC_INCAIP_DSP_SCIR0   (1 << 25)  /* IM1_IRL25  */
#define  EXC_INCAIP_DSP_SCIR8   (1 << 26)  /* IM1_IRL26  */
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Interrupt register values for MBC_ISR                                     */
/*---------------------------------------------------------------------------*/
#define OAK_MBC_ISR_B3DA      (1 << 31)
#define OAK_MBC_ISR_B2DA      (1 << 30)
#define OAK_MBC_ISR_B1E       (1 << 29)
#define OAK_MBC_ISR_B0E       (1 << 28)
#define OAK_MBC_ISR_WDT       (1 << 27)
#define OAK_MBC_ISR_SP_TRA    (1 << 26)
#define OAK_MBC_ISR_SP_IDLE   (1 << 25)
#define OAK_MBC_ISR_SP_DBL    (1 << 24)
#define OAK_MBC_ISR_CMDCERR   (1 << 23)
#define OAK_MBC_ISR_MIPS_OL   (1 << 22)
#define OAK_MBC_ISR_AFE_OF    (1 << 21)
#define OAK_MBC_ISR_DWL_END   (1 << 20)
#define OAK_MBC_ISR_CIDTR_ACT (1 << 18)
#define OAK_MBC_ISR_CIDTR_REQ (1 << 17)
#define OAK_MBC_ISR_CIDTR_BUF (1 << 16)
#define OAK_MBC_ISR_TG_ACT    (1 << 14)
#define OAK_MBC_ISR_DTMFR_DTV (1 << 13)
#define OAK_MBC_ISR_DTMFR_DTP (1 << 12)
#define OAK_MBC_ISR_DTMFR_DTC (0xF << 8)
#define OAK_MBC_ISR_ENC1_OF   (1 << 7)
#define OAK_MBC_ISR_ENC2_OF   (1 << 6)
#define OAK_MBC_ISR_DEC1_ERR  (1 << 5)
#define OAK_MBC_ISR_SR1_ERR   (1 << 5)
#define OAK_MBC_ISR_DEC2_ERR  (1 << 4)
#define OAK_MBC_ISR_SR2_ERR   (1 << 4)
#define OAK_MBC_ISR_DEC1_REQ  (1 << 3)
#define OAK_MBC_ISR_SR1_REQ   (1 << 3)
#define OAK_MBC_ISR_DS1_UF    (1 << 2)
#define OAK_MBC_ISR_SR1_IVW   (1 << 2)
#define OAK_MBC_ISR_DEC2_REQ  (1 << 1)
#define OAK_MBC_ISR_SR2_REQ   (1 << 1)
#define OAK_MBC_ISR_DS2_UF    (1 << 0)
#define OAK_MBC_ISR_SR2_IVW   (1 << 0)
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Register bits definition of the Control Register                          */
/*---------------------------------------------------------------------------*/
#define OAK_MBC_DCTRL_BA      (1 << 0)
#define OAK_MBC_DCTRL_BMOD    (1 << 1)
#define OAK_MBC_DCTRL_RES     (1 << 15)
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Register bits definition of the Configuration Register                    */
/*---------------------------------------------------------------------------*/
#define OAK_MBC_CFG_SWAP2     (2 << 6)
#define OAK_MBC_CFG_RES       (1 << 5)
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Bit definition of the driver-internal status flags                        */
/*---------------------------------------------------------------------------*/
#define OAK_STATUS_EVENT             (1 << 0)
#define OAK_STATUS_                  (1 << 1)
/*---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*/
/* Enum for type of the connection for the device driver                     */
/*---------------------------------------------------------------------------*/
typedef enum
{
   unknown,
   command,
   voice0,
   voice1
}DEVTYPE;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* structure which describes the read- or write- mailbox of the OAK          */
/*---------------------------------------------------------------------------*/
typedef struct
{
   /* Mailbox */
   volatile UINT32   MBC_IOD;
   volatile UINT32   MBC_CR;
   volatile UINT32   MBC_FS;
   volatile UINT32   MBC_DA;
   volatile UINT32   MBC_IABS;
   volatile UINT32   MBC_ITMP;
   volatile UINT32   MBC_OABS;
   volatile UINT32   MBC_OTMP;
}OAK_RW_MB;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* FIFO structure                                                            */
/*---------------------------------------------------------------------------*/
typedef struct
{
   UINT32            *pStart;
   UINT32            *pEnd;
   UINT32            *pWrite;
   UINT32            *pRead;
   UINT32            FifoSize;
}OAK_FIFO_MB;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* structure which describes global register of the OAK                      */
/*---------------------------------------------------------------------------*/
typedef struct
{
   OAK_RW_MB            Reserved[4];
   volatile UINT32      MBC_CFG;       /* Offset 0x80 */
   volatile UINT32      MBC_ISR;       /* Offset 0x84 */
   volatile UINT32      MBC_MSK;       /* Offset 0x88 */
   volatile UINT32      MBC_MSK01;     /* Offset 0x8C */
   volatile UINT32      MBC_MSK10;     /* Offset 0x90 */
   volatile UINT32      MBC_CMD;       /* Offset 0x94 */
   volatile UINT32      MBC_ACK;       /* Offset 0x98 */
   volatile UINT32      Reserved2;     /* Offset 0x9C */
   volatile UINT32      MBC_DCTRL;     /* Offset 0xA0 */
   volatile UINT32      MBC_DSTA;      /* Offset 0xA4 */
   volatile UINT32      MBC_DTST1;     /* Offset 0xA8 */
}OAK_GLOBAL_REG;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Device connection structure                                               */
/*---------------------------------------------------------------------------*/
typedef struct
{
   /* void pointer to the base device driver structure */
   void              *pOAK_DEV;

   /* Mutex semaphore to access the device */
   SEM_ID            sem_dev;

   /* Mutex semaphore to access the device */
   SEM_ID            sem_read_fifo;
   
   /* Wakeuplist for the select mechanism */
   SYS_POLL          oak_wklist;

   /* Base Address of the Mailbox Access */
   OAK_RW_MB         *pBaseReadMB;
   OAK_RW_MB         *pBaseWriteMB;
      
   /* Base Address of the Global Register Access */
   OAK_GLOBAL_REG    *pBaseGlobal;
      
   /* missing space in fifo */
   /* used if the interrupt is disabled and waiting for this amount of  */
   /* free bytes in the fifo to get enabled again                       */
   UINT32            irqMissingBytes;
   UINT32            VoiceChannelStream;
   
   /* Mask Bits for the IRQ register of the mailbox */
   UINT32            MBC_Mask;
   UINT32            MBC_Mask01;
   UINT32            MBC_Mask10;
   
   /* FIFO for read (Upstream) Buffer */
   OAK_FIFO_MB       upstrm_fifo;
   
   /* FIFO for write (Downstream) Buffer */
   OAK_FIFO_MB       dwstrm_fifo;
   
#ifdef OAK_FIFO_BLOCKING_WRITE
   SEM_ID           sem_write_fifo;
#endif /* OAK_FIFO_BLOCKING_WRITE */

   UINT32            errnum;
   
#ifdef CONFIG_PROC_FS
   /* interrupt counter */
   volatile UINT32   RxnumIRQs;
   volatile UINT32   TxnumIRQs;

   volatile UINT32   RxnumMiss;
   volatile UINT32   TxnumMiss;
  
   volatile UINT32   RxnumBytes;
   volatile UINT32   TxnumBytes;
#endif /* CONFIG_PROC_FS */

   /* Device ID  1->command 
		 2->voice chan 0, 
                 3->voice chan 1 */            
   DEVTYPE           devID;
   
   UINT32            HeaderORMask;
   UINT32            HeaderANDMask;

#ifdef OAK_FIFO_BLOCKING_WRITE
   volatile BOOL     full_write_fifo;   
#endif /* OAK_FIFO_BLOCKING_WRITE */

   /* installed or not? */
   BOOL              Installed;
   
   /* missing voice package for downstream */
   BOOL              VoiceMissingPackage;
   
#ifdef OAK_FIFO_BLOCKING_WRITE
   /* variable if the driver should block on write to the transmit FIFO */
   BOOL              bBlockWriteMB;
#endif /* OAK_FIFO_BLOCKING_WRITE */
   void              (*down_callback)(DEVTYPE type);
   void              (*up_callback)(DEVTYPE type);
   UINT              event;
   UINT              event_mask;
   void              (*event_callback)(UINT mask);
   
} OAK_MB_DEV_STR;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* Device structure                                                          */
/*---------------------------------------------------------------------------*/
typedef struct
{
#ifdef VXWORKS
   /* VxWorks dependent structure */
   DEV_HDR           oak_dev_hdr;
#endif /* #ifdef VXWORKS */

   /* global register pointer for the ISR */
   OAK_GLOBAL_REG    *pBaseGlobal;
   UINT32	     flags;
   
   /* Command and Voice supported device connections */
   OAK_MB_DEV_STR    VoiceMB[2];  /* for chan0 and chan1 */
   OAK_MB_DEV_STR    CommandMB;
      
}OAK_DEV;

/*---------------------------------------------------------------------------*/
#endif /* _INCAOAK_DEVICEDRIVER_H */


