#ifndef _SYS_DRV_NOOS_H
#define _SYS_DRV_NOOS_H
/****************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for
   the program, to the extent permitted by applicable law.  Except when
   otherwise stated in writing the copyright holders and/or other parties
   provide the program "as is" without warranty of any kind, either
   expressed or implied, including, but not limited to, the implied
   warranties of merchantability and fitness for a particular purpose. The
   entire risk as to the quality and performance of the program is with
   you.  should the program prove defective, you assume the cost of all
   necessary servicing, repair or correction.

   In no event unless required by applicable law or agreed to in writing
   will any copyright holder, or any other party who may modify and/or
   redistribute the program as permitted above, be liable to you for
   damages, including any general, special, incidental or consequential
   damages arising out of the use or inability to use the program
   (including but not limited to loss of data or data being rendered
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or
   other party has been advised of the possibility of such damages.
 ****************************************************************************
   Module      : $RCSfile: sys_drv_noOS.h,v $
   Date        : $Date: 2004/03/30 10:59:32 $
   Description : This file contains the includes and the defines
                 specific to the vxWorks OS

   $Log: sys_drv_noOS.h,v $
   Revision 1.1  2004/03/30 10:59:32  thomas
   Working version of common ...

   Revision 1.1  2003/12/19 15:34:52  tauche
   OS-less driver abstraction layer

   Revision 1.35  2003/12/05 09:52:45  langer
   - make sure that __BYTE_ORDER, __LITTLE_ENDIAN
     and __BIG_ENDIAN are always defined
   - removed unused include (vxWorks.h is enough for byte order)

   Revision 1.34  2003/11/21 14:12:17  kamdem
   comments adjusted

   Revision 1.33  2003/11/21 13:06:58  kamdem
   endianess for vxworks to make it compatible to linux ... still in check !!!

   Revision 1.32  2003/10/13 18:07:33  neumann
   * Added Wake_Down_Queue() for select wakeup on write

   Revision 1.31  2003/08/14 15:10:12  langer
   - removed config.h again
   - removed "driver/" from some path
   -> more independant from BSP

   Revision 1.30  2003/08/08 16:51:41  wachendorf
   added #include <config.h>

   Revision 1.29  2003/07/24 14:57:59  tauche
   Added definitions for SUCCESS and ERR for compatibility with linux code

   Revision 1.28  2003/07/07 09:58:57  rutkowski
   - Implement a lock/unlock mechanism for global interrupts of the system.

   Revision 1.27  2003/06/27 13:03:55  rutkowski
   - Implement a filled waitqueue to use as mutex semaphore.

   Revision 1.26  2003/06/23 17:17:50  rutkowski
   - Replace the '#ifdef PPC' macro against '#if (CPU_FAMILY==PPC)',
     because the PPC macro is always declared. This change was required
     to be compatible to the MIPS architecture.

*******************************************************************************/

/** \file
   This file contains the includes and the defines specific to the vxWorks OS
*/

/** \defgroup SYS_DRV_VXWORKS Definitions for VxWorks
   These features are base on VxWorks 5.4.
*/

/* ============================= */
/* Global Includes               */
/* ============================= */

/* System includes*/
#if 0
#if (CPU_FAMILY==PPC)
#include <drv/intrCtl/ppc860Intr.h>
#include <drv/multi/ppc860Siu.h>
#include <drv/multi/ppc860Cpm.h>
#include <drv/sio/ppc860Sio.h>
#include <arch/ppc/ivPpc.h>
#endif
#endif

#if 0
#include <drv/timer/timerDev.h>
#endif

#if 0
#include <selectLib.h>
#include <iosLib.h>
#include <stdio.h>
#include <time.h>
#include <taskLib.h>
#include <cacheLib.h>
#include <logLib.h>
#include <intLib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <tickLib.h>
#include <math.h>
#endif

/* ============================= */
/*          Endianess            */
/* ============================= */

/** \defgroup SYS_DRV_VXWORKS_ENDIAN Endianess
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/
#if !defined ( __LITTLE_ENDIAN )
   #define __LITTLE_ENDIAN _LITTLE_ENDIAN
#endif /* defined ( __LITTLE_ENDIAN ) */

#if !defined ( __BIG_ENDIAN )
   #define __BIG_ENDIAN    _BIG_ENDIAN
#endif /* defined ( __LITTLE_ENDIAN ) */

#if (_BYTE_ORDER == _LITTLE_ENDIAN)
   /** byte order is little endian */
   #define __BYTE_ORDER __LITTLE_ENDIAN
#elif (_BYTE_ORDER == _BIG_ENDIAN )
   /** byte order is big endian */
   #define __BYTE_ORDER __BIG_ENDIAN
#else
   #error "Unknown Byteorder!"
#endif
/*@}*/

/* ============================= */
/* Compiler features             */
/* ============================= */

/** \defgroup SYS_DRV_VXWORKS_FEAT Compiler features
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/
/** Do not inline functions. */
#define INLINE
/** Define functions only local. */
#define LOCAL  static
#define IMPORT extern
/*@}*/

/* ============================= */
/* Function Macros               */
/* ============================= */

#if 0
#if (CPU_FAMILY==PPC)
   #ifndef GetImmr
      #define GetImmr()                         vxImmrGet()
   #endif

   #define IMAP_ADDR                            vxImmrGet()
#endif
#endif

/** \defgroup SYS_DRV_VXWORKS_DELAY Delay functions
   \ingroup SYS_DRV_VXWORKS

   This section defines different delay functions
   with active waiting or with enabling the scheduler.

   \note "Wait_Milli_Sec" and "Wait_Micro_Sec" use the delay library
   "delayLib" from the "VxWorks / Tornado II FAQ"
    (Available at http://www.xs4all.nl/~borkhuis/vxworks/vxw_pt6.html#6.1)
*/
/*@{*/

/** Wait milliseconds with rescheduling.
   \note VxWorks taskdelay() is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick, but Wait() does not need to be very accurate
 */
/* STST */
#define Wait(time) No_OS_Wait(time);
/** Wait active milliseconds without rescheduling. */
#define Wait_Milli_Sec(time)              No_OS_Wait(time);
/** Wait active microseconds without rescheduling. */
#define Wait_Micro_Sec(time)              No_OS_Wait(time);

/*@}*/

/** \defgroup SYS_DRV_VXWORKS_MEMORY Memory functions
   \ingroup SYS_DRV_VXWORKS

   This section defines functions for allocating and freeing memory.
   If DEBUG is defined some functions to trace memory allocation are used
   (s. sys_debug.c)
*/
/*@{*/
#ifdef DEBUG
   /* define own functions for allocation and free */
   #define Alloc_Mem(size)                sys_dbgMalloc(size, __LINE__, __FILE__)
   #define Free_Mem(ptr)                  sys_dbgFree((void*)ptr, __LINE__, __FILE__)
   /* wrapper for os malloc and free in case of debug */
   #define sysdebug_malloc(size)          malloc(size)
   #define sysdebug_free(ptr)             free((void*)ptr)
#else
   /** Allocate memory. */
   #define Alloc_Mem(size)                malloc(size)
   /** Free memory. */
   #define Free_Mem(ptr)                  free((void*)ptr)
#endif


/*@}*/

/** \defgroup SYS_DRV_VXWORKS_EVENT Event handling functions
   \ingroup SYS_DRV_VXWORKS

   This section defines functions for event handling.
*/
/*@{*/

/** Initialize an event. */
#define Init_Event(queue) \
   (queue) = (noOSEventCnt < MAX_NUMBER_OF_EVENTS)?&(noOSEvent[noOSEventCnt++]):NULL;

/** Signal an event. */
#define Wake_Up_Event(queue)  

/** Reset event to initial state. */
#define Clear_Event(queue)

/** Wait for an event with timeout.
   \attention This is an old definition, it will be removed in future!
   Returns FALSE / 0 on timeout.
   \note VxWorks delay is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick */
#define Sleep_On(sem, timeout) (TRUE)

/** Wait for an event with timeout.
   Returns FALSE / 0 on timeout.
   \note VxWorks delay is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick */
#define Sleep_On_Event(sem, timeout)   (TRUE)

/* STST */
/** Delete an event. */
#define Delete_Event(queue)   (queue) = NULL

/*@}*/

/** Initialize a queue. */
#define Init_WakeupQueue(queue)           queue = NULL;
/** Signal a queue. */
#define Wake_Up_Queue(queue)              
#define Wake_Down_Queue(queue)            

/* STST */
#define Wake_Up_Node(node)                


/** \defgroup SYS_DRV_VXWORKS_MUTEX Mutex functions
   \ingroup SYS_DRV_VXWORKS

   This section has mutex definitions.
*/
/*@{*/

/** Create a mutex element. */
/* STST */
#define Sem_MutexInit(sem)                /*(sem) = &(OS_EventCnt++) */
                                       
#define Sem_BinaryInit(queue)          (queue) =                              \
                                       (noOSEventCnt < MAX_NUMBER_OF_EVENTS) ?\
                                       &(noOSEvent[noOSEventCnt++]):NULL;


/** Lock a mutex section. */
#define Sem_Lock(sem)                     (OK)

/** Unlock a mutex section. */
#define Sem_Unlock(sem)                   

/** Delete a mutex element. */
#define Sem_MutexDelete(sem)                 \
                           &(OS_EventCnt--); \
                           queue=0;
/*@}*/

/** \defgroup SYS_DRV_VXWORKS_CLOCK Time/Clock functions
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/



/** Returns system tick in milliseconds. */
#define Get_Tick()                        NoOStickGet()
/** Get CPU Frequency from BSP */
IMPORT UINT32 sys_Cpu_Clk;
/** Returns CPU clock in Hz. */
#define Cpu_Clk                           sys_Cpu_Clk

/*@}*/

/** \defgroup SYS_DRV_VXWORKS_INTERRUPT Enable/Disable global interrupts
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/

/*----------------------------------------------------------------------------*/
/* Lock mechanism for general processor interrupts                            */
/* Note: Please avoid to use the mechanism, because disabling all interrupts  */
/*       of the system is not a good solution.                                */
/*----------------------------------------------------------------------------*/
#define Lock_Variable                     int
#define Lock_Interrupts(var)              var = noOSLockCnt++;
#define Unlock_Interrupts(var)            noOSLockCnt--;
/*----------------------------------------------------------------------------*/

/* STST */
#define Lock_Tasks()                      
#define Unlock_Tasks()                    
/*----------------------------------------------------------------------------*/
/*@}*/


/* STST */
#define SYNC  __asm__("	sync")


/* ST ST */
#define OS_Start_Task      

#define OS_Delete_Task(taskId) 


/** Use semaphores for events. */
typedef INT*                     SYS_EVENT;
/** Polling is used to wakeup an application. */
typedef INT                      SYS_POLL;
/* STST */
typedef INT                      SYS_POLL_NODE;

typedef INT                 		(*FUNCPTR) ();	    
typedef VOID                		(*VOIDFUNCPTR) ();  

#define TIMESTAMP int

#define  SYS_NOSELECT   0x00000000
#define  SYS_DATAIN     0x00000001
#define  SYS_EXCEPT     0x00000002
#define  SYS_DATAOUT    0x00000004

/* return values */
#define SUCCESS 0
#define ERR  -1

#define TRUE   1
#define FALSE  0

/* to be removed */
#define OK      0
#define ERROR  -1


IMPORT  INT OS_EventCnt;
IMPORT  INT noOSLockCnt;
IMPORT  VOID No_OS_Wait(INT time);


#endif /* _SYS_DRV_NOOS_H */

