#ifndef _SYS_DRV_VXWORKS_H
#define _SYS_DRV_VXWORKS_H
/****************************************************************************
       Copyright (c) 2003, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for
   the program, to the extent permitted by applicable law.  Except when
   otherwise stated in writing the copyright holders and/or other parties
   provide the program "as is" without warranty of any kind, either
   expressed or implied, including, but not limited to, the implied
   warranties of merchantability and fitness for a particular purpose. The
   entire risk as to the quality and performance of the program is with
   you.  should the program prove defective, you assume the cost of all
   necessary servicing, repair or correction.

   In no event unless required by applicable law or agreed to in writing
   will any copyright holder, or any other party who may modify and/or
   redistribute the program as permitted above, be liable to you for
   damages, including any general, special, incidental or consequential
   damages arising out of the use or inability to use the program
   (including but not limited to loss of data or data being rendered
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or
   other party has been advised of the possibility of such damages.
 ****************************************************************************
   Module      : $RCSfile: sys_drv_vxworks.h,v $
   Date        : $Date: 2004/03/30 10:59:32 $
   Description : This file contains the includes and the defines
                 specific to the vxWorks OS

   $Log: sys_drv_vxworks.h,v $
   Revision 1.1  2004/03/30 10:59:32  thomas
   Working version of common ...

   Revision 1.40  2004/01/08 11:30:34  wachendorf
   fixed numerical problem in calculation of the WaitEvent timeout (for timeouts < 1000 ms the division was zero)
   removed unnecessary code in WaitEvent

   Revision 1.39  2003/12/21 09:52:59  hempel
   fix: added defines for (kernel) threads
   fix: added define for deleting an event / not functional yet
   fix: adedd define for node wake-up

   Revision 1.38  2003/12/17 13:24:48  martin
   Linux line breaks eliminated

   Revision 1.37  2003/12/15 17:15:29  wachendorf
   added #ifndefs around endianness defines

   Revision 1.36  2003/12/10 10:23:12  rah
   Changed OS_Int-Related for more device independance
   Added it to sys_drv_<any>.h
   For PSOS and OSE they are between #if 0, but compiler warning apears

   Revision 1.35  2003/12/05 09:52:45  langer
   - make sure that __BYTE_ORDER, __LITTLE_ENDIAN
     and __BIG_ENDIAN are always defined
   - removed unused include (vxWorks.h is enough for byte order)

   Revision 1.34  2003/11/21 14:12:17  kamdem
   comments adjusted

   Revision 1.33  2003/11/21 13:06:58  kamdem
   endianess for vxworks to make it compatible to linux ... still in check !!!

   Revision 1.32  2003/10/13 18:07:33  neumann
   * Added Wake_Down_Queue() for select wakeup on write

   Revision 1.31  2003/08/14 15:10:12  langer
   - removed config.h again
   - removed "driver/" from some path
   -> more independant from BSP

   Revision 1.30  2003/08/08 16:51:41  wachendorf
   added #include <config.h>

   Revision 1.29  2003/07/24 14:57:59  tauche
   Added definitions for SUCCESS and ERR for compatibility with linux code

   Revision 1.28  2003/07/07 09:58:57  rutkowski
   - Implement a lock/unlock mechanism for global interrupts of the system.

   Revision 1.27  2003/06/27 13:03:55  rutkowski
   - Implement a filled waitqueue to use as mutex semaphore.

   Revision 1.26  2003/06/23 17:17:50  rutkowski
   - Replace the '#ifdef PPC' macro against '#if (CPU_FAMILY==PPC)',
     because the PPC macro is always declared. This change was required
     to be compatible to the MIPS architecture.

*******************************************************************************/

/** \file
   This file contains the includes and the defines specific to the vxWorks OS
*/

/** \defgroup SYS_DRV_VXWORKS Definitions for VxWorks
   These features are base on VxWorks 5.4.
*/

/* ============================= */
/* Global Includes               */
/* ============================= */

/* System includes*/

#include <vxworks.h>

#if (CPU_FAMILY==PPC)
#include <drv/intrCtl/ppc860Intr.h>
#include <drv/multi/ppc860Siu.h>
#include <drv/multi/ppc860Cpm.h>
#include <drv/sio/ppc860Sio.h>
#include <arch/ppc/ivPpc.h>
#endif

#include <drv/timer/timerDev.h>

#include <selectLib.h>
#include <iosLib.h>
#include <stdio.h>
#include <time.h>
#include <taskLib.h>
#include <cacheLib.h>
#include <logLib.h>
#include <intLib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <tickLib.h>
#include <math.h>
#include <common/src/sys_delayLib_vxworks.h>
#include <common/src/driverlib.h>


/* ============================= */
/*          Endianess            */
/* ============================= */

/** \defgroup SYS_DRV_VXWORKS_ENDIAN Endianess
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/
#ifndef __LITTLE_ENDIAN
#define __LITTLE_ENDIAN _LITTLE_ENDIAN
#endif
#ifndef __BIG_ENDIAN
#define __BIG_ENDIAN    _BIG_ENDIAN
#endif

#if (_BYTE_ORDER == _LITTLE_ENDIAN)
   /** byte order is little endian */
   #define __BYTE_ORDER __LITTLE_ENDIAN
#elif (_BYTE_ORDER == _BIG_ENDIAN )
   /** byte order is big endian */
   #define __BYTE_ORDER __BIG_ENDIAN
#else
   #error "Unknown Byteorder!"
#endif
/*@}*/

/* ============================= */
/* Compiler features             */
/* ============================= */

/** \defgroup SYS_DRV_VXWORKS_FEAT Compiler features
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/
/** Do not inline functions. */
#define INLINE
/** Define functions only local. */
#define LOCAL  static
/*@}*/

/* ============================= */
/* Function Macros               */
/* ============================= */

#if (CPU_FAMILY==PPC)
   #ifndef GetImmr
      #define GetImmr()                         vxImmrGet()
   #endif

   #define IMAP_ADDR                            vxImmrGet()
#endif

/** \defgroup SYS_DRV_VXWORKS_DELAY Delay functions
   \ingroup SYS_DRV_VXWORKS

   This section defines different delay functions
   with active waiting or with enabling the scheduler.

   \note "Wait_Milli_Sec" and "Wait_Micro_Sec" use the delay library
   "delayLib" from the "VxWorks / Tornado II FAQ"
    (Available at http://www.xs4all.nl/~borkhuis/vxworks/vxw_pt6.html#6.1)
*/
/*@{*/

/** Wait milliseconds with rescheduling.
   \note VxWorks taskdelay() is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick, but Wait() does not need to be very accurate
 */
#define Wait(time)                        taskDelay((time * sysClkRateGet() / 1000)+1)
/** Wait active milliseconds without rescheduling. */
#define Wait_Milli_Sec(time)              delayMsec (time)
/** Wait active microseconds without rescheduling. */
#define Wait_Micro_Sec(time)              delayUsec(time)

/*@}*/

/** \defgroup SYS_DRV_VXWORKS_MEMORY Memory functions
   \ingroup SYS_DRV_VXWORKS

   This section defines functions for allocating and freeing memory.
   If DEBUG is defined some functions to trace memory allocation are used
   (s. sys_debug.c)
*/
/*@{*/
#ifdef DEBUG
   /* define own functions for allocation and free */
   #define Alloc_Mem(size)                sys_dbgMalloc(size, __LINE__, __FILE__)
   #define Free_Mem(ptr)                  sys_dbgFree((void*)ptr, __LINE__, __FILE__)
   /* wrapper for os malloc and free in case of debug */
   #define sysdebug_malloc(size)          malloc(size)
   #define sysdebug_free(ptr)             free((void*)ptr)
#else
   /** Allocate memory. */
   #define Alloc_Mem(size)                malloc(size)
   /** Free memory. */
   #define Free_Mem(ptr)                  free((void*)ptr)
#endif


/** copy from user to kernel space
   \note Different return codes! */
#define copy_from_user(to, from, n)       memcpy(to, from, n)
/** copy from kernel to user space
   \note Different return codes! */
#define copy_to_user(to, from, n)         memcpy(to, from, n)

/*@}*/

/** \defgroup SYS_DRV_VXWORKS_EVENT Event handling functions
   \ingroup SYS_DRV_VXWORKS

   This section defines functions for event handling.
*/
/*@{*/

/** Initialize an event. */
#define Init_Event(queue) \
   (queue) = semBCreate(SEM_Q_FIFO, SEM_EMPTY)

/* FIXME RH, delete an event */
#define Delete_Event(queue)

/** Signal an event. */
#define Wake_Up_Event(queue)              {\
   int ret;                               \
   ret = semGive ((queue));   \
   if (ret != OK) \
   {  \
      logMsg("Failed to set event 0x%x\n",(UINT32)queue,0,0,0,0,0);\
   }  \
}

/** Reset event to initial state. */
#define Clear_Event(queue)                {  \
   semFlush (*(queue)); \
   semTake (*(queue), NO_WAIT);  \
}

/** Wait for an event with timeout.
   \attention This is an old definition, it will be removed in future!
   Returns FALSE / 0 on timeout.
   \note VxWorks delay is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick */
#define Sleep_On(sem, timeout)            ({int __ret = TRUE;     \
   if (ERROR == semTake((sem), (((timeout) * sysClkRateGet() / 1000)+1)) )  \
      __ret = FALSE;                                              \
   __ret;                                                         \
})

/** Wait for an event with timeout.
   Returns FALSE / 0 on timeout.
   \note VxWorks delay is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick */
#define Sleep_On_Event(event, timeout)            ({int __ret = TRUE;     \
   if (ERROR == semTake((event), (((timeout) * sysClkRateGet() / 1000)+1)))   \
      __ret = FALSE;                                              \
   __ret;                                                         \
})

/*@}*/

/** Initialize a queue. */
#define Init_WakeupQueue(queue)           selWakeupListInit(&queue)
/** Signal a read event */
#define Wake_Up_Queue(queue)              selWakeupAll(&queue, SELREAD)
/** Signal a write event */
#define Wake_Down_Queue(queue)            selWakeupAll(&queue, SELWRITE)
/** Wake up a single element in the queue */
#define Wake_Up_Node(node)                selWakeup(node)


/** \defgroup SYS_DRV_VXWORKS_MUTEX Mutex functions
   \ingroup SYS_DRV_VXWORKS

   This section has mutex definitions.
*/
/*@{*/

/** Create a mutex element. */
#define Sem_MutexInit(sem)                ((sem) = semMCreate(SEM_Q_FIFO))

#define Sem_BinaryInit(queue) \
   (queue) = semBCreate(SEM_Q_FIFO, SEM_FULL)

/** Lock a mutex section. */
#define Sem_Lock(sem)                     semTake(sem, WAIT_FOREVER)

/** Unlock a mutex section. */
#define Sem_Unlock(sem)                   semGive(sem)

/** Delete a mutex element. */
#define Sem_Free(sem)              	  semDelete(sem)
/*@}*/

/** \defgroup SYS_DRV_VXWORKS_CLOCK Time/Clock functions
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/



/** Returns system tick in milliseconds. */
#define Get_Tick() \
   (UINT32)(tickGet() * 1000 / sysClkRateGet())
/** Get CPU Frequency from BSP */
IMPORT UINT32 sys_Cpu_Clk;
/** Returns CPU clock in Hz. */
#define Cpu_Clk                           sys_Cpu_Clk

/*@}*/

/** \defgroup SYS_DRV_VXWORKS_INTERRUPT Enable/Disable global interrupts
   \ingroup SYS_DRV_VXWORKS
*/
/*@{*/

/*----------------------------------------------------------------------------*/
/* Lock mechanism for general processor interrupts                            */
/* Note: Please avoid to use the mechanism, because disabling all interrupts  */
/*       of the system is not a good solution.                                */
/*----------------------------------------------------------------------------*/
#define Lock_Variable                     int
#define Lock_Interrupts(var)              var = intLock()
#define Unlock_Interrupts(var)            intUnlock(var)
/*----------------------------------------------------------------------------*/

/*@}*/

/* lock task switching */
#define Lock_Tasks()                      taskLock()
/* unlock task switching */
#define Unlock_Tasks()                    taskUnlock()
/* start (kernel) thread */
#define OS_Start_Task(name, prio, ssize,funcptr, arg1, arg2) taskSpawn(name, prio, ssize,funcptr, arg1, arg2)
/* delete (kernel) thread */
#define OS_Delete_Task(taskId)            taskDelete(taskId)


/** Waits for a specified event to occur or timeout. Used in WaitEvent.
   \ingroup SYS_DRV_VXWORKS_EVENT

   \param wq         - wait queue
   \param condition  - condition to occur
   \param timeout    - time out in ms
   \note VxWorks delay is very inaccurate, and cannot wait for less than one tick
   so we must wait at least one tick
*/
#define __WaitEvent(wq, condition, timeout)                 \
do {                                                        \
   int _timeout = (((timeout * sysClkRateGet()) / 1000)+1); \
   while (!(condition))                                     \
   {                                                        \
      if (semTake (wq, _timeout) == ERROR)                  \
      {                                                     \
         /* terminate loop on timeout */                    \
         break;                                             \
      }                                                     \
   }                                                        \
} while (0)

/** Waits for a specified event to occur or timeout.
   \ingroup SYS_DRV_VXWORKS_EVENT

   \param wq         - wait queue
   \param condition  - condition to occur
   \param timeout    - time out in ms
   \return  1 if event occured and condition is true, otherwise 0
*/
#define WaitEvent(wq, condition, timeout)             \
({                                                    \
   int __ret = 0;                                     \
   if (!(condition))                                  \
      __WaitEvent(wq, condition, (timeout));          \
   if ((condition))                                   \
      __ret = 1;                                      \
   __ret;                                             \
})


#if PPC

   /** function to allocate dualported memory (only PPC) */
   #define CPM_Dpalloc(size)                 sysGetDPRam(size)
   /** error code for CPM_Dpalloc (only PPC) */
   #define CPM_DP_NOSPACE                    ERROR
   /** function to allocate uncached memory (only PPC) */
   #define Host_Alloc(size)                  cacheDmaMalloc(size)

#endif


#ifdef DEBUG
   /** Makes a printout if the expression is not TRUE. */
   #define ASSERT(expr) \
      if(!(expr)) { \
         logMsg ( "\n" __FILE__ ":%d: Assertion " #expr " failed!\n\r",__LINE__,0,0,0,0,0); \
      }
#else
   #define ASSERT(cond)
#endif



/* ============================= */
/* Global function declaration   */
/* ============================= */

/* VxWorks BSP */
IMPORT UINT32 vxImmrGet(void);              /* get the base address through the IMMR register     */
IMPORT int sysGetDPRam(int size);           /* get an area of the CPM DPRAM for parameters and BD */

IMPORT int  Enable_Irq (int irq);           /* Enable Irq */
IMPORT void Disable_Irq(int irq);           /* Disable Irq */

/* Device Init. Called form Board_DevOpen when the device has been opened for
   the first time. The device structure will be cached */
extern int OS_InitDevice (int nDevn);
/* Device is stopped. Called on device close */
extern void OS_StopDevice (void* pDevice);
/* Install VINETIC ISR   */
extern void OS_Install_IRQ_Handler(void* pDevice, int nIrq);
/* Uninstall VINETIC ISR */
extern void OS_UnInstall_IRQ_Handler(void* pDevice);

/* ============================= */
/* Global structures declaration */
/* ============================= */

/* ============================= */
/* typedefs                      */
/* ============================= */
/*--------------------------------------------------*/
/* Timer element needed for the Service Timer.      */
/* Not yet supported for VxWorks.                   */
/*--------------------------------------------------*/
typedef struct
{
    timer_t   Timer;                    /* Timer handle for subsequent accesses */
    struct    itimerspec     timeToSet; /* time to program */
} TIMER_ELEMENT;

/** Use semaphores for events. */
typedef SEM_ID                   SYS_EVENT;
/** Polling is used to wakeup an application. */
typedef SEL_WAKEUP_LIST          SYS_POLL;
/** element in the wake-up list */
typedef SEL_WAKEUP_NODE          SYS_POLL_NODE;

#define TIMESTAMP int

#define  SYS_NOSELECT   0x00000000
#define  SYS_DATAIN     0x00000001
#define  SYS_EXCEPT     0x00000002
#define  SYS_DATAOUT    0x00000004

/* return values */
#define SUCCESS 0
#define ERR  -1

#endif /* _SYS_DRV_VXWORKS_H */

