/******************************************************************************
       Copyright (c) 2004, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for 
   the program, to the extent permitted by applicable law.  Except when     
   otherwise stated in writing the copyright holders and/or other parties
   provide the program "as is" without warranty of any kind, either         
   expressed or implied, including, but not limited to, the implied         
   warranties of merchantability and fitness for a particular purpose. The
   entire risk as to the quality and performance of the program is with     
   you.  should the program prove defective, you assume the cost of all     
   necessary servicing, repair or correction.
                                                                            
   In no event unless required by applicable law or agreed to in writing    
   will any copyright holder, or any other party who may modify and/or
   redistribute the program as permitted above, be liable to you for
   damages, including any general, special, incidental or consequential
   damages arising out of the use or inability to use the program
   (including but not limited to loss of data or data being rendered
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or
   other party has been advised of the possibility of such damages.

******************************************************************************
   File        : $RCSfile: drv_inca_oak_vxworks.c,v $
   Revision    : $Revision: 1.2 $
   Date        : $Date: 2004/05/07 14:48:52 $
   Description : VxWorks driver for the INCA-IP OAK DSP

******************************************************************************/
#ifdef VXWORKS

#include <vxworks.h>
#include <stdlib.h>
#include <string.h>
#include <logLib.h>
#include <iosLib.h>
#include <selectLib.h>
#include <semLib.h>

#include "INCAOAK_DeviceDriver.c"

#include <ioctl.h>
#include "drv_inca_oak_interface.h"
#include "drv_inca_oak_api.h"
#include "inca_oak_version.h"

/*----------------------------------------------------------------------------*/
/* File access names                                                          */
/*----------------------------------------------------------------------------*/
LOCAL const char string_oak_voice_chan0[] = {"/Voice/0"};
LOCAL const char string_oak_voice_chan1[] = {"/Voice/1"};
LOCAL const char string_oak_command[]     = {"/Command"};
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/


/****************************************************************************
Description : Open routine for the device driver
Arguments   :
Return      :
Remarks     :
****************************************************************************/
LOCAL STATUS OAK_Open(OAK_DEV *pDev, INT8 *pAnnex, INT32 flags)
{
   OAK_MB_DEV_STR    *pMBDev;
   BOOL     bcommand = FALSE;

   if (!memcmp(pAnnex, string_oak_voice_chan0,   
      sizeof(string_oak_voice_chan0) - 1))
   {
      /* voice channel 0 */
      pMBDev = &pDev->VoiceMB[0];
   }
   else if(!memcmp(pAnnex, string_oak_voice_chan1,
      sizeof(string_oak_voice_chan1) - 1))
   {
      /* voice channel 1 */
      pMBDev = &pDev->VoiceMB[1];
   }
   else if(!memcmp(pAnnex, string_oak_command,
      sizeof(string_oak_command) - 1))
   {
      /* command type */
      pMBDev = &pDev->CommandMB;
      bcommand = TRUE;
   }
   else
   {
      errno = S_iosLib_OAKMB_MSG_DWORD;
      return (ERROR);
   }
   
   /* device is already installed */
   if ((OK) == oak_common_open(pDev, pMBDev, bcommand))
   {
      /* installation successful */
       return (INT32) pMBDev;
   }
   else
   {
        /* installation failed */
      errno = S_iosLib_OAKMB_DEV_INST ;
      return (ERROR);
   }
}


/****************************************************************************
Description : ioctrl function of mailbox driver
Arguments   : VxWorks specific parameter
Return      : result of the operation ('OK' or 'ERROR')
Remarks     :
****************************************************************************/
LOCAL STATUS OAK_Ioctl(OAK_MB_DEV_STR *pMBDev, INT32 cmd, INT32 arg )
{
   void     *pTemp;
   STATUS   retvalue;
   int      irqLockNum;
   OAK_GLOBAL_REG  *pBaseGlobal;
   UINT32   reg;

   switch(cmd)
   {
   case FIOSELECT :
      selNodeAdd((SEL_WAKEUP_LIST*)&(pMBDev->oak_wklist),(SEL_WAKEUP_NODE*)arg);
      /* upstream queue */
      if( selWakeupType((SEL_WAKEUP_NODE*) arg ) == SELREAD )
      {
         /* queue is empty */
         if( pMBDev->upstrm_fifo.pWrite != pMBDev->upstrm_fifo.pRead )
            selWakeup((SEL_WAKEUP_NODE*) arg);
      }
      /* downstream queue */
      else
      {
         /* queue has space */
         if(oak_fifo_write_space(&pMBDev->dwstrm_fifo) != 0)
         {
            selWakeup((SEL_WAKEUP_NODE*) arg);
         }
      }
      return (OK);

   case FIOUNSELECT :
      selNodeDelete(&(pMBDev->oak_wklist), (SEL_WAKEUP_NODE*) arg);
      return (OK);

   case FIO_INCAIP_OAK_EVENT_REG:
      pMBDev->event_mask = arg;
      return( OK );

   case FIO_INCAIP_OAK_EVENT_UNREG:
      pMBDev->event_mask = 0xFFFFFFFF;
      return (OK);

   case FIO_INCAIP_OAK_MB_READ:
      /* Read the MailBox */
      return oak_read_fifo(pMBDev,(OAK_READWRITE*)arg, NO_WAIT);

   case FIO_INCAIP_OAK_MB_WRITE:
      /* Write into the mailbox */
      if(pMBDev->devID == command) 
         return oak_write_cmd_mailbox(pMBDev, (OAK_READWRITE*) arg);
      else
         return oak_write_fifo(pMBDev, (OAK_READWRITE*) arg);
         
   case FIO_INCAIP_OAK_MB_WR_ACCESS:
      /* Commands are sent to the mailbox directly    */
      /* or direct mailbox access for both mailboxes  */
      pBaseGlobal = pMBDev->pBaseGlobal;
      reg         = pMBDev->MBC_Mask01;

      irqLockNum = intLock();

      /* disable the interrupts of the mailbox to make the write access */
      pBaseGlobal->MBC_MSK01 |= reg;
      pBaseGlobal->MBC_MSK10 |= reg;

      intUnlock(irqLockNum);

      /* write the package know directly to the mailbox */
      retvalue = oak_write_cmd_mailbox(pMBDev, (OAK_READWRITE*) arg);

      irqLockNum = intLock();

      /* enable the interrupts of the mailbox after the write access */
      pBaseGlobal->MBC_MSK01 &= ~reg;
      pBaseGlobal->MBC_MSK10 &= ~reg;

      intUnlock(irqLockNum);

      return retvalue;

   case FIO_INCAIP_OAK_DWNLOAD:
      /* DownLoad the File to OAK */
      if(pMBDev->devID == command)
         return oak_download(pMBDev, (OAK_FWDWNLD*)arg);
      else
         return (ERROR);

   case FIO_INCAIP_OAK_GETVERSION:
      *((char**)arg) = ((char*)string_oak_version) + 5;
      return (OK);

   case FIO_INCAIP_RST_QUEUE :
      /* Reset the QUEUE Upstream/downstream Note : The interrupts and the
      tasks should be not be running if the queues have to be disabled */
      if( arg == UPSTREAM )
         pTemp = &(pMBDev->upstrm_fifo);
      else
         pTemp = &(pMBDev->dwstrm_fifo);
      oak_fifo_clear((OAK_FIFO_MB *) pTemp);
      return (OK);

      /* Reset of the DSP */
   case  FIO_INCAIP_OAK_RESET:
      return oak_reset_dsp(pMBDev->pBaseGlobal);

      /* Restart of the DSP */
   case  FIO_INCAIP_OAK_RESTART:
      return oak_restart_dsp(pMBDev->pBaseGlobal);

#if (OAK_FIFO_BLOCKING_WRITE == TRUE)
      /* Set the mailbox TX FIFO to blocking */
   case  FIO_INCAIP_TXFIFO_SET:
      if(pMBDev->devID == command)
      {
         return (ERROR); /* not supported for this command MB */
      }
      else
      {
         if (arg > 0)
         {
            pMBDev->bBlockWriteMB = TRUE;
         }
         else
         {
            pMBDev->bBlockWriteMB = FALSE;
            semGive(pMBDev->sem_write_fifo);
         }
         return (OK);
      }

      /* Get the mailbox TX FIFO to blocking */
   case  FIO_INCAIP_TXFIFO_GET:
      if(pMBDev->devID == command)
      {
         return (ERROR); /* not supported for this command MB */
      }
      else
      {
         *((UINT32*)arg) = pMBDev->bBlockWriteMB;
         return (OK);
      }
#endif /* OAK_FIFO_BLOCKING_WRITE */

   default:
      break ;
   }
   /* set errno to unknown ioctrl */
   errno =  S_iosLib_OAKMB_UNK_IOCTL ;
   return (ERROR);
}

/****************************************************************************
Description:
   Allows the upper layer to register a callback function either for
   downstream (tranmsit mailbox space available) or for upstream (read data 
   available)
Arguments:
   type     - DSP device entity ( 1 - command, 2 - voice0, 3 - voice1)
   dir      - Direction (1 - upstream, 2 - downstream)
   callback - Callback function to register
Return Value:
   OK    - Callback registered successfully
   ERROR - Wrong DSP device entity (only 1-3 supported)
           or Callback already registered
Remarks:
   None.
****************************************************************************/
int inca_oak_data_callback_register(OAK_MB_DEV_STR *pMBDev, UINT dir, void (*callback)(DEVTYPE type))
{
   if (callback == NULL) {
      return (ERROR);
   }
   /* Enter the desired callback function */
   switch(dir) {
   case 1:
      if (pMBDev->up_callback != NULL) {
         return (ERROR);
      } else {
         pMBDev->up_callback = callback;
      }
      break;
   case 2:
       if (pMBDev->down_callback != NULL) {
         return (ERROR);
      } else {
         pMBDev->down_callback = callback;
      }
      break;
   default:
      break;
   }
   return(OK);
}

/****************************************************************************
Description:
   Allows the upper layer to un-register the callback function previously
   registered using inca_oak_data_callback_register
Arguments:
   type     - DSP device entity ( 1 - command, 2 - voice0, 3 - voice1)
   dir      - Direction (1 - upstream, 2 - downstream)
Return Value:
   OK    - Callback registered successfully
   ERROR - Wrong DSP device entity (only 1-3 supported)
           or Callback not registered
Remarks:
   None.
****************************************************************************/
int inca_oak_data_callback_unregister(OAK_MB_DEV_STR *pMBDev, UINT dir)
{
   /* Delete the desired callback function */
   switch(dir) {
   case 1:
      if (pMBDev->up_callback == NULL) {
         return (ERROR);
      } else {
         pMBDev->up_callback = NULL;
      }
      break;
   case 2:
       if (pMBDev->down_callback == NULL) {
         return (ERROR);
      } else {
         pMBDev->down_callback = NULL;
      }
      break;
   default:
      break;
   }
   return(OK);
}

/****************************************************************************
Description:
   Allows the upper layer to register a callback function for event
   notification. 
Arguments:
   type     - DSP device entity ( 1 - command, 2 - voice0, 3 - voice1)
   mask     - mask according to MBC_ISR content
   callback - Callback function to register
Return Value:
   OK    - Callback registered successfully
   ERROR - Wrong DSP device entity (only 1-3 supported)
           or Callback already registered
Remarks:
   None.
****************************************************************************/
int inca_oak_event_callback_register(OAK_MB_DEV_STR *pMBDev, UINT mask, void (*callback)(UINT mask))
{
   if (callback == NULL) {
      return (ERROR);
   }
   /* Enter the desired callback function */
   if (pMBDev->event_callback != NULL) {
      return (ERROR);
   } else {
      pMBDev->event_callback = callback;
   }
   return(OK);
}

/****************************************************************************
Description:
   Allows the upper layer to un-register the callback function previously
   registered using inca_oak_event_callback_register
Arguments:
   type     - DSP device entity ( 1 - command, 2 - voice0, 3 - voice1)
   mask     - mask according to MBC_ISR content
Return Value:
   OK    - Callback registered successfully
   ERROR - Wrong DSP device entity (only 1-3 supported)
           or Callback not registered
Remarks:
   None.
****************************************************************************/
int inca_oak_event_callback_unregister(OAK_MB_DEV_STR *pMBDev, UINT mask)
{
   /* Delete the desired callback function */
    if (pMBDev->event_callback == NULL) {
      return (ERROR);
   } else {
      pMBDev->event_callback = NULL;
   }
   return(OK);
}


/****************************************************************************
Description : Initialization of Device Driver
Arguments   :
Return      :
Remarks     :
****************************************************************************/
STATUS OAK_DevCreate(void)
{
   int      drv_num;
   OAK_DEV  *pDev ;

   /* reset the device before initializing the device driver */
   if ((ERROR) == oak_reset_dsp((OAK_GLOBAL_REG *)OAK_BASEADDRESS))
      return (ERROR);

   /* get structure element for the device */
   pDev = malloc(sizeof(OAK_DEV));

   if (pDev == NULL)
   {
      return (ERROR) ;
   }

   /* init the parameter in the device structure */
   oak_init_structures(pDev);
   
   /* connect the interrupt service routine */
   if (intConnect( INUM_TO_IVEC(INT_NUM_IM1_IRL20), 
      (VOIDFUNCPTR)oak_upstream_cmd_isr, (int)pDev) == (ERROR))
   {
      errno = S_iosLib_OAKMB_INIT_DRV ;
      return (ERROR);
   }

   if (intConnect( INUM_TO_IVEC(INT_NUM_IM1_IRL21), 
      (VOIDFUNCPTR)oak_upstream_voice_isr, (int)pDev) == (ERROR))
   {
      errno = S_iosLib_OAKMB_INIT_DRV ;
      return (ERROR);
   }

   if (intConnect( INUM_TO_IVEC(INT_NUM_IM1_IRL25), 
      (VOIDFUNCPTR)oak_downstream_voice_isr, (int)pDev) == (ERROR))
   {
      errno = S_iosLib_OAKMB_INIT_DRV ;
      return (ERROR);
   }

   /* register the driver */
   drv_num = iosDrvInstall( NULL, NULL, OAK_Open, (FUNCPTR)oak_common_close, 
      NULL , NULL, OAK_Ioctl);

   if (drv_num <= 0)
   {
      errno = S_iosLib_OAKMB_INIT_DRV ;
      return (ERROR);
   }

   if(iosDevAdd( (DEV_HDR*) pDev, DRV_INCA_OAK_NAME,drv_num) == (ERROR))
   {
      errno = S_iosLib_OAKMB_INIT_DRV ;
      return (ERROR);
   }

   return (OK);
}
























#endif /* VXWORKS */

