/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
#ifndef __INCA_PHY_REGS_H__
#define __INCA_PHY_REGS_H__

/***********************************************************************
    MDIO PHY registers 
*/


/*----- PHY register 0: Basic Control Register -----*/
/* PHYReg0 - reserved bits: [0..6] */
#define  INCA_IP_PHY_REG0_BCR_RESERVE_MASK   0x0000007F 

/* PHYReg0 - Bit 15: RESET */
/* 0: normal operation, 1: software reset */
#define  INCA_IP_PHY_REG0_BCR_RESET          15 
#define  INCA_IP_PHY_REG0_BCR_RESET_MASK     (1 << INCA_IP_PHY_REG0_BCR_RESET)

/* PHYReg0 - Bit 13: Speed Select */
/* 0: 10 Mbit/s, 1: 100 Mbit/s */
#define  INCA_IP_PHY_REG0_BCR_SS             13
#define  INCA_IP_PHY_REG0_BCR_SS_MASK        (1 << INCA_IP_PHY_REG0_BCR_SS)

/* PHYReg0 - Bit 12: Auto Negotiation Enable */
#define  INCA_IP_PHY_REG0_BCR_ANE            12
#define  INCA_IP_PHY_REG0_BCR_ANE_MASK       (1 << INCA_IP_PHY_REG0_BCR_ANE)

/* PHYReg0 - Bit 9: Restart Negotiation Enable */
#define  INCA_IP_PHY_REG0_BCR_RAN            9
#define  INCA_IP_PHY_REG0_BCR_RAN_MASK       (1 << INCA_IP_PHY_REG0_BCR_RAN)

/* PHYReg0 - Bit 8: Duplex Mode */
/* 0: Half duplex, 1: Full duplex */
#define  INCA_IP_PHY_REG0_BCR_DM             8
#define  INCA_IP_PHY_REG0_BCR_DM_MASK        (1 << INCA_IP_PHY_REG0_BCR_DM)



/*----- PHY register 1: Basic Status Register  -----*/
/* PHYReg1 - reserved bits: [6..10] */
#define  INCA_IP_PHY_REG1_BSR_RESERVE_MASK   0x000003C0

/* PHYReg 1 - Bit 0: Extended Capabilities */
/* 0: does not support ext. capabilities, 1: support */
#define INCA_IP_PHY_REG1_BSR_EXC          0
#define INCA_IP_PHY_REG1_BSR_EXC_MASK     (1 << INCA_IP_PHY_REG1_BSR_EXC)

/* PHYReg 1 - Bit 1: Jabber Detect */
/* 0: No jabber condition detected, 1: jabber condition detected */
#define INCA_IP_PHY_REG1_BSR_JD           1
#define INCA_IP_PHY_REG1_BSR_JD_MASK      (1 << INCA_IP_PHY_REG1_BSR_JD)

/* PHYReg 1 - Bit 2: Link Status */
/* 0: link down, 1: link up */
#define INCA_IP_PHY_REG1_BSR_LS           2
#define INCA_IP_PHY_REG1_BSR_LS_MASK      (1 << INCA_IP_PHY_REG1_BSR_LS)

/* PHYReg 1 - Bit 3: Auto-Negotiate Ability */
/* 0: Unable to perform auto-negotiate function, 1: Able to perform auto-negotiate function */
#define INCA_IP_PHY_REG1_BSR_ANA          3
#define INCA_IP_PHY_REG1_BSR_ANA_MASK     (1 << INCA_IP_PHY_REG1_BSR_ANA)

/* PHYReg 1 - Bit 4: Remote Fault */
/* 0: No remote fault, 1: Remote fault condition detected */
#define INCA_IP_PHY_REG1_BSR_RF           4
#define INCA_IP_PHY_REG1_BSR_RF_MASK      (1 << INCA_IP_PHY_REG1_BSR_RF)

/* PHYReg 1 - Bit 5: Auto-Negotiate Complete */
/* 0: Auto-negotiate process not completed, 1: Auto-negotiate process completed */
#define INCA_IP_PHY_REG1_BSR_ANC          5
#define INCA_IP_PHY_REG1_BSR_ANC_MASK     (1 << INCA_IP_PHY_REG1_BSR_ANC)


/*----- PHY register 2: Register 2, PHY Identifier 1 -----*/
/* PHYReg0 - reserved bits:  */
#define  INCA_IP_PHY_REG2_ID1_RESERVE_MASK   0x00000000 

/*----- PHY register 3: Basic Control Register -----*/
/* PHYReg0 - reserved bits:  */
#define  INCA_IP_PHY_REG0_ID2_RESERVE_MASK   0x00000000 


/*----- PHY register 4: Auto Negotiation Advertisement Register (PHY_ANAR)  -----*/
/* PHYReg4 - reserved bits: [11..12] */
#define INCA_IP_PHY_REG4_ANAR_RESERVE_MASK   0x00001800

/* PHYReg 4 - Bit 10: Pause Operation */
/* 0: Pause Operation is not supported by MAC , 1: Pause Operation is supported by MAC */
#define INCA_IP_PHY_REG4_ANAR_PO       10
#define INCA_IP_PHY_REG4_ANAR_PO_MASK  (1 << INCA_IP_PHY_REG4_ANAR_PO)

/* PHYReg 4 - Bit 13: Remote Fault */
/* 0: No remote fault, 1: Remote fault detected */
#define INCA_IP_PHY_REG4_ANAR_RF       13
#define INCA_IP_PHY_REG4_ANAR_RF_MASK  (1 << INCA_IP_PHY_REG4_ANAR_RF)

/* PHYReg 4 - Bit 15: Next Page */
/* 0: No next page ability, 1: Next page capable */
#define INCA_IP_PHY_REG4_ANAR_NP       15
#define INCA_IP_PHY_REG4_ANAR_NP_MASK  (1 << INCA_IP_PHY_REG4_ANAR_NP)


/*----- PHY register 5: Auto Negotiation Link Partner Ability Register (PHY_ANLPAR)  -----*/
/* PHYReg5 - reserved bits: [11..12] */
#define INCA_IP_PHY_REG5_ANLPAR_RESERVE_MASK   0x00001800

/* PHYReg 5 - Bit 10: Pause Operation */
/* 0: Pause Operation is not supported by remote MAC , 1: Pause Operation is supported */
#define INCA_IP_PHY_REG5_ANLPAR_PO        10
#define INCA_IP_PHY_REG5_ANLPAR_PO_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_PO)

/* PHYReg 5 - Bit 13: Remote Fault */
/* 0: No remote fault, 1: Remote fault detected */
#define INCA_IP_PHY_REG5_ANLPAR_RF        13
#define INCA_IP_PHY_REG5_ANLPAR_RF_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_RF)

/* PHYReg 5 - Bit 14: Acknowledge */
/* 0: Link code word not yet received, 1: Link code word received from partner */
#define INCA_IP_PHY_REG5_ANLPAR_ACK       14
#define INCA_IP_PHY_REG5_ANLPAR_ACK_MASK  (1 << INCA_IP_PHY_REG5_ANLPAR_ACK)

/* PHYReg 5 - Bit 15: Next Page */
/* 0: No next page ability, 1: Next page capable */
#define INCA_IP_PHY_REG5_ANLPAR_NP        15
#define INCA_IP_PHY_REG5_ANLPAR_NP_MASK   (1 << INCA_IP_PHY_REG5_ANLPAR_NP)


/*----- PHY register 6: Auto Negotiation Expansion Register (PHY_ANER) -----*/
/* PHYReg6 - reserved bits: [5..15] */
#define INCA_IP_PHY_REG6_ANER_RESERVE_MASK   0x0000FFE0

/* PHYReg 6 - Bit 0: Link Partner Auto-Negotiation Able */
/* 0: Link partner does not have auto-negotiation ability, 
   1: Link partner has auto-negotiation ability */
#define INCA_IP_PHY_REG6_ANER_LPANA       0
#define INCA_IP_PHY_REG6_ANER_LPANA_MASK  (1 << INCA_IP_PHY_REG6_ANER_LPANA)

/* PHYReg 6 - Bit 1: Page Received */
/* 0: New page not yet received, 1: New page received */
#define INCA_IP_PHY_REG6_ANER_PR          1
#define INCA_IP_PHY_REG6_ANER_PR_MASK     (1 << INCA_IP_PHY_REG6_ANER_PR)

/* PHYReg 6 - Bit 2: Next Page Able */
/* 0: Local device does not have next page ability, 
   1: Local device has next page ability */
#define INCA_IP_PHY_REG6_ANER_NPA         2
#define INCA_IP_PHY_REG6_ANER_NPA_MASK    (1 << INCA_IP_PHY_REG6_ANER_NPA)

/* PHYReg 6 - Bit 3: Link Partner Next Page Able */
/* 0: Link partner does not have next page ability, 
   1: Link partner has next page ability */
#define INCA_IP_PHY_REG6_ANER_LPNPA       3
#define INCA_IP_PHY_REG6_ANER_LPNPA_MASK  (1 << INCA_IP_PHY_REG6_ANER_LPNPA)

/* PHYReg 6 - Bit 4: Parallel Detection Fault */
/* 0: No fault detected by parallel detection logic, 
   1: Fault detected by parallel detection logic */
#define INCA_IP_PHY_REG6_ANER_PDF         4
#define INCA_IP_PHY_REG6_ANER_PDF_MASK    (1 << INCA_IP_PHY_REG6_ANER_PDF)


/*----- PHY register 16: Silicon Revision Register (PHY_SRR) -----*/
/* PHYReg16 - reserved bits: [0..5], [10..15] */
#define INCA_IP_PHY_REG16_SRR_RESERVE_MASK   0x0000FC3F

/*----- PHY register 17: Mode Control / Status register (PHY_MCSR) -----*/
/* PHYReg17 - reserved bits: [2..12], [14..15] */
#define INCA_IP_PHY_REG17_MCSR_RESERVE_MASK   0x0000DFFC

/* PHYReg 17 - Bit 1: ENERGYON signal state */
/* 0: no valid energy from the line within 256ms, 1: signal detected */
#define INCA_IP_PHY_REG17_MCSR_ENRG_BIT	   1
#define INCA_IP_PHY_REG17_MCSR_ENRG_MASK     (1 << INCA_IP_PHY_REG17_MCSR_ENRG_BIT)

/*----- PHY register 18: Special Modes Register (PHY_SMR) -----*/
/* PHYReg18 - reserved bits: [8..15] */
#define INCA_IP_PHY_REG18_SMR_RESERVE_MASK   0x000000FF

/*----- PHY register 29: Interrupt Source Register (PHY_ISR) -----*/
/*----- PHY register 30: Interrupt Mask Register (PHY_IMR)   -----*/
/* PHYReg29/30 - reserved bits: [0] [8..15] */
#define INCA_IP_PHY_REG_INT_RESERVE_MASK   0x0000FF01

/* PHYReg 30: interrupt mask bit field [1..7] */
#define  INCA_IP_MDIO_IMR_FIELD_MASK     0x000000FE


/* PHYReg 29/30 - Bit 1: Auto-Negotiation Page Received */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANPREC_BIT       1
#define INCA_IP_PHY_REG_INT_ANPREC_MASK      (1 << INCA_IP_PHY_REG_INT_ANPREC_BIT)

/* PHYReg 29/30 - Bit 2: Parallel Detection Fault */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_PDFAULT_BIT      2
#define INCA_IP_PHY_REG_INT_PDFAULT_MASK     (1 << INCA_IP_PHY_REG_INT_PDFAULT_BIT)

/* PHYReg 29/30 - Bit 3: Auto-Negotiation LP Acknowledge */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANLPACK_BIT      3
#define INCA_IP_PHY_REG_INT_ANLPACK_MASK     (1 << INCA_IP_PHY_REG_INT_ANLPACK_BIT)

/* PHYReg 29/30 - Bit 4: Link Down (link status negated) */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_LDOWN_BIT        4
#define INCA_IP_PHY_REG_INT_LDOWN_MASK       (1 << INCA_IP_PHY_REG_INT_LDOWN_BIT)

/* PHYReg 29/30 - Bit 5: Remote Fault Detected */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_RFDETECT_BIT     5
#define INCA_IP_PHY_REG_INT_RFDETECT_MASK    (1 << INCA_IP_PHY_REG_INT_RFDETECT_BIT)

/* PHYReg 29/30 - Bit 6: Auto-Negotiation Complete */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ANCOMPL_BIT      6
#define INCA_IP_PHY_REG_INT_ANCOMPL_MASK     (1 << INCA_IP_PHY_REG_INT_ANCOMPL_BIT)

/* PHYReg 29/30 - Bit 7: ENERGYON Generated */
/* 0: No interrupt / Interrupt source is masked, 
   1: Interrupt generated / Interrupt source is enabled */
#define INCA_IP_PHY_REG_INT_ENRGON_BIT       7
#define INCA_IP_PHY_REG_INT_ENRGON_MASK      (1 << INCA_IP_PHY_REG_INT_ENRGON_BIT)

/*----- PHY register 31: PHY Special Control/Status Register (PHY_SCSR) -----*/
/* PHYReg31 - reserved bits: [0..1], [5..11], [13..15] */
#define INCA_IP_PHY_REG31_SCSR_RESERVE_MASK   0x0000EFE3

/* PHYReg31 - Bit [2..4]: Speed Indication */
/* 001B: 10Base-T half-duplex
   101B: 10Base-T full-duplex
   010B: 100Base-TX half-duplex
   110B: 100Base-TX full-duplex */
#define INCA_IP_PHY_REG31_SCSR_SPEED_GET(value)  ( ((value) >> 2) & ((1 << 3) - 1) )
#define INCA_IP_PHY_REG31_SCSR_SPEED_SET(value)  ( ( ((1 << 3) - 1) &(value) ) << 2 )

/* PHYReg31 - Bit 12: Auto-Negotiation Done indication (Autodone) */
/* 0: Auto-negotiation is not done or disabled 
   1: IAuto-negotiation is done */
#define INCA_IP_PHY_REG31_SCSR_AUTODONE         12
#define INCA_IP_PHY_REG31_SCSR_AUTODONE_MASK    (1<<INCA_IP_PHY_REG31_SCSR_AUTODONE)

/*
    END: MDIO PHY registers 
*************************************************************************/
#endif /* __INCA_PHY_REGS_H__ */
