/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   Module      : IncaSwitchAmdix.h
   Date        : 01.12.2003
   Description :
   
   $Log: IncaSwitchAmdix.h,v $


   
******************************************************************************/
#ifndef _INCASWITCHAMDIX_H
#define _INCASWITCHAMDIX_H

/* switch ON/OFF debugging, only for testing */
//#define AMDIX_DEBUG_ENABLE
#undef  AMDIX_DEBUG_ENABLE

/* -------------------------------------------------------------------------- */
/* Basic macro definitions                                                    */
/* -------------------------------------------------------------------------- */
/* tbd */

/* mode definitions of the Auto MDIX */
#define INCA_IP_AUTOMDIX_MODE_RESET       0
#define INCA_IP_AUTOMDIX_MODE_MDI         1
#define INCA_IP_AUTOMDIX_MODE_MDIX        2


/* control struct for handle the Ethernet PHY port (PC/LAN) */
typedef struct
{
   UINT32            AutoMDIX_ON ;  /* switch ON / OFF the autoMDIX feature */
   UINT32            currMode ;     /* RESET-mode: PC: DETECT RX and TX; LAN: MDI
                                       MDI-mode:   TX --> INCA_IP RX
                                                   incoming RX --> INCA_IP TX
                                       MDIX-mode:  incoming TX --> INCA_IP TX
                                                   incoming RX --> INCA_IP RX 
                                    */
   UINT32            currState ;    /* reset:             after power on, 
                                       wait for "linkup": do auto MDIX, 
                                       "linkup":          it works 
                                    */
   sw21553_PhyCallbackFunction oldCallback;
   
   /* external functions used for external board configuration */
   UINT32 (*pGetAutoMDIXSupportPOL)(void);
   UINT32 (*pSetAutoMDIXModeStartup)(void);
   void   (*pSetupAutoMDIXModeGPIO)(void);
   UINT32 (*pSetAutoMDIXModeMDI)(void);
   UINT32 (*pSetAutoMDIXModeMDIX)(void);
   UINT32 (*pSetAutoMDIXModeRESET)(void);
   int    (*pGetAutoMDIXMode)(void);
   int    (*pCheckAutoMDIXPortSetup)(void);

} swAutoMDIXstruct;


int sw21553_AmdixInit(void);
void sw21553_AmdixStart(int port);
void sw21553_AmdixEnergy(int port);
void sw21553_AmdixRetry(int port);
void sw21553_AmdixTriesTout(int port);

/*******************************************************************************
*   forward declaration for the external user-defined init functions 
*   to set the auto MDIX featuere. This functions must be provided by the BSP
*/
/* PC-Port */
extern UINT32 sysGetAutoMDIX_PC_ON_OFF(void);
extern UINT32 sysGetAutoMDIX_PC_StatusDelay(void);

extern UINT32 sysGetAutoMDIX_PC_Support_POL(void);
extern UINT32 sysGetAutoMDIX_PC_LinkTries(void);
extern UINT32 sysSetAutoMDIX_PC_StartupMode(void);
extern void   sysSetupAutoMDIX_PC_GPIO(void);
extern UINT32 sysSetAutoMDIX_PC_MDIMode(void);
extern UINT32 sysSetAutoMDIX_PC_MDIXMode(void);
extern UINT32 sysSetAutoMDIX_PC_RESETMode(void);
extern int    sysGetAutoMDIX_PC_Mode(void);
extern int    sysCheckAutoMDIX_PC_PortSetup(void);

/* LAN-Port */
extern UINT32 sysGetAutoMDIX_LAN_ON_OFF(void);
extern UINT32 sysGetAutoMDIX_LAN_StatusDelay(void);

extern UINT32 sysGetAutoMDIX_LAN_Support_POL(void);
extern UINT32 sysGetAutoMDIX_LAN_LinkTries(void);
extern UINT32 sysSetAutoMDIX_LAN_StartupMode(void);
extern void   sysSetupAutoMDIX_LAN_GPIO(void);
extern UINT32 sysSetAutoMDIX_LAN_MDIMode(void);
extern UINT32 sysSetAutoMDIX_LAN_MDIXMode(void);
extern UINT32 sysSetAutoMDIX_LAN_RESETMode(void);
extern int    sysGetAutoMDIX_LAN_Mode(void);
extern int    sysCheckAutoMDIX_LAN_PortSetup(void);

/* configure and setup the used AutoMDIX Ports */
extern UINT32 sysGetAutoMDIX_PHY_ResetDelay(void);
extern void   sysSetupAutoMDIX_Ports(void);

#endif /* _INCASWITCHAMDIX_H */
