/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   Module      : IncaSwitchApi.c
   Date        : 13.05.2003
   Description : provide an API to read and configure the INCA-IP ethernet parts
                 like PHY's and switch module



   $Log: inca_switch_api.c,v $
   Revision 1.1.2.1  2004/11/19 10:38:04  de6wmigr
   Several QoS related api functions added.

   Revision 1.1  2004/03/10 14:59:05  de6wanme
   Added first INCA-IP support

   Revision 1.3  2003/07/25 10:02:40  rjanse_a
   Bug fix (UnDestPort modification/display)

   Revision 1.2  2003/07/15 11:09:33  rjanse_a
   Bug fixes.
   Improvements in API handling.

   Revision 1.1  2003/07/14 08:57:09  rjanse_a
   Initial revision of ported vxWorks switch driver (inca_switch_hash, inca_switch_access) and a simple control interface (inca_switch_api, api.sh)

   Revision 1.5  2003/07/07 17:41:38  mangele
   - Add Switch LED API

   Revision 1.4  2003/06/04 15:55:20  mangele
   Add further API functions for access switch INCA IP module

   Revision 1.3  2003/06/02 21:52:33  mangele
   Add further functionallity for the switch API

   Revision 1.2  2003/05/23 13:35:42  mangele
   Splitt of vxWorks user functions and switch/PHY API functions (see IncaSwitchAccess.c)
   Rename vxWorks user functions (sysPhy..., sysSw..)

   Revision 1.1  2003/05/20 13:34:08  mangele
   add PHY API: access LAN and PC PHY
   add SWITCH API: switch MAC table



   
******************************************************************************/
/*  #include "vxWorks.h" */
/*  #include "stdio.h" */
/*  #include "string.h" */

/*  #include "RegIncaIP.h" */
/*  #include "IncaSwitchEnd.h" */
/*  #include "IncaSwitchHash.h" */
/*  #include "IncaSwitchAccess.h" */
/*  #include "IncaSwitchApi.h" */

#define EXPORT_SYMTAB

#include <asm/incaip/inca-ip.h>

#include <linux/module.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/sched.h>

#include "vxWorks_type.h"
#include "inca_switch_access.h"
#include "inca_switch_settings.h"
#include "inca_switch_hash.h"
#include "inca_switch_phy.h"

/*  #include "vxWorks_type.h" */
/*  #include "inca_switch_access.h" */
/*  #include "inca_phy_regs.h" */

/*  #include "inca_mdio_settings.h" */
/*  #include "inca_switch_actl.h" */
/*  #include "inca_switch_hash.h" */

#undef DEBUG

/******************************************************************************
   Common definitions
******************************************************************************/

volatile sw21553_VlanSwPrioPerPort_t VlanSwPrioPerPort;
volatile int stream_classification; 


/******************************************************************************
   QoS API
******************************************************************************/


/******************************************************************************
*  InitVLANPortPrio()
*     initializes the VlanSwPrioPerPort structure that contains the port 
*     related vlan IDs and priorities
*
*  Parameter:
*     none
*
*  Returns:
*     nothing
*
*  REMARK:
*
*/
void InitVLANPortPrio(void)
{
    unsigned int flags = 0;

    save_and_cli(flags);
    
    VlanSwPrioPerPort.mng.port_nr_min  = MANAGEMENT_DEFAULT_PORT;
    VlanSwPrioPerPort.mng.port_nr_max  = MANAGEMENT_DEFAULT_PORT;    
    VlanSwPrioPerPort.mng.priority = MANAGEMENT_DEFAULT_VLANPRIO;
    VlanSwPrioPerPort.mng.vlan_id  = MANAGEMENT_DEFAULT_VLANID;
    
    VlanSwPrioPerPort.voip.port_nr_min  = VOIP_DEFAULT_PORT_MIN;
    VlanSwPrioPerPort.voip.port_nr_max  = VOIP_DEFAULT_PORT_MAX;    
    VlanSwPrioPerPort.voip.priority = VOIP_DEFAULT_VLANPRIO; 
    VlanSwPrioPerPort.voip.vlan_id  = VOIP_DEFAULT_VLANID;
    
    VlanSwPrioPerPort.sig.port_nr_min  = SIGNALLING_DEFAULT_PORT_MIN;
    VlanSwPrioPerPort.sig.port_nr_max  = SIGNALLING_DEFAULT_PORT_MAX;    
    VlanSwPrioPerPort.sig.priority = SIGNALLING_DEFAULT_VLANPRIO;
    VlanSwPrioPerPort.sig.vlan_id  = SIGNALLING_DEFAULT_VLANID;
    
/*    VlanSwPrioPerPort.data.port_nr  =  needed ? */ 
    VlanSwPrioPerPort.data_priority = DATA_DEFAULT_VLANPRIO;
    VlanSwPrioPerPort.data_vlan_id  = DATA_DEFAULT_VLANID;
  
    restore_flags(flags);   
/*    
    printk("VlanSwPrioPerPort.mng.port_nr_min: %d\n", 
            VlanSwPrioPerPort.mng.port_nr_min);
    printk("VlanSwPrioPerPort.mng.port_nr_max: %d\n", 
            VlanSwPrioPerPort.mng.port_nr_max);
    printk("VlanSwPrioPerPort.mng.priority: %d\n", 
            VlanSwPrioPerPort.mng.priority);
    printk("VlanSwPrioPerPort.mng.vlan_id: %d\n", 
            VlanSwPrioPerPort.mng.vlan_id);    
    
    printk("VlanSwPrioPerPort.voip.port_nr_min: %d\n",
            VlanSwPrioPerPort.voip.port_nr_min);      
    printk("VlanSwPrioPerPort.voip.port_nr_max: %d\n",
            VlanSwPrioPerPort.voip.port_nr_max); 
    printk("VlanSwPrioPerPort.voip.priority: %d\n",
            VlanSwPrioPerPort.voip.priority); 
    printk("VlanSwPrioPerPort.voip.vlan_id: %d\n",
            VlanSwPrioPerPort.voip.vlan_id);     
            
    printk("VlanSwPrioPerPort.sig.port_nr_min: %d\n",
            VlanSwPrioPerPort.sig.port_nr_min);      
    printk("VlanSwPrioPerPort.sig.port_nr_max: %d\n",
            VlanSwPrioPerPort.sig.port_nr_max); 
    printk("VlanSwPrioPerPort.sig.priority: %d\n",
            VlanSwPrioPerPort.sig.priority); 
    printk("VlanSwPrioPerPort.sig.vlan_id: %d\n",
            VlanSwPrioPerPort.sig.vlan_id);     
            
    printk("VlanSwPrioPerPort.data_priority: %d\n",
            VlanSwPrioPerPort.data_priority);      
    printk("VlanSwPrioPerPort.data_vlan_id: %d\n",
            VlanSwPrioPerPort.data_vlan_id); 
*/    
    return;
}


int get_vlan_prio_and_id( int data_type, int* identifier, int* priority )
{
    int retval=0;
    unsigned int flags = 0;

    save_and_cli(flags);
    
    switch(data_type)
    {
        case 0:  /* Management Data */
            *priority=   (int)VlanSwPrioPerPort.mng.priority;
            *identifier= (int)VlanSwPrioPerPort.mng.vlan_id;
            break;
        case 1:  /* Signalling Data */
            *priority=VlanSwPrioPerPort.sig.priority;
            *identifier=VlanSwPrioPerPort.sig.vlan_id;
            break;
        case 2:  /* VoIP Data */
            *priority=VlanSwPrioPerPort.voip.priority;
            *identifier=VlanSwPrioPerPort.voip.vlan_id;
            break;
        case 3:  /* Other Data */
            *priority=VlanSwPrioPerPort.data_priority;
            *identifier=VlanSwPrioPerPort.data_vlan_id;
            break;
        default:
            printk("Invalid Parameter %d \n", data_type);
            printk("Must be between 0 ... 3 \n");
            printk("0: Management Data\n");
            printk("1: Signalling Data\n");
            printk("2: VoIP Data\n");
            printk("3: Other Data\n"); 
            *priority=-1; 
            *identifier=-1;                       
            retval=-1;  
            break;
    }
/*
    printk("get_vlan_prio_and_id(%d)-> piority %d, vid %d\n",
            data_type, *priority, *identifier); 
*/    
    restore_flags(flags);
    return(retval);
}

int get_port_range( int data_type, UINT32* port_min, UINT32* port_max )
{
    int retval=0;
    unsigned int flags = 0;

    save_and_cli(flags);
    
    switch(data_type)
    {
        case 0:  /* Management Data */
            *port_min=(UINT32)VlanSwPrioPerPort.mng.port_nr_min;
            *port_max=(UINT32)VlanSwPrioPerPort.mng.port_nr_max;
            break;
        case 1:  /* Signalling Data */
            *port_min=(UINT32)VlanSwPrioPerPort.sig.port_nr_min;
            *port_max=(UINT32)VlanSwPrioPerPort.sig.port_nr_max;
            break;
        case 2:  /* VoIP Data */
            *port_min=(UINT32)VlanSwPrioPerPort.voip.port_nr_min;
            *port_max=(UINT32)VlanSwPrioPerPort.voip.port_nr_max;  
            break;
        default:
            printk("Invalid Parameter %d \n", data_type);
            printk("Must be between 0 ... 2 \n");
            printk("0: Management Data\n");
            printk("1: Signalling Data\n");
            printk("2: VoIP Data\n");
            *port_min=0;
            *port_max=0;            
            retval=-1;       
    }

    restore_flags(flags);
    return(retval);
}




int stream_classification_enabled( void )
{
    int retval;
    unsigned int flags = 0;

    save_and_cli(flags);
    retval = stream_classification;
    restore_flags(flags);
    return(retval);
}



/******************************************************************************
*  sysSwQosSetDscpCos()
*     set DSCP CoS mapping table for layer 3 QoS
*
*  Parameter:
*    
*  cos_select: specify the register to be accessed
*              1: value will be written to DSCP_COS1 register 
*              2: value will be written to DSCP_COS2 register 
*
*  cos_value:  32 bit value to be written to Cos register
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwQosSetDscpCos(int cos_select, UINT32 cos_value)
{
    int command;
    
    if(cos_select == 1)   
        command = eSwIoSetDscpCos1; 
    else
        if(cos_select == 2)
            command = eSwIoSetDscpCos2; 
        else 
	   { 
		  printk("Invalid Parameter %d \n", cos_select);
		  printk("Must be 1 or 2 to access reg. DSCP_COS1 or 2 \n");
		  return(-1); 
        }
    sw21553_SwIoSetControl(command, (void*) cos_value);
    return (0);
}


/******************************************************************************
*  sysSwQosSetPrioCos()
*     set mapping table for layer 2 QoS
*
*  Parameter:
*
*  cos_value:  8 bit value to be written to PrioCos register
*
*  Returns:
*     0 - Success
*
*/
int sysSwQosSetPrioCos(UINT32 cos_value)
{
    if(cos_value>255)
        printk("Warning: Leading bits of parameter 0x%08X are ineffektive !\n",
               cos_value);
    sw21553_SwIoSetControl(eSwIoSetPrioCoS, (void*) cos_value);
    return(0);
}

/******************************************************************************
*  sysSwQosSelectFinalPrio()
*     this function sets or erases a bit that decides on the final priority 
*     when data is tagged and contains a dscp field
*
*  Parameter:
*
*  dscp_tos_select:  qos priority selector (0:tag prio is used, 1: DSCP is used)
*
*  Returns:
*     0 - Success
*
*/
int sysSwQosSelectFinalPrio(int dscp_tos_select)
{
    void* dummy_ptr;
    
    if( dscp_tos_select == 0)
        dummy_ptr=NULL;
    else
        if( dscp_tos_select == 1)
	        dummy_ptr=(void*)1;
        else
	    { 
            printk("Invalid Parameter %d \n", dscp_tos_select);
            printk("Must be 0 or 1 to select TOS or DSCP for final priority\n");
            return(-1); 
        }
	   
    sw21553_SwIoSetControl(eSwIoUpdArlCtl_PS, dummy_ptr);
    return(0);
}

/******************************************************************************
*  sysSwQosSetPortPrio()
*     this function sets the port specific priority (basic qos)
*  Parameter:
*
*     port_select: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*
*     priority:    port priority (0: low priority, 1: high priority) 
*  Returns:
*     0 - Success
*
*/
int sysSwQosSetPortPrio(int port_select, int priority)
{
    struct sw21553_sw_io_reg_upd io_data;
    
    if(priority==0)
    { 
        io_data.newRegSettings = 0x00000000;
    }
    else
        if(priority==1)
	    {
	        io_data.newRegSettings = 0xFFFFFFFF;
	    }
	    else
	    {
            printk("Invalid Parameter %d \n", priority);
            printk("Must be 0 or 1 to select low or high prio. for selected port\n");
            return(-1);             
	    }
    /* set port specific mask */
    switch(port_select)
    {
        case INCA_IP_SWITCH_PORT_SELECT_PC:
            io_data.updRegMask = INCA_IP_Switch_ARL_CTL_CP;
	        break;
        case INCA_IP_SWITCH_PORT_SELECT_LAN:
            io_data.updRegMask = INCA_IP_Switch_ARL_CTL_CL;
	        break;
        case INCA_IP_SWITCH_PORT_SELECT_CPU:
            io_data.updRegMask = INCA_IP_Switch_ARL_CTL_CC;
	        break;
        default:
            printk("Invalid Parameter %d \n", port_select);
            printk("Must be 0, 1 or 2 to select PC, LAN or CPU port\n");
            return(-1);  
    }
	sw21553_SwIoSetControl(eSwIoUpdArlCtl, (void*) &io_data);
	return(0);  
}



/******************************************************************************
*  sysSwQosSetSchedMode()
*     this function sets the scheduling mode to either strict priority scheduling
*     or weighted fair Queueing for all switch ports.
*
*  Parameter:
*
*     sched_select: scheduling mode selector
*                    0: strict priority scheduling
*                    1: weighted fair queue scheduling
*
*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwQosSetSchedMode( int sched_select )
{
    sw21553_SwIoCoSWfqCtl SwIoCoSWfqCtl; 

    /* set desired scheduling mode */
    if( sched_select == 0 )
    {
        SwIoCoSWfqCtl.scheme = INCA_IP_Switch_XXX_WFQ_CTL_STRICT_SCHED;
    }
    else
        if( sched_select == 1 )
        {
            SwIoCoSWfqCtl.scheme = INCA_IP_Switch_XXX_WFQ_CTL_WFQ_SCHED;
        }            
        else
	    {
            printk("Invalid Parameter %d \n", sched_select);
            printk("Must be 0 or 1 to select strict priority or WFQ scheduling\n");
            return(-1);             
	    }   
        
    /* set parameters that should remain unchanged */   
    SwIoCoSWfqCtl.weightQueue_0 = -1;
    SwIoCoSWfqCtl.weightQueue_1 = -1;
    SwIoCoSWfqCtl.prioQueue_0   = -1;
    SwIoCoSWfqCtl.prioQueue_1   = -1;        
   
    SwIoCoSWfqCtl.portSelect    = INCA_IP_SWITCH_PORT_SELECT_PC; 
    if( sw21553_SwIoSetControl( eSwIoSetPortWfqCtl, (void*) &SwIoCoSWfqCtl ) != ERROR)
    {
        SwIoCoSWfqCtl.portSelect    = INCA_IP_SWITCH_PORT_SELECT_LAN; 
        if( sw21553_SwIoSetControl( eSwIoSetPortWfqCtl, (void*) &SwIoCoSWfqCtl )  != ERROR)
        {
            SwIoCoSWfqCtl.portSelect    = INCA_IP_SWITCH_PORT_SELECT_CPU; 
            if( sw21553_SwIoSetControl( eSwIoSetPortWfqCtl, (void*) &SwIoCoSWfqCtl ) != ERROR)  
                return 0;  
        }
    }
    printk("sw21553_SwIoSetControl(eSwIoSetPortWfqCtl, arg) failed\n");
    return (-1);
}

/******************************************************************************
*  sysSwSetPmacL2HeaderHandling()
*     this function enables or disables the built-in L2 VLAN header removal 
*     functionality of the switch
*
*  Parameter:
*
*     handle: enable or disable L2 header removal
*                    0: do not remove header
*                    1: remove header
*
*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwSetPmacL2HeaderHandling( int handle )
{
    struct sw21553_sw_io_reg_upd io_data;
/*    void*  dummy_ptr;  */
    
    /* modify VLAN-awareness bit and PMAC L2 header removal bit 
     * according to passed selection parameter
     */
    if( handle == 0 )
    {
        io_data.newRegSettings = 0x00000000;
/*        dummy_ptr=NULL;  */
    }
    else
        if( handle == 1 )  
        {
            io_data.newRegSettings =0xFFFFFFFF;
/*            dummy_ptr=(void*)1;  */
        }            
        else
	    {
            printk("Invalid Parameter %d \n", handle);
            printk("Must be 0 or 1 to select header remain or removal\n");
            return(-1);             
	    }
        
    /* enable/disable VLAN header removal */    
    io_data.updRegMask = INCA_IP_Switch_PMAC_HD_CTL_RL2;
    sw21553_SwIoSetControl( eSwIoUpdMacHdCtl, (void*) &io_data );
    /* enable/disable VLAN awareness */
/*    sw21553_SwIoSetControl( eSwIoSetVlanCntrl_VA, dummy_ptr );  */
    
    return (0);
}


/******************************************************************************
*  sysSwSetPmacAddTagEnDisable()
*     this function enables or disables the built-in L2 VLAN header removal 
*     functionality of the switch
*
*  Parameter:
*
*     handle: enable or disable L2 header removal
*                    0: do not remove header
*                    1: remove header
*
*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwSetPmacAddTagEnDisable( int handle )
{
    struct sw21553_sw_io_reg_upd io_data;
    void*  dummy_ptr;
    
    /* 
     * 
     */
    if( handle == 0 )
    {
        io_data.newRegSettings = 0x00000000;
        dummy_ptr=NULL;
    }
    else
        if( handle == 1 )  
        {
            io_data.newRegSettings =0xFFFFFFFF;
            dummy_ptr=(void*)1;
        }            
        else
	    {
            printk("Invalid Parameter %d \n", handle);
            printk("Must be 0 or 1 to avoid or use tag\n");
            return(-1);             
	    }
        
    /* enable/disable VLAN tagging */    
    io_data.updRegMask = INCA_IP_Switch_PMAC_HD_CTL_ADD;
    sw21553_SwIoSetControl( eSwIoUpdMacHdCtl_ADD, (void*) &io_data );
    io_data.updRegMask = INCA_IP_Switch_PMAC_HD_CTL_TAG;
    sw21553_SwIoSetControl( eSwIoUpdMacHdCtl_TAG, (void*) &io_data );    
    /* enable/disable VLAN awareness */
    sw21553_SwIoSetControl( eSwIoSetVlanCntrl_VA, dummy_ptr );
    
    return (0);
}




/******************************************************************************
*  sysSwQosSetPrioId()
*     this function assigns the passed vlan-priority and vlan-ID to the appr. 
*     field of the VlanSwPrioPerPort data structure
*
*  Parameter:
*
*     data_type: data type selector
*                0: identifier and priority will be be assigned to management field
*                1: identifier and priority will be be assigned to signalling field
*                2: identifier and priority will be be assigned to VoIP field
*                3: identifier and priority will be be assigned to data field
*
*     vlan_prio: vlan priority to be assigned to outgoing frames of type data_type
*                range 0 ... 7
*
*     vlan_id:   vlan identifier to be assigned to outgoing frames of type data_type
*                range 1 ... 4095
*

*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwQosSetPrioId( int data_type, int vlan_prio, int vlan_id )
{

    if( vlan_prio < 0 ||
        vlan_prio > INCA_IP_VLAN_MAX_PRIO )
	{
        printk("Invalid Parameter %d \n", vlan_prio);
        printk("Must be between 0 ... %d \n", INCA_IP_VLAN_MAX_PRIO);
        return(-1);             
	}
    if( vlan_id < 0 ||
        vlan_id > INCA_IP_VLAN_MAX_VLAN_ID )
	{
        printk("Invalid Parameter %d \n", vlan_id);
        printk("Must be between 1 ... %d \n", INCA_IP_VLAN_MAX_VLAN_ID);
        return(-1);             
	}

    switch(data_type)
    {
        case 0:  /* Management Data */
            VlanSwPrioPerPort.mng.priority = (UCHAR) vlan_prio;
            VlanSwPrioPerPort.mng.vlan_id  = (unsigned short) vlan_id;
            break;
        case 1:  /* Signalling Data */
            VlanSwPrioPerPort.sig.priority = (UCHAR) vlan_prio;
            VlanSwPrioPerPort.sig.vlan_id  = (unsigned short) vlan_id;
            break;
        case 2:  /* VoIP Data */
            VlanSwPrioPerPort.voip.priority = (UCHAR) vlan_prio;
            VlanSwPrioPerPort.voip.vlan_id  = (unsigned short) vlan_id;
            break;
        case 3:  /* Other Data */
            VlanSwPrioPerPort.data_priority = (UCHAR) vlan_prio;
            VlanSwPrioPerPort.data_vlan_id  = (unsigned short) vlan_id;
            break;
        default:
            printk("Invalid Parameter %d \n", data_type);
            printk("Must be between 0 ... 3 \n");
            printk("0: Management Data\n");
            printk("1: Signalling Data\n");
            printk("2: VoIP Data\n");
            printk("3: Other Data\n");                        
            return(-1);       
    }
    return(0);
}




/******************************************************************************
*  sysSwQosSetVoicePortRange()
*     this function assigns the passed port range to the appr. 
*     field of the VlanSwPrioPerPort data structure
*
*  Parameter:
*
*     port_min: lower VoIP port number
*
*     port_max: upper VoIP port number
*
*  Remark:
*     Use port_min = port_max if a single port number should be assigned
*
*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwQosSetVoicePortRange( UINT32 port_min, UINT32 port_max )
{

    if( port_min == 0 ||
        port_max == 0 ||
        port_min > port_max )
	{
        printk("Invalid Parameters %d, %d \n", port_min, port_max);
        printk("Must be != 0 and port_min <= port_max \n");
        return(-1);             
	}        
    
    VlanSwPrioPerPort.voip.port_nr_min  = port_min;
    VlanSwPrioPerPort.voip.port_nr_max  = port_max;  
    return(0);        
}


/******************************************************************************
*  sysSwQosStreamClassification()
*     this function starts or stops the data type dependent assignment of 
*     VLAN ID and priority (stream classification)
*
*  Parameter:
*
*     handle: enable/disable handle
*             0: disable stream classification
*             1: enable stream classification
*
*  Returns:
*     0  - Success
*     -1 - invalid parameter
*/
int sysSwQosStreamClassification( int handle )
{
    if( handle == 0 || handle == 1)
    {
        stream_classification = handle;
    }
    else
    {
        printk("Invalid Parameter %d \n", handle);
        printk("Must be 0 or 1 to disable or enable stream classification\n");
        return(-1);             
	}
    return(0);    
}




/******************************************************************************
   PHY API
******************************************************************************/



/******************************************************************************
*  sw21553_DisplayPhyProperties()
*     display PHY properties previous read
*
*  Parameter:
*     properties: encoded properties of a PHY
*           bit  0.. 4: line properties (current state, advertise, partner)
*           bit  5.. 7: currently not used
*           bit  8:     support pause operation
*           bit  9:     support next page
*           bit 10:     support auto negotiation
*           bit 11..15: currently not used
*           bit 16:     extended capabilities
*           bit 17:     auto negotiation able (PHY Reg 1)
*           bit 18..19: currently not used       
*           bit 20:     link status
*           bit 21:     remote fault (advertise: signal partner a fault)
*           bit 22:     auto negotiation complete
*           bit 23:     currently not used       
*           bit 24..26: current speed (PHY reg 31)
*           bit 27:     auto negotiation done (PHY reg 31)
*           bit 28..30: currently not used       
*           bit 21:     data may be corrupted
*
*  REMARK:
*
*/
void sw21553_DisplayPhyProperties(UINT32 properties)
{
   printk(" data  |link  |remote|A.Neg |         line properties \n") ;
   printk(" valid |state |fault |compl.| 100T4| 100FD| 100HD| 10FD | 10HD\n") ;
   printk("----------------------------------------------------------------\n") ;
   printk("   %c      %c      %c      %c      %c      %c      %c      %c      %c\n",
          ((properties & sw21553_PHY_CTRL_DATA_CORRUPTED) ? '0' : '1'), 
          ((properties & sw21553_PHY_CTRL_LINK_STATE) ? '1' : '0'), 
          ((properties & sw21553_PHY_CTRL_REMOTE_FAULT) ? '1' : '0'), 
          ((properties & sw21553_PHY_CTRL_AUTO_NEG_COMPL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_T4) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_FULL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_100_HALF) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_10_FULL) ? '1' : '0'), 
          ((properties & sw21553_ETHERNET_LINE_10_HALF) ? '1' : '0') ) ;
   /* display current speed and duplex mode */
   printk("\n Current Line: %s MBit/s %s DUPLEX (Auto-Neg. %s)\n\n",
          ( (properties & sw21553_PHY_CTRL_SPEED_100_IND) ? "100" : "10" ),
          ( (properties & sw21553_PHY_CTRL_SPEED_DUPLEX_IND) ? "FULL" : "HALF" ),
          ( (properties & sw21553_PHY_CTRL_AUTO_NEG_DONE) ? "DONE" : "NOT DONE" ) ) ;

   return ;
}


/******************************************************************************
*  sysPhyShowLan()
*     collect and display the appropriate LAN PHY properties
*
*  Parameter:
*     phyInfo: specify the properties for display
*             0: properties of the given PHY
*             1: advertised PHY properties used for auto-neg.
*             2: partner PHY properties
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyShowLan(int phyInfo)
{
   UINT32  properties ;

   if ( (properties = sw21553_PhyGetProperties(INCA_IP_MDIO_LAN_PHY, phyInfo)) != ERROR)
   {
      /* display and interprete informations */
      printk("Status LAN Port PHY (%s): 0x%08x\n",
              (phyInfo==sw21553_PHY_PROPS_CURRENT) ? 
                    (sw21553_PHY_PROPS_CURRENT_STR):( 
                        (phyInfo==sw21553_PHY_PROPS_ADVERTISE)?
                             (sw21553_PHY_PROPS_ADVERTISE_STR):(sw21553_PHY_PROPS_PARTNER_STR)
                                                    ),

               properties ) ;
      sw21553_DisplayPhyProperties(properties) ;
   }
   else
   {
      printk("ERROR - while request LAN PHY informations\n") ;
   }
   return 0 ;
}

/******************************************************************************
*  sysPhyShowPc()
*     collect and display the appropriate PC PHY properties
*
*  Parameter:
*     phyInfo: specify the properties for display
*             0: properties of the given PHY
*             1: advertised PHY properties used for auto-neg.
*             2: partner PHY properties
*
*  Returns:
*     0 - no dummy*
*  REMARK:
*
*/
int sysPhyShowPc(int phyInfo)
{
   UINT32  properties ;

   if ( (properties = sw21553_PhyGetProperties(INCA_IP_MDIO_PC_PHY, phyInfo)) != ERROR)
   {
      /* display and interprete informations */
      printk("Status PC Port PHY (%s): 0x%08x\n",
              (phyInfo==sw21553_PHY_PROPS_CURRENT) ? 
                    (sw21553_PHY_PROPS_CURRENT_STR):( 
                        (phyInfo==sw21553_PHY_PROPS_ADVERTISE)?
                             (sw21553_PHY_PROPS_ADVERTISE_STR):(sw21553_PHY_PROPS_PARTNER_STR)
                                                    ),

               properties ) ;
      sw21553_DisplayPhyProperties(properties) ;
   }
   else
   {
      printk("ERROR - while request PC PHY informations\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhySetLanAdvertise()
*     restart the auto negotiation for the LAN PHY
*
*  Parameter:
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*     doRestartAutoNeg: if set do auto negotiation of the LAN PHY
*
*  Returns:
*     0 - no dummy
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*/
int sysPhySetLanAdvertise(UINT32 newSettings, int doRestartAutoNeg)
{
   UINT32 tempReg ;
   tempReg = sw21553_PhySetAdvertiseProperties(INCA_IP_MDIO_LAN_PHY, newSettings) ;
   
   if (doRestartAutoNeg)
      if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_LAN_PHY) == ERROR)
         printk("LAN PHY: advertise auto-neg. properties - ERROR while restart auto-neg.\n") ;
      else
         printk("LAN PHY: advertise auto-neg. properties - restart auto neg. done\n") ;
   else
      printk("LAN PHY: advertise auto-neg. properties set - NO restart auto neg.\n") ;

   return(0) ;
}


/******************************************************************************
*  sysPhySetPcAdvertise()
*     restart the auto negotiation for the PC PHY
*
*  Parameter:
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*     doRestartAutoNeg: if set do auto negotiation of the PC PHY
*
*  Returns:
*     0 - no dummy
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*/
int sysPhySetPcAdvertise(UINT32 newSettings, int doRestartAutoNeg)
{
   UINT32 tempReg ;
   tempReg = sw21553_PhySetAdvertiseProperties(INCA_IP_MDIO_PC_PHY, newSettings) ;
   
   if (doRestartAutoNeg)
      if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_PC_PHY) == ERROR)
         printk("PC PHY: advertise auto-neg. properties - ERROR while restart auto-neg.\n") ;
      else
         printk("PC PHY: advertise auto-neg. properties - restart auto neg. done\n") ;
   else
      printk("PC PHY: advertise auto-neg. properties set - NO restart auto neg.\n") ;

   return(0) ;
}

/******************************************************************************
*  sysPhySetLanFixed()
*     set the LAN PHY speed settings used for not auto-neg connections
*
*  Parameter:
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*     doDisableAutoNeg: if set disable auto negotiation of the LAN PHY
*                        else keep current value
*
*  Returns:
*     0 - Success
*     -1  ERROR
*  REMARK:
*/
int sysPhySetLanFixed(UINT32 newSpeed, int doDisableAutoNeg)
{
   if (sw21553_PhySetFixSpeed(INCA_IP_MDIO_LAN_PHY, newSpeed) == ERROR)
   {
      printk("ERROR while change fixed LAN PHY speed settings\n") ;
      return(-1) ;
   }

   if (doDisableAutoNeg==1)
   {
      if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, doDisableAutoNeg) == ERROR)
      {
         printk("LAN PHY: fixed speed changed - ERROR while disable auto-neg.\n") ;
      }
      else
      {
         printk("LAN PHY: fixed speed changed, auto-negotiation disabled\n") ;
      }
   }
   else
   {
      printk("LAN PHY: fixed speed changed\n") ;
   }

   return(OK) ;
}

/******************************************************************************
*  sysPhySetPcFixed()
*     set the PC PHY speed settings used for not auto-neg connections
*
*  Parameter:
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*     doDisableAutoNeg: if set disable auto negotiation of the PC PHY
*                        else keep current value
*
*  Returns:
*     0 - Success
*     -1  ERROR
*  REMARK:
*/
int sysPhySetPcFixed(UINT32 newSpeed, int doDisableAutoNeg)
{
   if (sw21553_PhySetFixSpeed(INCA_IP_MDIO_PC_PHY, newSpeed) == ERROR)
   {
      printk("ERROR while change fixed PC PHY speed settings\n") ;
      return(ERROR) ;
   }

   if (doDisableAutoNeg==1)
   {
      if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, doDisableAutoNeg) == ERROR)
      {
         printk("PC PHY: fixed speed changed - ERROR while disable auto-neg.\n") ;
      }
      else
      {
         printk("PC PHY: fixed speed changed, auto-negotiation disabled\n") ;
      }
   }
   else
   {
      printk("PC PHY: fixed speed changed\n") ;
   }

   return(OK) ;
}


/******************************************************************************
*  sysPhyLanReAutoNeg()
*     restart the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanReAutoNeg(void)
{
   if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_LAN_PHY) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot RESTART auto negotiation - check if auto neg is enabled\n") ;
   }
   else
   {
      printk("LAN PHY: RESTART auto negotiation done\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcReAutoNeg()
*     restart the auto negotiation for the PC PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcReAutoNeg(void)
{
   if (sw21553_PhyRestartAutoNeg(INCA_IP_MDIO_PC_PHY) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot RESTART auto negotiation - check if auto neg is enabled\n") ;
   }
   else
   {
      printk("PC PHY: RESTART auto negotiation done\n") ;
   }
   return(0) ;
}


/******************************************************************************
*  sysPhyLanDisableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanDisableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, 1) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot DISABLE auto negotiation\n") ;
   }
   else
   {
      printk("LAN PHY: auto negotiation DISABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyLanEnableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanEnableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_LAN_PHY, 0) == ERROR)
   {
      printk("ERROR - LAN PHY: Cannot ENABLE auto negotiation\n") ;
   }
   else
   {
      printk("LAN PHY: auto negotiation ENABLED\n") ;
   }
   return(0) ;
}


/******************************************************************************
*  sysPhyPcDisableAutoNeg()
*     disable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcDisableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, 1) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot DISABLE auto negotiation\n") ;
   }
   else
   {
      printk("PC PHY: auto negotiation DISABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcEnableAutoNeg()
*     enable the auto negotiation for the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcEnableAutoNeg(void)
{
   if (sw21553_PhyDisEnAutoNeg(INCA_IP_MDIO_PC_PHY, 0) == ERROR)
   {
      printk("ERROR - PC PHY: Cannot ENABLE auto negotiation\n") ;
   }
   else
   {
      printk("PC PHY: auto negotiation ENABLED\n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyLanReset()
*     reset the LAN PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyLanReset(void)
{
   if (sw21553_PhyReset(INCA_IP_MDIO_LAN_PHY) == ERROR)
   {
      printk("LAN PHY: ERROR - while PHY RESET\n") ;
   }
   else
   {
      printk("LAN PHY: PHY RESET done \n") ;
   }
   return(0) ;
}

/******************************************************************************
*  sysPhyPcReset()
*     reset the PC PHY
*
*  Parameter:
*
*  Returns:
*     0 - no dummy
*  REMARK:
*
*/
int sysPhyPcReset(void)
{
   if (sw21553_PhyReset(INCA_IP_MDIO_PC_PHY) == ERROR)
   {
      printk("PC PHY: ERROR - while PHY RESET\n") ;
   }
   else
   {
      printk("PC PHY: PHY RESET done \n") ;
   }
   return(0) ;
}



/******************************************************************************
   Switch API: MAC table handling
******************************************************************************/

/* definitions of the MAC table entry */
#define INCA_IP_MAC_TABLE_ENTRY_STATIC       0
#define INCA_IP_MAC_TABLE_ENTRY_AGEDOUT      7

#define INCA_IP_MAC_TABLE_ENTRY_STATIC_STR   "static"
#define INCA_IP_MAC_TABLE_ENTRY_AGEDOUT_STR  "aged out"
#define INCA_IP_MAC_TABLE_ENTRY_DYNAMIC_STR  "dynamic"


/* FORMAT: MAC ENTRY Control Field
   BIT:  7 6 5 4   3 2 1 0
               C    Port
          AGE  R   V M L P
         0 0 0 0   0 0 0 0
*/
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_P       0
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_P_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_P )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_L       1
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_L_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_L )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_V       2
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_V_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_V )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_M       3
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_M_MASK  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_M )

#define INCA_IP_MAC_TABLE_ENTRY_CTRL_CR      4
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_CR_MASK (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_CR )
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_POS     5
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_WIDTH   3
#define INCA_IP_MAC_TABLE_ENTRY_CTRL_GET_AGE(ageValue) \
                            ( ( (ageValue) >> INCA_IP_MAC_TABLE_ENTRY_CTRL_POS ) &\
                              (  (1<<INCA_IP_MAC_TABLE_ENTRY_CTRL_WIDTH) - 1   )   )



/******************************************************************************
*  sw21553_SwDisplayMacTableHeader()
*     display a header for show a MAC table
*
*/
LOCAL void sw21553_SwDisplayMacTableHeader(void)
{
   /*     
          " INCA-IP MAC table entries ||MAC Entry | PORTS |"
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */
 printk("\n INCA-IP MAC table entries ||MAC Entry | PORTS |\n") ;
   printk("INDEX |     MAC Address    || Cntrl -> |M|V|L|P| C| age\n") ;
   printk("----------------------------------------------------------------\n") ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}


/******************************************************************************
*  sw21553_SwDisplayMacTableTrailer()
*     display a trailer for show a MAC table
*
*/
LOCAL void sw21553_SwDisplayMacTableTrailer(int count)
{
   /*     
          "        INCA-IP MAC table entries"                   
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */

   printk("----------------------------------------------------------------\n") ;
   printk("  ==>  MAC TABLE: %d entries found\n", count) ;
   printk("----------------------------------------------------------------\n") ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}


/******************************************************************************
*  sw21553_SwDisplayMacEntry()
*     display an MAX entry 
*
*/
LOCAL void sw21553_SwDisplayMacEntry(int index, sw21553_MacTableEntry MacEntry)
{
   UINT32 tempAge ;

   tempAge = INCA_IP_MAC_TABLE_ENTRY_CTRL_GET_AGE((UCHAR)MacEntry.Cntrl) ;
   /*     
          "        INCA-IP MAC table entries"                   
          "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
          "----------------------------------------------------------------"
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
           [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static  
           [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
   */
   printk("[%3d] |  %02x:%02x:%02x:%02x:%02x:%02x"
                                    " ||  0x%02x "
                                              "-> |%c|%c|%c|%c| %c"
                                                            "| %2d %s\n", 
           index, 
           MacEntry.MacAddr[0], MacEntry.MacAddr[1], MacEntry.MacAddr[2],
           MacEntry.MacAddr[3], MacEntry.MacAddr[4], MacEntry.MacAddr[5],
           MacEntry.Cntrl,
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_M_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_V_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_L_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_P_MASK)?'+':' ',
           (MacEntry.Cntrl&INCA_IP_MAC_TABLE_ENTRY_CTRL_CR_MASK)?'1':'0',
           tempAge, 
           (tempAge == INCA_IP_MAC_TABLE_ENTRY_STATIC) ? (INCA_IP_MAC_TABLE_ENTRY_STATIC_STR) : ( 
               (tempAge == INCA_IP_MAC_TABLE_ENTRY_AGEDOUT) ? (INCA_IP_MAC_TABLE_ENTRY_AGEDOUT_STR) : 
                                                              (INCA_IP_MAC_TABLE_ENTRY_DYNAMIC_STR) )
           ) ;
   /*
          "----------------------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "----------------------------------------------------------------") ;
   */
}

/******************************************************************************
*  sw21553_SwReadDisplMacEntry()
*     get the MAC entry from the MAC table and display
*
*/
LOCAL void sw21553_SwReadDisplMacEntry(int index)
{
   sw21553_MacTableEntry MacEntry ;

   if ( sw21553_SwMacReadTableEntry(index, &MacEntry) == 1 )
   {
      /* entry found --> display */
      printk("[%3d] |  %02x:%02x:%02x:%02x:%02x:%02x |   0x%02x\n", 
              index, 
              MacEntry.MacAddr[0], MacEntry.MacAddr[1], MacEntry.MacAddr[2],
              MacEntry.MacAddr[3], MacEntry.MacAddr[4], MacEntry.MacAddr[5],
              MacEntry.Cntrl) ;
   }

}

/******************************************************************************
*  sw21553_SwDisplayUnDest()
*     display the UnDest register settings
*
*/
LOCAL UINT32 sw21553_SwDisplayUnDest(UINT32 regValue)
{
   if (regValue == ERROR)
   {
      printk("Error - cannot show \"Unknown Destination Register\" \n") ;
      return(ERROR) ; 
   }

   /*     
          "  Unknown Destination Register: 0x00000000"
          "                  | Broad | Multi | Uni"
          "  Forward unknown | C|L|P | C|L|P | C|L|P"
          "------------------------------------------"
          "       packets to |  | |  |  | |  |       "
   */
   printk("\n Unknown Destination Register: 0x%08x\n", regValue) ;
   printk("                 || Broad | Multi | Uni \n") ;
   printk(" Forward unknown || C|L|P | C|L|P | C|L|P \n") ;
   printk(" ----------------||-------|-------|------ \n") ;
   printk("      packets to || %c|%c|%c | %c|%c|%c | %c|%c|%c \n\n",
           /* broadcast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CB)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LB)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PB)?'+':'-',
           /* multicast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CM)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LM)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PM)?'+':'-',
           /* unicast flags */
           (regValue&INCA_IP_Switch_UN_DEST_CU)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_LU)?'+':'-',
           (regValue&INCA_IP_Switch_UN_DEST_PU)?'+':'-'
         ) ;
   return(OK) ;
}

/******************************************************************************
*  sw21553_SwDisplaySpPt()
*     display the SpPt register settings
*
*/

LOCAL UINT32 sw21553_SwDisplaySpPt(int regValue)
{
   int tempState ;

   if (regValue == ERROR)
   {
      printk("Error - cannot show \"Spanning Tree Port Status Register\" \n") ;
      return(ERROR) ; 
   }

   printk("\nSpanning Tree Port Status Register: 0x%08x\n", regValue) ;

   tempState = INCA_IP_Switch_ST_PT_PPS_GET(regValue) ;
   printk("   PC-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   tempState = INCA_IP_Switch_ST_PT_LPS_GET(regValue) ;
   printk("  LAN-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   tempState = INCA_IP_Switch_ST_PT_CPS_GET(regValue) ;
   printk("  CPU-port state:   \"%s\" \n",
          (tempState == INCA_IP_Switch_SP_PT_STATE_DISABLED)?("DISABLED"):
             ( 
               (tempState == INCA_IP_Switch_SP_PT_STATE_LISTEN)?("LISTENING"):
                  ( 
                    (tempState == INCA_IP_Switch_SP_PT_STATE_LEARN)?("LEARNING"):("FORWARDING")
                  )         
             )
         ) ;

   return(OK) ;
}


/******************************************************************************
*  sysSwUnDestSetPorts()
*     set and change the UN_DEST register per port
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     portBehaviour: 3 bit field
*                 forwarding of unknown packets for broadcast/multicast/unicast packets
*                 0: do not forward a unknown packet 
*                 1: forward a unknown packet 
*
*     3 bit field:
*     ============
*        Bit 0: unicast packets
*        Bit 1: multicast packets
*        Bit 2: broadcast packets
*
*     Example: forwarding unknown packets to <portSelect> port
*        portBehaviour = 0x5 = 100b
*                              |||---> do not forward unknown unicast packets 
*                              ||----> do not forward unknown multicast packets 
*                              |-----> do     forward unknown broadcast packets 
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*
*/
int sysSwUnDestSetPorts(int portSelect, int portBehaviour)
{
   UINT32 regValue ;
   regValue = sw21553_SwUpdUnDestPorts(portSelect, portBehaviour) ;

   return(sw21553_SwDisplayUnDest(regValue) ) ;
}

/******************************************************************************
*  sysSwUnDestShow()
*     read and display the UN_DEST register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwUnDestShow(void)
{
   UINT32 regValue ;
   /* arg 3: read only settings */
   regValue = sw21553_SwIOUnDest(0,0,0) ;

   return(sw21553_SwDisplayUnDest(regValue) ) ;
}

/******************************************************************************
*  sysSwSpPtSetPorts()
*     update the port state within the Spanning Tree Port Status 
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     newPortState: possible states are
*                    INCA_IP_Switch_SP_PT_STATE_DISABLED  0
*                    INCA_IP_Switch_SP_PT_STATE_LISTEN    1
*                    INCA_IP_Switch_SP_PT_STATE_LEARN     2
*                    INCA_IP_Switch_SP_PT_STATE_FORWARD   3
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*/
int sysSwSpPtSetPorts(int portSelect, int newPortState)
{

   UINT32 regValue ;
   regValue = sw21553_SwUpdSpPtPortState(portSelect, newPortState) ;

   return(sw21553_SwDisplaySpPt(regValue) ) ;
}

/******************************************************************************
*  sysSwSpPtShow()
*     read and display the ST_PT register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwSpPtShow(void)
{
   UINT32 regValue ;
   /* arg 3: read only settings */
   regValue = sw21553_SwIOSpPt(0,0,0) ;

   return(sw21553_SwDisplaySpPt(regValue) ) ;
}

/******************************************************************************
*  sysSwPktLenSet()
*     set the PKT_LEN register (Packet Length Register)
*
*  Parameter:
*     newPktLen: contains the new maximal packet lenght
*     addTagLen: ADD 4 to MAX_PKT_LEN for Tagged Packets
*                0: MAC_PKT_LEN to be used for tagged and untagged packets
*                1: MAX_PKT_LEN + 4 is used for tagged packets.
*
*  Returns:
*     0: Success and display new settings
*     else ERROR (-1)
*/
int sysSwPktLenSet(UINT32 newPktLen, int addTagLen)
{

   UINT32 regValue ;
   regValue = sw21553_SwIOPacketLen(newPktLen, addTagLen, 1) ;

   if(regValue != ERROR)
   {
      printk("PKT_LEN register (0x%08x): max packet len = 0x%x (%d), TAG length %s\n",
              regValue, 
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
              (regValue&INCA_IP_Switch_PKT_LEN_ADD) ? "added" : "not added"
            ) ;
      return(OK) ;
   }
   else
   {
      printk("ERROR - cannot set new MAX_PKT_LEN = %d (add = %d), max len = 0x%x (%d)\n",
              newPktLen, addTagLen, 
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN,
              INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN ) ;

      return(ERROR) ;
   }
   return(ERROR) ;
}

/******************************************************************************
*  sysSwPktLenShow()
*     read and display the PKT_LEN register
*
*  Returns:
*     0: success and display settings
*
*/
int sysSwPktLenShow(void)
{
   UINT32 regValue ;
   regValue = sw21553_SwIOPacketLen(0, 0, 0) ;

   printk("PKT_LEN register (0x%08x): max packet len = 0x%x (%d), TAG length %s\n",
           regValue, 
           INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
           INCA_IP_Switch_PKT_LEN_MAX_PKT_LEN_GET(regValue),
           (regValue&INCA_IP_Switch_PKT_LEN_ADD) ? "added" : "not added"
         ) ;
   return(OK) ;
}



/******************************************************************************
*  sysSwMacShowTable()
*     Display the entries of the switch MAC table
*  Returns:
*     0 - no dummy
*
*/
int sysSwMacShowTable(void)
{
   UINT32 index, count ;
   sw21553_MacTableEntry MacEntry ;
   count = 0 ;

   sw21553_SwDisplayMacTableHeader() ;

   for (index = 0; index < INCA_IP_MAX_MAC_TABLE_ENTRIES; index++)
   {
      if (
           sw21553_SwMacReadTableEntry(index, &MacEntry) == 1
         )
      {
         count++ ;
         /* entry found --> display */
         sw21553_SwDisplayMacEntry(index, MacEntry) ;
      }
   }

   sw21553_SwDisplayMacTableTrailer(count) ;

   return(0) ;
}

/******************************************************************************
*  sysSwMacAddTableEntry()
*     Add an given MAC address to the switch MAC table
*     interprete and set the given parameters                              
*
*  Parameter:
*     pStrEtherAdr:  ptr to MAC address string with the form 00:11:22:aa:bb:cc
*     port:       defines the port to which the address is assigned
*                 possible values are:
*                 0: PC port
*                 1: LAN port
*                 2: CPU port VOICE
*                 3: CPU port MANAGEMENT
*     classification: calssify the frames
*                 0: packets will be classified as normal packets
*                 1: packets will be classified as critical packets
*     aging:      defines the aging behaviour of the entry
*                 0: no aging --> static entry into the MAC table
*                 1: normal aging --> entry will normal timed out
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwMacAddTableEntry(char *pStrEtherAdr, int port, int classification, int aging )
{
   sw21553_MacTableEntry tempMacTableEntry ;

   
   /* create a MAC table entry struct and init the data */
   if (pStrEtherAdr != NULL)
   {
      /* get and copy mac-addr */
      sw21553_SwMacConvertStr2Hex(pStrEtherAdr, tempMacTableEntry.MacAddr) ;
   }
   else
   {
      printk("ERROR - sysSwMacAddTableEntry(): no MAC address given\n") ;
      return(ERROR) ;
   }

   /* check and set assigned port */
   if ( sw21553_SwMacSetTableEntryCntrl(
                                       port, classification, aging, &tempMacTableEntry.Cntrl
                                     ) == ERROR )
   {
      /* parameter error */
      printk("ERROR - sysSwMacAddTableEntry(): invalid parameter\n") ;
      return(ERROR) ;
   }

   printk("sysSwMacAddTableEntry(): add MAC table entry: MAC = %02x:%02x:%02x:%02x:%02x:%02x, control = %2x ",
           tempMacTableEntry.MacAddr[0], tempMacTableEntry.MacAddr[1], 
           tempMacTableEntry.MacAddr[2], tempMacTableEntry.MacAddr[3], 
           tempMacTableEntry.MacAddr[4], tempMacTableEntry.MacAddr[5],
           tempMacTableEntry.Cntrl ) ;
   if (sw21553_SwMacAddEntry(tempMacTableEntry, 0) == OK) 
      printk("SUCCESSFUL\n") ;
   else
      printk("NOT SUCCESSFUL !!!\n") ;
   
   return(OK) ;
}



/******************************************************************************
*  sysSwMacClearEntryByMac()
*     Clear all MAC table entries with the given MAC address
*
*  Parameter:
*     pStrEtherAdr:  ptr to MAC address string with the form 00:11:22:aa:bb:cc
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     Via modulation there are 4 possible MAC entry indexes for one MAC address
*     This functions checks all 4 possibilities for clearance.
*
*/
int sysSwMacClearEntryByMac(char *pStrEtherAdr)
{
   int count ;
   sw21553_MacTableEntry tempMacTableEntry ;


   /* create a MAC table entry struct and init the data */
   if (pStrEtherAdr != NULL)
   {
      /* get and copy mac-addr */
      sw21553_SwMacConvertStr2Hex(pStrEtherAdr, tempMacTableEntry.MacAddr) ;
      /* clear all entries with this MAC */
      tempMacTableEntry.Cntrl = 0 ;
   }
   else
   {
      printk("ERROR - sysSwMacClearEntryByMac(): no MAC address given\n") ;
      return(ERROR) ;
   }

   count = sw21553_SwMacClearEntryByMac(tempMacTableEntry) ;
   printk("sysSwMacClearEntryByMac(%02x:%02x:%02x:%02x:%02x:%02x): %d MAC table entries cleared \n ",
           tempMacTableEntry.MacAddr[0], tempMacTableEntry.MacAddr[1], 
           tempMacTableEntry.MacAddr[2], tempMacTableEntry.MacAddr[3], 
           tempMacTableEntry.MacAddr[4], tempMacTableEntry.MacAddr[5],
           count ) ;

   return(OK) ;
}



/******************************************************************************
*  sysSwMacClearEntryByIndex()
*     Clear a MAC table entry identified by the given index
*
*  Parameter:
*     index:  index of the MAC table entry for clearance
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     The index must be a valid index for the MAC table (currently [0..255]
*
*/
int sysSwMacClearEntryByIndex(int index)
{
   /* create a MAC table entry struct and init the data */
   if (index < 0 || index >= INCA_IP_MAX_MAC_TABLE_ENTRIES)
   {
      printk("ERROR - sysSwMacClearEntryByIndex(): no valid MAC table index\n") ;
      return(ERROR) ;
   }

   if (sw21553_SwMacClearEntryByIndex(index) == OK)
   {
      printk("Success cleared entry: ") ;
      sw21553_SwReadDisplMacEntry(index) ;
   }
   else
   {
      printk("ERROR: sysSwMacClearEntryByIndex(index = %d): contains no vaid entry \n", index) ;
      return(ERROR) ;
   }

   return(OK) ;
}

/******************************************************************************
*  sysSwMacTableReset()
*     reset the complete MAC table
*
*  Returns:
*     0 - Success
*     -1 ERROR:  ERROR while reset write
*
*/
int sysSwMacTableReset(void)
{
   /* reset complete MAC table: 0 - (INCA_IP_MAX_MAC_TABLE_ENTRIES-1) */
   return( sw21553_SwMacResetMacTable() ) ;
}



/******************************************************************************
   Switch API: VLAN control
******************************************************************************/

/******************************************************************************
*  sw21553_SwVlanDisplayVLanCtrl()
*     display the VLAN_CTRL register
*
*/
LOCAL void sw21553_SwVlanDisplayVLanCtrl(UINT32 regValue)
{
   /* 
        "                              PCx  LAN  CPU"
        "      send only to Members:
        "accept no untagged packets:
        "               VLAN awware:

   */
 printk("\n              VLAN_CTRL:    PC   LAN  CPU \n") ;
   printk("   send only to Members:     %c    %c    %c \n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_SP) ? '+' : '-',
           (regValue&INCA_IP_Switch_VLAN_CTRL_SL) ? '+' : '-',
           (regValue&INCA_IP_Switch_VLAN_CTRL_SC) ? '+' : '-'
         ) ;
   printk("accept untagged packets:     %c    %c    %c \n\n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_TP) ? '-' : '+',
           (regValue&INCA_IP_Switch_VLAN_CTRL_TL) ? '-' : '+',
           (regValue&INCA_IP_Switch_VLAN_CTRL_TC) ? '-' : '+'
         ) ;
   printk("             VLAN aware:    %s \n\n",
           (regValue&INCA_IP_Switch_VLAN_CTRL_VA) ? "YES" : "NO") ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayPortVLanHeader()
*     display header for VLAN port configuration
*/
LOCAL void sw21553_SwVlanDisplayVLanTagHeader(void)
{
   /* 
        "Port      VLAN ID      PRIO"
        "PC  :
        "LAN :
        "CPU :  
   */
   printk("\nPort      VLAN ID       PRIO\n") ;
   return ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanCtrl()
*     display the xx_VLAN port register
*
*/
LOCAL void sw21553_SwVlanDisplayPortVLan(int portSelect, UINT32 regValue)
{
   /* 
         "  Port      VLAN ID      PRIO "
         "  PC  :   0x0234 (xxxx)   4"
         "  LAN :
         "  CPU :  
   */
   printk("%s :   0x%04x (%4d)    %d\n", 
           (portSelect==INCA_IP_SWITCH_PORT_SELECT_PC) ? "PC " : 
               ( (portSelect==INCA_IP_SWITCH_PORT_SELECT_LAN)? "LAN" : 
                     ( (portSelect==INCA_IP_SWITCH_PORT_SELECT_CPU) ? "CPU" : "ERROR" ) 
               ) ,
           INCA_IP_Switch_PC_VLAN_VLAN_ID_GET(regValue),
           INCA_IP_Switch_PC_VLAN_VLAN_ID_GET(regValue),
           INCA_IP_Switch_PC_VLAN_PRI_GET(regValue)
         ) ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableHeader()
*     display header for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableHeader(void)
{
   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M|"
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P|"
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
   */
 printk("\n INCA-IP VLAN table\n") ;
   printk("INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M| \n") ;
   printk("INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P| \n") ;
   printk("----------------------------------------------------\n") ;
   /*
          "---------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "---------------------------------------------------") ;
   */
   return ;
}



/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableEntry()
*     display header for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableEntry(int index, UINT32 vLanEntry)
{
   UINT32 vLanId, vLanCntrl ;

   vLanId    = INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vLanEntry) ;
   vLanCntrl = INCA_IP_VLAN_TABLE_ENTRY_GET_CTRL_FLAGS(vLanEntry) ;

   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M| "
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P| "
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | | 
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | | 
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |  
   */

   printk("[%3d] |  0x%03x (%4d)  ||  0x%02x  -> | %c|%c|%c|%c|%c|%c|%c|\n", 
           index, vLanId, (int)vLanId, vLanCntrl,
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK) ?'1':'0',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EC_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EL_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_EP_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_MC_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_ML_MASK)?'+':' ',
           (vLanCntrl&INCA_IP_VLAN_TABLE_ENTRY_CTRL_MP_MASK)?'+':' '
         ) ;

   /*
          "---------------------------------------------------") ;
          "  ==>  MAC TABLE: %d entries found"
          "---------------------------------------------------") ;
   */
   return ;
}

/******************************************************************************
*  sw21553_SwVlanDisplayVLanTableTrailer()
*     display trailer for a VLAN table
*
*/
LOCAL void sw21553_SwVlanDisplayVLanTableTrailer(int count)
{
   /*     
          " INCA-IP VLAN table"
          "INDEX |     VLAN ID    || VLAN Ctrl |  |E|E|E|M|M|M|"
          "INDEX |   hex    dez   || Cntrl  -> | V|C|L|P|C|L|P|"
          "----------------------------------------------------"
           [  0] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  1] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |
           [  2] |  0x000 (1111)  ||  0xe2  -> |  | | | | | | |

          "----------------------------------------------------") ;
          "  ==>  VLAN TABLE: %d entries found"
          "----------------------------------------------------") ;
   */
   printk("----------------------------------------------------\n") ;
   printk("  ==>  VLAN TABLE: %d entries found\n", count) ;
   printk("----------------------------------------------------\n") ;
   return ;
}


/******************************************************************************
*  sysSwVLanShowVLanCtrl()
*     display the VLAN control settings
*
*  Parameter:
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanShowVLanCtrl(void)
{
   /* arg 3 = 0: get the register value --> arg 1, arg 2 not used */
   sw21553_SwVlanDisplayVLanCtrl( sw21553_SwVLanIOVLanCntrl(0, 0, 0) ) ;

   return(OK) ;
}

/******************************************************************************
*  sysSwVLanUpdVLanCtrlVAware()
*     switch ON/OFF VLAN aware (register VLAN_CTRL)
*
*  Parameter:
*     vLanAware: 0: set VLAN unaware
*                1: set VLAN aware
*/
int sysSwVLanUpdVLanCtrlVAware(int vLanAware)
{

   sw21553_SwVlanDisplayVLanCtrl( sw21553_SwIoSetControl( eSwIoSetVlanCntrl_VA, 
                                                          (void *)vLanAware )  ) ;
   return(OK) ;
}


/******************************************************************************
*  sysSwVLanSetVLanCtrlPort()
*     set the VLAN settings for an given port
*
*  Parameter:
*     portSelect:            select a port 
*                               PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                               LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                               CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*     sendOnlyToMember:      Symmetrical VLAN Settings for this port
*                                0: can send packets to VLANs of which it is not a member
*                                1: can only send packets to VLANs of which it is a member
*     notAcceptUntaggedPkt:  Tag Packets only on this Port
*                                0: Untagged packets will be accepted on this port
*                                1: Untagged packets will not be accepted on this port
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanSetVLanCtrlPort(int portSelect, int sendOnlyToMember, int notAcceptUntaggedPkt)
{
   UINT32 regValue ;
   sw21553_SwIoVLanCtrl tempVLanCtrl ;

   tempVLanCtrl.portSelect = portSelect ;
   tempVLanCtrl.sendOnlyToMember = sendOnlyToMember ;
   tempVLanCtrl.notAcceptUntaggedPkt = notAcceptUntaggedPkt ;

   if ( (regValue = sw21553_SwIoSetControl( eSwIoSetVlanCntrl, 
                                         (void *)(&tempVLanCtrl) ) ) == ERROR )
   {
      /* wrong parameter */
      printk("ERROR: sysSwVLanSetVLanCtrlPort(port = %d, ...): no valid port number (%d .. %d) \n", 
              portSelect, 
              INCA_IP_SWITCH_PORT_SELECT_PC,
              INCA_IP_SWITCH_PORT_SELECT_CPU) ;
      return(ERROR) ;
   }
   else
   {
      sw21553_SwVlanDisplayVLanCtrl(regValue) ;
      return(OK) ;
   }
}

/******************************************************************************
*  sysSwVLanShowPortConfig()
*     show the current port (PC, LAN, CPU) VLAN settings
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanShowPortConfig(void)
{
   UINT32 regValue ;

   sw21553_SwVlanDisplayVLanTagHeader() ;
   /* read and display PC-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_PC) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_PC, regValue) ;
   /* read and display LAN-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_LAN) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_LAN, regValue) ;
   /* read and display CPU-port */
   regValue = sw21553_SwVLanGetVLanConfig(INCA_IP_SWITCH_PORT_SELECT_CPU) ;
   sw21553_SwVlanDisplayPortVLan(INCA_IP_SWITCH_PORT_SELECT_CPU, regValue) ;

   return(OK) ;
}

/******************************************************************************
*  sysSwVLanSetPortConfig()
*     set the VLAN settings for an given port
*
*  Parameter:
*     portSelect: select a port 
*                    PC:  INCA_IP_SWITCH_PORT_SELECT_PC     0
*                    LAN: INCA_IP_SWITCH_PORT_SELECT_LAN    1
*                    CPU: INCA_IP_SWITCH_PORT_SELECT_CPU    2
*
*     newPrio:    new priority value to set (range 0 - 7)
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanSetPortConfig(int portSelect, UINT32 newPrio, UINT32 newVLanId)
{
   UINT32 regValue ;
   sw21553_SwIoVLanCfg tempPortVLan ;

   tempPortVLan.portSelect = portSelect ;
   tempPortVLan.newVLanId  = newVLanId ;
   tempPortVLan.newPrio    = newPrio ;


   if ( (regValue = sw21553_SwIoSetControl( eSwIoSetPortVLanConfig, 
                                         (void *)(&tempPortVLan) ) ) == ERROR )
   {
      /* wrong parameter */
      printk("ERROR: sysSwVLanSetPortConfig(port = %d, ...): no valid port number (%d .. %d) \n", 
              portSelect, 
              INCA_IP_SWITCH_PORT_SELECT_PC,
              INCA_IP_SWITCH_PORT_SELECT_CPU) ;
      return(ERROR) ;
   }
   else
   {
      sw21553_SwVlanDisplayVLanTagHeader() ;
      sw21553_SwVlanDisplayPortVLan(portSelect, regValue) ;
      return(OK) ;
   }
}


/******************************************************************************
*  sysSwVLanTableReset()
*     reset the complete VLAN table
*
*  Returns:
*     0 - Success
*     -1 ERROR:  ERROR while reset write
*
*/
int sysSwVLanTableReset(void)
{
   /* reset complete VLAN table: 0 - (INCA_IP_MAX_VLAN_TABLE_ENTRIES-1) */
   return( sw21553_SwVLanResetVLanTable() ) ;
}


/******************************************************************************
*  sysSwVLanTableSetTag()
*     set a VLAN tag within the VLAN tag table
*
*  Parameter:
*     index:      index of the VLAN Table
*                 first step: this version is only for testing
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*     newMemberInfo: 3 bit field
*                 Membership information for PC/LAN/CPU Port
*                 0: Packets from PC/LAN/CPU port is not a member of VLAN
*                 1: Packets from PC/LAN/CPU port is member of this VLAN
*     newEgressTagInfo: 3 bit field
*                 Egress Tagging information for PC/LAN/CPU Port
*                 0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
*                 1: Packets to PC/LAN/CPU egress port is tagged for this VLAN
*
*     3 bit field:
*     ============
*        Bit 0: represents PC port
*        Bit 1: represents LAN port
*        Bit 2: represents CPU port
*
*     Example: newMemberInfo = 0x5 = 101b
*            Bit 0   Bit 1   Bit 2
*        --> MP = 1, ML = 0, MC = 1
*
*     Example: newEgressTagInfo = 0x6 = 110b
*            Bit 0   Bit 1   Bit 2
*        --> EP = 1, EL = 1, EC = 0 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*/
int sysSwVLanTableSetTag( int index, 
                          UINT32 newVLanId, UINT32 newMemberInfo, UINT32 newEgressTagInfo)
{
   sw21553_VLanTableEntry vLanTableEntry ;

   /* check parameter */
   if (newMemberInfo > 7)
   {
      printk("ERROR: membership info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newMemberInfo) ;
      return(ERROR) ;
   }

   if (newEgressTagInfo > 7)
   {
      printk("ERROR: egress tag info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newEgressTagInfo) ;
      return(ERROR) ;
   }

   if ( (newVLanId & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK) )
   {
      printk("ERROR: VLAN ID (%x) out of range (max: %x) \n",
              newVLanId,
              INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK-1) ;
      return(ERROR) ;
   }

   /* create VLAN table struct */
   vLanTableEntry.vLanId = newVLanId ;
   vLanTableEntry.Cntrl  = (newMemberInfo & 0x7) | 
                           ( (newEgressTagInfo & 0x7) << 3 ) |
                           INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK ;

   if ( sw21553_SwVLanWriteTableEntry(index, vLanTableEntry) == OK )
   {
      /* add VLAN entry OK */
      printk("VLAN table entry added: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
   else
   {
      /* add VLAN entry OK */
      printk("ERROR: cannot add VLAN table entry: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
}





/******************************************************************************
*  sysSwVLanAddTableEntry()
*     add a VLAN tag within the VLAN tag table (index will be calculatet)
*
*  Parameter:
*     newVLanId:  new VLAN ID to set (range 0 - 4096)
*                 consider: 
*                    - 4096 (0xFFF): reserved
*                    -    0 (0x000): priority tag 
*     newMemberInfo: 3 bit field
*                 Membership information for PC/LAN/CPU Port
*                 0: Packets from PC/LAN/CPU port is not a member of VLAN
*                 1: Packets from PC/LAN/CPU port is member of this VLAN
*     newEgressTagInfo: 3 bit field
*                 Egress Tagging information for PC/LAN/CPU Port
*                 0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
*                 1: Packets to PC/LAN/CPU egress port is tagged for this VLAN
*
*     3 bit field:
*     ============
*        Bit 0: represents PC port
*        Bit 1: represents LAN port
*        Bit 2: represents CPU port
*
*     Example: newMemberInfo = 0x5 = 101b
*            Bit 0   Bit 1   Bit 2
*        --> MP = 1, ML = 0, MC = 1
*
*     Example: newEgressTagInfo = 0x6 = 110b
*            Bit 0   Bit 1   Bit 2
*        --> EP = 1, EL = 1, EC = 0 
*
*  Returns:
*     0 - Success
*     -1  ERROR
*
*  REMARK:
*     Because the VLAN table entries must be in a increasing order a local 
*     copy of the current VLAN table will be used for sorting.
*     After insertion of a new entry into the local VLAN table the
*     INCA IP VLAN table will be updated.
*
*/
int sysSwVLanAddTableEntry(UINT32 newVLanId, UINT32 newMemberInfo, UINT32 newEgressTagInfo)
{
   int index ;
   sw21553_VLanTableEntry vLanTableEntry ;

   /* check parameter */
   if (newMemberInfo > 7)
   {
      printk("ERROR: membership info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newMemberInfo) ;
      return(ERROR) ;
   }

   if (newEgressTagInfo > 7)
   {
      printk("ERROR: egress tag info (3-bit field --> PC/LAN/CPU): value %x out of range\n",
              newEgressTagInfo) ;
      return(ERROR) ;
   }

   if ( (newVLanId & ~INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK) )
   {
      printk("ERROR: VLAN ID (%x) out of range (max: %x) \n",
              newVLanId,
              INCA_IP_VLAN_TABLE_ENTRY_VLAN_ID_MASK-1) ;
      return(ERROR) ;
   }

   /* create VLAN table struct */
   vLanTableEntry.vLanId = newVLanId ;
   vLanTableEntry.Cntrl  = (newMemberInfo & 0x7) | 
                           ( (newEgressTagInfo & 0x7) << 3 ) |
                           INCA_IP_VLAN_TABLE_ENTRY_CTRL_V_MASK ;

   if ( (index=sw21553_SwVLanAddTableEntry(vLanTableEntry)) != ERROR )
   {
      /* add VLAN entry OK */
      printk("VLAN table entry added: index = %d, VLAN ID = %x (%d), Ctrl = %x \n",
              index, 
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(OK) ;
   }
   else
   {
      /* add VLAN entry OK */
      printk("ERROR: whilet add VLAN table entry: VLAN ID = %x (%d), Ctrl = %x \n",
              vLanTableEntry.vLanId,
              vLanTableEntry.vLanId,
              vLanTableEntry.Cntrl ) ;
      return(ERROR) ;
   }
}



/******************************************************************************
*  sysSwVLanShowTable()
*     show the VLAN table
*
*  Parameter:
*
*/
int sysSwVLanShowTable(void)
{
   int count, index ;
   UINT32 vlanTable[INCA_IP_MAX_VLAN_TABLE_ENTRIES] ;

   count =  sw21553_SwVLanReadTable(vlanTable, 0) ;

   if (count != 0)
   {
      /* display VLAN table entries */
      sw21553_SwVlanDisplayVLanTableHeader() ;

      for (index = 0; index < count; index++)
      {
         sw21553_SwVlanDisplayVLanTableEntry(index, vlanTable[index]) ;
      }
   }

   sw21553_SwVlanDisplayVLanTableTrailer(count) ;

   return(0) ;
}


/******************************************************************************
*  sysSwVLanTableReadTag()
*     read and show a VLAN tag within the VLAN tag table
*
*  Parameter:
*     index:      index of the VLAN Table
*
*/
int sysSwVLanTableReadTag( int index )
{
   UINT32 vlanTableEntry ;

   vlanTableEntry = sw21553_SwVLanReadTableEntry(index) ;

   switch (vlanTableEntry) 
   {
      case 0: /* no valid entry found at this index */
         printk("NO entry found at VLAN table[%d]\n", index ) ;
         return(0) ;

      case ERROR: /* index not valid (out of range) */
         printk("ERROR: index not valid (range 0..%d) - cannot read VLAN table[%d]\n", 
                INCA_IP_MAX_VLAN_TABLE_ENTRIES-1 ,
                index ) ;
         return(ERROR) ;

      default :
         printk("VLAN table[%d] = %08x:  VLAN ID = %x (%d), Ctrl = %x \n",
                index,
                vlanTableEntry, 
                INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vlanTableEntry),
                INCA_IP_VLAN_TABLE_ENTRY_GET_VLAN_ID(vlanTableEntry),
                INCA_IP_VLAN_TABLE_ENTRY_GET_CTRL_FLAGS(vlanTableEntry) ) ;
         return(OK) ;

   }

   return(ERROR) ;
}


#ifdef INCLUDE_LED_MULTIPLEXER

/******************************************************************************
   Switch/END LED's ACCESS: functions
******************************************************************************/


/******************************************************************************
*  sysSwLedMuxAllLan()
*     enable all LED's for signal LAN port properties
*
*  Parameter:
*     txAct: activate also the activities of transmit
*        0: default, signal only receive activities
*        1: signal also the transmit acitvities
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedMuxAllLan(int txAct)
{
   /* set the config register */
   if (txAct)
   {
      sw21553_SwLedSetLedMuxConfig( (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;
   }
   else
   {
      sw21553_SwLedSetLedMuxConfig( ((INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) & ~(INCA_IP_TSF_LEDMUX_CONF_ETL1) ), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;
   }

   return(0) ;
}

/******************************************************************************
*  sysSwLedDeMuxAllLan()
*     deactivate multiplexing of all LED's for signal LAN port properties
*
*  Parameter:
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedDeMuxAllLan(void)
{
   /* reset LEDMUX_CONF register */
   sw21553_SwLedSetLedMuxConfig( 0, (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;

   return(0) ;
}


/******************************************************************************
*  sysSwLedMuxLan()
*     configure and set the LED's for signal the LAN port properties
*
*  Parameter:
*     status: display the status of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: normal
*                LED on:  ehternet port is disabled
*                LED flashing: ERROR condition
*     speed:  displays the speed of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: 10 Mbit/s, LED on: 100 Mbit/s
*     Duplex: displays the duplex mode of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: Half Duplex, LED on: Full Duplex
*     Activity: signals any receive activities on the ethernet port
*             0: do not signal activities
*             1: do signal activities 
*                LED on (non-flashing): Link without activity
*                LED on (flashing):     Port receiving (or transmitting, see txAct)
*     txAct:  signals also the transmit activities
*             0: do not signal transmit activities
*             1: do signal transmit activities 
*
*  Returns:
*     always 0: success
*
*  REMARK:
*
*/
int sysSwLedMuxLan(int status, int speed, int duplex, int Activity, int txAct)
{
   UINT32 newSettings = 0 ;

   /* create LEDMUX_CONF settings */
   newSettings |= (status == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_ESTA1) : 0 ;
   newSettings |= (speed == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_EACT1) : 0 ;
   newSettings |= (duplex == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_EDPX1) : 0 ;
   newSettings |= (Activity == 1) ? (INCA_IP_TSF_LEDMUX_CONF_ESPD1) : 0 ;
   newSettings |= (txAct == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_ETL1)  : 0 ;

   sw21553_SwLedSetLedMuxConfig( (newSettings), 
                                 (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_LAN) ) ;

   return(0) ;
}



#if defined(INCA_STANDARD)

/******************************************************************************
*  sysSwLedMuxAllPc()
*     enable all LED's for signal PC port properties
*
*  Parameter:
*     txAct: activate also the activities of transmit
*        0: default, signal only receive activities
*        1: signal also the transmit acitvities
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedMuxAllPc(int txAct)
{
   /* set the config register */
   if (txAct)
   {
      sw21553_SwLedSetLedMuxConfig( (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;
   }
   else
   {
      sw21553_SwLedSetLedMuxConfig( ((INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) & ~(INCA_IP_TSF_LEDMUX_CONF_ETL0) ), 
                                    (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;
   }

   return(0) ;
}

/******************************************************************************
*  sysSwLedDeMuxAllPc()
*     deactivate all LED's for signal PC port properties
*
*  Parameter:
*
*  Returns:
*     ERROR: not expected here
*     0:     settings successful set
*
*  REMARK:
*
*/
int sysSwLedDeMuxAllPc(void)
{
   /* reset LEDMUX_CONF register */
   sw21553_SwLedSetLedMuxConfig( 0, (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;

   return(0) ;
}

/******************************************************************************
*  sysSwLedMuxPc()
*     configure and set the LED's for signal the PC port properties
*
*  Parameter:
*     status: display the status of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: normal
*                LED on:  ehternet port is disabled
*                LED flashing: ERROR condition
*     speed:  displays the speed of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: 10 Mbit/s, LED on: 100 Mbit/s
*     Duplex: displays the duplex mode of the ethernet port
*             0: do not display
*             1: do display 
*                LED off: Half Duplex, LED on: Full Duplex
*     Activity: signals any receive activities on the ethernet port
*             0: do not signal activities
*             1: do signal activities 
*                LED on (non-flashing): Link without activity
*                LED on (flashing):     Port receiving (or transmitting, see txAct)
*     txAct:  signals also the transmit activities
*             0: do not signal transmit activities
*             1: do signal transmit activities 
*
*  Returns:
*     always 0: success
*
*  REMARK:
*
*/
int sysSwLedMuxPc(int status, int speed, int duplex, int Activity, int txAct)
{
   UINT32 newSettings = 0 ;

   /* create LEDMUX_CONF settings */
   newSettings |= (status == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_ESTA0) : 0 ;
   newSettings |= (speed == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_EACT0) : 0 ;
   newSettings |= (duplex == 1)   ? (INCA_IP_TSF_LEDMUX_CONF_EDPX0) : 0 ;
   newSettings |= (Activity == 1) ? (INCA_IP_TSF_LEDMUX_CONF_ESPD0) : 0 ;
   newSettings |= (txAct == 1)    ? (INCA_IP_TSF_LEDMUX_CONF_ETL0)  : 0 ;

   sw21553_SwLedSetLedMuxConfig( (newSettings), 
                                 (INCA_IP_TSF_LEDMUX_CONF_SW_MASK_PC) ) ;

   return(0) ;
}

#endif      /* #if defined(INCA_STANDARD) */

#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */


/*
   for testing
   ===========
   calculate the appropriate hash value for the given 
   MAC address (string form 00:22:33:aa:bb:cc) and the 
   requested modulation

   Remark: 
      modulation = 3 --> calculate hash from original MAC address
*/
int sysSwMacCalcStrHashValue(int modulateNum, char *strMacAddr)
{
   UCHAR tempMacAddr[6] ;
   UCHAR hashValue ;
   
   sw21553_SwMacConvertStr2Hex(strMacAddr, tempMacAddr) ;
   printk("MAC Address: Str<%s> --> Hex %2x-%2x-%2x-%2x-%2x-%2x \n",
           strMacAddr,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5] ) ;

   hashValue = sw21553_IncaIpSwitchCalcHash(modulateNum, tempMacAddr) ;

   printk("Modulation %d: MAC addr = %2x-%2x-%2x-%2x-%2x-%2x --> hash = 0x%02x (%3d) \n",
           modulateNum,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5],
           hashValue, hashValue) ;
   return(OK) ;
}

/*
   for testing
   ===========
   calculate the appropriate hash value for the given 
   MAC address for all available modulations.

   Remark: 
      modulation = 3 --> calculate hash from original MAC address
*/
int sysSwMacCalcStrAllHashValue(char *strMacAddr)
{
   int modulateNum ;
   UCHAR tempMacAddr[6] ;
   UCHAR hashValue ;

   printk("\n-----------------------------------------\n") ;
   sw21553_SwMacConvertStr2Hex(strMacAddr, tempMacAddr) ;
   printk("MAC Address: Str<%s> --> Hex %2x-%2x-%2x-%2x-%2x-%2x \n",
           strMacAddr,
           tempMacAddr[0], tempMacAddr[1], tempMacAddr[2],
           tempMacAddr[3], tempMacAddr[4], tempMacAddr[5] ) ;
   for (modulateNum = 0; modulateNum < 4; modulateNum++)
   {
      printk("-----------------------------------------\n") ;
      hashValue = sw21553_IncaIpSwitchCalcHash(modulateNum, tempMacAddr) ;
      printk("Modulation %d: HASH value = 0x%02x (%3d) \n",
              modulateNum, hashValue, hashValue) ;
   }
   printk("-----------------------------------------\n\n") ;
   return(OK) ;
}

/* Linux specific enhancements */
void sw21553_PrintReg(UINT32 addr);
void sw21553_WrReg(UINT32 addr, UINT32 value);
static struct switch_api_command {
    const char *name;
    const void *function;
    const char *param;          /* i=int, l=long, s=string */
} commands[] = {
    { "sysSwQosSetPrioCos", sysSwQosSetPrioCos, "l" },	
    { "sysSwQosSetDscpCos", sysSwQosSetDscpCos, "il" },
    { "sysSwQosSelectFinalPrio", sysSwQosSelectFinalPrio, "i" },
    { "sysSwQosSetPortPrio", sysSwQosSetPortPrio, "ii" },	 
    { "sysSwQosSetSchedMode", sysSwQosSetSchedMode, "i" },	
    { "sysSwQosSetPrioId", sysSwQosSetPrioId, "iii" },	
    { "sysSwSetPmacL2HeaderHandling", sysSwSetPmacL2HeaderHandling, "i" },	
    { "sysSwSetPmacAddTagEnDisable", sysSwSetPmacAddTagEnDisable, "i" },
    { "sysSwQosSetVoicePortRange", sysSwQosSetVoicePortRange, "ii" },
    { "sysSwQosStreamClassification", sysSwQosStreamClassification, "i" },
	    
    { "sysPhyShowLan", sysPhyShowLan, "i" },
    { "sysPhyShowPc", sysPhyShowPc, "i" },
    { "sysPhySetLanAdvertise", sysPhySetLanAdvertise, "li" },
    { "sysPhySetPcAdvertise", sysPhySetPcAdvertise, "li" },
    { "sysPhySetLanFixed", sysPhySetLanFixed, "li" },
    { "sysPhySetPcFixed", sysPhySetPcFixed, "li" },
    { "sysPhyLanReAutoNeg", sysPhyLanReAutoNeg, "" },
    { "sysPhyPcReAutoNeg", sysPhyPcReAutoNeg, "" },
    { "sysPhyLanDisableAutoNeg", sysPhyLanDisableAutoNeg, "" },
    { "sysPhyLanEnableAutoNeg", sysPhyLanEnableAutoNeg, "" },
    { "sysPhyPcDisableAutoNeg", sysPhyPcDisableAutoNeg, "" },
    { "sysPhyPcEnableAutoNeg", sysPhyPcEnableAutoNeg, "" },
    { "sysPhyLanReset", sysPhyLanReset, "" },
    { "sysPhyPcReset", sysPhyPcReset, "" },
    { "sysSwUnDestShow", sysSwUnDestShow, "" },
    { "sysSwUnDestSetPorts", sysSwUnDestSetPorts, "ii" },
    { "sysSwSpPtSetPorts", sysSwSpPtSetPorts, "ii" },
    { "sysSwSpPtShow", sysSwSpPtShow, "" },
    { "sysSwPktLenSet", sysSwPktLenSet, "li" },
    { "sysSwPktLenShow", sysSwPktLenShow, "" },
    { "sysSwMacShowTable", sysSwMacShowTable, "" },
    { "sysSwMacAddTableEntry", sysSwMacAddTableEntry, "siii" },
    { "sysSwMacClearEntryByMac", sysSwMacClearEntryByMac, "s" },
    { "sysSwMacClearEntryByIndex", sysSwMacClearEntryByIndex, "i" },
    { "sysSwVLanShowVLanCtrl", sysSwVLanShowVLanCtrl, "" },
    { "sysSwVLanSetVLanCtrlPort", sysSwVLanSetVLanCtrlPort, "iii" },
    { "sysSwVLanUpdVLanCtrlVAware", sysSwVLanUpdVLanCtrlVAware, "i" },
    { "sysSwVLanShowPortConfig", sysSwVLanShowPortConfig, "" },
    { "sysSwVLanSetPortConfig", sysSwVLanSetPortConfig, "ill" },
    { "sysSwVLanTableReset", sysSwVLanTableReset, "" },
    { "sysSwVLanTableReadTag", sysSwVLanTableReadTag, "i" },
    { "sysSwVLanAddTableEntry", sysSwVLanAddTableEntry, "lll" },
    { "sysSwVLanShowTable", sysSwVLanShowTable, "" },
    { "sysSwVLanTableSetTag", sysSwVLanTableSetTag, "illl" },
#ifdef INCLUDE_LED_MULTIPLEXER
    { "sysSwLedMuxAllLan", sysSwLedMuxAllLan, "i" },
    { "sysSwLedDeMuxAllLan", sysSwLedDeMuxAllLan, "" },
    { "sysSwLedMuxLan", sysSwLedMuxLan, "iiiii" },
#if defined(INCA_STANDARD)
    { "sysSwLedMuxAllPc", sysSwLedMuxAllPc, "i" },
    { "sysSwLedDeMuxAllPc", sysSwLedDeMuxAllPc, "" },
    { "sysSwLedMuxPc", sysSwLedMuxPc, "iiiii" },
#endif      /* #if defined(INCA_STANDARD) */
#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */
    { "sw21553_SwMacResetMacTable", sw21553_SwMacResetMacTable, "" },
    { "sw21553_SwUpdSpPtPortState", sw21553_SwUpdSpPtPortState, "ii" },
    { "sw21553_PrintReg", sw21553_PrintReg, "l" },
    { "sw21553_WrReg", sw21553_WrReg, "ll" },
    { "sw21553_SwSetEPhy", sw21553_SwSetEPhy, "ll" }
};

static const char *get_string(const char **buffer, int *len)
{
    const char *s1 = *buffer;
    const char *s2;
    int quoted = 0;

    while (*s1 == ' ') { s1++; }

    if ((*s1 == '\0') || (*s1 == '\n')) /* end of string */
    {
        *len = 0;
        return NULL;
    }

    if (*s1 == '"')
    {
        quoted = 1;
        s1++;
    }

    s2 = s1;
    while ((*s2) && (*s2 != '\n'))
    {
        if (((*s2 == ' ') && (!quoted)) || 
            ((*s2 == '"') && ( quoted)) ||
            ((*s2 == ',') && (!quoted)))
        {
            break;
        }
        s2++;
    }

    *len = (s2 - s1);
    
    if (quoted && (*s2 == '"')) s2++;
    if (*s2 == ',') s2++;       /* remove comma between arguments */
    *buffer = s2;               /* save start of next token */
    
/*      { */
/*          int i; */
/*          printk("get_string: \""); */
/*          for (i = 0; i < *len; i++) printk("%c", s1[i]); */
/*          printk("\"\n"); */
/*      } */

    return s1;
}

static long atoi(const char *buffer, int len)
{
    long val = 0;
    int hex = 0;

    if ((len > 2) && (buffer[0]=='0') && (buffer[1]=='x'))
    {
        hex = 1;
        len -= 2;
        buffer += 2;
    }

    while (*buffer && len)
    {
        if ((*buffer >= '0') && (*buffer <= '9'))
        {
            val = val * ((hex) ? 16 : 10) + (*buffer - '0');
        }
        else if ((hex && (*buffer >= 'A') && (*buffer <= 'F')) ||
                 (hex && (*buffer >= 'a') && (*buffer <= 'f')))
        {
            val = val * 16 + ((*buffer & ~0x20) - 'A' + 10);
        }
        else
        {
            printk("atoi: invalid character: '%c'\n", *buffer);
            return 0;
        }
        len--;
        buffer++;
    }

    return val;
}

static int inca_switch_read_procmem(char *buf, char **start, off_t offset, 
                                        int count, int *eof, void *data)
{
    int len = 0, c;
    
    len += sprintf(buf+len,"Available commands:\n");

    for (c = 0; c < (sizeof(commands)/sizeof(commands[0])); c++)
    {
        len += sprintf(buf+len, "\t%s\t%s\n", 
                       commands[c].name, commands[c].param);
    }    
    *eof = 1;	// No more data available
    
    return len;
}

static int inca_switch_write_procmem(struct file *file, const char *buffer,
					unsigned long count, void *data)
{
    const char *buf = buffer;
    const char *cmd;
    const char *parg_string;
    const char *arg;
    int     slen, c;
    long    cmd_arg[10]={}; /* max. 10 command arguments (integer/long/string) */
    char    cmd_arg_strings[100]; /* max 100 characters as string parameters */
    int     cmd_arg_str_offset = 0;
    int     argc = 0;
    int     ret = 0;

    cmd = get_string(&buf, &slen);
    if ((cmd == NULL) || (slen == 0))
    {
        printk("No/invalid command specified (%s)\n", buffer);
        return -EINVAL;
    }
    
    for (c = 0; c < (sizeof(commands)/sizeof(commands[0])); c++)
    {
        if (strncmp(cmd, commands[c].name, slen) == 0)
        {
            /* yeah, we found our command */
/*              printk("Command found: name=\"%s\", fct=%p, param=\"%s\"\n", */
/*                     commands[c].name, commands[c].function, commands[c].param); */
            break;
        }
    }
    
    if (c >= (sizeof(commands)/sizeof(commands[0])))
    {
        int i;
        printk("Command not found (");
        for (i = 0; i < slen; i++) 
            printk("%c", cmd[i]);
        printk(")\n");
        return -EINVAL;
    }
    
    parg_string = commands[c].param;
    while ((*parg_string) && (argc < (sizeof(cmd_arg)/sizeof(cmd_arg[0]))))
    {
        arg = get_string(&buf, &slen);
        if ((arg == NULL) || (slen == 0))
        {
            printk("Too few parameters for command \"%s\", required are \"%s\"\n",
                   commands[c].name, commands[c].param);
            printk("arg=%p, slen=%d\n", arg, slen);
            return -EINVAL;
        }
        
        switch (*parg_string)
        {
            case 'i':
            case 'l':
                cmd_arg[argc++] = atoi(arg, slen);
                break;
            case 's':
                memcpy(&cmd_arg_strings[cmd_arg_str_offset], arg, slen);
                cmd_arg_strings[cmd_arg_str_offset + slen] = '\0';
                cmd_arg[argc++] = (long)&cmd_arg_strings[cmd_arg_str_offset];
                cmd_arg_str_offset += slen+1;
                break;
            default:
                printk("Invalid format string in module, please fix and re-compile\n");
                return -EINVAL;
        }
        
        parg_string++;
    }

    arg = get_string(&buf, &slen);
    if ((arg != NULL) && (slen != 0))
    {
        
        printk("Too many parameters for command \"%s\", required are \"%s\"\n",
               commands[c].name, commands[c].param);
               
        return -EINVAL;
    }
    
    /* This instructions casts the void pointer commands.function to a function that
     returns int and has 10 long parameters. Though the function actually called has
     a different argument list it works if the parameter are set up accordingly
     (e.g. int/long or string pointer) */
    ret = ((int (*)(long,long,long,long,long,long,long,long,long,long))(commands[c].function))
        (cmd_arg[0], cmd_arg[1], cmd_arg[2], cmd_arg[3], 
         cmd_arg[4], cmd_arg[5], cmd_arg[6], cmd_arg[7], 
         cmd_arg[8], cmd_arg[9]);
    printk("return value is: %d (0x%08x)\n", ret, ret);
    
    return count;
	
}

MODULE_LICENSE("Infineon");

static const char *dev_name="driver/switch_api";

int inca_switch_api_init(void)
{
    struct proc_dir_entry *inca_switch_proc;

    inca_switch_proc = create_proc_entry(dev_name, 0, 0);
    if (!inca_switch_proc) {
        printk(KERN_ERR "could not create /proc/%s!\n", dev_name);
        return -ENOENT;
    }
    inca_switch_proc->read_proc  = inca_switch_read_procmem;
    inca_switch_proc->write_proc = inca_switch_write_procmem;
#ifdef DEBUG
    printk(KERN_NOTICE "/proc/%s created!\n", dev_name);
#endif

    sw21553_IncaIpInitHashGenPoly();
    
    InitVLANPortPrio();
    stream_classification = 0;

    return 0;
}

void inca_switch_api_exit(void)
{
    remove_proc_entry(dev_name, 0);
#ifdef DEBUG
    printk(KERN_NOTICE "/proc/%s removed!\n", dev_name);
#endif
}

module_init(inca_switch_api_init);
module_exit(inca_switch_api_exit);


EXPORT_SYMBOL(sysSwQosSetPortPrio);
EXPORT_SYMBOL(sysSwQosSetPrioCos);	
EXPORT_SYMBOL(sysSwQosSetDscpCos);
EXPORT_SYMBOL(sysSwQosSelectFinalPrio);
EXPORT_SYMBOL(sysSwQosSetSchedMode);
EXPORT_SYMBOL(sysSwSetPmacL2HeaderHandling);
EXPORT_SYMBOL(sysSwQosSetPrioId);
EXPORT_SYMBOL(sysSwSetPmacAddTagEnDisable);
EXPORT_SYMBOL(sysSwQosSetVoicePortRange);
EXPORT_SYMBOL(sysSwQosStreamClassification);

EXPORT_SYMBOL(sysPhyShowLan);
EXPORT_SYMBOL(sysPhyShowPc);
EXPORT_SYMBOL(sysPhySetLanAdvertise);
EXPORT_SYMBOL(sysPhySetPcAdvertise);
EXPORT_SYMBOL(sysPhySetLanFixed);
EXPORT_SYMBOL(sysPhySetPcFixed);
EXPORT_SYMBOL(sysPhyLanReAutoNeg);
EXPORT_SYMBOL(sysPhyPcReAutoNeg);
EXPORT_SYMBOL(sysPhyLanDisableAutoNeg);
EXPORT_SYMBOL(sysPhyLanEnableAutoNeg);
EXPORT_SYMBOL(sysPhyPcDisableAutoNeg);
EXPORT_SYMBOL(sysPhyPcEnableAutoNeg);
EXPORT_SYMBOL(sysPhyLanReset);
EXPORT_SYMBOL(sysPhyPcReset);
EXPORT_SYMBOL(sysSwUnDestShow);
EXPORT_SYMBOL(sysSwUnDestSetPorts);
EXPORT_SYMBOL(sysSwSpPtSetPorts);
EXPORT_SYMBOL(sysSwSpPtShow);
EXPORT_SYMBOL(sysSwPktLenSet);
EXPORT_SYMBOL(sysSwPktLenShow);
EXPORT_SYMBOL(sysSwMacShowTable);
EXPORT_SYMBOL(sysSwMacAddTableEntry);
EXPORT_SYMBOL(sysSwMacClearEntryByMac);
EXPORT_SYMBOL(sysSwMacClearEntryByIndex);
EXPORT_SYMBOL(sysSwVLanShowVLanCtrl);
EXPORT_SYMBOL(sysSwVLanSetVLanCtrlPort);
EXPORT_SYMBOL(sysSwVLanUpdVLanCtrlVAware);
EXPORT_SYMBOL(sysSwVLanShowPortConfig);
EXPORT_SYMBOL(sysSwVLanSetPortConfig);
EXPORT_SYMBOL(sysSwVLanTableReset);
EXPORT_SYMBOL(sysSwVLanTableReadTag);
EXPORT_SYMBOL(sysSwVLanAddTableEntry);
EXPORT_SYMBOL(sysSwVLanShowTable);
EXPORT_SYMBOL(sysSwVLanTableSetTag);
#ifdef INCLUDE_LED_MULTIPLEXER
EXPORT_SYMBOL(sysSwLedMuxAllLan);
EXPORT_SYMBOL(sysSwLedDeMuxAllLan);
EXPORT_SYMBOL(sysSwLedMuxLan);
#if defined(INCA_STANDARD)
EXPORT_SYMBOL(sysSwLedMuxAllPc);
EXPORT_SYMBOL(sysSwLedDeMuxAllPc);
EXPORT_SYMBOL(sysSwLedMuxPc);
#endif      /* #if defined(INCA_STANDARD) */
#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */
