/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   Module      : IncaSwitchApi.h
   Date        : 13.05.2003
   Description : provide an API to read and configure the INCA-IP ethernet parts
                 like PHY's and switch module


   $Log: inca_switch_api.h,v $
   Revision 1.1.2.1  2004/11/19 10:39:34  de6wmigr
   Function declarations for QoS related api functions added.

   Revision 1.1  2004/03/10 14:59:05  de6wanme
   Added first INCA-IP support

   Revision 1.5  2003/07/07 17:41:38  mangele
   - Add Switch LED API

   Revision 1.4  2003/06/04 15:55:20  mangele
   Add further API functions for access switch INCA IP module

   Revision 1.3  2003/06/02 21:52:33  mangele
   Add further functionallity for the switch API

   Revision 1.2  2003/05/23 13:36:17  mangele
   Splitt of vxWorks user functions and switch/PHY API functions (see IncaSwitchAccess.c)
   Rename vxWorks user functions (sysPhy..., sysSw..)

   Revision 1.1  2003/05/20 13:34:08  mangele
   add PHY API: access LAN and PC PHY
   add SWITCH API: switch MAC table


   
******************************************************************************/
#ifndef _INCASWITCHAPI_H
#define _INCASWITCHAPI_H


/******************************************************************************
   PHY API: functions
******************************************************************************/

/* LAN - phyInfo: 0: current status, 1: advertise, 2: partner */
extern int sysPhyShowLan(int phyInfo) ;
/* PC - phyInfo: 0: current status, 1: advertise, 2: partner */
extern int sysPhyShowPc(int phyInfo) ;

/* LAN PHY advertise: newSettings bit[0..4] 
               10 MBit/s HD | 10 MBit/s FD | 100 MBit HD | 100 MBit/s FD | PAUSE */
extern int sysPhySetLanAdvertise(UINT32 newSettings, int doRestartAutoNeg) ;
/* PC PHY advertise: newSettings bit[0..4] 
               10 MBit/s HD | 10 MBit/s FD | 100 MBit HD | 100 MBit/s FD | PAUSE */
extern int sysPhySetPcAdvertise(UINT32 newSettings, int doRestartAutoNeg) ;

/* LAN PHY fixed (without Auto Neg.): 
      newSpeed bit[0..2]: 10 MBit | 100 MBit | FULL DUPLEX
      doDisableAutoNeg: 1 --> disable auto negotiation
*/
extern int sysPhySetLanFixed(UINT32 newSpeed, int doDisableAutoNeg) ;
/* PC PHY fixed (without Auto Neg.): 
      newSpeed bit[0..2]: 10 MBit | 100 MBit | FULL DUPLEX
      doDisableAutoNeg: 1 --> disable auto negotiation */
extern int sysPhySetPcFixed(UINT32 newSpeed, int doDisableAutoNeg) ;

/* LAN PHY: restart Auto Negotiation */
extern int sysPhyLanReAutoNeg() ;
/* PC PHY: restart Auto Negotiation */
extern int sysPhyPcReAutoNeg() ;

/* LAN PHY: disable Auto Negotiation */
extern int sysPhyLanDisableAutoNeg() ;
/* LAN PHY: enable Auto Negotiation */
extern int sysPhyLanEnableAutoNeg() ;

/* PC PHY: disable Auto Negotiation */
extern int sysPhyPcDisableAutoNeg() ;
/* PC PHY: enable Auto Negotiation */
extern int sysPhyPcEnableAutoNeg() ;

/* LAN PHY: reset phy */
extern int sysPhyLanReset() ;
/* PC PHY: reset phy */
extern int sysPhyPcReset() ;

/******************************************************************************
   SWITCH API: common switch commands
******************************************************************************/

/* read and display the UN_DEST register */
int sysSwUnDestShow() ;

/* set and change the UN_DEST register per port */
int sysSwUnDestSetPorts(int portSelect, int portBehaviour) ;

/* update the port state within the Spanning Tree Port Status  */
int sysSwSpPtSetPorts(int portSelect, int newPortState) ;

/* read and display the ST_PT register */
int sysSwSpPtShow() ;

/* set the PKT_LEN register (Packet Length Register) */
int sysSwPktLenSet(UINT32 newPktLen, int addTagLen) ;

/* read and display the PKT_LEN register */
int sysSwPktLenShow() ;

int stream_classification_enabled( void );

int get_vlan_prio_and_id( int data_type, int* identifier, int* priority );

int get_port_range( int data_type, UINT32* port_min, UINT32* port_max );





/******************************************************************************
   SWITCH API: MAC table and handling functions
******************************************************************************/

/* Display the entries of the switch MAC table */
extern int sysSwMacShowTable() ;

/* add entry to the switch MAC table:
   pStrEtherAdr:   MAC addr string (form "00:ab:12:34:4b:78")
   port:           0: PC, 1: LAN, 2: CPU voice, 3: CPU management
   classification: 0: normal packets, 1: critical packets
   aging:          0: no aging (static entry), 1: normal aging
*/

extern int sysSwMacAddTableEntry(char *pStrEtherAdr, int port, int classification, int aging ) ;

/* Clear all MAC table entries with the given MAC address:
      pStrEtherAdr:   MAC addr string (form "00:ab:12:34:4b:78")
*/
extern int sysSwMacClearEntryByMac(char *pStrEtherAdr) ;

/* Clear a MAC table entry identified by the given index 
      index:  index of the MAC table entry for clearance
*/
extern int sysSwMacClearEntryByIndex(int index) ;


/******************************************************************************
   SWITCH API: VLAN table and handling functions
******************************************************************************/

/* display the VLAN control settings (register VLAN_CTRL) */
extern int sysSwVLanShowVLanCtrl() ;

/* set the VLAN settings for an given port (register VLAN_CTRL) 
   portSelect:             0: PC, 1: LAN, 2: CPU
   sendOnlyToMember:       0: can send also to non member VLAN's, 
                           1: can send only to member VLAN's
   notAcceptUntaggedPkt:   0: accept untagged packets too, 
                           1: do not accept untagged packets
*/
extern int sysSwVLanSetVLanCtrlPort(int portSelect, int sendOnlyToMember, int notAcceptUntaggedPkt) ;

/* switch ON/OFF VLAN aware (register VLAN_CTRL) 
   vLanAware: 0: set VLAN unaware, 1: set VLAN aware
*/
extern int sysSwVLanUpdVLanCtrlVAware(int vLanAware) ;

/* show the current port (PC, LAN, CPU) VLAN settings */
extern int sysSwVLanShowPortConfig() ;

/* set the VLAN settings for an given port
   portSelect: 0: PC, 1: LAN, 2: CPU
   newPrio:    new priority value to set (range 0 - 7)
   newVLanId:  new VLAN ID to set (range 0 - 4096)
               consider: 4096 (0xFFF): reserved, 0 (0x000): priority tag 
*/
extern int sysSwVLanSetPortConfig(int portSelect, UINT32 newPrio, UINT32 newVLanId) ;

/* reset the complete VLAN table */
extern int sysSwVLanTableReset() ;

/* read and show a VLAN tag within the VLAN tag table */
extern int sysSwVLanTableReadTag( int index ) ;

/*
   add a VLAN tag within the VLAN tag table 
   !!! index will be calculatet !!!
   newVLanId:  new VLAN ID to set (range 0 - 4096)
               consider: 4096 (0xFFF): reserved, 0 (0x000): priority tag 
   newMemberInfo: Membership information for PC/LAN/CPU Port
   (3 bit field)     0: Packets from PC/LAN/CPU port is not a member of VLAN
                     1: Packets from PC/LAN/CPU port is member of this VLAN
   newEgressTagInfo: Egress Tagging information for PC/LAN/CPU Port
   (3 bit field)     0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
                     1: Packets to PC/LAN/CPU egress port is tagged for this VLAN

   3 bit field: Bit 0 ==> PC port, Bit 1 ==> LAN port, Bit 2 ==> CPU port
*/
extern int sysSwVLanAddTableEntry(UINT32 newVLanId, UINT32 newMemberInfo, UINT32 newEgressTagInfo) ;

/* show the VLAN table */
extern int sysSwVLanShowTable( ) ;



/* 
   only for testing
   ================
   set a VLAN tag within the VLAN tag table 
   index:      index of the VLAN Table
   newVLanId:  new VLAN ID to set (range 0 - 4096)
               consider: 4096 (0xFFF): reserved, 0 (0x000): priority tag 
   newMemberInfo: Membership information for PC/LAN/CPU Port
   (3 bit field)     0: Packets from PC/LAN/CPU port is not a member of VLAN
                     1: Packets from PC/LAN/CPU port is member of this VLAN
   newEgressTagInfo: Egress Tagging information for PC/LAN/CPU Port
   (3 bit field)     0: Packets to PC/LAN/CPU egress port is not tagged for this VLAN
                     1: Packets to PC/LAN/CPU egress port is tagged for this VLAN

   3 bit field: Bit 0 ==> PC port, Bit 1 ==> LAN port, Bit 2 ==> CPU port
*/
extern int sysSwVLanTableSetTag( int index, 
                                 UINT32 newVLanId, 
                                 UINT32 newMemberInfo, 
                                 UINT32 newEgressTagInfo) ;




#ifdef INCLUDE_LED_MULTIPLEXER
/******************************************************************************
   Switch/END LED's ACCESS: functions
******************************************************************************/

/* enable all LED's for signal LAN port properties */
extern int sysSwLedMuxAllLan(int txAct) ;
/* deactivate all LED's for signal LAN port properties */
extern int sysSwLedDeMuxAllLan() ;
/* configure and set the LED's for signal the LAN port properties */
extern int sysSwLedMuxLan(int status, int speed, int duplex, int Activity, int txAct) ;

#if defined(INCA_STANDARD)
/* enable all LED's for signal PC port properties */
extern int sysSwLedMuxAllPc(int txAct) ;
/* deactivate all LED's for signal PC port properties */
extern int sysSwLedDeMuxAllPc() ;
/* configure and set the LED's for signal the PC port properties */
extern int sysSwLedMuxPc(int status, int speed, int duplex, int Activity, int txAct) ;
#endif      /* #if defined(INCA_STANDARD) */

#endif      /* #ifdef INCLUDE_LED_MULTIPLEXER */

#endif      /* #ifndef _INCASWITCHAPI_H */
