/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/******************************************************************************
   Module      : IncaSwitchPhy.c
   Date        : 26.09.2003
   Description :
   
   $Log: inca_switch_phy.c,v $
   Revision 1.1  2004/03/10 14:59:05  de6wanme
   Added first INCA-IP support

   Revision 1.2  2004/01/15 12:59:59  de6wmigr
   several printks removed that occured on phy state changes

   Revision 1.1  2004/01/12 09:13:50  de6wmigr
   Phy driver routines taken from VxWorks BSP vers. 1.8 incl. parallel detection functionality..

   Revision 1.8  2003/11/26 13:16:14  aschmidt
   - disable switch ports for periods of unknown PHY state
   - implemented callback function for PHY events LINK_INACTIVE, LINK_ACTIVE and PHY_INIT
   - merged "PhyModifySpeed" into function "PhySetFixSpeed" again

   Revision 1.7  2003/11/24 12:42:33  aschmidt
   eliminated compiler warnings

   Revision 1.6  2003/11/20 14:35:29  aschmidt
   advertise all capabilities (incl. Pause capability)

   Revision 1.5  2003/11/07 15:51:27  aschmidt
   * revised the "Fixed Speed" operation of PHY:
     In "Fixed Speed" mode there is no interrupt for LinkUp situation. Updated   PhyTask to poll for LinkUp event if "Fixed Speed" is used.

   Revision 1.4  2003/10/16 11:05:54  aschmidt
   - corrected initial handling of AutoMDIX (no reconfig desired here if already configured earlier by other boot code)
   - check unsuccessful MDI/MDIX toggle with "LinkDown" instead of "LinkDown + EnergyOff"

   Revision 1.3  2003/10/13 19:06:07  neumann
   * Changed Auto MDIX behaviour in case link is up during initialization
   * Disabled Energy-Detect/Power-Down logic due to problems in unconnected state

   Revision 1.2  2003/09/26 17:25:23  mangele
   adaption to the new module "IncaSwitchPhy.*"

   Revision 1.1  2003/09/26 14:23:28  aschmidt
   This new file contains the PHY and AutoMDIX functions.
   The PHY-task is interrupt driven and updates the Switch ports with
   the correct Link Status.
   Interrupt control has been removed from AutoMDIX Task.

   
******************************************************************************/

/* includes */
#if 0
#include "vxWorks.h"
#include "stdlib.h"
#include "cacheLib.h"
#include "intLib.h"
#include "lstLib.h"        /* Needed to maintain protocol list. */
#include "wdLib.h"
#include "iv.h"
#include "semLib.h"
#include "logLib.h"
#include "netLib.h"
#include "stdio.h"
#include "sysLib.h"
#include "errno.h"
#include "errnoLib.h"
#include "memLib.h"
#include "iosLib.h"

#include "tickLib.h"
#include "timers.h"

#include "sys/socket.h"
#include "sys/ioctl.h"
#include "sys/times.h"
#include "RegIncaIP.h"


#endif

/* #define EXPORT_SYMTAB  */
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/spinlock.h>
#include "vxWorks_type.h" 
#include <asm/irq.h> 
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/irq.h>
#include <linux/interrupt.h>

#include <linux/malloc.h>


#include "inca_switch_access.h"
/* access function to the switch module */
#include "inca_switch_phy.h"   

#include "inca_switch_settings.h"
#include "inca_mdio_settings.h"

#include <linux/module.h>



/* #define PHY_DEBUG_ENABLE */





/* DEBUG MACROS */
#ifdef PHY_DEBUG_ENABLE
   /* debug print out masks */
   #define PHY_DEBUG_OFF          0x00000000
   #define PHY_DEBUG_INIT         0x00000001
   #define PHY_DEBUG_INT          0x00000040
   #define PHY_DEBUG_FSM          0x00000100
   #define PHY_DEBUG_FULL         0xFFFFFFFF

   #define PHY_DEBUG_MASK         (PHY_DEBUG_FSM | PHY_DEBUG_INT | PHY_DEBUG_INIT) 

   #define PHY_DEBUG(type,X)         if((UINT32)type & PHY_DEBUG_MASK) printf X
   #define PHY_LOG(type,S)           if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,0,0,0,0,0,0)
   #define PHY_LOG1(type,S,P1)       if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,P1,0,0,0,0,0)
   #define PHY_LOG2(type,S,P1,P2)    if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,P1,P2,0,0,0,0)
   #define PHY_LOG3(type,S,P1,P2,P3) if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,P1,P2,P3,0,0,0)
   #define PHY_LOG4(type,S,P1,P2,P3,P4)    if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,0,0)
   #define PHY_LOG5(type,S,P1,P2,P3,P4,P5) if((UINT32)type & PHY_DEBUG_MASK) logMsg(S,P1,P2,P3,P4,P5,0)

   #undef LOCAL
   #define LOCAL

#else
   #define PHY_DEBUG(type,X)          
   #define PHY_LOG(type,S)            
   #define PHY_LOG1(type,S,P1)        
   #define PHY_LOG2(type,S,P1,P2)     
   #define PHY_LOG3(type,S,P1,P2,P3)
   #define PHY_LOG4(type,S,P1,P2,P3,P4)
   #define PHY_LOG5(type,S,P1,P2,P3,P4,P5)

#endif /* PHY_DEBUG_ENABLE */


/* IMPORT   unsigned int sysIncaVer(void);  */
/*-------------------------------------------------------*/
swPhyStruct  devEthPhy[2];
/*
typedef task_str char[30];
task_str tasklet[PHY_STATE_MAX] [PHY_EVENT_MAX];
*/

void         ( *phyFsmAction [PHY_STATE_MAX] [PHY_EVENT_MAX] )( swPhyStruct *pPhy );
WDOG_ID wdId[2];
/*-------------------------------------------------------*/
void sw21553_PhyIsr(swPhyStruct *pPhy);
void sw21553_PhyCallbackFunctionSample(int event);
void sw21553_PhyLAN_Irq(int irq, void *dev_id, struct pt_regs *regs);
void sw21553_PhyPC_Irq(int irq, void *dev_id, struct pt_regs *regs);
/*
DECLARE_TASKLET(incaDMA_Rx_Tasklet0, incaDMA_Receive, 0 );
DECLARE_TASKLET(incaDMA_Rx_Tasklet1, incaDMA_Receive, 1 );


	phyFsmAction[AWAIT_ENERGY][ENERGY_DETECT  ] = fsmServeEnergy;
	phyFsmAction[AWAIT_LINK  ][LINK_ACTIVE    ] = fsmServeLinkUp;
	phyFsmAction[AWAIT_LINK  ][LINK_TRY_FAIL  ] = fsmServeLinkRetry;
	phyFsmAction[AWAIT_LINK  ][LINK_TRIES_TOUT] = fsmServeLinkTOut;
	phyFsmAction[LINK_UP     ][LINK_INACTIVE  ] = fsmServeLinkDown;
*/

typedef int FUNCPTR;

/* returns the number of ticks per second */
int sysClkRateGet( void )
{
   return 100;
}

struct timer_list*  wdCreate( void )
{
   struct timer_list* timer = NULL; 
    
   timer = (struct timer_list*) KSEG1ADDR( kmalloc(sizeof(struct timer_list), GFP_KERNEL));
   if(timer)
       init_timer( timer );
   return timer;
}

/*
 * This function removes the passed timer from the list
 */
void wdCancel( struct timer_list*  timer )
{
   del_timer( timer );
}

/*
 * This function starts a new timer acc. to the passed parameters
 */
void wdStart( struct timer_list*  timer,             /* wd id */ 
              int                 ticks,             /* retry ticks */
              void*               callback_function, /* call back function pointer */
              unsigned long int   cb_fnct_par ) /* callback function parameter address */
{

   timer->expires  = jiffies + ticks;
   timer->function = callback_function;
   timer->data     = cb_fnct_par;  

   add_timer( timer );

   
}


int sysIncaHWRegWrite(
   volatile UINT32 *pRegister,   /* Address of the register to write to */ 
   unsigned int value            /* 32-bit value to write */
)
{
   *pRegister = value;
   
   return 0;
}



int sysIncaHWRegSetBit(
   volatile UINT32 *pRegister,     /* Address of the register to write to */
   unsigned int bitMask            /* 32-bit value to write */
)
{
   unsigned int flags;

	save_and_cli( flags );
   
   *pRegister |= bitMask;

	restore_flags( flags );
   
   return 0;
}



int sysIncaHWRegResetBit(
   volatile UINT32 *pRegister, /* Address of the register to write to */ 
   unsigned int bitMask        /* 32-bit value to write */
)
{
   unsigned int flags;

   save_and_cli( flags );
   
   *pRegister &= ~bitMask;
   
	restore_flags( flags );
   
   return 0;
}







UINT32 sw21553_Write_MDIO_ACC_UnLocked( UINT32   value)
{
/*
   printf("MDIO_ACC (0x%08x): command = 0x%08x \n", 
          INCA_IP_Switch_MDIO_ACC, value ); */
	  
   SW_WRITE_REG_UNLOCKED(INCA_IP_Switch_MDIO_ACC, value);       
   do
   {
      SW_READ_REG_UNLOCKED(INCA_IP_Switch_MDIO_ACC, value);    
   } while (value & INCA_IP_Switch_MDIO_ACC_RA);
   
   return value;
}






/******************************************************************************
*  sw21553_PhyReadReg()
*     read a MDIO PHY register
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyRegIndex: select the MDIO PHY register
*
*  Returns:
*     ERROR: in case of invalid MDIO register index
*     else
*     REGISTER VALUE
*
*  REMARK:
*
*/
UINT32 sw21553_PhyReadReg(int portSelect, UINT32 phyRegIndex)
{
   UINT32   regValue ;
   UINT32   phyAddr = devEthPhy[portSelect % 2].PhyAddr;

   regValue = 0 ;

   /* read register value */
   switch (phyRegIndex)
   {
      case INCA_IP_MDIO_REG0_PHY_BCR   : /* basic control register PHY_BCR */
      case INCA_IP_MDIO_REG1_PHY_BSR   : /* basic status register  PHY_BSR */
      case INCA_IP_MDIO_REG2_PHY_ID1   : /* PHY identifier 1 PHY_ID1 */
      case INCA_IP_MDIO_REG3_PHY_ID2   : /* PHY identifier 2 PHY_ID2 */
      case INCA_IP_MDIO_REG4_PHY_ANAR  : /* Auto-Negotiation Advertisement Register (PHY_ANAR) */
      case INCA_IP_MDIO_REG5_PHY_ANLPAR: /* Auto-Negotiation Link Partner Ability Register (PHY_ANLPAR) */
      case INCA_IP_MDIO_REG6_PHY_ANER  : /* Auto-Negotiation Expansion Register (PHY_ANER) */
      case INCA_IP_MDIO_REG16_PHY_SRR  : /* Silicon Revision Register (PHY_SRR) */
      case INCA_IP_MDIO_REG17_PHY_MCSR : /* Mode Control/Status Register (PHY_MCSR) */
      case INCA_IP_MDIO_REG18_PHY_SMR  : /* Special Modes Register (PHY_SMR) */
      case INCA_IP_MDIO_REG29_PHY_ISR  : /* Interrupt Source Register (PHY_ISR) */
      case INCA_IP_MDIO_REG30_PHY_IMR  : /* Interrupt Mask Register (PHY_IMR) */
      case INCA_IP_MDIO_REG31_PHY_SCSR : /* PHY Special Control/Status Register (PHY_SCSR) */
         /* set MDIO_ACC read command */
         regValue = INCA_IP_Switch_MDIO_ACC_RA | 
                    INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
                    INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) ;

         regValue = sw21553_Write_MDIO_ACC(regValue) ;
         /* extract PHY register value */
         regValue = INCA_IP_Switch_MDIO_ACC_PHY_DATA_GET(regValue) ;

         return(regValue) ;
         break ;

      default:
         /* ERROR: invalid register index */
         return(ERROR) ;
   }
   return(ERROR) ;
}


/******************************************************************************
*  sw21553_PhyWriteReg()
*     read a MDIO PHY register
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyRegIndex: select the MDIO PHY register
*     newRegValue: new value to be set
*
*  Returns:
*     ERROR: in case of invalid MDIO register index
*     else
*     REGISTER VALUE
*
*  REMARK:
*
*/
UINT32 sw21553_PhyWriteReg(int portSelect, UINT32 phyRegIndex, UINT32 newRegValue)
{
   UINT32   regValue, reservedMask ;
   UINT32   phyAddr ;
   unsigned int flags = 0; 

   /* read PHY address */
   SW_READ_REG_UNLOCKED(INCA_IP_Switch_MDIO_CFG, regValue) ;  
   if (portSelect == 0)
   {
      phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_GET(regValue) ;
   }
   else
   {
      phyAddr = INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_GET(regValue) ;
   }

   /* set reseve bit mask or return with error */
   switch (phyRegIndex)
   {
      case INCA_IP_MDIO_REG0_PHY_BCR   : /* basic control register PHY_BCR */
         reservedMask = INCA_IP_PHY_REG0_BCR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG4_PHY_ANAR  : /* Auto-Negotiation Advertisement Register (PHY_ANAR) */
         reservedMask = INCA_IP_PHY_REG4_ANAR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG17_PHY_MCSR : /* Mode Control/Status Register (PHY_MCSR) */
         reservedMask = INCA_IP_PHY_REG17_MCSR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG18_PHY_SMR  : /* Special Modes Register (PHY_SMR) */
         reservedMask = INCA_IP_PHY_REG18_SMR_RESERVE_MASK ;
         break ;
      case INCA_IP_MDIO_REG30_PHY_IMR  : /* Interrupt Mask Register (PHY_IMR) */
         reservedMask = INCA_IP_PHY_REG_INT_RESERVE_MASK ;
         break ;

      /* invalid register index or register only read able */
      case INCA_IP_MDIO_REG1_PHY_BSR   : /* basic status register  PHY_BSR */
      case INCA_IP_MDIO_REG2_PHY_ID1   : /* PHY identifier 1 PHY_ID1 */
      case INCA_IP_MDIO_REG3_PHY_ID2   : /* PHY identifier 2 PHY_ID2 */
      case INCA_IP_MDIO_REG5_PHY_ANLPAR: /* Auto-Negotiation Link Partner Ability Register (PHY_ANLPAR) */
      case INCA_IP_MDIO_REG6_PHY_ANER  : /* Auto-Negotiation Expansion Register (PHY_ANER) */
      case INCA_IP_MDIO_REG16_PHY_SRR  : /* Silicon Revision Register (PHY_SRR) */
      case INCA_IP_MDIO_REG29_PHY_ISR  : /* Interrupt Source Register (PHY_ISR) */
      case INCA_IP_MDIO_REG31_PHY_SCSR : /* PHY Special Control/Status Register (PHY_SCSR) */
      default:
         /* ERROR: invalid register index or read only register */
         return(ERROR) ;
   }        /* switch (phyRegIndex) {...} */

   /* set MDIO_ACC read command */
   regValue = INCA_IP_Switch_MDIO_ACC_RA | 
              INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
              INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) ;
   save_and_cli(flags); 

   regValue = sw21553_Write_MDIO_ACC_UnLocked(regValue) ;

   /* set MDIO_ACC write command, keep reserved bits */
   regValue &= reservedMask ;
   regValue |= (INCA_IP_Switch_MDIO_ACC_RA | 
                INCA_IP_Switch_MDIO_ACC_RW |
                INCA_IP_Switch_MDIO_ACC_PHY_ADDR_SET(phyAddr) |
                INCA_IP_Switch_MDIO_ACC_REG_ADDR_SET(phyRegIndex) |
                INCA_IP_Switch_MDIO_ACC_PHY_DATA_SET(newRegValue)) ;

   regValue = sw21553_Write_MDIO_ACC_UnLocked(regValue) ;
   /* extract PHY register value */
   regValue = INCA_IP_Switch_MDIO_ACC_PHY_DATA_GET(regValue) ;
   restore_flags(flags);
   return(regValue) ;
}


/******************************************************************************
*  sw21553_PhySetCallback()
*     set callback function for a link-up/link-down indication 
*     to be called when link state is changed
*
*  Parameter:
*     portSelect : selects the MDIO PHY (LAN = 0 / PC = 1)
*     fct  : function to be called on state change
*
*  Returns:
*     -1 in case of error
*     old function ptr otherwise
*
*  REMARK:
*
*/
sw21553_PhyCallbackFunction sw21553_PhySetCallback(int portSelect, 
                                                   sw21553_PhyCallbackFunction fct)
{
	sw21553_PhyCallbackFunction old_fct = 0;

	if (fct == (void *)-1) 
		return (void *)-1;

	old_fct = devEthPhy[portSelect].fct;
	devEthPhy[portSelect].fct = fct;

	return old_fct;
}

/******************************************************************************
*  sw21553_PhySetLinkRetry()
*     set link retry parameters 
*
*  Parameter:
*     portSelect    : selects the MDIO PHY (LAN = 0 / PC = 1)
*     retryCount    : number of Link Retries
*     retryDelay_ms : delay between link retries
*
*  Returns:
*     OK
*
*  REMARK:
*
*/
UINT32 sw21553_PhySetLinkRetry(int portSelect, int retryCount, int retryDelay_ms)
{
	/* parameter check */
#if 0
	printk("sw21553_PhySetLinkRetry(): sysClkRateGet() has to be replaced by app. linux function !\n");
#endif
	
	if ( (retryCount < 0) ||
		  (retryDelay_ms <= 0) ||
		  (retryDelay_ms * sysClkRateGet() % 1000 != 0) )
		return ERROR;

	devEthPhy[portSelect].LinkRetriesReload = retryCount;
	devEthPhy[portSelect].RetryTicks  = retryDelay_ms * sysClkRateGet()/1000;
   PHY_LOG3 (PHY_DEBUG_INIT, "PhySetLinkRetry: Port%d LinkRetries = %d, RetryDelay = %dms !\n",
   					portSelect, retryCount, retryDelay_ms);

	return OK;
}

/******************************************************************************
*  sw21553_PhyGetProperties()
*     returns the selected PHY properties
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     phyInfo: specify the properties for return
*             0: current properties
*             1: advertised PHY properties for auto-neg.
*             2: partner PHY properties

*
*  Returns:
*     ERROR: in case of invalid selection
*     else
*     PHY properties
*
*  REMARK:
*
*/
UINT32 sw21553_PhyGetProperties(int portSelect, int phyInfo)
{
   UINT32   reg1Value, reg0Value, regAutoNegValue, reg6Value, reg31Value ;
   UINT32   returnState = 0 ;

   /* read current PHY properties form Basic Control Register (PHY_BCR) */
   reg0Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR) ;
   /* read current PHY properties form Basic Status Register */
   reg1Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG1_PHY_BSR) ;

   reg31Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG31_PHY_SCSR) ;
   returnState  = sw21553_PHY_CTRL_CURR_SPEED_SET(
                           INCA_IP_PHY_REG31_SCSR_SPEED_GET(reg31Value)
                                                 ) ;
   returnState |= (reg31Value & INCA_IP_PHY_REG31_SCSR_AUTODONE_MASK) ? 
                                      sw21553_PHY_CTRL_AUTO_NEG_DONE  : 0 ;
   switch(phyInfo)
   {
      case sw21553_PHY_PROPS_CURRENT:
         /* read speed and duplex settings */
         returnState |= INCA_IP_PHY_CTRL_LINE_PROPS_GET_BSR(reg1Value) ;
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;
         /* read Auto Neg. Ability, check if Auto Neg. Enable */
         if ((reg0Value & INCA_IP_PHY_REG0_BCR_ANE_MASK) &&
             (reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK))
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         /* check validity of the data */
         if ( (returnState & sw21553_PHY_PROPS_CURRENT_VALID_MASK) != 
                                                sw21553_PHY_PROPS_CURRENT_VALID_STATE )
         {
            /* valid mask does not match --> set invalid bit */
            returnState |= sw21553_PHY_CTRL_DATA_CORRUPTED ;
         }
         return(returnState) ;
         break ;

      case sw21553_PHY_PROPS_ADVERTISE:
         /* read current PHY properties form 
                        Auto Negotiation Advertisement Register (PHY_ANAR) */
         regAutoNegValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR) ;

         /* read speed and duplex settings */
         returnState |= (INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANAR(regAutoNegValue)) ;
         /* read Next Page */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG4_ANAR_NP_MASK) ? 
                                                 sw21553_PHY_CTRL_NEXT_PAGE : 0 ) ;
         /* read Pause Operation */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG4_ANAR_PO_MASK) ? 
                                                     sw21553_PHY_CTRL_PAUSE : 0 ) ;

         /* check if Auto Neg. Enable */
         if(reg0Value & INCA_IP_PHY_REG0_BCR_ANE_MASK)
         {
            /* Auto Neg. is enabled */
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_SUPPORT ;
         }

         /*--- read common link status data from PHYReg 1 ---*/
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;
         /* read Auto Neg. Ability */
         if(reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK)
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         return (returnState) ;
         break ;

      case sw21553_PHY_PROPS_PARTNER:
         /*
            # informations about the link partner are only valid if 
              --> link partner is able to perform auto negotiation
              --> auto neg. complete
         */
         /* read current PHY properties form 
                        Auto Negotiation Expansion Register (PHY_ANER) */
         reg6Value = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG6_PHY_ANER) ;
         /* read current PHY properties form 
                        Auto Negotiation Link Partner Ability Register (PHY_ANLPAR) */
         regAutoNegValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG5_PHY_ANLPAR) ;

         /* read current PHY properties form 
                     Auto Negotiation Link Partner Ability Register (PHY_ANLPAR) */
         returnState |= (INCA_IP_PHY_CTRL_LINE_PROPS_GET_ANLPAR(regAutoNegValue)) ;

         /* check if the link partner is able to do auto-negotiation */
         returnState |= ( (reg6Value & INCA_IP_PHY_REG6_ANER_LPANA_MASK) ? 
                                       sw21553_PHY_CTRL_AUTO_NEG_SUPPORT : 0 ) ;
         /* read Next Page */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG5_ANLPAR_NP_MASK) ? 
                                                   sw21553_PHY_CTRL_NEXT_PAGE : 0 ) ;
         /* read Pause Operation */
         returnState |= ( (regAutoNegValue & INCA_IP_PHY_REG5_ANLPAR_PO_MASK) ? 
                                                 sw21553_PHY_CTRL_PAUSE : 0 ) ;

         /*--- read common link status data from PHYReg 1 ---*/
         /* read Auto Neg. Ability */
         if(reg1Value & INCA_IP_PHY_REG1_BSR_ANA_MASK)
         {
            returnState |= sw21553_PHY_CTRL_AUTO_NEG_POSSIBLE ;
            /* read Auto Neg. complete */
            returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_ANC_MASK) ? 
                                         sw21553_PHY_CTRL_AUTO_NEG_COMPL : 0 ) ;
         }
         /* read link status */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_LS_MASK) ? 
                                        sw21553_PHY_CTRL_LINK_STATE : 0 ) ;
         /* read remote fault */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_RF_MASK) ? 
                                      sw21553_PHY_CTRL_REMOTE_FAULT : 0 ) ;
         /* read Extended Capabilities */
         returnState |= ( (reg1Value & INCA_IP_PHY_REG1_BSR_EXC_MASK) ? 
                                sw21553_PHY_CTRL_EXT_CAP : 0 ) ;

         /* check validity of the data */
         if ( (returnState & sw21553_PHY_PROPS_PARTNER_VALID_MASK) != 
                                                sw21553_PHY_PROPS_PARTNER_VALID_STATE )
         {
            /* valid mask does not match --> set invalid bit */
            returnState |= sw21553_PHY_CTRL_DATA_CORRUPTED ;
         }
         return (returnState) ;
         break ;

      default:
         /* no valid parameter */
         return(ERROR) ;

   }        /* switch(phyInfo) {...} */

   return(ERROR) ;
}

/******************************************************************************
*  sw21553_PhySetAdvertiseProperties()
*     set the advertise properties for auto negotiation (PHY reg 4)
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     newSettings: new properies to set
*                  --> line properties: bit[0..4]
*                       bit 0:  10 MBit/s Half Duplex
*                       bit 1:  10 MBit/s Full Duplex
*                       bit 2: 100 MBit/s Half Duplex
*                       bit 3: 100 MBit/s Full Duplex
*                      (bit 4: 100 MBit/s T4 - can not set )
*                      (bit 5..7: not used)
*                  --> support Pause Operation
*                       bit 8: Support Pause Operation
*
*  Returns:
*     ERROR: in case of invalid selection
*     else
*     PHY advertiese properties (PHY reg 4)
*
*  Example: 
*     sw21553_PhySetAdvertiseProperties( 0, 0x003)
*        --> select LAN PHY port
*        --> new settings: 0x003
*            - set "not support Pause"
*            - advertise: 10MBit/s Full Duplex | 10 MBit/s Half Duplex
*
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*
*/
UINT32 sw21553_PhySetAdvertiseProperties(int portSelect, UINT32 newSettings)
{
   UINT32   regValue ;

   /* clear all not writeable bits */
   newSettings &= sw21553_PHY_PROPS_ADVERTISE_SET_MASK ;

   /* read current settings */
   regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR) ;

   /* clear line properties and set the new settings */
   regValue &= ~(INCA_IP_MDIO_REG4_PROPERTY_MASK) ;
   regValue |= INCA_IP_PHY_CTRL_LINE_PROPS_SET_ANAR(newSettings) ;
   if(newSettings & sw21553_PHY_CTRL_PAUSE)
      regValue |= INCA_IP_PHY_REG4_ANAR_PO_MASK ;

   /* write new register settings */
   regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG4_PHY_ANAR, regValue) ;

   return(regValue) ;
}


/******************************************************************************
*  sw21553_PhySetFixSpeed()
*     set the fixed speed and duplex settings used without auto-neg
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     newSpeed: new speed and duplex properies to set
*                  - 001B: 10Base-T half-duplex
*                  - 101B: 10Base-T full-duplex
*                  - 010B: 100Base-TX half-duplex
*                  - 110B: 100Base-TX full-duplex
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:    
*
*  Example: 
*
*  REMARK:
*     The given settings will be set, but they are only become valid after
*     restart of the auto negotiation of the appropriate PHY.
*
*/
UINT32 sw21553_PhySetFixSpeed(int portSelect, UINT32 newSpeed)
{
   UINT32   regValue ;

   if ( (regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR)) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

	switch (newSpeed & 0x3) {
	case sw21553_PHY_SPEED_100_BIT:
      /* set PHY Reg0:Bit13 - set 100 MBit/S */
      regValue |= INCA_IP_PHY_REG0_BCR_SS_MASK ;
		break;
   case sw21553_PHY_SPEED_10_BIT:
      /* clear PHY Reg0:Bit13 - set 10 MBit/s */
      regValue &= ~(INCA_IP_PHY_REG0_BCR_SS_MASK) ;
		break;
   default:
      PHY_LOG (PHY_DEBUG_INIT, "Fix Speed: invalid setting !\n");
		return ERROR;
	}
   
   if (newSpeed & sw21553_PHY_SPEED_DUPLEX_BIT)
      /* set PHY Reg0:Bit8 - set full duplex */
      regValue |= INCA_IP_PHY_REG0_BCR_DM_MASK ;
   else
      /* clear PHY Reg0:Bit8 - set half duplex */
      regValue &= ~(INCA_IP_PHY_REG0_BCR_DM_MASK) ;

	/* disable corresponding switch port */
	if (portSelect == INCA_IP_PHY0_LAN)
		sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LL);
	else  /* INCA_IP_PHY1_PC */
	   sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LP);

	/* emit LINK_INACTIVE event. 
	   Note: if link was up and speed doesn't change 
	         -> PHY Link stays active, EPHY gets reactivated after WDOG timeout */

			 
/*	de6wmigr: to be replaced by tasklet_schedule() function call */
	
    phyFsmAction[devEthPhy[portSelect].State][LINK_INACTIVE] ( &devEthPhy[portSelect] );

/*													
	 netJobAdd((FUNCPTR) phyFsmAction[devEthPhy[portSelect].State][LINK_INACTIVE]
   													,(int)&devEthPhy[portSelect],2,3,4,5);
*/
	devEthPhy[portSelect].FixSpeed = newSpeed;

	/* Update the PHY register with new speed setting */
   if ( sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) == ERROR )
      /* wrong port select */
      return(ERROR) ;
	else
   	PHY_LOG1 (PHY_DEBUG_INIT, "Fix Speed: Wrote %04X into register 0 (BCR)\n", 
   			(UINT16) regValue);

   return(OK) ;
}

/******************************************************************************
*  sw21553_PhyDisEnAutoNeg()
*     disable / enable auto negotiation
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*     disableAutoNeg: enable / disable auto negotiation (PHY reg 0)
*                    1: disable auto neg.
*                    0: enable auto neg.
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:    
*
*
*/
UINT32 sw21553_PhyDisEnAutoNeg(int portSelect, int disableAutoNeg)
{
   UINT32   regValue ;

   if ( (regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR)) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

   if (disableAutoNeg == 1) {
      /* clear PHY Reg0:Bit12 - disable auto negotiation */
      regValue &= ~(INCA_IP_PHY_REG0_BCR_ANE_MASK) ;
		devEthPhy[portSelect].AutoNeg = FALSE;
	}
   else {
      /* set PHY Reg0:Bit12 - enable auto negotiation */
      regValue |= INCA_IP_PHY_REG0_BCR_ANE_MASK ;
		devEthPhy[portSelect].AutoNeg = TRUE;
	}

   if ( sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) == ERROR )
      /* wrong port select */
      return(ERROR) ;

   return(OK) ;
}

/******************************************************************************
*  sw21553_PhyRestartAutoNeg()
*     restart the auto negotiation for the selected PHY
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*
*  Returns:
*     ERROR: in case of invalid selection or restart not possible
*     OK:
*
*  REMARK:
*
*/
int sw21553_PhyRestartAutoNeg(int portSelect)
{
   UINT32 regValue ;

   /* read current register values */
   regValue = sw21553_PhyReadReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR) ;

   /* clear reset bit */
   regValue &= ~(INCA_IP_PHY_REG0_BCR_RESET_MASK) ;

   /* check if auto neg. is enabled and update restart auto neg. bit */
   if ( regValue & INCA_IP_PHY_REG0_BCR_ANE_MASK)
   {
      /* auto negotiation is enabled --> do restart */
      regValue |= INCA_IP_PHY_REG0_BCR_RAN_MASK ;
      regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) ;
      if (regValue != ERROR)
         return(OK) ;
   }
   else
   {
      return(ERROR) ;
   }

   return(ERROR) ;
}


/******************************************************************************
*  sw21553_PhyReset()
*     reset the selected PHY
*
*  Parameter:
*     portSelect: selects the MDIO PHY (LAN = 0 / PC = 1)
*
*  Returns:
*     ERROR: in case of invalid selection
*     OK:
*
*  REMARK:
*		This command also resets registers -> all interrupts get disabled
*/
int sw21553_PhyReset(int portSelect)
{
   UINT32 regValue ;

   /* set reset bit */
   regValue = INCA_IP_PHY_REG0_BCR_RESET_MASK ;

   regValue = sw21553_PhyWriteReg(portSelect, INCA_IP_MDIO_REG0_PHY_BCR, regValue) ;
   if (regValue != ERROR)
      return(OK) ;

   return(ERROR) ;
}



#undef PHYISR_NETTASK
/*******************************************************************************
*
* sw21553_PhyLan_Irq - Interrupt Handler for the LAN
*
* Interrupt of the LAN PHY
*
*/
void sw21553_PhyLAN_Irq( int irq, void *dev_id, struct pt_regs *regs )
{
   PHY_LOG (PHY_DEBUG_INT, "Got a LAN PHY interrupt!\n");
#ifdef PHYISR_NETTASK
  	netJobAdd ((FUNCPTR) sw21553_PhyIsr, (int) &devEthPhy[INCA_IP_PHY0_LAN], 0,0,0,0);
  	PHY_LOG (PHY_DEBUG_INT, "LAN PHY: netJob added!\n");
#else
   sw21553_PhyIsr(&devEthPhy[INCA_IP_PHY0_LAN]);
   SW_INTERRUPTS_CLEAR(INCA_IRQ_LINE_PHY0LAN);
#endif
}

/*******************************************************************************
*
* sw21553_PhyPc_Irq - Interrupt Handler for the PC
*
* Interrupt of the PC PHY
*
*/
void sw21553_PhyPC_Irq( int irq, void *dev_id, struct pt_regs *regs)
{
   PHY_LOG (PHY_DEBUG_INT, "Got a PC PHY interrupt!\n");

#ifdef PHYISR_NETTASK
  	netJobAdd ((FUNCPTR) sw21553_PhyIsr, (int) &devEthPhy[INCA_IP_PHY1_PC], 0,0,0,0);
   PHY_LOG (PHY_DEBUG_INT, "PC PHY: netJob added!\n");
#else
   sw21553_PhyIsr(&devEthPhy[INCA_IP_PHY1_PC]);
   SW_INTERRUPTS_CLEAR(INCA_IRQ_LINE_PHY1PC);
#endif
}

/*******************************************************************************
*
* sw21553_PhyIsr - Interrupt Service Routine for the both PC and LAN PHY
*
*
*/
void sw21553_PhyIsr(swPhyStruct *pPhy)
{
	volatile UINT32 phyIsr, phyReg1, phyReg6, phyReg17;
#ifdef PHYISR_NETTASK
	int lockKey;
#endif

	/* read and clear ISR */
	phyIsr = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG29_PHY_ISR);
	PHY_LOG2(PHY_DEBUG_INT, "PHYISR(%d): REG29_PHY_ISR = %04X\n", pPhy->Port, phyIsr);

	/* clear Link-Down status by reading BSR register ***/
	phyReg1 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG1_PHY_BSR);
	PHY_LOG2(PHY_DEBUG_INT, "PHYISR(%d): REG1_PHY_BSR = %04X\n", pPhy->Port, phyReg1);

	/* get Energy status by reading MCSR register */
	phyReg17 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG17_PHY_MCSR);

	/* clear Parallel detection fault by reading ANER register */
	phyReg6 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG6_PHY_ANER);

	/*
	 * Note: the PHY ISR keeps its status, even after clear-on-read!
	 * Thus it does not really show which event triggered the interrupt.
	 */

	/* if energy detect power-down is enabled, only energy status bit is valid! */
	if (phyReg17 & INCA_IP_PHY_REG17_MCSR_EDPD_MASK) 
	{
		/*** Energy On ***/
		if (phyReg17 & INCA_IP_PHY_REG17_MCSR_ENRG_MASK) 
		{
	      PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): ENERGY_DETECT event!\n", pPhy->Port);
/*       	netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][ENERGY_DETECT],(int)pPhy,2,3,4,5);  */
          phyFsmAction[pPhy->State][ENERGY_DETECT] (pPhy);
		}
		else
	      PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): No Signal detected!\n", pPhy->Port);
	}
	else
	{
	    /*** Autonegotiation/Parallel detection complete ***/
   	    if (phyIsr & INCA_IP_PHY_REG_INT_ANCOMPL_MASK) 
		{
	        PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): LINK_ACTIVE event!\n", pPhy->Port);
	        /*	netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][LINK_ACTIVE],(int)pPhy,2,3,4,5);  */
            phyFsmAction[pPhy->State][LINK_ACTIVE] ( pPhy );		 
		}

	   /*** Link Down ***/
   	   if (phyIsr & INCA_IP_PHY_REG_INT_LDOWN_MASK) 
       {
           PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): LINK_INACTIVE event!\n", pPhy->Port);
           /* immediately update EPHY register */
           if (pPhy->Port == INCA_IP_PHY0_LAN)
	   	       sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LL);
           else  /* INCA_IP_PHY1_PC */
	           sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LP);

        /*      	netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][LINK_INACTIVE],(int)pPhy,2,3,4,5);  */
           phyFsmAction[pPhy->State][LINK_INACTIVE] ( pPhy );		
	    }

	   /*** Parallel Detection Fault ***/
	   if (phyIsr & INCA_IP_PHY_REG_INT_PDFAULT_MASK) 
	   {
	       PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): PARDET_FAULT event!\n", pPhy->Port);
       	   /* netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][PARDET_FAULT],(int)pPhy,2,3,4,5);  */
		   phyFsmAction[pPhy->State][PARDET_FAULT] ( pPhy );
	   }

	   /*** Remote Fault Detected ***/
	   if (phyIsr & INCA_IP_PHY_REG_INT_RFDETECT_MASK) 
	   {
           PHY_LOG1(PHY_DEBUG_INT, "PHYISR(%d): REMOTE_FAULT event!\n", pPhy->Port);
           /* netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][REMOTE_FAULT],(int)pPhy,2,3,4,5); */
		   phyFsmAction[pPhy->State][REMOTE_FAULT] ( pPhy );
	   }
	}

#ifdef PHYISR_NETTASK
	save_and_cli( flags );
	if (pPhy->Port == INCA_IP_PHY0_LAN) {
	   SW_INTERRUPTS_ENABLE(INCA_IRQ_LINE_PHY0LAN);
   	SW_INTERRUPTS_CLEAR(INCA_IRQ_LINE_PHY0LAN);
	}
	else {
	   SW_INTERRUPTS_ENABLE(INCA_IRQ_LINE_PHY1PC);
   	SW_INTERRUPTS_CLEAR(INCA_IRQ_LINE_PHY1PC);
	}
	restore_flags( flags );
#endif
}

/*******************************************************************************
*
* sw21553_PhyWdIsr - Watchdog Interrupt Service Routine for the both 
*                    PC and LAN PHY. The watchdog timer is used to poll 
*                    for LinkUp event
*
*
*/
static void sw21553_PhyWdIsr(swPhyStruct *pPhy) 
{
    volatile UINT32 phyReg1;

    phyReg1 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG1_PHY_BSR);
	
    if (phyReg1 & INCA_IP_PHY_REG1_BSR_LS_MASK) 
    {
        PHY_LOG1(PHY_DEBUG_INT, "WDOG(%d): LINK_ACTIVE event!\n", pPhy->Port);
        /* netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][LINK_ACTIVE],(int)pPhy,2,3,4,5);  */
        phyFsmAction[pPhy->State][LINK_ACTIVE] ( pPhy );
    }
    else if (pPhy->LinkRetries > 0) 
    {
        PHY_LOG1(PHY_DEBUG_INT, "WDOG(%d): LINK_TRY_FAIL event!\n", pPhy->Port);
        /* netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][LINK_TRY_FAIL],(int)pPhy,2,3,4,5);  */
        phyFsmAction[pPhy->State][LINK_TRY_FAIL] ( pPhy );
    }
    else 
    {
        PHY_LOG1(PHY_DEBUG_INT, "WDOG(%d): LINK_TRIES_TOUT event!\n", pPhy->Port);
        /* netJobAdd((FUNCPTR) phyFsmAction[pPhy->State][LINK_TRIES_TOUT],(int)pPhy,2,3,4,5); */
        phyFsmAction[pPhy->State][LINK_TRIES_TOUT] ( pPhy );        
	}
}

/****************************************************************************
 *                     PHY Finite State Machine                             *
 ****************************************************************************/

static void fsmNoAction(swPhyStruct *pPhy ) 
{
   PHY_LOG1 (PHY_DEBUG_FSM, "FSM(%d): NoAction!\n", pPhy->Port);
}

static void fsmServeEnergy(swPhyStruct *pPhy )
{
   PHY_LOG1 (PHY_DEBUG_FSM, "FSM(%d): Energy (EDPD=0)!\n", pPhy->Port);
   sw21553_PhyWriteReg(pPhy->Port, INCA_IP_MDIO_REG17_PHY_MCSR, 0);

	pPhy->LinkRetries = pPhy->LinkRetriesReload;
	PHY_LOG1(PHY_DEBUG_FSM, "WDOG(%d): Start!\n", pPhy->Port);
	wdStart( (struct timer_list*) wdId[pPhy->Port], 
             pPhy->RetryTicks, 
             (void*)sw21553_PhyWdIsr, 
             (unsigned long int) pPhy);

	if (pPhy->fct) pPhy->fct(ENERGY_DETECT);

	pPhy->State = AWAIT_LINK;	
}

static void fsmServeLinkUp(swPhyStruct *pPhy )
{
    volatile UINT32 regEphy, phyReg1, phyReg4, phyReg5, phyReg6, phyReg31;
	UINT32 speed;
    char *linkStatus;

    PHY_LOG1 (PHY_DEBUG_FSM, "FSM(%d): LinkUp!\n", pPhy->Port);

    wdCancel( (struct timer_list*) wdId[pPhy->Port]);
    PHY_LOG1 (PHY_DEBUG_FSM, "WDOG(%d): Cancel!\n", pPhy->Port);

    phyReg1 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG1_PHY_BSR);
	
    /*** Fixed Speed ***********************************************/
    if (FALSE == pPhy->AutoNeg) 
    {
        speed = pPhy->FixSpeed;
    }
    /*** Autoneg/Parallel Detect ***********************************/
    else if (phyReg1 & INCA_IP_PHY_REG1_BSR_ANC_MASK) 
    {
        /* link partners agreed on settings -> tell settings to Ethernet MAC */
        phyReg31 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG31_PHY_SCSR);

        speed = INCA_IP_PHY_REG31_SCSR_SPEED_GET(phyReg31);
	}
	else 
        return; /* No change */
	   
    switch (speed) 
    {
        case sw21553_PHY_SPEED_IND_10_HALF: /* 10Base-T half-duplex */
            regEphy = 0;
            linkStatus = "10Base-T half-duplex";
            break;
        case sw21553_PHY_SPEED_IND_10_FULL: /* 10Base-T full-duplex */
            regEphy = INCA_IP_Switch_EPHY_DL_FULL | INCA_IP_Switch_EPHY_DP_FULL;
			linkStatus = "10Base-T full-duplex";
	    	break;
	case sw21553_PHY_SPEED_IND_100_HALF: /* 100Base-TX half-duplex */
	    	regEphy = INCA_IP_Switch_EPHY_SL_100MBIT | INCA_IP_Switch_EPHY_SP_100MBIT;
			linkStatus = "100Base-TX half-duplex";
	    	break;
	case sw21553_PHY_SPEED_IND_100_FULL: /* 100Base-TX full-duplex */
	    	regEphy = INCA_IP_Switch_EPHY_SL_100MBIT | INCA_IP_Switch_EPHY_SP_100MBIT |
	                INCA_IP_Switch_EPHY_DL_FULL | INCA_IP_Switch_EPHY_DP_FULL;
			linkStatus = "100Base-TX full-duplex";
	    	break;
	default:
			linkStatus = "invalid speed setting!";
	      PHY_DEBUG (PHY_DEBUG_FSM, ("FSM(%d): speed = %d is invalid !\n", pPhy->Port, speed));
			break;
	}
	/* in case of Autoneg, update the negotiated PAUSE support status */
	if ((phyReg1 & INCA_IP_PHY_REG1_BSR_ANA_MASK) && (pPhy->AutoNeg)) 
    {
	 	/* we are Autoneg-able. Is Link partner also able to autoneg? */
		phyReg6 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG6_PHY_ANER);
	 	if (phyReg6 & INCA_IP_PHY_REG6_ANER_LPANA_MASK) 
        { /* Link partner can also autoneg */
	   	  /* evaluate negotiated PAUSE support */
	      phyReg4 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG4_PHY_ANAR);
	   	  if (phyReg4 & INCA_IP_PHY_REG4_ANAR_PO_MASK) 
          { 
	   		 /* we advertise PAUSE capab. */
	         phyReg5 = sw21553_PhyReadReg(pPhy->Port, INCA_IP_MDIO_REG5_PHY_ANLPAR);
	      	 if (phyReg5 & INCA_IP_PHY_REG5_ANLPAR_PO_MASK) 
             { 
	      	    /* Link partner is PAUSE capab. */
	         	regEphy |= INCA_IP_Switch_EPHY_PL_PAUSE_SUP | INCA_IP_Switch_EPHY_PP_PAUSE_SUP;
	         	PHY_LOG2(PHY_DEBUG_FSM, "FSM(%d): Auto-Negotiation: %s, with PAUSE\n", pPhy->Port, (int) linkStatus);
                printk("Port %d: Auto-Negotiation: %s, with PAUSE\n", pPhy->Port, (int) linkStatus);
	      	 }
		  	 else 
             {
	         	PHY_LOG2(PHY_DEBUG_FSM, "FSM(%d): Auto-Negotiation: %s, no PAUSE\n", pPhy->Port, (int) linkStatus);
                printk("Port %d: Auto-Negotiation: %s, no PAUSE\n", pPhy->Port, (int) linkStatus);
		  	 }
	   	}
	   	else 
        {
	      	PHY_LOG2(PHY_DEBUG_FSM, "FSM(%d): Auto-Negotiation: %s, no PAUSE\n", pPhy->Port, (int) linkStatus);
            printk("Port %d: Auto-Negotiation: %s, no PAUSE\n", pPhy->Port, (int) linkStatus);
	   	}
		}
		else 
        {
	   	    PHY_LOG2(PHY_DEBUG_FSM, "FSM(%d): Parallel Detection: %s\n", pPhy->Port, (int) linkStatus);
            printk("Port %d: Parallel Detection: %s\n", pPhy->Port, (int) linkStatus);
		}
	}
	else 
    {
		PHY_LOG2(PHY_DEBUG_FSM, "FSM(%d): Fixed Setting: %s\n", pPhy->Port, (int) linkStatus);
        printk("Port %d: Fixed Setting: %s\n", pPhy->Port, (int) linkStatus);
	}

	/* Link is not down (-> it's up) */
	regEphy |= INCA_IP_Switch_EPHY_LL_LINK_UP | INCA_IP_Switch_EPHY_LP_LINK_UP;

	if (pPhy->Port == INCA_IP_PHY0_LAN) 
    {
	   regEphy = sw21553_SwSetEPhy(regEphy, 
	                              INCA_IP_Switch_EPHY_DL |
	                              INCA_IP_Switch_EPHY_SL |
	                              INCA_IP_Switch_EPHY_PL |
	                              INCA_IP_Switch_EPHY_LL);
	}
	else 
    {  /* INCA_IP_PHY1_PC */
	   regEphy = sw21553_SwSetEPhy(regEphy, 
	                              INCA_IP_Switch_EPHY_DP |
	                              INCA_IP_Switch_EPHY_SP |
	                              INCA_IP_Switch_EPHY_PP |
	                              INCA_IP_Switch_EPHY_LP);
	}

	if (pPhy->fct) pPhy->fct(LINK_ACTIVE);

	pPhy->State = LINK_UP;	
}

static void fsmServeLinkDown(swPhyStruct *pPhy )
{
   volatile UINT32 regEphy;

   PHY_LOG1(PHY_DEBUG_FSM, "FSM%d: LinkDown!\n", pPhy->Port);
	/* disable port */
	if (pPhy->Port == INCA_IP_PHY0_LAN)
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LL);
	else  /* INCA_IP_PHY1_PC */
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LP);

	if (pPhy->fct) 
        pPhy->fct(LINK_INACTIVE);

	pPhy->LinkRetries = pPhy->LinkRetriesReload;
	PHY_LOG1(PHY_DEBUG_FSM, "WDOG(%d): Start!\n", pPhy->Port);
	wdStart( (struct timer_list*) wdId[pPhy->Port],
             pPhy->RetryTicks, 
             (void*)sw21553_PhyWdIsr, 
             (unsigned long int) pPhy);

	pPhy->State = AWAIT_LINK;	
}

static void fsmServeLinkRetry(swPhyStruct *pPhy )
{
   volatile UINT32 regEphy;

   PHY_LOG1 (PHY_DEBUG_FSM, "FSM(%d): LinkRetry!\n", pPhy->Port);
	/* disable port */
	if (pPhy->Port == INCA_IP_PHY0_LAN)
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LL);
	else  /* INCA_IP_PHY1_PC */
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LP);

	if (pPhy->fct) pPhy->fct(LINK_TRY_FAIL);

	if (pPhy->LinkRetries > 0 ) 
    {
		pPhy->LinkRetries--;
		PHY_LOG1(PHY_DEBUG_FSM, "WDOG(%d): Start!\n", pPhy->Port);
		wdStart( (struct timer_list*) wdId[pPhy->Port], 
                 pPhy->RetryTicks, 
                 (void*) sw21553_PhyWdIsr, 
                 (unsigned long int) pPhy);
	}

	pPhy->State = AWAIT_LINK;	
}

static void fsmServeLinkTOut(swPhyStruct *pPhy )
{
   volatile UINT32 regEphy;

   PHY_LOG1 (PHY_DEBUG_FSM, "FSM(%d): LinkTOut (EDPD=1)!\n", pPhy->Port);
	/* disable port */
	if (pPhy->Port == INCA_IP_PHY0_LAN)
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LL);
	else  /* INCA_IP_PHY1_PC */
	   regEphy = sw21553_SwSetEPhy(0, INCA_IP_Switch_EPHY_LP);

	sw21553_PhyWriteReg(pPhy->Port, INCA_IP_MDIO_REG17_PHY_MCSR, INCA_IP_PHY_REG17_MCSR_EDPD_MASK);

	if (pPhy->fct) pPhy->fct(LINK_TRIES_TOUT);

	pPhy->State = AWAIT_ENERGY;	

}

/*******************************************************************************
*
* sw21553_PhyInit - initialize data structures and interrupt vectors for the PHYs
*
* This function is called by the switch END Load function
*
* RETURNS: OK or ERROR.
*
*/
int sw21553_PhyInit()
{
   int i, j, port;
    
   PHY_DEBUG (PHY_DEBUG_INIT, ("sw21553_PhyInit() called!\n"));

   SW_INTERRUPTS_DISABLE(INCA_IRQ_LINE_PHY0LAN);
   SW_INTERRUPTS_DISABLE(INCA_IRQ_LINE_PHY1PC);

   devEthPhy[INCA_IP_PHY0_LAN].PhyAddr = INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR;
   devEthPhy[INCA_IP_PHY1_PC ].PhyAddr = INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR;

	for (port = INCA_IP_PHY0_LAN; port <= INCA_IP_PHY1_PC; port++)
	{
		devEthPhy[port].Port        = port;
		devEthPhy[port].AutoNeg     = TRUE;
		devEthPhy[port].fct         = NULL;
		devEthPhy[port].State       = AWAIT_LINK;
		devEthPhy[port].LinkRetriesReload = 0x7FFFFFFF;
		devEthPhy[port].RetryTicks  = LINK_RETRY_TICKS;

		/* Create Watchdogs */
		wdId[port] = (WDOG_ID) wdCreate();

#ifdef PHY_DEBUG_ENABLE
		sw21553_PhySetCallback(port,sw21553_PhyCallbackFunctionSample);
#endif
	} /* for */

	/* Initialize PHY State Machine */
   for (i = 0; i < PHY_STATE_MAX; i++)       /* basic initialization to do */
      for (j = 0; j < PHY_EVENT_MAX; j++)    /* nothing for all possible   */
      {		  
         phyFsmAction[i][j] = fsmNoAction;   /* state-event combinations   */
/*         tasklet[i][j] ='\0';  */		 
      }
         
	phyFsmAction[AWAIT_ENERGY][ENERGY_DETECT  ] = fsmServeEnergy;
	phyFsmAction[AWAIT_LINK  ][LINK_ACTIVE    ] = fsmServeLinkUp;
	phyFsmAction[AWAIT_LINK  ][LINK_TRY_FAIL  ] = fsmServeLinkRetry;
	phyFsmAction[AWAIT_LINK  ][LINK_TRIES_TOUT] = fsmServeLinkTOut;
	phyFsmAction[LINK_UP     ][LINK_INACTIVE  ] = fsmServeLinkDown;
#if 0
	tasklet[AWAIT_ENERGY][ENERGY_DETECT  ] = 'ServeEnergyTasklet';
	tasklet[AWAIT_LINK  ][LINK_ACTIVE    ] = 'ServeLinkUpTasklet';
	tasklet[AWAIT_LINK  ][LINK_TRY_FAIL  ] = 'ServeLinkRetryTasklet';
	tasklet[AWAIT_LINK  ][LINK_TRIES_TOUT] = 'ServeLinkTOutTasklet';
	tasklet[LINK_UP     ][LINK_INACTIVE  ] = 'ServeLinkDownTasklet';	

    DECLARE_TASKLET( tasklet[AWAIT_ENERGY][ENERGY_DETECT  ], 
	                 phyFsmAction[AWAIT_ENERGY][ENERGY_DETECT  ],
					 (unsigned long) );	
    DECLARE_TASKLET( tasklet[AWAIT_LINK  ][LINK_ACTIVE    ] );	
    DECLARE_TASKLET( tasklet[AWAIT_LINK  ][LINK_TRY_FAIL  ] );		
    DECLARE_TASKLET( tasklet[AWAIT_LINK  ][LINK_TRIES_TOUT] );		
    DECLARE_TASKLET( tasklet[LINK_UP     ][LINK_INACTIVE  ] );	
#endif	
   /* connect PHY0 (LAN) interrupt line */
   if (request_irq(INCA_IP_IM_IRL_LAN, sw21553_PhyLAN_Irq, SA_INTERRUPT, "lan_phy", NULL))
   {
      PHY_DEBUG (PHY_DEBUG_INIT, ("sw2153_PHY0_IRQ: request_irq failed\n"));
      return ERROR;
   }
   /* connect PHY1 (PC) interrupt line */  
   if (request_irq(INCA_IP_IM_IRL_PC, sw21553_PhyPC_Irq, SA_INTERRUPT, "pc_phy", NULL))
   {
      PHY_DEBUG (PHY_DEBUG_INIT, ("sw2153_PHY1_IRQ: request_irq failed\n"));
      return ERROR;
   }
      
   return OK ;

}

/*******************************************************************************
*
* sw21553_PhyStart - disable MDIO hardware state machine, enable software 
*                    state machine
*
* This function gets added to the NetJob queue
*
* RETURNS: OK or ERROR.
*
*/
int sw21553_PhyStart()
{
   volatile UINT32 phyReg;
	int port;
    
   PHY_DEBUG (PHY_DEBUG_INIT, ("sw21553_PhyStart() called!\n"));

#if 0//def PHY_DEBUG_ENABLE
	printf("PHY registers before switching to SW state machine:\n");
	sysPhyShowLanRegs();
	sysPhyShowPcRegs();
#endif

   /* Disable MDIO State Machine (no automatic update of EPHY register) */
   sw21553_SwSetMdioConfig( INCA_IP_Switch_MDIO_CFG_PHY_LAN_ADDR_DEFAULT |
                            INCA_IP_Switch_MDIO_CFG_PHY_PC_ADDR_DEFAULT  |
                            INCA_IP_Switch_MDIO_CFG_UMM_DISABLED, 0xFFFFFFFF ) ;

	/* Disable Switch Ports (clear EPHY register) */
	sw21553_SwSetEPhy(0, 0xFFFFFFFF);

	for (port = INCA_IP_PHY0_LAN; port <= INCA_IP_PHY1_PC; port++)
	{
		if (devEthPhy[port].fct) 
			devEthPhy[port].fct(PHY_INIT);
	
   	/* Advertise all capabilities */
   	sw21553_PhySetAdvertiseProperties(port, sw21553_PHY_CTRL_PAUSE         |
                                              sw21553_ETHERNET_LINE_100_FULL |
                                              sw21553_ETHERNET_LINE_100_HALF |
                                              sw21553_ETHERNET_LINE_10_FULL  |
                                              sw21553_ETHERNET_LINE_10_HALF );

   	/* Disable Energy-Detect/Power-Down Logic */
   	sw21553_PhyWriteReg(port, INCA_IP_MDIO_REG17_PHY_MCSR, 0);

   	/* read and clear ISR */
   	sw21553_PhyReadReg(port, INCA_IP_MDIO_REG29_PHY_ISR);
		/* clear Link-Down status by reading BSR register */
		sw21553_PhyReadReg(port, INCA_IP_MDIO_REG1_PHY_BSR);
		/* clear Parallel detection fault by reading ANER register */
		sw21553_PhyReadReg(port, INCA_IP_MDIO_REG6_PHY_ANER);

		/* Is AutoNeg enabled ? */
		phyReg = sw21553_PhyReadReg(port, INCA_IP_MDIO_REG0_PHY_BCR);
		devEthPhy[port].AutoNeg = (phyReg & INCA_IP_PHY_REG0_BCR_ANE_MASK);

		/* load LinkRetries counter */
		devEthPhy[port].LinkRetries = devEthPhy[port].LinkRetriesReload;

		/* Do we already have Link? */
		phyReg = sw21553_PhyReadReg(port, INCA_IP_MDIO_REG1_PHY_BSR);

		if (phyReg & INCA_IP_PHY_REG1_BSR_LS_MASK) 
        { 
            PHY_DEBUG (PHY_DEBUG_INIT, ("sw21553_PhyStart: PHY%d Link already up!\n",port));
			/* Make sure EPHY gets updated */
			fsmServeLinkUp(&devEthPhy[port]);
		}
		else 
        {
		 /* Removing old timer (if any) */
		 wdCancel((struct timer_list*) wdId[devEthPhy[port].Port]);
		 
            PHY_DEBUG (PHY_DEBUG_INIT, ("sw21553_PhyStart: PHY%d Link is down!\n",port));
			/* Start Watchdog */
			
			fsmServeLinkDown(&devEthPhy[port]);
		}

	   /* set interrupt enable mask in PHYs */
		sw21553_PhyWriteReg(port, 
                 INCA_IP_MDIO_REG30_PHY_IMR, 
   			     INCA_IP_PHY_REG_INT_PDFAULT_MASK     /* Parallel Detection Fault */
   			   | INCA_IP_PHY_REG_INT_LDOWN_MASK       /* Link Down */
			   | INCA_IP_PHY_REG_INT_RFDETECT_MASK    /* Remote Fault Detected */
			   | INCA_IP_PHY_REG_INT_ANCOMPL_MASK     /* Auto-Negotiation Complete */
			   | INCA_IP_PHY_REG_INT_ENRGON_MASK );   /* ENERGYON Generated */
	} /* for */

   SW_INTERRUPTS_CLEAR (INCA_IRQ_LINE_PHY0LAN | INCA_IRQ_LINE_PHY1PC);
   SW_INTERRUPTS_ENABLE(INCA_IRQ_LINE_PHY0LAN | INCA_IRQ_LINE_PHY1PC);

   PHY_DEBUG (PHY_DEBUG_INIT, ("sw21553_PhyStart: INCA_IP_ICU_IM1_IER = 0x%08X!\n", *INCA_IP_ICU_IM1_IER));

   return OK ;
}

#ifdef PHY_DEBUG_ENABLE
/******************************************************************************
*  sw21553_PhyCallbackFunctionSample
*     Dummy callback for testing sw21553_PhySetCallback. Just does a printout
*     and returns.
*
*  Parameter: 
*     event: PHY_INIT, LINK_ACTIVE or LINK_INACTIVE
*
*  Returns:
*
*  REMARK:
*     The LINK_INACTIVE event might be called more than once!
*
*/
static void sw21553_PhyCallbackFunctionSample(int event)
{
	switch (event)
	{
	case PHY_INIT: 
		logMsg("myPhyCallback: PHY_INIT event!\n",1,2,3,4,5,6);
		break;

	case LINK_ACTIVE:
		logMsg("myPhyCallback: LINK_ACTIVE event!\n",1,2,3,4,5,6);
		break;

	case LINK_INACTIVE:
		logMsg("myPhyCallback: LINK_INACTIVE event!\n",1,2,3,4,5,6);
		break;

	default:
		logMsg("myPhyCallback: unsupported event (%d)!\n",event,2,3,4,5,6);
	}
}
#endif

/*
EXPORT_SYMBOL(sw21553_PhyReadReg);
EXPORT_SYMBOL(sw21553_PhyWriteReg);

EXPORT_SYMBOL(sw21553_Write_MDIO_ACC);

EXPORT_SYMBOL(sw21553_PhyGetProperties);
EXPORT_SYMBOL(sw21553_PhySetAdvertiseProperties);

EXPORT_SYMBOL(sw21553_PhySetFixSpeed);
EXPORT_SYMBOL(sw21553_PhyDisableAutoNeg);
EXPORT_SYMBOL(sw21553_PhyRestartAutoNeg);
EXPORT_SYMBOL(sw21553_PhyReset);
*/
