/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
//-----------------------------------------------------------------------
//Description:	
// 	Since the IKOS emulation is extremly slow, we can't wait until the 
// * whole system is up and then verify our drivers. Thus we decide to hack
// * the kernel so that it is traped in this function to do verification.
// * We borrow some parts from U-boot.
// * Test Cases see below

//-----------------------------------------------------------------------
//Author:	peng.liu@infineon.com
//Created:	12-April-2004
//-----------------------------------------------------------------------
/* History
 * Last changed on:
 * Last changed by: 
 */

#include <linux/config.h> 
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/delay.h>

#include <linux/interrupt.h>
#include <asm/time.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>

#include <linux/in.h>
#include <linux/netdevice.h> 
#include <linux/etherdevice.h> 
#include <linux/ip.h>
#include <linux/tcp.h> 
#include <linux/skbuff.h>
#include <linux/in6.h>
#include <linux/atmdev.h>
#include <linux/wait.h>

#ifdef CONFIG_DEBUG_MINI_BOOT
#define IKOS_MINI_BOOT
#endif

#if defined( IKOS_MINI_BOOT )

#if defined( CONFIG_INCAIP2 )
#include <linux/fs.h>
#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/switch.h>
#include <asm/incaip2/mps.h>
/* #include <asm/incaip2/incaip2_mailbox.h>  */

/* #include "INCAIP2_VCPU_DeviceDriver.h"  */

struct file command_file;
struct file voice0_file;
struct inode cmd_inode;
struct inode voice0_inode;
struct inode* voice0_inode_ptr;

#endif   /* CONFIG_INCAIP2 */

#define CMD_MAX 64
#define CFG_MAXARGS 6

#define INCAIP2_IKOS_EMSG(fmt,args...) printk("%s:" fmt, __FUNCTION__, ##args)
#define INCAIP2_IKOS_DMSG(fmt,args...) printk("%s:" fmt, __FUNCTION__, ##args)
#define INCAIP2_DMA_REG32(reg_num)  *((volatile u32*)(reg_num))


/* 
   #define USE_MAILBOX
*/
#undef USE_MAILBOX

/******************* global data ************************************************/
static struct sk_buff *g_send_skb;
static struct sk_buff * g_sw_send_skb;


/*********  external function declaration ********************/
//kgdb_serial.c
extern char getDebugChar(void);
extern int putDebugChar(char c);
extern int serial_tstc (void);
//in amazon_sw.c
extern struct net_device switch_devs[2];
extern int ifx_switch_init_module(void);
extern void  ifx_switch_cleanup(void);
extern int ifx_switch_init(struct net_device *dev);
extern int ifx_switch_open(struct net_device *dev);
extern int ifx_switch_tx(struct sk_buff *skb, struct net_device *dev);
extern int ifx_switch_release(struct net_device *dev);

extern s32 ifx_mps_open(struct inode *inode, struct file *file_p);
extern s32 ifx_mps_close(struct inode *inode, struct file *filp);
extern s32 ifx_mps_ioctl(struct inode *inode, struct file *file_p, u32 nCmd, unsigned long arg);
extern s32 ifx_mps_register_data_callback(mps_devices type, u32 dir, void (*callback)(mps_devices type));
extern s32 ifx_mps_unregister_data_callback(mps_devices type, u32 dir);
extern s32 ifx_mps_read_mailbox(mps_devices type, mps_message *rw);
extern s32 ifx_mps_write_mailbox(mps_devices type, mps_message *rw);
extern int __init ifx_mps_init_module(void);
extern s32 ifx_mps_debug_mailbox(void);
extern s32 ifx_mps_reset(void);

extern s32 ifx_get_MAC_table_entry_idx( u8 chipId, u32 macTableIndex, u8* macAddrPtr,
					BOOL* lan_portPtr,  BOOL* pc_portPtr, BOOL* cpu_portPtr,
					BOOL* criticalFlagPtr, u32* maStatePtr );

extern int dma_print_statistics( BOOL registers, BOOL channels, BOOL ports );

#ifdef CONFIG_PCI
//pci.
extern int pcibios_init(void);
#endif 



#ifdef CONFIG_INCAIP2

#if 0
extern s32  INCAIP2_MPS_MbxReadUpstrmFifo( MPS_MB_DEV_STR  *pMBDev, 
					   mps_message   *pPkg, 
					   s32             timeout);
extern s32 INCAIP2_MPS_MbxWriteDwnstrCmd( register MPS_MB_DEV_STR *pMBDev, 
					  mps_message *readWrite );
extern s32 INCAIP2_MPS_MbxWriteDwnstrData( register MPS_MB_DEV_STR *pMBDev, 
					   mps_message *readWrite );
extern u32 INCAIP2_MPS_InitStructures(MPS_COMM_DEV *pDev);
extern void INCAIP2_MPS_DSP_ReleaseStructures(MPS_COMM_DEV *pDev);   

extern void INCAIP2_MPS_DSP_MbxClearFifo(MailboxParam_s *pFifo);
extern u32 INCAIP2_MPS_MbxFifoMemAvailable( MailboxParam_s *mbx );
extern s32 INCAIP2_MPS_DSP_Restart(void);
extern s32 INCAIP2_MPS_DSP_Reset(void);
extern s32 INCAIP2_MPS_VCPU_DownloadFirmware(MPS_MB_DEV_STR *pDev, VCPU_FWDWNLD *pFWDwnld);
#endif


#endif

/*****  command type ****/
struct cmd_tbl_s{
    char * name; 					/* command name		*/
    int (*cmd)(struct cmd_tbl_s *, int, char *[]); 	/* command functionn	*/
    char * usage;					/* Usage message(short)	*/
    int	len;					/* minimum len		*/
};
typedef struct cmd_tbl_s cmd_tbl_t;

/*********  internal function declaration ********************/
static int do_help    (cmd_tbl_t *cmdtp, int argc, char *argv[]);
static int do_time( cmd_tbl_t *cmdtp, int argc, char *argv[]);
static int do_tpe_swie    (cmd_tbl_t *cmdtp, int argc, char *argv[]);
static int do_tpe_dma    (cmd_tbl_t *cmdtp, int argc, char *argv[]);
static int do_tpe_test   (cmd_tbl_t *cmdtp, int argc, char *argv[]);
static int do_sw_lp( cmd_tbl_t * cmdtp, int argc, char *argv[]);
static int do_fast_sw_lp( cmd_tbl_t * cmdtp, int argc, char *argv[]);
static int do_sw_tpe_lp ( cmd_tbl_t *cmdtp, int argc, char *argv[]);

#ifdef CONFIG_PCI
static int do_pci ( cmd_tbl_t *cmdtp, int argc, char *argv[]);
#endif 

#ifdef CONFIG_INCAIP2
static int do_incaip2_sw_mbx_bridge( cmd_tbl_t * cmdtp, int argc, char *argv[]);
static int do_incaip2_mbx_test( cmd_tbl_t * cmdtp, int argc, char *argv[]);
s32 incaip2_sw_to_mbx(struct sk_buff* skb);
s32 incaip2_mbx_to_sw( mps_message *rw );
void ikos(void);
#endif

#define SMALL_LOOP 0 /* ETH -> ETH */
#define BIG_LOOP   1 /* ETH -> MBX -> ETH */
#define MBX_LOOP   2 /* MBX -> MBX */
#define LOOP_LOOP  3 /* MBX -> MBX + ETH -> ETH */
static s32 loop=LOOP_LOOP;


static s32 mbx_rcv_cnt=0;
static s32 mbx_2_switch_cnt=0;
static u32 sw_2_mbx_discard_cnt=0;   
static s32 sw_2_mbx_rx_cnt=0;
static s32 sw_2_mbx_tx_cnt=0;
static s32 cmd_address_count=0;
static s32 mbx_voice_data_rcv_cnt=0;

/*
  typedef enum
  {
  unknown,
  command,
  voice0,
  voice1,
  voice2,
  voice3
  }DEVTYPE;
*/


typedef struct
{
    u32            *pStart;  /**< pointer to mailbox's read/write start address */
    u32            *pEnd;    /**< pointer to mailbox's read/write end address */
    u32            *pWrite;  /**< pointer to mailbox's write index location */
    u32            *pRead;   /**< pointer to mailbox's read index location */
    u32            FifoSize;
}MailboxParam_s;


typedef struct
{
    /* Mailbox */
    volatile u32   MBC_IOD;
    volatile u32   MBC_CR;
    volatile u32   MBC_FS;
    volatile u32   MBC_DA;
    volatile u32   MBC_IABS;
    volatile u32   MBC_ITMP;
    volatile u32   MBC_OABS;
    volatile u32   MBC_OTMP;
}OAK_RW_MB;


typedef wait_queue_head_t                 SYS_POLL;
#if 0
typedef struct
{
    /* void pointer to the base device driver structure */
    void              *pVCPU_DEV;

    /* Mutex semaphore to access the device */
    SEM_ID            sem_dev;

    /* Mutex semaphore to access the device */
    SEM_ID            sem_read_fifo;
   
    /* Wakeuplist for the select mechanism */
    SYS_POLL          oak_wklist;

    /* Base Address of the Mailbox Access */
    OAK_RW_MB         *pBaseReadMB;
    OAK_RW_MB         *pBaseWriteMB;
      
    /* Base Address of the Global Register Access */
    MBX_GLOBAL_REG    *pBaseGlobal;
      
    /* missing space in fifo */
    /* used if the interrupt is disabled and waiting for this amount of  */
    /* free bytes in the fifo to get enabled again                       */
    u32            irqMissingBytes;
    u32            VoiceChannelStream;
   
    /* Mask Bits for the IRQ register of the mailbox */
    u32            MBC_Mask;
    u32            MBC_Mask01;
    u32            MBC_Mask10;
   
    MailboxParam_s    upstrm_fifo;  /**< Data exchange FIFO for write (downstream) */ 
    MailboxParam_s    dwstrm_fifo;  /**< Data exchange FIFO for read (upstream) */
   
#ifdef OAK_FIFO_BLOCKING_WRITE
    SEM_ID           sem_write_fifo;
#endif /* OAK_FIFO_BLOCKING_WRITE */

    u32            errnum;
   
#ifdef CONFIG_PROC_FS
    /* interrupt counter */
    volatile u32   RxnumIRQs;
    volatile u32   TxnumIRQs;

    volatile u32   RxnumMiss;
    volatile u32   TxnumMiss;
  
    volatile u32   RxnumBytes;
    volatile u32   TxnumBytes;
#endif /* CONFIG_PROC_FS */

    mps_devices           devID;  /**< Device ID  1->command 
				     2->voice chan 0
				     3->voice chan 1 
				     4->voice chan 2 
				     5->voice chan 3 */
   
    u32            HeaderORMask;
    u32            HeaderANDMask;

#ifdef OAK_FIFO_BLOCKING_WRITE
    volatile BOOL     full_write_fifo;   
#endif /* OAK_FIFO_BLOCKING_WRITE */

    /* installed or not? */
    BOOL              Installed;
   
    /* missing voice package for downstream */
    BOOL              VoiceMissingPackage;
   
#ifdef OAK_FIFO_BLOCKING_WRITE
    /* variable if the driver should block on write to the transmit FIFO */
    BOOL              bBlockWriteMB;
#endif /* OAK_FIFO_BLOCKING_WRITE */
    void              (*down_callback)(mps_devices type);
    void              (*up_callback)(mps_devices type);
    UINT              event;
    UINT              event_mask;
    void              (*event_callback)(UINT mask);
   
} MPS_MB_DEV_STR;
#endif

typedef enum
{
    COMMAND=1,
    VOICE
}MbxMessageType_e;



typedef enum
{
    UPSTREAM,
    DOWNSTREAM
}MbxDirection_e;




typedef struct
{
    u32 rw      : 1;
    u32 res1    : 2;
    u32 type    : 5;
    u32 res2    : 4;
    u32 chan    : 4;
    u32 res3    : 1;
    u32 odd     : 2;
    u32 res4    : 5;
    u32 plength : 8;
} MbxMsgHd_s;

typedef union
{
    u32        val;
    MbxMsgHd_s hd;
} MbxMsgHd_u;


typedef struct
{
    MbxMsgHd_u header;
    u32        data[14];   /* maximum command length to be checked*/
}MbxMsg_s;


typedef enum
{
    WR,
    RD
} readwrite_e;



typedef enum
{
    DISABLE,
    ENABLE
} enable_disable_e;
/*
  MPS_COMM_DEV INCAIP2_MPS_dev;
*/

static u32 CMD_PCM_CTRL_Message[]  = { 0x06000004, 0x80000000 };

typedef u8 inp_sig_array_t;
inp_sig_array_t inp_sig_array[5]= { 0x00, 0x00, 0x00, 0x00, 0x00 };
static u32 CMD_PCM_CHAN_Message[]  = { 0x06000108, 0x80000000, 0x20002000 };

/*
  static u32 CMD_CODER_CHAN_SPEECH_COMPR_Message[]  = { 0x06007C08, 0x84080000, 0x1FFF1FFF };
*/
static u32 CMD_CODER_CHAN_SPEECH_COMPR_Message[]  = { 0x06007C08, 0x80180802, 0x20002000 };
static u32 CMD_CODER_CHAN_RTP_SUP_Message[]  = { 0x06007804, 0x01000000 };

static u32 CMD_AL_CHAN_Message[]  = { 0x06002108, 0x80182000, 0x20000000 };
static u32 OPMODE_CMD_Message[] = { 0x01000004, 0x00000000 };
static u32 DBG_CMD_read[] = {0x81000A10};
static u32 test_rtp_packet[] = 
  {{ 0x80010000, 0x00000000, 0x00000001, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
     0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 
     0x01010101, 0x01010101, 0x01010101},
   { 0x80010001, 0x00000010, 0x00000001, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202,
     0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 0x02020202, 
     0x02020202, 0x02020202, 0x02020202},
   { 0x80010002, 0x00000020, 0x00000001, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303,
     0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 0x03030303, 
     0x03030303, 0x03030303, 0x03030303}};
 
/* microphone gain 36dB */
/*
static u32 CMD_AUDIO_CHANNEL_CTRL_Message[]  = { 0x06008114, 0x80000600, 0x20002000, 0x20000000, 0x00000000, 0x00000000 };     
*/            
     
/* input signal looped via signal array */
/*
static u32 CMD_AUDIO_CHANNEL_CTRL_Message[]  = { 0x06008114, 0x80000100, 0x20002000, 0x20000000, 0x00400000, 0x00000000 }; 
*/     

/* input signal looped via signal array,
  microphone gain 6dB, loudspeaker gain -9dB
*/
static u32 CMD_AUDIO_CHANNEL_CTRL_Message[]  = { 0x06008118, 0x90000166, 0x20002000, 0x20000000, 0x00000000, 0x00000000, 0x00000000 };     
       
     
static cmd_tbl_t cmd_tbl []={

#ifdef CONFIG_PCI
    {"pci",do_pci,"pci: scan PCI bus", 3},
#endif
#ifdef CONFIG_INCAIP2
    {"sw_mbx",do_incaip2_sw_mbx_bridge,"sw_mbx: switch/mailbox bridging", 5},
    {"mbx",do_incaip2_mbx_test,"mbx: mailbox test", 5},    
#endif
    {"time", do_time,"time: print jiffies value",1},
    {"help",do_help, "h[elp]: this list", 1},
    {NULL, do_help, NULL,0},
};

/*********************  helpers **********************************************/
static void ifx_print_mps_message( mps_message  *dsp_rw_ptr )
{
    s32 k;


    printk("dsp_rw.nDataBytes= %d\n", dsp_rw_ptr->nDataBytes);
    printk("dsp_rw.pData=0x%08X\n", dsp_rw_ptr->pData);	
    for(k=0;k<dsp_rw_ptr->nDataBytes;k++)
    {
	printk("dsp_rw.pData[%d]=0x%02X\n", k, *((u8*)(KSEG0ADDR(dsp_rw_ptr->pData+k))));
    }

    printk("dsp_rw.cmd_type= %d\n", dsp_rw_ptr->cmd_type);
    printk("dsp_rw.RTP_PaylOffset=0x%08X\n", dsp_rw_ptr->RTP_PaylOffset);

    return;
}



static inline void dump_data(u8 * data, int len)
{
    int i;
    for(i=0;i<len;i++){	
	printk("%02x ",data[i]);
	if (i % 16 == 15)
	    printk("\n");
    }
    printk("\n");
}

#define IKOS_DUMP_SKB
/*
 * Brief:	dump skb data
 */
static inline void dump_skb(struct sk_buff * skb)
{
#ifdef IKOS_DUMP_SKB
    dump_data(skb->data, skb->len);
#endif
}
/*
 * Brief:	check skb data
 * Return:
 *		0
 *		-1 fails
 */
static inline int check_skb(struct atm_vcc *vcc, struct sk_buff *skb)
{
    int i;
    u8 * data;
    data = (u8 *) (skb->data);
    if (data[0]  != (u8) vcc->vpi) return -1;
    if (data[1]  != (u8) vcc->vci) return -1;
    for(i=2;i<skb->len;i++){
	if (data[i] != (u8) ((i-2)%0x100)) {
	    return -1;
	}
    }
    return 0;
}

/*
 * Brief:	fill in skb data
 */
static inline void fill_skb(struct atm_vcc *vcc, struct sk_buff *skb, u32 length)
{
    int i;
    u8 * data;
    data = (u8 *) skb_put(skb, length);
    data[0]  = ((u8) vcc->vpi);
    data[1]  = ((u8) vcc->vci);
    for(i=2;i<skb->len;i++){
	data[i] = ((u8) ((i-2)%0x100));
    }
}


#ifdef CONFIG_PCI
/* Brief:	pci
 */
static int do_pci ( cmd_tbl_t *cmdtp, int argc, char *argv[])
{
    pcibios_init();
    return 0;
}
#endif //CONFIG_PCI


#ifdef CONFIG_INCAIP2


s32 INCAIP2_MPS_DSP_Send_PCMCtrl_Command( readwrite_e readwrite,  enable_disable_e en_dis_pcm )
{
    MbxMsg_s msg;
    mps_message rw;

    printk("%s( readwrite=%d, en_dis_pcm=%d) \n",__FUNCTION__, readwrite, en_dis_pcm);
    
    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, (u8*)&CMD_PCM_CTRL_Message, msg.header.hd.plength+4);

    /*
     * adapt read/write bit
     */
    msg.header.hd.rw = readwrite;
    /*
     * adapt enable/disable bit
     */
    if( en_dis_pcm == ENABLE )
	msg.data[0] |= 0x80000000;
    else
	msg.data[0] &= ~0x80000000;

    
    printk("%s - msg.header=0x%08X \n" ,__FUNCTION__, msg.header.val);
    /*
     * pass message data to data transfer structure and send message
     */
    rw.pData = (u8*)&msg;
    if(readwrite==RD)
    {
        /* in case of Read Command only the header is sent */
        rw.nDataBytes = 4;
        msg.header.hd.plength = 4;
    }
    else    
        rw.nDataBytes = msg.header.hd.plength+4;
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */
/*
  ifx_mps_write_mailbox( voice0, &wrt_data[0] );
*/   

    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
	printk("Writing PCM Control Command Message failed !\n");
	return ERROR;
    }
    else
	printk("Writing PCM Control Command Message successful !\n");		

    return OK;
}


s32 INCAIP2_MPS_DSP_Send_PCMChan_Command( readwrite_e readwrite,
                                          u8 channel, enable_disable_e en_dis_chan,
                                          u8 *inp_sig_array, u8 xts, u8 rts )
{
    MbxMsg_s msg;
    mps_message rw;
/*	inp_sig_array_t *inp_sig_ptr;  */

    printk("%s( readwrite=%d, en_dis_chan=%d, channel=%d) \n",
           __FUNCTION__, readwrite, en_dis_chan, channel);

    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, (u8*)&CMD_PCM_CHAN_Message, sizeof(CMD_PCM_CHAN_Message));

    /*
     * adapt message header fields
     */
    msg.header.hd.rw   = readwrite; /* read/write operation */
    msg.header.hd.chan = channel;   /* channel select */
    /*
     * adapt message data[0]
     */
    msg.data[0] |= (((u32)(inp_sig_array[0] & 0x3F))<<16);  /* signal array address of inp. sig. 1 */

    if( en_dis_chan == ENABLE )      /* adapt channel enable/disable bit */
	msg.data[0] |= 0x80000000;
    else
	msg.data[0] &= ~0x80000000;

    msg.data[0] |= (((u32)xts & 0x0000007F) << 16) | ((u32)rts & 0x0000007F);

    /*
     * adapt message data[1] and message data[2]
     */
    msg.data[2] |= (((u32)(inp_sig_array[1] & 0x3F))<<24) |
	(((u32)(inp_sig_array[2] & 0x3F))<<16) |
	(((u32)(inp_sig_array[3] & 0x3F))<<8) |
	((u32)(inp_sig_array[4] & 0x3F));

    /*
     * pass message data to data transfer structure and send message
     */
    rw.pData = (u8*)&msg;
    if(readwrite==RD)
    {
        /* in case of Read Command only the header is sent */
        rw.nDataBytes = 4;
        msg.header.hd.plength = 8;
    }
    else    
        rw.nDataBytes = msg.header.hd.plength+4;    
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */


    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
	printk("Writing PCM Channel Control Command Message failed !\n");
	return ERROR;
    }

    return OK;

}


s32 INCAIP2_MPS_DSP_Send_CoderChanSpeechCompr_Command( readwrite_e readwrite,
                                                       u8 channel, 
                                                       enable_disable_e en_dis_chan )
{
    MbxMsg_s msg;
    mps_message rw;
/*	inp_sig_array_t *inp_sig_ptr;  */

    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, 
            (u8*)&CMD_CODER_CHAN_SPEECH_COMPR_Message, 
            sizeof(CMD_CODER_CHAN_SPEECH_COMPR_Message));

    /*
     * adapt message header fields
     */
    msg.header.hd.rw   = readwrite; /* read/write operation */
    msg.header.hd.chan = channel;   /* channel select */

    if( en_dis_chan == ENABLE )      /* adapt channel enable/disable bit */
	msg.data[0] |= 0x80000000;
    else
	msg.data[0] &= ~0x80000000;

    /*
     * pass message data to data transfer structure and send message
     */
    rw.pData = (u8*)&msg;
    if(readwrite==RD)
    {
        /* in case of Read Command only the header is sent */
        rw.nDataBytes = 4;
        msg.header.hd.plength = 8;        
    }
    else    
        rw.nDataBytes = msg.header.hd.plength+4;        
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */


    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
	printk("Writing PCM Channel Control Command Message failed !\n");
	return ERROR;
    }

    return OK;

}


s32 INCAIP2_MPS_DSP_Send_CoderChanRTPSupport( readwrite_e readwrite,
                                              u8 channel )
{
    MbxMsg_s msg;
    mps_message rw;
    /*	inp_sig_array_t *inp_sig_ptr;  */

    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, 
	    (u8*)&CMD_CODER_CHAN_RTP_SUP_Message, 
	    sizeof(CMD_CODER_CHAN_RTP_SUP_Message));

    /*
     * adapt message header fields
     */
    msg.header.hd.rw   = readwrite; /* read/write operation */
    msg.header.hd.chan = channel;   /* channel select */

    /*
     * pass message data to data transfer structure and send message
     */
    rw.pData = (u8*)&msg;
    if(readwrite==RD)
    {
	/* in case of Read Command only the header is sent */
	rw.nDataBytes = 4;
	msg.header.hd.plength = 8;        
    }        
    else    
	rw.nDataBytes = msg.header.hd.plength+4;            
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */


    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
	printk("Writing PCM Channel Control Command Message failed !\n");
	return ERROR;
    }

    return OK;

}
/* !!! LM  */
s32 INCAIP2_MPS_DSP_send_OPMODE_CMD( )
{
    MbxMsg_s msg;
    mps_message rw;
    u8 ch;
    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, (u8*)&OPMODE_CMD_Message, sizeof(OPMODE_CMD_Message));

    /*
     * adapt message header fields
     */
    msg.header.hd.rw = WR; /* read/write operation */
    /*
     * pass message data to data transfer structure and send message
     */
    ch = getDebugChar();
    msg.data[0] = (ch - 48) << 16; 
    rw.pData = (u8*)&msg;
    rw.nDataBytes = 8;
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */
    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
        printk("Writing OPMODE failed !\n");
        return ERROR;
    }
    return OK;
}


s32 INCAIP2_MPS_DSP_send_Audio_Channel_Loop_CMD( void )
{
    MbxMsg_s msg;
    mps_message rw;
    u8 ch;
    
    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, (u8*)&CMD_AUDIO_CHANNEL_CTRL_Message, sizeof(CMD_AUDIO_CHANNEL_CTRL_Message));    
 
    rw.pData = (u8*)&msg;
    rw.nDataBytes = msg.header.hd.plength+4;     
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */
    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
        printk("Writing Audio Channel Loop CMD failed !\n");
        return ERROR;
    }
    return OK;    
    
}

/* Read corrent DBG state and change it in recieve func */
s32 INCAIP2_MPS_DSP_Start_Loop( )
{
    MbxMsg_s msg;
    mps_message rw;
    memcpy( (u8*)&msg, (u8*)&DBG_CMD_read, sizeof(DBG_CMD_read));
    rw.pData = (u8*)&msg;
    rw.nDataBytes = 4;
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */

    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
        printk("Writing DBG Command Message failed !\n");
        return ERROR;
    }
    return OK;
}

s32 INCAIP2_MPS_DSP_read_cur_alm_state()
{
    MbxMsg_s msg;
    mps_message rw;

    memcpy( (u8*)&msg, (u8*)&OPMODE_CMD_Message, sizeof(OPMODE_CMD_Message));
    msg.header.hd.rw = RD;
    rw.pData = ((u8*)&msg);
    rw.nDataBytes = 4;

    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
        printk("Writing OPMODE read message failed !\n");
        return ERROR;
    }
    return OK;
}

/* !!! LM */
s32 INCAIP2_MPS_DSP_Send_AL_Chan_Command( readwrite_e readwrite,
                                          u8 channel, enable_disable_e en_dis_chan,
                                          u8 *inp_sig_array)
{
    MbxMsg_s msg;
    mps_message rw;
/*	inp_sig_array_t *inp_sig_ptr;  */

    printk("%s( readwrite=%d, en_dis_chan=%d, channel=%d) \n",
           __FUNCTION__, readwrite, en_dis_chan, channel);

    /* get default message data comprising fixed parameters */
    memcpy( (u8*)&msg, (u8*)&CMD_AL_CHAN_Message, sizeof(CMD_AL_CHAN_Message));

    /*
     * adapt message header fields
     */
    msg.header.hd.rw   = readwrite; /* read/write operation */
    msg.header.hd.chan = channel;   /* channel select */
    /*
     * adapt message data[0]
     */
    //    msg.data[0] |= (((u32)(inp_sig_array[0] & 0x3F)) << 16);  /* signal array address of inp. sig. 1 */

    if( en_dis_chan == ENABLE )      /* adapt channel enable/disable bit */
        msg.data[0] |= 0x80000000;
    else
        msg.data[0] &= ~0x80000000;

    /*
     * adapt message data[1] and message data[2]
     */
    msg.data[1] |= (((u32)(inp_sig_array[1] & 0x3F))<<8) |
        ((u32)(inp_sig_array[2] & 0x3F));
    msg.data[2] |= (((u32)(inp_sig_array[3] & 0x3F))<<24) |
        (((u32)(inp_sig_array[4] & 0x3F)) << 16);

    /*
     * pass message data to data transfer structure and send message
     */
    rw.pData = (u8*)&msg;
    if(readwrite==RD)
    {
        /* in case of Read Command only the header is sent */
        rw.nDataBytes = 4;
        msg.header.hd.plength = 12;
    }
    else    
        rw.nDataBytes = msg.header.hd.plength + 4;    
    rw.RTP_PaylOffset=0x00000000;  /* unused for command messages and  for downsteam dir. */
    rw.cmd_type=0xFF;  /* unused for command messages */


    if( ifx_mps_write_mailbox( command, &rw ) != OK )
    {
        printk("Writing AnalogLine Channel Control Command Message failed !\n");
        return ERROR;
    }

    return OK;

}
/* !!! LM  */


void INCAIP2_Switch_Print_MAC_TableHeader(void)
{

    printk("\nINCAIP2 MAC Table\n");
    printk("------------------------------------------------------------------\n");
    printk("| index |   MAC Address      || LAN PC  CPU crit  age           \n");
    printk("------------------------------------------------------------------\n");



}

void INCAIP2_Switch_Print_MAC_TableEntry( u32 macTableIndex, u8 *macAddr, 
                                          BOOL lan_port,  BOOL pc_port, BOOL cpu_port,
                                          BOOL criticalFlag, u32 maState )
{

    printk("[%3d] |  %02x:%02x:%02x:%02x:%02x:%02x    || %1c | %1c | %1c | %1c | %3d\n",
	   macTableIndex,
	   macAddr[0], macAddr[1], macAddr[2],
	   macAddr[3], macAddr[4], macAddr[5],
	   (lan_port)?'+':' ',  (pc_port)?'+':' ', (cpu_port)?'+':' ', 
	   (criticalFlag)?'1':'0', maState );
}


void INCAIP2_Switch_Print_MAC_TableTrailer(int count)
{
    /*
      "        INCA-IP MAC table entries"
      "INDEX |     MAC Address    || Cntrl -> |M|V|L|P|CR| age "
      "----------------------------------------------------------------"
      [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 aged out
      [212] |  00:a0:24:b8:b8:47 ||  0x14 -> | | | | |  | 000 static
      [182] |  00:50:bf:49:eb:83 ||  0xe2 -> | | | | |  | 111 dynamic
    */

    printk("----------------------------------------------------------------\n") ;
    printk("  ==>  MAC TABLE: %d entries found\n", count) ;
    printk("----------------------------------------------------------------\n") ;
    /*
      "----------------------------------------------------------------") ;
      "  ==>  MAC TABLE: %d entries found"
      "----------------------------------------------------------------") ;
    */
}



void INCAIP2_Switch_Print_MAC_Table(void)
{
    u8 chipId = 0;
    u32 macTableIndex;
    u8 macAddr[6];
    BOOL lan_port;
    BOOL pc_port;    
    BOOL cpu_port;    
    u32 mpid;
    BOOL criticalFlag;
/*	BOOL valid_entry=FALSE;  */
    u32 maState;
    s32 count = 0;

    INCAIP2_Switch_Print_MAC_TableHeader();
    for ( macTableIndex = 0; macTableIndex <= MAX_MACTABLEINDEX; macTableIndex++ )
    {
	if( ifx_get_MAC_table_entry_idx( chipId, macTableIndex, macAddr,
                                         &lan_port,  &pc_port, &cpu_port,
                                         &criticalFlag, &maState ) != 0 )
	{
	    INCAIP2_IKOS_EMSG("ifx_get_MAC_table_entry_idx failed for index %d\n", macTableIndex);
	    return;
	}


/*        
	  s32 j=0;
	  do
	  {
	  printk("maState=0x%08X\n", maState);
	  if(maState!=0xF)
	  valid_entry=TRUE;
	  j++;
	  } while( !valid_entry && j<6 );
*/        
    if(maState!=0xF)
	{
	    count++;

	    INCAIP2_Switch_Print_MAC_TableEntry( macTableIndex, macAddr, 
                                                 lan_port,  pc_port, cpu_port,
                                                 criticalFlag, maState );
                                                 
	}

    }
    INCAIP2_Switch_Print_MAC_TableTrailer(count);
    return;
}









void INCAIP2_Switch_Print_MIB_Counters( void )
{
    INCAIP2_SwitchPort_MIB_Table_t pPortMIBTable[2];
    INCAIP2_SwitchMac_MIB_Table_t  pMacMIBTable[2];
    s32 i;

    memset( &pPortMIBTable, 0, sizeof(INCAIP2_SwitchPort_MIB_Table_t)*2 );
    memset( &pMacMIBTable, 0, sizeof(INCAIP2_SwitchMac_MIB_Table_t)*2 );    
    
    for(i=0;i<2;i++)
    {
	if( INCAIP2_SwitchPortGetMIBCounters( i,
					      &pPortMIBTable[i] ) != 0 )
	{
	    INCAIP2_IKOS_EMSG("INCAIP2_Switch_Print_MIB_Counters: Port MIB Counter access failed\n");
	    return;
	}
	if( INCAIP2_SwitchMacGetMIBCounters( i,
					     &pMacMIBTable[i] ) != 0 )
	{
	    INCAIP2_IKOS_EMSG("INCAIP2_Switch_Print_MIB_Counters: Mac MIB Counter access failed\n");
	    return;
	}        

    }
    printk("\nInca-IP2 MIB Counters\n");
    printk("                              LAN Port                   PC Port\n");
    printk("----------------------------------------------------------------------------------\n");
    printk("rxBytes               %10d  (0x%08X)    %10d  (0x%08X)  \n",
	   pPortMIBTable[0].rxBytes, pPortMIBTable[0].rxBytes,
	   pPortMIBTable[1].rxBytes, pPortMIBTable[1].rxBytes);
    printk( "rxUnicastPkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxUnicastPkts, pPortMIBTable[0].rxUnicastPkts,
	    pPortMIBTable[1].rxUnicastPkts, pPortMIBTable[1].rxUnicastPkts);
    printk( "rxBroadcastPkts       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxBroadcastPkts, pPortMIBTable[0].rxBroadcastPkts,
	    pPortMIBTable[1].rxBroadcastPkts, pPortMIBTable[1].rxBroadcastPkts);
    printk( "rxMulticastPkts       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxMulticastPkts, pPortMIBTable[0].rxMulticastPkts, 
	    pPortMIBTable[1].rxMulticastPkts, pPortMIBTable[1].rxMulticastPkts); 
    printk( "rxCRCErrorPkts        %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxCRCErrorPkts, pPortMIBTable[0].rxCRCErrorPkts,
	    pPortMIBTable[1].rxCRCErrorPkts, pPortMIBTable[1].rxCRCErrorPkts);
    printk( "rxUnderSizeGoodPkts   %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxUnderSizeGoodPkts, pPortMIBTable[0].rxUnderSizeGoodPkts,
	    pPortMIBTable[1].rxUnderSizeGoodPkts, pPortMIBTable[1].rxUnderSizeGoodPkts);
    printk( "rxUnderSizeErrorPkts  %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxUnderSizeErrorPkts, pPortMIBTable[0].rxUnderSizeErrorPkts,
	    pPortMIBTable[1].rxUnderSizeErrorPkts, pPortMIBTable[1].rxUnderSizeErrorPkts);
    printk( "rx64BytePkts          %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rx64BytePkts, pPortMIBTable[0].rx64BytePkts,
	    pPortMIBTable[1].rx64BytePkts, pPortMIBTable[1].rx64BytePkts);
    printk( "rx127BytePkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rx127BytePkts, pPortMIBTable[0].rx127BytePkts,
	    pPortMIBTable[1].rx127BytePkts, pPortMIBTable[1].rx127BytePkts);
    printk( "rx255BytePkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rx255BytePkts, pPortMIBTable[0].rx255BytePkts,
	    pPortMIBTable[1].rx255BytePkts, pPortMIBTable[1].rx255BytePkts);
    printk( "rx511BytePkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rx511BytePkts, pPortMIBTable[0].rx511BytePkts,
	    pPortMIBTable[1].rx511BytePkts, pPortMIBTable[1].rx511BytePkts);
    printk( "rx1023BytePkts        %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rx1023BytePkts, pPortMIBTable[0].rx1023BytePkts,
	    pPortMIBTable[1].rx1023BytePkts, pPortMIBTable[1].rx1023BytePkts);
    printk( "rxMaxBytePkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxMaxBytePkts, pPortMIBTable[0].rxMaxBytePkts,
	    pPortMIBTable[1].rxMaxBytePkts, pPortMIBTable[1].rxMaxBytePkts);
    printk( "rxOversizeGoodPkts    %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxOversizeGoodPkts, pPortMIBTable[0].rxOversizeGoodPkts,
	    pPortMIBTable[1].rxOversizeGoodPkts, pPortMIBTable[1].rxOversizeGoodPkts);
    printk( "rxOversizeBadPkts     %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxOversizeBadPkts, pPortMIBTable[0].rxOversizeBadPkts,
	    pPortMIBTable[1].rxOversizeBadPkts, pPortMIBTable[1].rxOversizeBadPkts);
    printk( "rxGoodPausePkts       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxGoodPausePkts, pPortMIBTable[0].rxGoodPausePkts,
	    pPortMIBTable[1].rxGoodPausePkts, pPortMIBTable[1].rxGoodPausePkts);
    printk( "rxDroppedPkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].rxDroppedPkts, pPortMIBTable[0].rxDroppedPkts,  
	    pPortMIBTable[1].rxDroppedPkts, pPortMIBTable[1].rxDroppedPkts);
    printk( "txBytes               %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].txBytes, pPortMIBTable[0].txBytes,
	    pPortMIBTable[1].txBytes, pPortMIBTable[1].txBytes);
    printk( "txUnicastPkts         %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].txUnicastPkts, pPortMIBTable[0].txUnicastPkts,
	    pPortMIBTable[1].txUnicastPkts, pPortMIBTable[1].txUnicastPkts);
    printk( "txBroadcastPkts       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].txBroadcastPkts, pPortMIBTable[0].txBroadcastPkts,
	    pPortMIBTable[1].txBroadcastPkts, pPortMIBTable[1].txBroadcastPkts);
    printk( "txMulticastPkts       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pPortMIBTable[0].txMulticastPkts, pPortMIBTable[0].txMulticastPkts,
	    pPortMIBTable[1].txMulticastPkts, pPortMIBTable[1].txMulticastPkts);


    printk( "txPauseCount          %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txPauseCount, pMacMIBTable[0].txPauseCount,
	    pMacMIBTable[1].txPauseCount, pMacMIBTable[1].txPauseCount);
    printk( "txSingleCollCount     %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txSingleCollCount, pMacMIBTable[0].txSingleCollCount,
	    pMacMIBTable[1].txSingleCollCount, pMacMIBTable[1].txSingleCollCount);                     
    printk( "txMultCollCount       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txMultCollCount, pMacMIBTable[0].txMultCollCount,
	    pMacMIBTable[1].txMultCollCount, pMacMIBTable[1].txMultCollCount); 
    printk( "txExcessCollCount     %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txExcessCollCount, pMacMIBTable[0].txExcessCollCount,
	    pMacMIBTable[1].txExcessCollCount, pMacMIBTable[1].txExcessCollCount);                      
    printk( "txLateCollCount       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txLateCollCount, pMacMIBTable[0].txLateCollCount,
	    pMacMIBTable[1].txLateCollCount, pMacMIBTable[1].txLateCollCount);                      
    printk( "txDeferredCount       %10d  (0x%08X)    %10d  (0x%08X)  \n",
	    pMacMIBTable[0].txDeferredCount, pMacMIBTable[0].txDeferredCount,
	    pMacMIBTable[1].txDeferredCount, pMacMIBTable[1].txDeferredCount);                      
    return;
}




s32 incaip2_send_pck_to_coder0(void)
{
    mps_message rw;
    //    char num = getDebugChar();
    rw.pData          = (u8*)PHYSADDR((u8*) test_rtp_packet);
    rw.nDataBytes     = 92;
    rw.cmd_type       = CMD_RTP_VOICE_DATA_PACKET;

    if( ifx_mps_write_mailbox(voice0, &rw) != OK )
    {
	INCAIP2_IKOS_EMSG("ifx_mps_write_mailbox fails\n");
	return -1;
    }
}






/* static u32 g_sw_drop=0;  */
/* Brief:
 * passes data received from ethernet switch to VCPU mailbox
 * The function is called by switch_hw_receive() on occurence of a receive interrupt
 * in case of define IKOS_MINI_BOOT
 *
 *
 *
 */

s32 incaip2_sw_to_mbx(struct sk_buff* skb )
{
    struct net_device* net_dev_ptr;
    mps_message rw;
    struct ethhdr *eth_hdr;
    unsigned char tmp_src[6];
    static u32 rx_pck_cnt=0;
/*
static u32 discard_pck_cnt=0;   
    static s32 count=0;
*/
    s32 i=0;

    /* 
       INCAIP2_IKOS_DMSG("incaip2_sw_to_mbx\n");
    */       
    if(loop > 3)
    {
        printk("Invalid loop parameter %d\n", loop);
        return -1;
    }
    
    eth_hdr = (struct ethhdr*) (skb->data);  
    /*
      printk("eth_hdr->h_dest= %02X:%02X:%02X:%02X:%02X:%02X\n", 
      eth_hdr->h_dest[0], eth_hdr->h_dest[1],eth_hdr->h_dest[2],
      eth_hdr->h_dest[3],eth_hdr->h_dest[4],eth_hdr->h_dest[5]);
      printk("eth_hdr->h_source= %02X:%02X:%02X:%02X:%02X:%02X\n", 
      eth_hdr->h_source[0], eth_hdr->h_source[1],eth_hdr->h_source[2],
      eth_hdr->h_source[3],eth_hdr->h_source[4],eth_hdr->h_source[5]);            
      printk("eth_hdr->h_proto= 0x%04X\n", eth_hdr->h_proto);    
    */    
    sw_2_mbx_rx_cnt++;  
    
    if (skb_shinfo(skb)->frag_list)
    {
        printk("%s - packet is fragmented !!!\n", __FUNCTION__);
    }    
    
/*
if( ((sw_2_mbx_rx_cnt)%100 == 0) )          
        printk("%s - pck_cnt=%10d \n", __FUNCTION__, sw_2_mbx_rx_cnt );      
*/
#if 0
    if((eth_hdr->h_dest[0] != 0x00)  ||  (eth_hdr->h_dest[1] != 0x20 ) || 
       (eth_hdr->h_dest[2] != 0xda)  ||  (eth_hdr->h_dest[3] != 0x86 ) ||
       (eth_hdr->h_dest[4] != 0x23)  ||  (eth_hdr->h_dest[5] != 0x74 ))
    {
        printk("Received DATA not destined for INCAIP2 MAC address, -> discard\n");
        printk("eth_hdr->h_dest= %02X:%02X:%02X:%02X:%02X:%02X\n", 
	           eth_hdr->h_dest[0], eth_hdr->h_dest[1],eth_hdr->h_dest[2],
               eth_hdr->h_dest[3],eth_hdr->h_dest[4],eth_hdr->h_dest[5]);
        printk("eth_hdr->h_source= %02X:%02X:%02X:%02X:%02X:%02X\n", 
	           eth_hdr->h_source[0], eth_hdr->h_source[1],eth_hdr->h_source[2],
               eth_hdr->h_source[3],eth_hdr->h_source[4],eth_hdr->h_source[5]);            
        printk("eth_hdr->h_proto= 0x%04X\n", eth_hdr->h_proto);
        sw_2_mbx_discard_cnt++;
/*	printk("                   discard_pck_cnt=%d\n", sw_2_mbx_discard_cnt);  */      
	    dev_kfree_skb(skb);  
        return -1;        
    }
#endif    
//   printk("%s - received %d bytes\n", __FUNCTION__, skb->len);
    
    /* #ifdef USE_MAILBOX  */    
    if (loop == BIG_LOOP)
    {
        net_dev_ptr = skb->dev;

	/*    printk("%s - memset\n", __FUNCTION__);  */
	    memset(&rw, 0, sizeof(mps_message));
	/*    printk("%s - kmalloc\n", __FUNCTION__);  */    
#if 1
	/* allocate memory for received data */
	rw.pData = (u8*)kmalloc(skb->len, GFP_ATOMIC);  
	/* 
	 * allocate buffer for received message and copy data 
	 * into allocated area
	 */
	if( rw.pData == NULL )
    {
	    printk("%s - memory allocation failed !\n",__FUNCTION__);
	    return -1;
        }
	/*    printk("%s - memcpy\n", __FUNCTION__);  */      
	memcpy((u8*)rw.pData, (u8*) skb->data, skb->len);
#endif
/*    rw.pData          = (u8*)PHYSADDR((u8*) skb->data);  */  
	rw.nDataBytes     = skb->len;
	rw.RTP_PaylOffset = 0xFFFFFFFF;
	rw.cmd_type       = CMD_RTP_VOICE_DATA_PACKET;
    

	/*           
		     printk("%s: %d bytes to be sent to mailbox\n",__FUNCTION__, rw.nDataBytes);
		     dump_data( (u8*)(KSEG0ADDR(rw.pData)), rw.nDataBytes); 
 
		     printk("%s -> ifx_mps_write_mailbox(%d bytes)\n",
		     __FUNCTION__,rw.nDataBytes);        
	*/ 
        while( (ifx_mps_write_mailbox(voice0, &rw) != OK) && (i++ < 10));
        if( i>=10 )
        {
            sw_2_mbx_discard_cnt++;
            kfree(KSEG0ADDR(rw.pData));
            dev_kfree_skb(skb);                   
            INCAIP2_IKOS_EMSG("ifx_mps_write_mailbox failed 10 times\n");  
            while(1);
        }
        else
        {
            sw_2_mbx_tx_cnt++;
            dev_kfree_skb(skb);       
        }
    
    }
    else if (loop == SMALL_LOOP)
    {
	    /* change source and dest address */
    	/*        printk("received data packet directed to INCAIP2 -> loop the packet\n");  */
    	memcpy(tmp_src, eth_hdr->h_source, 6);
    	memcpy(eth_hdr->h_source, eth_hdr->h_dest, 6); 
    	memcpy(eth_hdr->h_dest, tmp_src, 6); 
    	ifx_switch_tx(skb, &switch_devs[0]);
    }    
    else if (loop == LOOP_LOOP)
    {
        struct sk_buff* nskb = skb_copy(skb, GFP_ATOMIC);
        dev_kfree_skb(skb);
	    /* change source and dest address */
    	/*        printk("received data packet directed to INCAIP2 -> loop the packet\n");  */
        eth_hdr->h_dest[0]++;
    	ifx_switch_tx(nskb, &switch_devs[0]);
        mbx_2_switch_cnt++;
    }    
   
    return 0;
}


#define SEND_CHAN 1
//#define SEND_SIZE 100
#define SEND_SIZE 84
void incaip2_mbx_send_data_message(void)
{
    mps_message  wrt_data[4];
    int i, j;

    for(j = 0; j < SEND_CHAN; j++)
    {
	wrt_data[j].pData          = (u8*) kmalloc(SEND_SIZE, GFP_ATOMIC);
	wrt_data[j].nDataBytes     = SEND_SIZE;
	wrt_data[j].cmd_type       = CMD_RTP_VOICE_DATA_PACKET;
	wrt_data[j].RTP_PaylOffset = 0x00000000;
#if 0
	for(i = 0; i < (SEND_SIZE / 4); i++)
	    *((u32*) KSEG1ADDR((wrt_data[j].pData+(i<<2)))) = 0xdeadbeef;
#else
   *((u32*) KSEG1ADDR((wrt_data[j].pData +  0))) = 0x00000000;
   *((u32*) KSEG1ADDR((wrt_data[j].pData +  4))) = 0x00010020;
   *((u32*) KSEG1ADDR((wrt_data[j].pData +  8))) = 0xDA862374;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 12))) = 0x81001112;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 16))) = 0x13141516;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 20))) = 0x1718191A;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 24))) = 0x1B1C1D1E;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 28))) = 0x1F202122;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 32))) = 0x23242526;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 36))) = 0x2728292A;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 40))) = 0x2B2C2D2E;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 44))) = 0x2F303132;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 48))) = 0x33343536;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 52))) = 0x3738393A;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 56))) = 0x3B3C3D3E;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 60))) = 0x3F404142;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 64))) = 0x43444546;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 68))) = 0x4748494A;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 72))) = 0x4B4C4D4E;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 76))) = 0x4F50DEAD;
   *((u32*) KSEG1ADDR((wrt_data[j].pData + 80))) = 0xBEEFD0F0;
#endif
    }
    INCAIP2_IKOS_DMSG("TX voice (0x%08x)\n",KSEG0ADDR(wrt_data[0].pData));
    ifx_mps_write_mailbox( voice0, &wrt_data[0] );
}




/* Brief:
 * mailbox receive callback function to read data from upstream mailbox
 * and write it to ethernet switch
 *
 *
 */
void incaip2_mbx_rcv_command_callback( MbxMessageType_e type )
{
    mps_message rw;
    static int count = 0;

/*	INCAIP2_IKOS_DMSG("Yippieh! %08d\n", count++);  */
    printk("%s - count=%d\n", __FUNCTION__, count++);  
    memset(&rw, 0, sizeof(mps_message));
    if( ifx_mps_read_mailbox( type, &rw ) != OK )
    {
	INCAIP2_IKOS_EMSG("ifx_mps_read_mailbox fails\n");
	return;
    }
    
    printk("%s - received %d bytes from mailbox\n", __FUNCTION__, rw.nDataBytes);
    ifx_print_mps_message( &rw );     
    
    
    switch (rw.cmd_type)
    {
    case CMD_ALI_PACKET:
	printk("%s - received CMD_ALI_PACKET\n", __FUNCTION__);        
	break;
    case CMD_COP_PACKET:
	printk("%s - received CMD_COP_PACKET\n", __FUNCTION__);        
	break;
    case CMD_EOP_PACKET:
	printk("%s - received CMD_EOP_PACKET\n", __FUNCTION__);        
	break;            

    default:
	printk("%s - received unknown packet (%x)\n", __FUNCTION__, rw.cmd_type);
	break;
    }

    if (((u32*)rw.pData)[0] == 0x81000A10)
    {
	((u32*)rw.pData)[0] = 0x01000A10; /* send write command */
	((u32*)rw.pData)[1] |= 0x00010000; /* set debug */
	((u32*)rw.pData)[3] |= 0x00000040; /* enable loop */
	if( ifx_mps_write_mailbox( command, &rw ) != OK )
	{
	    printk("Writing DBG Command write Message failed !\n");
	    return ERROR;
	}
	ifx_print_mps_message( &rw );     
    }

    kfree(KSEG0ADDR(rw.pData));   
    return;
}




/* Brief:
 * mailbox receive callback function to read data from upstream mailbox
 * and write it to ethernet switch
 *
 *
 */
void incaip2_mbx_rcv_data_callback( mps_devices type )
{
    mps_message rw;
    int i;
/*
static s32 count = 0;
    static s32 rtp_voice_data_count=0;
    
    static s32 cmd_address_count=0;
*/
/*	INCAIP2_IKOS_DMSG("Yippieh! %08d\n", count++);  */

    mbx_rcv_cnt++; 

    memset(&rw, 0, sizeof(mps_message));
    if( ifx_mps_read_mailbox( type, &rw ) != OK )
    {
	    INCAIP2_IKOS_EMSG("ifx_mps_read_mailbox fails\n");
	    return;
    }
/*       
	 printk("%s - received %d bytes from mailbox\n", __FUNCTION__, rw.nDataBytes);
	 ifx_print_mps_message( &rw );  
*/   
    
    
    switch (rw.cmd_type)
    {
    case CMD_RTP_VOICE_DATA_PACKET:
	/*
	 *
	 */    

	    mbx_voice_data_rcv_cnt++;
/*      if( rtp_voice_data_count % 10 == 0)   
            printk("%s - %d packets received\n", __FUNCTION__,rtp_voice_data_count); 
*/        
        if (loop == BIG_LOOP)
        {
	        if( incaip2_mbx_to_sw( &rw ) )
	        {
	            INCAIP2_IKOS_EMSG("%s - Sending data to switch fails\n", __FUNCTION__);
	            kfree(KSEG0ADDR(rw.pData));                
	            return;
	        }
            else
            {   
                if ((mbx_voice_data_rcv_cnt % 2000) == 0)
                    printk("\ro");
                else if ((mbx_voice_data_rcv_cnt % 1000) == 0)
                    printk("\rO");
            }
        }
        else if (loop == MBX_LOOP)
        {
            while( (ifx_mps_write_mailbox(voice0, &rw) != OK) && (i++ < 10));
            if( i>=10 )
            {
                INCAIP2_IKOS_EMSG("ifx_mps_write_mailbox failed 10 times\n");  
                while(1);
            }
            else
            {   
                if ((mbx_voice_data_rcv_cnt % 2000) == 0)
                    printk("\ro");
                else if ((mbx_voice_data_rcv_cnt % 1000) == 0)
                    printk("\rO");
            }
        }
        else if (loop == LOOP_LOOP)
        {
            static int didit = 0;

            if (!didit)
            {
               printk("Injecting packet to Eth...\n");
               incaip2_mbx_to_sw(&rw);
               didit = 1;
            }
            else
            {
                while( (ifx_mps_write_mailbox(voice0, &rw) != OK) && (i++ < 10));
                if( i>=10 )
                {
                    INCAIP2_IKOS_EMSG("ifx_mps_write_mailbox failed 10 times\n");  
                    while(1);
                }
                else
                {   
                    if ((mbx_voice_data_rcv_cnt % 2000) == 0)
                        printk("\ro");
                    else if ((mbx_voice_data_rcv_cnt % 1000) == 0)
                        printk("\rO");
                    sw_2_mbx_tx_cnt++;
                }
            }
        }

	    break;
    case CMD_ADDRESS_PACKET:
//		   INCAIP2_IKOS_DMSG("release address (0x%08x)\n",KSEG0ADDR(rw.pData));
	    cmd_address_count++;
/*        if( cmd_address_count % 10 == 0)  */ 
/*  
            printk("%s - %d CMD_ADDRESS_PACKETs received\n", __FUNCTION__, cmd_address_count);
*/            
    	kfree(KSEG0ADDR(rw.pData));
        break;
    case CMD_VOICEREC_STATUS_PACKET:
    case CMD_VOICEREC_DATA_PACKET:
    case CMD_RTP_EVENT_PACKET:
    case CMD_FAX_DATA_PACKET:
    case CMD_FAX_STATUS_PACKET:
    case CMD_P_PHONE_DATA_PACKET:
    case CMD_P_PHONE_STATUS_PACKET:
    case CMD_CID_DATA_PACKET:
	    printk("%s - unexpected packet (%x)\n",__FUNCTION__, rw.cmd_type);
	    INCAIP2_IKOS_EMSG("unexpected packet (%x)\n",rw.cmd_type);
	    dump_data(rw.pData, rw.nDataBytes);
	    break;
    default:
	    printk("%s - received unknown packet (%x)\n", __FUNCTION__, rw.cmd_type);
	    dump_data(rw.pData, rw.nDataBytes);            
	break;
    }
    return;
}



/* Brief: passes data received from mailbox to switch
 * The function needs to be registered as
 * mailbox upstream data callback handler in advance
 *
 *
 *
 */

s32 incaip2_mbx_to_sw( mps_message *rw )
{
    s32 i;
    unsigned char tmp_src[6] = { 0x00, 0x20, 0xda, 0x86, 0x23, 0x74};
    unsigned char tmp_dst[6] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x01};    
    unsigned char tmp_proto[2] = { 0x81, 0x00 };    
    struct ethhdr *eth_hdr;
/*    static s32 count=0;  */

    mbx_rcv_cnt++;
//    g_sw_send_skb = dev_alloc_skb(rw->nDataBytes + 14);
    g_sw_send_skb = dev_alloc_skb(rw->nDataBytes);
    if (g_sw_send_skb == NULL)
    {
	INCAIP2_IKOS_EMSG("no memory for skb\n");
	printk("no memory for skb\n");		
	return -ENOMEM;
    }

/*
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after dev_alloc_skb !!!\n", __FUNCTION__);
    }    
*/    
/*
	if(count % 100 == 0)        
        printk("%s - packet count=%d)\n",__FUNCTION__,count);    
*/
/*    printk("%s -> skb_put\n",__FUNCTION__);  */  
    skb_put(g_sw_send_skb,rw->nDataBytes + 14);   
//    skb_put(g_sw_send_skb,rw->nDataBytes);   
    
/*    
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after skb_put !!!\n", __FUNCTION__);
    }
*/    
/*    printk("%s -> copy data to skb\n",__FUNCTION__);  */  
    memcpy( (u8*)g_sw_send_skb->data +14, (u8*)KSEG0ADDR(rw->pData), rw->nDataBytes);
//    memcpy( (u8*)g_sw_send_skb->data, (u8*)KSEG0ADDR(rw->pData), rw->nDataBytes);
/*
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after memcpy 1 !!!\n", __FUNCTION__);
    }   
*/    
    
/*    printk("%s - g_sw_send_skb after dev_alloc_skb\n", __FUNCTION__);  
      dump_skb(g_sw_send_skb);
      printk("%s - rw as received\n", __FUNCTION__);
      printk("rw->pData=0x%08X\n", rw->pData);
      printk("rw->nDataBytes=%d\n", rw->nDataBytes);
      printk("data bytes\n");    
      dump_data( (u8*)KSEG0ADDR(rw->pData), rw->nDataBytes); 
*/

/*	skb_put(g_sw_send_skb, 2);  */  
/*    printk("%s -> memcpy((u8*)g_sw_send_skb->data+12, (u8*)tmp_proto, 2);\n",__FUNCTION__);  */  
    memcpy((u8*)g_sw_send_skb->data+12, (u8*)tmp_proto, 2);
/*
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after memcpy 1 !!!\n", __FUNCTION__);
    }      
*/    
/*	skb_put(g_sw_send_skb, 6);  */     
/*    printk("%s -> memcpy((u8*)g_sw_send_skb->data+6, (u8*)tmp_src, 6);\n",__FUNCTION__);  */  
    memcpy((u8*)g_sw_send_skb->data+6, (u8*)tmp_src, 6);
/*
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after memcpy 2 !!!\n", __FUNCTION__);
    }  
*/    
/*	skb_put(g_sw_send_skb, 6);  */   
/*    printk("%s -> memcpy((u8*)g_sw_send_skb->data, (u8*)tmp_dst, 6);\n",__FUNCTION__);  */  
    memcpy((u8*)g_sw_send_skb->data, (u8*)tmp_dst, 6);
/*
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after memcpy 3 !!!\n", __FUNCTION__);
    }      
*/
/*    
      eth_hdr = (struct ethhdr*) (g_sw_send_skb->data);      
      printk("eth_hdr->h_dest= %02X:%02X:%02X:%02X:%02X:%02X\n", 
      eth_hdr->h_dest[0], eth_hdr->h_dest[1],eth_hdr->h_dest[2],
      eth_hdr->h_dest[3],eth_hdr->h_dest[4],eth_hdr->h_dest[5]);
      printk("eth_hdr->h_source= %02X:%02X:%02X:%02X:%02X:%02X\n", 
      eth_hdr->h_source[0], eth_hdr->h_source[1],eth_hdr->h_source[2],
      eth_hdr->h_source[3],eth_hdr->h_source[4],eth_hdr->h_source[5]);            
      printk("eth_hdr->h_proto= 0x%04X\n", eth_hdr->h_proto);  
*/
/*    printk("%s -> ifx_switch_tx(g_sw_send_skb, &switch_devs[0]);     \n",__FUNCTION__);  */  

    if( ifx_switch_tx(g_sw_send_skb, &switch_devs[0]) != OK)
    {
        printk("ifx_switch_tx failed !\n");
        return -1;
    }
/*    
    if (skb_shinfo(g_sw_send_skb)->frag_list)
    {
            printk("%s - packet is fragmented after memcpy 3 !!!\n", __FUNCTION__);
    }
*/    
    kfree(KSEG0ADDR(rw->pData));    
    mbx_2_switch_cnt++;
    /*dev_kfree_skb(g_sw_send_skb);  */  
/*    if(count++ % 10 == 0)  */

    return 0;   
    
    
    
    
#if 0           
    if((eth_hdr->h_dest[0] == 0x00)  &&  (eth_hdr->h_dest[1] == 0x20 ) && 
       (eth_hdr->h_dest[2] == 0xda)  &&  (eth_hdr->h_dest[3] == 0x86 ) &&
       (eth_hdr->h_dest[4] == 0x23)  &&  (eth_hdr->h_dest[5] == 0x74 ))
    {

        /* change source and dest address */
/*        printk("received data packet directed to INCAIP2 -> loop the packet\n");  */

        memcpy(tmp_src, eth_hdr->h_source, 6);
        memcpy(eth_hdr->h_source, eth_hdr->h_dest, 6); 
        memcpy(eth_hdr->h_dest, tmp_src, 6); 
/*
  printk("%s: %d bytes to be sent to switch\n",__FUNCTION__, g_sw_send_skb->len);
  dump_data( (u8*)(KSEG0ADDR(g_sw_send_skb->data)), g_sw_send_skb->len); 
*/            
        
/*        printk("%s -> ifx_switch_tx\n",__FUNCTION__);  */
        ifx_switch_tx(g_sw_send_skb, &switch_devs[0]);        
    }
    else
    {
        printk("%s -> invalid packet received from mailbox\n",__FUNCTION__);
/*
  printk("eth_hdr->h_dest= %02X:%02X:%02X:%02X:%02X:%02X\n", 
  eth_hdr->h_dest[0], eth_hdr->h_dest[1],eth_hdr->h_dest[2],
  eth_hdr->h_dest[3],eth_hdr->h_dest[4],eth_hdr->h_dest[5]);
  printk("eth_hdr->h_source= %02X:%02X:%02X:%02X:%02X:%02X\n", 
  eth_hdr->h_source[0], eth_hdr->h_source[1],eth_hdr->h_source[2],
  eth_hdr->h_source[3],eth_hdr->h_source[4],eth_hdr->h_source[5]);            
  printk("eth_hdr->h_proto= 0x%04X\n", eth_hdr->h_proto);  
*/
        dump_data( (u8*)(KSEG0ADDR(g_sw_send_skb->data)), g_sw_send_skb->len); 
                  
        kfree(KSEG0ADDR(rw->pData));
    
	return -1;
    }
#endif

}




void print_data_flow_counters( void )
{
	printk("Received from DMA: %d\n", sw_2_mbx_rx_cnt);
	printk("Sent to Mbx:       %d\n", sw_2_mbx_tx_cnt);	
	printk("Discarded:         %d\n", sw_2_mbx_discard_cnt);	
	printk("Received from Mbx: %d\n", mbx_rcv_cnt);	
	printk("Data from Mbx:     %d\n", mbx_voice_data_rcv_cnt);    
	printk("Release from Mbx:  %d\n", cmd_address_count);     
	printk("Sent to DMA:       %d\n", mbx_2_switch_cnt);	

 
}

void PrintCommands( void )
{

    printk("The following commands are available\n");
    printk("q, Q : quit\n");    
    printk("c, C : Print MIB counters\n");    
    printk("m, M : Print MAC Table\n");  
    printk("b : mailbox pointers\n");     
    printk("w, W : write address\n");       
    printk("r, R : read address\n");      
/*    INCAIP2_IKOS_DMSG("h, H : Calculate Hash Index for MAC address 00:20:da:86:23:74\n");              
      INCAIP2_IKOS_DMSG("a, A : Add MAC Address 00:20:da:86:23:74 at pos. 25\n");
*/    
    printk("P : Enable PCM Module\n"); 
    printk("p : Read PCM Module Status\n");     
    printk("X : Enable PCM Channel 0\n"); 
    printk("x : Read PCM Channel 0 Status\n");     
    printk("Z : Coder Channel Speech Command\n"); 
    printk("z : Read Coder Channel Speech Status\n");         
    printk("A : Configure Coder Channel RTP Support\n"); 
    printk("a : Read Coder Channel RTP Support Status\n");     
/* !!! LM  */
    printk("E : Enable AL Channel 0\n"); 
    printk("e : Read AL Channel 0 Status\n");     
    printk("Y : Send OPMODE_CMD\n");     
    printk("y : Read current OPMODE\n");     
    printk("B : Start ALM Loop\n");     
    printk("F : Burst single packet to coder0\n");
    printk("L : Send Audio Channel Loop Command\n");   
    printk("f : Print Data Flow Counters\n");    
/*    printk("u : Print Upstream Buffer Level and State\n");  */        
    
/* !!! LM  */    
    printk("0...3: Print Switch Module's rx channel descriptor list \n");
    printk("4...7: Print Switch Module's tx channel descriptor list \n");    
    printk("d, D : Print DMA Statistics\n");   
    printk("g, G : Print Logical DMA Channel list\n"); 
    printk("t, T : Print tx_chan_list\n"); 
    printk("s, S : Print rx_chan_list\n");     
    printk("l :    Change Loop Mode\n");                
    printk("Current loop mode %i\n", loop);
}

/* Brief:	switch to mailbox and vice versa bridge
 * Parameter:
 * Return:	0 ok
 *		-EFAULT
 */
int do_incaip2_sw_mbx_bridge( cmd_tbl_t * cmdtp, int argc, char *argv[])
{
    u8 len = 80;
    int i;
    char c;
    int ret=0;
    bool_t keypressed = FALSE;
    u32 switch_p0_gmac;    
    u32 switch_p1_gmac;    
    u8 testmac[]="0020da862374";
    
    
    /*
     * Init switch and DMA for communication
     */
    if ((ret=ifx_switch_init_module()))
    {
	INCAIP2_IKOS_EMSG("switch_init_module fails\n");
	goto do_sw_lp_exit;  
    }

    if ((ret=ifx_switch_open(&switch_devs[0])))
    {
	INCAIP2_IKOS_EMSG("ifx_switch_open eth0 fails\n");
	printk("%s - ifx_switch_open eth0 fails\n", __FUNCTION__);
	ifx_switch_cleanup();
	goto do_sw_lp_exit;
    }

    if ((ret=ifx_switch_open(&switch_devs[1])))
    {
	INCAIP2_IKOS_EMSG("ifx_switch_open eth1 fails\n");
	printk("%s - ifx_switch_open eth1 fails\n", __FUNCTION__);        
	ifx_switch_release(&switch_devs[0]);
	ifx_switch_cleanup();
	goto do_sw_lp_exit;
    }

    INCAIP2_IKOS_DMSG("switch config ok\n");
    /*	loopback_mode = SWITCH_PORT_LOOPBACK;  */


    /* #ifdef USE_MAILBOX */
    /*
     * Init MPS/mailbox for communication
     */
    if( ifx_mps_init_module() )
    {
	INCAIP2_IKOS_EMSG("ifx_mps_init_module fails\n");
	printk("ifx_mps_init_module fails\n");
	goto do_sw_lp_exit;
    }
    /*
      voice0_inode_ptr = &voice0_inode;
      voice0_inode_ptr = (struct inode*) voice0;
    */    
    if( ifx_mps_open( (struct inode *)voice0, &voice0_file ) != OK )
	/*       voice0_inode_ptr, &voice0_file ) != OK )  */
    {
	INCAIP2_IKOS_EMSG("ifx_mps_open fails\n");
	printk("ifx_mps_open fails\n");
	goto do_sw_lp_exit;
    }

    if( ifx_mps_open( (struct inode *)command, &command_file ) != OK )
	/*       voice0_inode_ptr, &voice0_file ) != OK )  */
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Open fails for command\n");
	printk("INCAIP2_MPS_DSP_Open fails\n");
	goto do_sw_lp_exit;
    }    
    
    
    if( ifx_mps_register_data_callback( voice0,
					1 /* upstream */,
					incaip2_mbx_rcv_data_callback) != OK )
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_RegisterDataCallback fails\n");
	printk("INCAIP2_MPS_DSP_RegisterDataCallback fails\n");
	goto do_sw_lp_exit;
    }
    
    if( ifx_mps_register_data_callback( command,
					1 /* upstream */,
					incaip2_mbx_rcv_command_callback) != OK )
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_RegisterDataCallback fails\n");
	printk("INCAIP2_MPS_DSP_RegisterDataCallback fails\n");
	goto do_sw_lp_exit;
    }    
    
    
    /* #endif  */


    INCAIP2_IKOS_EMSG("Switch to Mailbox bridging started ...\n");
    
    /*
     * all further streaming actions are interrupt driven ->
     * Callback functions perform the data transfer, so the main loop
     * waits for user input (abort or print requests)
     */
    PrintCommands();
    do
    {
	/*		printk("%s -> Check keypressed\n",__FUNCTION__);  */
	if(keypressed)
        {
	    printk("%s -> PrintCommands\n",__FUNCTION__);
	    PrintCommands();  
	    keypressed = FALSE;
        }
	c = getDebugChar();
	if (c == 'q' || c == 'Q')
	{
	    INCAIP2_IKOS_EMSG("terminated by user\n");
	    keypressed = TRUE;
	    break;
	}
	else if (c == 'c' || c == 'C')
	{
	    INCAIP2_Switch_Print_MIB_Counters();
	    keypressed = TRUE;
	}
	else if(c == 'm' || c == 'M')
	{
	    INCAIP2_Switch_Print_MAC_Table();
	    keypressed = TRUE;
	}
	else if (c == 'w' || c == 'W')
	{
	    /* write value to address */
	    unsigned int addr, val;
	    char string[9];
	    printk("Address: 0x");
	    for (i = 0; i < 8; i++)
	    {
		string[i] = getDebugChar();
		putDebugChar(string[i]);
	    }   
	    string[9] = 0;
	    sscanf(string, "%08x", &addr);
	    printk("Value: 0x");
	    for (i = 0; i < 8; i++)
	    {
		string[i] = getDebugChar();
		putDebugChar(string[i]);
	    }   
	    string[9] = 0;
	    sscanf(string, "%08x", &val);
	    *((volatile unsigned int *)addr) = val;
	    printk("\n");
	    keypressed = TRUE;         
	}
	else if (c == 'r' || c == 'R')
	{
	    /* read value from address */
	    unsigned int addr, val;
	    char string[9];
	    printk("Address: 0x");
	    for (i = 0; i < 8; i++)
	    {
		string[i] = getDebugChar();
		putDebugChar(string[i]);
	    }   
	    string[9] = 0;
	    sscanf(string, "%08x", &addr);
	    printk("\nValue: 0x%08x\n", *((volatile unsigned int *)addr));
	    keypressed = TRUE;
	}
#if 0
	else if (c == 'h' || c == 'H')
	{

	    printk("MAC Address: 0020da862374-> Hash Index:0x%02X\n", 
		   crc8(testmac, strlen(testmac)) );
	    keypressed = TRUE;
	}        

	else if (c == 'a' || c == 'A')
	{

	    printk("Add MAC Address 0020da862374 at pos. 25\n");
	    ifx_add_MAC_table_entry_idx( 0, /* chipid = 0 */
					 25,
					 testmac,
					 FALSE,
					 FALSE, 
					 TRUE, 
					 TRUE,
					 STATIC );            
            
	    keypressed = TRUE;
	}    
#endif
	else if (c == 'd' || c == 'D')
	{
	    /* display complete DMA information */
	    dma_print_statistics( TRUE, TRUE, TRUE );     
	    keypressed = TRUE;
	}             
	else if (c == 'g' || c == 'G')
	{
	    print_g_log_channel_list(  );   
	    keypressed = TRUE;
	}      
	else if (c == '0')
	{
	    print_channel_desc_list( 0 );    
	    keypressed = TRUE;
	}     
	else if (c == '1')
	{
	    print_channel_desc_list( 1 );    
	    keypressed = TRUE;
	}   
	else if (c == '2')
	{
	    print_channel_desc_list( 2 );    
	    keypressed = TRUE;
	}     
	else if (c == '3')
	{
	    print_channel_desc_list( 3 );    
	    keypressed = TRUE;
	} 
	else if (c == '4')
	{
	    print_channel_desc_list( 10 );    
	    keypressed = TRUE;
	}     
	else if (c == '5')
	{
	    print_channel_desc_list( 11 );    
	    keypressed = TRUE;
	}   
	else if (c == '6')
	{
	    print_channel_desc_list( 12 );    
	    keypressed = TRUE;
	}     
	else if (c == '7')
	{
	    print_channel_desc_list( 13 );    
	    keypressed = TRUE;
	}     
	else if (c == 'l' )
	{
        loop = (loop + 1) % 3;
        printk("Change Loop Mode to %i\n", ++loop);
        switch (loop) {
        case BIG_LOOP:
            *(volatile u32*)0xb8008070 = 0x00000000; /* PMAC_HD_CTL */
            break;
        case SMALL_LOOP:
            *(volatile u32*)0xb8008070 = 0x00040000; /* PMAC_HD_CTL */
            break;
        case LOOP_LOOP:
        {
            u8 tmpmac[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x01 };
            *(volatile u32*)0xb8008014 = 0x000001B6; /* UN_DEST */
            *(volatile u32*)0xb8008070 = 0x00040000; /* PMAC_HD_CTL */
            ifx_add_MAC_table_entry(tmpmac, FALSE, FALSE, TRUE, TRUE, 0);
            break;
        }
        }    
	} 
    else if (c == '*')
	{
        INCAIP2_IKOS_DMSG("Packets: 0x%08x\n", mbx_voice_data_rcv_cnt);
    }
    else if (c == 'S')
	{
         /* send message to voice CPU */
         INCAIP2_IKOS_DMSG("Send out data message...\n");
         incaip2_mbx_send_data_message();
    }
	else if (c == 's')
	{
	    dma_print_rx_chan_list();
	    keypressed = TRUE;
	}        
	else if (c == 't' || c == 'T')
	{
	    dma_print_tx_chan_list();
	    keypressed = TRUE;
	}                
	else if (c == 'b')
	{
	    /* send message to voice CPU */
	    INCAIP2_IKOS_DMSG("Debug mailbox...\n");
	    ifx_mps_debug_mailbox();
	    keypressed = TRUE;         
	}   
	else if (c == 'P')
	{
	    printk("Sending PCM Ctrl Write\n");
	    if(INCAIP2_MPS_DSP_Send_PCMCtrl_Command( WR, ENABLE ) != OK)
	    {
		printk("Sending PCM Ctrl Write fails\n");
		/*                INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_PCMCtrl_Command fails\n");  */
	    }
	    else
	    {
		asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");			
		printk("Sending PCM Ctrl Write successful\n");
		asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");				
	    }
	    asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");
	    keypressed = TRUE;   
	    asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop;");            
	}
	else if(c == 'p')
	{
	    printk("Sending PCM Ctrl Read \n");
	    if(INCAIP2_MPS_DSP_Send_PCMCtrl_Command( RD, ENABLE ) != OK)
		printk("INCAIP2_MPS_DSP_Send_PCMCtrl_Command fails \n");
	    /*                INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_PCMCtrl_Command fails\n");  */
	    keypressed = TRUE;
	}
	else if (c == 'x')
	{
	    printk("Sending PCM Chan Ctrl Read \n");   
	    if( INCAIP2_MPS_DSP_Send_PCMChan_Command( RD, 
						      0, 
						      ENABLE,
						      inp_sig_array, 
						      0,   /* xts,  */ 
						      0   /* rts  */ )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_PCMChan_Command fails\n");
	    keypressed = TRUE;                     
	}
	else if(c == 'X')  
	{
	    printk("Sending PCM Chan Ctrl Write\n");                      
	    if( INCAIP2_MPS_DSP_Send_PCMChan_Command( WR, 
						      0, 
						      ENABLE,
						      inp_sig_array, 
						      0,   /* xts,  */ 
						      0   /* rts  */ )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_PCMChan_Command fails\n");
	    keypressed = TRUE;                     
	}

	/* !!! LM: commands for ALM looping */                 
	else if(c == 'Y')  
	{
	    printk("Send OPMODE_CMD\n");                      
	    if( INCAIP2_MPS_DSP_send_OPMODE_CMD()  != OK )
		INCAIP2_IKOS_EMSG("send OPMODE failed\n");
	    keypressed = TRUE;                     
	}                 
	else if(c == 'y')  
	{
	    printk("read OPMODE\n");                      
	    if( INCAIP2_MPS_DSP_read_cur_alm_state()  != OK )
		INCAIP2_IKOS_EMSG("read OPMODE failed\n");
	    keypressed = TRUE;                     
	}                 
	else if(c == 'B')  
	{
	    printk("Start ALM loop\n");                      
	    if( INCAIP2_MPS_DSP_Start_Loop() != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_DCCTLDBG_Command failed\n");
	    keypressed = TRUE;                     
	}                 
	else if(c == 'F')  
	{
	    printk("Sending test packet\n");                      
	    if( incaip2_send_pck_to_coder0()  != OK )
		INCAIP2_IKOS_EMSG("send packet failed\n");
	    keypressed = TRUE;                     
	}                 
	else if (c == 'e')
	{
	    printk("Sending AL Chan Ctrl Read \n");   
	    if( INCAIP2_MPS_DSP_Send_AL_Chan_Command( RD, /* read */
						      0, /* channel number */
						      ENABLE, /* enable */
						      inp_sig_array  )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_AL_Chan_Command fails\n");
	    keypressed = TRUE;                     
	}
	else if(c == 'E')  
	{
	    printk("Sending AL Chan Ctrl Write\n");   
	    inp_sig_array[0] = 4;
	    if( INCAIP2_MPS_DSP_Send_AL_Chan_Command( WR, /* write */
						      0, /* channel number */
						      ENABLE, /* enable */
						      inp_sig_array )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_AL_Chan_Command fails\n");
	    inp_sig_array[0] = 0;
	    keypressed = TRUE;                     
	}                 
	/* !!! LM: commands for ALM Looping*/
	else if (c == 'Z')
	{
	    printk("Sending PCM Coder Chan Speech Compr Write \n");                                                     
	    if( INCAIP2_MPS_DSP_Send_CoderChanSpeechCompr_Command( WR, 
								   0, 
								   ENABLE )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_CoderChanSpeechCompr_Command fails\n");   
	    keypressed = TRUE;         
	}
	else if (c == 'z')
	{
	    printk("Sending PCM Coder Chan Speech Compr Read \n");                                               
	    if( INCAIP2_MPS_DSP_Send_CoderChanSpeechCompr_Command( RD, 
								   0, 
								   ENABLE )  != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_CoderChanSpeechCompr_Command fails\n");               
	    keypressed = TRUE;         
	} 
	else if (c == 'A')
	{
	    printk("Sending Coder Channel RTP Support Write \n");                                                     
	    if( INCAIP2_MPS_DSP_Send_CoderChanRTPSupport( WR, 0 ) != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_CoderChanRTPSupport fails\n");   
	    keypressed = TRUE;         
	}
	else if (c == 'a')
	{
	    printk("Sending Coder Channel RTP Support Read \n");                                           
	    if( INCAIP2_MPS_DSP_Send_CoderChanRTPSupport( RD, 0 ) != OK )
		INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Send_CoderChanRTPSupport fails\n");               
	    keypressed = TRUE;         
	}         
	else if (c == 'L')
	{
	    printk("Sending Audio Channel Loop Command\n");                                           
	    if( INCAIP2_MPS_DSP_send_Audio_Channel_Loop_CMD( ) != OK )
		INCAIP2_IKOS_EMSG("Sending Audio Channel Loop Command fails\n");               
	    keypressed = TRUE;         
	}         
    
        else 
		if (c == 'f' )
		{
			printk("Packet Counters \n");         
			print_data_flow_counters();     
//            keypressed = TRUE;         
        } 
/*         
        else
		if (c == 'u' )
		{
			printk("Upstream Buffers at VCPU \n");         
			printk("Buffer Level: %d \n",ifx_mps_get_buffer_level());   
			printk("Buffer State: ");
			switch(ifx_mps_get_buffer_state())
			{
				case MPS_BUF_UNINITIALIZED:
					printk("MPS_BUF_UNINITIALIZED\n");
					break;
				case MPS_BUF_EMPTY:
					printk("MPS_BUF_EMPTY\n");
					break;
				case MPS_BUF_LOW:
					printk("MPS_BUF_LOW\n");
					break;
				case MPS_BUF_OK:
					printk("MPS_BUF_OK\n");
					break;
				case MPS_BUF_OV:
					printk("MPS_BUF_OV\n");
					break;
				case MPS_BUF_ERR:
					printk("MPS_BUF_ERR\n");
					break;

			}
            keypressed = TRUE;         
        }      
*/
    
    
    } while(1);

    ifx_switch_release(&switch_devs[0]);
    ifx_switch_release(&switch_devs[1]);  
    ifx_switch_cleanup();
    
    /*#ifdef USE_MAILBOX  */
    if( ifx_mps_unregister_data_callback(voice0,
					 1 /* upstream */) !=	OK)
    {
	INCAIP2_IKOS_EMSG("ifx_mps_unregister_data_callback fails\n");
	printk("ifx_mps_unregister_data_callback fails\n");
	goto do_sw_lp_exit;
    }

    if( ifx_mps_unregister_data_callback(command,
					 1 /* upstream */) !=	OK)
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_UnregisterDataCallback fails for command channel\n");
	printk("INCAIP2_MPS_DSP_UnregisterDataCallback fails for command channel\n");
	goto do_sw_lp_exit;
    }
    
    if( ifx_mps_close( (struct inode *)voice0, &voice0_file ) != 0 )
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Close fails for voice channel 0\n");
	printk("INCAIP2_MPS_DSP_Close fails\n");
	goto do_sw_lp_exit;
    }
    if( ifx_mps_close( (struct inode *)command, &command_file ) != 0 )
    {
	INCAIP2_IKOS_EMSG("INCAIP2_MPS_DSP_Close fails for command channel\n");
	printk("INCAIP2_MPS_DSP_Close fails\n");
	goto do_sw_lp_exit;
    }    
    
    /* #endif  */
    
    
#if 0
    INCAIP2_IKOS_EMSG("Press 0 or 1 to send packet to respective port\n");
    INCAIP2_IKOS_EMSG("Ready to loop back packet from port 0 and port1\n");
    c = getDebugChar();
    if (c == 'q' || c == 'Q'){
	INCAIP2_IKOS_EMSG("terminated by user\n");
	break;
    }else if (c == '0' || c == '1'){
	g_sw_send_skb = dev_alloc_skb(len);
	if (g_sw_send_skb == NULL){
	    INCAIP2_IKOS_EMSG("no memory for skb\n");
	    ret = -ENOMEM;
	    goto do_sw_lp_exit;
	}
	for(i=0;i<len;i++){
	    g_sw_send_skb->data[i] = i;
	}
	skb_put(g_sw_send_skb,len);
	ifx_switch_tx(g_sw_send_skb, &switch_devs[(c-'0')]);
    }
};

ifx_switch_release(&switch_devs[0]);
ifx_switch_release(&switch_devs[1]);
ifx_switch_cleanup();
#endif
do_sw_lp_exit:
return ret;
}

#endif

void mbx_print_commands( void )
{
    INCAIP2_IKOS_DMSG("The following commands are available\n");
    INCAIP2_IKOS_DMSG("q, Q : quit\n");    
    INCAIP2_IKOS_DMSG("w, W : write address\n");       
    INCAIP2_IKOS_DMSG("r, R : read address\n");       
    INCAIP2_IKOS_DMSG("d, D : dump print area\n");       
    INCAIP2_IKOS_DMSG("s, S : send message\n");       
    INCAIP2_IKOS_DMSG("t, T : set trace level\n");       
    INCAIP2_IKOS_DMSG("m, M : mailbox pointers\n");       
}

/* Brief:	switch to mailbox and vice versa bridge
 * Parameter:
 * Return:	0 ok
 *		-EFAULT
 */
int do_incaip2_mbx_test( cmd_tbl_t * cmdtp, int argc, char *argv[])
{
	u8 len = 80;
	int i;
	char c;
	int ret=0;
    
	/*
	 * Init MPS/mailbox for communication
	 */
	if( ifx_mps_init_module() )
	{
		INCAIP2_IKOS_EMSG("ifx_mps_init_module fails\n");
		goto do_sw_lp_exit;
	}
	if( ifx_mps_open( (struct inode *)voice0, &voice0_file ) != OK )
	{
		INCAIP2_IKOS_EMSG("ifx_mps_open fails\n");
		goto do_sw_lp_exit;
	}

	if( ifx_mps_register_data_callback( voice0,
	                                          1 /* upstream */,
											  incaip2_mbx_rcv_data_callback) != OK )
	{
		INCAIP2_IKOS_EMSG("ifx_mps_register_data_callback fails\n");
		goto do_sw_lp_exit;
	}

   INCAIP2_IKOS_EMSG("Mailbox init done...\n");

	/*
	 * all further streaming actions are interrupt driven ->
	 * Callback functions perform the data transfer, so the main loop
	 * waits for user input (abort or print requests)
	 */
   mbx_print_commands();
	do
	{
      c = getDebugChar();
		if (c == 'q' || c == 'Q')
		{
			INCAIP2_IKOS_EMSG("terminated by user\n");
			break;
		}
		else
		if (c == 'w' || c == 'W')
		{
         /* write value to address */
         unsigned int addr, val;
         char string[9];
         printk("Address: 0x");
         for (i = 0; i < 8; i++)
         {
            string[i] = getDebugChar();
            putDebugChar(string[i]);
         }   
         string[9] = 0;
         sscanf(string, "%08x", &addr);
         printk("Value: 0x");
         for (i = 0; i < 8; i++)
         {
            string[i] = getDebugChar();
            putDebugChar(string[i]);
         }   
         string[9] = 0;
         sscanf(string, "%08x", &val);
         *((volatile unsigned int *)addr) = val;
         printk("\n");
		}
		if (c == 'r' || c == 'R')
		{
         /* read value from address */
         unsigned int addr, val;
         char string[9];
         printk("Address: 0x");
         for (i = 0; i < 8; i++)
         {
            string[i] = getDebugChar();
            putDebugChar(string[i]);
         }   
         string[9] = 0;
         sscanf(string, "%08x", &addr);
         printk("\nValue: 0x%08x\n", *((volatile unsigned int *)addr));
		}
		if (c == 'd' || c == 'D')
		{
         INCAIP2_IKOS_EMSG("Print VCPU debug string:\n");
         INCAIP2_IKOS_EMSG("%s\n", (char*)0xA0E0EC00);
         INCAIP2_IKOS_EMSG("EOM\n");
		}
		if (c == 's' || c == 'S')
		{
         /* send message to voice CPU */
         INCAIP2_IKOS_DMSG("Send out data message...\n");
         incaip2_mbx_send_data_message();
      }
		if (c == 't' || c == 'T')
		{
         char level;
         /* send message to voice CPU */
         INCAIP2_IKOS_DMSG("new level: ");
         level = getDebugChar();
         printk("\n");
      }
		if (c == 'm' || c == 'M')
		{
         /* send message to voice CPU */
         INCAIP2_IKOS_DMSG("Debug mailbox...\n");
         ifx_mps_debug_mailbox();
      }
	} while(1);

    
	if( ifx_mps_unregister_data_callback(voice0,
	                                           1 /* upstream */) !=	OK)
	{
		INCAIP2_IKOS_EMSG("ifx_mps_unregister_data_callback fails\n");
	}

	if( ifx_mps_close( (struct inode *)voice0, &voice0_file ) != 0 )
	{
		INCAIP2_IKOS_EMSG("ifx_mps_close fails\n");
	}
   ifx_mps_reset();

do_sw_lp_exit:
	return ret;
}




/**************************************************************************/
/* Brief:	help
 */
static int do_help ( cmd_tbl_t *cmdtp, int argc, char *argv[])
{
	int i=0;
	printk("test cases:\n");
	while ( cmd_tbl[i].name ){
		printk("\t%s\n", cmd_tbl[i].usage);
		i++;
	}
	return 0;
}
/***************************************************************************
 * Brief:	 find command table entry for a command
 */
cmd_tbl_t *find_cmd(const char *cmd)
{
	cmd_tbl_t *cmdtp;
	/* Search command table - Use linear search - it's a small table */
	for (cmdtp = &cmd_tbl[0]; cmdtp->name; cmdtp++) {
		if (strncmp (cmd, cmdtp->name, cmdtp->len) == 0){
			return cmdtp;
		}
	}
	return NULL;	/* not found */
}
/****************************************************************************
 * Brief: 	parse command line to args
 */
int parse_line (char *line, char *argv[])
{
	int nargs = 0;
	
#ifdef DEBUG_PARSER
	INCAIP2_IKOS_DMSG ("parse_line: \"%s\"\n", line);
#endif

	while (nargs < CFG_MAXARGS) {

		/* skip any white space */
		while ((*line == ' ') || (*line == '\t')) {
			++line;
		}

		if (*line == '\0') {	/* end of line, no more args	*/
			argv[nargs] = NULL;
#ifdef DEBUG_PARSER
		INCAIP2_IKOS_DMSG ("parse_line: nargs=%d\n", nargs);
#endif
			return (nargs);
		}

		argv[nargs++] = line;	/* begin of argument string	*/

		/* find end of string */
		while (*line && (*line != ' ') && (*line != '\t')) {
			++line;
		}

		if (*line == '\0') {	/* end of line, no more args	*/
			argv[nargs] = NULL;
#ifdef DEBUG_PARSER
		INCAIP2_IKOS_DMSG ("parse_line: nargs=%d\n", nargs);
#endif
			return (nargs);
		}

		*line++ = '\0';		/* terminate current arg	 */
	}

	printk ("** Too many args (max. %d) **\n", CFG_MAXARGS);

#ifdef DEBUG_PARSER
	INCAIP2_IKOS_DMSG ("parse_line: nargs=%d\n", nargs);
#endif
	return (nargs);
}
/* ********************************************************************/
/* Brief:	process a line of commond
 * Parameter:	cmd_line, current command line
 * Return:	0 on successful
 * 		1 quit command
 *		-1 error
 */
int process_command(char * cmd_line)
{
	cmd_tbl_t *cmdtp;
	char *argv[CFG_MAXARGS + 1];	/* NULL terminated	*/
	int argc;
	
	//ignore comments
	if (*cmd_line == '#'|| strlen(cmd_line) == 0) {
		return 0;
	}

	/* Extract arguments */
	argc = parse_line (cmd_line, argv);

	/* Look up command in command table */
	if ((cmdtp = find_cmd(argv[0])) == NULL) {
		printk ("Unknown command '%s' - try 'help'\n", argv[0]);
		return -1;	/* give up after bad command */
	}
	
	/* OK - call function to do the command */
	return (cmdtp->cmd) (cmdtp,argc, argv);
}

/* Brief:	read a line from console (null terminated)
 * Paramter:	buf, hold the return string
 *		limit, size of the buffer (including the null char)
 * Return:	number of bytes read, 0 means error
	
 */
static int read_line(char * buf, int limit) {
	int len=0;
	char c;
	while(1){
		c = getDebugChar();
		if (c == '\r' || c == '\n') {
			printk("\n");
			break;
		}
		//echo back
		putDebugChar(c);
		buf[len++] = c;
		if (len == limit - 1) break;
	}
	buf[len] = '\0';	
	return len;
}

/**************************************************************************/
/* Brief:	help
 */
extern unsigned long volatile jiffies;
static int do_time( cmd_tbl_t *cmdtp, int argc, char *argv[])
{
	printk("%16ul\n", (unsigned int) jiffies);
	return 0;
}

void __ikos(void)
{
	char line_buffer[CMD_MAX+1];
	printk("IKOS verification...\n");
	while (1)
    {
		printk(">");
		if (read_line(line_buffer, CMD_MAX) != 0)
        {
			//process the command
			process_command(line_buffer);
		}
	}
}

/* Brief: testcase entry
 * Parameter: no
 * Return: never
 * Description:
 * 	Since the IKOS emulation is extremly slow, we can't wait until the
 * whole system is up and then verify our drivers. Thus we decide to hack
 * the kernel so that it is traped in this function to do verification.
 * We borrow some parts from U-boot.
 */
void ikos(void)
{
	__ikos();
}

#endif /* IKOS_MINI_BOOT */


