/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* 
 * Initial version
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 */

/*
 * Reset the INCA-IP2 board.
 */

#include <linux/kernel.h>
#include <asm/reboot.h>
#include <asm/system.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/mps.h>


static void incaip2_machine_restart(char *command);
static void incaip2_machine_halt(void);
static void incaip2_machine_power_off(void);

#define MBX_SIZE_IN_WORDS 112

typedef struct
{
    volatile u32      MPS_BOOT_RVEC;   /**< CPU reset vector */
    volatile u32      MPS_RES[7];
} reset_mps_boot_cfg_reg;

typedef struct
{
    volatile u32      MBX[MBX_SIZE_IN_WORDS];
    reset_mps_boot_cfg_reg  MBX_CPU0_BOOT_CFG;    /**< CPU0 Boot Configuration */
    reset_mps_boot_cfg_reg  MBX_CPU1_BOOT_CFG;    /**< CPU1 Boot Configuration */  
} reset_mps_mbx;

static void incaip2_machine_restart(char *command)
{
    u32 version;
    
    reset_mps_mbx *ifx_mps_reg = (reset_mps_mbx*) INCA_IP2_MPS_SRAM;
    
	local_irq_disable();

    /* Set correct reset vector in bootrom */
    version=INCA_IP2_MPS_CHIPID_VERSION_GET(*INCA_IP2_MPS_CHIPID);
    if ( version == 1 ) { 
        ifx_mps_reg->MBX_CPU0_BOOT_CFG.MPS_BOOT_RVEC = (u32)0xBFC00550;
    }
    if ( version >= 2 ) { 
        ifx_mps_reg->MBX_CPU0_BOOT_CFG.MPS_BOOT_RVEC = (u32)0xBFC0056C;
    }
    
	printk(KERN_NOTICE "Restarting...\n");
	*INCA_IP2_RCU_RST_REQ = INCA_IP2_RCU_RST_REQ_ALL;
	for (;;) ;
}

static void incaip2_machine_halt(void)
{
	/* Disable interrupts and loop forever */
	printk(KERN_NOTICE "System halted.\n");
	local_irq_disable();
	for (;;) ;
}

static void incaip2_machine_power_off(void)
{
	/* We can't power off without the user's assistance */
	printk(KERN_NOTICE "Please turn off the power now.\n");
	local_irq_disable();
	for (;;) ;
}

void incaip2_reboot_setup(void)
{
	_machine_restart = incaip2_machine_restart;
	_machine_halt = incaip2_machine_halt;
	_machine_power_off = incaip2_machine_power_off;
}
