/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* 
 * Initial version
 * Gary Jennejohn (C) 2003 <gj@denx.de>
 */

/*
 * INCA-IP2 specific setup.
 */
 
#include <linux/config.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/module.h>
#include <linux/kernel_stat.h>
#include <linux/trace.h>

#include <linux/interrupt.h>
#include <asm/time.h>

#include <asm/cpu.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/irq.h>
#include <asm/debug.h>

#ifdef CONFIG_ILATENCY
#include <linux/ilatency.h>
#define ILATENCY_RET_FROM_EXCEPTION() intr_ret_from_exception()
#else
#define ILATENCY_RET_FROM_EXCEPTION()
#endif

extern void ifx_mux_init(void);
extern void incaip2_reboot_setup(void);
extern void prom_printf(const char * fmt, ...);
void incaip2_timer_setup(struct irqaction *irq);

#ifdef CONFIG_KGDB
extern void rs_kgdb_hook(int);
int remote_debug = 0;
#endif

#define COUNTER_WORKS
#define MIPS_CPU_TIMER_IRQ INT_NUM_IM5_IRL18
#define ALLINTS (IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5)

#define FREQ_262MHZ 262144000
#define FREQ_327MHZ 327685000
#define FREQ_360MHZ 360448000
#define FREQ_393MHZ 393216000
#define FREQ_133MHZ (FREQ_393MHZ/3)
#define FREQ_100MHZ (FREQ_393MHZ/4)

#define FREQ_EMULATION 10000000
#define FREQ_VENUS 10000000
//#define FREQ_EMULATION 2000000

static unsigned int r4k_offset; /* Amount to increment compare reg each time */
static unsigned int r4k_cur;    /* What counter should be at next timer irq */

/**
 * Bus Error Init.
 * This function reads all bus error CON registers, thus initializing them to 0.
 */
void __init bus_error_init(void) 
{ 
    u32 reg=0;
    
	reg = *(INCA_IP2_BCU0_ECON);
	reg = *(INCA_IP2_BCU1_ECON);
	reg = *(INCA_IP2_MC_ERRCAUSE);
}

/**
 * Get CPU clock divider.
 * This function returns the currently set divider for the CPU clock.
 *
 * \return 1 Divider is 1 
 * \return 2 Divider is 2 
 * \return 3 Divider is 3 
 * \return 4 Divider is 1.5 
 */
int incaip2_get_cpu_clock_divider(void)
{
    u32 div=0;
    u32 fuse=0;
    int retval=1;
    
    div=INCA_IP2_CGU_PLL2CR_MIPS0_DIV_GET(*INCA_IP2_CGU_PLL2CR);
    fuse=INCA_IP2_CGU_FUSE_MIPS0DIS_GET(*INCA_IP2_CGU_FUSE);
    
    if (fuse == 1) {
        switch (div) {
            case 1: retval=2;
                    break;
            case 2: retval=3;
                    break;
            default:
                    printk("Unknown clock divider mode in PLL register. Using 2\n");
                    return 2;
        }
    }
    else {
        switch (div) {
            case 0: retval=1;
                    break;
            case 1: retval=2;
                    break;
            case 2: retval=3;
                    break;
            case 3: retval=4;
                    break;
            default:
                    printk("Unknown clock divider mode in PLL register. Using 1\n");
                    return 1;
        }
    }
    
    return retval;
}

uint incaip2_get_pllclk (void)
{
    u32 pll2_cr = (*INCA_IP2_CGU_PLL2CR & 0x018FFFFF);
    switch (pll2_cr)
    {
    case 0x01828BE3:
        return FREQ_393MHZ;
    case 0x008D8383:
        return FREQ_360MHZ;
    case 0x0085BB33:
        return FREQ_327MHZ;
    case 0x01845E93:
        return FREQ_262MHZ;
    }
    return 0;
}

uint incaip2_get_cpuclk (void)
{
    int pllclock=0;
    int cpuclock=0;
    int divider=1;
    
    pllclock=incaip2_get_pllclk();
    divider = incaip2_get_cpu_clock_divider();
    
    switch (divider) {
        case 1:
                cpuclock = pllclock;
                break;
        case 2:
                cpuclock = pllclock >> 1;
                break;
        case 3:
                cpuclock = pllclock / 3;
                break;
        case 4:
                cpuclock = (2 * pllclock) / 3; /* changed from pllclock/1.5 to avoid float */
                break;
        default:
                printk("Unknown CPU clock divider mode. Using PLL clock.\n");
                cpuclock = pllclock;
    }
    
    return cpuclock;
}

uint incaip2_get_lpbsclk (void)
{
	uint fpi_div[] = {2,3,4,5};
	uint lpbsdiv = INCA_IP2_CGU_FBSCR_LPBSDIV_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_pllclk()/fpi_div[lpbsdiv];
}

uint incaip2_get_fbs0clk (void)
{
	uint fbs0_div[] = {1,2,4,8};
	uint div0 = INCA_IP2_CGU_FBSCR_DIV0_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_lpbsclk()/fbs0_div[div0];
}

uint incaip2_get_fbs1clk (void)
{
	uint fbs1_div[] = {1,2,4,8};
	uint div1 = INCA_IP2_CGU_FBSCR_DIV1_GET(*INCA_IP2_CGU_FBSCR);

	return incaip2_get_lpbsclk()/fbs1_div[div1];
}

/**
 * Get CPU clock rate.
 * This function returns the default frequency of the chip
 *
 * \return FREQ_393MHZ for 393MHz 
 */
unsigned int incaip2_get_cpu_hz(void)
{
#ifdef CONFIG_INCAIP2_USE_IKOS
#ifdef CONFIG_INCAIP2_VENUS
    return FREQ_VENUS;
#else
    return FREQ_EMULATION;
#endif
#else
	return incaip2_get_cpuclk();
#endif
}

/**
 * Get FPI bus clock rate.
 * This function returns the default frequency of the FPI bus
 *
 * \return FREQ_133MHZ for 133MHz 
 */
unsigned int incaip2_get_fpi_hz(void)
{
#ifdef CONFIG_INCAIP2_USE_IKOS
	return (unsigned int)FREQ_EMULATION/3;
#else
	return incaip2_get_fbs1clk();
#endif
}

/**
 * Get CPU version number.
 * This function returns the version of the Inca-IP2
 *
 * \return Version of Inca-IP2 (1=v1.1, 2=v1.2 ...) 
 */
unsigned int incaip2_get_cpu_ver(void)
{
	static unsigned int cpu_ver = 0;
	
	cpu_ver = INCA_IP2_MPS_CHIPID_VERSION_GET(*INCA_IP2_MPS_CHIPID);
	return cpu_ver;
}

/**
 * Initialize timer values.
 * This function checks the current frequency and set some internal 
 * variables accordingly.
 */
void incaip2_time_init(void)
{
	unsigned int ver;
	unsigned int freq;

	ver = incaip2_get_cpu_ver();
	printk("INCA-IP2 CPU Ver. 1.%d ",ver);
	
	freq=incaip2_get_cpu_hz();
#ifdef CONFIG_INCAIP2_USE_IKOS
	printk("200kHz\n");
	r4k_offset = freq/HZ;
#else
    printk("%dMHz\n",freq/1000000);
	r4k_offset = freq/HZ;
#endif

    if ((read_c0_prid() & 0xffff00) != (PRID_COMP_MIPS | PRID_IMP_20KC))
		r4k_offset = r4k_offset >> 1;

	mips_hpt_frequency = r4k_offset*HZ;
}

/**
 * Setup and start timer.
 * This function initializes the CP0_COUNT and CP0_COMPARE registers
 * and issues the first timer interrupt to get things running. 
 * Also all HW interrupts are enabled in CP0_STATUS
 *
 * \param irq Structure for timer interrupt action.
 */
void incaip2_timer_setup(struct irqaction *irq)
{
	int retval;

	irq->handler = no_action;

	retval = setup_irq(MIPS_CPU_TIMER_IRQ, irq);
	if (retval)
		printk("setup_irq failed\n");

	/* Enable timer interrupt in IM5 */
	enable_irq(MIPS_CPU_TIMER_IRQ);

	write_c0_count(0);
	/* to generate the first timer interrupt */
	r4k_cur = (read_c0_count() + r4k_offset);
	write_c0_compare(r4k_cur);
	set_c0_status(ALLINTS);
	set_c0_status(ALLINTS);
	write_c0_cache(0xf);
}

/**
 * Handler for timer interrupt.
 * This is the handler for a timer interrupt. 
 *
 * \param regs register context structure.
 */
asmlinkage void incaip2_timer_interrupt(struct pt_regs *regs)
{
	int cpu = smp_processor_id();
	int irq = MIPS_CPU_TIMER_IRQ;
    
#if defined(CONFIG_PREEMPT)
	preempt_disable();
#endif

	TRACE_IRQ_ENTRY(irq, ((regs->cp0_status & ST0_KSU) == KSU_KERNEL));
	irq_enter(cpu, irq);
	kstat.irqs[cpu][irq]++;

	/* we keep interrupt disabled all the time */
	timer_interrupt(irq, NULL, regs);
	ack_incaip2_irq(MIPS_CPU_TIMER_IRQ);

	irq_exit(cpu, irq);

	TRACE_IRQ_EXIT();

	if (softirq_pending(cpu))
		do_softirq();

#if defined(CONFIG_PREEMPT)
	for(;;) {
		preempt_enable_no_resched();
		if (preempt_is_disabled() || !need_resched())
			break;

		db_assert(intr_off());
		db_assert(!in_interrupt());

		preempt_disable();
		__sti();
		preempt_schedule();
		__cli();
	}
#endif
	ILATENCY_RET_FROM_EXCEPTION();
}

#ifdef CONFIG_DEBUG_MINI_BOOT
#define MINI_BOOT
#endif

#if defined(CONFIG_KGDB) || defined(CONFIG_INCAIP2_USE_IKOS) || defined(MINI_BOOT)
/**
 * Print debug character.
 * This function is used by KGDB to send data over the serial port
 *
 * \param  c is the character to be transmitted
 * \return Always returns 1 
 */
int putDebugChar(char c)
{
	while ((INCA_IP2_ASC0_FSTAT_TXFREE_GET(*INCA_IP2_ASC0_FSTAT)) == 0x00) ; 
	*((char*)INCA_IP2_ASC0_TBUF + 3) = c;
	return 1;
}

/**
 * Read debug character.
 * This function is used by KGDB to read data from the serial port
 *
 * \return Returns the read character 
 */
char getDebugChar(void)
{
	while ((INCA_IP2_ASC0_FSTAT_RXFFL_GET(*INCA_IP2_ASC0_FSTAT)) == 0x00) ; 
	return *INCA_IP2_ASC0_RBUF;
}
#endif

void prom_putchar(char c)
{
#if defined( CONFIG_IFX_ASC_CONSOLE_ASC0 )
    /* Wait for FIFO to empty */
    while ((INCA_IP2_ASC0_FSTAT_TXFREE_GET(*INCA_IP2_ASC0_FSTAT)) == 0x00) ; 
    /* Crude cr/nl handling is better than none */
    *(((char*)INCA_IP2_ASC0_TBUF)+3)=(c);
#elif defined( CONFIG_IFX_ASC_CONSOLE_ASC1 )
    /* Wait for FIFO to empty */
    while ((INCA_IP2_ASC1_FSTAT_TXFREE_GET(*INCA_IP2_ASC1_FSTAT)) == 0x00) ; 
    /* Crude cr/nl handling is better than none */
    *INCA_IP2_ASC1_TBUF=(c);
#endif
}

/**
 * Chip setup.
 * In the first boot phase this function is called to setup chip specific stuff
 */
void __init incaip2_setup(void)
{
#ifdef CONFIG_KGDB
	char *argptr;
#endif
	
	ifx_mux_init();
	incaip2_reboot_setup();
	board_time_init = incaip2_time_init;
	board_timer_setup = incaip2_timer_setup;

	prom_printf("Inca-IP2 Linux starting.\n");
    
#ifndef CONFIG_MIPS_FORCE_CACHE_WRITE_THROUGH
    if (incaip2_get_cpu_ver() < 2) {
		prom_printf("FATAL ERROR.\n");
		prom_printf("Inca-IP2 version too old for this kernel. Need at least v1.2\n");
		prom_printf("Died.\n");
		panic("Help!");
	}
#endif
    
#ifdef CONFIG_KGDB
	/*
	 * We could set the serial speed here, but we leave it
	 * at the speed set by the bootloader.
	 */
	prom_printf("KGDB: Using serial line /dev/ttyS0 for session, "
		    "please connect your debugger\n");
	remote_debug = 1;
	/* Breakpoints are in init_IRQ() */
#endif
}

EXPORT_SYMBOL(incaip2_get_cpu_hz);
EXPORT_SYMBOL(incaip2_get_fpi_hz);
EXPORT_SYMBOL(incaip2_get_fbs0clk);
EXPORT_SYMBOL(incaip2_get_fbs1clk);
EXPORT_SYMBOL(incaip2_get_cpu_ver);

