/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/addrspace.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/mux.h>
#include "mux_defs.h"

extern int incaip2_get_cpu_ver(void);

static int mux_led = 0;
static int mux_key = 0;
static u32 mux_exin = 0;
static u32 mux_gp = 0;
static u32 mux_ebucs = 0;
static u32 mux_ebuaddr = 0;
static u32 mux_altclock = 0;
static u32 mux_ssc = 0;

static u32 mux_altsel0 = 0;
static u32 mux_altsel1 = 0;
static u32 mux_altsel2 = 0;
static u32 mux_altsel3 = 0;
static u32 mux_altsel4 = 0;
static u32 mux_altsel5 = 0;
static u32 mux_altsel6 = 0;
static u32 mux_altsel7 = 0;
static u32 mux_altsel8 = 0;
static u32 mux_altsel9 = 0;
static u32 mux_altsel10 = 0;
static u32 mux_altsel11 = 0;
static u32 mux_altsel12 = 0;
static u32 mux_altsel13 = 0;
static u32 mux_altsel14 = 0;
static u32 mux_altsel15 = 0;
static u32 mux_altsel16 = 0;
static u32 mux_altsel17 = 0;
static u32 mux_altsel18 = 0;
static u32 mux_altsel19 = 0;
static u32 mux_altsel20 = 0;
static u32 mux_altsel21 = 0;
static u32 mux_altsel22 = 0;
static u32 mux_altsel23 = 0;
static u32 mux_altsel24 = 0;
static u32 mux_altsel25 = 0;
static u32 mux_altsel26 = 0;
static u32 mux_altsel27 = 0;

static u32 mux_dir = 0;
static u32 mux_puden = 0;
static u32 mux_dir_mask = 0;

/**
 * Init LED count.
 * This functions initializes the number of available LEDs.
 * \ingroup Internal  
 */
void ifx_mux_led_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_LED0 
    mux_led = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED1 
    mux_led = 2;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED2 
    mux_led = 3;
#endif 
#ifdef CONFIG_INCAIP2_MUX_LED3 
    mux_led = 4;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED4 
    mux_led = 5;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED5 
    mux_led = 6;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED6 
    mux_led = 7;
    mux_altsel19 = 0;
    mux_dir |= 0x00080000;
    mux_dir_mask |= 0x00080000;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED7 
    mux_led = 8;
    mux_altsel20 = 0;
    mux_dir |= 0x00100000;
    mux_dir_mask |= 0x00100000;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED8 
    mux_led = 9;
    mux_altsel21 = 0;
    mux_dir |= 0x00200000;
    mux_dir_mask |= 0x00200000;
#endif
#ifdef CONFIG_INCAIP2_MUX_LED9 
    mux_led = 10;
    mux_altsel22 = 0;
    mux_dir |= 0x00400000;
    mux_dir_mask |= 0x00400000;
#endif
} 

/**
 * Init key count.
 * This functions initializes the number of available keys. 
 * \ingroup Internal  
 */
void ifx_mux_key_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_KEY0 
    mux_key = 1;
#endif 
#ifdef CONFIG_INCAIP2_MUX_KEY1 
    mux_key = 2;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY2 
    mux_key = 3;
#endif 
#ifdef CONFIG_INCAIP2_MUX_KEY3 
    mux_key = 4;
#endif 
#ifdef CONFIG_INCAIP2_MUX_KEY4 
    mux_key = 5;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY5 
    mux_key = 6;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY6 
    mux_key = 7;
    mux_puden |= 0x00000400;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY7 
    mux_key = 8;
    mux_altsel11 = 0;
    mux_puden |= 0x00000800;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY8 
    mux_key = 9;
    mux_altsel12 = 0;
    mux_puden |= 0x00001000;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY9 
    mux_key = 10;
    mux_altsel13 = 0;
    mux_puden |= 0x00002000;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY10 
    mux_key = 11;
    mux_altsel14 = 0;
    mux_puden |= 0x00004000;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY11 
    mux_key = 12;
    mux_altsel15 = 0;
    mux_puden |= 0x00008000;
#endif 
#ifdef CONFIG_INCAIP2_MUX_KEY12 
    mux_key = 13;
    mux_altsel16 = 0;
    mux_puden |= 0x00010000;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY13 
    mux_key = 14;
    mux_altsel17 = 0;
    mux_puden |= 0x00020000;
#endif
#ifdef CONFIG_INCAIP2_MUX_KEY14 
    mux_key = 15;
    mux_altsel18 = 0;
    mux_puden |= 0x00040000;
#endif
} 

/**
 * Init external interrupts.
 * This functions initializes bitfield of available external interrups. 
 * \ingroup Internal  
 */
void ifx_mux_exin_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_EXIN0 
    mux_exin |= CONFIG_INCAIP2_MUX_BIT_EXIN0;
    mux_altsel25 = 0;
    mux_dir_mask |= 0x02000000;
#endif
#ifdef CONFIG_INCAIP2_MUX_EXIN1 
    mux_exin |= CONFIG_INCAIP2_MUX_BIT_EXIN1;
    mux_altsel22 = 2;
    mux_dir_mask |= 0x00400000;
#endif
#ifdef CONFIG_INCAIP2_MUX_EXIN2 
    mux_exin |= CONFIG_INCAIP2_MUX_BIT_EXIN2;
    mux_altsel21 = 2;
    mux_dir_mask |= 0x00200000;
#endif
} 

/**
 * Init PWM.
 * This functions initializes altsel and direction values for PWM.
 * \ingroup Internal  
 */
void ifx_mux_pwm_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_PWM1 
    mux_altsel9 = 2;
    if (incaip2_get_cpu_ver() < 2) {
        mux_dir |= 0x00000200;
    }
    mux_dir_mask |= 0x00000200;
#endif
#ifdef CONFIG_INCAIP2_MUX_PWM2 
    mux_altsel8 = 2;
    if (incaip2_get_cpu_ver() < 2) {
        mux_dir |= 0x00000100;
    }
    mux_dir_mask |= 0x00000100;
#endif
} 

/**
 * Init SSC
 * This functions initializes altsel and direction values for SSCs. 
 * \ingroup Internal  
 */
void ifx_mux_ssc_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_MRST0
    mux_altsel18 = 2;
    mux_puden |= 0x00040000;
    mux_dir_mask |= 0x00040000;
#endif
#ifdef CONFIG_INCAIP2_MUX_MTSR0
    mux_altsel17 = 2;
    mux_dir |= 0x00020000;
    mux_dir_mask |= 0x00020000;
#endif
#ifdef CONFIG_INCAIP2_MUX_SCLK0
    mux_altsel16 = 2;
    mux_dir |= 0x00010000;
    mux_dir_mask |= 0x00010000;
#endif

#ifdef CONFIG_INCAIP2_MUX_USE_BOOT_GPIO
	/* We are relying on uboot to have set GPIO pins 12,13,14,15 correctly
	 * so that we can read our hardware type from the board
	 * If We are an OpenStage 15 we do not want to configure the SCC0 pins
	 */
	if( (*INCA_IP2_GPIO_PIN & 0xF000 ) == 0x04000 )
	{
		/* Set GPIO pins 16,17,18 to output (16 not actual used) */
	    mux_gp |= (CONFIG_INCAIP2_MUX_BIT_GP16 | CONFIG_INCAIP2_MUX_BIT_GP17 | CONFIG_INCAIP2_MUX_BIT_GP18);
	    mux_altsel16 = 1;
	    mux_altsel17 = 1;
	    mux_altsel18 = 1;
	    mux_puden &= ~0x00070000;
	    mux_dir   |=  0x00070000;
	    mux_dir_mask |= 0x00070000;
	}
#endif

#ifdef CONFIG_INCAIP2_MUX_SSC0_CS1
    mux_altsel15 = 2;
    mux_dir |= 0x00008000;
    mux_dir_mask |= 0x00008000;
#endif
#ifdef CONFIG_INCAIP2_MUX_SSC0_CS0 
    mux_altsel14 = 2;
    mux_dir |= 0x00004000;
    mux_dir_mask |= 0x00004000;
#endif
    if ((mux_altsel18 == 2) && (mux_altsel17 == 2) &&
        (mux_altsel16 == 2) ) {
//        (mux_altsel16 == 2) && (mux_altsel15 == 2) &&
//        (mux_altsel14 == 2) ) {
            mux_ssc |= CONFIG_INCAIP2_MUX_BIT_SSC0;
    }
     
#ifdef CONFIG_INCAIP2_MUX_MRST1 
    mux_altsel7 = 2;
    mux_puden |= 0x00000080;
    mux_dir_mask |= 0x00000080;
#endif
#ifdef CONFIG_INCAIP2_MUX_MTSR1 
    mux_altsel6 = 2;
    mux_dir |= 0x00000040;
    mux_dir_mask |= 0x00000040;
#endif
#ifdef CONFIG_INCAIP2_MUX_SCLK1 
    mux_altsel5 = 2;
    mux_dir |= 0x00000020;
    mux_dir_mask |= 0x00000020;
#endif
#ifdef CONFIG_INCAIP2_MUX_SSC1_CS0 
    mux_altsel4 = 2;
    mux_dir |= 0x00000010;
    mux_dir_mask |= 0x00000010;
#endif
#ifdef CONFIG_INCAIP2_MUX_SSC1_CS1 
    mux_altsel3 = 2;
    mux_dir |= 0x00000008;
    mux_dir_mask |= 0x00000008;
#endif
    if ((mux_altsel7 == 2) && (mux_altsel6 == 2) &&
        (mux_altsel5 == 2) ) {
            mux_ssc |= CONFIG_INCAIP2_MUX_BIT_SSC1;
    }

} 

/**
 * Init ASC.
 * This functions initializes altsel and direction values for ASCs. 
 * \ingroup Internal  
 */
void ifx_mux_asc_init(void)
{
#if defined(CONFIG_INCAIP2_MUX_TXD0)
    mux_altsel23 = 0;
    mux_dir |= 0x00800000;
    mux_dir_mask |= 0x00800000;
#endif
#if defined(CONFIG_INCAIP2_MUX_RXD0)
    mux_altsel24 = 0;
    mux_dir_mask |= 0x01000000;
#endif 
#if defined(CONFIG_INCAIP2_MUX_TXD1)
    mux_altsel17 = 3;
    mux_dir |= 0x00020000;
    mux_dir_mask |= 0x00020000;
#endif
#if defined(CONFIG_INCAIP2_MUX_RXD1)
    mux_altsel18 = 3;
    mux_dir_mask |= 0x00040000;
#endif 
}

/**
 * Init NAND.
 * This functions initializes altsel value for NAND. 
 * \ingroup Internal  
 */
void ifx_mux_nand_init(void)
{
#if defined(CONFIG_INCAIP2_MUX_NAND_RDBY)
    mux_altsel10 = 0;
#endif
}

/**
 * Init USB.
 * This functions initializes altsel and direction values for USB. 
 * \ingroup Internal  
 */
void ifx_mux_usb_init(void)
{
#if defined(CONFIG_INCAIP2_MUX_DP_USB)
    mux_altsel26 = 0;
#endif
#if defined(CONFIG_INCAIP2_MUX_DN_USB)
    mux_altsel27 = 0;
#endif 
}

/**
 * Init alternate clock.
 * This functions initializes altsel value for altenate clock. 
 * \ingroup Internal  
 */
void ifx_mux_clock_out_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_CLK_OUT2
    mux_altclock |= CONFIG_INCAIP2_MUX_BIT_ALT_CLK2;
    mux_altsel16 = 3;
    mux_dir |= 0x00010000;
    mux_dir_mask |= 0x00010000;
#endif
#ifdef CONFIG_INCAIP2_MUX_CLK_OUT1
    mux_altclock |= CONFIG_INCAIP2_MUX_BIT_ALT_CLK1;
    mux_altsel6 = 1;
    mux_dir |= 0x00000080;
    mux_dir_mask |= 0x00000080;
#endif
} 

/**
 * Init PCM.
 * This functions initializes altsel values for PCM. 
 * \ingroup Internal  
 */
void ifx_mux_pcm_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_RES
    mux_altsel5 = 1;
    mux_dir |= 0x00000020;
    mux_dir_mask |= 0x00000020;
#endif
#ifdef CONFIG_INCAIP2_MUX_TC
    mux_altsel4 = 1;
    mux_dir |= 0x00000010;
#endif
#ifdef CONFIG_INCAIP2_MUX_DO
    mux_altsel3 = 1;
    mux_dir |= 0x00000008;
    mux_dir_mask |= 0x00000008;
    mux_puden |= 0x00000008;
#endif
#ifdef CONFIG_INCAIP2_MUX_DI
    mux_altsel2 = 1;
    mux_dir_mask |= 0x00000004;
    mux_puden |= 0x00000004;
#endif
#ifdef CONFIG_INCAIP2_MUX_DCL
    mux_altsel1 = 1;
    mux_dir |= 0x00000002;
    mux_dir_mask |= 0x00000002;
#endif
#ifdef CONFIG_INCAIP2_MUX_DCL_SLAVE
    mux_altsel1 = 1;
    mux_dir_mask |= 0x00000002;
#endif
#ifdef CONFIG_INCAIP2_MUX_FSC
    mux_altsel0 = 1;
    mux_dir |= 0x00000001;
    mux_dir_mask |= 0x00000001;
#endif
#ifdef CONFIG_INCAIP2_MUX_FSC_SLAVE
    mux_altsel0 = 1;
    mux_dir_mask |= 0x00000001;
#endif
} 

/**
 * Init real time clock.
 * This functions initializes altsel value for real time clock. 
 * \ingroup Internal  
 */
void ifx_mux_rtc_clock_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_CLK32 
    mux_altsel7 = 1;
#endif
} 

/**
 * Init EBU.
 * This functions initializes bitfield of available chip selects and address pins. 
 * \ingroup Internal  
 */
void ifx_mux_ebu_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_EBU_CS3 
    mux_ebucs |= CONFIG_INCAIP2_MUX_BIT_EBU_CS3;
    mux_altsel21 = 3;
    mux_dir |= 0x00200000;
    mux_dir_mask |= 0x00200000;
#endif
#ifdef CONFIG_INCAIP2_MUX_EBU_CS2 
    mux_ebucs |= CONFIG_INCAIP2_MUX_BIT_EBU_CS2;
    mux_altsel22 = 3;
    mux_dir |= 0x00400000;
    mux_dir_mask |= 0x00400000;
#endif
#ifdef CONFIG_INCAIP2_MUX_EBU_A25 
    mux_ebuaddr |= CONFIG_INCAIP2_MUX_BIT_EBU_A25;
    mux_altsel9 = 0;
    if (incaip2_get_cpu_ver() < 2) {
        mux_dir |= 0x00000200;
    }
    mux_dir_mask |= 0x00000200;
#endif
#ifdef CONFIG_INCAIP2_MUX_EBU_A24 
    mux_ebuaddr |= CONFIG_INCAIP2_MUX_BIT_EBU_A24;
    mux_altsel8 = 0;
    if (incaip2_get_cpu_ver() < 2) {
        mux_dir |= 0x00000100;
    }
    mux_dir_mask |= 0x00000100;
#endif
} 

/**
 * Init GP bitfield.
 * This functions initializes bitfield of available general purpose pins. 
 * \ingroup Internal  
 */
void ifx_mux_gp_init(void)
{
#ifdef CONFIG_INCAIP2_MUX_GP0 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP0;
    mux_altsel0 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP1 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP1;
    mux_altsel1 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP2 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP2;
    mux_altsel2 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP3 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP3;
    mux_altsel3 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP4 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP4;
    mux_altsel4 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP5 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP5;
    mux_altsel5 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP6 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP6;
    mux_altsel6 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP7 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP7;
    mux_altsel7 = 0;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP8 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP8;
    mux_altsel8 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP9 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP9;
    mux_altsel9 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP10 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP10;
    mux_altsel10 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP11 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP11;
    mux_altsel11 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP12 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP12;
    mux_altsel12 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP13 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP13;
    mux_altsel13 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP14 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP14;
    mux_altsel14 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP15 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP15;
    mux_altsel15 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP16 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP16;
    mux_altsel16 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP17 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP17;
    mux_altsel17 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP18 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP18;
    mux_altsel18 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP19 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP19;
    mux_altsel19 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP20 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP20;
    mux_altsel20 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP21 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP21;
    mux_altsel21 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP22 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP22;
    mux_altsel22 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP23 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP23;
    mux_altsel23 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP24 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP24;
    mux_altsel24 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP25 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP25;
    mux_altsel25 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP26 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP26;
    mux_altsel26 = 1;
#endif
#ifdef CONFIG_INCAIP2_MUX_GP27 
    mux_gp |= CONFIG_INCAIP2_MUX_BIT_GP27;
    mux_altsel27 = 1;
#endif
} 

/**
 * Availability of PWM1.
 * This functions reports whether support for PWM1 is enabled in the multiplexer 
 * settings.
 *
 * \return 1 PWM1 is available
 * \return 0 PWM1 is not available.
 * \ingroup API
 */
u32 ifx_mux_pwm1_pins(void)
{
#ifdef CONFIG_INCAIP2_MUX_PWM1
    return 1;
#else
    return 0;
#endif 
}

/**
 * Availability of PWM2.
 * This functions reports whether support for PWM2 is enabled in the multiplexer 
 * settings.
 *
 * \return 1 PWM2 is available
 * \return 0 PWM2 is not available.
 * \ingroup API
 */
u32 ifx_mux_pwm2_pins(void)
{
#ifdef CONFIG_INCAIP2_MUX_PWM2
    return 1;
#else
    return 0;
#endif 
}

/**
 * Availability of SSC Ports.
 * This functions reports which SSC ports are available.
 *
 * \return mux_ssc bitfield of available SSC ports
 * \ingroup API
 */
u32 ifx_mux_ssc_pins(void)
{
    return mux_ssc;
}

/**
 * Availability of ASC1.
 * This functions reports whether support for ASC1 is enabled in the multiplexer 
 * settings.
 *
 * \return 1 ASC1 is available
 * \return 0 ASC1 is not available.
 * \ingroup API
 */
u32 ifx_mux_asc1_pins(void)
{
#if defined(CONFIG_INCAIP2_MUX_TXD1) && defined(CONFIG_INCAIP2_MUX_RXD1) 
    return 1;
#else
    return 0;
#endif 
}

/**
 * Availability of ASC0.
 * This functions reports whether support for ASC0 is enabled in the multiplexer 
 * settings.
 *
 * \return 1 ASC0 is available
 * \return 0 ASC0 is not available.
 * \ingroup API
 */
u32 ifx_mux_asc0_pins(void)
{
#if defined(CONFIG_INCAIP2_MUX_TXD0) && defined(CONFIG_INCAIP2_MUX_RXD0) 
    return 1;
#else
    return 0;
#endif 
}

/**
 * Availability of USB.
 * This functions reports whether support for USB   is enabled in the multiplexer 
 * settings.
 *
 * \return 1 USB is available
 * \return 0 USB is not available.
 * \ingroup API
 */
u32 ifx_mux_usb_pins(void)
{
#if defined(CONFIG_INCAIP2_MUX_DP_USB) && defined(CONFIG_INCAIP2_MUX_DN_USB) 
    return 1;
#else
    return 0;
#endif 
}

/**
 * Availability of Clock Output.
 * This functions reports which alternate clock pins are available.
 *
 * \return mux_altclock bitfield of available alternate clock pins
 * \ingroup API
 */
u32 ifx_mux_clock_out_pins(void)
{
    return mux_altclock;
}

/**
 * Availability of RTC clock input.
 * This functions reports whether support for the 32kHz RTC input clock (CLK32)
 * is enabled in the multiplexer settings.
 *
 * \return 1 CLK32 is available
 * \return 0 CLK32 is not available.
 * \ingroup API
 */
u32 ifx_mux_rtc_clock_pins(void)
{
#if defined(CONFIG_INCAIP2_MUX_CLK32) 
    return 1;
#else
    return 0;
#endif 
}

/**
 * Number of LEDs.
 * This functions reports the number of available LEDs 
 *
 * \return num Number of available LEDs
 * \ingroup API
 */
u32 ifx_mux_led_pins(void)
{
    return mux_led;
}

/**
 * Number of Keypad pins.
 * This functions reports the number of available keypad pins 
 *
 * \return num Number of available keypad pins
 * \ingroup API
 */
u32 ifx_mux_key_pins(void)
{
    return mux_key;
}

/**
 * Number of EXIN pins.
 * This functions returns the bitfield of available external interrupt pins
 *
 * \return mux_exin bitfield of available external interrupts pins
 * \ingroup API
 */
u32 ifx_mux_exin_pins(void)
{
    return mux_exin;
}

/**
 * Number of EBU chip select pins.
 * This functions returns the bitfield of available EBU chip select pins
 *
 * \return mux_ebucs bitfield of available EBU chip select pins
 * \ingroup API
 */
u32 ifx_mux_ebucs_pins(void)
{
    return mux_ebucs;
}

/**
 * Number of EBU address pins.
 * This functions returns the bitfield of available EBU address pins
 *
 * \return mux_ebuaddr bitfield of available EBU address pins
 * \ingroup API
 */
u32 ifx_mux_ebuaddr_pins(void)
{
    return mux_ebuaddr;
}

/**
 * Number of GP pins.
 * This functions returns the bitfield of available general purpose pins
 *
 * \return mux_gp bitfield of available general purpose pins
 * \ingroup API
 */
u32 ifx_mux_gp_pins(void)
{
    return mux_gp;
}

/**
 * Initialize multiplex unit.
 * This function sets a static multiplex configuration
 * \ingroup Internal  
 */
void ifx_mux_init(void)
{
    u32 reg = 0;
    u32 alt1, alt2, pdir;

    
    MUX_READ_REG(INCA_IP2_GPIO_ALTSEL1,alt1);
    MUX_READ_REG(INCA_IP2_GPIO_ALTSEL2,alt2);
    printk("Inca-IP2 Multiplex Initialization (1:%08x 2:%08x)\n", alt1, alt2);
    if( (alt2 & 0xff000000) != 0x55000000 )
    {
    	printk(KERN_ERR "ERROR:Phone Type pins not setup by uboot.\n");
        MUX_WRITE_REG(INCA_IP2_GPIO_ALTSEL2, alt2 | 0x55000000 );
        MUX_READ_REG(INCA_IP2_GPIO_PDIR,pdir);
        MUX_WRITE_REG(INCA_IP2_GPIO_PDIR, (pdir & ~0xF000) );
    }
    ifx_mux_gp_init();
    ifx_mux_key_init();
    ifx_mux_led_init();
    ifx_mux_asc_init();
    ifx_mux_clock_out_init();
    ifx_mux_ebu_init();
    ifx_mux_exin_init();
    ifx_mux_nand_init();
    ifx_mux_pcm_init();
    ifx_mux_pwm_init();
    ifx_mux_rtc_clock_init();
    ifx_mux_ssc_init();
    ifx_mux_usb_init();
    
    reg = (mux_altsel0  <<  0) | (mux_altsel1  <<  2) | (mux_altsel2  <<  4) |
          (mux_altsel3  <<  6) | (mux_altsel4  <<  8) | (mux_altsel5  << 10) |
          (mux_altsel6  << 12) | (mux_altsel7  << 14) | (mux_altsel8  << 16) |
          (mux_altsel9  << 18) | (mux_altsel10 << 20) | (mux_altsel11 << 22) |
          (mux_altsel12 << 24) | (mux_altsel13 << 26) | (mux_altsel14 << 28) |
          (mux_altsel15 << 30);
    MUX_WRITE_REG(INCA_IP2_GPIO_ALTSEL2, reg);

    reg = (mux_altsel16 <<  0) | (mux_altsel17 <<  2) | (mux_altsel18 <<  4) |
          (mux_altsel19 <<  6) | (mux_altsel20 <<  8) | (mux_altsel21 << 10) |
          (mux_altsel22 << 12) | (mux_altsel23 << 14) | (mux_altsel24 << 16) |
          (mux_altsel25 << 18) | (mux_altsel26 << 20) | (mux_altsel27 << 22);
    MUX_WRITE_REG(INCA_IP2_GPIO_ALTSEL1, reg);

#ifdef CONFIG_INCAIP2_MUX_USE_BOOT_GPIO
    MUX_READ_REG(INCA_IP2_GPIO_PDIR,reg);
    reg &= ~(mux_dir_mask);
    reg |= mux_dir;
    MUX_WRITE_REG(INCA_IP2_GPIO_PDIR, reg );
    
    MUX_READ_REG(INCA_IP2_GPIO_PUDEN,reg);
    reg |= mux_puden;
    MUX_WRITE_REG(INCA_IP2_GPIO_PUDEN, reg);
#else
    MUX_WRITE_REG(INCA_IP2_GPIO_PDIR, mux_dir);
    MUX_WRITE_REG(INCA_IP2_GPIO_PUDEN, mux_puden);
#endif

}

/**
 * Create proc file output.
 * This function creates the output for the multiplexer proc file, by 
 * reading all appropriate registers and variables and displaying the 
 * content as a table.
 *
 * \param   buf         Buffer to write the string to
 * \param   start   not used (Linux internal)
 * \param   offset  not used (Linux internal)
 * \param   count   not used (Linux internal)
 * \param   eof         Set to 1 when all data is stored in buffer
 * \param   data    not used (Linux internal)
 * \return len  Lenght of data in buffer 
 * \ingroup Internal  
 */
int ifx_mux_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
    long    len = 0;
    int     t=0;
    u32 bit=0;
    u32 reg_tmp,bits_tmp;

    len = sprintf(buf,"\nINCA-IP2 Multiplex Settings\n");

    len=len+sprintf(buf+len,"              1         0\n");
    len=len+sprintf(buf+len,"         5432109876543210\n");
    len=len+sprintf(buf+len,"-------------------------\n");
    len=len+sprintf(buf+len,"ADDSEL2: ");
    MUX_READ_REG(INCA_IP2_GPIO_ALTSEL2, reg_tmp);
    bit = 0xC0000000;
    for (t = 15; t != -1; t--) {
        bits_tmp = (reg_tmp & bit) >> (t*2);
        len=len+sprintf(buf+len,"%d",bits_tmp);
        bit = bit >> 2;
    }
    len=len+sprintf(buf+len,"\n\n");

    len=len+sprintf(buf+len,"                    2   1\n");
    len=len+sprintf(buf+len,"         xxxx765432109876\n");
    len=len+sprintf(buf+len,"-------------------------\n");
    len=len+sprintf(buf+len,"ADDSEL1: ");
    MUX_READ_REG(INCA_IP2_GPIO_ALTSEL1, reg_tmp);
    bit = 0xC0000000;
    for (t = 15; t != -1; t--) {
        bits_tmp = (reg_tmp & bit) >> (t*2);
        len=len+sprintf(buf+len,"%d",bits_tmp);
        bit = bit >> 2;
    }
    
    len=len+sprintf(buf+len,"\n\n");

    len=len+sprintf(buf+len,"ASC0 : %d\n",ifx_mux_asc0_pins());
    len=len+sprintf(buf+len,"ASC1 : %d\n",ifx_mux_asc1_pins());
    len=len+sprintf(buf+len,"PWM1 : %d\n",ifx_mux_pwm1_pins());
    len=len+sprintf(buf+len,"PWM2 : %d\n",ifx_mux_pwm2_pins());
    len=len+sprintf(buf+len,"USB  : %d\n",ifx_mux_usb_pins());
    len=len+sprintf(buf+len,"RTC  : %d\n",ifx_mux_rtc_clock_pins());
    len=len+sprintf(buf+len,"LED  : %d\n",ifx_mux_led_pins());
    len=len+sprintf(buf+len,"KEY  : %d\n",ifx_mux_key_pins());
    len=len+sprintf(buf+len,"EXCLK: 0x%08x\n",ifx_mux_clock_out_pins());
    len=len+sprintf(buf+len,"EXINT: 0x%08x\n",ifx_mux_exin_pins());
    len=len+sprintf(buf+len,"EBUAD: 0x%08x\n",ifx_mux_ebuaddr_pins());
    len=len+sprintf(buf+len,"GPIO : 0x%08x\n",ifx_mux_gp_pins());
    len=len+sprintf(buf+len,"SSC  : 0x%08x\n",ifx_mux_ssc_pins());

    len=len+sprintf(buf+len,"\n");
    *eof = 1;   // No more data available

    return len;
}


/**
 *  Initialize mux proc file.
 *  This function will create the proc file entry.
 *
 * \return 0    OK
 * \ingroup Internal  
 */
int __init ifx_mux_init_proc(void)
{
    
    create_proc_read_entry("driver/ifx_mux", 0, NULL, ifx_mux_read_procmem, NULL);
    return 0;
}

module_init(ifx_mux_init_proc);

EXPORT_SYMBOL(ifx_mux_asc0_pins);
EXPORT_SYMBOL(ifx_mux_asc1_pins);
EXPORT_SYMBOL(ifx_mux_clock_out_pins);
EXPORT_SYMBOL(ifx_mux_ebuaddr_pins);
EXPORT_SYMBOL(ifx_mux_ebucs_pins);
EXPORT_SYMBOL(ifx_mux_exin_pins);
EXPORT_SYMBOL(ifx_mux_gp_pins);
EXPORT_SYMBOL(ifx_mux_key_pins);
EXPORT_SYMBOL(ifx_mux_led_pins);
EXPORT_SYMBOL(ifx_mux_pwm1_pins);
EXPORT_SYMBOL(ifx_mux_pwm2_pins);
EXPORT_SYMBOL(ifx_mux_rtc_clock_pins);
EXPORT_SYMBOL(ifx_mux_ssc_pins);
EXPORT_SYMBOL(ifx_mux_usb_pins);

