/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/*
 * port_defs.h
 *
 * Local Inca-IP2 port header file
 *
 * Andre Messerschmidt 
 *
 * For questions about this file please contact your local Infineon 
 * representative.
 *
 */

#ifndef PORT_DEFS_H
#define PORT_DEFS_H

#define OK	0

#define PORT_WRITE_REG(reg, value)   \
	*((volatile u32*)(reg)) = (u32)value; 
#define PORT_READ_REG(reg, value)    \
	value = (u32)*((volatile u32*)(reg)); 

#define PORT_MODULE_ID	0xff
#define PORT_IOC_CALL(ret,port,pin,func) 	\
	ret=ifx_port_reserve_pin(port,pin,PORT_MODULE_ID); \
	if (ret == 0) ret=func(port,pin,PORT_MODULE_ID); \
	if (ret == 0) ret=ifx_port_free_pin(port,pin,PORT_MODULE_ID);

#define OUT_REG_BIT		0
#define IN_REG_BIT		1
#define DIR_REG_BIT		2
#define ALTSEL1_REG_BIT	3
#define ALTSEL2_REG_BIT	4
#define STOFF_REG_BIT	5
#define OD_REG_BIT		6
#define PUDEN_REG_BIT	7
#define PUDSEL_REG_BIT	8

#define OUT_REG_MASK	(1 << OUT_REG_BIT)
#define IN_REG_MASK		(1 << IN_REG_BIT)
#define DIR_REG_MASK	(1 << DIR_REG_BIT)
#define ALTSEL1_REG_MASK	(1 << ALTSEL1_REG_BIT)
#define ALTSEL2_REG_MASK	(1 << ALTSEL2_REG_BIT)
#define STOFF_REG_MASK	(1 << STOFF_REG_BIT)
#define OD_REG_MASK		(1 << OD_REG_BIT)
#define PUDEN_REG_MASK	(1 << PUDEN_REG_BIT)
#define PUDSEL_REG_MASK	(1 << PUDSEL_REG_BIT)

#if defined(CONFIG_INCAIP2)

#define PORT_REGISTERS		OUT_REG_MASK | IN_REG_MASK | DIR_REG_MASK | \
						ALTSEL1_REG_MASK | STOFF_REG_MASK | OD_REG_MASK | \
						PUDEN_REG_MASK
						
#define INCA_IP2_PORT_BASE    0xbf102600
#define PORT_OUT_REG		0x00000000
#define PORT_IN_REG			0x00000004
#define PORT_DIR_REG		0x00000008
#define PORT_ALTSEL1_REG		0x0000000C
#define PORT_ALTSEL2_REG		0x00000010
#define PORT_STOFF_REG		0x00000014
#define PORT_OD_REG			0x00000018
#define PORT_PUDEN_REG		0x0000001C
#define PORT_PUDSEL_REG		0x00000020

#define PORT_OUT_PINS		0x0fffffff
#define PORT_IN_PINS		0x0fffffff
#define PORT_DIR_PINS		0x0fffffff
#define PORT_STOFF_PINS		0x0fffffff
#define PORT_OD_PINS		0x0fffffff
#define PORT_PUDEN_PINS		0x0fffffff
#define PORT_PUDSEL_PINS		0x00000000

#define MAX_PORTS		1	// Number of ports in system
#define PINS_PER_PORT	27	// Number of available pins per port
static u32 ifx_port_bases[MAX_PORTS] 
	= {INCA_IP2_PORT_BASE}; // Base addresses for ports
static u32 ifx_port_reg_avail[MAX_PORTS] 
	= {PORT_REGISTERS}; // Which registers are available
static u32 ifx_port_out_pins[MAX_PORTS] 
	= {PORT_OUT_PINS}; // Which pins are supported by register
static u32 ifx_port_in_pins[MAX_PORTS] 
	= {PORT_IN_PINS}; // Which pins are supported by register
static u32 ifx_port_dir_pins[MAX_PORTS] 
	= {PORT_DIR_PINS}; // Which pins are supported by register
static u32 ifx_port_stoff_pins[MAX_PORTS] 
	= {PORT_STOFF_PINS}; // Which pins are supported by register
static u32 ifx_port_od_pins[MAX_PORTS] 
	= {PORT_OD_PINS}; // Which pins are supported by register
static u32 ifx_port_puden_pins[MAX_PORTS] 
	= {PORT_PUDEN_PINS}; // Which pins are supported by register
static u32 ifx_port_pudsel_pins[MAX_PORTS] 
	= {PORT_PUDSEL_PINS}; // Which pins are supported by register

#define IFX_GPIO_PINS(index) ifx_mux_gp_pins() 
#endif

	
	
#endif /* PORT_DEFS_H */
