/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Mananmohan   		1/9/2003					        	
 *
 * Manamohan			10/8/2004					Added support for re-entrancy
 *
 * DESCRIPTION :This module contains the Flood Table Configuration API's
 *
 ******************************************************************************/
#include <asm/types.h>
#include <asm/addrspace.h>
#include <asm/incaip2/incaip2.h>
#include "struct.h"
#include "const.h"
#include "flood.h"
#include "err.h"
#include "var.h"
#include "api_init_prv.h"

/* #define INCAIP2_DEBUG_FLOOD */
#undef INCAIP2_DEBUG_FLOOD

#ifdef INCAIP2_DEBUG_FLOOD
#include <linux/kernel.h>
#endif
/**
 * Set Unknown unicast ID.
 * Set the Id for unknown unicast destination address. 
 *
 * \param   chipId           Chip Id
 * \param   UnicastId        Unknown unicast destination address ID(range is 0-7)
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_UNKN_DEST_ID - If the unicast destination address ID
 *                                is not in the range of 0 to 7
 * \return  PR_ERR_NULL_PTR     - Base address is a null pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_set_unknown_ucast_ID( u8 chipId, u32 UnicastId)
{	
	u32 Temp =0;
	u32* ptr_unknown_dest;  /* pointer to Unknown destination register */
	s32 retflag = PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_FLOOD
    printk("%s - chipId = %ud, UnicastId=%ul\n", __FUNCTION__, chipId, UnicastId);	    
#endif   
#ifdef PARAMETER_CHECK
	if(UnicastId > MAX_UNKN_DEST_ID)
		return PR_ERR_UNKN_DEST_ID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId]+UNKNOWN_DEST_REG);

	if((retflag = ifx_sem_take(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;
	PR_Reg_Read(ptr_unknown_dest,Temp);
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif	
   /* read unknown destination register content and modify unicast bits */
	Temp &= UNKWN_UCASTID_MASK;
	Temp |= (UnicastId << UNKWN_UCASTID_POS);

	PR_Reg_Write(ptr_unknown_dest,Temp);
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - wrote 0x%08X to UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif 	

	if((retflag = ifx_sem_give(chipId,SEM_FLOOD)) != PR_ERR_NONE)
        return retflag;


	return PR_ERR_NONE ;	

}


/**
 * Get Unknown unicast destination address ID.
 * Get the Id for Unknown unicast destination address. 
 *                
 * \param   chipId         Chip Id
 * \param   UnicastIdPtr   Pointer to unknown unicast destination address ID
 *                         bit 0 - LAN Port is destination(1)/is not destination(0)
 *                                 for packets with unknown unicast destination 
 *                         bit 1 - PC Port is destination(1)/is not destination(0)
 *                                 for packets with unknown unicast destination
 *                         bit 2 - CPU Port is destination(1)/is not destination(0)
 *                                 for packets with unknown unicast destination
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_NULL_PTR     - Base address or MulticastIdPtr is a null pointer
 * \return  PR_ERR_CHIP_ID      - Chip Id is invalid
 * \ingroup Internal
 */
s32 ifx_get_unknown_ucast_ID(u8 chipId,u32* UnicastIdPtr)
{	
	u32 Temp =0;
	u32* ptr_unknown_dest;  /* pointer to Unknown destination register */

#ifdef INCAIP2_DEBUG_FLOOD
    printk("%s - chipId    = %d\n", __FUNCTION__, chipId);	    
#endif    
#ifdef PARAMETER_CHECK
	if(UnicastIdPtr == NULL)
		return PR_ERR_NULL_PTR;
    
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
          
#endif

   /* read unknown destination register content and modify unicast bits */
	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId]+UNKNOWN_DEST_REG);
	PR_Reg_Read(ptr_unknown_dest,Temp);
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif	
	*UnicastIdPtr = (Temp >> UNKWN_UCASTID_POS) & 0x00000007;
	
	return PR_ERR_NONE ;	
}



/**
 * Set Unknown multicast ID.
 * Set the Id for unknown multicast destination address. 
 *
 * \param   chipId        Chip Id
 * \param   MulticastId   Unknown multicast destination address ID(range is 0-7)
 *                        bit 0 - LAN Port is destination(1)/is not destination(0)
 *                                for packets with unknown multicast destination address
 *                        bit 1 - PC Port is destination(1)/is not destination(0)
 *                                for packets with unknown multicast destination address
 *                        bit 2 - CPU Port is destination(1)/is not destination(0)
 *                                for packets with unknown multicast destination address 
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_UNKN_DEST_ID - If the multicast destination address ID
 *                                is not in the range of 0 to 7
 * \return  PR_ERR_NULL_PTR     - Base address is a null pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_set_unknown_mcast_ID(u8 chipId,u32 MulticastId)
{	
	u32 Temp =0;
	u32* ptr_unknown_dest; /* pointer to Unknown destination register */
	s32 retflag = PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_FLOOD
    printk("%s - chipId = %d, MulticastId=%u\n", \
           __FUNCTION__, chipId, MulticastId);	    
#endif   
#ifdef PARAMETER_CHECK
	if(MulticastId > MAX_UNKN_DEST_ID)
		return PR_ERR_UNKN_DEST_ID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId]+UNKNOWN_DEST_REG);

	if((retflag = ifx_sem_take(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;

   /* read unknown destination register content and modify multicast bits */
	PR_Reg_Read(ptr_unknown_dest,Temp); 
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif    
    Temp &= UNKWN_MCASTID_MASK;
	Temp |= (MulticastId << UNKWN_MCASTID_POS);
	PR_Reg_Write(ptr_unknown_dest,Temp);
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - wrote 0x%08X to UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif 
	retflag = ifx_sem_give(chipId,SEM_FLOOD);
   
   return retflag;
}

/**
 * Get Unknown multicast destination address ID ID.
 * Get the Id for Unknown multicast destination address address. 
 *                
 * \param   chipId        Chip Id
 * \param   MulticastIdPtr Pointer to unknown multicast destination address ID
 *                         bit 0 - LAN Port is destination(1)/is not destination(0)
 *                                 for packets with unknown multicast destination
 *                         bit 1 - PC Port is destination(1)/is not destination(0)
 *                                 for packets with unknown multicast destination
 *                         bit 2 - CPU Port is destination(1)/is not destination(0)
 *                                 for packets with unknown multicast destination
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_NULL_PTR     - Base address or MulticastIdPtr is a null pointer
 * \return  PR_ERR_CHIP_ID      - Chip Id is invalid
 * \ingroup Internal
 */
s32 ifx_get_unknown_mcast_ID( u8 chipId, u32* MulticastIdPtr )
{	
	u32 Temp =0;
	u32* ptr_unknown_dest; /* pointer to Unknown destination register */

#ifdef PARAMETER_CHECK
	if(MulticastIdPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

   /* read unknown destination register content and extract multicast id field */
	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId]+UNKNOWN_DEST_REG);
	PR_Reg_Read(ptr_unknown_dest,Temp); 
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif       
	*MulticastIdPtr = (Temp >> UNKWN_MCASTID_POS) & 0x00000007;
	
	return PR_ERR_NONE ;	
}


/**
 * Set broadcast ID.
 * Set the Id for broadcast destination address. 
 *
 * \param   chipId        Chip Id
 * \param   BroadcastId   Broadcast destination address ID(range is 0-7)
 *                        bit 0 - LAN Port is destination(1)/is not destination(0)
 *                                for broadcast packets
 *                        bit 1 - PC Port is destination(1)/is not destination(0)
 *                                for broadcast packets
 *                        bit 2 - CPU Port is destination(1)/is not destination(0)
 *                                for broadcast packets
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_UNKN_DEST_ID - If the broadcast destination address ID
 *                                is not in the range of 0 to 7
 * \return  PR_ERR_NULL_PTR     - Base address is a null pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_set_bcast_ID( u8 chipId, u32 BroadcastId )
{	
	u32 Temp =0;
	u32* ptr_unknown_dest;  /* pointer to Unknown destination register */
	s32 retflag = PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_FLOOD
    printk("%s - chipId = %d, BroadcastId=%u\n", \
           __FUNCTION__, chipId, BroadcastId);	    
#endif   
#ifdef PARAMETER_CHECK	
	if(BroadcastId > MAX_UNKN_DEST_ID)
		return PR_ERR_UNKN_DEST_ID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId] + UNKNOWN_DEST_REG);

	if((retflag = ifx_sem_take(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;
      
   /* read unknown destination register content and modify broadcast bits */
	PR_Reg_Read(ptr_unknown_dest,Temp); 
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif
	Temp &= BCASTID_MASK;
	Temp |= BroadcastId;
	PR_Reg_Write(ptr_unknown_dest,Temp);
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - wrote 0x%08X to UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif 
	if((retflag = ifx_sem_give(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;
	
	return PR_ERR_NONE ;	

}


/**
 * Get broadcast ID.
 * Get the Id for broadcast destination address. 
 *
 * \param   chipId        Chip Id
 * \param   BroadcastIdPtr Pointer to Broadcast destination address ID
 *                         bit 0 - LAN Port is destination(1)/is not destination(0)
 *                                 for broadcast packets
 *                         bit 1 - PC Port is destination(1)/is not destination(0)
 *                                 for broadcast packets
 *                         bit 2 - CPU Port is destination(1)/is not destination(0)
 *                                 for broadcast packets
 * \return  PR_ERR_NONE         - Success
 * \return  PR_ERR_NULL_PTR     - Base address is a null pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_get_bcast_ID(u8 chipId,u32* BroadcastIdPtr)
{
	u32 Temp =0;
	u32* ptr_unknown_dest;  /* pointer to Unknown destination register */

#ifdef PARAMETER_CHECK
	if(BroadcastIdPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

	/* read unknown destination register content and extract broadcast id field */
	ptr_unknown_dest = (u32*) (Chip_Base_Address[chipId] + UNKNOWN_DEST_REG);
	PR_Reg_Read(ptr_unknown_dest,Temp); 
#ifdef INCAIP2_DEBUG_FLOOD    
    printk("%s - read 0x%08X from UNKNOWN_DEST_REG\n", __FUNCTION__, Temp);
#endif       
  	*BroadcastIdPtr = Temp & 0x00000007;

	return PR_ERR_NONE ;
}

#if 0
/******************************************************************************
  *
  *	FUNCTION:	 ifx_add_flood_entry
  *
  *	DESCRIPTION: Add an entry in the Flood table. 
  *              
  *	PARAMETERS:
  *		IN:
  *         chipId          Chip Id
  *			floodId   		floodId(range is 0-255)
  *			portListPtr		Pointer to port List.Port List indicates whether a port 
  *							is member of the flood map index. Port 0 occupies the least 
  *						    significant bit.
  *		OUT:
  *			None
  * 						 
  *	RETURNS:
  *		PR_ERR_NONE					Success		
  *		PR_ERR_UNKN_DEST_ID		 		If the FloodId is not in the range of 0 to 255	
  *		PR_ERR_NULL_PTR				Null pointer
  *		PR_ERR_TIME_OUT				Time out
  *     PR_ERR_CHIP_ID			    Invalid Chip Id
  *		
  ******************************************************************************/

s32 ifx_add_flood_entry(u8 chipId,u32 floodId, u32* portListPtr)
{

/* Define a pointer to a structure & assign the base address of Flood Table 
   Command register */
	volatile fd_tbl_cmd_reg_t*  ptr_fd_tbl_cmd;

/* Define a pointer to Flood Table data register */
	u32* ptr_fd_tbl_data;
	u32 Temp =0;
	s32 i=0;

	volatile u32 temp1 = 0;
	u32* ptr_reg1;
	s32 retflag = PR_ERR_NONE;


#ifdef PARAMETER_CHECK

	if(floodId > MAX_UNKN_DEST_ID)
		return PR_ERR_UNKN_DEST_ID;

	if(portListPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;

#endif

	ptr_fd_tbl_data = (u32*) (Chip_Base_Address[chipId] + FD_TBL_DATA_REG);

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + FD_TBL_CMD_REG);

	ptr_fd_tbl_cmd = (volatile fd_tbl_cmd_reg_t*)&temp1;

	
/* Write the Port membership for ports 0 to 30 in data register  */

	Temp = *(portListPtr);


/* Configuration of Flood Table Command register for writing */
/* Flood ID(7:0) - 0->255 */
	ptr_fd_tbl_cmd -> FloodId = floodId;

/* Selection(29) - 0 -> Port Membership for ports 0 to 31 */
	ptr_fd_tbl_cmd -> Select = 0;

/* Read/Write(30) : 0-Write */
	ptr_fd_tbl_cmd -> Read_write = WRITEPR;

/* Request/Acknowledge(31): 1-Request */
    ptr_fd_tbl_cmd -> Req_ack = REQ;
	

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;

/* Added MACRO for indirect register write 7/7/2004 */
	PR_Indirect_Reg_Write_Sem(ptr_reg1,ptr_fd_tbl_data,ptr_fd_tbl_cmd,temp1,Temp,SEM_FLOOD);



/* Write the Port membership for ports 32 to 50 in data register  */
	Temp = *(portListPtr+1);

/* Selection(29) - 1 -> Port Membership for ports 32 to 50 */
	ptr_fd_tbl_cmd -> Select = 1;

/* Request/Acknowledge(31): 1-Request */
    ptr_fd_tbl_cmd -> Req_ack = REQ;

/* Added MACRO for indirect register write 7/7/2004 */
	PR_Indirect_Reg_Write_Sem(ptr_reg1,ptr_fd_tbl_data,ptr_fd_tbl_cmd,temp1,Temp,SEM_FLOOD);	

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;

	return PR_ERR_NONE ;

}


/******************************************************************************
  *
  *	FUNCTION:	 ifx_get_flood_entry
  *
  *	DESCRIPTION: Get an entry in the Flood table. 
  *              
  *	PARAMETERS:
  *		IN:
  *         chipId          Chip Id
  *			floodId   		floodId(range is 0-255)
  *		OUT:
  *			portListPtr		Pointer to port List.Port List indicates whether a port 
  *							is member of the flood map index. Port 0 occupies the least 
  *						    significant bit.
  * 						 
  *	RETURNS:
  *		PR_ERR_NONE					Success		
  *		PR_ERR_UNKN_DEST_ID		 		If the FloodId is not in the range of 0 to 255	
  *		PR_ERR_NULL_PTR				Null pointer		
  *		PR_ERR_TIME_OUT				Time out
  *     PR_ERR_CHIP_ID			    Invalid Chip Id
  *
  ******************************************************************************/

s32 ifx_get_flood_entry(u8 chipId,u32 floodId, u32* portListPtr)
{

/* Define a pointer to a structure & assign the base address of Flood Table 
   Command register */
	volatile fd_tbl_cmd_reg_t*  ptr_fd_tbl_cmd;

/* Define a pointer to Flood Table data register */
	u32* ptr_fd_tbl_data;
	u32 Temp =0;
	s32  i=0;

	volatile u32 temp1 = 0;
	u32* ptr_reg1;
	s32 retflag = PR_ERR_NONE;


#ifdef PARAMETER_CHECK

	if(floodId > MAX_UNKN_DEST_ID)
		return PR_ERR_UNKN_DEST_ID;

	if(portListPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;
#endif

	ptr_fd_tbl_data = (u32*) (Chip_Base_Address[chipId] + FD_TBL_DATA_REG);

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + FD_TBL_CMD_REG);

	ptr_fd_tbl_cmd = (volatile fd_tbl_cmd_reg_t*)&temp1;

	
/* Configuration of Flood Table Command register for reading */
/* Flood ID(7:0) - 0->255 */
	//printf("\n");
	ptr_fd_tbl_cmd -> FloodId = floodId;

/* Selection(29) - 0 -> Port Membership for ports 0 to 31 */
	ptr_fd_tbl_cmd -> Select = 0;

/* Read/Write(30) : 1-Read */
	ptr_fd_tbl_cmd -> Read_write = READPR;

/* Request/Acknowledge(31): 1-Request */
    ptr_fd_tbl_cmd -> Req_ack = REQ;
	

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;

/* Added MACRO for indirect register read 7/7/2004 */
	PR_Indirect_Reg_Read_Sem(ptr_reg1,ptr_fd_tbl_data,ptr_fd_tbl_cmd,temp1,Temp,SEM_FLOOD);

/* Read the Port membership for ports 0 to 30 */
	*(portListPtr) = Temp;

/* Selection(29) - 1 -> Port Membership for ports 32 to 50 */
	ptr_fd_tbl_cmd -> Select = 1;

/* Request/Acknowledge(31): 1-Request */
    ptr_fd_tbl_cmd -> Req_ack = REQ;
	
/* Added MACRO for indirect register read 7/7/2004 */
	PR_Indirect_Reg_Read_Sem(ptr_reg1,ptr_fd_tbl_data,ptr_fd_tbl_cmd,temp1,Temp,SEM_FLOOD);

/* Read Port membership for ports 32 to 50 in data register  */
	*(portListPtr+1) = Temp;


/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_FLOOD)) != PR_ERR_NONE)
    		return retflag;


	return PR_ERR_NONE ;

}	
#endif

