/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>

#include <asm/incaip2/ifx_types.h>
#include "switch_api.h"
#include "ioctl.h"
#include "ioctl_struct.h"

#include <vlan.h>
#include <cos.h>
#include <flood.h>
#include <stp.h>
#include <switch.h>
#include <vlan_prv.h>

/**
 * Address resolution IOCTL handler.
 * IOCTL handler for address resolution configuration.
 * The following IOCTLS are supported:
 * - #IFX_MAP_INGRESS_PRIORITY_COS
 * - #IFX_GET_INGRESS_PRIORITY_COS
 * - #IFX_MAP_COS_EGRESS_PRIORITY
 * - #IFX_GET_COS_EGRESS_PRIORITY
 * - #IFX_MAP_COS_REDUCTION
 * - #IFX_GET_COS_REDUCTION
 * - #IFX_VLAN_CHECK
 * - #IFX_SET_VLAN_AWARE
 * - #IFX_GET_VLAN_AWARE
 * - #IFX_GET_VLAN_IDX
 * - #IFX_GET_VLAN_MIB
 *
 * \param   file   - File structure of device
 * \param   cmd    - IOCTL command 
 * \param   arg    - Argument for some IOCTL commands
 * \return  0      - OK
 * \return  ENOTTY - Invalid command
 * \ingroup API  
 */
int addrresl_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
	int err=0;
	/*structures for arg*/
/*    port_stp_state_api pss_api;  */
	cos_pr_api cp_api;
	cos_rd_api cr_api;
	xcast_fid_api xf_api;
/*	id_api i_api;  */
	vlan_aware_api va_api;
/*	pvst_state_api ps_api;  */
	vlan_idx_api vi_api;
/*	vlan_mib_api vm_api;  */
/*	vlan_filter_api vf_api;  */
/*	free_vid_api fv_api;  */
/* 	stpPvst_api sP_api;  */
	mib_idx_api mi_api;
	vlan_status_api vs_api;
/*	vlanStpMstp_api vsm_api;  */
	vlan_tableentry_api vte_api;
    pmac_vlan_api       pm_vlan_api;

	switch(cmd)
	{
/*        
		case IFX_SET_STP_PORT_STATE:
			DPRINTK("%s: IFX_SET_STP_PORT_STATE\n",__FUNCTION__);
			copy_from_user(&pss_api,(port_stp_state_api *) arg,sizeof (port_stp_state_api));
			err = ifx_set_STP_port_state(pss_api.chipId,pss_api.portId,pss_api.portState);
			break;
		case IFX_GET_STP_PORT_STATE:
			DPRINTK("%s: IFX_GET_STP_PORT_STATE\n",__FUNCTION__);
			copy_from_user(&pss_api,(port_stp_state_api *) arg,sizeof (port_stp_state_api));
			err = ifx_get_STP_port_state(pss_api.chipId,pss_api.portId,&pss_api.portState);
			if(err == 0)
		              copy_to_user((port_stp_state_api *) arg, &pss_api, sizeof(port_stp_state_api));
			break;
*/            
		case IFX_MAP_INGRESS_PRIORITY_COS:
			DPRINTK("%s: IFX_MAP_INGRESS_PRIORITY_COS\n",__FUNCTION__);
			copy_from_user(&cp_api,(cos_pr_api *) arg,sizeof (cos_pr_api));
			err = ifx_map_ingress_priority_CoS(cp_api.chipId,cp_api.Priority,cp_api.CoS);
			break;
		case IFX_GET_INGRESS_PRIORITY_COS:
			DPRINTK("%s: IFX_GET_INGRESS_PRIORITY_COS\n",__FUNCTION__);
			copy_from_user(&cp_api,(cos_pr_api *) arg,sizeof (cos_pr_api));
			err = ifx_get_ingress_priority_CoS(cp_api.chipId,cp_api.Priority, &cp_api.CoS);
			if(err == 0)
		              copy_to_user((cos_pr_api *) arg, &cp_api, sizeof(cos_pr_api));
			break;
		case IFX_MAP_COS_EGRESS_PRIORITY:
			DPRINTK("%s: IFX_MAP_COS_EGRESS_PRIORITY\n",__FUNCTION__);
			copy_from_user(&cp_api,(cos_pr_api *) arg,sizeof (cos_pr_api));
			err=ifx_map_CoS_egress_priority(cp_api.chipId,cp_api.CoS, cp_api.Priority);
			break;
		case IFX_GET_COS_EGRESS_PRIORITY:
			DPRINTK("%s: IFX_GET_COS_EGRESS_PRIORITY\n",__FUNCTION__);
			copy_from_user(&cp_api,(cos_pr_api *) arg,sizeof (cos_pr_api));
			err=ifx_get_CoS_egress_priority(cp_api.chipId,cp_api.CoS,&cp_api.Priority);
			if(err == 0)
                copy_to_user((cos_pr_api *) arg, &cp_api, sizeof(cos_pr_api));
			break;
		case IFX_MAP_COS_REDUCTION:
			DPRINTK("%s: IFX_MAP_COS_REDUCTION\n",__FUNCTION__);
			copy_from_user(&cr_api,(cos_rd_api *) arg,sizeof (cos_rd_api));
			err=ifx_map_CoS_reduction(cr_api.chipId,cr_api.CoS,cr_api.reduction);
			break;
		case IFX_GET_COS_REDUCTION:
			DPRINTK("%s: IFX_GET_COS_REDUCTION\n",__FUNCTION__);
			copy_from_user(&cr_api,(cos_rd_api *) arg,sizeof (cos_rd_api));
            err=ifx_get_CoS_reduction(cr_api.chipId,cr_api.CoS,&cr_api.reduction );
            if(err == 0)
                copy_to_user((cos_rd_api *) arg, &cr_api, sizeof(cos_rd_api));
            break;
        case IFX_SET_UNKNOWN_UCAST_ID:
            DPRINTK("%s: IFX_SET_UNKNOWN_UCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_set_unknown_ucast_ID(xf_api.chipId,xf_api.floodId);
            break;
        case IFX_GET_UNKNOWN_UCAST_ID: 
            DPRINTK("%s: IFX_GET_UNKNOWN_UCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_get_unknown_ucast_ID(xf_api.chipId,&xf_api.floodId);
            if(err == 0)
                copy_to_user((xcast_fid_api *) arg, &xf_api, sizeof(xcast_fid_api));
            break;
        case IFX_SET_UNKNOWN_MCAST_ID:
            DPRINTK("%s: IFX_SET_UNKNOWN_MCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_set_unknown_mcast_ID(xf_api.chipId,xf_api.floodId);
            break;
        case IFX_GET_UNKNOWN_MCAST_ID:
            DPRINTK("%s: IFX_GET_UNKNOWN_MCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_get_unknown_mcast_ID(xf_api.chipId,&xf_api.floodId);
            if(err == 0)
                copy_to_user((xcast_fid_api *) arg, &xf_api, sizeof(xcast_fid_api));
            break;
		case IFX_SET_BCAST_ID: 
            DPRINTK("%s: IFX_SET_BCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_set_bcast_ID(xf_api.chipId,xf_api.floodId);
            break;
        case IFX_GET_BCAST_ID:
            DPRINTK("%s: IFX_GET_BCAST_ID\n",__FUNCTION__);
            copy_from_user(&xf_api,(xcast_fid_api *) arg,sizeof (xcast_fid_api));
            err = ifx_get_bcast_ID(xf_api.chipId,&xf_api.floodId);
            if(err == 0)
                copy_to_user((xcast_fid_api *) arg, &xf_api, sizeof(xcast_fid_api));
            break;
/*            
		case IFX_ADD_FLOOD_ENTRY:
			DPRINTK("%s: IFX_ADD_FLOOD_ENTRY\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_add_flood_entry(i_api.chipId,i_api.Id, i_api.portList);
			break;
		case IFX_GET_FLOOD_ENTRY:
			DPRINTK("%s: IFX_GET_FLOOD_ENTRY\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_get_flood_entry(i_api.chipId,i_api.Id, i_api.portList);
			if(err == 0)
		              copy_to_user((id_api *) arg, &i_api, sizeof(id_api));
			break;
*/
        case IFX_VLAN_CHECK:
			DPRINTK("%s: IFX_VLAN_CHECK\n",__FUNCTION__);
			copy_from_user(&vs_api,(vlan_status_api *) arg,sizeof (vlan_status_api));
			err = ifx_vlan_check(vs_api.chipId,vs_api.vlanId,&vs_api.flag);
			if(err == 0)
		              copy_to_user((vlan_status_api *) arg, &vs_api, sizeof(vlan_status_api));
			break;
/*
        case IFX_GET_VLAN_STP_MSTP:
			DPRINTK("%s: IFX_GET_VLAN_STP_MSTP\n",__FUNCTION__);
			copy_from_user(&vsm_api,(vlanStpMstp_api *) arg,sizeof (vlanStpMstp_api));
			err = ifx_get_VLAN_STP_MSTP(vsm_api.chipId,vsm_api.vlanId,&vsm_api.stp_mst);
			if(err == 0)
		              copy_to_user((vlanStpMstp_api *) arg, &vsm_api, sizeof(vlanStpMstp_api));
			break;
		case IFX_GET_VLAN_VID_SHARE:
			DPRINTK("%s: IFX_GET_VLAN_VID_SHARE\n",__FUNCTION__);
			copy_from_user(&vs_api,(vlan_status_api *) arg,sizeof (vlan_status_api));
			err = ifx_get_VLAN_VID_share(vs_api.chipId,vs_api.vlanId,&vs_api.flag);
			if(err == 0)
		              copy_to_user((vlan_status_api *) arg, &vs_api, sizeof(vlan_status_api));
			break;

		case IFX_SET_VLAN_TABLE:
			DPRINTK("%s: IFX_SET_VLAN_TABLE\n",__FUNCTION__);
			copy_from_user(&vte_api,(vlan_tableentry_api *) arg,sizeof (vlan_tableentry_api));
			err = ifx_set_vlan_table(vte_api.chipId,
                                     vte_api.vlanId,
                                     (u32*)&vte_api.portmemberList,
                                     (u32*)&vte_api.portegressList,
                                     vte_api.mstp_stp,
                                     vte_api.shar_ind);
			break;
*/
		case IFX_ADD_VLAN_TABLE_ENTRY:
			DPRINTK("%s: IFX_ADD_VLAN_TABLE_ENTRY\n",__FUNCTION__);
			copy_from_user(&vte_api,(vlan_tableentry_api *) arg,sizeof (vlan_tableentry_api));
			err = ifx_add_vlan_entry(vte_api.chipId,
                                     vte_api.vlanId,
                                     (u32*)&vte_api.portmemberList,
                                     (u32*)&vte_api.portegressList, 
                                     vte_api.valid );
			break;
        case IFX_DEL_VLAN_TABLE_ENTRY:
			DPRINTK("%s: IFX_DEL_VLAN_TABLE_ENTRY\n",__FUNCTION__);
			copy_from_user(&vte_api,(vlan_tableentry_api *) arg,sizeof (vlan_tableentry_api));
			err = ifx_delete_vlan_entry(vte_api.chipId,
                                        vte_api.vlanId );
			break;
            
		case IFX_GET_VLAN_TABLE_ENTRY: 
			DPRINTK("%s: IFX_GET_VLAN_TABLE_ENTRY\n",__FUNCTION__);
			copy_from_user(&vte_api,(vlan_tableentry_api *) arg,sizeof (vlan_tableentry_api));
			err = ifx_get_vlan_table_entry(vte_api.chipId,
                                           vte_api.vlanId,
                                           (u32*)&vte_api.portmemberList,
                                           (u32*)&vte_api.portegressList,
                                           &vte_api.valid);
			if(err == 0)
		        copy_to_user((vlan_tableentry_api *) arg, &vte_api, sizeof(vlan_tableentry_api));
            DPRINTK("%s: IFX_GET_VLAN_TABLE_ENTRY failed, returned %d\n",__FUNCTION__, err);
			break;
            
		case IFX_SET_VLAN_AWARE: 
			DPRINTK("%s: IFX_SET_VLAN_AWARE\n",__FUNCTION__);
			copy_from_user(&va_api,(vlan_aware_api *) arg,sizeof (vlan_aware_api));
			err = ifx_set_VLAN_aware(va_api.chipId,va_api.VlanAwareFlag);
			break;
		case IFX_GET_VLAN_AWARE: 
			DPRINTK("%s: IFX_GET_VLAN_AWARE\n",__FUNCTION__);
			copy_from_user(&va_api,(vlan_aware_api *) arg,sizeof (vlan_aware_api));
			err = ifx_get_VLAN_aware(va_api.chipId,&va_api.VlanAwareFlag);
			if(err == 0)
		        copy_to_user((vlan_aware_api *) arg, &va_api, sizeof(vlan_aware_api));
			break;

		case IFX_SET_PMAC_TAG_INSERTION: 
			DPRINTK("%s: IFX_SET_PMAC_TAG_INSERTION\n",__FUNCTION__);
			copy_from_user(&pm_vlan_api,(pmac_vlan_api *) arg,sizeof (pmac_vlan_api));
			err = ifx_pmac_vlan_tag_insertion( pm_vlan_api.chipId, pm_vlan_api.en_dis, 
                                               pm_vlan_api.VlanId, pm_vlan_api.prio, 
                                               0 /* cfi */ );
			break;

/*
        case IFX_SET_PVST_PORT_STATE: 
			DPRINTK("%s: IFX_SET_PVST_PORT_STATE\n",__FUNCTION__);
			copy_from_user(&ps_api,(pvst_state_api *) arg,sizeof (pvst_state_api));
			err = ifx_set_PVST_port_state(ps_api.chipId,ps_api.vlanId, ps_api.portId,ps_api.portState);
			break;
		case IFX_GET_PVST_PORT_STATE: 
			DPRINTK("%s: IFX_GET_PVST_PORT_STATE\n",__FUNCTION__);
			copy_from_user(&ps_api,(pvst_state_api *) arg,sizeof (pvst_state_api));
			err = ifx_get_PVST_port_state(ps_api.chipId,ps_api.vlanId,ps_api.portId, &ps_api.portState);
			if(err == 0)
		              copy_to_user((pvst_state_api *) arg, &ps_api, sizeof(pvst_state_api));
			break;
*/
		case IFX_GET_VLAN_IDX: 
			DPRINTK("%s: IFX_GET_VLAN_IDX\n",__FUNCTION__);
			copy_from_user(&vi_api,(vlan_idx_api *) arg,sizeof (vlan_idx_api));
			err = ifx_get_VLAN_idx(vi_api.chipId,vi_api.VlanId, &vi_api.VlanIndex);
			if(err == 0)
		              copy_to_user((vlan_idx_api *) arg, &vi_api, sizeof(vlan_idx_api));
			break;
		case IFX_GET_VLAN_MIB: 
			DPRINTK("%s: IFX_GET_VLAN_MIB\n",__FUNCTION__);
			copy_from_user(&mi_api,(mib_idx_api *) arg,sizeof (mib_idx_api));
			err = ifx_get_VLAN_mib(mi_api.chipId,mi_api.mibIdx, mi_api.counterId, &mi_api.count);
			if(err == 0)
		              copy_to_user((mib_idx_api *) arg, &mi_api, sizeof(mib_idx_api));
			break;
/*
        case IFX_SET_VLAN_MIB_IDX: 
			DPRINTK("%s: IFX_SET_VLAN_MIB_IDX\n",__FUNCTION__);
			copy_from_user(&vm_api,(vlan_mib_api *) arg,sizeof (vlan_mib_api));
			err = ifx_set_VLAN_MIB_idx(vm_api.chipId,vm_api.VlanMIBIdx);
			break;
		case IFX_GET_VLAN_MIB_IDX: 
			DPRINTK("%s: IFX_GET_VLAN_MIB_IDX\n",__FUNCTION__);
			copy_from_user(&vm_api,(vlan_mib_api *) arg,sizeof (vlan_mib_api));
			err = ifx_get_VLAN_MIB_idx(vm_api.chipId,&vm_api.VlanMIBIdx);
			if(err == 0)
		              copy_to_user((vlan_mib_api *) arg, &vm_api, sizeof(vlan_mib_api));
			break;

        case IFX_SET_VLAN_FILTER_ID: 
			DPRINTK("%s: IFX_SET_VLAN_FILTER_ID\n",__FUNCTION__);
			copy_from_user(&vf_api,(vlan_filter_api *) arg,sizeof (vlan_filter_api));
			err = ifx_set_VLAN_filter_id(vf_api.chipId,vf_api.vlanIdx, vf_api.filterId);
			break;
		case IFX_GET_VLAN_FILTER_ID: 
			DPRINTK("%s: IFX_GET_VLAN_FILTER_ID\n",__FUNCTION__);
			copy_from_user(&vf_api,(vlan_filter_api *) arg,sizeof (vlan_filter_api));
			err = ifx_get_VLAN_filter_id(vf_api.chipId,vf_api.vlanIdx, &vf_api.filterId);
			if(err == 0)
		              copy_to_user((vlan_filter_api *) arg, &vf_api, sizeof(vlan_filter_api));
			break;
		case IFX_SET_VLAN_PORT_MEMBERS: 
			DPRINTK("%s: IFX_SET_VLAN_PORT_MEMBERS\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_set_VLAN_port_members(i_api.chipId,i_api.Id, i_api.portList);
			break;
		case IFX_GET_VLAN_PORT_MEMBERS: 
			DPRINTK("%s: IFX_GET_VLAN_PORT_MEMBERS\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_get_VLAN_port_members(i_api.chipId,i_api.Id,i_api.portList);
			if(err == 0)
		              copy_to_user((id_api *) arg, &i_api, sizeof(id_api));
			break;
		case IFX_SET_PORT_EGRESS_VLAN_TAG: 
			DPRINTK("%s: IFX_SET_PORT_EGRESS_VLAN_TAG\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_set_port_egress_VLAN_tag(i_api.chipId,i_api.Id, i_api.portList);
			break;
        
		case IFX_GET_PORT_EGRESS_VLAN_TAG: 
			DPRINTK("%s: IFX_GET_PORT_EGRESS_VLAN_TAG\n",__FUNCTION__);
			copy_from_user(&i_api,(id_api *) arg,sizeof (id_api));
			err = ifx_get_port_egress_VLAN_tag(i_api.chipId,i_api.Id, i_api.portList);
			if(err == 0)
		              copy_to_user((id_api *) arg, &i_api, sizeof(id_api));
			break;
          
		case IFX_GET_FREE_VID_LIST: 
			DPRINTK("%s: IFX_GET_FREE_VID_LIST\n",__FUNCTION__);
			copy_from_user(&fv_api,(free_vid_api *) arg,sizeof (free_vid_api));
			err = ifx_get_free_VID_list(fv_api.chipId,fv_api.vidList,&fv_api.free_cnt);
			if(err == 0)
		              copy_to_user((free_vid_api *) arg, &fv_api, sizeof(free_vid_api));
			break;

       case IFX_SET_STP_PVST: 
			DPRINTK("%s: IFX_SET_STP_PVST\n",__FUNCTION__);
			copy_from_user(&sP_api,(stpPvst_api *) arg,sizeof (stpPvst_api));
			err = ifx_set_stp_pvst(sP_api.chipId, sP_api.stpPvst);
			break;
		case IFX_GET_STP_PVST: 
			DPRINTK("%s: IFX_GET_STP_PVST\n",__FUNCTION__);
			copy_from_user(&sP_api,(stpPvst_api *) arg,sizeof (stpPvst_api));
			err = ifx_get_stp_pvst(sP_api.chipId, &sP_api.stpPvst);
			if(err == 0)
		              copy_to_user((stpPvst_api *) arg, &sP_api, sizeof(stpPvst_api));
			break;
*/

			
		default:
            printk("%s - Invalid command 0x%08X passed\n", __FUNCTION__, cmd);
			return -ENOTTY;
			
		break;

	}


    
	return err;
					
}

