/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

 /***********************************************************************
 * This module contains the STP Configuration API's
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup Internal Internally used functions */

#include <asm/types.h>
#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/incaip2.h>
#include "struct.h"
#include "const.h"
#include "stp.h"
#include "stp_prv.h"
#include "err.h"
#include "var.h"
#include "api_init_prv.h"

extern s32 ifx_get_VLAN_aware(u8 chipId,BOOL* VlanAwareFlagPtr);
extern s32 ifx_set_port_ingress_VLAN_filter(u8 chipId,u32 ifx_portId,BOOL IngressFilter);
extern s32 ifx_get_VLAN_idx(u8 chipId,u32 vlanId,u32* vidPtr);
extern s32 ifx_set_port_lock(u8 chipId,u32 portId,BOOL lock);
extern s32 ifx_set_SYM_VLAN(u8 chipId,u32 portId, BOOL IngressFilter);
extern s32 ifx_set_Vlan_table_portstate(u32 chipId,u32 vid,u32 portId, u32 portState);

#if 0
/**
 * Set STP port state.
 * Set the port state for the individual port for all the allocated VID's in 
 * the VLAN table if the port is member of it.
 *
 * STP port states:
 * Disabled   - Manually disabled port. No data is sent or received.
 * Blocking   - If a port causes a loop, it is put into blocking state. No data 
 *              is sent or received.
 * Listening  - The switch tries to determine the network topology
 * Learning   - Incoming packets are learned and put into the MAC table
 * Forwarding - During normal operation the port is in forwarding state. Data is
 *              sent and received
 *
 * \param   chipId            Chip Id
 * \param   portId            PortId(range is 0-2)
 * \param   portState         Port state
 *                              -  0x1  Disabled    
 *                              -  0x2  Blocking 
 *                              -  0x3  Listening
 *                              -  0x4  Learning
 *                              -  0x5  Forward
 * \return  PR_ERR_NONE      - Success                 
 * \return  PR_ERR_PORTID    - If the Port Id is not in the range of 0 to 2       
 * \return  PR_ERR_PORTSTATE - If the Portstate is not in the range of 1 to 5
 * \return  PR_ERR_TIME_OUT  - Time out
 * \return  PR_ERR_CHIP_ID   - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_set_STP_port_state( u8 chipId, u32 portId, u32 portState )
{   
    u32 vid =0;
    s32 retflag =0; 
    u32 vlanId = 0; 
    BOOL VlanAwareFlag;
    BOOL symflag = TRUE;
    u32 ifx_portId = portId;
    
#ifdef PARAMETER_CHECK  
    if(portId > MAX_PORTID) 
        return PR_ERR_PORTID;

    if(portState > MAX_PORTSTATE)
        return PR_ERR_PORTSTATE;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

#endif


    ifx_get_VLAN_aware(chipId,&VlanAwareFlag);


    if((retflag = ifx_sem_take(chipId,SEM_VLAN)) != PR_ERR_NONE)
            return retflag;
    

/* Set the Symmetric Vlan bit to 1 for the specified port if the port state is other than forwarding  */
    if(portState != IFX_PORT_FORWARDING) 
        ifx_set_SYM_VLAN(chipId,ifx_portId,symflag);

/* Writing the portstate in software table */
    PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus((DEFAULT_VID - 1),portId)].port_state = portState;

/* Writing the portstate in Vlan table */
    if((retflag = ifx_set_port_state(chipId,(DEFAULT_VID - 1),ifx_portId,portState)) != PR_ERR_NONE)
    {
        ifx_sem_give(chipId,SEM_VLAN);  
        return retflag;
    }

    for(vlanId = 1;vlanId <= (MAX_VLANID+1);vlanId++)
    {
        if(PR_VLAN_VID_ASC_TBL[chipId][vlanId] == 1)
        {
/* VLANID -> VID */ 
            if(VlanAwareFlag == TRUE)   
                ifx_get_VLAN_idx(chipId,vlanId, &vid);
            else
                vid = PR_VLAN_VID_TBL[chipId][vlanId];
    
            /* Writing the portstate in software table,PR_TABLE_PORTSTATE[1024][51] */
            if((PR_CHIP_MAJR_VER_ID[chipId] == MAJOR_ID_1) && 
               (PR_CHIP_MINR_VER_ID[chipId] == MINOR_ID_3) )
            {
                PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(vid,portId)].port_state = portState;
            
                if((retflag = ifx_set_port_state(chipId,vid,ifx_portId,portState)) != PR_ERR_NONE)
                {
                    ifx_sem_give(chipId,SEM_VLAN);
                    return retflag;
                }                           
            }

            else
            {
                if(PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(vid,portId)].vlanmember == 1)
                {
                    PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(vid,portId)].port_state = portState;
            
                    if((retflag = ifx_set_port_state(chipId,vid,ifx_portId,portState)) != PR_ERR_NONE)
                    {
                        ifx_sem_give(chipId,SEM_VLAN);
                        return retflag;
                    }
                }
            }
        }
    }


    if((retflag = ifx_sem_give(chipId,SEM_VLAN)) != PR_ERR_NONE)
            return retflag;

    
    return PR_ERR_NONE ;
}
#endif 


#if 0
/**
 * Get STP port state.
 * Get the port state for the individual port. 
 *
 * \param   chipId            Chip Id
 * \param   portId            PortId(range is 0-2)
 * \param   portStatePtr      Port state
 *                              -  0x1  Disabled    
 *                              -  0x2  Blocking 
 *                              -  0x3  Listening
 *                              -  0x4  Learning
 *                              -  0x5  Forward
 * \return  PR_ERR_NONE      - Success                 
 * \return  PR_ERR_PORTID    - If the Port Id is not in the range of 0 to 2       
 * \return  PR_ERR_NULL_PTR  - portStatePtr is a null pointer
 * \return  PR_ERR_CHIP_ID   - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_get_STP_port_state( u8 chipId, u32 portId, u32* portStatePtr )
{
    
    s32 retflag = PR_ERR_NONE;

#ifdef PARAMETER_CHECK
    if(portId > MAX_PORTID) 
        return PR_ERR_PORTID;
    
    if(portStatePtr == NULL ) 
        return PR_ERR_NULL_PTR;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

#endif


    if((retflag = ifx_sem_take(chipId,SEM_VLAN)) != PR_ERR_NONE)
            return retflag;
    
/* Reading the portstate from software table */
    *portStatePtr = PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(DEFAULT_VID,portId)].port_state;


    if((retflag = ifx_sem_give(chipId,SEM_VLAN)) != PR_ERR_NONE)
            return retflag;
    
    return PR_ERR_NONE;
}
#endif


/*************************************** Internal API *************************************************/

/**
 * Set port state by VID.
 * Set the port state in the Hardware VLAN table for the specific port 
 * corresponding to VID.
 *
 * \param   chipId            Chip Id
 * \param   vid               VID
 * \param   portId            PortId(range is 0-2)
 * \param   portState         Port state
 *                              -  0x1  Disabled    
 *                              -  0x2  Blocking 
 *                              -  0x3  Listening
 *                              -  0x4  Learning
 *                              -  0x5  Forward
 * \return  PR_ERR_NONE      - Success 
 * \return  PR_ERR_TIME_OUT  - Time out
 * \return  PR_ERR_NULL_PTR  - Base address is invalid
 * \return  PR_ERR_CHIP_ID   - If the chip Id invalid
 * \ingroup Internal
 * \internal
 */
s32 ifx_set_port_state(u8 chipId,u32 vid,u32 portId, u32 portState)
{   
    
    BOOL lock;
    
#ifdef PARAMETER_CHECK

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;
#endif

    

    if((PR_CHIP_MAJR_VER_ID[chipId] == MAJOR_ID_1) && (PR_CHIP_MINR_VER_ID[chipId] == MINOR_ID_3) )
    {
        switch(portState)
        {
            case IFX_PORT_DISABLED:
                    portState = 0x0;
                    break;

            case IFX_PORT_BLOCKING:
                    portState = 0x1;
                    break;
                    
            case IFX_PORT_LISTENING:
                    portState = 0x1;
                    break;

            case IFX_PORT_LEARNING:
                {
                    if(PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(vid,portId)].vlanmember == 1)
                        portState = 0x2;
                    else
                        portState = 0x1;
                    break;
                }

            case IFX_PORT_FORWARDING:
                {
                    if(PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(vid,portId)].vlanmember == 1)
                        portState = 0x3;
                    else
                        portState = 0x1;
                    break;
                }
            default:
                return PR_ERR_PORTSTATE;
            
        }
            
    }
    else
    {
/* Added this part of the code on 6/3/03 to over come the Port state dependency Hard ware problem  */
/* Disabled(1) - Set port state to 2(10) and set port Locking  */
/* Blocking(2) & Listening(3) - Set port state to 2(10) and set port locking */
/* Learning(4) - Set port state to 2(10) */
/* Forwarding(5) - Set port state to 3(11) */

        lock = (portState > IFX_PORT_LISTENING) ? FALSE : TRUE;
        ifx_set_port_lock(chipId,portId,lock);

        portState = (portState < IFX_PORT_FORWARDING) ? 2 : 3;

    }


/* Added the new Internal API call for accessing the hardware vlan table on 13/7/2004 */

    return (ifx_set_Vlan_table_portstate(chipId,vid,portId,portState));


}

