/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Manamohan shetty		5/8/2002						Phase I Release
 *                      4/12/2002                       Phase II Release 
 *
 *
 * Manamohan shetty     10/4/2003                 		Changes:
 *														Added one more parameter to ifx_rcv_CPU_data
 *														for tag information.      
 *
 *                      03/06/2003                      Included pr_const.h that contains all constant 
 *                                                      declerations  			
 *                      24/11/2003                      Add support for B-DMA
 *
 * Manamohan			10/8/2004						Added support for re-entrancy
 *
 * DESCRIPTION :This module contains the CPU Port Packet I/O API's
 ******************************************************************************/
#include <asm/types.h>
#include <asm/addrspace.h>
#include "const.h"
#include "struct.h"
#include <asm/incaip2/incaip2.h>
#include "cpuport.h"
#include "err.h"
#include "var.h"
#include "api_init_prv.h"



/******************************************************************************
  *
  *	FUNCTION:		ifx_set_autocast_state
  *
  *	DESCRIPTION:	Set the CPU Autocast state.Whenever the autocast bit is set  
  *					all mirrored packets to the CPU are truncated to 64 Bytes.
  *
  *	PARAMETERS:
  *		IN:
  *         ChipId    	ChipId.
  *			stateFlag		CPU Autocast state flag
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE			  Success					
  *		PR_ERR_AUTOCAST_FLAG  Auto cast flag is invalid
  *     PR_ERR_CHIP_ID        Invaid Chip Id
  *     PR_ERR_NULL_PTR       Null Pointer
  *
  ****************************************************************************/
#if 0
s32 ifx_set_autocast_state(u8 chipId,bool stateFlag )
{
	cmac_ctx_ctrl_reg_t* ptr_cmac_ctx_ctrl;		/* Pointer to the CMAC TRX struct */
	u32 temp1 =0;
	u32* ptr_reg1;

	s32 retflag = PR_ERR_NONE;

#ifdef PARAMETER_CHECK
	
	if((stateFlag != TRUE) && (stateFlag != FALSE))
		return PR_ERR_AUTOCAST_FLAG;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + CMAC_CTX_CTRL_REG);

	ptr_cmac_ctx_ctrl = (cmac_ctx_ctrl_reg_t*)&temp1;	

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	PR_Reg_Read(ptr_reg1,temp1);

	
	switch(stateFlag)
	{
	case FALSE:	
		ptr_cmac_ctx_ctrl->autocast = 0; /*	Clear the autocast flag */
		break;

	case TRUE:	
		ptr_cmac_ctx_ctrl->autocast = 1; /* The autocast flag has to be set */
		break;

	default:	
		return PR_ERR_AUTOCAST_FLAG;
	}

	PR_Reg_Write(ptr_reg1,temp1);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	return PR_ERR_NONE;
}	

/******************************************************************************
  *
  *	FUNCTION:		ifx_get_autocast_state
  *
  *	DESCRIPTION:	Get CPU Autocast state.If autocast is set then all packet to 
  *					CPU are only 64 bytes. 
  *
  *	PARAMETERS:
  *		IN:
  *			 ChipId    	ChipId.	
  *		OUT:
  *			stateFlagPtr	Pointer to CPU Autocast state flag 
  *
  *	RETURNS:
  *		PR_ERR_NONE			Success		
  *		PR_ERR_NULL_PTR		Null pointer
  *		PR_ERR_CHIP_ID      Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_get_autocast_state(u8 chipId,bool* stateFlagPtr)
{

	cmac_ctx_ctrl_reg_t* ptr_cmac_ctx_ctrl;		/* Pointer to the CMAC TRX struct */
	
	u32 temp1 =0;
	u32* ptr_reg1;

#ifdef PARAMETER_CHECK

	if(stateFlagPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + CMAC_CTX_CTRL_REG);	

	ptr_cmac_ctx_ctrl = (cmac_ctx_ctrl_reg_t*)&temp1;

	PR_Reg_Read(ptr_reg1,temp1);
	

	*stateFlagPtr = ptr_cmac_ctx_ctrl->autocast;	/* Value assignment */

	return PR_ERR_NONE;
}	
#endif

/******************************************************************************
  *
  *	FUNCTION:		ifx_set_Mbox_command
  *
  *	DESCRIPTION:	Set the command pending bit in the Mail box status register 
  *					by writing to command register.  
  *					 
  *
  *	PARAMETERS:
  *		IN:
  *         ChipId    	ChipId.
  *			mboxid		Mailbox Id(range is 0-3)
  *			mboxCmd		Mailbox command
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		 PR_ERR_NONE				Success	
  *		 PR_ERR_MBOXID				If the MailBox Id is not in the range of 0 to 3
  *      PR_ERR_NULL_PTR            Null Pointer
  *      PR_ERR_CHIP_ID             Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_set_Mbox_command(u8 chipId,u32 mboxId,u32 mboxCmd)
{
	u32 *ptr_mbox_cmd_reg;

#ifdef PARAMETER_CHECK	
	if(mboxId > TOTAL_MAILBOX)				/* mboxid whose valid range is 0-3	*/
		return PR_ERR_MBOXID;
	
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif
	
	switch (mboxId)
	{
	
	case 0:	/* MBOX 1 */
		ptr_mbox_cmd_reg = (u32*)(Chip_Base_Address[chipId] + MB1_CMD_REG);
		break;

	case 1:	/* MBOX 2 */
		ptr_mbox_cmd_reg = (u32*)(Chip_Base_Address[chipId] + MB2_CMD_REG);
		break;

	case 2:	/* MBOX 3 */
		ptr_mbox_cmd_reg = (u32*)(Chip_Base_Address[chipId] + MB3_CMD_REG);
		break;

	case 3:	/* MBOX 4 */
		ptr_mbox_cmd_reg = (u32*)(Chip_Base_Address[chipId] + MB4_CMD_REG);
		break;

	default:	
		return PR_ERR_MBOXID;
		
	}

	PR_Reg_Write(ptr_mbox_cmd_reg,mboxCmd);
	
	return PR_ERR_NONE;

}	


/******************************************************************************
  *
  *	FUNCTION:		ifx_set_Mbox_owner
  *
  *	DESCRIPTION:	Set the mail box owner. 
  *
  *	PARAMETERS:
  *		IN:
  *		    ChipId    	ChipId.
  *			mboxid				Mailbox Id(range is 0-3)
  *         mboxOwnerId			Mailbox owner Id(range is 0-7)
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE				Success
  *		PR_ERR_MBOX_BUSY		Mailbox is in use
  *		PR_ERR_MBOXID			If the MailBox Id is not in the range of 0 to 3
  *     PR_ERR_NULL_PTR         Null Pointer
  *     PR_ERR_CHIP_ID          Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_set_Mbox_owner(u8 chipId,u32 mboxId,u32 mboxOwnerId)
{
	mbox_stat_reg_t* ptr_mbox_stat_reg;
	u32 temp1 =0;
	u32* ptr_reg1;


#ifdef PARAMETER_CHECK
	
	if(mboxId > TOTAL_MAILBOX)	/* mboxid valid range is 0-3	*/
		return PR_ERR_MBOXID;
		
	if((mboxOwnerId < 0) || (mboxOwnerId > MAX_MBOXOWNERID))	/*	mboxOwnerId valid range is 0-7. */
		return PR_ERR_OWNERID_INVALID;	

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif
	

	switch (mboxId)
	{
	case 0:	/* MBOX 1	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB1_STAT_REG);
		break;

	case 1:	/* MBOX 2	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB2_STAT_REG);
		
		break;

	case 2:	/* MBOX 3	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB3_STAT_REG);
		
		break;

	case 3:	/* MBOX 4	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB4_STAT_REG);
		
		break;

	default:	
		return PR_ERR_MBOXID;
		
	}

	PR_Reg_Read(ptr_reg1,temp1);
	ptr_mbox_stat_reg = (mbox_stat_reg_t*)&temp1;



/*	Mbox buzy set or DMA active set then mboxOwnerId cannot be changed */
	if ((ptr_mbox_stat_reg->MB_busy == 0) && (ptr_mbox_stat_reg->DMA_active == 0))	
		ptr_mbox_stat_reg->OwnerId = mboxOwnerId;
	else
		return PR_ERR_MBOX_BUSY;


	PR_Reg_Write(ptr_reg1,temp1);


		
	return PR_ERR_NONE;
} 



/******************************************************************************
  * 
  *	FUNCTION:		ifx_get_Mbox_state
  * 
  *	DESCRIPTION:	Get the mailbox state.
  *					 
  *	PARAMETERS:
  *		IN:
  *         ChipId    	ChipId.
  *			mboxid			Mailbox Id(range is 0-3)
  *		OUT:	
  *			mboxStatPtr		Pointer to Mailbox State
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success	
  *		PR_ERR_MBOXID				If the MailBox Id is not in the range of 0 to 3
  *		PR_ERR_NULL_PTR				Null pointer
  *		PR_ERR_CHIP_ID              Invalid Chip Id
  *		
  ****************************************************************************/

s32 ifx_get_Mbox_state(u8 chipId,u32 mboxId,u32* mboxStatePtr)
{
	u32* ptr_mbox_stat_reg;
	u32 Temp=0;


#ifdef PARAMETER_CHECK

	if(mboxStatePtr == NULL)
		return PR_ERR_NULL_PTR;
	
	if(mboxId > TOTAL_MAILBOX)	/* mboxid valid range is 0-3	*/
		return PR_ERR_MBOXID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

	switch (mboxId)
	{
	case 0:	/* MBOX 1 */
		ptr_mbox_stat_reg = (u32*)(Chip_Base_Address[chipId] + MB1_STAT_REG);
		break;

	case 1:	/* MBOX 2 */
		ptr_mbox_stat_reg = (u32*)(Chip_Base_Address[chipId] + MB2_STAT_REG);
		break;

	case 2:	/* MBOX 3 */
		ptr_mbox_stat_reg = (u32*)(Chip_Base_Address[chipId] + MB3_STAT_REG);
		break;

	case 3:	/* MBOX 4 */
		ptr_mbox_stat_reg = (u32*)(Chip_Base_Address[chipId] + MB4_STAT_REG);
		break;

	default:	
		return PR_ERR_MBOXID;
		
	}
	
	PR_Reg_Read(ptr_mbox_stat_reg,Temp);

	*mboxStatePtr = Temp;
	
	return	PR_ERR_NONE;
}



/******************************************************************************
  * 
  *	FUNCTION:		ifx_send_CPU_data
  * 
  *	DESCRIPTION:	Transmit one packet to CPU port.As part of this API,CPU writes the
  *					packet to mailbox 1 or 3 depending upon the criticality and sets the
  *					command pending bit in the mail box status register.CMAC reads the
  *					data from mailbox 1 or 3 and transfers the packet to CRX.Once the 
  *					packet transfer is complete,reset the command pending bit.
  *					 
  *	PARAMETERS:
  *		IN:
  *			ChipId			ChipId.
  *			txPktPtr		Pointer to Transmit packet buffer
  *			txSize			Size of Transmit packet buffer
  *			txPriority		0 (non-critical), 1 (critical)
  *			crcGenFlag		enable/disable ASIC CRC Generation
  *			destTypeFlag	0 (portID), 1(floodID)
  *			destID			portID or FloodID
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success
  *     PR_ERR_CPU_PORT             Unable to set the Command Pending bit
  *		PR_ERR_NULL_PTR				Null pointer
  *		PR_ERR_TIME_OUT				Time out
  *     PR_ERR_CHIP_ID              Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_send_CPU_data(u8 chipId,u8* txPktPtr,u32 txSize,u32 txPriority,bool crcGenFlag,
					    bool destTypeFlag, u32 destID,bool enDaLookup)
{

	u32* ptr_mailbox;

	u32 mboxId =0;
    	u32 mboxState=0;

	u32 i=0;
	u32 Temp =0;
	s32 timeout = PR_TIME_OUT;

	s32 retflag = PR_ERR_NONE;

/* Pointer to Destination lookup structure*/
	dst_lookup_reg_t* ptr_dst_lookup_reg;

/* Pointer to CMAC CRX structure */
	cmac_crx_ctrl_reg_t* ptr_cmac_crx_ctrl;

	u32 temp1=0;
	u32 temp2=0;
	u32* ptr_reg1;
	u32* ptr_reg2;


#ifdef PARAMETER_CHECK
	if(txPktPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif


	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + CMAC_CRX_CTRL_REG);

	ptr_reg2 = (u32*)(Chip_Base_Address[chipId] + DST_LOOKUP_REG);

	ptr_cmac_crx_ctrl = (cmac_crx_ctrl_reg_t*)&temp1;
	PR_Reg_Read(ptr_reg1,temp1);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_CPU_TX)) != PR_ERR_NONE)
    		return retflag;


	ptr_dst_lookup_reg = (dst_lookup_reg_t*)&temp2;
	PR_Reg_Read(ptr_reg2,temp2);

/* Intialization of Mailbox owner ID */
	ifx_set_Mbox_owner(chipId,MAILBOX_1,0);
	ifx_set_Mbox_owner(chipId,MAILBOX_3,1);


/* Configuration of CMAC_CRX_CONTROL register  */
	if((crcGenFlag == TRUE) && (txSize < PACKET_LEN_60))
	{
		ptr_cmac_crx_ctrl -> Pkt_padding = 1;
		ptr_cmac_crx_ctrl -> Pkt_len = PACKET_LEN_64;
	}
	else
		if((crcGenFlag == FALSE) && (txSize < PACKET_LEN_64))
		{
			ptr_cmac_crx_ctrl -> Pkt_padding = 1;
			ptr_cmac_crx_ctrl -> Pkt_len = PACKET_LEN_64;
		}
		else
			if((crcGenFlag == TRUE) && (txSize >= PACKET_LEN_60))
			{
				ptr_cmac_crx_ctrl -> Pkt_padding = 0;
				ptr_cmac_crx_ctrl -> Pkt_len = txSize + 4;
			}
			else
				if((crcGenFlag == FALSE) && (txSize >= PACKET_LEN_64))
				{
					ptr_cmac_crx_ctrl -> Pkt_padding = 0;
					ptr_cmac_crx_ctrl -> Pkt_len = txSize;
				}

/* Configuration of Destination lookup register*/

	ptr_dst_lookup_reg->En_ma_learn = 0;	/*  Normal Source Address learning */

	ptr_dst_lookup_reg->En_da_lookup = enDaLookup;	/*  No Normal destination lookup */

	ptr_dst_lookup_reg->Da_crit = txPriority;	/* Set Criticalness */

/*	ptr_dst_lookup_reg->Da_dest = destID;  */

#if 0

	switch(destTypeFlag)					/*  destination is multicast or not.*/
	{
	case FALSE:
		ptr_dst_lookup_reg->Da_is_mcast = 0;
		break;

	case TRUE:
		ptr_dst_lookup_reg->Da_is_mcast = 1;
		break;

	default:
		ifx_sem_give(chipId,SEM_CPU_TX);
		return PR_ERR_CPU_PORT;
	}
#endif
	ptr_dst_lookup_reg->Da_state = 1;		/*  Valid State for MA Entry */

	switch(crcGenFlag)						/*  CRC Generation */
	{
	case FALSE:
		ptr_cmac_crx_ctrl->Crc_gen = 0;
		break;

	case TRUE:
		ptr_cmac_crx_ctrl->Crc_gen = 1;
		break;

	default:
		ifx_sem_give(chipId,SEM_CPU_TX);
		return PR_ERR_CPU_PORT;
	}

	PR_Reg_Write(ptr_reg1,temp1);

	PR_Reg_Write(ptr_reg2,temp2);




/* Select the mailbox depending upon the transmission priority */
	if(txPriority == 1)
	{
		mboxId = 0;
		ptr_mailbox = (u32*)(Chip_Base_Address[chipId] + MBOX1_MEM);  /* Mailbox 1 */
	 	
	}
	else
	{
		mboxId = 2;
		ptr_mailbox = (u32*) (Chip_Base_Address[chipId] + MBOX3_MEM);  /* Mailbox 3 */
		
	}

	


/* Wait till the MB_busy bit is reset to access the mail box */
	do
	{
		if(ifx_get_Mbox_state(chipId,mboxId,&mboxState) != PR_ERR_NONE)
		{
			ifx_sem_give(chipId,SEM_CPU_TX);
			return PR_ERR_CPU_PORT;
		}
		timeout--;
					
	}while(((mboxState & 0x1) == 1) && (timeout > 0));

	if(timeout == 0)
	{
		ifx_sem_give(chipId,SEM_CPU_TX);
		return PR_ERR_TIME_OUT;
	}
#ifdef DMA_USE
	
#ifdef SWAP_ON //if the external CPU works in differnt endian mode than Purple.

	//All buffer must be swapped
	for(i = 0; i < txSize/4; ++i)
	{
		PR_Reg_Read(((u32*)txPktPtr +i), Temp);
		*((u32*) txPktPtr + i) = Temp;
	}

	if(txSize % 4 != 0)
	{
		PR_Reg_Read(((u32*)txPktPtr +i), Temp);
		*((u32*) txPktPtr + i) = Temp;
	}
	
#endif	

	DmaDataTransfer(ptr_mailbox, txSize, false /*send PCI ---> FPI*/, 
					txPktPtr, Chip_Base_Address[chipId]);
#ifdef SWAP_ON //if the external CPU works in differnt endian mode than Purple.
	//All buffer must be swapped for correct print on the screen
	for(i = 0; i < txSize/4; ++i)
	{
		PR_Reg_Read(((u32*)txPktPtr +i), Temp);
		*((u32*) txPktPtr + i) = Temp;
	}

	if(txSize % 4 != 0)
	{
		PR_Reg_Read(((u32*)txPktPtr +i), Temp);
		*((u32*) txPktPtr + i) = Temp;
	}
	
#endif	
#else // there is no DMA acitvization is required
/* Write the data to Mailbox  */	
	for(i = 0; i < txSize/4; i++,txPktPtr += 4)
	{
		Temp = *((u32*)txPktPtr);
		PR_Reg_Write((ptr_mailbox+i),Temp);
	}
	

	if(txSize % 4 != 0)
	{
		Temp = *((u32*)txPktPtr);	
		PR_Reg_Write((ptr_mailbox+i),Temp);
	}
#endif // DMA_USE
			
/* Set the command pending bit in the MBOX Status Register */
	if(ifx_set_Mbox_command(chipId,mboxId,i) != PR_ERR_NONE)
	{
		ifx_sem_give(chipId,SEM_CPU_TX);
		return PR_ERR_CPU_PORT;
	}


/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_CPU_TX)) != PR_ERR_NONE)
    		return retflag;

	return PR_ERR_NONE;
}


/******************************************************************************
  * 
  *	FUNCTION:		ifx_rcv_CPU_data
  * 
  *	DESCRIPTION:	Receive one packet from CPU port.
  *					 
  *	PARAMETERS:
  *		IN:
  *			 ChipId    	ChipId.
  *		OUT:
  *			txPktPtr		Pointer to the receive packet buffer			
  *			txSizePtr		pointer to the receive packet size
  *			SrcIdPtr		pointer to Source Port ID  
  *			timeStampPtr	pointer to the time stamp of the packet
  *         tagPtr			pointer to Tag(TRUE - tagged packet,FALSE - untagged packet) 
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success	
  *		PR_ERR_CPU_PORT		        Time out
  *		PR_ERR_NULL_PTR				Null pointer
  *		PR_ERR_TIME_OUT				Time out
  *     PR_ERR_CHIP_ID              Invalid Chip Id
  *
  ****************************************************************************/


s32 ifx_rcv_CPU_data(u8 chipId,u8* rxPktPtr,u32* rxSizePtr,u32* SrcIdPtr,u32* timeStampPtr,bool* tagPtr)
{	
	
	u32 mboxId2=1,mboxId4=3;
    u32 mboxId =0;
	u32 mboxState2=0;
	u32 mboxState4=0;
    u32 value =0; 
	u32* ptr_mailbox;
	u32 i =0;
	bool stateFlag = FALSE ;

	s32 retflag = PR_ERR_NONE; 

/* added by Bingtao to avoid endless waiting*/
	int timeout = PR_TIME_OUT;

#ifdef PARAMETER_CHECK
	if(rxPktPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(rxSizePtr == NULL)
		return PR_ERR_NULL_PTR;
	if(SrcIdPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(timeStampPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_CPU_RX)) != PR_ERR_NONE)
    		return retflag;


/* Intialization of Mailbox owner ID */	
	ifx_set_Mbox_owner(chipId,MAILBOX_2,0);
	ifx_set_Mbox_owner(chipId,MAILBOX_4,1);
	
/* Wait till cmd_Pending bit is set either for MBOX2 or MBOX4 */		
	do
	{
		if(ifx_get_Mbox_state(chipId,mboxId2,&mboxState2) != PR_ERR_NONE)
		{
			ifx_sem_give(chipId,SEM_CPU_RX);
			return PR_ERR_CPU_PORT;
		}
		
		if(ifx_get_Mbox_state(chipId,mboxId4,&mboxState4) != PR_ERR_NONE)
		{
			ifx_sem_give(chipId,SEM_CPU_RX);
			return PR_ERR_CPU_PORT;
		}

		timeout--;				
	}while((((mboxState2 >> COMMAND_PEND_POS) & 0x1) == 0) && (((mboxState4 >> COMMAND_PEND_POS) & 0x1) == 0) && (timeout>0));
/* added by Bingtao */
	if (timeout == 0)
	{ 
		ifx_sem_give(chipId,SEM_CPU_RX);
		return PR_ERR_TIME_OUT;
	}

	if(((mboxState2 >> COMMAND_PEND_POS) & 0x1) == 1)
	{	

		ptr_mailbox = (u32*)(Chip_Base_Address[chipId] + MBOX2_MEM);  /* Mailbox 2 */
		mboxId = 1;
	}
	else
	{
		ptr_mailbox = (u32*)(Chip_Base_Address[chipId] + MBOX4_MEM);  /* Mailbox 4 */ 
		mboxId = 3;
	}

/* Extract the SPID,packet length,Timestamp from the Packet Header */

/* SPID*/
/* SPID(1:0)->Word0(7-6),SPID(5:2)->Word0(11-8) */

	PR_Reg_Read(ptr_mailbox,value);
	*SrcIdPtr = (value & 0xfc0)>>6; 

/* TAG -> Word0(31) */
	if((value & 0x80000000) != 0)
		*tagPtr = TRUE;
	else
		*tagPtr = FALSE;


/* Packet length*/
/* PKT_LEN(7:0)->Word1(7-0),PKT_LEN(13:8)->Word1(13-8) */
	ptr_mailbox++;

	ifx_get_autocast_state(chipId,&stateFlag);
    if(stateFlag == TRUE)
		*rxSizePtr = 64;
	else
	{
		PR_Reg_Read(ptr_mailbox,value);
		*rxSizePtr = (value & 0x3fff);
	}

/* Timestamp*/
/* TIME_STAMP(31:0)->Word2(31-0) */

	ptr_mailbox++;
	PR_Reg_Read(ptr_mailbox,value);
	*timeStampPtr = LONGSWAP_P(value);

#ifdef DMA_USE

	DmaDataTransfer(++ptr_mailbox, *rxSizePtr, true /*receive FPI ---> PCI*/, 
					rxPktPtr, Chip_Base_Address[chipId]);

#ifdef SWAP_ON //if the external CPU works in differnt endian mode than Purple.
	
	//All buffer must be swapped
	for(i = 0; i < *rxSizePtr/4; ++i)
	{
		PR_Reg_Read(((u32*)rxPktPtr +i), Temp);
		*((u32*) rxPktPtr + i) = Temp;
	}

	if(*rxSizePtr % 4 != 0)
	{
		PR_Reg_Read(((u32*)rxPktPtr +i), Temp);
		*((u32*) rxPktPtr + i) = Temp;
	}

#endif	
#else // there is no DMA acitvization is required
/* Read the data from MBOX and write it to receive buffer */
	ptr_mailbox++;
   	for(i = 0; i < *rxSizePtr/4; i++)
	{
		PR_Reg_Read(ptr_mailbox,value);
		*(u32 *)rxPktPtr = value;
		ptr_mailbox++;
		rxPktPtr = rxPktPtr+4;
	}

	if(*rxSizePtr % 4 != 0)
	{
		PR_Reg_Read(ptr_mailbox,value);
		*(u32 *)rxPktPtr = value;
	}
#endif // DMA_USE

/* Clear the Command pending bit in the MBOX status register */
	if(ifx_clear_Mbox_command(chipId,mboxId) != PR_ERR_NONE)
	{
		ifx_sem_give(chipId,SEM_CPU_RX);
		return PR_ERR_CPU_PORT;
	}

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_CPU_RX)) != PR_ERR_NONE)
   		return retflag;

	return PR_ERR_NONE;
}




/******************************************************************************
  * 
  *	FUNCTION:		ifx_clear_Mbox_command
  * 
  *	DESCRIPTION:	Clear the command pending bit in the Mail box status register.
  *					 
  *	PARAMETERS:
  *		IN:
  *			ChipId    	ChipId.
  *			mboxid			Mailbox Id(range is 0-3)
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success	
  *		PR_ERR_MBOXID				If the MailBox Id is not in the range of 0 to 3
  *     PR_ERR_NULL_PTR             Null Pointer
  *     PR_ERR_CHIP_ID              Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_clear_Mbox_command(u8 chipId,u32 mboxId)
{
	mbox_stat_reg_t* ptr_mbox_stat_reg;

	u32 temp1 =0;
	u32* ptr_reg1;

	
#ifdef PARAMETER_CHECK

	if(mboxId > TOTAL_MAILBOX)	/* mboxid valid range is 0-3 */
		return PR_ERR_MBOXID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif
		

	switch (mboxId)
	{
	case 0:	/* MBOX 1	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB1_STAT_REG);
			
		break;

	case 1:	/* MBOX 2	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB2_STAT_REG);
		
		break;

	case 2:	/* MBOX 3	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB3_STAT_REG);
	
		break;

	case 3:	/* MBOX 4	*/
		ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + MB4_STAT_REG);
	
		break;

	default:	
		return PR_ERR_MBOXID;
		
	}

	PR_Reg_Read(ptr_reg1,temp1);
	ptr_mbox_stat_reg = (mbox_stat_reg_t*)&temp1;


/* Clear the Command pending bit in the MBOX status register */
	
	ptr_mbox_stat_reg->Cmd_pending = 0;

	PR_Reg_Write(ptr_reg1,temp1);
		
	return PR_ERR_NONE;

}

/******************************************************************************
  * 
  *	FUNCTION:		ifx_set_autocast_time_stamp
  * 
  *	DESCRIPTION:	Set CPU Autocast Time Stamp.
  *					 
  *	PARAMETERS:
  *		IN:
  *			ChipId    	ChipId.
  *			timeStamp			Time stamp(range is 0-2)
  *								0 -> millisecond
  *								1 -> second
  *								2 -> minute
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success	
  *		PR_ERR_TIMESTMP				If the Time stamp is invalid
  *		PR_ERR_NULL_PTR             Null Pointer
  *		PR_ERR_CHIP_ID              Invalid Chip Id
  *
  ****************************************************************************/

s32 ifx_set_autocast_time_stamp(u8 chipId,u32 timeStamp)
{

	cmac_ctx_ctrl_reg_t* ptr_cmac_ctx_ctrl;		/* Pointer to the CMAC TRX struct */

	u32 temp1 =0;
	u32* ptr_reg1;

	s32 retflag = PR_ERR_NONE;

#ifdef PARAMETER_CHECK
	if(timeStamp > MAX_TIMESTAMP)
		return PR_ERR_TIMESTMP;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + CMAC_CTX_CTRL_REG);

	ptr_cmac_ctx_ctrl = (cmac_ctx_ctrl_reg_t*)&temp1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	PR_Reg_Read(ptr_reg1,temp1);
	
	ptr_cmac_ctx_ctrl ->Cp_timestamp = timeStamp;

	PR_Reg_Write(ptr_reg1,temp1);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;


	return PR_ERR_NONE;
}

/******************************************************************************
  * 
  *	FUNCTION:		ifx_get_autocast_time_stamp
  * 
  *	DESCRIPTION:	Get CPU Autocast Time Stamp.
  *					 
  *	PARAMETERS:
  *		IN:
  *			ChipId    	ChipId.
  *			timeStampPtr		Pointer to Time stamp(range is 0-2)
  *								0 -> millisecond
  *								1 -> second
  *								2 -> minute
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE					Success	
  *		PR_ERR_NULL_PTR				Null pointer
  *		Pr_ERR_CHIP_ID              Invalid Chip ID
  *
  ****************************************************************************/

s32 ifx_get_autocast_time_stamp(u8 chipId,u32* timeStampPtr)
{	
		
	cmac_ctx_ctrl_reg_t* ptr_cmac_ctx_ctrl;		/* Pointer to the CMAC TRX struct */
	u32 temp1 =0;
	u32* ptr_reg1;

#ifdef PARAMETER_CHECK
	if(timeStampPtr == NULL)
		return PR_ERR_NULL_PTR;
	
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + CMAC_CTX_CTRL_REG);

	
	ptr_cmac_ctx_ctrl = (cmac_ctx_ctrl_reg_t*)&temp1;

	PR_Reg_Read(ptr_reg1,temp1);
	

	*timeStampPtr = ptr_cmac_ctx_ctrl ->Cp_timestamp;

	return PR_ERR_NONE;
}



	





