/*
 * Copyright (c) 2002, Infineon Technologies AG, All rights reserved
 * NO WARRANTY
 * BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
 * THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
 * ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH
 * YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
 * NECESSARY SERVICING, REPAIR OR CORRECTION.
 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
 * WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
 * REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR
 * DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM
 * (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING RENDERED
 * INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF
 * THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR
 * OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>

#include <asm/incaip2/ifx_types.h>					     
#include "switch_api.h"
#include "ioctl.h"
#include "ioctl_struct.h"

#include <cpuport.h>

int cpuport_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
	int err=0;
	autocast_api au_api;
	mbox_api mb_api;
	mbox_id_api mi_api;
	auto_timestamp_api ats_api;
	sendCpuData_api scd_api;
	rcvCpuData_api rcd_api;
	
	switch(cmd)
	{
		case IFX_SET_AUTOCAST_STATE:
			DPRINTK("%s: IFX_SET_AUTOCAST_STATE\n",__FUNCTION__);
			copy_from_user(&au_api,(autocast_api *) arg,sizeof (autocast_api));
			err = ifx_set_autocast_state(au_api.chipId,au_api.stateFlag );
		       break;
		case IFX_GET_AUTOCAST_STATE: 
			DPRINTK("%s: IFX_GET_AUTOCAST_STATE\n",__FUNCTION__);
			copy_from_user(&au_api,(autocast_api *) arg,sizeof (autocast_api));
			err = ifx_get_autocast_state(au_api.chipId,&au_api.stateFlag);
			if(err == 0)
		              copy_to_user((autocast_api *) arg, &au_api, sizeof(autocast_api));
		       break;
		case IFX_SET_MBOX_COMMAND:
			DPRINTK("%s: IFX_SET_MBOX_COMMAND\n",__FUNCTION__);
			copy_from_user(&mb_api,(mbox_api *) arg,sizeof (mbox_api));
			err = ifx_set_Mbox_command(mb_api.chipId,mb_api.mboxId,mb_api.mboxData);
		       break;
		case IFX_SET_MBOX_OWNER:
			DPRINTK("%s: IFX_SET_MBOX_OWNER\n",__FUNCTION__);
			copy_from_user(&mb_api,(mbox_api *) arg,sizeof (mbox_api));
			err = ifx_set_Mbox_owner(mb_api.chipId,mb_api.mboxId,mb_api.mboxData);
		       break;
		case IFX_GET_MBOX_STATE:
			DPRINTK("%s: IFX_GET_MBOX_STATE\n",__FUNCTION__);
			copy_from_user(&mb_api,(mbox_api *) arg,sizeof (mbox_api));
			err = ifx_get_Mbox_state(mb_api.chipId,mb_api.mboxId,&mb_api.mboxData);
			if(err == 0)
		              copy_to_user((mbox_api *) arg, &mb_api, sizeof(mbox_api));
		       break;
		case IFX_SEND_CPU_DATA:
		       DPRINTK("%s: IFX_SEND_CPU_DATA\n",__FUNCTION__);
		       copy_from_user(&scd_api,(sendCpuData_api *) arg,sizeof (sendCpuData_api));
		       err = ifx_send_CPU_data(scd_api.chipId,scd_api.txPktPtr,scd_api.txSize,scd_api.txPriority,
				       scd_api.crcGenFlag,scd_api.destTypeFlag, scd_api.destID,scd_api.enDaLookup);
		       break;
		case IFX_RCV_CPU_DATA: 
			DPRINTK("%s: IFX_RCV_CPU_DATA\n",__FUNCTION__);
			copy_from_user(&rcd_api,(rcvCpuData_api *) arg,sizeof (rcvCpuData_api));
			err = ifx_rcv_CPU_data(rcd_api.chipId,rcd_api.rxPktPtr,&rcd_api.rxSize,&rcd_api.SrcId,
					&rcd_api.timeStamp,&rcd_api.tagPtr);
			if(err == 0)
		              copy_to_user((rcvCpuData_api *) arg, &rcd_api, sizeof(rcvCpuData_api));
		       break;
		case IFX_CLEAR_MBOX_COMMAND:
			DPRINTK("%s: IFX_CLEAR_MBOX_COMMAND\n",__FUNCTION__);
			copy_from_user(&mi_api,(mbox_id_api *) arg,sizeof (mbox_id_api));
			err = ifx_clear_Mbox_command(mi_api.chipId,mi_api.mboxId);
		       break;
		case IFX_SET_AUTOCAST_TIME_STAMP: 
			DPRINTK("%s: IFX_SET_AUTOCAST_TIME_STAMP\n",__FUNCTION__);
			copy_from_user(&ats_api,(auto_timestamp_api *) arg,sizeof (auto_timestamp_api));
			err = ifx_set_autocast_time_stamp(ats_api.chipId,ats_api.timeStamp);
		       break;
		case IFX_GET_AUTOCAST_TIME_STAMP: 
			DPRINTK("%s: IFX_GET_AUTOCAST_TIME_STAMP\n",__FUNCTION__);
			copy_from_user(&ats_api,(auto_timestamp_api *) arg,sizeof (auto_timestamp_api));
			err = ifx_get_autocast_time_stamp(ats_api.chipId,&ats_api.timeStamp);
			if(err == 0)
		              copy_to_user((auto_timestamp_api *) arg, &mb_api, sizeof(auto_timestamp_api));
		       break;
		default:
			return -ENOTTY;
		break;

	}
					
	return err;
}

