/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

#ifndef __GLOBAL_H
#define __GLOBAL_H   

#include "depend.h"

/* Swap function for PCI buses */	

	#define LLSB_P(x)	((x) & 0xff)		/* 32bit word byte/word swap macros */
	#define LNLSB_P(x) (((x) >> 8) & 0xff)  /* LNLSB_P defined in vxWorks.h */
	#define LNMSB_P(x) (((x) >> 16) & 0xff)
	#define LMSB_P(x)	 (((x) >> 24) & 0xff)
	#define LONGSWAP_P(x) ((LLSB_P(x) << 24) | \
		     (LNLSB_P(x) << 16)| \
		     (LNMSB_P(x) << 8) | \
		     (LMSB_P(x)))

#ifdef	SWAP_ON

	#define	PR_Reg_Write(address,reg)  (*address) =  LONGSWAP_P(reg)
	#define	PR_Reg_Read(address,reg) reg= (*address);reg =  LONGSWAP_P(reg)    

#else

	#define	PR_Reg_Write(address,reg) (*address) = reg
	#define	PR_Reg_Read(address,reg)   reg = (*address)

#endif


#define PR_Indirect_Reg_Read(ptr_cmd,ptr_data,ptr_struct,cmd,data) \
		PR_Reg_Write(ptr_cmd,cmd); \
		for (i=0; i<PR_TIME_OUT;i++) \
		{ \
			PR_Reg_Read(ptr_cmd,cmd); \
			if(ptr_struct -> Req_ack== ACK) \
			break; \
		} \
		if((ptr_struct -> Req_ack) != ACK) \
			return PR_ERR_TIME_OUT; \
		PR_Reg_Read(ptr_data,data);


#define PR_Indirect_Reg_Write(ptr_cmd,ptr_data,ptr_struct,cmd,data) \
		PR_Reg_Write(ptr_data,data); \
		PR_Reg_Write(ptr_cmd,cmd); \
		for(i=0; i<PR_TIME_OUT;i++) \
		{ \
			PR_Reg_Read(ptr_cmd,cmd); \
			if(ptr_struct -> Req_ack == ACK) \
			break; \
		} \
		if((ptr_struct -> Req_ack) != ACK) \
			return PR_ERR_TIME_OUT;


/* Macro with semaphore release in case of error  */

#define RETURN_ERROR(ifx_sem) \
		{ \
			ifx_sem_give(chipId,ifx_sem); \
			return PR_ERR_TIME_OUT; \
		}

#define PR_Indirect_Reg_Read_Sem(ptr_cmd,ptr_data,ptr_struct,cmd,data,ifx_sem) \
		PR_Reg_Write(ptr_cmd,cmd); \
		for (i=0; i<PR_TIME_OUT;i++) \
		{ \
			PR_Reg_Read(ptr_cmd,cmd); \
			if(ptr_struct -> Req_ack== ACK) \
			break; \
		} \
		if((ptr_struct -> Req_ack) != ACK) \
			RETURN_ERROR(ifx_sem) \
		PR_Reg_Read(ptr_data,data);


#define PR_Indirect_Reg_Write_Sem(ptr_cmd,ptr_data,ptr_struct,cmd,data,ifx_sem) \
		PR_Reg_Write(ptr_data,data); \
		PR_Reg_Write(ptr_cmd,cmd); \
		for(i=0; i<PR_TIME_OUT;i++) \
		{ \
			PR_Reg_Read(ptr_cmd,cmd); \
			if(ptr_struct -> Req_ack == ACK) \
			break; \
		} \
		if((ptr_struct -> Req_ack) != ACK) \
			RETURN_ERROR(ifx_sem)



/*.........................................................
..........Typedef Declerations for all Global Structures......
............................................. .............*/


/******  Software VLAN Table ******/
	
typedef struct	{
				u8	vlanmember:1;
				u8	port_state:3;
				u8   Rsv:4;
				}port_status_per_vlan;


/* MACROs for Software Tables */

#define PR_Table_Portstatus(vid,portId)\
	((vid) * (TOTAL_PORTS) + (portId))


#ifdef LINUX /*for debuging*/
//#define S_API_DEBUG
#ifdef S_API_DEBUG
#define DPRINTK(fmt, args...) printk(KERN_ERR "--- %s: " fmt, __FUNCTION__ , ## args)
#else
#define DPRINTK(fmt, args...)
#endif
#endif
#endif // end of __GLOBAL_H 



