/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup IOCTL Defined IOCTLs */

#ifndef __IOCTL_API
#define __IOCTL_API

#include <linux/ioctl.h>
#include <linux/fs.h>


/*extern int parm;*/
#define IFX_ZERO_SIZE 0
#define IFX_IOC_MAGIC 0xAE
#define IFX_IOC_MAXNR 255

/*IOCTL defines*/
#define IFX_IOC_HARDRESET      _IO(IFX_IOC_MAGIC, 0)

/************************************************ AddrResl ******************************************************/
#define ADDRRESL_BEGIN 1
#define ADDRRESL_END  40
/*38-40 reserved*/

/*STP.h*/
#if 0
/**
 * Set STP port state IOCTL.
 * Calls #ifx_set_STP_port_state(arg.chipId, arg.portId, arg.portState)
 * \param arg #port_stp_state_api
 * \ingroup IOCTL
 */
#define IFX_SET_STP_PORT_STATE  _IOW(IFX_IOC_MAGIC, 1, port_stp_state_api)
/**
 * Get STP port state IOCTL.
 * Calls #ifx_get_STP_port_state(arg.chipId, arg.portId, &arg.portState);
 * \param arg #port_stp_state_api
 * \ingroup IOCTL
 */
#define IFX_GET_STP_PORT_STATE _IOR(IFX_IOC_MAGIC,2,port_stp_state_api)
#endif

/*cos.h*/
/**
 * Map ingress priority class of service IOCTL.
 * Calls #ifx_map_ingress_priority_CoS(arg.chipId, arg.Priority, arg.CoS);
 * \param arg #cos_pr_api
 * \ingroup IOCTL
 */
#define IFX_MAP_INGRESS_PRIORITY_COS _IOW(IFX_IOC_MAGIC,3,cos_pr_api)
/**
 * Get ingress priority class of service IOCTL.
 * Calls #ifx_get_ingress_priority_CoS(arg.chipId, arg.Priority, &arg.CoS);
 * \param arg #cos_pr_api
 * \ingroup IOCTL
 */
#define IFX_GET_INGRESS_PRIORITY_COS _IOR(IFX_IOC_MAGIC,4,cos_pr_api)
/**
 * Map egress priority class of service IOCTL.
 * Calls #ifx_map_CoS_egress_priority(arg.chipId, arg.CoS, arg.Priority);
 * \param arg #cos_pr_api
 * \ingroup IOCTL
 */
#define IFX_MAP_COS_EGRESS_PRIORITY _IOW(IFX_IOC_MAGIC,5,cos_pr_api)
/**
 * Get egress priority class of service IOCTL.
 * Calls #ifx_get_CoS_egress_priority(arg.chipId, arg.CoS, &arg.Priority);
 * \param arg #cos_pr_api
 * \ingroup IOCTL
 */
#define IFX_GET_COS_EGRESS_PRIORITY _IOR(IFX_IOC_MAGIC,6,cos_pr_api)
/**
 * Map class of service reduction IOCTL.
 * Calls #ifx_map_CoS_reduction(arg.chipId, arg.CoS, arg.reduction);
 * \param arg #cos_rd_api
 * \ingroup IOCTL
 */
#define IFX_MAP_COS_REDUCTION _IOW(IFX_IOC_MAGIC,7,cos_rd_api)
/**
 * Get class of service reduction IOCTL.
 * Calls #ifx_get_CoS_reduction(arg.chipId, arg.CoS, &arg.reduction);
 * \param arg #cos_rd_api
 * \ingroup IOCTL
 */
#define IFX_GET_COS_REDUCTION _IOR(IFX_IOC_MAGIC,8,cos_rd_api)

/* flood.h*/
/**
 * Set the Id for unknown unicast destination address. 
 * Calls #ifx_set_unknown_ucast_ID(arg.chipId, arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_SET_UNKNOWN_UCAST_ID _IOW(IFX_IOC_MAGIC,9,IFX_ZERO_SIZE)


/**
 * Get the Id for unknown unicast destination address. 
 * Calls #ifx_get_unknown_ucast_ID(arg.chipId, &arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_GET_UNKNOWN_UCAST_ID _IOR(IFX_IOC_MAGIC,10,IFX_ZERO_SIZE)


/**
 * Set the Id for unknown multicast destination address. 
 * Calls #ifx_set_unknown_mcast_ID(arg.chipId, arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_SET_UNKNOWN_MCAST_ID _IOW(IFX_IOC_MAGIC,11,IFX_ZERO_SIZE)


/**
 * Get the Id for unknown multicast destination address. 
 * Calls #ifx_get_unknown_mcast_ID(arg.chipId, &arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_GET_UNKNOWN_MCAST_ID _IOR(IFX_IOC_MAGIC,12,IFX_ZERO_SIZE)


/**
 * Set the Id for broadcast destination address. 
 * Calls #ifx_set_bcast_ID(arg.chipId, arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_SET_BCAST_ID _IOW(IFX_IOC_MAGIC,13,IFX_ZERO_SIZE)


/**
 * Get the Id for broadcast destination address. 
 * Calls #ifx_get_bcast_ID(arg.chipId, &arg.floodId);
 * \param arg #xcast_fid_api
 * \ingroup IOCTL
 */
#define IFX_GET_BCAST_ID _IOR(IFX_IOC_MAGIC,14,IFX_ZERO_SIZE)


/*vlan.h*/

/**
 * Set VLAN Tag insertion at PMAC IOCTL.
 * Calls #ifx_pmac_vlan_tag_insertion(arg.chipId, arg.en_dis,arg.VlanId,
   arg.prio,0);
 * \param arg #pmac_vlan_api
 * \ingroup IOCTL
 */
#define IFX_SET_PMAC_TAG_INSERTION _IOW(IFX_IOC_MAGIC,16,pmac_vlan_api)


/**
 * Set VLAN awarenes IOCTL.
 * Calls #ifx_set_VLAN_aware(arg.chipId, arg.VlanAwareFlag);
 * \param arg #vlan_aware_api
 * \ingroup IOCTL
 */
#define IFX_SET_VLAN_AWARE _IOW(IFX_IOC_MAGIC,17,vlan_aware_api)
/**
 * Get VLAN awarenes IOCTL.
 * Calls #ifx_get_VLAN_aware(arg.chipId, &arg.VlanAwareFlag);
 * \param arg #vlan_aware_api
 * \ingroup IOCTL
 */
#define IFX_GET_VLAN_AWARE _IOR(IFX_IOC_MAGIC,18,vlan_aware_api)

/**
 * Get VLAN index IOCTL.
 * Calls #ifx_get_VLAN_idx(arg.chipId, arg.vlanId, &arg.VlanIndex);
 * \param arg #vlan_idx_api
 * \ingroup IOCTL
 */
#define IFX_GET_VLAN_IDX _IOR(IFX_IOC_MAGIC,21,vlan_idx_api)
/**
 * Get MIB counters for VLAN/Flow IOCTL.
 * Calls #ifx_get_VLAN_mib(arg.chipId, arg.mibIdx, arg.counterId, &arg.count);
 * \param arg #mib_idx_api
 * \ingroup IOCTL
 */

#define IFX_GET_VLAN_MIB _IOR(IFX_IOC_MAGIC,22,mib_idx_api)
/**
 * Set VLAN MIB start index IOCTL.
 * Calls #ifx_set_VLAN_MIB_idx(arg.chipId, arg.VlanMIBIdx);
 * \param arg #vlan_mib_api
 * \ingroup IOCTL
 */

#if 0  
#define IFX_SET_VLAN_MIB_IDX _IOW(IFX_IOC_MAGIC,23,vlan_mib_api)
/**
 * Get VLAN MIB start index IOCTL.
 * Calls #ifx_get_VLAN_MIB_idx(arg.chipId, &arg.VlanMIBIdx);
 * \param arg #vlan_mib_api
 * \ingroup IOCTL
 */
#define IFX_GET_VLAN_MIB_IDX _IOR(IFX_IOC_MAGIC,24,vlan_mib_api)


/**
 * Set VLAN port members IOCTL.
 * Calls #ifx_set_VLAN_port_members(arg.chipId, arg.Id, arg.portList);
 * \param arg #id_api
 * \ingroup IOCTL
 */
#define IFX_SET_VLAN_PORT_MEMBERS _IOW(IFX_IOC_MAGIC,27,id_api)
/**
 * Get VLAN port members IOCTL.
 * Calls #ifx_get_VLAN_port_members(arg.chipId, arg.Id, arg.portList);
 * \param arg #id_api
 * \ingroup IOCTL
 */
#define IFX_GET_VLAN_PORT_MEMBERS _IOR(IFX_IOC_MAGIC,28,id_api)


/**
 * Get egress port VLAN tag IOCTL.
 * Calls #ifx_get_port_egress_VLAN_tag(arg.chipId, arg.Id, arg.portList);
 * \param arg #id_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_EGRESS_VLAN_TAG _IOR(IFX_IOC_MAGIC,29,id_api)
/**
 * Set egress port VLAN tag IOCTL.
 * Calls #ifx_set_port_egress_VLAN_tag(arg.chipId, arg.Id, arg.portList);
 * \param arg #id_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_EGRESS_VLAN_TAG _IOW(IFX_IOC_MAGIC,30,id_api)


/**
 * Get free VID list IOCTL.
 * Calls #ifx_get_free_VID_list(arg.chipId, arg.vidList, &arg.free_cnt);
 * \param arg #free_vid_api
 * \ingroup IOCTL
 */
#define IFX_GET_FREE_VID_LIST _IOR(IFX_IOC_MAGIC,31,free_vid_api)
#endif

/**
 * Check VLAN existance IOCTL.
 * Calls #ifx_vlan_check(arg.chipId, arg.vlanId, &arg.flag);
 * \param arg #vlan_status_api
 * \ingroup IOCTL
 */
#define IFX_VLAN_CHECK _IOR(IFX_IOC_MAGIC,35,vlan_status_api)

/**
 * Add VLAN table entry IOCTL.
 * Calls #ifx_add_vlan_entry(arg.chipId, arg.vlanId, &arg.portmemberList, 
                            &arg.portegressList, arg.valid);
 * \param arg #vlan_tableentry_api
 * \ingroup IOCTL
 */
#define IFX_ADD_VLAN_TABLE_ENTRY _IOW(IFX_IOC_MAGIC,38,vlan_tableentry_api) 

/**
 * Delete VLAN table entry IOCTL.
 * Calls #ifx_delete_vlan_entry(arg.chipId, arg.vlanId );
 * \param arg #vlan_tableentry_api
 * \ingroup IOCTL
 */
#define IFX_DEL_VLAN_TABLE_ENTRY _IOW(IFX_IOC_MAGIC,39,vlan_tableentry_api) 

/**
 * Get VLAN table entry IOCTL.
 * Calls #ifx_get_vlan_table_entry(vte_api.chipId, vte_api.vlanId,
 *                                 (u32*)&vte_api.portmemberList,
 *                                 (u32*)&vte_api.portegressList,
 *                                  &vte_api.valid);
 * \param arg #vlan_tableentry_api
 * \ingroup IOCTL
 */
#define IFX_GET_VLAN_TABLE_ENTRY _IOR(IFX_IOC_MAGIC,40,vlan_tableentry_api)

/************************************************ Mac ******************************************************/
#define MAC_BEGIN 66
#define MAC_END 75


/*mac.h*/
/**
 * Get MAC table index IOCTL.
 * Calls #ifx_get_MAC_table_idx(arg.macAddr, arg.tableIndex);
 * \param arg #macTidx_api
 * \ingroup IOCTL
 */
#define IFX_GET_MAC_TABLE_IDX _IOR(IFX_IOC_MAGIC,66,macTidx_api)
/**
 * Add MAC table entry with index IOCTL.
 * Calls #ifx_add_MAC_table_entry_idx(arg.chipId, arg.macTableIndex, arg.macAddr,
 *                                    arg.lan_port, arg.pc_port, arg.cpu_port,
 *                                    arg.criticalFlag, arg.maState);
 * \param arg #macTenIdex_api
 * \ingroup IOCTL
 */
#define IFX_ADD_MAC_TABLE_ENTRY_IDX _IOW(IFX_IOC_MAGIC,67,macTenIdex_api)
/**
 * Get MAC table entry index IOCTL.
 * Calls #ifx_get_MAC_table_entry_idx(arg.chipId, arg.macTableIndex, arg.macAddr,
 *                                    arg.lan_port, arg.pc_port, arg.cpu_port,
 *                                    arg.criticalFlag, arg.maState);
 * \param arg #macTenIdex_api
 * \ingroup IOCTL
 */
#define IFX_GET_MAC_TABLE_ENTRY_IDX _IOR(IFX_IOC_MAGIC,68,macTenIdex_api)
#if 0
/**
 * Get MAC table entry hash IOCTL.
 * Calls #ifx_add_MAC_table_entry_hash(arg.chipId, arg.macAddr, arg.mpid,
 *                                    arg.criticalFlag, arg.maState, arg.fId);
 * \param arg #macTenHash_api
 * \ingroup IOCTL
 */
#define IFX_ADD_MAC_TABLE_ENTRY_HASH _IOW(IFX_IOC_MAGIC,69,macTenHash_api)
#endif
/**
 * Get MAC table statistics IOCTL.
 * Calls #ifx_get_MAC_table_stat(arg.chipId, arg.staticCount, arg.valid);
 * \param arg #macTstat_api
 * \ingroup IOCTL
 */
#define IFX_GET_MAC_TABLE_STAT _IOR(IFX_IOC_MAGIC,70,macTstat_api)
/**
 * Add MAC table entry IOCTL.
 * Calls #ifx_add_MAC_table_entry(arg.macAddr, arg.lan_port, arg.pc_port, 
                                  arg.cpu_port, arg.criticalFlag, arg.maState);
 * \param arg #macTenIdex_api
 * \ingroup IOCTL
 */
#define IFX_ADD_MAC_TABLE_ENTRY _IOW(IFX_IOC_MAGIC,71,macTenIdex_api)


/**
 * Control MAC table aging IOCTL.
 * Calls #ifx_add_MAC_table_entry(arg.macAddr, arg.lan_port, arg.pc_port, 
                                  arg.cpu_port, arg.criticalFlag, arg.maState);
 * \param arg #macAging_api
 * \ingroup IOCTL
 */
#define IFX_CONTROL_MAC_TABLE_AGING _IOW(IFX_IOC_MAGIC,72,macAging_api)

/************************************************ Port ******************************************************/
#define PORT_BEGIN 76
#define PORT_END 110
/*104-110 reserved*/


/* fe_port.h */
/**
 * Set Port Fast Ethernet Mode IOCTL.
 * Calls #ifx_set_port_FE_mode( arg.chipId, arg.portId, arg.feature, arg.mode );
 * \param arg #portMode_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_FE_MODE _IOW(IFX_IOC_MAGIC,76,portMode_api)

/**
 * Get Port Fast Ethernet Mode IOCTL.
 * Calls #ifx_get_port_FE_mode arg.chipId, arg.portId, arg.feature, &arg.mode)
 * \param arg #portMode_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_FE_MODE _IOR(IFX_IOC_MAGIC,77,portMode_api)


/*ge_port.h*/
/**
 * Set Port Gigabit Ethernet Mode IOCTL.
 * Calls #ifx_set_port_GE_mode( arg.chipId, arg.portId, arg.feature, arg.mode );
 * \param arg #portMode_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_GE_MODE _IOW(IFX_IOC_MAGIC,78,portMode_api)

/**
 * Get Port Gigabit Ethernet Mode IOCTL.
 * Calls #ifx_get_port_GE_mode arg.chipId, arg.portId, arg.feature, &arg.mode)
 * \param arg #portMode_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_GE_MODE _IOR(IFX_IOC_MAGIC,79,portMode_api)


/*monitoring.h*/
/**
 * Set ingress monitor flag IOCTL.
 * Calls #ifx_set_ingress_monitor_flag(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_INGRESS_MONITOR_FLAG _IOW(IFX_IOC_MAGIC,81,Flag_api)
/**
 * Get ingress monitor flag IOCTL.
 * Calls #ifx_get_ingress_monitor_flag(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_INGRESS_MONITOR_FLAG _IOR(IFX_IOC_MAGIC,82,Flag_api)
/**
 * Set egress monitor flag IOCTL.
 * Calls #ifx_set_egress_monitor_flag(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_EGRESS_MONITOR_FLAG _IOW(IFX_IOC_MAGIC,83,Flag_api)
/**
 * Get egress monitor flag IOCTL.
 * Calls #ifx_get_egress_monitor_flag(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_EGRESS_MONITOR_FLAG _IOR(IFX_IOC_MAGIC,84,Flag_api)

/*phy.h*/
/**
 * Read PHY register IOCTL.
 * Calls #ifx_phy_read(arg.chipId, arg.mdio_id, arg.phy_addr, arg.RegOffset, &arg.Data);
 * \param arg #phyAction_api
 * \ingroup IOCTL
 */
#define IFX_PHY_READ _IOR(IFX_IOC_MAGIC,85,phyAction_api)
/**
 * Write PHY register IOCTL.
 * Calls #ifx_phy_write(arg.chipId, arg.mdio_id, arg.phy_addr, arg.RegOffset, arg.Data);
 * \param arg #phyAction_api
 * \ingroup IOCTL
 */
#define IFX_PHY_WRITE _IOW(IFX_IOC_MAGIC,86,phyAction_api)




/*ports.h*/
/**
 * Set port locking bit IOCTL.
 * Calls #ifx_set_port_lock(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_LOCK _IOW(IFX_IOC_MAGIC,87,Flag_api)

/**
 * Get port locking bit IOCTL.
 * Calls #ifx_get_port_lock(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_LOCK _IOR(IFX_IOC_MAGIC,88,Flag_api)

/**
 * Set port VLAN ID IOCTL.
 * Calls #ifx_set_port_VLANId(arg.chipId, arg.portId, arg.vlanid);
 * \param arg #portVlan_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_VLANID _IOW(IFX_IOC_MAGIC,89,portVlan_api)

/**
 * Get port VLAN ID IOCTL.
 * Calls #ifx_get_port_VLANId(arg.chipId, arg.portId, &arg.vlanid);
 * \param arg #portVlan_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_VLANID _IOR(IFX_IOC_MAGIC,90,portVlan_api)

/**
 * Set VLAN Ingress Tag IOCTL.
 * Calls #ifx_set_port_ingress_VLAN_tag(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_INGRESS_VLAN_TAG _IOW(IFX_IOC_MAGIC,91,Flag_api)

/**
 * Get VLAN Ingress Tag IOCTL.
 * Calls #ifx_get_port_ingress_VLAN_tag(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_INGRESS_VLAN_TAG _IOR(IFX_IOC_MAGIC,92,Flag_api)

/**
 * Set VLAN Ingress Filter IOCTL.
 * Calls #ifx_set_port_ingress_VLAN_filter(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_INGRESS_VLAN_FILTER _IOW(IFX_IOC_MAGIC,93,Flag_api)

/**
 * Get VLAN Ingress Filter IOCTL.
 * Calls #ifx_get_port_ingress_VLAN_filter(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_INGRESS_VLAN_FILTER _IOR(IFX_IOC_MAGIC,94,Flag_api)

 /**
 * Set default CoS IOCTL.
 * Calls #ifx_set_port_CoS(arg.chipId, arg.portId, arg.CoS);
 * \param arg #portCoS_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_COS _IOW(IFX_IOC_MAGIC,95,portCoS_api)

 /**
 * Get default CoS IOCTL.
 * Calls #ifx_get_port_CoS(arg.chipId, arg.portId, &arg.CoS);
 * \param arg #portCoS_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_COS _IOR(IFX_IOC_MAGIC,96,portCoS_api)

/**
 * Set Jumbo Enable flag IOCTL.
 * Calls #ifx_set_port_jumbo_enable(arg.chipId, arg.portId, arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_JUMBO_ENABLE _IOW(IFX_IOC_MAGIC,97,Flag_api)
/**
 * Get Jumbo Enable flag IOCTL.
 * Calls #ifx_get_port_jumbo_enable(arg.chipId, arg.portId, &arg.flag);
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_JUMBO_ENABLE _IOR(IFX_IOC_MAGIC,98,Flag_api)


/**
 * Get Port Configuration IOCTL.
 * Calls #ifx_get_port_default_configuration(arg.chipId, arg.portId, &arg.CoS));
 * \param arg #Flag_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_CONFIGURATION _IOR(IFX_IOC_MAGIC,99,portCoS_api)


/**
 * Get MDIO and autonegotiation mode IOCTL.
 * Calls #ifx_get_MDIO_mode(arg.chipId, arg.port_group, &arg.mdioMode, &arg.autoNegMode);
 * \param arg #mdiomode_api
 * \ingroup IOCTL
 */
#define IFX_GET_MDIO_MODE _IOR(IFX_IOC_MAGIC,100,mdiomode_api)
/**
 * Set MDIO and autonegotiation mode IOCTL.
 * Calls #ifx_set_MDIO_mode(arg.chipId, arg.port_group, arg.mdioMode, arg.autoNegMode);
 * \param arg #mdiomode_api
 * \ingroup IOCTL
 */
#define IFX_SET_MDIO_MODE _IOW(IFX_IOC_MAGIC,101,mdiomode_api)


/*MIB_counters.h*/
/**
 * Get MIB counters for MAC IOCTL.
 * Calls #ifx_get_mac_MIB_counters(arg.chipId, arg.portId, arg.counterId, &arg.count);
 * \param arg #mibCounters_api
 * \ingroup IOCTL
 */
#define IFX_GET_MAC_MIB_COUNTERS _IOR(IFX_IOC_MAGIC,102,mibCounters_api)

/**
 * Get MIB counters for port IOCTL.
 * Calls #ifx_get_port_MIB_counters(arg.chipId, arg.portId, arg.counterId, &arg.count);
 * \param arg #mibCounters_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_MIB_COUNTERS _IOR(IFX_IOC_MAGIC,103,mibCounters_api)


/**
 * Set Port Ethernet Configuration IOCTL.
 * Calls #ifx_set_port_eth_conf( arg.chipId, arg.portId, arg.speed, arg.duplex,
 *                               arg.autoneg, arg.int_ext_phy);
 * \param arg #port_eth_conf_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_ETH_CONF _IOW(IFX_IOC_MAGIC,104,port_eth_conf_api)

/**
 * Get Port Ethernet Configuration IOCTL.
 * Calls #ifx_get_port_eth_conf( arg.chipId, arg.portId, &arg.speed, &arg.duplex,
 *                               &arg.autoneg, &arg.int_ext_phy, &arg.link_status);
 * \param arg #port_eth_conf_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_ETH_CONF _IOR(IFX_IOC_MAGIC,105,port_eth_conf_api)

/************************************************ QoS ******************************************************/
#define QOS_BEGIN 111
#define QOS_END 150
/*145-150 reserved*/

/*flow_engine.h*/
/**
 * Set port keys IOCTL.
 * Calls #ifx_set_port_key(arg.chipId, arg.portId, arg.IKey);
 * \param arg #portKey_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_KEY _IOW(IFX_IOC_MAGIC,111,portKey_api)
/**
 * Get port keys IOCTL.
 * Calls #ifx_get_port_key(arg.chipId, arg.portId, &arg.IKey);
 * \param arg #portKey_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_KEY _IOR(IFX_IOC_MAGIC,112,portKey_api)

/**
 * Set rule for flow ID IOCTL.
 * Calls #ifx_set_flow_pattern(arg.chipId, arg.flowId, arg.IKeyData, arg.IKeyMask);
 * \param arg #flowPattern_api
 * \ingroup IOCTL
 */
#define IFX_SET_FLOW_PATTERN _IOW(IFX_IOC_MAGIC,113,flowPattern_api)
/**
 * Get rule for flow ID IOCTL.
 * Calls #ifx_get_flow_pattern(arg.chipId, arg.flowId, arg.IKeyData, arg.IKeyMask);
 * \param arg #flowPattern_api
 * \ingroup IOCTL
 */
#define IFX_GET_FLOW_PATTERN _IOR(IFX_IOC_MAGIC,114,flowPattern_api)

/**
 * Delete rule for flow ID IOCTL.
 * Calls #ifx_delete_flow_pattern(arg.chipId, arg.flowId);
 * \param arg #flowId_api
 * \ingroup IOCTL
 */
#define IFX_DELETE_FLOW_PATTERN _IOW(IFX_IOC_MAGIC,115,flowId_api)

/**
 * Set action parameters for flow ID IOCTL.
 * Calls #ifx_set_flow_action_param(arg.chipId, arg.flowId, arg.feature, arg.flowAction);
 * \param arg #flowParam_api
 * \ingroup IOCTL
 */
#define IFX_SET_FLOW_ACTION_PARAM _IOW(IFX_IOC_MAGIC,116,flowParam_api)
/**
 * Get action for flow ID IOCTL.
 * Calls #ifx_get_flow_action_param(arg.chipId, arg.flowId, arg.feature, &arg.flowAction);
 * \param arg #flowParam_api
 * \ingroup IOCTL
 */
#define IFX_GET_FLOW_ACTION_PARAM  _IOR(IFX_IOC_MAGIC,117,flowParam_api)


#define IFX_DELETE_FLOW_ACTION_PARAM  _IOW(IFX_IOC_MAGIC,118,flowParam_api)




/**
 * Set rule for port ID IOCTL.
 * Calls #ifx_set_port_rule(arg.chipId, arg.portId, arg.portRule);
 * \param arg #portRule_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_RULE _IOW(IFX_IOC_MAGIC,120,portRule_api)
/**
 * Get rule for port ID IOCTL.
 * Calls #ifx_get_port_rule(arg.chipId, arg.portId, arg.portRule);
 * \param arg #portRule_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_RULE _IOR(IFX_IOC_MAGIC,121,portRule_api)

/**
 * Set rule mask for port ID IOCTL.
 * Calls #ifx_set_port_mask(arg.chipId, arg.portId, arg.portRule);
 * \param arg #portRule_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_MASK _IOW(IFX_IOC_MAGIC,122,portRule_api)
/**
 * Get rule mask for port ID IOCTL.
 * Calls #ifx_get_port_mask(arg.chipId, arg.portId, arg.portRule);
 * \param arg #portRule_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_MASK _IOR(IFX_IOC_MAGIC,123,portRule_api)

/**
 * Set offset for port ID IOCTL.
 * Calls #ifx_set_port_offset(arg.chipId, arg.portId, arg.portOffset);
 * \param arg #portOffset_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_OFFSET _IOW(IFX_IOC_MAGIC,124,portOffset_api)
/**
 * Get offset for port ID IOCTL.
 * Calls #ifx_get_port_offset(arg.chipId, arg.portId, arg.portOffset);
 * \param arg #portOffset_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_OFFSET _IOR(IFX_IOC_MAGIC,125,portOffset_api)
/**
 * Set the RX configuration IOCTL.
 * Calls #ifx_set_Rx_config(arg.chipId, arg.portId, arg.L3Program, 
 *                          arg.L3StartVld, arg.L3Start);
 * \param arg #RxConfig_api
 * \ingroup IOCTL
 */
#define IFX_SET_RX_CONFIG _IOW(IFX_IOC_MAGIC,126,RxConfig_api)
/**
 * Get the RX configuration IOCTL.
 * Calls #ifx_get_Rx_config(arg.chipId, arg.portId, &arg.L3Program, 
 *                          &arg.L3StartVld, &arg.L3Start);
 * \param arg #RxConfig_api
 * \ingroup IOCTL
 */
#define IFX_GET_RX_CONFIG _IOR(IFX_IOC_MAGIC,127,RxConfig_api)

/*qos.h*/
/**
 * Set values for token bucket ID IOCTL.
 * Calls #ifx_set_QoS_bucket(arg.chipId, arg.tBucketId, arg.tBucketCnt, 
 *                           arg.tReplenishRateIdx, arg.tBurstSizeIdx);
 * \param arg #QoSbucket_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_BUCKET _IOW(IFX_IOC_MAGIC,128,QoSbucket_api)
/**
 * Get values for token bucket ID IOCTL.
 * Calls #ifx_get_QoS_bucket(arg.chipId, arg.tBucketId, &arg.tBucketCnt, 
 *                           &arg.tReplenishRateIdx, &arg.tBurstSizeIdx);
 * \param arg #QoSbucket_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_BUCKET _IOR(IFX_IOC_MAGIC,129,QoSbucket_api)
/**
 * Set flow control for token bucket ID IOCTL.
 * Calls #ifx_set_QoS_bucket_flow_control(arg.chipId, arg.portRuleBaseIdx, 
 *                                        arg.actionLC, arg.actionNC);
 * \param arg #QoSBflowCntr_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_BUCKET_FLOW_CONTROL _IOW(IFX_IOC_MAGIC,130,QoSBflowCntr_api)
/**
 * Get flow control for token bucket ID IOCTL.
 * Calls #ifx_get_QoS_bucket_flow_control(arg.chipId, &arg.portRuleBaseIdx, 
 *                                        &arg.actionLC, &arg.actionNC);
 * \param arg #QoSBflowCntr_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_BUCKET_FLOW_CONTROL _IOR(IFX_IOC_MAGIC,131,QoSBflowCntr_api)
/**
 * Set parameters for WFQ IOCTL.
 * Calls #ifx_set_QoS_WF_queue(arg.chipId, arg.portId, arg.queueId, 
 *                             arg.weightFactor, arg.deficitCount);
 * \param arg #QoSWFqueue_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_WF_QUEUE _IOW(IFX_IOC_MAGIC,132,QoSWFqueue_api)
/**
 * Get parameters for WFQ IOCTL.
 * Calls #ifx_get_QoS_WF_queue(arg.chipId, arg.portId, arg.queueId, 
 *                             &arg.weightFactor, &arg.deficitCount);
 * \param arg #QoSWFqueue_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_WF_QUEUE _IOR(IFX_IOC_MAGIC,133,QoSWFqueue_api)
/**
 * Set parameters for rate shaping IOCTL.
 * Calls #ifx_set_QoS_shaping_queue(arg.chipId, arg.portId, arg.queueId, 
 *                             arg.bucketCnt, arg.replenishRate, arg.tokenValue, 
 *                             arg.tReplenishTimer);
 * \param arg #QoSshapingQ_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_SHAPING_QUEUE _IOW(IFX_IOC_MAGIC,134,QoSshapingQ_api)
/**
 * Get parameters for rate shaping IOCTL.
 * Calls #ifx_get_QoS_shaping_queue(arg.chipId, arg.portId, arg.queueId, 
 *                             &arg.bucketCnt, &arg.replenishRate, &arg.tokenValue, 
 *                             &arg.tReplenishTimer);
 * \param arg #QoSshapingQ_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_SHAPING_QUEUE _IOR(IFX_IOC_MAGIC,135,QoSshapingQ_api)

/**
 * Set token bucket replenish rate IOCTL.
 * Calls #ifx_set_QoS_replenish_rate(arg.chipId, arg.Idx, arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_REPLENISH_RATE _IOW(IFX_IOC_MAGIC,136,idx_api)
/**
 * Get token bucket replenish rate IOCTL.
 * Calls #ifx_get_QoS_replenish_rate(arg.chipId, arg.Idx, &arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_REPLENISH_RATE _IOR(IFX_IOC_MAGIC,137,idx_api)
/**
 * Set token bucket burst size IOCTL.
 * Calls #ifx_set_QoS_burst_size(arg.chipId, arg.Idx, arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_BURST_SIZE _IOW(IFX_IOC_MAGIC,138,idx_api)
/**
 * Get token bucket burst size IOCTL.
 * Calls #ifx_get_QoS_burst_size(arg.chipId, arg.Idx, &arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_BURST_SIZE _IOR(IFX_IOC_MAGIC,139,idx_api)
/**
 * Set token bucket extended burst size IOCTL.
 * Calls #ifx_set_QoS_extend_burst_size(arg.chipId, arg.Idx, arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_SET_QOS_EXTEND_BURST_SIZE _IOW(IFX_IOC_MAGIC,140,idx_api)
/**
 * Get token bucket extended burst size IOCTL.
 * Calls #ifx_get_QoS_extend_burst_size(arg.chipId, arg.Idx, &arg.info);
 * \param arg #idx_api
 * \ingroup IOCTL
 */
#define IFX_GET_QOS_EXTEND_BURST_SIZE _IOR(IFX_IOC_MAGIC,141,idx_api)

/**
 * Get rule memory IOCTL.
 * Calls #ifx_get_rule_mem(arg.chipId, arg.bank, arg.address, arg.section, 
 *                         &arg.rule32_val);
 * \param arg #rule_mem_api
 * \ingroup IOCTL
 */
#define IFX_GET_RULE_MEM _IOR(IFX_IOC_MAGIC,144,rule_mem_api)




/************************************************ Switch ******************************************************/
#define SWITCH_BEGIN 151 
#define SWITCH_END 200
/*195-200 reserved*/
#if 0
/**
 * Get Chip Number IOCTL.
 * Calls #ifx_get_chip_number(arg.chipId);
 * \param arg #chip_num_api
 * \ingroup IOCTL
 */
#define IFX_GET_CHIP_NUMBER _IOR(IFX_IOC_MAGIC,151,chip_num_api)


/**
 * Set Chip Base Adress IOCTL.
 * Calls #ifx_set_base_address(arg.chipId, arg.Base_Address);
 * \param arg #baseAddr_api
 * \ingroup IOCTL
 */
#define IFX_SET_BASE_ADDRESS _IOW(IFX_IOC_MAGIC,152,baseAddr_api)

/**
 * Get Chip Base Adress IOCTL.
 * Calls #ifx_get_base_address(arg.chipId, arg.Base_Address);
 * \param arg #baseAddr_api
 * \ingroup IOCTL
 */
#define IFX_GET_BASE_ADDRESS _IOR(IFX_IOC_MAGIC,153,baseAddr_api)

/**
 * Set Chip Number IOCTL.
 * Calls #ifx_set_chip_number(arg.chipId);
 * \param arg #chip_num_api
 * \ingroup IOCTL
 */
#define IFX_SET_CHIP_NUMBER _IOW(IFX_IOC_MAGIC,154,chip_num_api)
#endif


#define IFX_SET_SWITCH_MODE _IOW(IFX_IOC_MAGIC,155,feature_mode_api)

#define IFX_GET_SWITCH_MODE _IOR(IFX_IOC_MAGIC,156,feature_mode_api)



/**
 * Set Ingress watermark IOCTL.
 * Calls #ifx_set_ingress_watermark(arg.chipId, arg.waterMarkType, 
 *                                  arg.portId, arg.waterMark);
 * \param arg #watermark_api
 * \ingroup IOCTL
 */
#define IFX_SET_INGRESS_WATERMARK _IOW(IFX_IOC_MAGIC,184,watermark_api)
/**
 * Get Ingress watermark IOCTL.
 * Calls #ifx_get_ingress_watermark(arg.chipId, arg.waterMarkType, 
 *                                  arg.portId, &arg.waterMark);
 * \param arg #watermark_api
 * \ingroup IOCTL
 */
#define IFX_GET_INGRESS_WATERMARK _IOR(IFX_IOC_MAGIC,185,watermark_api)
/**
 * Set Egress watermark IOCTL.
 * Calls #ifx_set_egress_watermark(arg.chipId, arg.portId, arg.queueId,
 *                                 arg.waterMark);
 * \param arg #Ewatermark_api
 * \ingroup IOCTL
 */
#define IFX_SET_EGRESS_WATERMARK _IOW(IFX_IOC_MAGIC,186,Ewatermark_api)
/**
 * Get Egress watermark IOCTL.
 * Calls #ifx_get_egress_watermark(arg.chipId, arg.portId, arg.queueId,
 *                                 &arg.waterMark);
 * \param arg #Ewatermark_api
 * \ingroup IOCTL
 */
#define IFX_GET_EGRESS_WATERMARK _IOR(IFX_IOC_MAGIC,187,Ewatermark_api)
/**
 * Set Tx config IOCTL.
 * Calls #ifx_set_Tx_config(arg.chipId, arg.portId, arg.xmitDropCount,
 *                          arg.lateCollisionCount);
 * \param arg #TxConfig_api
 * \ingroup IOCTL
 */
 
 
 
#define IFX_SET_TX_CONFIG _IOW(IFX_IOC_MAGIC,188,TxConfig_api)
/**
 * Get Tx config IOCTL.
 * Calls #ifx_get_Tx_config(arg.chipId, arg.portId, &arg.xmitDropCount,
 *                          &arg.lateCollisionCount);
 * \param arg #TxConfig_api
 * \ingroup IOCTL
 */
#define IFX_GET_TX_CONFIG _IOR(IFX_IOC_MAGIC,189,TxConfig_api)

/**
 * Enable port rate shaping IOCTL.
 * Calls #ifx_set_PORT_rate_shape(arg.chipId, arg.en_dis, arg.PRE);
 * \param arg #rateShape_api
 * \ingroup IOCTL
 */
#define IFX_SET_PORT_RATE_SHAPE _IOW(IFX_IOC_MAGIC,190,rateShape_api)
/**
 * Get port rate shaping flag IOCTL.
 * Calls #ifx_get_PORT_rate_shape(arg.chipId, &arg.PRE);
 * \param arg #rateShape_api
 * \ingroup IOCTL
 */
#define IFX_GET_PORT_RATE_SHAPE _IOR(IFX_IOC_MAGIC,191,rateShape_api)

/**
 * Enable/Disable Pause Frame generation IOCTL.
 * Calls #ifx_set_pause_frame_gen(arg.chipId, arg.portId, arg.en_dis);
 * \param arg #PauseFrame_api
 * \ingroup IOCTL
 */
#define IFX_SET_PAUSE_FRAME_GEN _IOW(IFX_IOC_MAGIC,192,PauseFrame_api)
/**
 * Get port rate shaping flag IOCTL.
 * Calls #ifx_get_pause_frame_gen(arg.chipId, arg.portId, &arg.en_dis);
 * \param arg #PauseFrame_api
 * \ingroup IOCTL
 */
#define IFX_GET_PAUSE_FRAME_GEN _IOR(IFX_IOC_MAGIC,193,PauseFrame_api)




/*****************************function decleration***************************************************/

int addrresl_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int cpuport_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int led_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int mac_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int port_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int qos_ioctl(struct file *file,unsigned int cmd, unsigned long arg);
int switch_ioctl(struct file *file,unsigned int cmd, unsigned long arg);

#endif /*__IOCTL_API*/

