/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
/* Group definitions for Doxygen */
/** \addtogroup IOCTLstructs Structures used as parameters for IOCTLs */

#if 0
/**
 * port_stp_state IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_STP_PORT_STATE
 * - #IFX_GET_STP_PORT_STATE
 * \ingroup IOCTLstructs
 */ 
typedef struct port_stp_state{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 portState;     /**<  0x1=Disabled,  0x2=Blocking 
                             0x3=Listening, 0x4=Learning
                             0x5=Forward */
}port_stp_state_api;
#endif


/**
 * cos_pr IOCTL parameter struct.
 * Used by: 
 * - #IFX_MAP_INGRESS_PRIORITY_COS
 * - #IFX_GET_INGRESS_PRIORITY_COS
 * - #IFX_MAP_COS_EGRESS_PRIORITY
 * - #IFX_GET_COS_EGRESS_PRIORITY
 * \ingroup IOCTLstructs
 */ 
typedef struct cos_pr{
	u8 chipId;         /**< ChipID (always 0) */
	u32 Priority;      /**< IngressPriority (0-7) */
	u32 CoS;           /**< Class of service (0-3) */
}cos_pr_api;

/**
 * cos_rd IOCTL parameter struct.
 * Used by: 
 * - #IFX_MAP_COS_REDUCTION
 * - #IFX_GET_COS_REDUCTION
 * \ingroup IOCTLstructs
 */ 
typedef struct cos_rd{
	u8 chipId;         /**< ChipID (always 0) */
	u32 reduction;     /**< Reduced class of service (0-3) */
	u32 CoS;           /**< Class of service (0-3) */
}cos_rd_api;

/**
 * xcast_fid IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_UNKNOWN_UCAST_ID
 * - #IFX_GET_UNKNOWN_UCAST_ID
 * - #IFX_SET_UNKNOWN_MCAST_ID
 * - #IFX_GET_UNKNOWN_MCAST_ID
 * - #IFX_SET_BCAST_ID
 * - #IFX_GET_BCAST_ID 
 * \ingroup IOCTLstructs
 */ 
typedef struct ucast_fid{
	u8 chipId;         /**< ChipID (always 0) */
	u32 floodId;
}xcast_fid_api;

#if 0
/**
 * id IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_PORT_EGRESS_VLAN_TAG
 * - #IFX_SET_PORT_EGRESS_VLAN_TAG
 * \ingroup IOCTLstructs
 */ 
typedef struct id{
	u8 chipId;         /**< ChipID (always 0) */
	u32 Id;            /**< VLANID (1-4094) */
	u32 portList[2];   /**< Bit 2:0  -> 0:Not a member,1:Member,
                            Bit 63   -> 0:VLAN create,1:Delete */
}id_api;
#endif

/**
 * Port Ethernet Configuration IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_ETH_CONF
 * - #IFX_GET_PORT_ETH_CONF
 * \ingroup IOCTLstructs
 */ 
typedef struct port_eth_conf{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC */
	u32 speed;         /**< 0-10Mbit/s, 1-100Mbit/s, 2-1000Mbit/s */
    u32 duplex;        /**< half(0) or full duplex mode(1) */
    BOOL autoneg;      /**< disable(0) or enable(1) autonegotiation  */
    u32 int_ext_phy;   /**< use internal(0) or external(1) PHY   */
	u32 link_status;   /**< link down(0) or link up(1) only for IFX_GET_PORT_ETH_CONF */                          
}port_eth_conf_api;



/**
 * vlan_status IOCTL parameter struct.
 * Used by: 
 * - #IFX_VLAN_CHECK
 * \ingroup IOCTLstructs
 */ 
typedef struct vlan_status{
	u8 chipId;         /**< ChipID (always 0) */
	u32 vlanId;        /**< VLANID (0-4094) */
	BOOL flag;         /**< TRUE=created, FALSE=not created */
}vlan_status_api;

#if 0
typedef struct vlanStpMstp{
	u8 chipId;         /**< ChipID (always 0) */
	u32 vlanId;
	u8 stp_mst;
}vlanStpMstp_api;
#endif

/**
 * vlan_tableentry IOCTL parameter struct.
 * Used by: 
 * - #IFX_ADD_VLAN_TABLE_ENTRY
 * - #IFX_GET_VLAN_TABLE_ENTRY 
 * - #IFX_DEL_VLAN_TABLE_ENTRY
 * \ingroup IOCTLstructs
 */ 
typedef struct vlan_tableentry{
	u8 chipId;              /**< ChipID (always 0) */
	u32 vlanId;             /**< VLANID (0-4094) */
	u32 portmemberList[2];  /**< Port Membership -> 0:Not a member,1:Member 
	                             (Bit 0=LAN port, bit 1=PC port, bit 2=CPU port) */
	u32 portegressList[2];  /**< Port Egress Tagging -> 0:do not tag packet,1:tag packet */
    BOOL valid;             /**< Entry-valid-flag
	                             -> 0:table entry invalid,1:table entry valid */
}vlan_tableentry_api;
 
/**
 * vlan_aware IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_VLAN_AWARE
 * - #IFX_GET_VLAN_AWARE
 * \ingroup IOCTLstructs
 */ 
typedef struct vlan_aware{
	u8 chipId;          /**< ChipID (always 0) */
	BOOL VlanAwareFlag; /**< TRUE=enable VLAN awareness, FALSE=disable */
}vlan_aware_api;

#if 0
typedef struct pvst_state{
	u8 chipId;         /**< ChipID (always 0) */
	u32 vlanId;
	u32 portId;
	u32 portState;
}pvst_state_api;
#endif

/**
 * vlan_idx IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_VLAN_IDX
 * \ingroup IOCTLstructs
 */ 
typedef struct pmac_vlan{
    u8   chipId;        /**< ChipID (always 0) */
	BOOL en_dis;        /**< 0- disable, 1-enable tag insertion */
	u32  VlanId;        /**< VLANID (0-4094) */
    u8   prio;          /**< Priority (0-7) */
	u8   cfi;           /**< canonical format indicator,
                             0-little endian, 1-big endian */
}pmac_vlan_api;	


/**
 * vlan_idx IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_VLAN_IDX
 * \ingroup IOCTLstructs
 */ 
typedef struct vlan_idx{
	u8 chipId;         /**< ChipID (always 0) */
	u32 VlanId;        /**< VLANID (0-4094) */
	u32 VlanIndex;     /**< VLAN Index (0-63) */
}vlan_idx_api;	
 
/**
 * mib_idx IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_VLAN_MIB
 * \ingroup IOCTLstructs
 */ 
typedef struct mib_idx{
	u8 chipId;         /**< ChipID (always 0) */
	u32 mibIdx;        /**< VLAN Index (0-63) */
	u32 counterId;     /**< Counter ID 0=Byte counter, 1=Packet counter,
                                       2=Dropped packet counter,
                                       3=Non unicast packet counter */
	u32 count;	       /**< Counter value */
}mib_idx_api;

#if 0
/**
 * vlan_mib IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_VLAN_MIB_IDX
 * - #IFX_GET_VLAN_MIB_IDX
 * \ingroup IOCTLstructs
 */ 
typedef struct vlan_mib{
	u8 chipId;         /**< ChipID (always 0) */
	u32 VlanMIBIdx;    /**< VLAN MIB start index (0-23) */
}vlan_mib_api;


typedef struct vlan_filter{
	u8 chipId;         /**< ChipID (always 0) */
	u32 vlanIdx;
	u32 filterId;
}vlan_filter_api;


/**
 * free_Vid IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_FREE_VID_LIST
 * \ingroup IOCTLstructs
 */ 
typedef struct free_Vid{
	u8 chipId;         /**< ChipID (always 0) */
	u32 vidList[1023]; /**< Free VID list */
	u32 free_cnt;      /**< Free VID count */
}free_vid_api;



typedef struct stpPvst{
	u8 chipId;         /**< ChipID (always 0) */
	u8 stpPvst;
}stpPvst_api;

typedef struct autocast{
	u8 chipId;         /**< ChipID (always 0) */
	BOOL stateFlag;
}autocast_api;

typedef struct mbox{
	u8 chipId;         /**< ChipID (always 0) */
	u32 mboxId;
	u32 mboxData;
}mbox_api;

typedef struct mbox_id{
	u8 chipId;         /**< ChipID (always 0) */
	u32 mboxId;
}mbox_id_api;

typedef struct auto_timestamp{
	u8 chipId;         /**< ChipID (always 0) */
	u32 timeStamp;
}auto_timestamp_api;

typedef struct sendCPUData{
	u8 chipId;         /**< ChipID (always 0) */
	u8 txPktPtr[1536];
	u32 txSize;
	u32 txPriority;
	BOOL crcGenFlag;
	BOOL destTypeFlag;
	u32 destID;
	BOOL enDaLookup;
}sendCpuData_api;

typedef struct rcvCPUData{
	u8 chipId;         /**< ChipID (always 0) */
	u8 rxPktPtr[1536];
	u32 rxSize;
	u32 SrcId;
	u32 timeStamp;
	BOOL tagPtr;
}rcvCpuData_api;
		    
typedef struct ledsysmode{
	u8 chipId;         /**< ChipID (always 0) */
	BOOL sysMode;
}ledSysMode_api;

typedef struct ledhwmode{
	u8 chipId;         /**< ChipID (always 0) */
	u32 feature;
	u32 LEDmode;
}ledHWmode_api;
		
typedef struct setLedSW{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;
	u32 LEDdata;
}setLedSW_api;
#endif

/**
 * macTidx IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_MAC_TABLE_IDX
 * \ingroup IOCTLstructs
 */ 
typedef struct macTidx{
	u8 macAddr[6];    /**< MAC address macAddrPtr[0] -> MAC address byte 5
                                       macAddrPtr[1] -> MAC address byte 4
                                       macAddrPtr[2] -> MAC address byte 3
                                       macAddrPtr[3] -> MAC address byte 2
                                       macAddrPtr[4] -> MAC address byte 1
                                       macAddrPtr[5] -> MAC address byte 0 */
	u32 tableIndex;   /**< MAC table index (0-63) */
}macTidx_api;
 
/**
 * macTenIdx IOCTL parameter struct.
 * Used by:
 * - #IFX_ADD_MAC_TABLE_ENTRY
 * - #IFX_GET_MAC_TABLE_ENTRY_IDX
 * \ingroup IOCTLstructs
 */ 
typedef struct macTenIdx{
	u8 chipId;          /**< ChipID (always 0) */
	u32 macTableIndex;  /**< MAC table index (0-63) */
	u8 macAddr[6];      /**< MAC address macAddrPtr[0] -> MAC address byte 5
                                         macAddrPtr[1] -> MAC address byte 4
                                         macAddrPtr[2] -> MAC address byte 3
                                         macAddrPtr[3] -> MAC address byte 2
                                         macAddrPtr[4] -> MAC address byte 1
                                         macAddrPtr[5] -> MAC address byte 0 */
	BOOL lan_port;      /**< LAN port assignment 
	                        -> FALSE: not assigned, TRUE: assigned */
	BOOL pc_port;       /**< PC port assignment 
	                        -> FALSE: not assigned, TRUE: assigned */
	BOOL cpu_port;      /**< CPU port assignment 
	                        -> FALSE: not assigned, TRUE: assigned */
	BOOL criticalFlag;  /**< Priority Status of the packet 0: normal, 1:critical */
	u32 maState;        /**< aging status: 0(static),1...14(dynamic),15(aged out) */
}macTenIdex_api;


#if 0
/**
 * macTenHash IOCTL parameter struct.
 * Used by: 
 * - #IFX_ADD_MAC_TABLE_ENTRY_HASH
 * \ingroup IOCTLstructs
 */ 
typedef struct macTenHash{
	u8 chipId;         /**< ChipID (always 0) */
	u8 macAddr[6];     /**< MAC address macAddrPtr[0] -> MAC address byte 5
                                        macAddrPtr[1] -> MAC address byte 4
                                        macAddrPtr[2] -> MAC address byte 3
                                        macAddrPtr[3] -> MAC address byte 2
                                        macAddrPtr[4] -> MAC address byte 1
                                        macAddrPtr[5] -> MAC address byte 0 */
	u32 mpid; 
 	BOOL criticalFlag; /**< Status of the packet */ 
	u32 maState;       /**< aging/priority data */
	u32 fId; 
}macTenHash_api;
#endif


/**
 * macAging IOCTL parameter struct.
 * Used by: 
 * - #IFX_CONTROL_MAC_TABLE_AGING
 * \ingroup IOCTLstructs
 */ 
typedef struct macAging{
	u8  chipId;         /**< ChipID (always 0) */
	u32 aging_period;   /**< MAC table aging period */
}macAging_api;


/**
 * macTstat IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_MAC_TABLE_STAT
 * \ingroup IOCTLstructs
 */ 
typedef struct macTstat{
	u8 chipId;         /**< ChipID (always 0) */
	u32 staticCount;   /**< Number of static entries */
	u32 valid;         /**< Number of valid entries */
}macTstat_api;


/**
 * portMode IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_FE_MODE
 * - #IFX_GET_PORT_FE_MODE
 * - #IFX_SET_PORT_GE_MODE
 * - #IFX_GET_PORT_GE_MODE
 * \ingroup IOCTLstructs
 */ 
typedef struct portMode{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC */
	u32 feature;       /**< Port feature to be configured/read (see mode descr.) */
	u32 mode;          /**< Feature related modes:
                            Fast Ethernet(FE):
                              0 - FE/GE_PORT_AUTONEGOTIATION
                                  0 - disable autonegotiation
                                  1 - enable autonegotiation                                  
                              1 - FE/GE_PORT_LINK_OK (only IFX_GET_PORT_FE_MODE)
                                  0 - link down
                                  1 - link up                                    
                              2 - FE/GE_PORT_DUPLEX_MODE
                                  0 - half duplex
                                  1 - full duplex                                  
                              3 - FE/GE_PORT_SPEED
                                  0 - 10 MBit/s
                                  1 - 100 MBit/s
                                  2 - 1 GBit/s  (only in GE mode)                                 
                              4 - FE/GE_PORT_PAUSE_FLOW_CTRL
                                  0 - disable flow control
                                  1 - enable flow control
                              5 - FE/GE_PORT_PRIORITY
                                  0...3 - lowest...highest prio cos
                              6 - FE/GE_PORT_MONITOR_INGRESS
                                  0 - disable ingress monitoring for port
                                  1 - enable ingress monitoring for port
                              7 - FE/GE_PORT_MONITOR_EGRESS
                                  0 - disable egrress monitoring for port
                                  1 - enable egress monitoring for port
                              8 - FE/GE_PORT_AUTHENTICATION 
                                  0 - not authorized to receive or transmit
                                  1 - authorized to transmit but not to receive
                                  2 - authorized to receive but not to transmit
                                  3 - authorized to transmit and receive  */
}portMode_api;

/**
 * mibCounters IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_PORT_MIB_COUNTERS
 * - #IFX_GET_MAC_MIB_COUNTERS
 * \ingroup IOCTLstructs
 */ 
typedef struct mibCounters{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 counterId;     /**< Port MIB counter ID (IFX_GET_PORT_MIB_COUNTERS)
                            0 - rx Bytes;
                            1 - rx Unicast Packets
                            2 - rx Broadcast Packets
							3 - rx Multicast Packets
                            4 - rx CRC Errored Packets
                            5 - rx Undersized Good Packets
                            6 - rx UnderSized Errored Packets   
                            7 - rx Packets with length <= 64 Byte        
							8 - rx Packets with 64 byte < length <= 127 byte 
                            9 - rx Packets with 127 byte < length <= 255 byte 
                            10 - rx Packets with 255 byte < length <= 511 byte 
                            11 - rx Packets with 511 byte < length <= 1023 byte 
                            12 - rx Packets with 1023 byte < length <= MAX_PKT_LEN byte 
                            13 - rx Oversized Good Packets (Packet length > MAX_PKT_LEN bytes)
                            14 - rx Oversized Bad Packets (Packet length > MAX_PKT_LEN bytes)
                            15 - rx Good Pause Packets
                            16 - rx Dropped Packets    
                            17 - tx Bytes
                            18 - tx Unicast Pkts
                            19 - tx Broadcast Pkts
                            20 - tx Multicast Pkts 
							
							MAC MIB counter ID (IFX_GET_MAC_MIB_COUNTERS)
                            0 - tx Pause Frames
                            1 - tx single Collisions
                            2 - tx multiple Collisions
							3 - tx Excessive Collisions
                            4 - tx Later Collisions
                            5 - tx deferred  */
	u32 count;         /**< Value of MIB counter */
}mibCounters_api;
 
/**
 * Flagstat IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_INGRESS_MONITOR_FLAG
 * - #IFX_GET_INGRESS_MONITOR_FLAG
 * - #IFX_SET_EGRESS_MONITOR_FLAG
 * - #IFX_GET_EGRESS_MONITOR_FLAG
 * - #IFX_SET_PORT_LOCK
 * - #IFX_GET_PORT_LOCK
 * - #IFX_SET_PORT_INGRESS_VLAN_TAG
 * - #IFX_GET_PORT_INGRESS_VLAN_TAG
 * - #IFX_SET_PORT_INGRESS_VLAN_FILTER
 * - #IFX_GET_PORT_INGRESS_VLAN_FILTER
 * - #IFX_SET_PORT_JUMBO_ENABLE
 * - #IFX_GET_PORT_JUMBO_ENABLE
 * \ingroup IOCTLstructs
 */
typedef struct Flagstat{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	BOOL flag;         /**< TRUE or FALSE */
}Flag_api;
 
/**
 * phyAction IOCTL parameter struct.
 * Used by: 
 * - #IFX_PHY_READ
 * - #IFX_PHY_WRITE
 * \ingroup IOCTLstructs
 */ 
typedef struct phyAction{
	uint chipId;         /**< ChipID (always 0) */
	u8 mdio_id;          /**< MDIO ID */
	u8 phy_addr;         /**< MDIO address of the PHY device */
	u8 RegOffset;        /**< PHY register address */
	u16 Data;            /**< Register data */
}phyAction_api;

/**
 * portVlan IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_VLANID
 * - #IFX_GET_PORT_VLANID
 * \ingroup IOCTLstructs
 */ 
typedef struct portVlan{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 vlanid;        /**< VLANID (1-4094) */
}portVlan_api;

/**
 * portCoS IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_COS
 * - #IFX_GET_PORT_COS
 * \ingroup IOCTLstructs
 */ 
typedef struct portCoS{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 CoS;           /**< Class of service (0-3) */
}portCoS_api;


#if 0
typedef struct tbiInit{
	u8 chipId;         /**< ChipID (always 0) */
	u8 portGroupId;
}tbiInit_api;

typedef struct trunckMember{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;
	u32 groupSel;
}trunckMember_api;
#endif

/**
 * rule_mem IOCTL parameter struct.
 * Used by: 
 * - #IFX_GET_RULE_MEM
 * \ingroup IOCTLstructs
 */ 
typedef struct rule_mem{
	u8 chipId;         /**< ChipID (always 0) */
	u32 bank;
	u32 address;
	u32 section;
	u32 rule32_val;
}rule_mem_api;

/**
 * portKey IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_KEY
 * - #IFX_GET_PORT_KEY
 * \ingroup IOCTLstructs
 */ 
typedef struct portKey{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 IKey;          /**< IKEY value */
}portKey_api;

/**
 * flowPattern IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_FLOW_PATTERN
 * - #IFX_GET_FLOW_PATTERN
 * \ingroup IOCTLstructs
 */ 
typedef struct flowPattern{
	u8 chipId;         /**< ChipID (always 0) */
	u32 flowId;        /**< Flow ID (0-23) */
	u32 IKeyData[4];   /**< Rule vector (0-127) */
	u32 IKeyMask[4];   /**< Mask vector (0-127) */
}flowPattern_api;

/**
 * flowId IOCTL parameter struct.
 * Used by: 
 * - #IFX_DELETE_FLOW_PATTERN
 * \ingroup IOCTLstructs
 */ 
typedef struct flowId{
	u8 chipId;         /**< ChipID (always 0) */
	u32 flowId;        /**< Flow ID (0-23) */
}flowId_api;

/**
 * flowParam IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_FLOW_ACTION_PARAM
 * - #IFX_GET_FLOW_ACTION_PARAM
 * - #IFX_DELETE_FLOW_ACTION_PARAM
 * \ingroup IOCTLstructs
 */ 
typedef struct flowParam{
	u8 chipId;         /**< ChipID (always 0) */
	u32 flowId;        /**< Flow ID (0-23) */
	u32 feature;       /**< Possible Features
    
                            - REDIRECT_MCAST:
                                - bit 2 = LAN Port
                                - bit 1 = PC Port
                                - bit 0 = CPU port
                                .
                            - MIRROR:   
                                - 1 = Mirror Enable
                                - 0 = Mirror Disable
                                .
                            - COS_ASSIGNMENT: 
                                - 0-3 CoS Value to be assigned to this rule.
                            - VLAN_ASSIGNMENT: 
                                - 0-4094 VLAN value to be associated for this rule.
                            - TKN_BKT_ID: 
                                - 0-62 Token bucket ID for this rule.
                                - 63 Token bucket ID is flow ID
                            - DELETE_ACTION_ENTRY: 
                              (valid only for IFX_DELETE_FLOW_ACTION_PARAM)
                                - removes the entry assigned by flowId 
                                  from the action table.  */
	u32 flowAction;    /**< Action value for the selected feature */
}flowParam_api;

#if 0
typedef struct flowAction{
	u8 chipId;         /**< ChipID (always 0) */
	u32 flowId;        
	u32 flowAction;
}flowAction_api;
#endif

/**
 * portRule IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_RULE
 * - #IFX_GET_PORT_RULE
 * - #IFX_SET_PORT_MASK
 * - #IFX_GET_PORT_MASK
 * \ingroup IOCTLstructs
 */ 
typedef struct portRule{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 portRule[4];   /**< 128 bit port rule */
}portRule_api;

/**
 * portOffset IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_OFFSET
 * - #IFX_GET_PORT_OFFSET
 * \ingroup IOCTLstructs
 */ 
typedef struct portOffset{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 portOffset[2]; /**< Port Offset 0...7 */
}portOffset_api;

/**
 * RxConfig IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_RX_CONFIG
 * - #IFX_GET_RX_CONFIG
 * \ingroup IOCTLstructs
 */ 
typedef struct RxConfig{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	BOOL L3Program;    
	BOOL L3StartVld;   /**< Layer3 start valid flag */
	u32 L3Start;       /**< Layer3 start value in bytes */
}RxConfig_api;

/**
 * QoSbucket IOCTL parameter struct.
 * Used by:
 * - #IFX_SET_QOS_BUCKET
 * - #IFX_GET_QOS_BUCKET
 * \ingroup IOCTLstructs
 */ 
typedef struct QoSbucket{
	u8 chipId;             /**< ChipID (always 0) */
	u32 tBucketId;         /**< Token Bucket ID (0-63) */
	u32 tBucketCnt;        /**< 32bit Bucket count */
	u32 tReplenishRateIdx; /**< Replinish Rate */
	u32 tBurstSizeIdx;     /**< Index of Replinish Rate register */
}QoSbucket_api;

/**
 * QoSBflowCntr IOCTL parameter struct.
 * Used by:
 * - #IFX_SET_QOS_BUCKET_FLOW_CONTROL
 * - #IFX_GET_QOS_BUCKET_FLOW_CONTROL
 * \ingroup IOCTLstructs
 */ 
typedef struct QoSBflowCntr{
	u8 chipId;           /**< ChipID (always 0) */
	u32 portRuleBaseIdx; /**< Port Rule base Id (0-511) */
	u32 actionLC;        /**< Flow Control Action for loosley confirming packets */
	u32 actionNC;        /**< Flow Control Action for non confirming packets */
}QoSBflowCntr_api;

/**
 * QoSWFqueue IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_QOS_WF_QUEUE
 * - #IFX_GET_QOS_WF_QUEUE
 * \ingroup IOCTLstructs
 */ 
typedef struct QoSWFqueue{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 queueId;       /**< Queue ID (0-3) */
	u32 weightFactor;  /**< 16 bit Weight Factor associated with each queue for 
 *                          WFQ scheduling */
	u32 deficitCount;  /**< 21 bit DeficitCount for WFQ scheduling */
}QoSWFqueue_api;

/**
 * QoSshapingQ IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_QOS_SHAPING_QUEUE
 * - #IFX_GET_QOS_SHAPING_QUEUE
 * \ingroup IOCTLstructs
 */ 
typedef struct QoSshapingQ{
	u8 chipId;             /**< ChipID (always 0) */
	u32 portId;            /**< PortID 0=LAN 1=PC 2=CPU */
	u32 queueId;           /**< Queue ID (0-3) */
	u32 bucketCnt;         /**< 21 bit Bucket count for rate shaping */
	u32 replenishRate;     /**< 20 bit Replinish rate for rate shaping */
	u32 tokenValue;        /**< 20 bit Token value for rate shaping */
	u32 tReplenishTimer;   /**< Replinish rate timer for rate shaping */
}QoSshapingQ_api;
 
/**
 * idx IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_QOS_REPLENISH_RATE
 * - #IFX_GET_QOS_REPLENISH_RATE
 * - #IFX_SET_QOS_BURST_SIZE
 * - #IFX_GET_QOS_BURST_SIZE
 * - #IFX_SET_QOS_EXTEND_BURST_SIZE
 * - #IFX_GET_QOS_EXTEND_BURST_SIZE
 * \ingroup IOCTLstructs
 */ 
typedef struct idx{
	u8 chipId;         /**< ChipID (always 0) */
	u32 Idx;           /**< Index to replenish rate register */
	u32 info;          /**< Replenish rate value */
}idx_api;

#if 0 
typedef struct portQdata{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;
	u32 queueId;
	u32 queueType;
	u32 count;
}portQdata_api;

typedef struct portQstatus{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;
	u32 queueId;
	u32 queueStatusType;
	BOOL queueStatus;
}portQstatus_api;
#endif

#if 0
/**
 * chip_num_api IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_CHIP_NUMBER
 * \ingroup IOCTLstructs
 */ 
typedef struct chipNum{
       u8 num_of_chips;         /**< number of chips */
}chip_num_api;

/**
 * baseAddr_api IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_BASE_ADDRESS
 * \ingroup IOCTLstructs
 */ 
typedef struct baseAddr{
       u8 chipId;         /**< ChipID (always 0) */
       u32 Base_Address;
}baseAddr_api;


typedef struct intReg{
	u8 chipId;         /**< ChipID (always 0) */
	u32 regHi;
	u32 regLo;
}intReg_api;

typedef struct intRegW{
	u8 chipId;         /**< ChipID (always 0) */
	u32 reg;
}intRegW_api;

typedef struct portReg{
	u32 chipId;         /**< ChipID (always 0) */
	u32 portId;
	u32 reg;
}portReg_api;

typedef struct packetBuf{
	u8 chipId;         /**< ChipID (always 0) */
	u32 readPtr;
	u32 writePtr;
	u32 count;
}packetBuf_api;

typedef struct packetLinkList{
	u8 chipId;         /**< ChipID (always 0) */
	u32 PBAddr;
	u32 PBLink;
}packetLinkList_api;

typedef struct chipversion{
	u8 chipId;         /**< ChipID (always 0) */
	u32 Majr;
	u32 Minr;
}chipversion_api;

typedef struct minorversion{
	u8 chipId;         /**< ChipID (always 0) */
	u32 minr;
}minorversion_api;
#endif
					
/**
 * set/get switch mode IOCTL parameter struct.
 * Used by:
 * - #IFX_SET_SWITCH_MODE
 * - #IFX_GET_SWITCH_MODE
 * \ingroup IOCTLstructs
 */ 
typedef struct feature_mode{
	u8 chipId;         /**< ChipID (always 0) */
	u32 feature;       /**<  2 - GE_MCASTQ_THRESHOLD
                             6 - AGING_SOURCE
                             7 - AGING_TIME_SELECT
                             8 - MA_FREEZE_ALL
                             9 - MA_FREEZE_NEW
                            10 - CRCERROR_PACKET_MONITOR
                            11 - MACHANGE_PACKET_MONIOR
                            12 - PAUSE_PACKET_TO_CPU                            
                            13 - FORWARD_CFI_PACKET			
                            14 - MA_REPLACE_THRESHOLD		
                            33 - UNKNOWN_PROTOCOL_TO_CPU		
                            34 - MONITORING_PORT */
	u32 mode;
}feature_mode_api;

/**
 * watermark IOCTL parameter struct.
 * Used by:
 * - #IFX_SET_INGRESS_WATERMARK
 * - #IFX_GET_INGRESS_WATERMARK
 * \ingroup IOCTLstructs
 */ 
typedef struct watermark{
	u8 chipId;          /**< ChipID (always 0) */
	u32 waterMarkType;  /**< Type of watermark 
                             - 1=GLOBAL_DROP_WATERMARK,
                             - 2=PORTGROUP_FC_WATERMARK,
                             - 3=PORTGROUP_DROP_WATERMARK */ 
	u32 portId;         /**< PortID 0=LAN 1=PC 2=CPU */
	u32 waterMark;      /**< Watermark value */
}watermark_api;

/**
 * Ewatermark IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_EGRESS_WATERMARK
 * - #IFX_GET_EGRESS_WATERMARK
 * \ingroup IOCTLstructs
 */ 
typedef struct Ewatermark{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC 2=CPU */
	u32 queueId;       /**< Queue ID (0-3) */
	u32 waterMark;     /**< Watermark value */
}Ewatermark_api;


/**
 * Pause Frame Generation IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PAUSE_FRAME_GEN
 * - #IFX_GET_PAUSE_FRAME_GEN 
 * \ingroup IOCTLstructs
 */ 
typedef struct PauseFrame{
	u8 chipId;         /**< ChipID (always 0) */
	u32 portId;        /**< PortID 0=LAN 1=PC */
	BOOL en_dis;       /**< Enable/disable pause frame generation 
                            (0-disable, 1-enable) */
}PauseFrame_api;


/**
 * TxConfig IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_TX_CONFIG
 * - #IFX_GET_TX_CONFIG
 * \ingroup IOCTLstructs
 */ 
typedef struct TxConfig{
	u8 chipId;              /**< ChipID (always 0) */
	u32 portId;             /**< PortID 0=LAN 1=PC 2=CPU */
	u32 xmitDropCount;      /**< Number of retransmission attempts before the 
                                 packet is discarded */
	u32 lateCollisionCount; /**< Threshold of decision between Early and Late 
                                 collision */
}TxConfig_api;

/**
 * rateShape IOCTL parameter struct.
 * Used by: 
 * - #IFX_SET_PORT_RATE_SHAPE
 * - #IFX_GET_PORT_RATE_SHAPE
 * \ingroup IOCTLstructs
 */ 
typedef struct rateShape{
	u8 chipId;         /**< ChipID (always 0) */
    BOOL en_dis;       /**< rate shaping enable(1)/disable(0) flag */
	BOOL PRE;          /**< Port rate shaping enable flag */
}rateShape_api;

typedef struct mdiomode{
	u8 chipId;         /**< ChipID (always 0) */
	u32 port_group; 
	BOOL mdioMode; 
	BOOL autoNegMode;
}mdiomode_api;

