/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>

#include <asm/incaip2/ifx_types.h>	
#include <asm/incaip2/switch.h>				     
#include "switch_api.h"
#include "ioctl.h"
#include "ioctl_struct.h"

#include <mac.h>


/**
 * Mac IOCTL handler.
 * IOCTL handler for Mac address table modification.
 * The following IOCTLs are supported:
 * - #IFX_GET_MAC_TABLE_IDX
 * - #IFX_ADD_MAC_TABLE_ENTRY
 * - #IFX_ADD_MAC_TABLE_ENTRY_IDX
 * - #IFX_GET_MAC_TABLE_ENTRY_IDX
 * - #IFX_GET_MAC_TABLE_STAT
*
 * \param   file   - File structure of device
 * \param   cmd    - IOCTL command 
 * \param   arg    - Argument for some IOCTL commands
 * \return  0      - OK
 * \return  ENOTTY - Invalid command
 * \ingroup API
 */
int mac_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
	int err=0;
	/*structures for arg*/
	macTidx_api mti_api;
	macTenIdex_api mtei_api;
	macTstat_api mts_api;
	
	switch(cmd)
	{
		case IFX_GET_MAC_TABLE_IDX:
			DPRINTK("%s: IFX_GET_MAC_TABLE_IDX\n",__FUNCTION__);
			copy_from_user(&mti_api,(macTidx_api *) arg,sizeof (macTidx_api));
			err = ifx_get_MAC_table_idx( mti_api.macAddr,
                                      /*   mti_api.fId,  */
                                         &mti_api.tableIndex);
			if(err == 0)
				copy_to_user((macTidx_api *) arg, &mti_api, sizeof(macTidx_api));
			break;
		case IFX_ADD_MAC_TABLE_ENTRY_IDX:
			DPRINTK("%s: IFX_ADD_MAC_TABLE_ENTRY_IDX\n",__FUNCTION__);
			copy_from_user( &mtei_api, (macTenIdex_api *) arg, sizeof (macTenIdex_api) );
			err = ifx_add_MAC_table_entry_idx( mtei_api.chipId, 
                                               mtei_api.macTableIndex,
                                               mtei_api.macAddr, 
                                               mtei_api.lan_port,
                                               mtei_api.pc_port,
                                               mtei_api.cpu_port,
                                               mtei_api.criticalFlag,
                                               mtei_api.maState );
			break;
		case IFX_ADD_MAC_TABLE_ENTRY:
			DPRINTK("%s: IFX_ADD_MAC_TABLE_ENTRY\n",__FUNCTION__);
			copy_from_user( &mtei_api, (macTenIdex_api *) arg, sizeof (macTenIdex_api) );
			err = ifx_add_MAC_table_entry( mtei_api.macAddr,
                                           mtei_api.lan_port,
                                           mtei_api.pc_port,
                                           mtei_api.cpu_port,
                                           mtei_api.criticalFlag,
                                           mtei_api.maState );
			break;
		case IFX_GET_MAC_TABLE_ENTRY_IDX:
/*			DPRINTK("%s: IFX_GET_MAC_TABLE_ENTRY_IDX\n",__FUNCTION__);  */
			copy_from_user(&mtei_api,(macTenIdex_api *) arg,sizeof(macTenIdex_api));
			err = ifx_get_MAC_table_entry_idx( mtei_api.chipId,
                                               mtei_api.macTableIndex,
                                               mtei_api.macAddr,
				       	                       &mtei_api.lan_port,
                                               &mtei_api.pc_port,
                                               &mtei_api.cpu_port,
                                               &mtei_api.criticalFlag,
                                               &mtei_api.maState );
            if(err == 0)
				copy_to_user((macTenIdex_api *) arg, &mtei_api, sizeof(macTenIdex_api));
			break;
/*
		case IFX_ADD_MAC_TABLE_ENTRY_HASH:
			DPRINTK("%s: IFX_ADD_MAC_TABLE_ENTRY_HASH\n",__FUNCTION__);
			copy_from_user(&mteh_api,(macTenHash_api *) arg,sizeof (macTenHash_api));
			err = ifx_add_MAC_table_entry_hash( mteh_api.chipId,
                                                mteh_api.macAddr,
                                                mteh_api.mpid,
		                                        mteh_api.criticalFlag,
                                                mteh_api.maState,
                                                mteh_api.fId);
			break;
*/
		case IFX_GET_MAC_TABLE_STAT:
			DPRINTK("%s: IFX_GET_MAC_TABLE_STAT\n",__FUNCTION__);
			copy_from_user(&mts_api,(macTstat_api *) arg,sizeof(macTstat_api));
			err = ifx_get_MAC_table_stat( mts_api.chipId,
                                          /* &mts_api.multiCast,  */
                                          &mts_api.staticCount,
                                          &mts_api.valid);
			if(err == 0)
				copy_to_user((macTstat_api *) arg, &mts_api, sizeof(macTstat_api));
			break;
	}
					
	return err;
}
