/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Aparna   		    1/9/2003					        	
 *
 *
 * Manamohan			10/8/2004					Added support for re-entrancy
 *
 *
 * DESCRIPTION :This module contains the Trunking Configuration API's
 *
 ******************************************************************************/
#include <asm/types.h>
#include <asm/addrspace.h>
#include "struct.h"
#include "const.h"
#include "trunk.h"
#include <asm/incaip2/incaip2.h>
#include "err.h"
#include "var.h"
#include "api_init_prv.h"

/******************************************************************************
  *
  *	FUNCTION:	ifx_set_port_trunk_member
  *				 	
  *	DESCRIPTION: Set the trunk group membership for the port.
  *				 
  *	PARAMETERS:	
  *		IN:
  *         chipId      Chip Id
  *			portId		port Id (range is 0 - 49)
  *			groupSel	Indicates trunk group membership
  *		OUT
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE				Success							
  *		PR_ERR_PORTID			If portId is not in the range of 0 - 49
  *     PR_ERR_NULL_PTR			Null pointer
  *     PR_ERR_CHIP_ID			Invalid chip Id		 				
  *
  ******************************************************************************/

s32  ifx_set_port_trunk_member(u8 chipId,u32 portId, u32 groupSel)
{
	u32 *ptr_trunk_ctrl;
	u32 Temp=0;
	u32 pos=0;
	s32 retflag = PR_ERR_NONE;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif	


	ptr_trunk_ctrl = (u32*) (Chip_Base_Address[chipId] + TRUNK_CTRL_REGS) + (portId >> 4) ;

	pos = (portId & 0xF) << 1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	PR_Reg_Read(ptr_trunk_ctrl, Temp);
	Temp &= ~(3 << pos) ;
	Temp |= groupSel << pos ;		

	PR_Reg_Write(ptr_trunk_ctrl, Temp);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE ;

	
}

/******************************************************************************
  *
  *	FUNCTION:	 ifx_get_port_trunk_member
  *				 	
  *	DESCRIPTION: Get the trunk group membership for the port.
  *				 
  *	PARAMETERS:	
  *		IN:
  *         chipId      	Chip Id
  *			portId			port Id (range is 0 - 49)
  *		OUT
  *			groupSelPtr		Pointer to trunk group membership
  *
  *	RETURNS:
  *		PR_ERR_NONE				Success							
  *		PR_ERR_PORTID			If portId is not in the range of 0 - 49
  *     PR_ERR_CHIP_ID			Invalid chip Id
  *     PR_ERR_NULL_PTR			Null pointer		 		 				
  *
  ******************************************************************************/

s32  ifx_get_port_trunk_member(u8 chipId, u32 portId, u32 *groupSelPtr)
{
	u32 *ptr_trunk_ctrl;
	u32 groupSel=0;
	u32 pos=0;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;

	if(groupSelPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif	


	ptr_trunk_ctrl = (u32*) (Chip_Base_Address[chipId] + TRUNK_CTRL_REGS) + (portId >> 4) ;

	pos = (portId & 0xF) << 1;

	PR_Reg_Read(ptr_trunk_ctrl, groupSel);	
	*groupSelPtr = (groupSel >> pos) & 0x3;

	return PR_ERR_NONE ;
}
