/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>

#include <asm/incaip2/ifx_types.h>
#include "switch_api.h"
#include "ioctl.h"
#include "ioctl_struct.h"

#include <flow_engine.h>
#include <qos.h>  
#include <flow_engine_prv.h>

/**
 * QoS IOCTL handler.
 * IOCTL handler for QoS configuration. 
 * The following IOCTLs are supported:
 * - #IFX_GET_RULE_MEM
 * - #IFX_SET_PORT_KEY
 * - #IFX_GET_PORT_KEY
 * - #IFX_SET_FLOW_PATTERN
 * - #IFX_GET_FLOW_PATTERN
 * - #IFX_DELETE_FLOW_PATTERN
 * - #IFX_SET_FLOW_ACTION_PARAM
 * - #IFX_GET_FLOW_ACTION_PARAM
 * - #IFX_SET_PORT_RULE: 
 * - #IFX_GET_PORT_RULE: 
 * - #IFX_SET_PORT_MASK:
 * - #IFX_GET_PORT_MASK: 
 * - #IFX_SET_PORT_OFFSET: 
 * - #IFX_GET_PORT_OFFSET: 
 * - #IFX_SET_RX_CONFIG: 
 * - #IFX_GET_RX_CONFIG: 
 * - #IFX_SET_QOS_BUCKET: 
 * - #IFX_GET_QOS_BUCKET: 
 * - #IFX_SET_QOS_BUCKET_FLOW_CONTROL: 
 * - #IFX_GET_QOS_BUCKET_FLOW_CONTROL: 
 * - #IFX_SET_QOS_WF_QUEUE: 
 * - #IFX_GET_QOS_WF_QUEUE:
 * - #IFX_SET_QOS_SHAPING_QUEUE:
 * - #IFX_GET_QOS_SHAPING_QUEUE:
 * - #IFX_SET_QOS_REPLENISH_RATE:
 * - #IFX_GET_QOS_REPLENISH_RATE: 
 * - #IFX_SET_QOS_BURST_SIZE: 
 * - #IFX_GET_QOS_BURST_SIZE: 
 * - #IFX_SET_QOS_EXTEND_BURST_SIZE: 
 * - #IFX_GET_QOS_EXTEND_BURST_SIZE: 
 *
 * \param   file   - File structure of device
 * \param   cmd    - IOCTL command 
 * \param   arg    - Argument for some IOCTL commands
 * \return  0      - OK
 * \return  ENOTTY - Invalid command
 * \ingroup API  
 */
int qos_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
	int err=0;
	/*structures for arg*/
	portKey_api pk_api;
	flowPattern_api fp_api;
	flowId_api fi_api;
	flowParam_api fpm_api;
/*	flowAction_api fa_api;  */
	portRule_api pr_api;
	portOffset_api po_api;
	RxConfig_api rc_api;
	QoSbucket_api qb_api;
	QoSBflowCntr_api qfc_api;
	QoSWFqueue_api qwf_api;
	QoSshapingQ_api qsq_api;
	idx_api ia_api;
/*	portQdata_api pqd_api;  */
/*	portQstatus_api pqs_api;  */
	rule_mem_api rm_api;

	switch(cmd)
	{
		case IFX_GET_RULE_MEM:
			DPRINTK("%s: IFX_GET_RULE_MEM\n",__FUNCTION__);
			copy_from_user(&rm_api,(rule_mem_api *)arg,sizeof(rule_mem_api));
			err = ifx_get_rule_mem(rm_api.chipId, rm_api.bank, rm_api.address, rm_api.section, &rm_api.rule32_val);
			if(err == 0)
		              copy_to_user((rule_mem_api *) arg, &rm_api, sizeof(rule_mem_api));
			break;
		case IFX_SET_PORT_KEY:
			DPRINTK("%s: IFX_SET_PORT_KEY\n",__FUNCTION__);
			copy_from_user(&pk_api,(portKey_api *) arg,sizeof (portKey_api));
			err = ifx_set_port_key(pk_api.chipId,pk_api.portId, pk_api.IKey);
			break;
		case IFX_GET_PORT_KEY :
			DPRINTK("%s: IFX_GET_PORT_KEY\n",__FUNCTION__);
			copy_from_user(&pk_api,(portKey_api *)arg,sizeof(portKey_api));
			err = ifx_get_port_key(pk_api.chipId,pk_api.portId, &pk_api.IKey);
			if(err == 0)
		              copy_to_user((portKey_api *) arg, &pk_api, sizeof(portKey_api));
			break;
		case IFX_SET_FLOW_PATTERN:
			DPRINTK("%s: IFX_SET_FLOW_PATTERN\n",__FUNCTION__);
			copy_from_user(&fp_api,(flowPattern_api *) arg,sizeof (flowPattern_api));
            DPRINTK("%s - fp_api.IKeyData[0]=0x%08X\n", __FUNCTION__, fp_api.IKeyData[0]);
            DPRINTK("%s - fp_api.IKeyData[1]=0x%08X\n", __FUNCTION__, fp_api.IKeyData[1]);
            DPRINTK("%s - fp_api.IKeyData[2]=0x%08X\n", __FUNCTION__, fp_api.IKeyData[2]);
            DPRINTK("%s - fp_api.IKeyData[3]=0x%08X\n", __FUNCTION__, fp_api.IKeyData[3]); 
            DPRINTK("%s - fp_api.IKeyMask[0]=0x%08X\n", __FUNCTION__, fp_api.IKeyMask[0]);
            DPRINTK("%s - fp_api.IKeyMask[1]=0x%08X\n", __FUNCTION__, fp_api.IKeyMask[1]);
            DPRINTK("%s - fp_api.IKeyMask[2]=0x%08X\n", __FUNCTION__, fp_api.IKeyMask[2]);
            DPRINTK("%s - fp_api.IKeyMask[3]=0x%08X\n", __FUNCTION__, fp_api.IKeyMask[3]);                     
			err = ifx_set_flow_pattern(fp_api.chipId,fp_api.flowId, fp_api.IKeyData, fp_api.IKeyMask);
			break;
		case IFX_GET_FLOW_PATTERN:
			DPRINTK("%s: IFX_GET_FLOW_PATTERN\n",__FUNCTION__);
			copy_from_user(&fp_api,(flowPattern_api *)arg,sizeof(flowPattern_api));
			err = ifx_get_flow_pattern(fp_api.chipId,fp_api.flowId,fp_api.IKeyData,fp_api.IKeyMask);
			if(err == 0)
		              copy_to_user((flowPattern_api *) arg, &fp_api, sizeof(flowPattern_api));
			break;
		case IFX_DELETE_FLOW_PATTERN:
			DPRINTK("%s: IFX_DELETE_FLOW_PATTERN\n",__FUNCTION__);
			copy_from_user(&fi_api,(flowId_api *) arg,sizeof (flowId_api));
			err = ifx_delete_flow_pattern(fi_api.chipId,fi_api.flowId);
			break;
		case IFX_SET_FLOW_ACTION_PARAM:
			DPRINTK("%s: IFX_SET_FLOW_ACTION_PARAM\n",__FUNCTION__);
			copy_from_user(&fpm_api,(flowParam_api *) arg,sizeof (flowParam_api));
			err = ifx_set_flow_action_param( fpm_api.chipId, fpm_api.flowId, 
                                             fpm_api.feature, fpm_api.flowAction);
			break;

		case IFX_DELETE_FLOW_ACTION_PARAM:
			DPRINTK("%s: IFX_DELETE_FLOW_ACTION_PARAM\n",__FUNCTION__);
			copy_from_user(&fpm_api,(flowParam_api *) arg,sizeof (flowParam_api));
			err = ifx_delete_flow_action_param( fpm_api.chipId,
                                                fpm_api.flowId, 
                                                fpm_api.feature);
			break;            
            
		case IFX_GET_FLOW_ACTION_PARAM:
			DPRINTK("%s: IFX_GET_FLOW_ACTION_PARAM\n",__FUNCTION__);
			copy_from_user(&fpm_api,(flowParam_api *)arg,sizeof(flowParam_api));
			err = ifx_get_flow_action_param(fpm_api.chipId,fpm_api.flowId, fpm_api.feature,&fpm_api.flowAction);
			if(err == 0)
		              copy_to_user((flowParam_api *) arg, &fpm_api, sizeof(flowParam_api));
			break;

		case IFX_SET_PORT_RULE: 
			DPRINTK("%s: IFX_SET_PORT_RULE\n",__FUNCTION__);
			copy_from_user(&pr_api,(portRule_api *) arg,sizeof (portRule_api));
			err = ifx_set_port_rule(pr_api.chipId,pr_api.portId, pr_api.portRule);
			break;
            
		case IFX_GET_PORT_RULE: 
			DPRINTK("%s: IFX_GET_PORT_RULE\n",__FUNCTION__);
			copy_from_user(&pr_api,(portRule_api *)arg,sizeof(portRule_api));
			err = ifx_get_port_rule(pr_api.chipId,pr_api.portId, pr_api.portRule);
			if(err == 0)
		              copy_to_user((portRule_api *) arg, &pr_api, sizeof(portRule_api));
			break;
            
		case IFX_SET_PORT_MASK:
			DPRINTK("%s: IFX_SET_PORT_MASK\n",__FUNCTION__);
			copy_from_user(&pr_api,(portRule_api *) arg,sizeof (portRule_api));
			err = ifx_set_port_mask( pr_api.chipId, pr_api.portId, pr_api.portRule );
			break;
            
		case IFX_GET_PORT_MASK: 
			DPRINTK("%s: IFX_GET_PORT_MASK\n",__FUNCTION__);
			copy_from_user(&pr_api,(portRule_api *)arg,sizeof(portRule_api));
			err = ifx_get_port_mask(pr_api.chipId,pr_api.portId, pr_api.portRule);
			if(err == 0)
		              copy_to_user((portRule_api *) arg, &pr_api, sizeof(portRule_api));
			break;
            
		case IFX_SET_PORT_OFFSET: 
			DPRINTK("%s: IFX_SET_PORT_OFFSET\n",__FUNCTION__);
			copy_from_user(&po_api,(portOffset_api *) arg,sizeof (portOffset_api));
			err = ifx_set_port_offset(po_api.chipId,po_api.portId, po_api.portOffset);
			break;
            
		case IFX_GET_PORT_OFFSET: 
			DPRINTK("%s: IFX_GET_PORT_OFFSET\n",__FUNCTION__);
			copy_from_user(&po_api,(portOffset_api *)arg,sizeof(portOffset_api));
			err =  ifx_get_port_offset(po_api.chipId,po_api.portId, po_api.portOffset);
			if(err == 0)
		              copy_to_user((portOffset_api *) arg, &po_api, sizeof(portOffset_api));
			break;
            
		case IFX_SET_RX_CONFIG: 
			DPRINTK("%s: IFX_SET_RX_CONFIG\n",__FUNCTION__);
			copy_from_user(&rc_api,(RxConfig_api *) arg,sizeof (RxConfig_api));
            printk("%s -> ifx_set_Rx_config(portid=%d, L3Prog=%d, L3startvld=%d, L3Start=%d)\n",
                    __FUNCTION__, rc_api.portId,rc_api.L3Program, rc_api.L3StartVld, rc_api.L3Start );
			err = ifx_set_Rx_config( rc_api.chipId, rc_api.portId, 
                                     (BOOL)rc_api.L3Program, 
                                     (BOOL) rc_api.L3StartVld, 
					                 rc_api.L3Start);
			break;
            
		case IFX_GET_RX_CONFIG: 
			DPRINTK("%s:  IFX_GET_RX_CONFIG\n",__FUNCTION__);
			copy_from_user(&rc_api,(RxConfig_api *)arg,sizeof(RxConfig_api));
			err = ifx_get_Rx_config( rc_api.chipId,rc_api.portId,&rc_api.L3Program,&rc_api.L3StartVld,
					  &rc_api.L3Start);
			if(err == 0)
		              copy_to_user((RxConfig_api *) arg, &rc_api, sizeof(RxConfig_api));
			break;
            
		case IFX_SET_QOS_BUCKET: 
			DPRINTK("%s: IFX_SET_QOS_BUCKET\n",__FUNCTION__);
			copy_from_user(&qb_api,(QoSbucket_api *) arg,sizeof (QoSbucket_api));
			err = ifx_set_QoS_bucket(qb_api.chipId,qb_api.tBucketId, qb_api.tBucketCnt,
					  qb_api.tReplenishRateIdx, qb_api.tBurstSizeIdx);
			break;
            
		case IFX_GET_QOS_BUCKET: 
			DPRINTK("%s: IFX_GET_QOS_BUCKET\n",__FUNCTION__);
			copy_from_user(&qb_api,(QoSbucket_api *)arg,sizeof(QoSbucket_api));
			err = ifx_get_QoS_bucket(qb_api.chipId,qb_api.tBucketId,&qb_api.tBucketCnt,
				  &qb_api.tReplenishRateIdx,&qb_api.tBurstSizeIdx);
			if(err == 0)
                copy_to_user((QoSbucket_api *) arg, &qb_api, sizeof(QoSbucket_api));
			break;
            
		case IFX_SET_QOS_BUCKET_FLOW_CONTROL: 
			DPRINTK("%s: IFX_SET_QOS_BUCKET_FLOW_CONTROL\n",__FUNCTION__);
			copy_from_user(&qfc_api,(QoSBflowCntr_api *) arg,sizeof (QoSBflowCntr_api));
			err = ifx_set_QoS_bucket_flow_control(qfc_api.chipId,qfc_api.portRuleBaseIdx, qfc_api.actionLC,
								 qfc_api.actionNC);
			break;
            
		case IFX_GET_QOS_BUCKET_FLOW_CONTROL: 
			DPRINTK("%s: IFX_GET_QOS_BUCKET_FLOW_CONTROL\n",__FUNCTION__);
			copy_from_user(&qfc_api,(QoSBflowCntr_api *)arg,sizeof(QoSBflowCntr_api));
			err = ifx_get_QoS_bucket_flow_control(qfc_api.chipId,&qfc_api.portRuleBaseIdx, 
			   		&qfc_api.actionLC, &qfc_api.actionNC);
			if(err == 0)
		              copy_to_user((QoSBflowCntr_api *) arg, &qfc_api, sizeof(QoSBflowCntr_api));
			break;
            
		case IFX_SET_QOS_WF_QUEUE: 
			DPRINTK("%s: IFX_SET_QOS_WF_QUEUE\n",__FUNCTION__);
			copy_from_user(&qwf_api,(QoSWFqueue_api *) arg,sizeof (QoSWFqueue_api));
			err = ifx_set_QoS_WF_queue(qwf_api.chipId,qwf_api.portId, qwf_api.queueId, 
						  qwf_api.weightFactor, qwf_api.deficitCount);
			break;
            
		case IFX_GET_QOS_WF_QUEUE:
			DPRINTK("%s: IFX_GET_QOS_WF_QUEUE\n",__FUNCTION__);
			copy_from_user(&qwf_api,(QoSWFqueue_api *)arg,sizeof(QoSWFqueue_api));
			err = ifx_get_QoS_WF_queue(qwf_api.chipId,qwf_api.portId, qwf_api.queueId, 
					&qwf_api.weightFactor, &qwf_api.deficitCount);
			if(err == 0)
		              copy_to_user((QoSWFqueue_api *) arg, &qwf_api, sizeof(QoSWFqueue_api));
			break;
            
		case IFX_SET_QOS_SHAPING_QUEUE:
			DPRINTK("%s: IFX_SET_QOS_SHAPING_QUEUE\n",__FUNCTION__);
			copy_from_user(&qsq_api,(QoSshapingQ_api *) arg,sizeof (QoSshapingQ_api));
			err = ifx_set_QoS_shaping_queue(qsq_api.chipId,qsq_api.portId, qsq_api.queueId, qsq_api.bucketCnt,
					  qsq_api.replenishRate, qsq_api.tokenValue,qsq_api.tReplenishTimer);
			break;
            
		case IFX_GET_QOS_SHAPING_QUEUE:
			DPRINTK("%s: IFX_GET_QOS_SHAPING_QUEUE\n",__FUNCTION__);
			copy_from_user(&qsq_api,(QoSshapingQ_api *)arg,sizeof(QoSshapingQ_api));
			err = ifx_get_QoS_shaping_queue(qsq_api.chipId,qsq_api.portId, qsq_api.queueId, &qsq_api.bucketCnt, 
					 &qsq_api.replenishRate, &qsq_api.tokenValue,&qsq_api.tReplenishTimer);
			if(err == 0)
		              copy_to_user((QoSshapingQ_api *) arg, &qsq_api, sizeof(QoSshapingQ_api));
			break;
            
		case IFX_SET_QOS_REPLENISH_RATE:
			DPRINTK("%s: IFX_SET_QOS_REPLENISH_RATE\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *) arg,sizeof (idx_api));
			err = ifx_set_QoS_replenish_rate(ia_api.chipId,ia_api.Idx, ia_api.info);
			break;
            
		case IFX_GET_QOS_REPLENISH_RATE: 
			DPRINTK("%s: IFX_GET_QOS_REPLENISH_RATE:\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *)arg,sizeof(idx_api));
			err = ifx_get_QoS_replenish_rate(ia_api.chipId,ia_api.Idx,&ia_api.info);
			if(err == 0)
		              copy_to_user((idx_api *) arg, &ia_api, sizeof(idx_api));
			break;
            
		case IFX_SET_QOS_BURST_SIZE: 
			DPRINTK("%s: IFX_SET_QOS_BURST_SIZE:\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *) arg,sizeof (idx_api));
			err = ifx_set_QoS_burst_size(ia_api.chipId,ia_api.Idx, ia_api.info);
			break;
            
		case IFX_GET_QOS_BURST_SIZE: 
			DPRINTK("%s: IFX_GET_QOS_BURST_SIZE:\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *)arg,sizeof(idx_api));
			err = ifx_get_QoS_burst_size(ia_api.chipId,ia_api.Idx, &ia_api.info);
			if(err == 0)
		              copy_to_user((idx_api *) arg, &ia_api, sizeof(idx_api));
			break;
            
		case IFX_SET_QOS_EXTEND_BURST_SIZE: 
			DPRINTK("%s: IFX_SET_QOS_EXTEND_BURST_SIZE:\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *) arg,sizeof (idx_api));
			err = ifx_set_QoS_extend_burst_size(ia_api.chipId,ia_api.Idx, ia_api.info);
			break;
            
		case IFX_GET_QOS_EXTEND_BURST_SIZE: 
			DPRINTK("%s: IFX_GET_QOS_EXTEND_BURST_SIZE:\n",__FUNCTION__);
			copy_from_user(&ia_api,(idx_api *)arg,sizeof(idx_api));
			err = ifx_get_QoS_extend_burst_size(ia_api.chipId,ia_api.Idx, &ia_api.info);
			if(err == 0)
		              copy_to_user((idx_api *) arg, &ia_api, sizeof(idx_api));
			break;
            
		default:
			return -ENOTTY;
		break;

	}
		
	return err;
}
