/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup Internal Internally used functions */

 
#include <asm/types.h>
#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/switch.h>
#include "const.h"
#include "api_init.h"
#include "api_init_prv.h"
#include "struct.h"
#include "err.h"
#include "var.h"
#include "switch.h"
#include "vlan_prv.h"
#include "vlan.h"
#include "cos.h"
#include "ports.h"
#include "flow_engine_prv.h"

/*
extern s32 ifx_set_flow_action_all(u8 chipId,u32 flowId,u32 flowAction );

extern s32 ifx_add_flood_entry(u8 chipId,u32 floodId, u32* portListPtr);
*/




/******************************************************************************
  *
  *	FUNCTION:	 ifx_API_init
  *
  *	DESCRIPTION: This function calls following initialization routines:
  *              ifx_add_flood_entry
  *              ifx_Init_Tables
  *				 ifx_VlanFlow_MIBTbl_init
  *				 ifx_ActionTB_init
  *				 ifx_RedCoS_CoS_Tbl_init
  *				 ifx_PVID_init
  *              tbiInit
  *
  *	PARAMETERS:  
  *		IN
  *              Base_Addresses[]: Array of base addresses for all the chips.
  *				 ChipId:    	   ChipId
  *
  *		OUT						   
  *             None
  *
  *	RETURNS:		
  *      PR_ERR_NONE		 	If the API is successful
  *		 PR_ERR_NULL_PTR        If the memory allocation fails  
  *
  *
  ******************************************************************************/

s32 ifx_API_init(u8 chipId)
{
	s32 retflag = 0;

/*    printk("%s\n", __FUNCTION__);  */
    
	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
    {
        printk("%s - Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID\n", __FUNCTION__);
		return PR_ERR_NULL_PTR;
    }

    /* Software Table Initialization */
	if(ifx_Init_Tables(chipId) == PR_ERR_NULL_PTR)
    {
        printk("%s - ifx_Init_Tables() failed\n", __FUNCTION__);
		return PR_ERR_NULL_PTR;
    }


/* Vlan/Flow MIB Table initialization to reset all VLAN / FLow MIB counters 
   to zero, since the counters after system on reset show some dummy values	*/
	retflag = ifx_VlanFlow_MIBTbl_init(chipId);
	if(retflag != PR_ERR_NONE)
    {
        printk("%s - ifx_VlanFlow_MIBTbl_init() failed\n", __FUNCTION__);            
        return retflag;
    }

    /* Action Table initialization - reset values to zero */
	retflag = ifx_ActionTB_init(chipId);
	if(retflag != PR_ERR_NONE)
    {
        printk("%s - ifx_ActionTB_init() failed\n", __FUNCTION__);            
		return retflag;
    }
 
    /* Reduced CoS for CoS register initialization */
	retflag = ifx_RedCoS_CoS_Tbl_init(chipId);
	if(retflag != PR_ERR_NONE)
    {
        printk("%s - ifx_RedCoS_CoS_Tbl_init() failed\n", __FUNCTION__);            
		return retflag;
    }

/* Default PortVlanId initialization for the Ports to one */
		retflag = ifx_PVID_init(chipId);
		if(retflag != PR_ERR_NONE)
        {
            printk("%s - ifx_PVID_init() failed\n", __FUNCTION__);            
			return retflag;
        }
	

/* change the againg defualt to 5 minuts*/
		retflag = ifx_set_switch_mode(chipId,AGING_TIME_SELECT,3);
		if(retflag != PR_ERR_NONE)
        {
            printk("%s - ifx_set_switch_mode(AGING_TIME_SELECT) failed\n", __FUNCTION__);            
			return retflag;
        }

/* allow reserved MAC addr. send to CPU */
		retflag = ifx_set_switch_mode(chipId,SEND_RESERVED_PACKETS_2_CPU,1);
		if(retflag != PR_ERR_NONE)
        {
            printk("%s - ifx_set_switch_mode(SEND_RESERVED_PACKETS_2_CPU) failed\n", __FUNCTION__);
			return retflag;
        }

#if 0
/* FID Learn check bit is set to zero */
    printk("%s -> ifx_set_switch_mode(FID_LEARN_CHECK)\n", __FUNCTION__);

		retflag = ifx_set_switch_mode(chipId,FID_LEARN_CHECK,0);
		if(retflag != PR_ERR_NONE)
        {
            printk("%s - ifx_set_switch_mode(FID_LEARN_CHECK) failed\n", __FUNCTION__);            
			return retflag;
        }
#endif
#ifdef UCLINUX 
/*Nirint: setting Unknown unicast and mcast flood id to 254, broadcast to 255*/
	ifx_set_unknown_ucast_FID(0, 254);
	ifx_set_unknown_mcast_FID(0, 254);
	ifx_set_bcast_FID(0, 255);
#endif

     return PR_ERR_NONE;

}





/*****************************************************************************
 *
 * FUNCTION:  ifx_get_base_address
 *				
 * DESCRIPTION: Gets the base address for the specified chip ID.  
 *			
 * PARAMETERS:
 *				IN:
 *					chipId - ChipId
 *				OUT:
 *					Base_AddressPtr - Pointer to the base address of the specified Purple
 *									  IC.
 *			
 * RETURNS:
 *		 PR_ERR_NONE		 	If the API is successful
 *		 PR_ERR_CHIP_ID 		If the chip ID is invalid
 *       PR_ERR_NULL_PTR        If the memory allocation fails
 *
 ***************************************************************************/

s32 ifx_get_base_address(u8 chipId,u32* Base_Address)
{

#ifdef PARAMETER_CHECK

	if(chipId >= Total_Chips) 
			return PR_ERR_CHIP_ID;
/*
	if(Base_Address == NULL_PTR)
		return PR_ERR_NULL_PTR;
*/
#endif

	*Base_Address = Chip_Base_Address[chipId];	


	return PR_ERR_NONE;
}


/*****************************************************************************
 *
 * FUNCTION:  ifx_set_base_address
 *				
 * DESCRIPTION: Sets the base address for the specified chip ID.  
 *			
 * PARAMETERS:
 *				IN:
 *					chipId -       Chipid
 *                  
 *                  Base_Address - Base address of the specified chip ID
 *
 *				OUT:
 *					None
 *			
 * RETURNS:
 *		 	PR_ERR_NONE 		If the API is successful
 *			PR_ERR_CHIP_ID 		If the chip ID is invalid
 *			PR_ERR_NULL_PTR 	If the memory allocation fails
 *			PR_ERR_CHIP_VID 	If the chip ID is greater than the maximum number of
 *								Purple ICs on board.
 *
 ***************************************************************************/

s32 ifx_set_base_address(u8 chipId,u32 Base_Address)
{

    printk("%s\n", __FUNCTION__);
#ifdef PARAMETER_CHECK
	if(chipId >= Total_Chips) 
			return PR_ERR_CHIP_ID;

#endif		


    /* 
     * if base address is already used 
     * free data structures before re-initialization 
     */
	if(Chip_Base_Address[chipId] != CHIP_ADDRESS_INVALID)
	{
		kfree(PR_TABLE_PORTSTATUS[chipId]);
		kfree(PR_TABLE_VID[chipId]) ;
		kfree(PR_VLAN_VID_ASC_TBL[chipId]) ;
		kfree(PR_INGRESS_FILTER[chipId]);
		kfree(PR_VLAN_VID_TBL[chipId]) ;
		kfree(cache_speed[chipId]) ;
		kfree(cache_duplex[chipId]);
		kfree(Ikey_Sel_Reg[chipId]) ;		

	}
    Chip_Base_Address[chipId] = Base_Address;
    return(ifx_API_init(chipId));
}


/*****************************************************************************
 *
 * FUNCTION:  ifx_set_chip_number
 *				
 * DESCRIPTION: Sets the total number of on-board Purple ICs.  
 *			
 * PARAMETERS:
 *				IN:
 *                  No_of_Chips - Total number of chips supported on-board
 *
 *				OUT:
 *					None
 *			
 * RETURNS:
 *		 	PR_ERR_NONE 		If the API is successful
 *			PR_ERR_NULL_PTR 	If the memory allocation fails
 *          PR_ERR_CHIP_NUM     If the Chip number is greater then the maximum numbers.
 *
 ***************************************************************************/


s32 ifx_set_chip_number(u8 No_of_Chips)
{
	
	u32 * temp1 = NULL ;
	port_status_per_vlan ** temp_port_stat = NULL ;
	u8 ** temp8 = NULL ;
	u32 ** temp32 = NULL ;
	u8 old_num = 0;
	u8 copy_num = 0;
	u8 i = 0;
	static BOOL allocated = FALSE ;

    printk("%s\n", __FUNCTION__);
    
	if(No_of_Chips > Total_Chips)
		return PR_ERR_CHIP_NUM ;

    old_num = Total_Chips ;
	Total_Chips = No_of_Chips ;
	copy_num = (old_num < No_of_Chips) ? old_num : No_of_Chips ;

		
	if(allocated == FALSE)
	{
		Chip_Base_Address = (u32*)(MALLOC(sizeof(u32) * No_of_Chips)) ;
		if(Chip_Base_Address == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			Chip_Base_Address[i] = CHIP_ADDRESS_INVALID ;

		PR_TABLE_PORTSTATUS = (port_status_per_vlan **) MALLOC(
					       sizeof(u32) * No_of_Chips) ;
		if(PR_TABLE_PORTSTATUS == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_TABLE_PORTSTATUS[i] = NULL ;

		PR_TABLE_VID =	(u32**) MALLOC( sizeof(u32) * No_of_Chips ) ;
		if(PR_TABLE_VID == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_TABLE_VID[i] = NULL ;

		PR_VLAN_VID_ASC_TBL = (u8**) MALLOC( sizeof(u32) * No_of_Chips);
		if(PR_VLAN_VID_ASC_TBL == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_VLAN_VID_ASC_TBL[i] = NULL ;

		PR_INGRESS_FILTER = (u8**) MALLOC( sizeof(u32) * No_of_Chips);
		if(PR_INGRESS_FILTER == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_INGRESS_FILTER[i] = NULL ;
	
		PR_VLAN_VID_TBL = (u32**) MALLOC( sizeof(u32) * No_of_Chips);
		if(PR_VLAN_VID_TBL == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_VLAN_VID_TBL[i] = NULL ;

        PR_VLAN_SW_TBL = (vlan_tbl_entry_u**) MALLOC( sizeof(vlan_tbl_entry_u) * No_of_Chips);
		if(PR_VLAN_SW_TBL == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			PR_VLAN_SW_TBL[i] = NULL ;
         
		PR_CHIP_MAJR_VER_ID = (u32 *) MALLOC(sizeof(u32) * No_of_Chips) ;
		if(PR_CHIP_MAJR_VER_ID == NULL)
			return PR_ERR_NULL_PTR ;

		PR_CHIP_MINR_VER_ID = (u32 *) MALLOC(sizeof(u32) * No_of_Chips) ;
		if(PR_CHIP_MINR_VER_ID == NULL)
			return PR_ERR_NULL_PTR ;


		readIndex = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(readIndex == NULL)
			return PR_ERR_NULL_PTR ;

		writeIndex = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(writeIndex == NULL)
			return PR_ERR_NULL_PTR ;

		freeEntries = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(freeEntries == NULL)
			return PR_ERR_NULL_PTR ;

		fill_level = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(fill_level == NULL)
			return PR_ERR_NULL_PTR ;        
        
		cache_speed = (u8 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(cache_speed == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			cache_speed[i] = NULL ;

		cache_duplex = (u8 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(cache_duplex == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			cache_duplex[i] = NULL ;

		Ikey_Sel_Reg = (u32 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(Ikey_Sel_Reg == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<	No_of_Chips; i++)
			Ikey_Sel_Reg[i] = NULL ;

		if(ifx_sem_create() != PR_ERR_NONE)
			return PR_SEM_CRT_FAIL;

		allocated = TRUE ;
	}

	else
	{

/* Added for re-entrancy  */
		if(ifx_sem_delete(old_num) != PR_ERR_NONE)
			return PR_SEM_DEL_FAIL;
		
		if(ifx_sem_create() != PR_ERR_NONE)
			return PR_SEM_CRT_FAIL;		

		
		temp_port_stat= (port_status_per_vlan **) MALLOC(
					       sizeof(u32) * No_of_Chips) ;
		if(temp_port_stat == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp_port_stat[i] = PR_TABLE_PORTSTATUS [i] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp_port_stat[i] = NULL ;

		}
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID )
					kfree(PR_TABLE_PORTSTATUS[i]) ;
			}
		}
		kfree(PR_TABLE_PORTSTATUS) ;
		PR_TABLE_PORTSTATUS = temp_port_stat ;



		temp32 =	(u32**) MALLOC( sizeof(u32) * No_of_Chips ) ;
		if(temp32 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp32[i] = PR_TABLE_VID[i] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp32[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(PR_TABLE_VID[i]) ;
			}
		}
		kfree(PR_TABLE_VID);
		PR_TABLE_VID = temp32 ;



		temp8 = (u8**) MALLOC( sizeof(u32) * No_of_Chips);
		if(temp8 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp8[i] = PR_VLAN_VID_ASC_TBL [i ] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp8[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(PR_VLAN_VID_ASC_TBL[i]) ;
			}
		}
		kfree(PR_VLAN_VID_ASC_TBL);
		PR_VLAN_VID_ASC_TBL = temp8 ;



		temp8 = (u8**) MALLOC( sizeof(u32) * No_of_Chips);
		if(temp8 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp8[i] = PR_INGRESS_FILTER[i];
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp8[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(PR_INGRESS_FILTER[i]) ;
			}
		}
		kfree(PR_INGRESS_FILTER);
		PR_INGRESS_FILTER = temp8;


		

		temp32 = (u32**) MALLOC( sizeof(u32) * No_of_Chips);
		if(temp32 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp32[i] = PR_VLAN_VID_TBL [i ] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp32[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(PR_VLAN_VID_TBL[i]) ;
			}
		}
		kfree(PR_VLAN_VID_TBL);
		PR_VLAN_VID_TBL = temp32 ;

		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;		
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = PR_CHIP_MAJR_VER_ID[i] ;

		kfree(PR_CHIP_MAJR_VER_ID);
		PR_CHIP_MAJR_VER_ID = temp1 ;



		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;		
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = PR_CHIP_MINR_VER_ID[i] ;

		kfree(PR_CHIP_MINR_VER_ID);
		PR_CHIP_MINR_VER_ID = temp1 ;



	

		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = readIndex [i ];
		kfree(readIndex);
		readIndex = temp1 ;


		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;	
			if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = writeIndex [i ] ;
		kfree(writeIndex);
		writeIndex = temp1 ;
	

		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = freeEntries [i] ;
		kfree(freeEntries);
		freeEntries = temp1 ;

		temp1 = (u32 *) MALLOC(sizeof(u32) * No_of_Chips ) ;
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = fill_level [i] ;
		kfree(fill_level);
		fill_level = temp1 ;

        
		temp8 = (u8 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(temp8 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp8[i] = cache_speed [i ] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp8[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)		
					kfree(cache_speed[i]) ;
			}
		}
		kfree(cache_speed);
		cache_speed = temp8 ;


		temp8 = (u8 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(temp8 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp8[i] = cache_duplex [i] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp8[i] = NULL ;
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(cache_duplex[i]) ;
			}
		}
		kfree(cache_duplex);
		cache_duplex = temp8 ;


	   	temp32 = (u32 **) MALLOC(sizeof(u32) * No_of_Chips ) ;	
		if(temp32 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp32[i] = Ikey_Sel_Reg [i ] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp32[i] = NULL ;
			
		}	
		else
		{
			for(i=No_of_Chips; i<old_num; i++)
			{
				if(Chip_Base_Address[i] != CHIP_ADDRESS_INVALID)
					kfree(Ikey_Sel_Reg[i]) ;
			}
		}
		kfree(Ikey_Sel_Reg);
		Ikey_Sel_Reg = temp32 ;


		temp1 = (u32*)( MALLOC(sizeof(u32) * No_of_Chips)) ;
		if(temp1 == NULL)
			return PR_ERR_NULL_PTR ;
		for(i=0; i<copy_num; i++)
			temp1[i] = Chip_Base_Address[i] ;
		if(old_num < Total_Chips)
		{
			for(i=old_num;  i<No_of_Chips; i++)
				temp1[i] = CHIP_ADDRESS_INVALID ;			
		}
		kfree(Chip_Base_Address);
		Chip_Base_Address = temp1 ;



	}

	return PR_ERR_NONE ;

}

/*****************************************************************************
 *
 * FUNCTION:  ifx_get_chip_number
 *				
 * DESCRIPTION: Sets the total number of on-board Purple ICs.  
 *			
 * PARAMETERS:
 *				IN:
 *                 None 
 *
 *				OUT:
 *					No_of_Chips - Total number of chips supported on-board
 *			
 * RETURNS:
 *		 	PR_ERR_NONE 		If the API is successful
 *			PR_ERR_NULL_PTR 	If the Pointer is NULL 
 *
 ***************************************************************************/


s32 ifx_get_chip_number(u8* No_of_Chips)
{

#ifdef PARAMETER_CHECK
	if(No_of_Chips == NULL)
		return PR_ERR_NULL_PTR;
#endif			

	*No_of_Chips  = Total_Chips	;
	return PR_ERR_NONE;

}



/*****************************************************************************
 *
 * FUNCTION:  ifx_release_API_mem
 *				
 * DESCRIPTION: This function is used to Realse the memory allocated for one chip  
 *			
 * PARAMETERS:
 *				IN:
 *                  No_of_Chips - Total number of chips supported on-board
 *
 *				OUT:
 *					None
 *
 * RETURNS:
 *		 PR_ERR_NONE           If the API is successful
 *
 ***************************************************************************/
s32 ifx_release_API_mem(u8 No_of_Chips)
{
	u32 i=0;
	for(i=0;i<No_of_Chips;i++)
	{
		ifx_release_chip_mem(i);
	}
	
	if(PR_TABLE_PORTSTATUS!=NULL)
		kfree(PR_TABLE_PORTSTATUS);

	if(PR_TABLE_VID!=NULL)
		kfree(PR_TABLE_VID) ;   
	
	if(PR_VLAN_VID_ASC_TBL!=NULL)
		kfree(PR_VLAN_VID_ASC_TBL) ;
	
	if(PR_VLAN_VID_TBL!=NULL)
		kfree(PR_VLAN_VID_TBL) ;
	
	if(cache_speed!=NULL)
		kfree(cache_speed) ;
	
	if(cache_duplex!=NULL)
		kfree(cache_duplex);
	
	if(Ikey_Sel_Reg!=NULL)
		kfree(Ikey_Sel_Reg) ;

	return PR_ERR_NONE;

}


/********************************************* Internal API Definition **************************************************/


/******************************************************************************
  *
  *	FUNCTION:	 ifx_Init_Tables
  *
  *	DESCRIPTION: This function initializes the following software tables:
  *              Vlanindex(VID) Table,
  *              vlanId to vid association table, 
  *              vlan membership and port status table, 
  *              Hardware table: vlanId to vid table
  *              In order to support priority tagged packet vlan 0 is created
  *              with all ports as members and with egress tag cleared.
  *              The global variable stp_pvst is initialized to support STP.        
  *              
  *	PARAMETERS:  
  *				 ChipId : ChipId.
  *			
  *	RETURNS:	 
  *         PR_ERR_NULL_PTR          If the memory allocation fails
  *         PR_ERR_NONE              If the API is successful 		
  *		
  ******************************************************************************/

s32 ifx_Init_Tables(u8 chipId)
{	

	s32 i=0;
   	s32 j=0; 
	u32 chipMjrId = 0;
    u32 chipMnrId = 0;

    /* Allocate memory for all the Software Tables */
	if(ifx_Allocate_ChipMem(chipId) == PR_ERR_NULL_PTR)
	    return PR_ERR_NULL_PTR  ;

    /* Initialization of VLAN related Software Tables: 
       VID Table,Vlan membership & Port Status Table and VLANID to VID association Table */	
    for(i = 0; i < NUMBER_VIDS; i++)
        PR_TABLE_VID[chipId][i] = MAX_VID-i;
			
    /* Initialization of Vlan membership & Port Status Table */	
	for(i=0;i<(TOTAL_VIDS);i++)
	{
		for(j=0;j<(TOTAL_PORTS);j++)
		{
			PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(i,j)] . vlanmember = 0;
			PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(i,j)] . port_state = IFX_PORT_FORWARDING;
			PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(i,j)] . Rsv = 0;
		}
	} 
/* Initialize the software vlanmembership table to 1 for VID 1022 for all the ports  */
//		for(j=0;j<(TOTAL_PORTS);j++)
//			PR_TABLE_PORTSTATUS[chipId][PR_Table_Portstatus(DEFAULT_VID_1022,j)] . vlanmember = 1;


    /* Initialization of VLANID to VID association Table */		
	for(i=0;i<(TOTAL_VLAN_IDS);i++)
		PR_VLAN_VID_ASC_TBL[chipId][i] = 0;

    /* For vlan Id 0 association is made 1 */
//	PR_VLAN_VID_ASC_TBL[chipId][0] = 1;		


    /* Configure the VlanID to VID table to DEFAULT_VID_1022 */
	for(i=0;i<(TOTAL_VLAN_IDS);i++)
	{
		/*	printf("");*/
/*			retflag = ifx_set_VLAN_idx(chipId,i,DEFAULT_VID_1022);  
			if(retflag != PR_ERR_NONE)
				return retflag;
*/
        PR_VLAN_VID_TBL[chipId][i] = DEFAULT_VID;
	}


    /* Set the VID for Priority-tagged Packet as 1022 in the software table */
	PR_VLAN_VID_TBL[chipId][0] = DEFAULT_VID_1022;


/* Write the Portstate 00 in the Vlan table for DEFAULT_VID 1023 */
//		portState = 0x0;
//		for(i=0;i<(TOTAL_PORTS);i++)
//			ifx_set_Vlan_table_portstate(chipId,DEFAULT_VID,i,portState);

	
/* Initialize the Ingress Filter software table to zero */
	for(i=0;i<TOTAL_PORTS; i++)
		PR_INGRESS_FILTER[chipId][i] = 0;

/* Initialize the chip major and minor version Id */
		ifx_get_chip_version(chipId, &chipMjrId,&chipMnrId);
		PR_CHIP_MAJR_VER_ID[chipId] = chipMjrId;
		PR_CHIP_MINR_VER_ID[chipId] = chipMnrId;
		
    
	for( i=0; i<TOTAL_VIDS; i++)
    {
        PR_VLAN_SW_TBL[chipId][i].val = VLAN_ENTRY_INVALID; 
    }

        
        
        
        
/* Global index to a currently available vlanindex(VID) in the PR_TABLE_VID table */	
		readIndex[chipId] = 0;  
	
/* Global index to add a deleted vlanindex(VID) in the PR_TABLE_VID table */
		writeIndex[chipId] = WRITE_INDEX; 

/* Number of VID's supported */
		freeEntries[chipId] = NUMBER_VIDS;

/* Number of VID's supported */
		fill_level[chipId] = 0;        
        
/* Default is STP */
//		stp_pvst[chipId] = STP;


/* Duplex - Speed initialization for Ports 0 - 49 */
	for(i=0; i<(TOTAL_PORTS-3); i++)
		cache_speed[chipId][i] = 1;

	cache_speed[chipId][GE_PORT_0] = 2;
	cache_speed[chipId][GE_PORT_1] = 2;

	for(i=0; i<(TOTAL_PORTS-1); i++)
		cache_duplex[chipId][i] = 1; 

/* IKEY Selection register - implemented in software for read(hardware bug - this is write only 
	in hardware). Store zero  in this software implemented register */

	for(i=0; i<(TOTAL_PORTS - 1) ; i++)
		Ikey_Sel_Reg[chipId][i] = 0;

	return PR_ERR_NONE;
}

/******************************************************************************
  *
  *	FUNCTION:	     ifx_VlanFlow_MIBTbl_init
  *
  *	DESCRIPTION:     Vlan/Flow MIB initialization routine. 
  *              		 
  *              
  *	PARAMETERS:
  *        IN   
  *               ChipId :   ChipId.
  *          
  *       OUT
  *                 None
  *			
  *	RETURNS:	    PR_ERR_NONE           If the API is successful
  *		
  *		
  ******************************************************************************/

s32 ifx_VlanFlow_MIBTbl_init(u8 chipId)
{
	u32 idx =0;
	u32 counterID=0; 
	s32 retflag;

/*  VLAN/FLOW MIB Table Initialization */
	for (idx = 0;idx <= MAX_VID;idx++)
	{
		for ( counterID = 0; counterID <= 2; counterID++ )
		{
			retflag = ifx_set_VLAN_FLOW_mib(chipId, idx, counterID,0); 
			if(retflag != PR_ERR_NONE)
				return retflag;
		}
			
	}
	
	return PR_ERR_NONE;
}




/******************************************************************************
  *
  *	FUNCTION:	     ifx_set_VLAN_FLOW_mib
  *
  *	DESCRIPTION:     Vlan/Flow MIB routine. 
  *              		 
  *              
  *	PARAMETERS:
  *        IN   
  *               ChipId :   ChipId.
  *               VlanFlowIdx: Range 0-1023
  *               CounterId : Range  0-2
  *               count     :  count range is 0-8190
  *          
  *       OUT
  *                 None
  *			
  *	RETURNS:	    PR_ERR_VLAN_MIBIDX
  *                 PR_ERR_MIB_CNTID
  *                 PR_ERR_CHIP_ID
  *                 PR_ERR_NULL_PTR
  *                 PR_ERR_TIME_OUT
  *                 PR_ERR_NONE
  *                  
  *		
  ******************************************************************************/


s32 ifx_set_VLAN_FLOW_mib(u8 chipId,u32 VlanFlowIdx, u32 counterId, u32 count)
{
	

/* Define a pointer to a structure & assign the base address of VLAN/Rule MIBs 
   Command register */
	volatile vlan_mibs_cmd_reg_t*  ptr_vlan_mibs_cmd;

/* Define a pointer to VLAN/Rule MIBs data register */
	u32* ptr_vlan_mibs_data;
	
	volatile u32 temp1=0;
	s32  i=0;
	u32* ptr_reg1;

#ifdef PARAMETER_CHECK

	if(VlanFlowIdx > MAX_VlanFlowIdx)
		return PR_ERR_VLAN_MIBIDX;

	if(counterId > 2)
		return PR_ERR_MIB_CNTID;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

    
	if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
		return PR_ERR_NULL_PTR;

#endif

	ptr_vlan_mibs_data = (u32*) (Chip_Base_Address[chipId] + VLAN_MIBS_DATA_REG);

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + VLAN_MIBS_CMD_REG);


	ptr_vlan_mibs_cmd = (volatile vlan_mibs_cmd_reg_t*)&temp1;
	PR_Reg_Read(ptr_reg1,temp1);
	

/* Write Counter value of MIB selected from Data register */
	PR_Reg_Write(ptr_vlan_mibs_data,count);


/* Configuration of VLAN/Rule MIBs Command register for reading */
/* Address(9:0) - Index of MIB set */
	ptr_vlan_mibs_cmd -> Address = VlanFlowIdx;

/* MIB ID(29:28) - 00 -> Byte Count MIB
                   01 -> Packet Count MIB  
				   10 -> Dropped Packet Count MIB */
	ptr_vlan_mibs_cmd -> MIBId = counterId;

/* Read/Write(30) : 1-Read */
	ptr_vlan_mibs_cmd -> Read_write = WRITEPR;

/* Request/Acknowledge(31): 1-Request */
	ptr_vlan_mibs_cmd -> Req_ack = REQ;

	PR_Reg_Write(ptr_reg1,temp1);
	
	for (i=0; i<PR_TIME_OUT;i++)
	{
		PR_Reg_Read(ptr_reg1,temp1);
		if(ptr_vlan_mibs_cmd -> Req_ack == ACK) 
		break;

	}

	if((ptr_vlan_mibs_cmd -> Req_ack) != ACK) 
 		return PR_ERR_TIME_OUT;

	return PR_ERR_NONE ;

}


/*****************************************************************************
 *
 * FUNCTION:  ifx_ActionTB_init
 *				
 * DESCRIPTION: This function is used to clear the Action table Entries to zero  
 *			
 * PARAMETERS:
 *
 *      IN 
 *       	 ChipId:    	   ChipId
 *		OUT 
 *            None
 *
 * RETURNS:
 *		 PR_ERR_NONE
 *
 ***************************************************************************/

s32 ifx_ActionTB_init(u8 chipId)
{
	s32 i=0;
	s32 retflag = 0;

	for(i=0; i < TOTAL_FLOW_ID; i++)
	{
		retflag = ifx_set_flow_action_all(chipId,i,0);
		if(retflag != PR_ERR_NONE)
			return retflag;	
	}

	return PR_ERR_NONE;

}



/******************************************************************************
  *
  *	FUNCTION:	 		ifx_RedCoS_CoS_Tbl_init
  *
  *	DESCRIPTION:     	Reduced Priority for CoS table initialization routine. 
  *              		 
  *              
  *	PARAMETERS:  	
  *      IN
  *        	 ChipId:    	   ChipId
  *		 OUT	
  *          None
  *  
  *	RETURNS:	 		PR_ERR_NONE
  *		
  *		
  ******************************************************************************/
s32 ifx_RedCoS_CoS_Tbl_init(u8 chipId)
{
	s32 i=0;
    s32 retflag =0;

	for(i=0;i<4;i++)
	{
		retflag = ifx_map_CoS_reduction(chipId,i,i);
		if(retflag != PR_ERR_NONE)
			return retflag;
	}
	return PR_ERR_NONE;
}




/******************************************************************************
  *
  *	FUNCTION:	     ifx_PVID_init
  *
  *	DESCRIPTION:     Initialize the default PVID for the port as 1. 
  *              		 
  *              
  *	PARAMETERS:  
  *      IN   
  *        	 ChipId:    	   ChipId
  *     OUT
  *           None
  *			
  *	RETURNS:	     PR_ERR_NONE
  *		
  *		
  ******************************************************************************/

s32 ifx_PVID_init(u8 chipId)
{
	u32 pvid = 1;
    s32 i=0;
	s32 retflag=0;
	
	for(i=0;i<TOTAL_PORTS;i++)
	{
		retflag = ifx_set_port_VLANId(chipId,i,pvid);
		if(retflag != PR_ERR_NONE)
		{
			if(retflag != PR_ERR_VLAN_NOTCONFG)
				return retflag;
		}
	}
	
	return PR_ERR_NONE;
}
		

/******************************************************************************
  *
  *	FUNCTION:	     ifx_release_chip_mem
  *
  *	DESCRIPTION:     This function is  used to release a allocated memory for a chip. 
  *              		 
  *              
  *	PARAMETERS:  
  *      IN   
  *        	 ChipId:    	   ChipId.
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	     PR_ERR_NONE
  *		
  *		
  ******************************************************************************/



s32 ifx_release_chip_mem(u8 chipId)
{
	if(PR_TABLE_PORTSTATUS[chipId]!=NULL)
		kfree(PR_TABLE_PORTSTATUS[chipId]);

    if(PR_TABLE_VID[chipId]!=NULL)
		kfree(PR_TABLE_VID[chipId]) ;   
	
	if(PR_VLAN_VID_ASC_TBL[chipId]!=NULL)
		kfree(PR_VLAN_VID_ASC_TBL[chipId]) ;
	
	if(PR_VLAN_VID_TBL[chipId]!=NULL)
		kfree(PR_VLAN_VID_TBL[chipId]) ;
	
	if(cache_speed[chipId]!=NULL)
		kfree(cache_speed[chipId]) ;
	
	if(cache_duplex[chipId]!=NULL)
		kfree(cache_duplex[chipId]);
	
	if(Ikey_Sel_Reg[chipId]!=NULL)
		kfree(Ikey_Sel_Reg[chipId]) ;
	return PR_ERR_NONE;
				
}
/******************************************************************************
  *
  *	FUNCTION:	 		ifx_Allocate_ChipMem
  *
  *	DESCRIPTION:     	Allocate the memory for all Tables & variables used in 
  *						the chip. 
  *              		 
  *              
  *	PARAMETERS: 
  *      IN   
  *        	 ChipId:    	   ChipId.
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	 		PR_ERR_NONE
  *                    PR_ERR_NULL_PTR
  *		
  *		
  ******************************************************************************/

s32 ifx_Allocate_ChipMem(u8 chipId)
{

	PR_TABLE_PORTSTATUS[chipId] = 
        (port_status_per_vlan *) MALLOC( sizeof(port_status_per_vlan) \
                                         *TOTAL_PORTS * TOTAL_VIDS) ;
	if(PR_TABLE_PORTSTATUS[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	PR_TABLE_VID[chipId] =	(u32*) MALLOC( sizeof(u32) * NUMBER_VIDS) ;
	if(PR_TABLE_VID[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	PR_VLAN_VID_ASC_TBL[chipId] = (u8*) MALLOC( sizeof(u8) * TOTAL_VLAN_IDS);
	if(PR_VLAN_VID_ASC_TBL[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	PR_INGRESS_FILTER[chipId] = (u8*) MALLOC( sizeof(u8) * TOTAL_PORTS);
	if(PR_INGRESS_FILTER[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	PR_VLAN_VID_TBL[chipId] = (u32*) MALLOC( sizeof(u32) * TOTAL_VLAN_IDS );
	if(PR_VLAN_VID_TBL[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

    PR_VLAN_SW_TBL[chipId] =
       (vlan_tbl_entry_u*) MALLOC( sizeof(vlan_tbl_entry_u) * TOTAL_VIDS );
	if(PR_VLAN_SW_TBL[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

/* Added the following 3 tables for 4k vlan support on 24-3-2004   */
#if 0
	PR_VLAN_MULTIPLE_VID[chipId] = (u32*) MALLOC( sizeof(u32) * TOTAL_VIDS );
	if(PR_VLAN_MULTIPLE_VID[chipId] == NULL)
		return PR_ERR_NULL_PTR ;


	PR_VLAN_SHARE_IND[chipId] = (u8 *) MALLOC(sizeof(u8) * TOTAL_VLAN_IDS) ;
	if(PR_VLAN_SHARE_IND[chipId] == NULL)
		return PR_ERR_NULL_PTR ;



	PR_VLAN_STP_MSTP[chipId] = (u8 *) MALLOC(sizeof(u8) * TOTAL_VLAN_IDS) ;
	if(PR_VLAN_STP_MSTP[chipId] == NULL)
		return PR_ERR_NULL_PTR ;
#endif
	
	
	cache_speed[chipId] = (u8 *) MALLOC(sizeof(u8) * (TOTAL_PORTS -1)) ;
	if(cache_speed[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	cache_duplex[chipId] = (u8 *) MALLOC(sizeof(u8) *(TOTAL_PORTS -1)) ;
	if(cache_duplex[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	Ikey_Sel_Reg[chipId] = (u32 *) MALLOC(sizeof(u32) * (TOTAL_PORTS - 1)) ;
	if(Ikey_Sel_Reg[chipId] == NULL)
		return PR_ERR_NULL_PTR ;

	

	return PR_ERR_NONE  ;

}


/******************************************************************************
  *
  *	FUNCTION:	 		ifx_sem_take
  *
  *	DESCRIPTION:     	It calls the respective semaphore take system call 
  *                     depending upon the Operating System used.
  *
  *              
  *	PARAMETERS: 
  *      IN   
  *        	 chipId		    - ChipId Id
  *			 sem_type       - Semaphore type Id,an index to semaphore array 
  *
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	 		 
  *			PR_ERR_NONE			If it is successfull	
  *			PR_ERR_SEM			If it fails
  *		
  *		
  ******************************************************************************/



s32 ifx_sem_take (u8 chipId,u8 sem_type)
{
/* Checked - Add real semaphore handling */
#ifdef  REENTRANCY

	#ifdef  WINDRIVER
			if(semTake (ifxsem_array[chipId][sem_type],SEM_TAKE_FLAGS) != OK)
				return	PR_ERR_SEM;
	#endif

#endif

	return PR_ERR_NONE;

}



/******************************************************************************
  *
  *	FUNCTION:	 		ifx_sem_give
  *
  *	DESCRIPTION:     	It calls the respective semaphore give system call 
  *                     depending upon the Operating System used.
  *
  *              
  *	PARAMETERS: 
  *      IN   
  *        	 chipId      - chipId
  *          sem_type    - Semaphore type Id,an index to semaphore array
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	 		 
  *			PR_ERR_NONE			If it is successfull	
  *			PR_ERR_SEM			If it fails
  *		
  *		
  ******************************************************************************/

s32 ifx_sem_give(u8 chipId,u8 sem_type)
{

#ifdef  REENTRANCY

	#ifdef  WINDRIVER
		if(semGive (ifxsem_array[chipId][sem_type]) != OK)
			return	PR_ERR_SEM;
	#endif

#endif

	return PR_ERR_NONE;

}


/******************************************************************************
  *
  *	FUNCTION:	 		ifx_sem_create
  *
  *	DESCRIPTION:     	It creates all the required semaphores by calling the 
  *                     respective semaphore create system call depending upon 
  *                     the Operating System used
  *
  *
  *              
  *	PARAMETERS: 
  *      IN   
  *        	  None
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	 		 
  *				PR_ERR_NONE			If it is successfull	
  *				PR_SEM_CRT_FAIL		If semaphore creation fails
  *				PR_ERR_NULL_PTR		If the memory allocation is failed
  *
  ******************************************************************************/

s32 ifx_sem_create(void)
{

#ifdef REENTRANCY

	s32 i=0;
	s32 j=0;
	u32 no_chip = Total_Chips; 


	ifxsem_array =	(SEM_TYPE **) MALLOC(sizeof(u32) * no_chip );

	if(ifxsem_array == NULL)
		return PR_ERR_NULL_PTR ;

	for(i=0;i<no_chip;i++)
	{
		ifxsem_array[i] = (SEM_TYPE *) MALLOC(sizeof(SEM_TYPE) *  TOTAL_SEM);	

		if(ifxsem_array[i] == NULL)
			return PR_ERR_NULL_PTR ;
	}

	
	for(i=0;i < no_chip;i++)
	{
		for(j=0;j < TOTAL_SEM;j++)
		{
			ifxsem_array[i][j] = SEMCREATE(SEM_CREATE_FLAGS);
			if(ifxsem_array[i][j] == NULL)
				return PR_SEM_CRT_FAIL;
		}
	}
  
#endif
	
	return PR_ERR_NONE;

}


/******************************************************************************
  *
  *	FUNCTION:	 		ifx_sem_delete
  *
  *	DESCRIPTION:     	It  deletes all the semaphores by calling the respective 
  *                     semaphore delete system call depending upon the Operating 
  *                     System used
  *
  *
  *              
  *	PARAMETERS: 
  *      IN   
  *        	  old_num	Number of purple chips
  *								   
  *     OUT
  *           None
  *			
  *	RETURNS:	 		 
  *				PR_ERR_NONE			If it is successfull	
  *				PR_SEM_DEL_FAIL		If semaphore deletion fails
  *
  ******************************************************************************/	
 
s32 ifx_sem_delete(u8 old_num)
{

#ifdef REENTRANCY 

	s32 i=0;
	s32 j=0;

	for(i=0;i < old_num;i++)
	{
		for(j=0;j < TOTAL_SEM;j++)
		{
			if(SEMDELETE(ifxsem_array[i][j]) != OK)
				return PR_SEM_DEL_FAIL;
		}
		
		kfree(ifxsem_array[i]);

	}

	kfree(ifxsem_array);

#endif

	return PR_ERR_NONE;

}





