/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Aparna   		    1/9/2003					        	
 *
 *
 * Manamohan			10/8/2004					Added support for re-entrancy
 *
 * DESCRIPTION :This module contains the Interrupt Configuration API's
 *
 ******************************************************************************/
#include <asm/types.h>
#include <asm/addrspace.h>
#include "struct.h"
#include "const.h"
#include "interrupt.h"
#include <asm/incaip2/incaip2.h>
#include "err.h"
#include "var.h"
#include "api_init_prv.h"

#if 0
/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_CPU_IMR
 * 
 * DESCRIPTION:	Set the interrupt mask for MIPS High/Low Priority interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *          ChipId:    	ChipId.
 *			IMRHi :		MIPS High Priority Interrupt Mask
 *			IMRLo :		MIPS low priority Interrupt Mask
 *            
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_NULL_PTR         Null Pointer
 *		PR_ERR_CHIP_ID          Invalid Chip Id 
 *
 *****************************************************************************/

s32 ifx_set_CPU_IMR(u8 chipId,u32 IMRHi, u32 IMRLo)			
{

	mips_imr_hi_reg_t*   ptr_mips_imr_hi;
	mips_imr_lo_reg_t*   ptr_mips_imr_lo;

	s32 retflag=PR_ERR_NONE;

#ifdef	PARAMETER_CHECK
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_mips_imr_hi  = (mips_imr_hi_reg_t*) (Chip_Base_Address[chipId] + MIPS_IMR_HI_REG);
	ptr_mips_imr_lo  = (mips_imr_lo_reg_t*) (Chip_Base_Address[chipId] + MIPS_IMR_LO_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;


	PR_Reg_Write(ptr_mips_imr_hi,IMRHi);
	PR_Reg_Write(ptr_mips_imr_lo,IMRLo);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE;

	return -1;
}
#endif    	
/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_CPU_IMR
 * 
 * DESCRIPTION:	Get the interrupt mask for MIPS High/Low Priority interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId:    	ChipId.
 *		OUT:
 *			IMRHiPtr
 *			IMRLoptr
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer
 *		 
 *****************************************************************************/
#if 0
s32 ifx_get_CPU_IMR(u8 chipId,u32* IMRHiPtr, u32* IMRLoPtr)
{

	mips_imr_hi_reg_t*   ptr_mips_imr_hi;
	mips_imr_lo_reg_t*   ptr_mips_imr_lo;
	u32 temp1=0;

	s32 retflag=PR_ERR_NONE;


#ifdef	PARAMETER_CHECK
	
	if(IMRHiPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(IMRLoPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
	
#endif

	ptr_mips_imr_hi = (mips_imr_hi_reg_t*) (Chip_Base_Address[chipId] + MIPS_IMR_HI_REG);
	ptr_mips_imr_lo = (mips_imr_lo_reg_t*) (Chip_Base_Address[chipId] + MIPS_IMR_LO_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;


	PR_Reg_Read(ptr_mips_imr_hi,temp1);
	*IMRHiPtr = temp1;

	PR_Reg_Read(ptr_mips_imr_lo,temp1);
	*IMRLoPtr = temp1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;
	

	return PR_ERR_NONE;

	return -1;    
}
#endif


/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_CPU_ISR
 * 
 * DESCRIPTION:	Get the interrupt status for MIPS High/Low Priority interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			   ChipId:    	ChipId.
 *		OUT:
 *			ISRHiPtr
 *			ISRLoPtr
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/
	
s32 ifx_get_CPU_ISR(u8 chipId,u32 *ISRHiPtr, u32 *ISRLoPtr)
{

	mips_isr_hi_reg_t*   ptr_mips_isr_hi;
	mips_isr_lo_reg_t*   ptr_mips_isr_lo;
	u32 temp1;

	s32 retflag=PR_ERR_NONE;

#ifdef	PARAMETER_CHECK
	
	if(ISRHiPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(ISRLoPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_mips_isr_hi = (mips_isr_hi_reg_t*) (Chip_Base_Address[chipId] + MIPS_ISR_HI_REG);
	ptr_mips_isr_lo = (mips_isr_lo_reg_t*) (Chip_Base_Address[chipId] + MIPS_ISR_LO_REG);	

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	
	PR_Reg_Read(ptr_mips_isr_hi,temp1);
	*ISRHiPtr = temp1;

	PR_Reg_Read(ptr_mips_isr_lo,temp1);
	*ISRLoPtr = temp1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_PCI_IMR
 * 
 * DESCRIPTION:	Set the interrupt mask for PCI High/Low Priority interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *          ChipId:    	ChipId.
 *			IMRHi:		PCI High Priority Interrupt Mask	
 *			IMRLo:		PCI low priority Interrupt Mask
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *      PR_ERR_NULL_PTR         Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_set_PCI_IMR(u8 chipId,u32 IMRHi, u32 IMRLo)
{
	pci_imr_hi_reg_t*   ptr_pci_imr_hi;
	pci_imr_lo_reg_t*   ptr_pci_imr_lo;
	
	s32 retflag=PR_ERR_NONE;


#ifdef	PARAMETER_CHECK

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_pci_imr_hi = (pci_imr_hi_reg_t*) (Chip_Base_Address[chipId] + PCI_IMR_HI_REG);
	ptr_pci_imr_lo = (pci_imr_lo_reg_t*) (Chip_Base_Address[chipId] + PCI_IMR_LO_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	PR_Reg_Write(ptr_pci_imr_hi,IMRHi);
	PR_Reg_Write(ptr_pci_imr_lo,IMRLo);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_PCI_IMR
 * 
 * DESCRIPTION:	Get the interrupt mask for PCI High/Low Priority interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId:    	ChipId.
 *		OUT:
 *			IMRHiPtr		Pointer to PCI High Priority Interrupt Mask
 *			IMRLoPtr		Pointer to PCI low priority Interrupt Mask
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer 
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_PCI_IMR(u8 chipId,u32 *IMRHiPtr, u32 *IMRLoPtr)
{

#if 0
	pci_imr_hi_reg_t*   ptr_pci_imr_hi;
	pci_imr_lo_reg_t*   ptr_pci_imr_lo;
	u32 temp1=0;

	s32 retflag=PR_ERR_NONE;

#ifdef	PARAMETER_CHECK
	
	if(IMRHiPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(IMRLoPtr == NULL)
		return PR_ERR_NULL_PTR;
	
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif
	ptr_pci_imr_hi = (pci_imr_hi_reg_t*) (Chip_Base_Address[chipId] + PCI_IMR_HI_REG);
	ptr_pci_imr_lo = (pci_imr_lo_reg_t*) (Chip_Base_Address[chipId] + PCI_IMR_LO_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	PR_Reg_Read(ptr_pci_imr_hi,temp1);
	*IMRHiPtr = temp1;

	PR_Reg_Read(ptr_pci_imr_lo,temp1);
	*IMRLoPtr = temp1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
#endif
    return -1;    
}

/******************************************************************************
 * 
 * FUNCTION:	ifx_get_PCI_ISR
 * 
 * DESCRIPTION:	Get the current interrupt status for PCI High/Low Priority 
 *				interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			 ChipId:    	ChipId.
 *		OUT:
 *			ISRHiPtr		Pointer to PCI High Priority Interrupt Status
 *			ISRLoPtr		Pointer to PCI low priority Interrupt Status
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id  
 *		 
 *****************************************************************************/
	
s32 ifx_get_PCI_ISR(u8 chipId,u32 *ISRHiPtr, u32 *ISRLoPtr)
{

	pci_isr_hi_reg_t*   ptr_pci_isr_hi;
	pci_isr_lo_reg_t*   ptr_pci_isr_lo;
	u32 temp1=0;

	s32 retflag=PR_ERR_NONE;

#ifdef	PARAMETER_CHECK
	
	if(ISRHiPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(ISRLoPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	ptr_pci_isr_hi = (pci_isr_hi_reg_t*) (Chip_Base_Address[chipId] + PCI_ISR_HI_REG);
	ptr_pci_isr_lo = (pci_isr_lo_reg_t*) (Chip_Base_Address[chipId] + PCI_ISR_LO_REG);

	PR_Reg_Read(ptr_pci_isr_hi,temp1);
	*ISRHiPtr = temp1;
	
	PR_Reg_Read(ptr_pci_isr_lo,temp1);
	*ISRLoPtr =  temp1;

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_chip_IMR
 * 
 * DESCRIPTION:	Set the switch interrupt mask for PR interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			IMR		    Packet Resolution Interrupt Mask
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR         Null Pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_set_chip_IMR(u8 chipId,u32 IMR)	
{
	pr_imr_reg_t*   ptr_pr_imr;

#ifdef	PARAMETER_CHECK

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_pr_imr = (pr_imr_reg_t*) (Chip_Base_Address[chipId] + PR_IMR_REG);

	PR_Reg_Write(ptr_pr_imr,IMR);

	return PR_ERR_NONE;

}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_chip_IMR
 * 
 * DESCRIPTION:	Get the interrupt mask for Packet resolution interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *		    ChipId    	ChipId.
 *		OUT:
 *			IMRPtr		Pointer to Packet Resolution Interrupt Mask		
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_chip_IMR(u8 chipId,u32 *IMRPtr)
{

	pr_imr_reg_t*   ptr_pr_imr;
	u32 temp1=0;

#ifdef	PARAMETER_CHECK
	
	if(IMRPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_pr_imr = (pr_imr_reg_t*) (Chip_Base_Address[chipId] + PR_IMR_REG);

	PR_Reg_Read(ptr_pr_imr,temp1);
	*IMRPtr = temp1;

	return PR_ERR_NONE;
}

/******************************************************************************
 * 
 * FUNCTION:	ifx_get_chip_ISR
 * 
 * DESCRIPTION:	Get the current interrupt status for Packet Resolution interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *		OUT:
 *			ISRPtr		Pointer to Packet Resolution Interrupt Status
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer 
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_chip_ISR( u8 chipId, u32 *ISRPtr )

{

	pr_isr_reg_t* ptr_pr_isr;
	u32 temp1=0;

#ifdef	PARAMETER_CHECK
	
	if(ISRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif	

	ptr_pr_isr = (pr_isr_reg_t*)  PR_ISR_REG;
	
	PR_Reg_Read(ptr_pr_isr,temp1);
	*ISRPtr = temp1;	
	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_switch_engine_IMR
 * 
 * DESCRIPTION:	Set the switch engine interrupt mask.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			IMR		Switch Engine Interrupt Mask
 *		OUT:
 *			None
 *
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *      PR_ERR_NULL_PTR         Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_set_switch_engine_IMR(u8 chipId,u32 IMR)
{
	
	se_imr_reg_t*   ptr_se_imr;

#ifdef	PARAMETER_CHECK

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif
	ptr_se_imr = (se_imr_reg_t*) SE_INT_MSK_REG_WR; /* Bug in the read write logic for this register */

	PR_Reg_Write(ptr_se_imr,IMR);

	return PR_ERR_NONE;

}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_switch_engine_IMR
 * 
 * DESCRIPTION:	Get the interrupt mask for switch engine interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *		OUT:
 *			IMRPtr		Pointer to Switch Engine Interrupt Mask
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer 
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_switch_engine_IMR(u8 chipId,u32 *IMRPtr)
{

	
	se_imr_reg_t* ptr_se_imr;
	u32 temp1=0;

#ifdef	PARAMETER_CHECK
	
	if(IMRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif

	ptr_se_imr = (se_imr_reg_t*)SE_INT_MSK_REG_RD;

	PR_Reg_Read(ptr_se_imr,temp1);
	*IMRPtr = temp1;

	return PR_ERR_NONE;


}

/******************************************************************************
 * 
 * FUNCTION:	ifx_get_switch_engine_ISR
 * 
 * DESCRIPTION:	Get the current interrupt status for switch engine interrupt group.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *		OUT:
 *			ISRPtr		Pointer to Switch Engine Interrupt Status
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id 
 *		 
 *****************************************************************************/


s32 ifx_get_switch_engine_ISR(u8 chipId,u32* ISRPtr)
{


	se_isr_reg_t*   ptr_se_isr;
	u32 temp1=0;

#ifdef	PARAMETER_CHECK
	
	if(ISRPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_se_isr = (se_isr_reg_t*) SE_INT_STS_REG;

	PR_Reg_Read(ptr_se_isr,temp1);
	*ISRPtr = temp1;

	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_Rx_IMR
 * 
 * DESCRIPTION:	Set the switch interrupt mask for RX interrupt for a port (0-49). 
 *								
 * PARAMETERS:
 *		IN:
 *
 *			ChipId    	ChipId.
 *			IMR			Rx Interrupt Mask
 *			PortId		Port Id (range is 0 - 49)
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *      PR_ERR_NULL_PTR         Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_set_Rx_IMR(u8 chipId,u32 portId, u32 IMR)
{
	u32* ptr ;
	u32 mask1=0;
	u32 mask2=0;
	u32 temp=0;
	u32 fe_port=0;

	s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;
	

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx Imr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_IMR1_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, temp);
		 fe_port = portId & 7 ;
	     
	
	     mask1 =  ~((0x01010101)   << fe_port);
		 mask2 =  ((IMR >> 7) & 1) << RX_CRC_ERR_POS;
		 mask2 |= ((IMR >> 6) & 1) << RX_NO_RXD_CELL_POS;
		 mask2 |= ((IMR >> 5) & 1) << RX_NO_PKT_DISC_POS;
		 mask2 |= ((IMR >> 4) & 1) ;
		
		 temp &= mask1;
		 temp |= (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

/* rx imr register 2 */

		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_IMR0_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, temp);
		     
	
	   	 mask2 =  ((IMR >> 3) & 1) << RX_CRC_ERR_POS;
		 mask2 |= ((IMR >> 2) & 1) << RX_NO_RXD_CELL_POS;
		 mask2 |= ((IMR >> 1) & 1) << RX_NO_PKT_DISC_POS;
		 mask2 |= IMR  & 1;
		
		 temp &= mask1;
		 temp |= (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_IMR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, temp);
		 temp &= ~(0xff);
		 temp |= IMR & 0xff;
		 PR_Reg_Write(ptr, temp);
	
	}

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_Rx_IMR
 * 
 * DESCRIPTION:	Get the switch interrupt mask for RX interrupt for a port (0-49).
 *								
 *				
 * PARAMETERS:
 *		IN:		
 *			ChipId    	ChipId.
 *			PortId		Port Id (range is 0 - 49)			
 *		OUT:
 *			IMRPtr		Pointer to Rx Interrupt Mask			
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_Rx_IMR(u8 chipId,u32 portId, u32 * IMRPtr)
{
	u32* ptr ;
	u32 IMR=0;
	u32  temp=0;
	u32 fe_port=0;

	s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
	if(IMRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif
	
/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx Imr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_IMR1_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, IMR);
		 fe_port = portId & 7 ;
	
		 temp =  ((IMR >>( RX_CRC_ERR_POS + fe_port)) & 1) <<  7;
		 temp |= ((IMR >> (RX_NO_RXD_CELL_POS + fe_port)) & 1) <<  6;
		 temp |= ((IMR >> (RX_NO_PKT_DISC_POS + fe_port)) & 1)  <<  5;
		 temp |= ((IMR >> (fe_port)) & 1)  <<  4;
		
		 
/* rx imr register 2 */

		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_IMR0_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, IMR);
		     

		 temp |=  ((IMR >> (RX_CRC_ERR_POS + fe_port)) & 1) << 3;
		 temp |= ((IMR >> (RX_NO_RXD_CELL_POS + fe_port)) & 1) << 2;
		 temp |= ((IMR >> (RX_NO_PKT_DISC_POS + fe_port)) & 1) << 1;
		 temp |= (IMR >> fe_port) & 1 ;
	
		 *IMRPtr = temp;
	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_IMR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, IMR);
		 temp = IMR & 0xff;
		 *IMRPtr = temp;
	
	}
/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
}


/******************************************************************************
 * 
 * FUNCTION:	ifx_get_Rx_ISR
 * 
 * DESCRIPTION:	Get the current interrupt status for RX interrupt. 
 *								
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			PortId		Port Id (range is 0 - 49) 
 *		OUT:
 *			ISRPtr		Pointer to Rx Interrupt Status
 *			
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_Rx_ISR(u8 chipId,u32 portId, u32 *ISRPtr)
{
	u32* ptr ;
	u32 ISR=0;
	u32  temp=0;
	u32 fe_port=0;

	s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;

	if(ISRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif
	
/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx Isr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_ISR1_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, ISR);
		 fe_port = portId & 7 ;
	     

		 temp =  ((ISR >> (RX_CRC_ERR_POS + fe_port)) & 1) <<  7;
		 temp |= ((ISR >> (RX_NO_RXD_CELL_POS + fe_port)) & 1) <<  6;
		 temp |= ((ISR >> (RX_NO_PKT_DISC_POS + fe_port)) & 1)  <<  5;
		 temp |= ((ISR >> fe_port) & 1)  <<  4;
		
		 
/* rx imr register 2 */

		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_ISR0_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, ISR);
		     

		 temp |=  ((ISR >> (RX_CRC_ERR_POS + fe_port)) & 1) << 3;
		 temp |= ((ISR >> (RX_NO_RXD_CELL_POS + fe_port)) & 1) << 2;
		 temp |= ((ISR >> (RX_NO_PKT_DISC_POS + fe_port)) & 1) << 1;
		 temp |= (ISR >> fe_port) & 1 ;
	
		 *ISRPtr = temp;
	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, ISR);
		 temp = ISR & 0xff;
		 *ISRPtr = temp;
	
	}

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE;

}

/******************************************************************************
 * 
 * FUNCTION:	ifx_clear_Rx_ISR
 * 
 * DESCRIPTION:	Clear the current interrupt status for RX interrupt. 
 *					
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			ISR 		Rx Interrupt Status
 *			portId		Port Id (range is 0 - 49)
 *		OUT:
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_CHIP_ID          Invalid  Chip Id
 *		PR_ERR_NULL_PTR         Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_clear_Rx_ISR(u8 chipId,u32 portId, u32 ISR)

{
	u32* ptr ;

	u32 mask2=0;
	u32  temp=0;
	u32 fe_port=0;

	s32 retflag=PR_ERR_NONE;


#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif
	
/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx ISR register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_ISR1_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
/* no need to read,if you want to clear all interrupt,just read and write back
		 PR_Reg_Read(ptr, temp);
*/
		 fe_port = portId & 7 ;
	     
		 mask2 =  ((ISR >> 7) & 1) << RX_CRC_ERR_POS;
		 mask2 |= ((ISR >> 6) & 1) << RX_NO_RXD_CELL_POS;
		 mask2 |= ((ISR >> 5) & 1) << RX_NO_PKT_DISC_POS;
		 mask2 |= ((ISR >> 4) & 1) ;

		 temp =  (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

/* rx ISR register 2 */

		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RX_ISR0_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );

	     

		 mask2 =  ((ISR >> 3) & 1) << RX_CRC_ERR_POS;
		 mask2 |= ((ISR >> 2) & 1) << RX_NO_RXD_CELL_POS;
		 mask2 |= ((ISR >> 1) & 1) << RX_NO_PKT_DISC_POS;
		 mask2 |= ISR  & 1;

		 temp = (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );

		 temp = ISR & 0xff;
		 PR_Reg_Write(ptr, temp);
	
	}

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_set_Tx_IMR
 * 
 * DESCRIPTION:	Set the TX interrupt mask.				
 *				
 * PARAMETERS:
 *		IN:
 *
 *			ChipId    	ChipId.
 *			IMR			Tx Interrupt Mask		
 *			portId		Port Id (range is 0 - 49)
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *      PR_ERR_NULL_PTR			Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_set_Tx_IMR(u8 chipId,u32 portId, u32 IMR)
{
	u32* ptr ;
	u32 mask1=0;
	u32 mask2=0;
	u32  temp=0;
	u32 fe_port=0;

	s32 retflag=PR_ERR_NONE;


#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif
	
/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx Imr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_IMR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, temp);
		 fe_port = portId & 7 ;
	     
	
	     mask1 =  ~((0x01010101)   << fe_port);
		 mask2 =  ((IMR >> 3) & 1) << TX_MAX_ATTEMPTS_POS;
		 mask2 |= ((IMR >> 2) & 1) << TX_LATECOL_POS;
		 mask2 |= ((IMR >> 1) & 1) << TX_EARLYCOL_POS;
		 mask2 |= IMR  & 1;
		
		 temp &= mask1;
		 temp |= (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_IMR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, temp);
		 temp &= ~(0xf00);
		 temp |= (IMR & 0xf) << 8;
		 PR_Reg_Write(ptr, temp);
	
	}

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;

}

/*****************************************************************************
 * 
 * FUNCTION:	ifx_get_Tx_IMR
 * 
 * DESCRIPTION:	Get the TX interrupt mask.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			portId		Port Id (range is 0 - 49)			 
 *		OUT:
 *			IMRPtr		Pointer to Tx Interrupt Mask		
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_Tx_IMR(u8 chipId,u32 portId, u32 *IMRPtr) 
{
	u32* ptr ;
	u32 IMR=0;
	u32  temp=0;
	u32 fe_port=0;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;

	if(IMRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif


	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* tx Imr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_IMR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, IMR);
		 fe_port = portId & 7 ;

		 temp =  ((IMR >> (TX_MAX_ATTEMPTS_POS + fe_port)) & 1) <<  3;
		 temp |= ((IMR >> (TX_LATECOL_POS + fe_port)) & 1) <<  2;
		 temp |= ((IMR >> (TX_EARLYCOL_POS + fe_port)) & 1)  <<  1;
		 temp |= ((IMR >> fe_port) & 1)  <<  0;
		
		 *IMRPtr = temp;

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_IMR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, IMR);
		 temp = (IMR & 0xf00) >> 8;
		 *IMRPtr = temp;
	
	}


	return PR_ERR_NONE;
}




/******************************************************************************
 * 
 * FUNCTION:	ifx_get_Tx_ISR
 * 
 * DESCRIPTION:	Get the TX interrupt status.				
 *				
 * PARAMETERS:
 *		IN:
 *			portId			Port Id (range is 0 - 49) 
 *          ChipId    	ChipId.
 *      OUT:
 *			ISRPtr			Pointer to Tx Interrupt Status
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_NULL_PTR			Null pointer
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *		 
 *****************************************************************************/

s32 ifx_get_Tx_ISR(u8 chipId, u32 portId,  u32 *ISRPtr)
{
	u32* ptr ;
	u32 ISR=0;
	u32  temp=0;
	u32 fe_port=0;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;

	if(ISRPtr == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif
	
	 

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* tx ISR register */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_ISR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, ISR);
		 fe_port = portId & 7 ;
	     
	
		 temp =  ((ISR >> (TX_MAX_ATTEMPTS_POS + fe_port)) & 1) <<  3;
		 temp |= ((ISR >> (TX_LATECOL_POS + fe_port)) & 1) <<  2;
		 temp |= ((ISR >> (TX_EARLYCOL_POS + fe_port)) & 1)  <<  1;
		 temp |= ((ISR >> fe_port) & 1)  <<  0;
		
		 *ISRPtr = temp;

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, ISR);
		 temp = (ISR & 0xf00) >> 8;
		 *ISRPtr = temp;
	
	}

	return PR_ERR_NONE ;

}




/******************************************************************************
 * 
 * FUNCTION:	ifx_clear_Tx_ISR
 * 
 * DESCRIPTION:	Clear the TX interrupt status.				
 *				
 * PARAMETERS:
 *		IN:
 *			ChipId    	ChipId.
 *			ISRPtr		Tx Interrupt Status
 *			portId		Port Id (0 - 49)
 *		OUT:
 *			None
 * 
 * RETURNS:
 *		PR_ERR_NONE				Success
 *		PR_ERR_PORTID	        If the portId is not in the range of 0-49
 *      PR_ERR_CHIP_ID          Invlaid Chip Id
 *		PR_ERR_NULL_PTR         Null Pointer
 *		 
 *****************************************************************************/

s32 ifx_clear_Tx_ISR(u8 chipId,u32 portId, u32 ISR)
{

	u32* ptr ;
	u32 mask2=0;
	u32  temp=0;
	u32 fe_port=0;

#ifdef PARAMETER_CHECK

	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif
	

	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx ISR register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_ISR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );

		 fe_port = portId & 7 ;
	     

		 mask2 =  ((ISR >> 3) & 1) << TX_MAX_ATTEMPTS_POS;
		 mask2 |= ((ISR >> 2) & 1) << TX_LATECOL_POS;
		 mask2 |= ((ISR >> 1) & 1) << TX_EARLYCOL_POS;
		 mask2 |= ISR  & 1;


		 temp = (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );

		 temp = (ISR & 0xf) << 8;
		 PR_Reg_Write(ptr, temp);
	
	}


	return PR_ERR_NONE ;


}



s32 ifx_get_RxTx_IMR( u32 portId, u32 *IMRPtr) 
{
	u32* ptr ;
	u32 IMR=0;
	u32  temp=0;
	u32 fe_port=0;

#ifdef PARAMETER_CHECK
/*
	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
*/
	if(IMRPtr == NULL)
		return PR_ERR_NULL_PTR;
#endif

    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            ptr = (u32*) SWITCH_P0_RTX_INT_MASK;
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            ptr = (u32*) SWITCH_P1_RTX_INT_MASK;
            break;            
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            ptr = (u32*) SWITCH_P2_RTX_INT_MASK;
            break;            
        default:
            return PR_ERR_PORTID;
    }
    
    PR_Reg_Read(ptr, IMR);
    *IMRPtr = IMR;
    
	return PR_ERR_NONE;    
    
#if 0
	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* tx Imr register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_IMR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, IMR);
		 fe_port = portId & 7 ;

		 temp =  ((IMR >> (TX_MAX_ATTEMPTS_POS + fe_port)) & 1) <<  3;
		 temp |= ((IMR >> (TX_LATECOL_POS + fe_port)) & 1) <<  2;
		 temp |= ((IMR >> (TX_EARLYCOL_POS + fe_port)) & 1)  <<  1;
		 temp |= ((IMR >> fe_port) & 1)  <<  0;
		
		 *IMRPtr = temp;

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_IMR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, IMR);
		 temp = (IMR & 0xf00) >> 8;
		 *IMRPtr = temp;
	
	}


	return PR_ERR_NONE;
#endif    
}






s32 ifx_get_RxTx_ISR( u32 portId,  u32 *ISRPtr)
{
	u32* ptr ;
	u32 ISR=0;
	u32  temp=0;
	u32 fe_port=0;

#ifdef PARAMETER_CHECK
/*
	if(portId >= (TOTAL_PORTS - 1))
		return PR_ERR_PORTID;
*/
	if(ISRPtr == NULL)
		return PR_ERR_NULL_PTR;
#endif

    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            ptr = (u32*) SWITCH_P0_RTX_INT_STATUS;
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            ptr = (u32*) SWITCH_P1_RTX_INT_STATUS;
            break;            
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            ptr = (u32*) SWITCH_P2_RTX_INT_STATUS;
            break;            
        default:
            return PR_ERR_PORTID;
    }

    PR_Reg_Read(ptr, ISR);
    *ISRPtr = ISR; 

	return PR_ERR_NONE ;
	 
#if 0
	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* tx ISR register */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_ISR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );
		 PR_Reg_Read(ptr, ISR);
		 fe_port = portId & 7 ;
	     
	
		 temp =  ((ISR >> (TX_MAX_ATTEMPTS_POS + fe_port)) & 1) <<  3;
		 temp |= ((ISR >> (TX_LATECOL_POS + fe_port)) & 1) <<  2;
		 temp |= ((ISR >> (TX_EARLYCOL_POS + fe_port)) & 1)  <<  1;
		 temp |= ((ISR >> fe_port) & 1)  <<  0;
		
		 *ISRPtr = temp;

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );
		 PR_Reg_Read(ptr, ISR);
		 temp = (ISR & 0xf00) >> 8;
		 *ISRPtr = temp;
	
	}

	return PR_ERR_NONE ;
#endif
}


s32 ifx_clear_RxTx_ISR( u32 portId, u32 ISR )
{

	u32* ptr ;
	u32 mask2=0;
	u32  temp=0;
	u32 fe_port=0;

    if( ISR == 0x00000000 )
    {
        printk("ifx_clear_RxTx_ISR: nothing to clear\n");
    }
	
    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            ptr = (u32*) SWITCH_P0_RTX_INT_STATUS;
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            ptr = (u32*) SWITCH_P1_RTX_INT_STATUS;
            break;            
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            ptr = (u32*) SWITCH_P2_RTX_INT_STATUS;
            break;            
        default:
            return PR_ERR_PORTID;
    }
    
    temp = ISR & RX_TX_INTERRUPT_MASK;
    PR_Reg_Write(ptr, temp);

	return PR_ERR_NONE ;
    
#if 0    
	 if(portId < IFX_TOTAL_FE_PORTS)
	 {

/* rx ISR register1 */
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + TX_ISR_REG + 
					TOTAL_PORT_REGS * (portId >> 3) );

		 fe_port = portId & 7 ;
	     

		 mask2 =  ((ISR >> 3) & 1) << TX_MAX_ATTEMPTS_POS;
		 mask2 |= ((ISR >> 2) & 1) << TX_LATECOL_POS;
		 mask2 |= ((ISR >> 1) & 1) << TX_EARLYCOL_POS;
		 mask2 |= ISR  & 1;


		 temp = (mask2 << fe_port);
		 PR_Reg_Write(ptr, temp);

	}
	else
	{
		 ptr = (u32*) (Chip_Base_Address[chipId] + PORTREG_BASE  + RTX_ISR_REG + 
					TOTAL_PORT_REGS * (portId - IFX_TOTAL_FE_PORTS + 6) );

		 temp = (ISR & 0xf) << 8;
		 PR_Reg_Write(ptr, temp);
	
	}


	return PR_ERR_NONE ;
#endif

}


/*
 * Processes Packet Resolution Interrupts of the Ethernet Switch
 *
 */
void ifx_switch_PR_ISR(s32 irq, void *dev_id, struct pt_regs *regs)
{
    u8 chipId = 0;
    pr_isr_t pr_isr;
        
    if( ifx_get_chip_ISR( chipId, &pr_isr.val ) == PR_ERR_NONE )
    {
        if(pr_isr.val)
        {
            if( pr_isr.fld.dmf)
            {
                printk("ifx_switch_PR_ISR: Drop all multicast off interrupt\n");
            }
            if( pr_isr.fld.dmn)
            {
                printk("ifx_switch_PR_ISR: Drop all multicast on interrupt\n");
            }
            if( pr_isr.fld.stf)
            {
                printk("ifx_switch_PR_ISR: Stall off interrupt\n");
            }
            if( pr_isr.fld.stn)
            {
                printk("ifx_switch_PR_ISR: Stall on interrupt\n");
            }
            if( pr_isr.fld.mit)
            {
                printk("ifx_switch_PR_ISR: MAC table intrusion interrupt\n");
            }
            if( pr_isr.fld.mch)
            {
                printk("ifx_switch_PR_ISR: MAC table change interrupt\n");
            }
            if( pr_isr.fld.mfl)
            {
                printk("ifx_switch_PR_ISR: MAC table change full interrupt\n");
            }            
        }
        else
        {
            printk("ifx_switch_PR_ISR: no interrupt bit set !!!");
        }

    }
    return;

}

/*
 * Processes Switch Engine Interrupts of the Ethernet Switch
 *
 */
void ifx_switch_engine_ISR(s32 irq, void *dev_id, struct pt_regs *regs)
{
    u8 chipId = 0;
    se_isr_t se_isr;
        
    if( ifx_get_switch_engine_ISR( chipId, &se_isr.val ) == PR_ERR_NONE )
    {
        if(se_isr.val)
        {
            if( se_isr.fld.rf)
            {
                printk("ifx_switch_engine_ISR: Receive Full Interrupt\n");
            }
            if( se_isr.fld.rnf)
            {
                printk("ifx_switch_engine_ISR: Receive Near Full Interrupt\n");
            }
            if( se_isr.fld.qf_lan)
            {
                printk("ifx_switch_engine_ISR: LAN Port Transmit Queue Full Interrupt\n");
            }
            if( se_isr.fld.qf_pc)
            {
                printk("ifx_switch_engine_ISR: PC Port Transmit Queue Full Interrupt\n");
            }
            if( se_isr.fld.qf_cpu)
            {
                printk("ifx_switch_engine_ISR: CPU Port Transmit Queue Full Interrupt\n");
            }            
            if( se_isr.fld.qnf_lan)
            {
                printk("ifx_switch_engine_ISR: LAN Port Transmit Queue Near Full Interrupt\n");
            }
            if( se_isr.fld.qnf_pc)
            {
                printk("ifx_switch_engine_ISR: PC Port Transmit Queue Near Full Interrupt\n");
            }
            if( se_isr.fld.qnf_cpu)
            {
                printk("ifx_switch_engine_ISR: CPU Port Transmit Queue Near Full Interrupt\n");
            }            
            if( se_isr.fld.pfqe)
            {
                printk("ifx_switch_engine_ISR: Packet Buffer Free Queue Empty Interrupt\n");
            }
 
        }
        else
        {
            printk("ifx_switch_engine_ISR: no interrupt bit set !!!");
        }

    }
    return;
}



s32 ifx_process_switch_port_interrupt( u32 portId )
{
    rtx_isr_t rtx_isr;
    u32 portId=0;  /* always select LAN Port */
    s32 retflag=PR_ERR_NONE;

    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            printk("LAN Port Interrupt occurred...\n");
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            printk("PC Port Interrupt occurred...\n");
            break;            
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            printk("CPU Port Interrupt occurred...\n");
            break;            
        default:
            return PR_ERR_PORTID;
    }

        
    if( ifx_get_RxTx_ISR( portId,  &rtx_isr.val) == PR_ERR_NONE )
    {
        if(rtx_isr.val)
        {
            if( rtx_isr.fld.txma)
            {
                printk("Tx Maximum Attempts Interrupt\n");
            }
            if( rtx_isr.fld.txlc)
            {
                printk("Tx Late Collision Interrupt\n");
            }
            if( rtx_isr.fld.txec)
            {
                printk("Tx Early Collision Interrupt\n");
            }
            if( rtx_isr.fld.txuf)
            {
                printk("Transmit Underflow Interrupt\n");
            }
            if( rtx_isr.fld.rr)
            {
                printk("Rkey received Interrupt\n");
            }            
            if( rtx_isr.fld.noeof)
            {
                printk("No EOF detected Interrupt\n");
            }
            if( rtx_isr.fld.nosof)
            {
                printk("No SOF detected Interrupt\n");
            }
            if( rtx_isr.fld.rxfo)
            {
                printk("Receive FIFO overflow Interrupt\n");
            }            
            if( rtx_isr.fld.rxce)
            {
                printk("Receive CRC Error Interrupt\n");
            }
            if( rtx_isr.fld.rxnrc)
            {
                printk("No RxD Cell in Receive Interrupt\n");
            }
            if( rtx_isr.fld.rxnpd)
            {
                printk("No Packet Descriptor available Interruptn");
            }
            if( rtx_isr.fld.rxnpn)
            {
                printk("No PBNUM available Interrupt\n");
            } 
            /* Finally clear all interrupts */
            ifx_clear_RxTx_ISR( portId, rtx_isr.val );            
        }
        else
        {
            printk("no interrupt bit set !!!");
        }
   
    }
    return PR_ERR_NONE ;;    
}


/*
 * Processes LAN Port Interrupts of the Ethernet Switch
 *
 */
void ifx_switch_lan_port_ISR(s32 irq, void *dev_id, struct pt_regs *regs)
{
    ifx_process_switch_port_interrupt( INCAIP2_SWITCH_PORT_SELECT_LAN );
    return;    
}

/*
 * Processes PC Port Interrupts of the Ethernet Switch
 *
 */
void ifx_switch_pc_port_ISR(s32 irq, void *dev_id, struct pt_regs *regs)
{
    ifx_process_switch_port_interrupt( INCAIP2_SWITCH_PORT_SELECT_PC );
    return;    
}

/*
 * Processes CPU Port Interrupts of the Ethernet Switch
 *
 */
void ifx_switch_cpu_port_ISR(s32 irq, void *dev_id, struct pt_regs *regs)
{
    ifx_process_switch_port_interrupt( INCAIP2_SWITCH_PORT_SELECT_CPU );
    return;    
}


