/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/version.h>
#include <linux/smp_lock.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>

#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/switch.h>
#include "switch_api.h"
#include "ioctl.h"
#include "ioctl_struct.h"

#include <interrupt.h>
#include <switch.h>
#include <api_init.h>
#include <pbll.h>
#include <switch_prv.h>


/**
 * Switch IOCTL handler.
 * IOCTL handler for switch configuration.
 * The following IOCTLs are supported:
 * - #IFX_SET_INGRESS_WATERMARK
 * - #IFX_GET_INGRESS_WATERMARK
 * - #IFX_SET_EGRESS_WATERMARK
 * - #IFX_GET_EGRESS_WATERMARK
 * - #IFX_SET_TX_CONFIG
 * - #IFX_GET_TX_CONFIG
 * - #IFX_SET_PORT_RATE_SHAPE
 * - #IFX_GET_PORT_RATE_SHAPE
 *
 * \param   file   - File structure of device
 * \param   cmd    - IOCTL command
 * \param   arg    - Argument for some IOCTL commands
 * \return  0      - OK
 * \return  ENOTTY - Invalid command
 * \ingroup API
 */
int switch_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
    int err=0;
    /*structures for arg*/
/*
    u8 NumOfChips;
    baseAddr_api ba_api;
    intReg_api ir_api;
    intRegW_api irw_api;
    portReg_api pra_api;
    packetBuf_api pb_api;
    packetLinkList_api pll_api;
    chipversion_api cv_api;
    minorversion_api minver_api;
*/
    feature_mode_api fm_api;
    watermark_api wm_api;
    Ewatermark_api ewm_api;
    TxConfig_api tc_api;
    rateShape_api rs_api;
    PauseFrame_api pfg_api;

/*    printk("%s\n", __FUNCTION__);  */

    switch(cmd)
    {
#if 0
        case IFX_GET_CHIP_NUMBER:
            DPRINTK("%s: IFX_GET_CHIP_NUMBER\n",__FUNCTION__);
            copy_from_user(&NumOfChips,(u8 *) arg,sizeof (u8));
            err = ifx_get_chip_number(&NumOfChips);
            if(err == 0)
                      copy_to_user((u8 *) arg, &NumOfChips, sizeof(u8));
               break;
        case IFX_SET_BASE_ADDRESS:
            DPRINTK("%s: IFX_SET_BASE_ADDRESS\n",__FUNCTION__);
            printk("%s - IFX_SET_BASE_ADDRESS\n", __FUNCTION__);
            copy_from_user(&ba_api,(baseAddr_api *) arg,sizeof (baseAddr_api));
            err = ifx_set_base_address(ba_api.chipId,ba_api.Base_Address);
               break;
        case IFX_GET_BASE_ADDRESS:
            DPRINTK("%s: IFX_GET_BASE_ADDRESS\n",__FUNCTION__);
            copy_from_user(&ba_api,(baseAddr_api *) arg,sizeof (baseAddr_api));
            err = ifx_get_base_address(ba_api.chipId,&ba_api.Base_Address);
            if(err == 0)
                      copy_to_user((baseAddr_api *) arg, &ba_api, sizeof(baseAddr_api));
               break;
        case IFX_SET_CHIP_NUMBER:
            DPRINTK("%s: IFX_SET_CHIP_NUMBER\n",__FUNCTION__);
            printk("%s - IFX_SET_CHIP_NUMBER\n", __FUNCTION__);
            copy_from_user(&NumOfChips,(u8 *) arg,sizeof (u8));
            err = ifx_set_chip_number(NumOfChips);
               break;
#endif
/*
        case IFX_SET_CPU_IMR:
            DPRINTK("%s: IFX_SET_CPU_IMR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_set_CPU_IMR(ir_api.chipId,ir_api.regHi, ir_api.regLo);
               break;
        case IFX_GET_CPU_IMR:
            DPRINTK("%s: IFX_GET_CPU_IMR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_get_CPU_IMR(ir_api.chipId,&ir_api.regHi, &ir_api.regLo);
            if(err == 0)
                      copy_to_user((intReg_api *) arg, &ir_api, sizeof(intReg_api));
               break;
        case IFX_GET_CPU_ISR:
            DPRINTK("%s: IFX_GET_CPU_ISR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_get_CPU_ISR(ir_api.chipId,&ir_api.regHi, &ir_api.regLo);
            if(err == 0)
                      copy_to_user((intReg_api *) arg, &ir_api, sizeof(intReg_api));
               break;
        case IFX_SET_PCI_IMR:
            DPRINTK("%s: IFX_SET_PCI_IMR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_set_PCI_IMR(ir_api.chipId,ir_api.regHi, ir_api.regLo);
               break;
        case IFX_GET_PCI_IMR:
            DPRINTK("%s: IFX_GET_PCI_IMR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_get_PCI_IMR(ir_api.chipId,&ir_api.regHi, &ir_api.regLo);
            if(err == 0)
                      copy_to_user((intReg_api *) arg, &ir_api, sizeof(intReg_api));
               break;
        case IFX_GET_PCI_ISR:
            DPRINTK("%s: IFX_GET_PCI_ISR\n",__FUNCTION__);
            copy_from_user(&ir_api,(intReg_api *) arg,sizeof (intReg_api));
            err = ifx_get_PCI_ISR(ir_api.chipId,&ir_api.regHi, &ir_api.regLo);
            if(err == 0)
                      copy_to_user((intReg_api *) arg, &ir_api, sizeof(intReg_api));
               break;
        case IFX_SET_CHIP_IMR:
            DPRINTK("%s: IFX_SET_CHIP_IMR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_set_chip_IMR(irw_api.chipId,irw_api.reg);
               break;
        case IFX_GET_CHIP_IMR:
            DPRINTK("%s: IFX_GET_CHIP_IMR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_get_chip_IMR(irw_api.chipId,&irw_api.reg);
            if(err == 0)
                      copy_to_user((intRegW_api *) arg, &irw_api, sizeof(intRegW_api));
               break;
        case IFX_GET_CHIP_ISR:
            DPRINTK("%s: IFX_GET_CHIP_ISR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_get_chip_ISR(irw_api.chipId,&irw_api.reg);
            if(err == 0)
                      copy_to_user((intRegW_api *) arg, &irw_api, sizeof(intRegW_api));
               break;
        case IFX_SET_SWITCH_ENGINE_IMR:
            DPRINTK("%s: IFX_SET_SWITCH_ENGINE_IMR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_set_switch_engine_IMR(irw_api.chipId,irw_api.reg);
               break;
        case IFX_GET_SWITCH_ENGINE_IMR:
            DPRINTK("%s: IFX_GET_SWITCH_ENGINE_IMR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_get_switch_engine_IMR(irw_api.chipId,&irw_api.reg);
            if(err == 0)
                      copy_to_user((intRegW_api *) arg, &irw_api, sizeof(intRegW_api));
               break;
        case IFX_GET_SWITCH_ENGINE_ISR:
            DPRINTK("%s: IFX_GET_SWITCH_ENGINE_IMR\n",__FUNCTION__);
            copy_from_user(&irw_api,(intRegW_api *) arg,sizeof (intRegW_api));
            err = ifx_get_switch_engine_ISR(irw_api.chipId,&irw_api.reg);
            if(err == 0)
                      copy_to_user((intRegW_api *) arg, &irw_api, sizeof(intRegW_api));
               break;
        case IFX_SET_RX_IMR:
            DPRINTK("%s: IFX_SET_RX_IMR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_set_Rx_IMR(pra_api.chipId,pra_api.portId,   pra_api.reg);
               break;
        case IFX_GET_RX_IMR:
            DPRINTK("%s: IFX_GET_RX_IMR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_get_Rx_IMR(pra_api.chipId,pra_api.portId,  &pra_api.reg);
            if(err == 0)
                      copy_to_user((portReg_api *) arg, &pra_api, sizeof(portReg_api));
               break;
        case IFX_GET_RX_ISR:
            DPRINTK("%s: IFX_GET_RX_ISR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_get_Rx_ISR(pra_api.chipId,pra_api.portId,  &pra_api.reg);
            if(err == 0)
                      copy_to_user((portReg_api *) arg, &pra_api, sizeof(portReg_api));
               break;
        case IFX_SET_CLEAR_RX_ISR:
            DPRINTK("%s: IFX_SET_CLEAR_RX_ISR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_clear_Rx_ISR(pra_api.chipId,pra_api.portId,   pra_api.reg);
               break;
        case IFX_SET_TX_IMR:
            DPRINTK("%s: IFX_SET_TX_IMR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_set_Tx_IMR(pra_api.chipId,pra_api.portId,   pra_api.reg);
               break;
        case IFX_GET_TX_IMR:
            DPRINTK("%s: IFX_GET_TX_IMR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_get_Tx_IMR(pra_api.chipId,pra_api.portId,  &pra_api.reg);
            if(err == 0)
                      copy_to_user((portReg_api *) arg, &pra_api, sizeof(portReg_api));
               break;
        case IFX_GET_TX_ISR:
            DPRINTK("%s: IFX_GET_TX_ISR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_get_Tx_ISR(pra_api.chipId,pra_api.portId,  &pra_api.reg);
            if(err == 0)
                      copy_to_user((portReg_api *) arg, &pra_api, sizeof(portReg_api));
               break;
        case IFX_SET_CLEAR_TX_ISR:
            DPRINTK("%s: IFX_SET_CLEAR_TX_ISR\n",__FUNCTION__);
            copy_from_user(&pra_api,(portReg_api *) arg,sizeof (portReg_api));
            err = ifx_clear_Tx_ISR(pra_api.chipId,pra_api.portId,   pra_api.reg);
               break;
        case IFX_SET_PACKET_BUF:
            DPRINTK("%s: IFX_SET_PACKET_BUF\n",__FUNCTION__);
            copy_from_user(&pb_api,(packetBuf_api *) arg,sizeof (packetBuf_api));
            err = ifx_set_packet_buf(pb_api.chipId,pb_api.readPtr, pb_api.writePtr, pb_api.count);
               break;
        case IFX_GET_PACKET_BUF:
            DPRINTK("%s: IFX_GET_PACKET_BUF\n",__FUNCTION__);
            copy_from_user(&pb_api,(packetBuf_api *) arg,sizeof (packetBuf_api));
            err = ifx_get_packet_buf(pb_api.chipId,&pb_api.readPtr, &pb_api.writePtr, &pb_api.count);
            if(err == 0)
                      copy_to_user((packetBuf_api *) arg, &pb_api, sizeof(packetBuf_api));
               break;
        case IFX_SET_PACKET_BUF_LINK_LIST:
            DPRINTK("%s: IFX_SET_PACKET_BUF_LINK_LIST\n",__FUNCTION__);
            copy_from_user(&pll_api,(packetLinkList_api *) arg,sizeof (packetLinkList_api));
            err = ifx_set_packet_buf_link_list(pll_api.chipId,pll_api.PBAddr, pll_api.PBLink);
               break;
        case IFX_GET_PACKET_BUF_LINK_LIST:
            DPRINTK("%s: IFX_GET_PACKET_BUF_LINK_LIST\n",__FUNCTION__);
            copy_from_user(&pll_api,(packetLinkList_api *) arg,sizeof (packetLinkList_api));
            err = ifx_get_packet_buf_link_list(pll_api.chipId,pll_api.PBAddr, &pll_api.PBLink);
            if(err == 0)
                      copy_to_user((packetLinkList_api *) arg, &pll_api, sizeof(packetLinkList_api));
               break;
        case IFX_GET_CHIP_MINOR_VERSION:
            DPRINTK("%s: IFX_GET_CHIP_MINOR_VERSION\n",__FUNCTION__);
            copy_from_user(&minver_api,(minorversion_api *) arg,sizeof (minorversion_api));
            err = ifx_get_chip_minor_version(minver_api.chipId,&minver_api.minr);
            if(err == 0)
                      copy_to_user((minorversion_api *) arg, &minver_api, sizeof(minorversion_api));
               break;
        case IFX_GET_CHIP_VERSION:
            DPRINTK("%s: IFX_GET_CHIP_VERSION\n",__FUNCTION__);
            copy_from_user(&cv_api,(chipversion_api *) arg,sizeof (chipversion_api));
            err = ifx_get_chip_version(cv_api.chipId,&cv_api.Majr,&cv_api.Minr);
            if(err == 0)
                      copy_to_user((chipversion_api *) arg, &cv_api, sizeof(chipversion_api));
               break;
        case IFX_SET_CHIP_MODE:
            DPRINTK("%s: IFX_SET_CHIP_MODE\n",__FUNCTION__);
            copy_from_user(&fm_api,(feature_mode_api *) arg,sizeof (feature_mode_api));
            err = ifx_set_chip_mode(fm_api.chipId,fm_api.feature, fm_api.mode);
               break;
        case IFX_GET_CHIP_MODE:
            DPRINTK("%s: IFX_GET_CHIP_MODE\n",__FUNCTION__);
            copy_from_user(&fm_api,(feature_mode_api *) arg,sizeof (feature_mode_api));
            err = ifx_get_chip_mode(fm_api.chipId,fm_api.feature, &fm_api.mode);
            if(err == 0)
                      copy_to_user((feature_mode_api *) arg, &fm_api, sizeof(feature_mode_api));
               break;
*/
        case IFX_SET_SWITCH_MODE:
            DPRINTK("%s: IFX_SET_SWITCH_MODE\n",__FUNCTION__);
            copy_from_user(&fm_api,(feature_mode_api *) arg,sizeof (feature_mode_api));
            err = ifx_set_switch_mode(fm_api.chipId,fm_api.feature, fm_api.mode);
               break;

        case IFX_GET_SWITCH_MODE:
            DPRINTK("%s: IFX_GET_SWITCH_MODE\n",__FUNCTION__);
            copy_from_user(&fm_api,(feature_mode_api *) arg,sizeof (feature_mode_api));
            err = ifx_get_switch_mode(fm_api.chipId,fm_api.feature, &fm_api.mode);
            if(err == 0)
                      copy_to_user((feature_mode_api *) arg, &fm_api, sizeof(feature_mode_api));
               break;

        case IFX_SET_INGRESS_WATERMARK:
            DPRINTK("%s: IFX_SET_INGRESS_WATERMARK\n",__FUNCTION__);
            copy_from_user(&wm_api,(watermark_api *) arg,sizeof (watermark_api));
            err = ifx_set_ingress_watermark(wm_api.chipId,wm_api.waterMarkType, wm_api.portId,
                                  wm_api.waterMark);
               break;
        case IFX_GET_INGRESS_WATERMARK:
            DPRINTK("%s: IFX_GET_INGRESS_WATERMARK\n",__FUNCTION__);
            copy_from_user(&wm_api,(watermark_api *) arg,sizeof (watermark_api));
            err = ifx_get_ingress_watermark(wm_api.chipId,wm_api.waterMarkType,wm_api.portId,
                              &wm_api.waterMark);
            if(err == 0)
                      copy_to_user((watermark_api *) arg, &wm_api, sizeof(watermark_api));
               break;
        case IFX_SET_EGRESS_WATERMARK:
            DPRINTK("%s: IFX_SET_EGRESS_WATERMARK\n",__FUNCTION__);
            copy_from_user(&ewm_api,(Ewatermark_api *) arg,sizeof (Ewatermark_api));

            DPRINTK("%s: ewm_api.chipId=%d\n",__FUNCTION__, ewm_api.chipId);
            DPRINTK("%s: ewm_api.portId=%d\n",__FUNCTION__, ewm_api.portId);
            DPRINTK("%s: ewm_api.queueId=%d\n",__FUNCTION__, ewm_api.queueId);
            DPRINTK("%s: ewm_api.waterMark=%d\n",__FUNCTION__, ewm_api.waterMark);

            err = ifx_set_egress_watermark( ewm_api.chipId,
                                            ewm_api.portId,
                                            ewm_api.queueId,
                                            ewm_api.waterMark);
            break;
        case IFX_GET_EGRESS_WATERMARK:
            DPRINTK("%s: IFX_GET_EGRESS_WATERMARK\n",__FUNCTION__);
            copy_from_user(&ewm_api,(Ewatermark_api *) arg,sizeof (Ewatermark_api));
            err = ifx_get_egress_watermark(ewm_api.chipId,ewm_api.portId, ewm_api.queueId,
                             &ewm_api.waterMark);
            if(err == 0)
                copy_to_user((Ewatermark_api *) arg, &ewm_api, sizeof(Ewatermark_api));
            break;

        case IFX_SET_PAUSE_FRAME_GEN:
            DPRINTK("%s: IFX_SET_PAUSE_FRAME_GEN\n",__FUNCTION__);
            copy_from_user( &pfg_api, (PauseFrame_api *) arg, sizeof(PauseFrame_api));
            err = ifx_set_pause_frame_gen( pfg_api.chipId,
                                           pfg_api.portId,
                                           pfg_api.en_dis );
            break;

        case IFX_GET_PAUSE_FRAME_GEN:
            DPRINTK("%s: IFX_GET_PAUSE_FRAME_GEN\n",__FUNCTION__);
            copy_from_user( &pfg_api, (PauseFrame_api *) arg, sizeof(PauseFrame_api));
            err = ifx_get_pause_frame_gen( pfg_api.chipId,
                                           pfg_api.portId,
                                           &pfg_api.en_dis);
            if(err == 0)
                copy_to_user((PauseFrame_api *) arg, &pfg_api, sizeof(PauseFrame_api));
            break;

        case IFX_SET_TX_CONFIG:
            DPRINTK("%s: IFX_SET_TX_CONFIG\n",__FUNCTION__);
            copy_from_user(&tc_api,(TxConfig_api *) arg,sizeof (TxConfig_api));
            err = ifx_set_Tx_config( tc_api.chipId,
                                     tc_api.portId,
                                     tc_api.xmitDropCount,
                                     tc_api.lateCollisionCount );
               break;
        case IFX_GET_TX_CONFIG:
            DPRINTK("%s: IFX_GET_TX_CONFIG\n",__FUNCTION__);
            copy_from_user(&tc_api,(TxConfig_api *) arg,sizeof (TxConfig_api));
            err = ifx_get_Tx_config( tc_api.chipId,
                                     tc_api.portId,
                                     &tc_api.xmitDropCount,
                                     &tc_api.lateCollisionCount );
            if(err == 0)
                      copy_to_user((TxConfig_api *) arg, &tc_api, sizeof(TxConfig_api));
               break;
        case IFX_SET_PORT_RATE_SHAPE:
            DPRINTK("%s: IFX_SET_PORT_RATE_SHAPE\n",__FUNCTION__);
            copy_from_user(&rs_api,(rateShape_api *) arg,sizeof (rateShape_api));
            err = ifx_set_PORT_rate_shape(rs_api.chipId,rs_api.en_dis,rs_api.PRE);
            break;
        case IFX_GET_PORT_RATE_SHAPE:
            DPRINTK("%s: IFX_GET_PORT_RATE_SHAPE\n",__FUNCTION__);
            copy_from_user(&rs_api,(rateShape_api *) arg,sizeof (rateShape_api));
            err =  ifx_get_PORT_rate_shape(rs_api.chipId,&rs_api.en_dis,&rs_api.PRE);
            if(err == 0)
                copy_to_user((rateShape_api *) arg, &rs_api, sizeof(rateShape_api));
            break;
        default:
            return -ENOTTY;
        break;

    }


    return err;
}

