/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Aparna   		    1/9/2003					        	
 *
 *
 * Manamohan			10/8/2004					Added support for re-entrancy
 *
 * DESCRIPTION :This module contains the Packet Buffer Link List Configuration API's
 *
 ******************************************************************************/
#include <asm/types.h>
#include <asm/addrspace.h>
#include "struct.h"
#include "const.h"
#include "pbll.h"
#include <asm/incaip2/incaip2.h>
#include "err.h"
#include "var.h"
#include "api_init_prv.h"



/******************************************************************************
  *
  *	FUNCTION:	 ifx_set_packet_buf
  *
  *	DESCRIPTION: Set the PB (Packet Buffer) Link List Free Queue parameters.
  *
  *	PARAMETERS:
  *		IN:
  *         ChipId:    	ChipId.
  *			readPtr		readPtr range is 0-8190
  *			writePtr	writePtr range is 0-8190	
  *			count		count range is 0-8190
  *		OUT:
  *			None
  * 		
  *	RETURNS:
  *		PR_ERR_NONE						Success	
  *		PR_ERR_PBLL_READPTR				If the Read pointer is not in the range of 0 to 8190
  *		PR_ERR_PBLL_WRITEPTR			If the write pointer is not in the range of 0 to 8190
  *		PR_ERR_PBLL_COUNT				If the free packet buffers count is not in the range 
  *										of 0 to 8190
  *     PR_ERR_CHIP_ID					Invlaid Chip Id
  *		 				
  ******************************************************************************/

s32 ifx_set_packet_buf(u8 chipId,u32 readPtr, u32 writePtr, u32 count)
{		
	u32* ptr_freepb_rptr;			
	u32* ptr_freepb_wptr;
	u32* ptr_freepb_cnt;
	s32 retflag=PR_ERR_NONE;


#ifdef PARAMETER_CHECK
	
	if(readPtr > (TOTAL_PBNUM - 1))
		return PR_ERR_PBLL_READPTR;
	if(writePtr >(TOTAL_PBNUM - 1))
		return PR_ERR_PBLL_WRITEPTR;
	if(count > TOTAL_PBNUM)
		return PR_ERR_PBLL_COUNT;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;

#endif
	ptr_freepb_rptr = (u32*)(Chip_Base_Address[chipId] + FREEPB_RPTR_REG);			
	ptr_freepb_wptr = (u32*)(Chip_Base_Address[chipId] + FREEPB_WPTR_REG);
	ptr_freepb_cnt = (u32*)(Chip_Base_Address[chipId] + FREEPB_CNT_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

/* PBLL Free Read Pointer */

	PR_Reg_Write(ptr_freepb_rptr, readPtr);

/* PBLL Free Write Pointer */

	PR_Reg_Write(ptr_freepb_wptr, writePtr);

/* PBLL Free Queue Counter */
	PR_Reg_Write(ptr_freepb_cnt, count);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;


}


/******************************************************************************
  *
  *	FUNCTION:	 ifx_get_packet_buf
  *
  *	DESCRIPTION: Get the PB (Packet Buffer) Link List Free Queue parameters.
  *
  *	PARAMETERS:
  *		IN:
  *			ChipId:    	ChipId.
  *		OUT:
  *			readPtr		readPtr (range is 0-8190)
  *			writePtr	writePtr (range is 0-8190)	
  *			count		count (range is 0-8190)
  * 		
  *	RETURNS:
  *		PR_ERR_NONE				Success							
  *		PR_ERR_NULL_PTR			Null pointer 			
  *		PR_ERR_CHIP_ID          Invalid Chip Id
  *
  ******************************************************************************/

s32 ifx_get_packet_buf(u8 chipId,u32* readPtr, u32* writePtr, u32* count)
{

	u32* ptr_freepb_rptr;			
	u32* ptr_freepb_wptr;
	u32* ptr_freepb_cnt;

	u32 temp=0;
	s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK
	if(readPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(writePtr == NULL)
		return PR_ERR_NULL_PTR;
	if(count == NULL)
		return PR_ERR_NULL_PTR;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_freepb_rptr = (u32*)(Chip_Base_Address[chipId] + FREEPB_RPTR_REG);			
	ptr_freepb_wptr = (u32*)(Chip_Base_Address[chipId] + FREEPB_WPTR_REG);
	ptr_freepb_cnt = (u32*)(Chip_Base_Address[chipId] + FREEPB_CNT_REG);

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
    		return retflag;

/* PBLL Free Queue Read Pointer */

	PR_Reg_Read(ptr_freepb_rptr ,temp);
	*readPtr = temp;

/* PBLL Free Queue Write Pointer */

	PR_Reg_Read(ptr_freepb_wptr ,temp);
	*writePtr = temp;
	 
/* PBLL Free Queue Counter */
	PR_Reg_Read(ptr_freepb_cnt, temp);
	*count =temp;

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
    	return retflag;


	return PR_ERR_NONE;

}

/******************************************************************************
  *
  *	FUNCTION:	 ifx_set_packet_buf_link_list
  *				 	
  *	DESCRIPTION: Set the PB (Packet Buffer) Link List.
  *				 
  *	PARAMETERS:
  *		IN: 
  *         ChipId:    	ChipId.
  *			PBAddr		Current Buffer number. (0 - 8190)
  *			PBLink		Poniter to next Packet Buffer number. (0 - 8190)
  *		OUT:
  *			None
  *
  *	RETURNS:
  *		PR_ERR_NONE				Success							
  *		PR_ERR_PBLL_ADDR		If the Packet buffer number is not in the range of 0 to 8190 
  *		PR_ERR_PBLL_LINK		If the next Packet buffer number is not in the range of 0 to 8190
  *     PR_ERR_CHIP_ID          Invalid Chip Id
  *     PR_ERR_NULL_PTR         Null Pointer
  *     PR_ERR_TIME_OUT         Time Out
  *
  ******************************************************************************/

s32 ifx_set_packet_buf_link_list(u8 chipId,u32 PBAddr, u32 PBLink)
{
#if 0
	
/* Pointer to PBLL command register */
	volatile pbll_cmd_reg_t* ptr_pbll_cmd;
		
/* Pointer to PBLL Data register */	
	u32* ptr_pbll_data;
	u32 Temp = 0;
	volatile u32 temp1=0;
	u32 i=0;

	u32* ptr_reg1;
	s32 retflag=PR_ERR_NONE;


#ifdef PARAMETER_CHECK
	
	if(PBAddr > (TOTAL_PBNUM - 1))
		return PR_ERR_PBLL_ADDR;
	if(PBLink > TOTAL_PBNUM)
		return PR_ERR_PBLL_LINK;

	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;
#endif

	ptr_pbll_data = (u32*)(Chip_Base_Address[chipId] + PBLL_DATA_REG);

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PBLL_CMD_REG);

	ptr_pbll_cmd = (volatile pbll_cmd_reg_t*)&temp1;

/* Configuration of PBLL Command register for reading */
	ptr_pbll_cmd -> Address = PBAddr;
	
	ptr_pbll_cmd -> Nptr_svcnt = 0;  /* 0 -> Next Pointer,spid */

	ptr_pbll_cmd -> Read_write = READPR;

	ptr_pbll_cmd -> Req_ack = REQ;

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_PBLL)) != PR_ERR_NONE)
    		return retflag;

/* Added MACRO for indirect register read 7/7/2004 */
	PR_Indirect_Reg_Read_Sem(ptr_reg1,ptr_pbll_data,ptr_pbll_cmd,temp1,Temp,SEM_PBLL);


/* Clear the old Nxt_pbnum */	
	Temp &= 0xFFFFE000;

/* Write the new value*/
	Temp |= PBLink;

/* Configuration of PBLL Command register for writing */
	
	ptr_pbll_cmd -> Nptr_svcnt = 0;  /* 0 -> Next Pointer,spid  */

	ptr_pbll_cmd -> Read_write = WRITEPR;

	ptr_pbll_cmd -> Req_ack = REQ;

/* Added MACRO for indirect register write 7/7/2004 */
	PR_Indirect_Reg_Write_Sem(ptr_reg1,ptr_pbll_data,ptr_pbll_cmd,temp1,Temp,SEM_PBLL);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_PBLL)) != PR_ERR_NONE)
    	return retflag;

	return PR_ERR_NONE;
#endif
    printk("%s: Check whether the functionality can be implemented w/o pbll_cmd_reg_t\n");
    return -1;
}

/******************************************************************************
  *
  *	FUNCTION:	 ifx_get_packet_buf_link_list
  *				 	
  *	DESCRIPTION: Get the PB (Packet Buffer) Link List.
  *				 
  *	PARAMETERS:	 
  *		IN:
  *         ChipId:    	ChipId.
  *			PBAddr		Current Buffer number. (0 - 8190)
  *		OUT:
  *			PBLinkPtr	Poniter to next Packet Buffer number. (0 - 8190)	
  *
  *	RETURNS:
  *		PR_ERR_NONE				Success							
  *		PR_ERR_PBLL_ADDR		If the Packet buffer number is not in the range of 0 to 8190
  *     PR_ERR_NULL_PTR			Null pointer			 				
  *		PR_ERR_CHIP_ID          Invalid Chip Id
  *     PR_ERR_NULL_PTR         Null Pointer
  *
  ******************************************************************************/

s32 ifx_get_packet_buf_link_list(u8 chipId,u32 PBAddr, u32* PBLinkPtr)
{	
/* Pointer to PBLL command register */
	volatile pbll_cmd_reg_t* ptr_pbll_cmd;
		
/* Pointer to PBLL Data register */	
	u32* ptr_pbll_data;
		
	u32 Temp = 0;
	volatile u32 temp1=0;
	u32 i=0;
	u32* ptr_reg1;
	s32 retflag=PR_ERR_NONE;

	
#ifdef PARAMETER_CHECK
	
	if(PBAddr > (TOTAL_PBNUM - 1))
		return PR_ERR_PBLL_ADDR;

	if(PBLinkPtr == NULL)
		return PR_ERR_NULL_PTR;
	if(chipId >= Total_Chips)
		return PR_ERR_CHIP_ID ;

	if(Chip_Base_Address[chipId] == NULL_PTR)
		return PR_ERR_NULL_PTR;	
#endif

	ptr_pbll_data = (u32*)(Chip_Base_Address[chipId] + PBLL_DATA_REG);

	ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PBLL_CMD_REG);

	PR_Reg_Read(ptr_reg1,temp1);
	ptr_pbll_cmd = (volatile pbll_cmd_reg_t*)&temp1;

	
/* Configuration of PBLL Command register for reading */
	ptr_pbll_cmd -> Address = PBAddr;
	
	ptr_pbll_cmd -> Nptr_svcnt = 0;  /* 0 -> Next Pointer,spid */

	ptr_pbll_cmd -> Read_write = READPR;

	ptr_pbll_cmd -> Req_ack = REQ;

/* Added for re-entrancy */
	if((retflag = ifx_sem_take(chipId,SEM_PBLL)) != PR_ERR_NONE)
    		return retflag;

/* Added MACRO for indirect register read 7/7/2004 */
	PR_Indirect_Reg_Read_Sem(ptr_reg1,ptr_pbll_data,ptr_pbll_cmd,temp1,Temp,SEM_PBLL);

/* Added for re-entrancy */
	if((retflag = ifx_sem_give(chipId,SEM_PBLL)) != PR_ERR_NONE)
    	return retflag;


	*PBLinkPtr = Temp & 0x1FFF;			/* Nxt_pbnum */
	
	return PR_ERR_NONE;

}

























