/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup Internal Internally used functions */

#include <asm/types.h>
#include <asm/addrspace.h>
#include <linux/kernel.h>
#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/incaip2.h>
#include "struct.h"
#include "const.h"
#include "switch.h"
#include "switch_prv.h"
#include "err.h"
#include "ports.h"
#include "var.h"
#include "api_init_prv.h"


#undef INCAIP2_DEBUG_SWITCH

/**
 * Get Chip version.
 * Get the Revision Id of the chip.
 *
 * \param   chipId           Chip Id 
 * \param   chip_Majr_idPtr  Pointer to chip major ID
 * \param   chip_Minr_idPtr  Pointer to chip minor ID
 * \return  PR_ERR_NONE      - Success 
 * \return  PR_ERR_NULL_PTR  - chip_Majr_idPtr or chip_Minr_idPtr is null pointer 
 * \return  PR_ERR_CHIP_ID   - If the chip Id invalid
 * \ingroup Internal
 */
s32 ifx_get_chip_version(u8 chipId,u32 *chip_Majr_idPtr,u32 *chip_Minr_idPtr)
{
/* Checked - OK */
#ifdef PARAMETER_CHECK
    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;
    
    if(chip_Majr_idPtr == NULL)
    return PR_ERR_NULL_PTR;

    if(chip_Minr_idPtr == NULL)
    return PR_ERR_NULL_PTR;
#endif  

    *(chip_Majr_idPtr) = INCA_IP2_MPS_CHIPID_VERSION_GET(*(INCA_IP2_MPS_CHIPID));
    *(chip_Minr_idPtr) = *(INCA_IP2_MPS_SUBVER);
    return PR_ERR_NONE;
}

#if 0  
/*****************************************************************************
 * 
 * FUNCTION:    ifx_set_chip_mode
 * 
 * DESCRIPTION: Set the operation mode of the chip features.                
 *              
 * PARAMETERS:
 *      IN:
 *          ChipId      ChipId.
 *          feature     This parameter indicates which of the features need to be set
 *          mode        Value of the parameter
 *      OUT:
 *          None        
 * 
 * RETURNS:
 *      PR_ERR_NONE             Success
 *      PR_ERR_CHIP_MODE        If the feature is Invalid.
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *      PR_ERR_NULL_PTR         Null Pointer
 *       
 *************************************************************************/

s32 ifx_set_chip_mode(u8 chipId,u32 feature, u32 mode)
{
     
    purple_config1_reg_t*  ptr_purple_congig1;
    purple_config3_reg_t*  ptr_purple_congig3;

        
    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;
    
#endif

    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PURPLE_CONFIG1_REG);
    ptr_reg2 = (u32*)(Chip_Base_Address[chipId] + PURPLE_CONFIG3_REG);


    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;


    PR_Reg_Read(ptr_reg1,temp1);
    ptr_purple_congig1 = (purple_config1_reg_t*)&temp1;

        
    PR_Reg_Read(ptr_reg2,temp2);
    ptr_purple_congig3 = (purple_config3_reg_t*)&temp2; 
            
        
        
    switch (feature) 
        {

        case CPU_PACKET_FORWARD: 
            ptr_purple_congig1->Intelligent = mode;
            break;

        case EDRAM_FPI_ACCESS: 
            ptr_purple_congig1->EDRAM_Access = mode;
            break;


        case SYSTEM_PLL_BYPASS: 
            ptr_purple_congig1->SYS_PLL_bypass = mode;
            break;


        case SYS_CLOCK_FEEDBACK_DIVIDER: 
            ptr_purple_congig1 -> Factor_n1 = mode;
            break;


        case SYS_CLOCK_INPUT_DIVIDER: 
            ptr_purple_congig1->Factor_m1 = mode;
            break;


        default:
            ifx_sem_give(chipId,SEM_GENRL);
            return PR_ERR_CHIP_MODE;
    }


    PR_Reg_Write(ptr_reg1,temp1);
    PR_Reg_Write(ptr_reg2,temp2);


    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;


    return PR_ERR_NONE;

}
#endif    

#if 0      
/*****************************************************************************
 * 
 * FUNCTION:    ifx_get_chip_mode
 * 
 * DESCRIPTION: Get the operation mode of the chip features.                
 *              
 * PARAMETERS:
 *      IN:
 *          ChipId      ChipId.  
 *          feature     This parameter indicates which of the features need to be read
 *      OUT:
 *          modePtr     Pointer to value of the parameter
 * 
 * RETURNS:
 *      PR_ERR_NONE             Success
 *      PR_ERR_CHIP_MODE        If the feature is Invalid
 *      PR_ERR_NULL_PTR         Null pointer 
 *      PR_ERR_CHIP_ID          Invalid Chip Id
 *       
 *************************************************************************/
s32 ifx_get_chip_mode(u8 chipId,u32 feature, u32 *modePtr)
{
 
    purple_config1_reg_t*  ptr_purple_congig1;
    purple_config3_reg_t*  ptr_purple_congig3;

    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;


#ifdef PARAMETER_CHECK

    if(modePtr == NULL)
    return PR_ERR_NULL_PTR;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;
#endif  

    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PURPLE_CONFIG1_REG);
    ptr_reg2 = (u32*)(Chip_Base_Address[chipId] + PURPLE_CONFIG3_REG);


    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    
    PR_Reg_Read(ptr_reg1,temp1);
    ptr_purple_congig1 = (purple_config1_reg_t*)&temp1;


    PR_Reg_Read(ptr_reg2,temp2);
    ptr_purple_congig3 = (purple_config3_reg_t*)&temp2;


    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

        
    
        switch (feature) 
        {

        case IIC_ADDRESS: 
             
            *modePtr = ptr_purple_congig3->IIC_address;
            break;

        case MEM_CONFIG_MODE:
            
            *modePtr = ptr_purple_congig3->Mem_Config;
            break;

        case ENDIAN_MODE:
            /*0=big endian , 1= little endian in purple*/
             
            *modePtr = ptr_purple_congig3->endian ;
            break;

        case EXT_ROM_BOOT:
            
            *modePtr = ptr_purple_congig3->Ext_Boot;
            break;

        case BOARD_ID: 
             
            *modePtr = ptr_purple_congig3->Board_Id;
            break;

        case EXT_DATA_EEPROM: 
             
            *modePtr = ptr_purple_congig3->Ext_Data_EPROM ;
            break;

        case EXT_SDRAM_EEPROM: 
             
            *modePtr = ptr_purple_congig3->Ext_SDRAM_EPROM ;
            break;

        case CLOCK_CONTROL: 
            
            *modePtr = ptr_purple_congig3->Clk_Ctrl;
            break;

        case PCI_SDRAM_SELECT:
             
            *modePtr = ptr_purple_congig3->PCI_SDRAM_sel ;
            break;

        case TEST_MODE: 
             
            *modePtr = ptr_purple_congig3->Test_Mode;
            break;
        
    
        case CPU_PACKET_FORWARD: 
             
            *modePtr = ptr_purple_congig1->Intelligent ;
            break;

        case EDRAM_FPI_ACCESS: 
             
            *modePtr = ptr_purple_congig1->EDRAM_Access;
            break;

        case MIPS_PLL_BYPASS: 
             
            *modePtr = ptr_purple_congig1->MIPS_PLL_bypass;
            break;

        case SYSTEM_PLL_BYPASS: 
             
            *modePtr = ptr_purple_congig1->SYS_PLL_bypass;
            break;

        case CPU_CLOCK_FEEDBACK_DIVIDER: 
             
            *modePtr = ptr_purple_congig1->Factor_n2 ;
            break;

        case CPU_CLOCK_INPUT_DIVIDER: 
             
            *modePtr = ptr_purple_congig1->Factor_m2 ;
            break;

        case SYS_CLOCK_FEEDBACK_DIVIDER: 
             
            
            *modePtr = ptr_purple_congig1 -> Factor_n1;
            break;


        case SYS_CLOCK_INPUT_DIVIDER: 
             
            *modePtr = ptr_purple_congig1->Factor_m1;
            break;

        default:
            return PR_ERR_CHIP_MODE;
    }
    return PR_ERR_NONE;

}
#endif    



/**
 * Set Switch operation modes
 * Sets operation modes for various switch features. The control bits assigned 
 * to the selectable features are located in the PR_CTRL_REG resp. GMAC_REG.
 
 *
 * \param   chipId         ChipId
 * \param   feature        Feature select
 *                         The modes for the following features can be read:
 *                             2 - GE_MCASTQ_THRESHOLD
 *                             6 - AGING_SOURCE
 *                             7 - AGING_TIME_SELECT
 *                             8 - MA_FREEZE_ALL
 *                             9 - MA_FREEZE_NEW
 *                             10 - CRCERROR_PACKET_MONITOR
 *                             11 - MACHANGE_PACKET_MONIOR
 *                             12 - PAUSE_PACKET_TO_CPU     
 *                             13 - FORWARD_CFI_PACKET	
 *                             19 - PORT0_GE_CONF
 *                             21 - PORT1_GE_CONF
 *                             14 - MA_REPLACE_THRESHOLD		
 *                             33 - UNKNOWN_PROTOCOL_TO_CPU	
 *                             34 - MONITORING_PORT 
 *                             35 - SEND_RESERVED_PACKETS_2_CPU
 * \param   mode           Mode value for the feature 
 *                              
 * \return  PR_ERR_NONE          - Success 
 * \return  PR_ERR_CHIP_ID       - Invalid Chip Id
 * \return  PR_ERR_CHIPSW_MODE   - Feature is Invalid  
 * \return  PR_ERR_NULL_PTR      - base address is null pointer
 * \ingroup Internal
 */
s32 ifx_set_switch_mode( u8 chipId, u32 feature, u32 mode )
{
    pr_ctrl_reg_t*  ptr_pr_ctrl;
	gmac_reg_t*	    ptr_gmac ;
    u32 temp1=0;
    u32* ptr_reg1;
    u32 *ptr_mirror;
    s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;

#endif

    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PR_CTRL_REG);
    ptr_mirror =(u32*)(Chip_Base_Address[chipId] + MIRROR_PORT_REG) ;

    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_pr_ctrl = (pr_ctrl_reg_t*)&temp1;

    switch(feature)
    {
        case GE_MCASTQ_THRESHOLD:
            ptr_pr_ctrl->ge_bcastq_threshold = mode;
            break;

        case AGING_SOURCE: 
            ptr_pr_ctrl->Age_tick_Sw= mode;
            break;

        case AGING_TIME_SELECT: 
            ptr_pr_ctrl->Age_Timer = mode;
            break;

        case MA_FREEZE_ALL: 
            ptr_pr_ctrl->MA_Freeze = mode;
            break;

        case MA_FREEZE_NEW:
            ptr_pr_ctrl->MA_Freez_New = mode;
            break;

        case MONITORING_PORT:
            PR_Reg_Write(ptr_mirror,mode);
            break;

        case CRCERROR_PACKET_MONITOR: 
            ptr_pr_ctrl->Monitor_crc = mode;
            break;

        case MACHANGE_PACKET_MONIOR: 
            ptr_pr_ctrl->Monitor_MA_change_pkt = mode;
            break;

        case PAUSE_PACKET_TO_CPU: 
            ptr_pr_ctrl->Fwd_pause2cpu = mode;
            break;

        case FORWARD_CFI_PACKET: 
            ptr_pr_ctrl->Fwd_cfi_pkts = mode;
            break;

        case UNKNOWN_PROTOCOL_TO_CPU: 
            ptr_pr_ctrl->Fwd_unknown2_cpu = mode;
            break;

        case MA_REPLACE_THRESHOLD: 
            ptr_pr_ctrl->MA_Replace_Threshold = mode;
            break;

        case PORT0_GE_CONF: /* write GE mode to P0 GMAC_REG */
			ptr_gmac = (gmac_reg_t *)&temp1;		
			PR_Reg_Read((u32*)SWITCH_P0_GMAC_REG,temp1);
#ifdef INCAIP2_DEBUG_SWITCH			
			printk("%s - read 0x%08X from SWITCH_P0_GMAC_REG\n", __FUNCTION__, temp1);
#endif			
            ptr_gmac->g_sel = mode;		
			PR_Reg_Write((u32*)SWITCH_P0_GMAC_REG, temp1);
            break;
            
        case PORT1_GE_CONF: /* write GE mode to P1 GMAC_REG */
			ptr_gmac = (gmac_reg_t *)&temp1;		
			PR_Reg_Read((u32*)SWITCH_P1_GMAC_REG,temp1);
#ifdef INCAIP2_DEBUG_SWITCH				
			printk("%s - read 0x%08X from SWITCH_P1_GMAC_REG\n", __FUNCTION__, temp1);
#endif			
            ptr_gmac->g_sel = mode;		
			PR_Reg_Write((u32*)SWITCH_P1_GMAC_REG, temp1);
            break;                
            
        case SEND_RESERVED_PACKETS_2_CPU: /* allow dot1x PAE MAC addr. send to CPU */
            ptr_pr_ctrl->Send_Res2_cpu = mode;
            break;

        default:
            ifx_sem_give(chipId,SEM_GENRL);
            return PR_ERR_CHIPSW_MODE;

    }

    PR_Reg_Write(ptr_reg1,temp1);

	if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;


    return PR_ERR_NONE;
}


/**
 * Get operation mode for switch feature.
 * Get the operation mode for various switch features. The control bits that are
 * read for the selectable features are located in the PR_CTRL_REG resp. GMAC_REG.
 *
 * \param   chipId              Chip Id
 * \param   feature             Feature select.
 *                              The modes for the following features can be read:
 *                                  2 - GE_MCASTQ_THRESHOLD
 *                                  6 - AGING_SOURCE
 *                                  7 - AGING_TIME_SELECT
 *                                  8 - MA_FREEZE_ALL
 *                                  9 - MA_FREEZE_NEW
 *                                  10 - CRCERROR_PACKET_MONITOR
 *                                  11 - MACHANGE_PACKET_MONIOR
 *                                  12 - PAUSE_PACKET_TO_CPU     
 *                                  13 - FORWARD_CFI_PACKET			
 *                                  14 - MA_REPLACE_THRESHOLD		
 *                                  20 - PORT0_GE_MODE
 *                                  22 - PORT1_GE_MODE
 *                                  33 - UNKNOWN_PROTOCOL_TO_CPU	
 *                                  34 - MONITORING_PORT
 *                                  35 - SEND_RESERVED_PACKETS_2_CPU
 * \param   modePtr             Pointer to feature value.
 * \return  PR_ERR_NONE        - Success 
 * \return  PR_ERR_CHIPSW_MODE - Feature is Invalid 
 * \return  PR_ERR_NULL_PTR    - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID     - Chip Id is invalid
 * \ingroup Internal
 */ 
s32 ifx_get_switch_mode( u8 chipId, u32 feature, u32* modePtr )
{
    pr_ctrl_reg_t*  ptr_pr_ctrl;
	gmac_reg_t*	    ptr_gmac;   
    u32 reg32=0;  /*,temp1=0,temp2=0,temp3=0; */
    u32* ptr_reg1;
    u32* ptr_mirror;
    s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

    if(modePtr == NULL)
    return PR_ERR_NULL_PTR;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;
#endif  

    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PR_CTRL_REG);
    ptr_mirror= (u32*)(Chip_Base_Address[chipId] + MIRROR_PORT_REG) ;
	
    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;
    
    PR_Reg_Read(ptr_reg1,reg32);
    ptr_pr_ctrl = (pr_ctrl_reg_t*)&reg32;

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

    switch(feature)
    {
        case GE_MCASTQ_THRESHOLD:
            *modePtr = ptr_pr_ctrl->ge_bcastq_threshold;
            break;

        case AGING_SOURCE: 
            *modePtr = ptr_pr_ctrl->Age_tick_Sw;
            break;

        case AGING_TIME_SELECT: 
            *modePtr = ptr_pr_ctrl->Age_Timer ;
            break;

        case MA_FREEZE_ALL: 
            *modePtr = ptr_pr_ctrl->MA_Freeze ;
            break;

        case MA_FREEZE_NEW:
            *modePtr = ptr_pr_ctrl->MA_Freez_New ;
            break;

        case MONITORING_PORT:
            PR_Reg_Read(ptr_mirror, reg32);
            *modePtr = reg32;
            break;

        case CRCERROR_PACKET_MONITOR: 
            *modePtr = ptr_pr_ctrl->Monitor_crc;
            break;

        case MACHANGE_PACKET_MONIOR: 
            *modePtr = ptr_pr_ctrl->Monitor_MA_change_pkt;
            break;

        case PAUSE_PACKET_TO_CPU: 
            *modePtr = ptr_pr_ctrl->Fwd_pause2cpu;
            break;

        case FORWARD_CFI_PACKET: 
            *modePtr = ptr_pr_ctrl->Fwd_cfi_pkts;
            break;

        case UNKNOWN_PROTOCOL_TO_CPU: 
            *modePtr = ptr_pr_ctrl->Fwd_unknown2_cpu;
            break;

        case MA_REPLACE_THRESHOLD: 
            *modePtr = ptr_pr_ctrl->MA_Replace_Threshold;
            break;

        case PORT0_GE_MODE: /* get GE mode from P0 GMAC_REG */
            ptr_gmac = (gmac_reg_t *)&reg32;
			PR_Reg_Read((u32*)SWITCH_P0_GMAC_REG, reg32);
#ifdef INCAIP2_DEBUG_SWITCH				
			printk("%s - read 0x%08X from SWITCH_P0_GMAC_REG\n", __FUNCTION__, reg32);
#endif			
            *modePtr = (u32)ptr_gmac->g_sel;
            break;
            
        case PORT1_GE_MODE: /* get GE mode from P1 GMAC_REG */
            ptr_gmac = (gmac_reg_t *)&reg32;
			PR_Reg_Read((u32*)SWITCH_P1_GMAC_REG, reg32);	
#ifdef INCAIP2_DEBUG_SWITCH				
			printk("%s - read 0x%08X from SWITCH_P1_GMAC_REG\n", __FUNCTION__, reg32);
#endif			
            *modePtr = (u32)ptr_gmac->g_sel;  
            break;            
            
        case SEND_RESERVED_PACKETS_2_CPU:
            *modePtr = ptr_pr_ctrl->Send_Res2_cpu;
            break;

        default:
			printk("%s - Invalid Feature %d passed\n", __FUNCTION__, feature);		
            return PR_ERR_CHIPSW_MODE;
    }
    return PR_ERR_NONE;
}

/**
 * Set Ingress watermark.
 * Set the Ingress watermark of the individual WaterMark Type.
 *
 * \param   chipId                 Chip Id
 * \param   waterMarkType          watermark type
 *                                   - 1 - GLOBAL_DROP_WATERMARK
 *                                   - 2 - PORTGROUP_FC_WATERMARK
 *                                   - 3 - PORTGROUP_DROP_WATERMARK 
 * \param   portId                 Value of the port Id (0 - 2)
 * \param   waterMark              Value of the watermark
 * \return  PR_ERR_NONE            - Success 
 * \return  PR_ERR_PORTID          - If the portId is not in the range of 0-2
 * \return  PR_ERR_INGRESS_WM_TYPE - If waterMarkType is not in the range of 1 -3
 * \return  PR_ERR_NULL_PTR        - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID         - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_set_ingress_watermark(u8 chipId,u32 waterMarkType, u32 portId, 
                                  u32 waterMark)
{

    port_rx_wm_regs_t*  ptr_port_rx_wm ;
    global_rx_wm_reg_t*  ptr_global_rx_wm ;

    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;

#ifdef  PARAMETER_CHECK
    if(portId >= TOTAL_PORTS)  
        return PR_ERR_PORTID;

    if(chipId >= Total_Chips) 
        return PR_ERR_CHIP_ID ;
    
    if( waterMark > MAX_INGR_WM_VAL )
        return PR_ERR_WM_VAL;            
    
    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif

    ptr_reg1 = (u32*) PORT_RX_WM_REGS + portId;
    ptr_reg2 = (u32*) GLOBAL_RX_WM_REG;
    
    
    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;
        
    PR_Reg_Read(ptr_reg1,temp1);
    ptr_port_rx_wm = (port_rx_wm_regs_t*)&temp1;
    
    PR_Reg_Read(ptr_reg2,temp2);
    ptr_global_rx_wm = (global_rx_wm_reg_t*)&temp2;
    
    switch(waterMarkType)
    {
        case GLOBAL_DROP_WATERMARK: 
            *ptr_global_rx_wm = waterMark;
            break;
    
        case PORTGROUP_FC_WATERMARK:    
            ptr_port_rx_wm->rx_wm1 = waterMark;
            break;
    
        case PORTGROUP_DROP_WATERMARK:  
            ptr_port_rx_wm->rx_wm2 = waterMark;
            break;

        default:
            ifx_sem_give(chipId,SEM_GENRL);
            return PR_ERR_INGRESS_WM_TYPE;
    }

    PR_Reg_Write(ptr_reg1,temp1);
    PR_Reg_Write(ptr_reg2,temp2);

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

    return PR_ERR_NONE;
}

/**
 * Get Ingress watermark.
 * Get the Ingress watermark of the individual WaterMark Type. 
 *
 * \param   chipId                 Chip Id
 * \param   waterMarkType          watermark type
 *                                   - 1 - GLOBAL_DROP_WATERMARK
 *                                   - 2 - PORTGROUP_FC_WATERMARK
 *                                   - 3 - PORTGROUP_DROP_WATERMARK 
 * \param   portId                 Value of the port Id (0 - 2)
 * \param   waterMarkPtr           Pointer to value of the watermark
 * \return  PR_ERR_NONE            - Success 
 * \return  PR_ERR_PORTID    - If the portId is not in the range of 0-2
 * \return  PR_ERR_INGRESS_WM_TYPE - If waterMarkType is not in the range of 1 -3
 * \return  PR_ERR_NULL_PTR        - waterMarkPtr or base address is null Pointer
 * \return  PR_ERR_CHIP_ID         - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_get_ingress_watermark(u8 chipId,u32 waterMarkType,u32 portId, 
                              u32 *waterMarkPtr)    
{

    port_rx_wm_regs_t*  ptr_port_rx_wm ;
    global_rx_wm_reg_t*  ptr_global_rx_wm ;

    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;
    
    
#ifdef  PARAMETER_CHECK
    
    if(portId >= TOTAL_PORTS)
        return PR_ERR_PORTID;
    if(waterMarkPtr == NULL)
        return PR_ERR_NULL_PTR;
    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif

    ptr_reg1 = (u32*) PORT_RX_WM_REGS + portId;
    ptr_reg2 = (u32*) GLOBAL_RX_WM_REG;
    
    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_port_rx_wm = (port_rx_wm_regs_t*)&temp1;

    PR_Reg_Read(ptr_reg2,temp2);
    ptr_global_rx_wm = (global_rx_wm_reg_t*)&temp2;

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;
    
    switch(waterMarkType)
    {
        case GLOBAL_DROP_WATERMARK: 
            *waterMarkPtr = *ptr_global_rx_wm ;
            break;
    
        case PORTGROUP_FC_WATERMARK:    
            *waterMarkPtr = ptr_port_rx_wm->rx_wm1 ;
            break;
    
        case PORTGROUP_DROP_WATERMARK:  
            *waterMarkPtr = ptr_port_rx_wm->rx_wm2 ;
            break;

        default:
            return PR_ERR_INGRESS_WM_TYPE;
    }

    return PR_ERR_NONE;
}

/**
 * Set Egress watermark.
 * Set the Egress watermark of the individual WaterMark Type
 *
 * \param   chipId              Chip Id
 * \param   queueId             queue Id (0 - 3)
 * \param   portId              Value of the port Id (0 - 2)
 * \param   waterMark           Value of watermark
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PORTID       - If the portId is not in the range of 0-2
 * \return  PR_ERR_QUEUEID      - If queueId is not in the range of 0 -3
 * \return  PR_ERR_NULL_PTR     - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_set_egress_watermark( u8 chipId, u32 portId, u32 queueId,
                              u32 waterMark)
{
    port_tx_wm_reg0_t*  ptr_port_tx_wm0;   
    port_tx_wm_reg1_t*  ptr_port_tx_wm1; 
    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;
	
#ifdef INCAIP2_DEBUG_SWITCH	  
    printk("%s( chipId=%d, portId=%d, queueId=%d, waterMark=0x%08X)\n",
           __FUNCTION__, chipId, portId, queueId, waterMark);
#endif    
#ifdef  PARAMETER_CHECK
    
    if(portId >=TOTAL_PORTS)
        return PR_ERR_PORTID;

    if(queueId >= TOTAL_EGRESS_QUEUES)
        return PR_ERR_QUEUEID;

    if(chipId >= Total_Chips)
    {
        printk("%s - invalid chipId\n", __FUNCTION__);
        return PR_ERR_CHIP_ID ;
    }

    if( waterMark > MAX_EGR_WM_VAL )
        return PR_ERR_WM_VAL;        
    
    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif

    ptr_reg1 = (u32*) (Chip_Base_Address[chipId] + PORT_TX_WM_REGS) + (portId<<1);
    ptr_reg2 = (u32*) (Chip_Base_Address[chipId] + PORT_TX_WM_REGS) + ((portId<<1) + 1);

    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_port_tx_wm0 = (port_tx_wm_reg0_t*)&temp1;

    PR_Reg_Read(ptr_reg2,temp2);
    ptr_port_tx_wm1 = (port_tx_wm_reg1_t*)&temp2;

    
    switch(queueId)
    {
        case QUEUE_0:
            ptr_port_tx_wm0->Queue0_tx_wm = waterMark;
            break;

        case QUEUE_1:
            ptr_port_tx_wm0->Queue1_tx_wm = waterMark;
            break;
    
        case QUEUE_2:
            ptr_port_tx_wm1->Queue2_tx_wm = waterMark;
            break;

        case QUEUE_3:
            ptr_port_tx_wm1->Queue3_tx_wm = waterMark;
            break;

        default :
            ifx_sem_give(chipId,SEM_GENRL);
            return PR_ERR_QUEUEID;
    }

    PR_Reg_Write(ptr_reg1,temp1);
    PR_Reg_Write(ptr_reg2,temp2); 
    

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

    return PR_ERR_NONE;
}

/**
 * Get Egress watermark.
 * Get the Egress watermark of the individual WaterMark Type
 *
 * \param   chipId              Chip Id
 * \param   queueId             queue Id (0 - 3)
 * \param   portId              Value of the port Id (0 - 2)
 * \param   waterMarkPtr        Pointer to value of watermark
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PORTID       - If the portId is not in the range of 0-2
 * \return  PR_ERR_QUEUEID      - If queueId is not in the range of 0 -3
 * \return  PR_ERR_NULL_PTR     - waterMarkPtr or base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_get_egress_watermark( u8 chipId, u32 portId, u32 queueId,
                              u32* waterMarkPtr)
{

    port_tx_wm_reg0_t*  ptr_port_tx_wm0;   
    port_tx_wm_reg1_t*  ptr_port_tx_wm1; 

    u32 temp1=0,temp2=0;
    u32* ptr_reg1;
    u32* ptr_reg2;
    s32 retflag=PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s( chipId=%d, portId=%d, queueId=%d )\n",
           __FUNCTION__, chipId, portId, queueId);    
#endif
#ifdef  PARAMETER_CHECK
    
    if(portId >= TOTAL_PORTS)
        return PR_ERR_PORTID;

    if(queueId >= TOTAL_EGRESS_QUEUES)
        return PR_ERR_QUEUEID;
    
    if(waterMarkPtr == NULL)
        return PR_ERR_NULL_PTR;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif

    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PORT_TX_WM_REGS) + (portId<<1);
    ptr_reg2 = (u32*)(Chip_Base_Address[chipId] + PORT_TX_WM_REGS) + ((portId<<1) + 1);
    
    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_port_tx_wm0 = (port_tx_wm_reg0_t*)&temp1;

    PR_Reg_Read(ptr_reg2,temp2);
    ptr_port_tx_wm1 = (port_tx_wm_reg1_t*)&temp2;

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;
    
    switch(queueId)
    {
        case QUEUE_0:
            *waterMarkPtr = ptr_port_tx_wm0->Queue0_tx_wm ;
            break;

        case QUEUE_1:
            *waterMarkPtr = ptr_port_tx_wm0->Queue1_tx_wm ;
            break;
    
        case QUEUE_2:
            *waterMarkPtr = ptr_port_tx_wm1->Queue2_tx_wm ;
            break;

        case QUEUE_3:
            *waterMarkPtr = ptr_port_tx_wm1->Queue3_tx_wm ;
            break;

        default:
            return PR_ERR_QUEUEID;
    }

    return PR_ERR_NONE;
}



/**
 * Set Pause Frame Generation.
 * Enable or disable Pause Frame Generation for selected port by configuring 
 * PAUSE_CTL register.
 *
 * \param   chipId              Chip Id
 * \param   portId              Value of the port Id (0 - 1)
 * \param   en_dis              Enable (1) or disable (0) pause frame generation
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PORTID       - If the portId is not in the range of 0-2
 * \return  PR_ERR_NULL_PTR     - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_set_pause_frame_gen( u8 chipId, u32 portId, BOOL en_dis)
{
    port_pause_ctl_reg_t*   ptr_pfg;
    u32 temp1=0;
    u32* ptr_reg1;
    s32 retflag=PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_SWITCH	  
    printk("%s( portId=%d, en_dis=%d ) \n", 
           __FUNCTION__, portId, en_dis);        
#endif
#ifdef PARAMETER_CHECK

    if(portId >= (TOTAL_PORTS - 1))
        return PR_ERR_PORTID;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;
    
    if((en_dis != TRUE) && (en_dis != FALSE))
        return PR_ERR_PAUSE_FRAME_GEN_INVALID;    

#endif
    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PAUSE_CTL);

    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_pfg = (port_pause_ctl_reg_t*)&temp1;
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s - read 0x%08X from PAUSE_CTL_REG\n", __FUNCTION__, temp1);  
#endif    
    if(portId==INCAIP2_SWITCH_PORT_SELECT_LAN)
    {
        if( en_dis )
        {
            ptr_pfg->pfg_lan = 1;
        }
        else
        {
            ptr_pfg->pfg_lan = 0;
        }
    }
    else
    {
        if( en_dis )
        {
            ptr_pfg->pfg_pc = 1;
        }
        else
        {
            ptr_pfg->pfg_pc = 0;
        }        
    }        

    PR_Reg_Write(ptr_reg1,temp1);
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s - wrote 0x%08X to PAUSE_CTL_REG\n", __FUNCTION__, temp1);  
#endif  
    
    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

    return PR_ERR_NONE;
}

/**
 * Get Pause Frame Generation.
 * Get the Pause Frame Generation status for selected port from PAUSE_CTL register.
 *
 * \param   chipId              Chip Id
 * \param   portId              Value of the port Id (0 - 1)
 * \param   en_dis              pointer to pause frame generation information
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PORTID       - If the portId is not in the range of 0-2
 * \return  PR_ERR_NULL_PTR     - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_get_pause_frame_gen( u8 chipId, u32 portId, BOOL *en_dis)
{
    port_pause_ctl_reg_t*   ptr_pfg;
    u32 temp1=0;
    u32* ptr_reg1;
    s32 retflag=PR_ERR_NONE;
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s( portId=%d ) \n", __FUNCTION__, portId); 
#endif      

#ifdef PARAMETER_CHECK

    if(portId >= (TOTAL_PORTS - 1))
        return PR_ERR_PORTID;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;
    
    if(en_dis == NULL)
        return PR_ERR_NULL_PTR;    

#endif
    ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + PAUSE_CTL);

    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_pfg = (port_pause_ctl_reg_t*)&temp1;

    if(portId==INCAIP2_SWITCH_PORT_SELECT_LAN)
    {
        *en_dis = ptr_pfg->pfg_lan;
     }
    else
    {
        *en_dis = ptr_pfg->pfg_pc;        
    }        

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;

    return PR_ERR_NONE;
}




/**
 * Set Tx config.
 * Set the TX configuration.
 *
 * \param   chipId              Chip Id
 * \param   portId              Value of the port Id (0 - 2)
 * \param   xmitDropCount       Number of retransmission attempts before the 
 *                              packet is discarded
 * \param   lateCollisionCount  Threshold of decision between Early and Late 
 *                              collision
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PORTID       - If the portId is not in the range of 0-2
 * \return  PR_ERR_NULL_PTR     - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_set_Tx_config(u8 chipId,u32 portId,u32 xmitDropCount, u32 lateCollisionCount)
{
    tx_config_reg_t*   ptr_tx_config;
    u32 temp1=0;
    u32*                ptr_reg1 = NULL;
    s32                 retflag=PR_ERR_NONE;

#ifdef INCAIP2_DEBUG_SWITCH	 
    printk("%s( portId=%d, xmitDropCount=%d, lateCollisionCount=%d ) \n",
        __FUNCTION__, portId, xmitDropCount, lateCollisionCount);        
#endif

#ifdef PARAMETER_CHECK
    if(portId > (TOTAL_PORTS - 1))
        return PR_ERR_PORTID;
    if(xmitDropCount >= RE_TRANSMIT_CNT)
        return PR_ERR_TX_CONFIG;
    if(lateCollisionCount >= LATE_COLLISION_CNT)
        return PR_ERR_TX_CONFIG;
    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;
    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID)
        return PR_ERR_NULL_PTR;
#endif

    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P0_TX_CONF);
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P1_TX_CONF);
            break;
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P2_TX_CONF);
            break;            
    }    

    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_tx_config = (tx_config_reg_t*)&temp1;


    ptr_tx_config->Attempt_limit = xmitDropCount ;
    ptr_tx_config->late_coll_thresh = lateCollisionCount;

    PR_Reg_Write(ptr_reg1,temp1);
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s - write 0x%08X to SWITCH_TX_CONF at address 0x%08X \n",
        __FUNCTION__, temp1, (u32)ptr_reg1);          
#endif        
 
	retflag = ifx_sem_give(chipId,SEM_GENRL);
        
	return retflag;

}

/**
 * Get Tx config.
 * Get the TX configuration.
 *
 * \param   chipId                Chip Id
 * \param   portId                Value of the port Id (0 - 2)
 * \param   xmitDropCountPtr      Pointer to Number of retransmission attempts 
 *                                before the packet is discarded
 * \param   lateCollisionCountPtr Pointer to Threshold of decision between Early
 *                                and Late collision
 * \return  PR_ERR_NONE           - Success 
 * \return  PR_ERR_PORTID         - If the portId is not in the range of 0-2
 * \return  PR_ERR_NULL_PTR       - xmitDropCountPtr or base address is null Pointer
 * \return  PR_ERR_CHIP_ID        - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_get_Tx_config(u8 chipId,u32 portId,u32 *xmitDropCountPtr,
                      u32 * lateCollisionCountPtr)
{
    tx_config_reg_t*   ptr_tx_config;
    u32                 temp1=0;
    u32*                ptr_reg1 = NULL;

#ifdef PARAMETER_CHECK

    if(portId > (TOTAL_PORTS - 1))
        return PR_ERR_PORTID;
    if(xmitDropCountPtr == NULL)
        return PR_ERR_NULL_PTR;
    if(lateCollisionCountPtr == NULL)
        return PR_ERR_NULL_PTR;
    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;
#endif
    switch(portId)
    {
        case INCAIP2_SWITCH_PORT_SELECT_LAN:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P0_TX_CONF);
            break;
        case INCAIP2_SWITCH_PORT_SELECT_PC:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P1_TX_CONF);
            break;
        case INCAIP2_SWITCH_PORT_SELECT_CPU:
            ptr_reg1 = (u32*)(Chip_Base_Address[chipId] + SWITCH_P2_TX_CONF);
            break;            
    }    

    PR_Reg_Read(ptr_reg1,temp1);
    ptr_tx_config = (tx_config_reg_t*)&temp1;
#ifdef INCAIP2_DEBUG_SWITCH	
    printk("%s - read 0x%08X from SWITCH_TX_CONF at address 0x%08X \n",
        __FUNCTION__, temp1, (u32)ptr_reg1);      
#endif
    *xmitDropCountPtr = ptr_tx_config->Attempt_limit ;
    *lateCollisionCountPtr = ptr_tx_config->late_coll_thresh;

    return PR_ERR_NONE;
}

/**
 * Enable port rate shaping.
 * Set the Port rate shaping enable flag.If this flag is set then Port based 
 * rate shaping is enabled otherwise Queue based rate shaping is enabled.
 *
 * \param   chipId              Chip Id
 * \param   en_dis              Enable or disable port rate shaping
 * \param   PRE                 Port rate shaping enable flag
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_PRE_INVALID  - If the Port rate shaping enable flag is invalid
 * \return  PR_ERR_NULL_PTR     - Base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_set_PORT_rate_shape( u8 chipId, BOOL en_dis, BOOL PRE )
{
    u32 Temp=0;
    u32* ptr_stack_port;
    s32 retflag=PR_ERR_NONE;

#ifdef PARAMETER_CHECK

    if((PRE != TRUE) && (PRE != FALSE))
        return PR_ERR_PRE_INVALID;

    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif  

    ptr_stack_port = (u32*) (Chip_Base_Address[chipId] + STACK_PORT_REG);


    /* 
     * read STACK_REG content clear rate shape enable and rate shape mode bits
     * and overwrite with  passed values  
     */
    if((retflag = ifx_sem_take(chipId,SEM_GENRL)) != PR_ERR_NONE)
            return retflag;

    PR_Reg_Read(ptr_stack_port,Temp);
    Temp &= PORT_PRE_MASK & PORT_RATE_EN_MASK;
    Temp |= (PRE << PORT_PRE_POS) | (en_dis<<PORT_RATE_EN_POS); 
    PR_Reg_Write(ptr_stack_port,Temp);

    if((retflag = ifx_sem_give(chipId,SEM_GENRL)) != PR_ERR_NONE)
        return retflag;
    
    return PR_ERR_NONE ;

}

/**
 * Get port rate shaping flags.
 * Get the Port rate shaping enable flag and the rate shaping mode flag.
 *
 * \param   chipId              Chip Id
 * \param   PREPtr              Pointer to Port rate shaping enable flag
 * \param   en_dis_ptr          Enabled or disabled bool pointer
 * \return  PR_ERR_NONE         - Success 
 * \return  PR_ERR_NULL_PTR     - PREPtr or base address is null Pointer
 * \return  PR_ERR_CHIP_ID      - If the chip Id invalid
 * \ingroup Internal
 */ 
s32 ifx_get_PORT_rate_shape( u8 chipId, BOOL* en_dis_ptr, BOOL* PREPtr)
{   

    volatile u32* ptr_stack_port;
    volatile u32 Temp=0;

#ifdef PARAMETER_CHECK
    
    if(PREPtr == NULL)
        return PR_ERR_NULL_PTR;
    
    if(chipId >= Total_Chips)
        return PR_ERR_CHIP_ID ;

    if(Chip_Base_Address[chipId] == CHIP_ADDRESS_INVALID) 
        return PR_ERR_NULL_PTR;

#endif

    ptr_stack_port = (u32*) (Chip_Base_Address[chipId] + STACK_PORT_REG);

    /*
     * Read STACK_REG content and extract rate shaping enable and 
     * rate shaping mode flag (port or queue based) 
     */
    PR_Reg_Read(ptr_stack_port,Temp);
    *PREPtr = (Temp & (~PORT_PRE_MASK)) !=0 ? TRUE:FALSE;
    *en_dis_ptr = (Temp & (~PORT_RATE_EN_MASK)) !=0 ? TRUE:FALSE;

    return PR_ERR_NONE ;
    
}



/**************************************** Internal API Definition *************************************************/


/*****************************************************************************
 * 
 * FUNCTION:    ifx_get_chip_minor_version
 * 
 * DESCRIPTION: get the chip minor version number
 *              
 * PARAMETERS:
 *      IN: 
 *            ChipId        ChipId.
 *      OUT:
 *             minr                 
 *
 * 
 * RETURNS:
 *      PR_ERR_NONE             Success
 *       
 *************************************************************************/

s32 ifx_get_chip_minor_version(u8 chipId,u32 *minr)
{
    *minr=PR_CHIP_MINR_VER_ID[chipId];
    return PR_ERR_NONE ;
}
/*****************************************************************************
 * 
 * FUNCTION:    ifx_get_chip_major_version
 * 
 * DESCRIPTION: get the chip major version number
 *              
 * PARAMETERS:
 *      IN:
 *              ChipId      ChipId.
 *      OUT:
 *              majr
 *                  
 * 
 * RETURNS:
 *      PR_ERR_NONE             Success
 *       
 *************************************************************************/

s32 ifx_get_chip_major_version(u8 chipId,u32 *majr)
{
    *majr=PR_CHIP_MAJR_VER_ID[chipId];
    return PR_ERR_NONE ;
}



