/******************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ******************************************************************************/
/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#define IFX_SSC_DRV_VERSION "0.2.1"

/* general issues: */
/* TODO: power management */
/* DONE: interrupt handling (direct/indirect) (test pending) */
/* DONE: pin/mux-handling (just overall concept due to project dependency) (test pending) */
/* DONE: multiple instances capability (test pending) */
/* TODO: slave functionality */

#ifndef EXPORT_SYMTAB
#define EXPORT_SYMTAB
#endif

#include <linux/config.h>
#include <linux/module.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/major.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/ptrace.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/spinlock.h>
#include <linux/slab.h>
#ifdef CHECKME_USE_POLL
#include <linux/poll.h>
#endif

#include <asm/system.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/bitops.h>

#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/version.h>

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif

#if defined(CONFIG_INCAIP2)    /* FIXME: move platform specific file */
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/irq.h>
#include <asm/incaip2/ifx_per.h>
#include <asm/incaip2/port.h>
#include <asm/incaip2/mux.h>
#include <asm/incaip2/ifx_ssc_defines.h>
#include <asm/incaip2/ifx_ssc.h>
#include <asm/incaip2/ifx_dma.h>

#define ifx_get_fpi_hz incaip2_get_fbs0clk
#define ifx_disable_irq disable_incaip2_irq
#define ifx_enable_irq enable_incaip2_irq
#define ifx_mask_and_ack_irq ack_incaip2_irq

#define IFX_SSC_MAX_PORT_NUM 2
#define IFX_SSC0_BASE INCA_IP2_SSC0_BASE
#define IFX_SSC1_BASE INCA_IP2_SSC1_BASE

#define IFX_SSC0_RIR INT_NUM_IM0_IRL14
#define IFX_SSC0_TIR INT_NUM_IM0_IRL15
#define IFX_SSC0_EIR INT_NUM_IM0_IRL16
#define IFX_SSC0_FIR INT_NUM_IM0_IRL17
//#define IFX_SSC0_FIR INT_NUM_IM3_IRL17

#define IFX_SSC1_RIR INT_NUM_IM3_IRL18
#define IFX_SSC1_TIR INT_NUM_IM3_IRL19
#define IFX_SSC1_EIR INT_NUM_IM3_IRL20
#define IFX_SSC1_FIR INT_NUM_IM3_IRL21

#else

#define IFX_GPIO_POUT INCA_IP2_GPIO_POUT
#define IFX_GPIO_PIN INCA_IP2_GPIO_PIN
#define IFX_GPIO_PDIR INCA_IP2_GPIO_PDIR
#define IFX_GPIO_ALTSEL0 INCA_IP2_GPIO_ALTSEL1
#define IFX_GPIO_ALTSEL1 INCA_IP2_GPIO_ALTSEL2
#define IFX_GPIO_PSTOFF INCA_IP2_GPIO_PSTOFF
#define IFX_GPIO_POD INCA_IP2_GPIO_POD
#define IFX_GPIO_PUDEN INCA_IP2_GPIO_PUDEN
#define IFX_GPIO_PUDSEL INCA_IP2_GPIO_PUDSEL

#endif /* CONFIG_INCAIP2 */

#define SSC_WRITE_REG(data, addr)   (*(volatile UINT32 *)(addr)) = data 
#define SSC_WRITE_REG_16(data, addr)(*(volatile UINT16 *)(addr)) = data 
#define SSC_WRITE_REG_8(data, addr) (*(volatile UINT8 *)(addr))  = data 
#define SSC_READ_REG(addr)          (*(volatile UINT32 *)(addr))

#ifdef SSC_FRAME_INT_ENABLE
#undef SSC_FRAME_INT_ENABLE
#endif
#define SSC_FRAME_INT_ENABLE

#define IFX_SSC_EMSG(fmt,arg...) printk("%s: "fmt,__FUNCTION__, ##arg)
#define DEFAULT_SSC_TX_ENDIANESS 0x03
#define DEFAULT_SSC_RX_ENDIANESS 0x03
#define SSC_DMA_BUFFER_SIZE 2048

#define ifx_port_ssc_cs_set(ssc_port,cs,master)
/*
 * Deal with CONFIG_MODVERSIONS
 */
#if CONFIG_MODVERSIONS==1
# include <linux/modversions.h>
#endif

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Infineon Technologies");
MODULE_DESCRIPTION("IFX SSC driver");
MODULE_SUPPORTED_DEVICE("ifx_ssc");
MODULE_PARM(maj, "i");
MODULE_PARM_DESC(maj, "Major device number");

/**
 * allow the user to set the major device number 
 */
static int maj = 0;

/**
 * SSC Port Structure
 * This is the per-channel data structure containing pointers, flags
 * and variables for the port. This driver supports a maximum of IFX_SSC_MAX_PORT_NUM.
 * isp is allocated in ifx_ssc_init() based on the chip version.
 */
static struct ifx_ssc_port *isp;
static int ifx_ssc_port_map=0;

u32 ifx_ssc_dma_rx=0;
u32 ifx_ssc_dma_alloc=0;
u32 ifx_ssc_dma_rx_err=0;
u32 ifx_ssc_dma_alloc_drop=0;
u32 ifx_ssc_dma_free=0;
u32 ifx_ssc_dma_tx=0;
u32 ifx_ssc_dma_tx_full=0;
u32 ifx_ssc_dma_tx_drop=0;
u32 ifx_ssc_dma_free_err=0;

/******************************************************************************
 * Function declaration
 ******************************************************************************/

extern unsigned int ifx_get_fpi_hz(void);
extern void ifx_disable_irq(unsigned int irq_nr);
extern void ifx_enable_irq(unsigned int irq_nr);
extern void ifx_mask_and_ack_irq(unsigned int irq_nr);

/******************************************************************************
 * Type/struct definitions
 ******************************************************************************/
typedef struct {
    int (*request)(unsigned int irq, 
                   void (*handler)(int, void *, struct pt_regs *), 
                   unsigned long irqflags,
                   const char * devname, 
                   void *dev_id);
    void (*free)(unsigned int irq, void *dev_id);
    void (*enable)(unsigned int irq);
    void (*disable)(unsigned int irq);
    void (*clear)(unsigned int irq);
} ifx_int_wrapper_t;

static ifx_int_wrapper_t ifx_int_wrapper = {
    request:  request_irq, 
    free:     free_irq,
    enable:   enable_irq,
    disable:  disable_irq,
    clear:    ifx_mask_and_ack_irq,
    /* end: */
};

typedef struct {
    u32 base_address;
    int rx_int;
    int tx_int;
    int err_int;
    int frm_int;
} ifx_ssc_port_specific_data_t ;

#if defined(CONFIG_INCAIP2)
static ifx_ssc_port_specific_data_t ifx_ssc_port[IFX_SSC_MAX_PORT_NUM] = { 
    { IFX_SSC0_BASE, IFX_SSC0_RIR, IFX_SSC0_TIR, IFX_SSC0_EIR, IFX_SSC0_FIR },
    { IFX_SSC1_BASE, IFX_SSC1_RIR, IFX_SSC1_TIR, IFX_SSC1_EIR, IFX_SSC1_FIR }
    };
#else
#error Unsupported platform! Please add specific data for this platform.
#endif

/**
* wait queue for going to sleep
*/
wait_queue_head_t wait_queue_sidecar_fastmode_finish;

/**
* sidecar_ifx_wait_sleep
* function (wrapper) for going to sleep (leave CPU) to wait for end of dma data transfer
*/
void sidecar_ifx_wait_sleep(void)
{
//    printk("going to sleep...\n");
    interruptible_sleep_on(&wait_queue_sidecar_fastmode_finish);
}

/******************************************************************************
 * Function implementations
 ******************************************************************************/

/**
 * Get SSC clock speed.
 * Return the current operating speed of the SSC peripheral, depending on chip
 * specific bus speed and RMC setting in CLC register.
 * 
 * \param  info Pointer to structure #ifx_ssc_port
 * \return  >0  Peripheral speed in HZ
 * \return  0   Error
 * \ingroup Internal
 */
static inline unsigned int ifx_ssc_get_kernel_clk(struct ifx_ssc_port *info)
{ 
    unsigned int rmc;

    rmc = (SSC_READ_REG(info->mapbase + IFX_SSC_CLC) & 
           IFX_CLC_RUN_DIVIDER_MASK) >> IFX_CLC_RUN_DIVIDER_OFFSET;
    if (rmc == 0){
        printk("ifx_ssc_get_kernel_clk rmc==0 \n");
        return (0);
    }
    return (ifx_get_fpi_hz() / rmc);
}


/**
 * Pseudo Interrupt handler for DMA.
 * This function processes DMA interrupts notified to the SSC device driver.
 * It is installed at the DMA core as interrupt handler for the SSC DMA device 
 * and handles the following DMA interrupts:
 * - Passes received data to the upper layer in case of rx interrupt,  
 * - In case of a DMA receive interrupt the received data is passed to the upper layer.
 * - In case of a transmit buffer full interrupt the transmit queue is stopped. 
 * - In case of a transmission complete interrupt the transmit queue is restarted.
 * 
 * \param   dma_dev pointer to DMA device structure
 * \param   status  type of interrupt being notified (RCV_INT: DMA receive 
 *                  interrupt, TX_BUF_FULL_INT: transmit buffer full interrupt, 
 *                  TRANSMIT_CPT_INT: transmission complete interrupt)
 * \return  OK      In case of successful data reception from DMA
 * \ingroup Internal
 */
int ifx_ssc_dma_int_handler( struct dma_device_info* dma_dev, int status )
{
    u32 flags;
    char* buf=NULL;
    struct ifx_ssc_port *info=(struct ifx_ssc_port *)dma_dev->priv;
    int len;
    
    switch(status)
    {
        case RCV_INT:        
            ifx_ssc_dma_rx++;
            save_and_cli(flags);
            /* read data from DMA device */
            len = dma_device_read( dma_dev, (u8**) &buf, NULL );
            restore_flags(flags);
            if (buf != NULL) {
                if ( (info != NULL) && (info->rxbuf != NULL) ) {
                    memcpy(info->rxbuf,buf,len);
                }
                kfree(buf);
            }
           /* wake up any processes waiting in read() */
            wake_up_interruptible(&info->rwait);
            break;  	  
        case TX_BUF_FULL_INT:
            ifx_ssc_dma_tx_full++;
            printk("%s - tx buffer full\n", __FUNCTION__);
            break;
        case TRANSMIT_CPT_INT:
	    ifx_ssc_dma_tx++;
//sidecar	    
	    if(info->port_nr==1){
//	    	printk("SSC DMA TX done\n");
		wake_up_interruptible(&wait_queue_sidecar_fastmode_finish);	
	    }
//end 	    	    
//            info->stats.txBytes += 1;
            if (info->txbuf != NULL) {
                save_and_cli(flags);
                kfree(info->txbuf);
                info->txbuf=NULL;
                restore_flags(flags);
            }
            break;
    }
    return 0;
}


/**
 * Allocates buffer for SSC buffer.
 * This function is installed as DMA callback function to be called on DMA 
 * receive interrupt.
 * 
 * \param   len          Length of packet
 * \param   *byte_offset Pointer to byte offset
 * \param   **opt        unused  
 * \return  NULL         In case of buffer allocation fails
 *          buffer       Pointer to allocated memory
 * \ingroup Internal
 */
u8* ifx_ssc_dma_buffer_alloc(int len, int* byte_offset,void** opt)
{
    u8* buffer=NULL;
    
    buffer=kmalloc(len,GFP_ATOMIC);
    *byte_offset=0;
    ifx_ssc_dma_alloc++;
    return buffer;
}

/**
 * Free DMA buffer.
 * This function frees a buffer previously allocated. 
 * 
 * \param   *dataptr Pointer to data buffer
 * \param   *opt     unused  
 * \return  0        OK
 * \ingroup Internal
 */
int ifx_ssc_dma_buffer_free(u8* dataptr,void* opt)
{
    if (dataptr == NULL) {
        ifx_ssc_dma_free_err++;
    }
    else {
        ifx_ssc_dma_alloc--;
        kfree(dataptr);
    }
    return 0;
} 

/**
 * Initialize SSC DMA device.
 * This function initializes the passed DMA device structure for usage as 
 * SSC DMA device. In advance the memory upper layer module has to be 
 * registered at the DMA core. 
 * 
 * \param   *dma_dev Pointer to dma device structure to be initialized
 * \return  0        OK
 * \ingroup Internal
 */
int ifx_ssc_init_dma_device(int line, struct dma_device_info *dma_dev)
{
    int i;

    /* 
     * configure switch DMA device acc. to default values in incaip2_dma.h 
     */
    dma_dev->priv = &isp[line];
    dma_dev->tx_weight=DEFAULT_SSC_PORT_WEIGHT; 
    dma_dev->num_tx_chan=1;
    dma_dev->num_rx_chan=1; 
    dma_dev->tx_burst_len=DEFAULT_SSC_TX_BURST_LEN;
    dma_dev->rx_burst_len=DEFAULT_SSC_RX_BURST_LEN;
    dma_dev->tx_endianness_mode = DEFAULT_SSC_TX_ENDIANESS;
    dma_dev->rx_endianness_mode = DEFAULT_SSC_RX_ENDIANESS;
    for(i=0;i<dma_dev->num_tx_chan;i++)  /* set tx channel parameters */
    {
        dma_dev->tx_chan[i]->tx_weight=DEFAULT_SSC_CHANNEL_WEIGHT; 
        dma_dev->tx_chan[i]->desc_len=DEFAULT_SSC_TX_CHANNEL_DESCR_NUM;  
        dma_dev->tx_chan[i]->packet_size=SSC_DMA_BUFFER_SIZE;

    }
    for(i=0;i<dma_dev->num_rx_chan;i++)   /* set rx channel parameters */
    {
        dma_dev->rx_chan[i]->tx_weight=DEFAULT_SSC_CHANNEL_WEIGHT;
        dma_dev->rx_chan[i]->desc_len=DEFAULT_SSC_RX_CHANNEL_DESCR_NUM;
        dma_dev->rx_chan[i]->packet_size=SSC_DMA_BUFFER_SIZE;

    }
    /* 
     * set DMA handler functions for rx-interrupts, 
     * buffer allocation and release
     */
    dma_dev->intr_handler=ifx_ssc_dma_int_handler;
    dma_dev->buffer_alloc=ifx_ssc_dma_buffer_alloc;
    dma_dev->buffer_free=ifx_ssc_dma_buffer_free;
    return 0;
}


/**
 * Receive Interrupt Handler.
 *
 * \param   info Pointer to structure #ifx_ssc_port
 * \return  none
 * \ingroup Internal
 */
inline static void rx_int(struct ifx_ssc_port *info)
{
    int     fifo_fill_lev, bytes_in_buf, i;
    unsigned long   tmp_val;
    unsigned long   *tmp_ptr;
    unsigned int    rx_valid_cnt;
    u32 flags;
    
    /* number of words waiting in the RX FIFO */
    fifo_fill_lev = (SSC_READ_REG(info->mapbase + IFX_SSC_FSTAT) & 
                         IFX_SSC_FSTAT_RECEIVED_WORDS_MASK) >>
                         IFX_SSC_FSTAT_RECEIVED_WORDS_OFFSET;
    /* Note: There are always 32 bits in a fifo-entry except for the last 
       word of a contigous transfer block and except for not in rx-only 
       mode and CON.ENBV set. But for this case it should be a convention 
       in software which helps:
       In tx or rx/tx mode all transfers from the buffer to the FIFO are 
       32-bit wide, except for the last three bytes, which could be a 
       combination of 16- and 8-bit access.
       => The whole block is received as 32-bit words as a contigous stream, 
       even if there was a gap in tx which has the fifo run out of data! 
       Just the last fifo entry *may* be partially filled (0, 1, 2 or 3 bytes)!
    */

    /* free space in the RX buffer */
    bytes_in_buf = info->rxbuf_end - info->rxbuf_ptr;
    /* transfer with 32 bits per entry */
//    save_and_cli(flags);
//    while ((bytes_in_buf >= 4) && (fifo_fill_lev > 0)) {
//        tmp_ptr = (unsigned long *)info->rxbuf_ptr;
//        *tmp_ptr = SSC_READ_REG(info->mapbase + IFX_SSC_RB);
//#if defined (__MIPSEB__)
//        *tmp_ptr = ___swab32(*tmp_ptr);
//#endif
//        info->rxbuf_ptr += 4;
//        info->stats.rxBytes += 4;
//        fifo_fill_lev --;
//        bytes_in_buf -= 4;
//    }
//    restore_flags(flags);
    /* now do the rest as mentioned in STATE.RXBV */
    while ((bytes_in_buf > 0) && (fifo_fill_lev > 0)) {
        rx_valid_cnt = (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) & 
                        IFX_SSC_STATE_RX_BYTE_VALID_MASK) >> 
                        IFX_SSC_STATE_RX_BYTE_VALID_OFFSET;
        if (rx_valid_cnt == 0) break;
        if (rx_valid_cnt > bytes_in_buf) {
            /* TODO: warning message: not block aligned data, other data 
               in this entry will be lost */
            rx_valid_cnt = bytes_in_buf;
        }
        tmp_val = SSC_READ_REG(info->mapbase + IFX_SSC_RB);
        
        save_and_cli(flags);
        for (i=0; i<rx_valid_cnt; i++) {        
            *info->rxbuf_ptr = (tmp_val >> ( 8 * (rx_valid_cnt - i-1))) & 0xff;
            bytes_in_buf--;
            info->rxbuf_ptr++;
        }
        restore_flags(flags);
        info->stats.rxBytes += rx_valid_cnt;
    }

    /* check if transfer is complete */
    if (info->rxbuf_ptr >= info->rxbuf_end) {
        /* wakeup any processes waiting in read() */
        wake_up_interruptible(&info->rwait);
#ifdef CHECKME_USE_POLL             
        /* and in poll() */
        wake_up_interruptible(&info->pwait);
#endif                
    } else if ((info->opts.modeRxTx == IFX_SSC_MODE_RX) && 
               (SSC_READ_REG(info->mapbase + IFX_SSC_RXCNT) == 0)) {
        /* if buffer not filled completely and rx request done initiate new 
           transfer */
        if  (info->rxbuf_end - info->rxbuf_ptr < IFX_SSC_RXREQ_BLOCK_SIZE)
            SSC_WRITE_REG((info->rxbuf_end - info->rxbuf_ptr)
                          << IFX_SSC_RXREQ_RXCOUNT_OFFSET, 
                          info->mapbase + IFX_SSC_RXREQ);
        else 
            SSC_WRITE_REG(IFX_SSC_RXREQ_BLOCK_SIZE 
                          << IFX_SSC_RXREQ_RXCOUNT_OFFSET, 
                          info->mapbase + IFX_SSC_RXREQ);
    }
}

/**
 * Transmit Interrupt Handler.
 *
 * \param   info Pointer to structure #ifx_ssc_port
 * \return  none
 * \ingroup Internal
 */
inline static void tx_int(struct ifx_ssc_port *info)
{
    int fifo_space, fill, i;
    u32 flags;
    
    fifo_space = ((SSC_READ_REG(info->mapbase + IFX_SSC_ID) & IFX_SSC_PERID_TXFS_MASK) >> IFX_SSC_PERID_TXFS_OFFSET) - 
                 ((SSC_READ_REG(info->mapbase + IFX_SSC_FSTAT) & IFX_SSC_FSTAT_TRANSMIT_WORDS_MASK) >> 
                   IFX_SSC_FSTAT_TRANSMIT_WORDS_OFFSET);

    if (fifo_space == 0)
        return;

    if (info->txbuf_ptr >= info->txbuf_end) {
        kfree(info->txbuf);
        info->txbuf = NULL;
#ifdef CHECKME_USE_POLL
        /* wake up any process waiting in poll() */
        wake_up_interruptible(&info->pwait);
#endif                
        return;
    }   

    if (info->opts.dataWidth == 32) {
        fifo_space = fifo_space < 2;
    }
    if (info->opts.dataWidth == 16) {
        fifo_space = fifo_space < 1;
    }
    else if (info->opts.dataWidth == 8) {
        fifo_space = fifo_space;
    }

    fill = info->txbuf_end - info->txbuf_ptr;
    if (fill < 0) printk("TRACE %s %s %d txbuf_end=0x%08X txbuf_ptr=0x%08X\n",__FILE__,__FUNCTION__,__LINE__,info->txbuf_end,info->txbuf_ptr);

    if (fill > fifo_space)
        fill = fifo_space;

    if (info->opts.dataWidth == 32) {
        for (i = 0; i < fill / 4; i++) {
            /* at first 32 bit access */
            SSC_WRITE_REG(*(UINT32 *)info->txbuf_ptr, info->mapbase + IFX_SSC_TB);
            info->txbuf_ptr += 4;
        }
        fifo_space -= fill;
        info->stats.txBytes += fill & ~0x3;
        fill &= 0x3;
        if ((fifo_space > 0) & (fill > 1)) {
           /* trailing 16 bit access */
            SSC_WRITE_REG_16(*(UINT16 *)info->txbuf_ptr, info->mapbase + IFX_SSC_TB);
            info->txbuf_ptr += 2;
            info->stats.txBytes += 2;
            fifo_space -=2;
            fill -=2;
        }

        if ((fifo_space > 0) & (fill > 0)) {
            /* trailing 8 bit access */
            SSC_WRITE_REG_8(*(UINT8 *)info->txbuf_ptr, info->mapbase + IFX_SSC_TB);
            info->txbuf_ptr ++;
            info->stats.txBytes ++;
            fifo_space --;
    
        }
    }
    else if (info->opts.dataWidth == 16) {
        for (i = 0; i < fill / 2; i++) {
            /* at first 16 bit access */
            SSC_WRITE_REG_16(*(UINT16 *)info->txbuf_ptr, info->mapbase + IFX_SSC_TB);
            info->txbuf_ptr += 2;
        }
        fifo_space -= fill;
        info->stats.txBytes += fill & ~0x1;
        fill &= 0x1;
        if ((fifo_space > 0) & (fill > 0)) {
            /* trailing 8 bit access */
            SSC_WRITE_REG_8(*(UINT8 *)info->txbuf_ptr, info->mapbase + IFX_SSC_TB);
            info->txbuf_ptr ++;
            info->stats.txBytes ++;
            fifo_space --;
        }
    }
    else if (info->opts.dataWidth == 8) {
        save_and_cli(flags);
        for (i = 0; (i < fill) && (info->txbuf_ptr < info->txbuf_end) ; i++) {
            /* 8 bit access */
            *(((UINT8*)info->mapbase + IFX_SSC_TB)+3)=*(UINT8 *)info->txbuf_ptr;
            info->txbuf_ptr += 1;
        }
        restore_flags(flags);
        fifo_space -= i;
        info->stats.txBytes += i;
    }

    if (info->opts.modeRxTx == IFX_SSC_MODE_RXTX) {
        rx_int(info);
    }
    
    /* check if transmission is complete */
    if (info->txbuf_ptr >= info->txbuf_end) {
        kfree(info->txbuf);
        info->txbuf = NULL;
#ifdef CHECKME_USE_POLL
        /* wake up any process waiting in poll() */
        wake_up_interruptible(&info->pwait);
#endif                
    }   

}

/**
 * RX Interrupt Handler.
 *
 * \param  irq    Interrupt number
 * \param  dev_id Pointer to info structure
 * \param  regs   Current MIPS register context
 * \return none
 * \ingroup Internal
 */
static void ifx_ssc_rx_int(int irq, void *dev_id, struct pt_regs *regs)
{
    struct ifx_ssc_port *info = (struct ifx_ssc_port *)dev_id;

    rx_int(info);
}

/**
 * TX Interrupt Handler.
 *
 * \param  irq    Interrupt number
 * \param  dev_id Pointer to info structure
 * \param  regs   Current MIPS register context
 * \return none
 * \ingroup Internal
 */
static void ifx_ssc_tx_int(int irq, void *dev_id, struct pt_regs *regs)
{
    struct ifx_ssc_port *info = (struct ifx_ssc_port *)dev_id;

    tx_int(info);
}

/**
 * SSC ERR Interrupt Handler.
 *
 * \param  irq    Interrupt number
 * \param  dev_id Pointer to info structure
 * \param  regs   Current MIPS register context
 * \return none
 * \ingroup Internal
 */
static void ifx_ssc_err_int(int irq, void *dev_id, struct pt_regs *regs)
{
    struct ifx_ssc_port *info = (struct ifx_ssc_port *)dev_id;
    unsigned int state;
    unsigned int write_back = 0;
    unsigned long flags;

    local_irq_save(flags);
    state = SSC_READ_REG(info->mapbase + IFX_SSC_STATE); 
    
    if (((state & (IFX_SSC_STATE_RX_UFL | IFX_SSC_STATE_RX_OFL | 
                IFX_SSC_STATE_TX_OFL | IFX_SSC_STATE_TX_UFL | 
                IFX_SSC_STATE_ABORT_ERR | IFX_SSC_STATE_MODE_ERR)) == 0) &&
        (regs != NULL) ) {
        info->stats.unknownErr++;
        local_irq_restore(flags);
        return;
    }

    if ((state & IFX_SSC_STATE_RX_UFL) != 0) {
        info->stats.rxUnErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_RX_UFL_ERROR;
    }
    if ((state & IFX_SSC_STATE_RX_OFL) != 0) {
        info->stats.rxOvErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_RX_OFL_ERROR;
    }
    if ((state & IFX_SSC_STATE_TX_OFL) != 0) {
        info->stats.txOvErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_TX_OFL_ERROR;
    }
    if ((state & IFX_SSC_STATE_TX_UFL) != 0) {
        info->stats.txUnErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_TX_UFL_ERROR;
    }
    if ((state & IFX_SSC_STATE_ABORT_ERR) != 0) {
        info->stats.abortErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_ABORT_ERROR;
    }
    if ((state & IFX_SSC_STATE_MODE_ERR) != 0) {
        info->stats.modeErr++;
        write_back |= IFX_SSC_WHBSTATE_CLR_MODE_ERROR;
    }

    
    if (write_back)
        SSC_WRITE_REG(write_back, 
                      info->mapbase + IFX_SSC_WHBSTATE);

    local_irq_restore(flags);
}

#ifdef SSC_FRAME_INT_ENABLE
/**
 * SSC Frame Interrupt Handler.
 *
 * \param  irq    Interrupt number
 * \param  dev_id Pointer to info structure
 * \param  regs   Current MIPS register context
 * \return none
 * \ingroup Internal
 */
static void ifx_ssc_frm_int(int irq, void *dev_id, struct pt_regs *regs)
{
    u32 tmp;
    struct ifx_ssc_port *info = (struct ifx_ssc_port *)dev_id;

    /* ack interrupt in SSC block */
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_IRN_CR);
    tmp |= IFX_SSC_F_BIT;
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_IRN_CR);
    /* Clear enable bit */
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON);
    tmp &= ~(IFX_SSC_SFCON_SF_ENABLE);
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_SFCON);
    ifx_int_wrapper.clear(info->frmirq);
    ifx_int_wrapper.disable(info->frmirq);
    if (info->frm_callback.IntCallback != NULL)
        info->frm_callback.IntCallback(info->port_nr);
    
//printk("TRACE %s %s %d info=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,info);
}
#endif

/**
 * SSC Abort.
 * Abort current data transfer on specified SSC port. Will disable/clear interrupts
 * and flush FIFOs in Tx and Rx direction.
 *
 * \param   info Pointer to structure #ifx_ssc_port
 * \return  none
 * \ingroup Internal
 */
static void ifx_ssc_abort(struct ifx_ssc_port *info)
{
    unsigned long flags;
    bool enabled;
    u32 reg;
    
    local_irq_save(flags);

    /* disable all int's */
    ifx_int_wrapper.disable(info->rxirq);
    ifx_int_wrapper.disable(info->txirq);
#ifdef SSC_FRAME_INT_ENABLE
//    ifx_int_wrapper.disable(info->frmirq);
#endif
    local_irq_restore(flags);

    /* Handle any pending errors */
    ifx_ssc_err_int(0,info,NULL);
    

    /* disable SSC (also aborts a receive request!) */
    /* TODO: Perhaps it's better to abort after the receiption of a 
       complete word. The disable cuts the transmission immediatly and 
       releases the chip selects. This could result in unpredictable 
       behavior of connected external devices!  */ 
    enabled = (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) 
           & IFX_SSC_STATE_IS_ENABLED) != 0;
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE, 
                  info->mapbase + IFX_SSC_WHBSTATE);

    /* flush fifos */
    reg=SSC_READ_REG(info->mapbase + IFX_SSC_TXFCON);
    SSC_WRITE_REG(reg | IFX_SSC_XFCON_FIFO_FLUSH, info->mapbase + IFX_SSC_TXFCON);
    reg=SSC_READ_REG(info->mapbase + IFX_SSC_RXFCON);
    SSC_WRITE_REG(reg | IFX_SSC_XFCON_FIFO_FLUSH, info->mapbase + IFX_SSC_RXFCON);

    /* free txbuf */
    if (info->txbuf != NULL) {
        kfree(info->txbuf);
        info->txbuf = NULL;
    }

    /* wakeup read process */
    if (info->rxbuf != NULL)
        wake_up_interruptible(&info->rwait);

    /* clear pending int's */
    ifx_int_wrapper.clear(info->rxirq);
    ifx_int_wrapper.clear(info->txirq);
#ifdef SSC_FRAME_INT_ENABLE
//    ifx_int_wrapper.clear(info->frmirq);
#endif
    /* clear error flags */
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ALL_ERROR,
                  info->mapbase + IFX_SSC_WHBSTATE);
#if 0 /* CHECKME */
    printk("IFX SSC%d: Transmission aborted\n", info->port_nr);
#endif    
    /* enable SSC */
    if (enabled)
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_ENABLE, 
                      info->mapbase + IFX_SSC_WHBSTATE);
}


/**
 * SSC Port Open.
 * This routine is called whenever a port is opened.  It enforces
 * exclusive opening of a port and enables interrupts, etc.
 *
 * \param   inode Inode of device or 0/1 for calls from kernel mode
 * \param   filp  File pointer of device
 * \return  0     OK
 * \return -ENXIO Invalid port number
 * \return -EBUSY Port already open
 * \ingroup API
 */
int ifx_ssc_open(struct inode *inode, struct file *filp)
{
    struct ifx_ssc_port  *info;
    int line;
    int from_kernel = 0;
    u32 tmp=0;

    if ((inode == (struct inode *)0) || (inode == (struct inode *)1)) {
        from_kernel = 1;
        line = (int)inode;
    }
    else {
        line = MINOR(filp->f_dentry->d_inode->i_rdev);
/* FIXME: Ist this really necessary? We provided the structure during registration */
//        filp->f_op = &ifx_ssc_fops;
    }

    /* don't open more minor devices than we can support */
    if (line < 0 || line >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    info = &isp[line];

    /* exclusive open */
    if (info->port_is_open != 0)
        return -EBUSY;
    info->port_is_open++;

    ifx_int_wrapper.enable(info->rxirq);
    ifx_int_wrapper.enable(info->txirq);
    ifx_int_wrapper.enable(info->errirq);
    
#ifdef SSC_FRAME_INT_ENABLE
//    ifx_int_wrapper.enable(info->frmirq);
#endif

    /* Flush and enable TX/RX FIFO */
    SSC_WRITE_REG((IFX_SSC_DEF_TXFIFO_FL << 
                   IFX_SSC_XFCON_ITL_OFFSET) | 
                  IFX_SSC_XFCON_FIFO_FLUSH   |
                  IFX_SSC_XFCON_FIFO_ENABLE, 
                  info->mapbase + IFX_SSC_TXFCON);
    SSC_WRITE_REG((IFX_SSC_DEF_RXFIFO_FL << 
                   IFX_SSC_XFCON_ITL_OFFSET) |
                  IFX_SSC_XFCON_FIFO_FLUSH   | 
                  IFX_SSC_XFCON_FIFO_ENABLE, 
                  info->mapbase + IFX_SSC_RXFCON);

    /* logically flush the software FIFOs */
    info->rxbuf_ptr = 0;
    info->txbuf_ptr = 0;

    /* DMA mode must be enabled explicitly via IOCTL */
    info->opts.dmamode = 0;
    tmp=SSC_READ_REG(info->mapbase + IFX_SSC_IRN_EN);
    SSC_WRITE_REG(tmp | IFX_SSC_DEF_IRNEN, info->mapbase + IFX_SSC_IRN_EN);

    /* Disable DMA for TX and Rx */
    tmp=SSC_READ_REG(info->mapbase + IFX_SSC_DMACON);
    tmp &= ~IFX_SSC_DMACON_TXON;
    tmp &= ~IFX_SSC_DMACON_RXON;
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_DMACON);

    /* clear all error bits */
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ALL_ERROR,
                  info->mapbase + IFX_SSC_WHBSTATE);

    /* clear pending data interrupts */
    ifx_int_wrapper.clear(info->rxirq);
    ifx_int_wrapper.clear(info->txirq);


#ifdef SSC_FRAME_INT_ENABLE
    ifx_int_wrapper.clear(info->frmirq);
#endif

	if( from_kernel )
	{    
		ifx_int_wrapper.disable(info->rxirq);
		ifx_int_wrapper.disable(info->txirq);
	}
    /* enable SSC */
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_ENABLE, 
                  info->mapbase + IFX_SSC_WHBSTATE);

    MOD_INC_USE_COUNT;

    return 0;
}

/**
 * SSC Port Close.
 * This routine is called when a particular device is closed.
 *
 * \param   inode Inode of device or 0 or 1 for calls from kernel mode
 * \param   filp  File pointer of device
 * \return  0     OK
 * \return -ENXIO Invalid port number
 * \ingroup API
 */
int ifx_ssc_close(struct inode *inode, struct file *filp)
{
    struct ifx_ssc_port *info;
        int idx;

    if ((inode == (struct inode *)0) || (inode == (struct inode *)1))
        idx = (int)inode;
    else
        idx = MINOR(filp->f_dentry->d_inode->i_rdev);

    if (idx < 0 || idx >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    info = &isp[idx];
    if (!info)
        return -ENXIO;

    /* disable SSC */
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE, 
                  info->mapbase + IFX_SSC_WHBSTATE);

    /* call abort function to disable int's, flush fifos... */
    ifx_ssc_abort(info);

    info->port_is_open --;
    MOD_DEC_USE_COUNT;

    return 0;
}

/**
 * SSC Read Helper (polling mode).
 * Helper routine to handle reads from the kernel or user-space
 * info->rxbuf : never kfree and contains valid data
 * should be points to NULL after copying data !!!
 *
 * \param   info        Pointer to structure #ifx_ssc_port
 * \param   buf         Pointer to read buffer
 * \param   len         Size of read buffer
 * \param   from_kernel 0 -> user mode, 1 -> kernel mode 
 * \return  ret_val     Number of bytes read
 * \return -EFAULT      Transmission only mode
 * \ingroup Internal
 */
static ssize_t ifx_ssc_read_helper_poll(struct ifx_ssc_port *info, char *buf, 
                                        size_t len, int from_kernel)
{
    ssize_t         ret_val;
    unsigned long   flags;

    if (info->opts.modeRxTx == IFX_SSC_MODE_TX) 
        return -EFAULT;

    local_irq_save(flags);
    info->rxbuf_ptr = info->rxbuf;
    info->rxbuf_end = info->rxbuf + len;
    local_irq_restore(flags);
    /* Vinetic driver always works in IFX_SSC_MODE_RXTX
       TXRX in poll mode */
	while(info->txbuf_ptr < info->txbuf_end) {
		tx_int(info);
	}
    while (info->rxbuf_ptr < info->rxbuf_end) {
        /* This is the key point, if you don't check this condition 
           kfree (NULL) will happen because tx only need write into 
           FIFO, it's much fast than rx
           So when rx still waiting , tx already finish and release buf */  
        rx_int(info);
    };

    ret_val = info->rxbuf_ptr - info->rxbuf; 
    return (ret_val);
}

/**
 * SSC Read Helper.
 * helper routine to handle reads from the kernel or user-space
 * info->rx_buf : never kfree and contains valid data
 * should be points to NULL after copying data !!!
 *
 * \param   info        Pointer to structure #ifx_ssc_port
 * \param   buf         Pointer to read buffer
 * \param   len         Size of read buffer
 * \param   from_kernel 0 -> user mode, 1 -> kernel mode 
 * \return  ret_val     Number of bytes read
 * \return -EFAULT      Transmission only mode or len greater than 64k
 * \ingroup Internal
 */
static ssize_t ifx_ssc_read_helper(struct ifx_ssc_port *info, char *buf, 
                                   size_t len, int from_kernel)
{
    ssize_t         ret_val;
    unsigned long   flags;
    DECLARE_WAITQUEUE(wait, current);
    u32 reg=0;

    if (info->opts.modeRxTx == IFX_SSC_MODE_TX) 
        return -EFAULT;
    
    if ( info->opts.dmamode == 1) {
        /* DMA is not supported for RXTX mode */
        if (info->opts.modeRxTx == IFX_SSC_MODE_RXTX)
            return -EFAULT;
        /* 64k max size */
        if (len > 64*1024) return -EFAULT;

        info->rxbuf_ptr = info->rxbuf;
        info->rxbuf_end = info->rxbuf + len;
        SSC_WRITE_REG(len, info->mapbase + IFX_SSC_RXREQ);
        
        /* Enable DMA for RX */
        reg=SSC_READ_REG(info->mapbase + IFX_SSC_DMACON);
        reg |= IFX_SSC_DMACON_RXON;
        SSC_WRITE_REG(reg, info->mapbase + IFX_SSC_DMACON);
    }
    else {   
        local_irq_save(flags);
        info->rxbuf_ptr = info->rxbuf;
        info->rxbuf_end = info->rxbuf + len;
        if (info->opts.modeRxTx == IFX_SSC_MODE_RXTX) {
            if ((info->txbuf == NULL) ||
                (info->txbuf != info->txbuf_ptr) ||
                (info->txbuf_end != len + info->txbuf)) {
                local_irq_restore(flags);
                printk("IFX SSC - %s: write must be called before calling "
                       "read in combined RX/TX!\n", __FUNCTION__);
                return -EFAULT;
            }
            local_irq_restore(flags);
            /* should enable tx, right?*/
            tx_int(info);
            if (info->txbuf_ptr < info->txbuf_end) {
                ifx_int_wrapper.enable(info->txirq);
            }
        } else { /* rx mode */
            local_irq_restore(flags);
            if (SSC_READ_REG(info->mapbase + IFX_SSC_RXCNT) &
                IFX_SSC_RXCNT_TODO_MASK)
                return -EBUSY;
            if (len < IFX_SSC_RXREQ_BLOCK_SIZE)
                SSC_WRITE_REG(len << IFX_SSC_RXREQ_RXCOUNT_OFFSET, 
                          info->mapbase + IFX_SSC_RXREQ);
            else 
                SSC_WRITE_REG(IFX_SSC_RXREQ_BLOCK_SIZE 
                              << IFX_SSC_RXREQ_RXCOUNT_OFFSET, 
                              info->mapbase + IFX_SSC_RXREQ);
    
        }
    
        ifx_int_wrapper.enable(info->rxirq);
    }
    add_wait_queue(&info->rwait, &wait);
    set_current_state(TASK_INTERRUPTIBLE);
    /* wakeup done in rx_int */
 
    do {
        local_irq_save(flags);
        if (info->rxbuf_ptr >= info->rxbuf_end)
            break;
        local_irq_restore(flags);
#if 0 /* CHECKME */
        if (filp->f_flags & O_NONBLOCK)
        {
            N = -EAGAIN;
            goto out;
        }
#endif
        if (signal_pending(current)) {
            ret_val = -ERESTARTSYS;
            goto out;
        }
        schedule();
    } while (1);

    ret_val = info->rxbuf_ptr - info->rxbuf; /* should be equal to len */
    local_irq_restore(flags);

out:
    current->state = TASK_RUNNING;
    __remove_wait_queue(&info->rwait, &wait);
    return (ret_val);
}

/**
 * SSC Write Helper.
 * Helper routine to handle writes to the kernel or user-space
 * info->txbuf has two cases:
 *  1) return value < 0 (-EFAULT), not touched at all
 *  2) kfree and points to NULL in interrupt routine (but maybe later )
 *
 * \param   info        Pointer to structure #ifx_ssc_port
 * \param   buf         Pointer to read buffer
 * \param   len         Size of read buffer
 * \param   from_kernel 0 -> user mode, 1 -> kernel mode 
 * \return  len         Number of bytes written
 * \return -EFAULT      Reception only mode
 * \ingroup Internal
 */
static ssize_t ifx_ssc_write_helper(struct ifx_ssc_port *info, const char *buf,
                                    size_t len, int from_kernel)
{
    ssize_t retval=0;
    u32 reg=0;
    struct dma_device_info* dma_dev=info->dma_dev;
    
    /* check if in rx mode */
    if (info->opts.modeRxTx == IFX_SSC_MODE_RX)
        return -EFAULT;

    if ( info->opts.dmamode == 1) {
        /* TODO: count number of descriptors needed? */
        
        info->txbuf_ptr = info->txbuf;
        /* DMA is not supported for RXTX mode */
        if (info->opts.modeRxTx == IFX_SSC_MODE_RXTX)
            return -EFAULT;
        /* Enable DMA for TX */
        reg=SSC_READ_REG(info->mapbase + IFX_SSC_DMACON);
        reg |= IFX_SSC_DMACON_TXON;
        SSC_WRITE_REG(reg, info->mapbase + IFX_SSC_DMACON);
        /* Send buffer to DMA */
//        printk("SSC DMA write %d\n",len);
        retval=dma_device_write( dma_dev, (u8*)buf, (s32)len, 0 );        
    }
    else {
        info->txbuf_ptr = info->txbuf;
        info->txbuf_end = len + info->txbuf;
        /* start the transmission (not in rx/tx, see read helper) */
        if (info->opts.modeRxTx == IFX_SSC_MODE_TX) {
            tx_int(info);
            if (info->txbuf_ptr < info->txbuf_end){
                ifx_int_wrapper.enable(info->txirq);
            }
        }
        retval=len;
    }

    return retval;
}

void ifx_ssc_frame_end_int_test(int port)
{
printk("TRACE %s %s %d port=%d\n",__FILE__,__FUNCTION__,__LINE__,port);
}

/**
 * Register SSC Frame callback.
 * This function registers the kernel callback function for the SSC frame end 
 * interrupt.
 *
 * \param   port   SSC Port number (0 = SSC1, 1 = SSC2)
 * \param   func   Pointer to callback function
 * \return  0     OK
 * \return -ENXIO  Invalid port number
 * \return -EBUSY  A callback function is already registered
 * \return -EINVAL Function Pointer is NULL
 * \ingroup API
 */
int ifx_ssc_register_frm_callback(int port, void (*func)(int port))
{
    struct ifx_ssc_port *info;

    if (func == NULL) {
        /* SSC Frame callback function must not be NULL! */
        return -EINVAL;
    }

    if (port < 0 || port >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    info = &isp[port];
    
    if (info->frm_callback.IntCallback != NULL)
        return -EBUSY;
    
    info->frm_callback.IntCallback  = func;
    return 0;
}

/**
 * Unegister SSC Frame callback.
 * This function unregisters the kernel callback function for the SSC frame end 
 * interrupt.
 *
 * \param   port   SSC Port number (0 = SSC1, 1 = SSC2)
 * \return  0     OK
 * \return -ENXIO  Invalid port number
 * \ingroup API
 */
int ifx_ssc_unregister_frm_callback(int port)
{
    struct ifx_ssc_port *info;

    if (port < 0 || port >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    info = &isp[port];
    
    info->frm_callback.IntCallback  = NULL;
    return 0;
}

/**
 * SSC Kernel Read.
 * Interface read function to be called from kernel mode.
 *
 * \param   port   SSC Port number (0 = SSC1, 1 = SSC2)
 * \param   kbuf   Pointer to receive buffer
 * \param   len    Length of receive buffer
 * \return  >0     Number of bytes received
 * \return -ENXIO  Invalid port number
 * \return -EBUSY  Reception currently ongoing
 * \return -EINVAL Invalid receive buffer
 * \ingroup API
 */
ssize_t ifx_ssc_kread(int port, char *kbuf, size_t len)
{
    struct ifx_ssc_port *info;
    ssize_t ret_val;

    if (port < 0 || port >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    if (len == 0)
        return 0;

    info = &isp[port];

    /* check if reception in progress */
    if (info->rxbuf != NULL){
        printk("SSC device busy\n");
        return -EBUSY;
    }

    info->rxbuf = kbuf;
    if (info->rxbuf == NULL){
        printk("SSC device error\n");
        return -EINVAL;
    }
#if 0 /* CHECKME */
    if (!in_irq()){
#endif    
    if (0) {
        ret_val = ifx_ssc_read_helper(info, kbuf, len, 1);
    } else {
        ret_val = ifx_ssc_read_helper_poll(info, kbuf, len, 1);
    };      
    info->rxbuf = NULL;

    /* TODO: perhaps warn if ret_val != len */

    return (ret_val);
}

/**
 * SSC Kernel Write.
 * Interface write function to be called from kernel mode.
 *
 * \param   port   SSC Port number (0 = SSC1, 1 = SSC2)
 * \param   kbuf   Pointer to receive buffer
 * \param   len    Length of receive buffer
 * \return  >0     Number of bytes received
 * \return -ENXIO  Invalid port number
 * \return -EBUSY  Reception currently ongoing
 * \return -EINVAL Invalid receive buffer
 * \ingroup API
 */
ssize_t ifx_ssc_kwrite(int port, const char *kbuf, size_t len)
{
    struct ifx_ssc_port *info;
    ssize_t ret_val;
    u32 tmp;

    if (port < 0 || port >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    if (len == 0)
        return 0;

    info = &isp[port];

    /* check if transmission in progress */
    if (info->txbuf != NULL)
        return -EBUSY;
        
    if (info->frm_opts.FrameEnable == 1) {
	    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON);
	    tmp |= IFX_SSC_SFCON_SF_ENABLE;
	    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_SFCON);
        ifx_int_wrapper.clear(info->frmirq);
        ifx_int_wrapper.enable(info->frmirq);
    }

    info->txbuf = (char *)kbuf;

    ret_val = ifx_ssc_write_helper(info, info->txbuf, len, 1);
    if (ret_val < 0) {
        info->txbuf = NULL;
    }
    return ret_val;
}

/**
 * SSC Read.
 *
 * \param   filp   File structure of device node
 * \param   ubuf   Buffer to read to
 * \param   len    Number of bytes to be read
 * \param   off    Current position in file
 * \return  >=0    Number of bytes read from FIFO
 * \return -EBUSY  SSC receiver busy
 * \return -ENOMEM Could not allocate Rx buffer
 * \return -EFAULT Transmission failed
 * \ingroup API
 */
static ssize_t ifx_ssc_read(struct file *filp, char *ubuf, size_t len, 
                            loff_t *off)
{
    ssize_t ret_val;
    int idx;
    struct ifx_ssc_port *info;

#if 0 /* CHECKME */
    if (len == 0)
        return (0);
#endif
    idx = MINOR(filp->f_dentry->d_inode->i_rdev);
    info = &isp[idx];

    /* check if reception in progress */
    if (info->rxbuf != NULL)
        return -EBUSY;

    info->rxbuf = kmalloc(len+ 3, GFP_KERNEL);
    if (info->rxbuf == NULL)
        return -ENOMEM;

    ret_val = ifx_ssc_read_helper(info, info->rxbuf, len, 0);
    /* TODO: perhaps warn if ret_val != len */
    if (copy_to_user((void*)ubuf, info->rxbuf, ret_val) != 0)
        ret_val = -EFAULT;

    kfree(info->rxbuf);
    info->rxbuf = NULL;
    return (ret_val);
}

/**
 * SSC Write.
 * As many bytes as we have free space for are copied from the user
 * into txbuf and the actual byte count is returned. The transmission is
 * always kicked off by calling the appropriate TX routine.
 *
 * \param   filp   File structure of device node
 * \param   ubuf   Buffer to read from
 * \param   len    Number of bytes to be written
 * \param   off    Current position in file
 * \return  >=0    Number of bytes written to FIFO
 * \return -EBUSY  SSC transmitter busy
 * \return -ENOMEM Could not allocate Tx buffer
 * \return -EFAULT Transmission failed
 * \ingroup API
 */
static ssize_t ifx_ssc_write(struct file *filp, const char *ubuf, size_t len, 
                             loff_t *off)
{
    int idx;
    struct ifx_ssc_port *info;
    int ret_val;

    if (len == 0)
        return (0);

    idx = MINOR(filp->f_dentry->d_inode->i_rdev);
    info = &isp[idx];

    /* check if transmission in progress */
    if (info->txbuf != NULL)
        return -EBUSY;
    info->txbuf = kmalloc(len+ 3, GFP_KERNEL);
    if (info->txbuf == NULL)
        return -ENOMEM;

    ret_val = copy_from_user(info->txbuf, ubuf, len);
    if (ret_val == 0)
        ret_val = ifx_ssc_write_helper(info, info->txbuf, len, 0);
    else
        ret_val = -EFAULT;
    if (ret_val < 0) {
        kfree(info->txbuf); /* otherwise will be done in ISR */
        info->txbuf = NULL;
    }
    return (ret_val);
}


/**
 * Framing Status Get.
 * Get the current status of the serial data framing.
 *
 * \param  info              Pointer to structure #ifx_ssc_port
 * \return &info->frm_status Pointer to structure #ifx_ssc_frm_status
 * \note   Returns a register value independent of framing is enabled or 
 *         not! Changes structure inside of info, so the return value isn't 
 *         needed at all, but could be used for simple access.
 * \ingroup Internal
 */
static struct ifx_ssc_frm_status *ifx_ssc_frm_status_get(struct ifx_ssc_port *info)
{
    unsigned long tmp;
    
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFSTAT);
    info->frm_status.DataBusy = (tmp & IFX_SSC_SFSTAT_IN_DATA) > 0;
    info->frm_status.PauseBusy = (tmp & IFX_SSC_SFSTAT_IN_PAUSE) > 0;
    info->frm_status.DataCount = (tmp & IFX_SSC_SFSTAT_DATA_COUNT_MASK) 
        >> IFX_SSC_SFSTAT_DATA_COUNT_OFFSET;
    info->frm_status.PauseCount = (tmp & IFX_SSC_SFSTAT_PAUSE_COUNT_MASK) 
        >> IFX_SSC_SFSTAT_PAUSE_COUNT_OFFSET;
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON);
    info->frm_status.EnIntAfterData = 
        (tmp & IFX_SSC_SFCON_FIR_ENABLE_BEFORE_PAUSE) > 0;
    info->frm_status.EnIntAfterPause = 
        (tmp & IFX_SSC_SFCON_FIR_ENABLE_AFTER_PAUSE) > 0;
    return (&info->frm_status);
}

/**
 * Framing Control Get.
 * Get the actual control values of the serial data framing.
 *
 * \param  info            Pointer to structure #ifx_ssc_port
 * \return &info->frm_opts Pointer to structure #ifx_ssc_frm_opts
 * \note   Changes structure inside of info, so the return value isn't 
 *         needed at all, but could be used for simple access.
 * \ingroup Internal
 */
static struct ifx_ssc_frm_opts *ifx_ssc_frm_control_get(struct ifx_ssc_port *info)
{
    unsigned long tmp;
    
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON);
    info->frm_opts.FrameEnable = (tmp & IFX_SSC_SFCON_SF_ENABLE) > 0;
    info->frm_opts.DataLength = (tmp & IFX_SSC_SFCON_DATA_LENGTH_MASK) 
        >> IFX_SSC_SFCON_DATA_LENGTH_OFFSET;
    info->frm_opts.PauseLength = (tmp & IFX_SSC_SFCON_PAUSE_LENGTH_MASK) 
        >> IFX_SSC_SFCON_PAUSE_LENGTH_OFFSET;
    info->frm_opts.IdleData = (tmp & IFX_SSC_SFCON_PAUSE_DATA_MASK) 
        >> IFX_SSC_SFCON_PAUSE_DATA_OFFSET;
    info->frm_opts.IdleClock = (tmp & IFX_SSC_SFCON_PAUSE_CLOCK_MASK) 
        >> IFX_SSC_SFCON_PAUSE_CLOCK_OFFSET;
    info->frm_opts.StopAfterPause = 
        (tmp & IFX_SSC_SFCON_STOP_AFTER_PAUSE) > 0;
    return (&info->frm_opts);
}


/**
 * Framing Control Set.
 * Set the actual control values of the serial data framing.
 *
 * \param  info    Pointer to structure #ifx_ssc_port
 * \return  0      OK
 * \return -EINVAL Invalid options provided
 * \ingroup Internal
 */
static int ifx_ssc_frm_control_set(struct ifx_ssc_port *info)
{
    unsigned long tmp;

    /* check parameters */
    if ((info->frm_opts.DataLength > IFX_SSC_SFCON_DATA_LENGTH_MAX) || 
        (info->frm_opts.DataLength < 1) ||
        (info->frm_opts.PauseLength > IFX_SSC_SFCON_PAUSE_LENGTH_MAX) || 
        (info->frm_opts.PauseLength < 1) )
/*
        ((info->frm_opts.IdleData & ~(IFX_SSC_SFCON_PAUSE_DATA_MASK >> 
                      IFX_SSC_SFCON_PAUSE_DATA_OFFSET)) != 0 ) || 
        ((info->frm_opts.IdleClock & ~(IFX_SSC_SFCON_PAUSE_CLOCK_MASK >> 
                       IFX_SSC_SFCON_PAUSE_CLOCK_OFFSET)) != 0 ))
*/      
        return -EINVAL;

#if 0
    /* FIXME: Why shouldn't we set these bits here ? */
    
    /* read interrupt bits (they're not changed here) */
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON) &
        (IFX_SSC_SFCON_FIR_ENABLE_BEFORE_PAUSE | 
         IFX_SSC_SFCON_FIR_ENABLE_AFTER_PAUSE);
#endif
    
    /* Disable framing before changing the setup */
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_SFCON);
    tmp &= ~(IFX_SSC_SFCON_SF_ENABLE);
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_SFCON);

    /* set all values with respect to it's bit position (for data and pause 
       length set N-1) */
    tmp = (info->frm_opts.DataLength - 1) << IFX_SSC_SFCON_DATA_LENGTH_OFFSET;
    tmp |= (info->frm_opts.PauseLength - 1) << IFX_SSC_SFCON_PAUSE_LENGTH_OFFSET;
    tmp |= info->frm_opts.IdleData << IFX_SSC_SFCON_PAUSE_DATA_OFFSET;
    tmp |= info->frm_opts.IdleClock << IFX_SSC_SFCON_PAUSE_CLOCK_OFFSET;
    tmp |= (info->frm_opts.FrameEnable&1) * IFX_SSC_SFCON_SF_ENABLE;
    tmp |= (info->frm_opts.StopAfterPause&1) * IFX_SSC_SFCON_STOP_AFTER_PAUSE;
    tmp |= (info->frm_opts.IntBefore&1) * IFX_SSC_SFCON_FIR_ENABLE_BEFORE_PAUSE;
    tmp |= (info->frm_opts.IntAfter&1) * IFX_SSC_SFCON_FIR_ENABLE_AFTER_PAUSE;
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_SFCON);

#ifdef SSC_FRAME_INT_ENABLE
    tmp = SSC_READ_REG(info->mapbase + IFX_SSC_IRN_EN);
    if (info->frm_opts.FrameEnable == 1) {
//        ifx_ssc_register_frm_callback(info->port_nr,ifx_ssc_frame_end_int_test);
        tmp |= IFX_SSC_F_BIT;
        ifx_int_wrapper.clear(info->frmirq);
        ifx_int_wrapper.enable(info->frmirq);
    }
    else {
//        ifx_ssc_unregister_frm_callback(info->port_nr);
        tmp &= ~(IFX_SSC_F_BIT);
        ifx_int_wrapper.clear(info->frmirq);
        ifx_int_wrapper.disable(info->frmirq);
    }
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_IRN_EN);
#endif

    return 0;
}


/**
 * Rx/Tx mode set.
 * Set the transmission mode while SSC is idle
 *
 * \param   info   Pointer to structure #ifx_ssc_port
 * \param   val    Rx/Tx mode
 * \return  0      OK
 * \return -EINVAL Invalid parameters supplied 
 * \return -EBUSY  Transmission or reception ongoing
 * \ingroup Internal
 */
static int ifx_ssc_rxtx_mode_set(struct ifx_ssc_port *info, unsigned int val)
{
    unsigned long tmp;

    /* check parameters */
    if (!(info) || (val & ~(IFX_SSC_MODE_MASK)))
        return -EINVAL;
    /*check BUSY and RXCNT*/    
    if (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) & IFX_SSC_STATE_BUSY ||
        SSC_READ_REG(info->mapbase + IFX_SSC_RXCNT) & IFX_SSC_RXCNT_TODO_MASK)
        return -EBUSY;
    /* modify */
    tmp = (SSC_READ_REG(info->mapbase + IFX_SSC_CON) &
          ~(IFX_SSC_CON_RX_OFF | IFX_SSC_CON_TX_OFF)) | (val);
    SSC_WRITE_REG(tmp, info->mapbase + IFX_SSC_CON);
    info->opts.modeRxTx = val;

    return 0;
}

#ifndef CONFIG_INCAIP2
/**
 * GPIO Init.
 * Initialize MUX settings to enable SPI interface
 *
 * \ingroup Internal
 */
void ifx_gpio_init(void) 
{
    u32 temp;
    /* set gpio pin p0.10(SPI_DIN) p0.11(SPI_DOUT) p0.12(SPI_CLK) p0.13(SPI_CS2)
       direction */
    temp = *(IFX_GPIO_PDIR) ;
    temp &= 0xFFFFFBFF;
    temp |= 0x3800;
    *(IFX_GPIO_PDIR) = temp;
    /* set port 0 alternate select register 0 */
    temp = *(IFX_GPIO_ALTSEL0) ;
    temp &= 0xFFFFC3FF;
    temp |= 0x00001c00;
    *(IFX_GPIO_ALTSEL0) = temp;

    /* set port 0 alternate select register 1 */
    temp = *(IFX_GPIO_ALTSEL1) ;
    temp &= 0xFFFFC3FF;
    temp |= 0x00002000;
    *(IFX_GPIO_ALTSEL1) = temp;

    /* set port 0 open drain mode register */
    temp = *(IFX_GPIO_POD);
    temp |= 0x00003800; /* set output pin normal mode */
    *(IFX_GPIO_POD)= temp;
}
#endif /* !CONFIG_INCAIP2 */

/**
 * SSC Set Hardware Options.
 * This routine intializes the SSC appropriately depending on slave/master and 
 * full-/half-duplex mode. It assumes that the SSC is disabled and the fifo's 
 * and buffers are flushed later on.
 *
 * \param   info   Pointer to structure #ifx_ssc_port
 * \return  0      OK
 * \return -EINVAL Invalid hardware options supplied
 * \ingroup API
 */
static int ifx_ssc_sethwopts(struct ifx_ssc_port *info)
{
    unsigned long flags, bits;
    struct ifx_ssc_hwopts *opts = &info->opts;

    /* sanity checks */
    if ((opts->dataWidth < IFX_SSC_MIN_DATA_WIDTH) ||
        (opts->dataWidth > IFX_SSC_MAX_DATA_WIDTH)) {
        printk("%s: sanity check failed\n", __FUNCTION__);
        return -EINVAL;
    }
    if ((!opts->masterSelect) && (opts->dmamode == 1)) {
        printk("%s: DMA mode only as SSC master possible\n", __FUNCTION__);
        return -EINVAL;
    }

    bits = (opts->dataWidth - 1) << IFX_SSC_CON_DATA_WIDTH_OFFSET;
    bits |= IFX_SSC_CON_ENABLE_BYTE_VALID;
#if 0
    if (opts->abortErrDetect)
        bits |= IFX_SSC_CON_ABORT_ERR_CHECK;
#endif
    if (opts->rxOvErrDetect)
        bits |= IFX_SSC_CON_RX_OFL_CHECK;
    if (opts->rxUndErrDetect)
        bits |= IFX_SSC_CON_RX_UFL_CHECK;
    if (opts->txOvErrDetect)
        bits |= IFX_SSC_CON_TX_OFL_CHECK;
    if (opts->txUndErrDetect)
        bits |= IFX_SSC_CON_TX_UFL_CHECK;
    if (opts->loopBack)
        bits |= IFX_SSC_CON_LOOPBACK_MODE;
    if (opts->echoMode)
        bits |= IFX_SSC_CON_ECHO_MODE_ON;
    if (opts->headingControl)
        bits |= IFX_SSC_CON_MSB_FIRST;
    if (opts->clockPhase)
        bits |= IFX_SSC_CON_LATCH_THEN_SHIFT;
    if (opts->clockPolarity)
        bits |= IFX_SSC_CON_CLOCK_FALL;
    switch (opts->modeRxTx) {
    case IFX_SSC_MODE_TX:
        bits |= IFX_SSC_CON_RX_OFF;
        break;
    case IFX_SSC_MODE_RX:
        bits |= IFX_SSC_CON_TX_OFF;
        break;
    } 
    local_irq_save(flags);
    SSC_WRITE_REG(bits, info->mapbase + IFX_SSC_CON);
    SSC_WRITE_REG((info->opts.gpoCs << IFX_SSC_GPOCON_ISCSB0_POS) | 
                  (info->opts.gpoInv << IFX_SSC_GPOCON_INVOUT0_POS), 
                  info->mapbase + IFX_SSC_GPOCON);
    /* master mode */
    if (opts->masterSelect) {
        ifx_port_ssc_cs_set(info->port_nr, 0, 1);
        ifx_port_ssc_cs_set(info->port_nr, 1, 1);
        ifx_port_ssc_master_set(info->port_nr,1);
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_MASTER_SELECT,info->mapbase + 
                      IFX_SSC_WHBSTATE);
    } else {
        ifx_port_ssc_cs_set(info->port_nr, 0, 0);
        ifx_port_ssc_cs_set(info->port_nr, 1, 0);
        ifx_port_ssc_master_set(info->port_nr,0);
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_MASTER_SELECT,info->mapbase + 
                      IFX_SSC_WHBSTATE);
    }
    /* init serial framing */
    SSC_WRITE_REG(0, info->mapbase + IFX_SSC_SFCON);
#ifndef CONFIG_INCAIP2
    /* set up the port pins */
    ifx_gpio_init();
#endif
    local_irq_restore(flags);

    return 0;
}

/**
 * SSC Set Baudrate.
 * Sets the baudrate of the corresponding port according to the passed
 * rate after reading out the current module speed.
 *
 * \param   info   Pointer to structure #ifx_ssc_port
 * \param   baud   Desired baudrate
 * \return  0      OK
 * \return -EINVAL Could not retrieve system clock or invalid baudrate setting
 * \ingroup Internal
 */
static int ifx_ssc_set_baud(struct ifx_ssc_port *info, unsigned int baud)
{
    unsigned int ifx_ssc_clock;
    unsigned int br;
    unsigned long flags;
    bool enabled;

    ifx_ssc_clock = ifx_ssc_get_kernel_clk(info);
    if (ifx_ssc_clock ==0)
        return -EINVAL;

    local_irq_save(flags);
    /* have to disable the SSC to set the baudrate */
    enabled = (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) 
               & IFX_SSC_STATE_IS_ENABLED) != 0;
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE, 
                  info->mapbase + IFX_SSC_WHBSTATE);

    /* compute divider */
    br = ((ifx_ssc_clock >> 1)/baud) - 1;
    asm("SYNC"); 
    if (br > 0xffff || 
        ((br == 0) && ((SSC_READ_REG(info->mapbase + IFX_SSC_STATE) & 
                        IFX_SSC_STATE_IS_MASTER) == 0))) {
        local_irq_restore(flags);
        printk("%s: illegal baudrate %u\n", __FUNCTION__, baud);
        return -EINVAL;
    }
    SSC_WRITE_REG(br, info->mapbase + IFX_SSC_BR);
    if (enabled)
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_ENABLE, 
                      info->mapbase + IFX_SSC_WHBSTATE);

    local_irq_restore(flags);
    return 0;
}

/**
 * SSC HW Initialization.
 * Initializes the SSC Port HW with the desired baudrate and transmission
 * options.
 *
 * \param   info   Pointer to structure #ifx_ssc_port
 * \return  0      OK
 * \return -EINVAL Error during initialization
 * \ingroup Internal
 */
static int ifx_ssc_hwinit(struct ifx_ssc_port *info)
{
    unsigned long flags;
    bool enabled;

    /* have to disable the SSC */
    enabled = (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) 
               & IFX_SSC_STATE_IS_ENABLED) != 0;
    SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE, 
                  info->mapbase + IFX_SSC_WHBSTATE);

    if (ifx_ssc_sethwopts(info) < 0)
    {
        printk("%s: setting the hardware options failed\n",
                       __FUNCTION__);
        return -EINVAL;
    }

    if (ifx_ssc_set_baud(info, info->baud) < 0) {
        printk("%s: setting the baud rate failed\n", __FUNCTION__);
        return -EINVAL;
    }
        local_irq_save(flags);
        /* TX FIFO */
        SSC_WRITE_REG((IFX_SSC_DEF_TXFIFO_FL << 
                       IFX_SSC_XFCON_ITL_OFFSET) | 
                       IFX_SSC_XFCON_FIFO_ENABLE, 
                      info->mapbase + IFX_SSC_TXFCON);
        /* RX FIFO */
        SSC_WRITE_REG((IFX_SSC_DEF_RXFIFO_FL << 
                       IFX_SSC_XFCON_ITL_OFFSET) | 
                       IFX_SSC_XFCON_FIFO_ENABLE, 
                      info->mapbase + IFX_SSC_RXFCON);
        local_irq_restore(flags);
    if (enabled)
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_ENABLE, 
                      info->mapbase + IFX_SSC_WHBSTATE);
    return 0;
}

/**
 * SSC IOCTL.
 * The following IOCTls are supported for the SSC device:
 * \li #IFX_SSC_STATS_READ
 * \li #IFX_SSC_STATS_RESET
 * \li #IFX_SSC_BAUD_SET
 * \li #IFX_SSC_BAUD_GET
 * \li #IFX_SSC_HWOPTS_SET
 * \li #IFX_SSC_HWOPTS_GET
 * \li #IFX_SSC_RXTX_MODE_SET
 * \li #IFX_SSC_RXTX_MODE_GET
 * \li #IFX_SSC_ABORT
 * \li #IFX_SSC_FIFO_FLUSH
 * \li #IFX_SSC_GPO_OUT_SET
 * \li #IFX_SSC_GPO_OUT_CLR
 * \li #IFX_SSC_GPO_OUT_GET
 * \li #IFX_SSC_FRM_STATUS_GET
 * \li #IFX_SSC_FRM_CONTROL_GET
 * \li #IFX_SSC_FRM_CONTROL_SET
 *
 * \param   inode       Inode of device 
 * \param   filp        File structure of device
 * \param   cmd         IOCTL command 
 * \param   data        Argument for some IOCTL commands
 * \return -ENOIOCTLCMD invalid IOCTL command
 * \ingroup API
 */
int ifx_ssc_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
                  unsigned long data)
{
    struct ifx_ssc_port *info;
    int line, ret_val = 0;
    unsigned long flags;
    unsigned long tmp;
    int from_kernel = 0;
    bool enabled;

    if ((inode == (struct inode *)0) || (inode == (struct inode *)1)) {
        from_kernel = 1;
        line = (int)inode;
    } else
        line = MINOR(filp->f_dentry->d_inode->i_rdev);

    /* don't use more minor devices than we can support */
    if (line < 0 || line >= IFX_SSC_MAX_PORT_NUM)
        return -ENXIO;

    info = &isp[line];

//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
    switch (cmd) 
    {
    case IFX_SSC_STATS_READ:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        /* data must be a pointer to a struct ifx_ssc_statistics */
        if (from_kernel)
            memcpy((void *)data, (void *)&info->stats, 
            sizeof(struct ifx_ssc_statistics));
        else
            if (copy_to_user((void *)data, (void *)&info->stats, 
                sizeof(struct ifx_ssc_statistics)))
                ret_val = -EFAULT;
        break;
    case IFX_SSC_STATS_RESET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        /* just resets the statistics counters */
        memset((void *)&info->stats, 0, sizeof(struct ifx_ssc_statistics));
        break;
    case IFX_SSC_BAUD_SET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        /* if the buffers are not empty then the port is */
        /* busy and we shouldn't change things on-the-fly! */
        if (info->txbuf || info->rxbuf || 
            (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) & 
                          IFX_SSC_STATE_BUSY)) {
            ret_val = -EBUSY;
            break;
        }
        /* misuse flags */
        if (from_kernel)
            flags = *((unsigned long *)data);
        else
            if (copy_from_user((void *)&flags, (void *)data, sizeof(flags)))
            {
                ret_val = -EFAULT;
                break;
            }
        if (flags == 0)
        {
            ret_val = -EINVAL;
            break;
        }

        if (ifx_ssc_set_baud(info, flags) < 0)
        {
            ret_val = -EINVAL;
            break;
        }
        info->baud = flags;
        break;
    case IFX_SSC_BAUD_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        if (from_kernel)
            *((unsigned int *)data) = info->baud;
        else
            if (copy_to_user((void *)data,
                     (void *)&info->baud,
                     sizeof(unsigned long)))
                ret_val = -EFAULT;
        break;
    case IFX_SSC_RXTX_MODE_SET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x data=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd,data);
        if (from_kernel)
            tmp = *((unsigned long *)data);
        else
            if (copy_from_user((void *)&tmp,
                       (void *)data, sizeof(tmp))) {
                ret_val = -EFAULT;
                break;
            }
        ret_val = ifx_ssc_rxtx_mode_set(info, tmp);
        break;
    case IFX_SSC_RXTX_MODE_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        tmp = SSC_READ_REG(info->mapbase + IFX_SSC_CON) &
            (~(IFX_SSC_CON_RX_OFF | IFX_SSC_CON_TX_OFF));
        if (from_kernel)
            *((unsigned int *)data) = tmp;
        else
            if (copy_to_user((void *)data,
                     (void *)&tmp,
                     sizeof(tmp)))
                ret_val = -EFAULT;
        break;
    case IFX_SSC_ABORT:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
       ifx_ssc_abort(info);
        break;
    case IFX_SSC_GPO_OUT_SET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        if (from_kernel)
            tmp = *((unsigned long *)data);
        else
            if (copy_from_user((void *)&tmp,
                       (void *)data, sizeof(tmp))) {
                ret_val = -EFAULT;
                break;
            }
//printk("TRACE %s %s %d tmp=%d\n",__FILE__,__FUNCTION__,__LINE__,tmp);
        if (tmp > IFX_SSC_MAX_GPO_OUT)
            ret_val = -EINVAL;
        else
            SSC_WRITE_REG (1<<(tmp + IFX_SSC_WHBGPOSTAT_SETOUT0_POS), 
             info->mapbase + IFX_SSC_WHBGPOSTAT);
        break;
    case IFX_SSC_GPO_OUT_CLR:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        if (from_kernel)
            tmp = *((unsigned long *)data);
        else
            if (copy_from_user((void *)&tmp,
                       (void *)data, sizeof(tmp))) {
                ret_val = -EFAULT;
                break;
            }
//printk("TRACE %s %s %d tmp=%d\n",__FILE__,__FUNCTION__,__LINE__,tmp);
        if (tmp > IFX_SSC_MAX_GPO_OUT)
            ret_val = -EINVAL;
        else {
            SSC_WRITE_REG
            (1<<(tmp + IFX_SSC_WHBGPOSTAT_CLROUT0_POS), 
             info->mapbase + IFX_SSC_WHBGPOSTAT);
        }
        break;
    case IFX_SSC_GPO_OUT_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        tmp = SSC_READ_REG
            (info->mapbase + IFX_SSC_GPOSTAT);
            if (from_kernel)
                *((unsigned int *)data) = tmp;
            else
                if (copy_to_user((void *)data,
                         (void *)&tmp,
                         sizeof(tmp)))
                    ret_val = -EFAULT;
        break;
    case IFX_SSC_FRM_STATUS_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        ifx_ssc_frm_status_get(info);
        if (from_kernel)
            memcpy((void *)data, (void *)&info->frm_status,
               sizeof(struct ifx_ssc_frm_status));
        else
            if (copy_to_user((void *)data,
                 (void *)&info->frm_status,
                 sizeof(struct ifx_ssc_frm_status)))
            ret_val = -EFAULT;
        break;
    case IFX_SSC_FRM_CONTROL_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        ifx_ssc_frm_control_get(info);
        if (from_kernel)
            memcpy((void *)data, (void *)&info->frm_opts,
               sizeof(struct ifx_ssc_frm_opts));
        else
            if (copy_to_user((void *)data,
                 (void *)&info->frm_opts,
                 sizeof(struct ifx_ssc_frm_opts)))
            ret_val = -EFAULT;
        break;
    case IFX_SSC_FRM_CONTROL_SET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x data=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd,data);
        if (from_kernel)
            memcpy((void *)&info->frm_opts, (void *)data, 
                   sizeof(struct ifx_ssc_frm_opts));
        else
            if (copy_from_user((void *)&info->frm_opts,
                             (void *)data,
                             sizeof(struct ifx_ssc_frm_opts))) {
                ret_val = -EFAULT;
                break;
            }
        ret_val = ifx_ssc_frm_control_set(info);
        break;
    case IFX_SSC_HWOPTS_SET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        /* data must be a pointer to a struct ifx_ssc_hwopts
           if the buffers are not empty then the port is
           busy and we shouldn't change things on-the-fly! */
		
	   
        if (info->txbuf || info->rxbuf || 
            (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) 
            & IFX_SSC_STATE_BUSY)) {
            ret_val = -EBUSY;
            break;
        }
        if (from_kernel)
            memcpy((void *)&info->opts, (void *)data,
                   sizeof(struct ifx_ssc_hwopts));
        else
            if (copy_from_user((void *)&info->opts,
                               (void *)data,
                               sizeof(struct ifx_ssc_hwopts)))
            {
                ret_val = -EFAULT;
                break;
            }
        if (ifx_ssc_hwinit(info) < 0)
        {
            ret_val = -EIO;
        }
        break;
    case IFX_SSC_HWOPTS_GET:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        /* data must be a pointer to a struct ifx_ssc_hwopts */
        if (from_kernel)
            memcpy((void *)data, (void *)&info->opts,
                   sizeof(struct ifx_ssc_hwopts));
        else
            if (copy_to_user((void *)data,
                             (void *)&info->opts,
                             sizeof(struct ifx_ssc_hwopts)))
                ret_val = -EFAULT;
        break;
    case IFX_SSC_FIFO_FLUSH:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        tmp=SSC_READ_REG(info->mapbase + IFX_SSC_RXFCON);
        SSC_WRITE_REG(tmp | IFX_SSC_XFCON_FIFO_FLUSH,info->mapbase + IFX_SSC_RXFCON);
        tmp=SSC_READ_REG(info->mapbase + IFX_SSC_TXFCON);
        SSC_WRITE_REG(tmp | IFX_SSC_XFCON_FIFO_FLUSH,info->mapbase + IFX_SSC_TXFCON);
        break;
    case IFX_SSC_DMA_ENABLE:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        if (info->opts.masterSelect) {
            SSC_WRITE_REG(0, info->mapbase + IFX_SSC_IRN_EN);
            info->opts.dmamode = 1;
        }
        else ret_val = -EFAULT;
        break;
    case IFX_SSC_DMA_DISABLE:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        info->opts.dmamode = 0;
        tmp=SSC_READ_REG(info->mapbase + IFX_SSC_IRN_EN);
//        if (info->frm_opts.FrameEnable == 1) tmp |= IFX_SSC_F_BIT;
        SSC_WRITE_REG(tmp | IFX_SSC_DEF_IRNEN, info->mapbase + IFX_SSC_IRN_EN);
        break;
    case IFX_SSC_RESURRECT:
        if (info->rxbuf != NULL) kfree(info->rxbuf);
        if (info->txbuf != NULL) kfree(info->txbuf);
        info->rxbuf = NULL;
        info->txbuf = NULL;
        
        tmp=SSC_READ_REG(info->mapbase + IFX_SSC_RXFCON);
        SSC_WRITE_REG(tmp | IFX_SSC_XFCON_FIFO_FLUSH,info->mapbase + IFX_SSC_RXFCON);
        tmp=SSC_READ_REG(info->mapbase + IFX_SSC_TXFCON);
        SSC_WRITE_REG(tmp | IFX_SSC_XFCON_FIFO_FLUSH,info->mapbase + IFX_SSC_TXFCON);
        /* wait 500us */
        current->state = TASK_INTERRUPTIBLE;
        schedule_timeout(500*HZ/1000);
        
        enabled = (SSC_READ_REG(info->mapbase + IFX_SSC_STATE) & IFX_SSC_STATE_IS_ENABLED) != 0;
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE, info->mapbase + IFX_SSC_WHBSTATE);
        /* wait 1s */
        current->state = TASK_INTERRUPTIBLE;
        schedule_timeout(1000*HZ/1000);
        if (enabled) SSC_WRITE_REG(IFX_SSC_WHBSTATE_SET_ENABLE,info->mapbase + IFX_SSC_WHBSTATE);
        break;
    default:
//printk("TRACE %s %s %d line=%d cmd=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,line,cmd);
        ret_val = -ENOIOCTLCMD;
    }

    return ret_val;
}

#ifdef CHECKME_USE_POLL
/**
 * SSC Poll Function.
 * Poll handler called by system stack on select call
 *
 * \param   filp File structure of device
 * \param   pts  Pointer to select structure
 * \return  Poll status
 * \ingroup Internal
 */
static unsigned int ifx_ssc_poll(struct file *filp, 
                                 struct poll_table_struct *pts)
{
    int unit = MINOR(filp->f_dentry->d_inode->i_rdev);
    struct ifx_ssc_port *info;
    unsigned int mask = 0; 
    int spc;

    info = &isp[unit];

    /* add event to the wait queues */
    /* DO NOT FORGET TO DO A WAKEUP ON THESE !!!! */
    poll_wait(filp, &info->pwait, pts);

    /* are there bytes in the RX SW-FIFO? */
    if (info->rxrp != info->rxwp)
        mask |= POLLIN | POLLRDNORM;

    /* free space in the TX SW-FIFO */
    spc = info->txrp - info->txwp - 1;
    if (spc < 0)
        spc += TX_BUFSIZE;
#ifdef IFX_SSC_USEDMA
    /* writing always works, except in the DMA case when all descriptors */
    /* are used up */
    if (unit == 1 && info->dma_freecnt == 0)
        spc = 0;
#endif
    if (spc > 0)
        mask |= POLLOUT | POLLWRNORM;

    return (mask);
}
#endif

/**
 * SSC Proc File Output.
 * 
 * \param   buf    Buffer to write the string to
 * \param   start  not used (Linux internal)
 * \param   offset not used (Linux internal)
 * \param   count  not used (Linux internal)
 * \param   eof    Set to 1 when all data is stored in buffer
 * \param   data   not used (Linux internal)
 * \return  len    Lenght of data in buffer
 * \ingroup Internal  
 */
static int ifx_ssc_read_proc(char *buf, char **start, off_t offset, int count,
                              int *eof, void *data)
{
    int off = 0;
    int t = 0;
    struct ifx_ssc_port *info;

    /* print statistics */
    off += sprintf(buf+off, "Statistics for Infineon Synchronous Serial Controller\n\n");
    
    for (t=0;t<IFX_SSC_MAX_PORT_NUM;t++) {
        if (isp[t].port_is_available != 0) {
            off += sprintf(buf+off, "SSC%d\n",isp[t].port_nr);
            off += sprintf(buf+off, "RX overflow errors %d\n", isp[t].stats.rxOvErr);
            off += sprintf(buf+off, "RX underflow errors %d\n", isp[t].stats.rxUnErr);
            off += sprintf(buf+off, "TX overflow errors %d\n", isp[t].stats.txOvErr);
            off += sprintf(buf+off, "TX underflow errors %d\n", isp[t].stats.txUnErr);
            off += sprintf(buf+off, "Abort errors %d\n", isp[t].stats.abortErr);
            off += sprintf(buf+off, "Mode errors %d\n", isp[t].stats.modeErr);
            off += sprintf(buf+off, "Unknown errors %d\n", isp[t].stats.unknownErr);
            off += sprintf(buf+off, "RX Bytes %d\n", isp[t].stats.rxBytes);
            off += sprintf(buf+off, "TX Bytes %d\n", isp[t].stats.txBytes);
            if (isp[t].opts.dmamode == 1) off += sprintf(buf+off, "DMA mode\n");
            off += sprintf(buf+off, "\n");
            
            info = &isp[t];
            off += sprintf(buf+off, "IFX_SSC_CLC 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_CLC));
            off += sprintf(buf+off, "IFX_SSC_ID 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_ID));
            off += sprintf(buf+off, "IFX_SSC_MCON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_CON));
            off += sprintf(buf+off, "IFX_SSC_STATE 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_STATE));
            off += sprintf(buf+off, "IFX_SSC_TB 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_TB));
            off += sprintf(buf+off, "IFX_SSC_FSTAT 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_FSTAT));
            off += sprintf(buf+off, "IFX_SSC_RXFCON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_RXFCON));
            off += sprintf(buf+off, "IFX_SSC_TXFCON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_TXFCON));
            off += sprintf(buf+off, "IFX_SSC_BR 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_BR));
            off += sprintf(buf+off, "IFX_SSC_SFCON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_SFCON));
            off += sprintf(buf+off, "IFX_SSC_SFSTAT 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_SFSTAT));
            off += sprintf(buf+off, "IFX_SSC_GPOCON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_GPOCON));
            off += sprintf(buf+off, "IFX_SSC_GPOSTAT 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_GPOSTAT));
            off += sprintf(buf+off, "IFX_SSC_RXREQ 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_RXREQ));
            off += sprintf(buf+off, "IFX_SSC_RXCNT 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_RXCNT));
            off += sprintf(buf+off, "IFX_SSC_DMACON 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_DMACON));
            off += sprintf(buf+off, "IFX_SSC_IRN_EN 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_IRN_EN));
            off += sprintf(buf+off, "IFX_SSC_IRN_CR 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_IRN_CR));
            off += sprintf(buf+off, "IFX_SSC_IRN_ICR 0x%08x\n", SSC_READ_REG(info->mapbase + IFX_SSC_IRN_ICR));
            
            off += sprintf(buf+off, "\n");
       }
    }
    
    *eof = 1;
    return (off);
}

/**
 * Show Version String.
 * This routine prints out the appropriate serial driver version number
 *
 * \return none
 * \ingroup Internal
 */
static inline void show_version(void)
{
#if 0
    printk("Infineon Technologies Synchronous Serial Controller (SSC) driver\n"
           "  version %s - built %s %s\n", IFX_SSC_DRV_VERSION, __DATE__, __TIME__);
#endif   
}

/* Fops-struct */
static struct file_operations ifx_ssc_fops = {
    owner:    THIS_MODULE,
    read:     ifx_ssc_read,
    write:    ifx_ssc_write,
#ifdef CHECKME_USE_POLL     
    poll:     ifx_ssc_poll,
#endif    
    ioctl:    ifx_ssc_ioctl,
    open:     ifx_ssc_open,
    release:  ifx_ssc_close,
};

/**
 * SSC Initialization.
 * Due to the fact that a port can be dynamically switched between slave
 * and master mode using an IOCTL the hardware is not initialized here,
 * but in ifx_ssc_hwinit() as a result of an IOCTL.
 *
 * \return  none
 * \ingroup Internal
 */
int __init ifx_ssc_init(void)
{
    struct ifx_ssc_port  *info;
    int i,t,nbytes;
    unsigned long flags;
    int ret_val;
    char dma_dev_name[10];
#ifdef CONFIG_INCAIP2
    u32 pins=0;
#endif
    
//init of sidecar wait_queue
    init_waitqueue_head(&wait_queue_sidecar_fastmode_finish);
//

    /* DONE: dynamic port count evaluation due to pin multiplexing */
    ret_val = -ENOMEM;
#ifdef CONFIG_INCAIP2
    /* Get avaiability of SSC ports, calculate correct number of bytes 
       for info structure and enable SSC in PMS if it is available */
    pins=ifx_mux_ssc_pins();
    if ((pins & CONFIG_INCAIP2_MUX_BIT_SSC0) != 0) {
        *INCA_IP2_PMS_PMS_GEN |= INCA_IP2_PMS_PMS_GEN_SSC0;
        ifx_ssc_port_map |= 0x00000001;
    }
    if ((pins & CONFIG_INCAIP2_MUX_BIT_SSC1) != 0) {
        *INCA_IP2_PMS_PMS_GEN |= INCA_IP2_PMS_PMS_GEN_SSC1;
        ifx_ssc_port_map |= 0x00000002;
    }
#else
    ifx_ssc_port_map=0x00000001;
#endif /* CONFIG_INCAIP2 */
    nbytes = IFX_SSC_MAX_PORT_NUM * sizeof(struct ifx_ssc_port);
    isp = (struct ifx_ssc_port *)kmalloc(nbytes, GFP_KERNEL);
    if (isp == NULL) {
        printk("%s: no memory for isp\n", __FUNCTION__);
        return (ret_val);
    }
    memset(isp, 0, nbytes);

    show_version();

    /* register the device */
    ret_val = -ENXIO;

#ifdef CONFIG_DEVFS_FS
    devfs_register_chrdev(maj , "ifx_ssc", &ifx_ssc_fops);
    maj=(int) devfs_register(NULL, "ifx_ssc0", DEVFS_FL_NONE, maj, 0, S_IFCHR | S_IRUGO | S_IWUGO, &ifx_ssc_fops, NULL);
    if (maj == 0) {
        printk("Unable to register SSC device\n");
        goto errout;
    }
    maj=(int) devfs_register(NULL, "ifx_ssc1", DEVFS_FL_NONE, maj, 1, S_IFCHR | S_IRUGO | S_IWUGO, &ifx_ssc_fops, NULL);
    if (maj == 0) {
        printk("Unable to register SSC device\n");
        goto errout;
    }
    printk("registered Infineon SSC device\n");
#else
    if ((i = register_chrdev(maj, "ifx_ssc", &ifx_ssc_fops)) < 0) {
        printk("Unable to register major %d for the Infineon SSC\n", maj);
        goto errout;
    }
    if (maj == 0) maj = i;
    printk("registered major %d for Infineon SSC\n", maj);
#endif

    /* set default values in ifx_ssc_port */
    for (i = 0; i < IFX_SSC_MAX_PORT_NUM; i++) {
        info = &isp[i];
        info->port_nr = i;
        /* default values for HwOpts */
        info->opts.AbortErrDetect   = IFX_SSC_DEF_ABRT_ERR_DETECT;
        info->opts.rxOvErrDetect    = IFX_SSC_DEF_RO_ERR_DETECT;
        info->opts.rxUndErrDetect   = IFX_SSC_DEF_RU_ERR_DETECT;
        info->opts.txOvErrDetect    = IFX_SSC_DEF_TO_ERR_DETECT;
        info->opts.txUndErrDetect   = IFX_SSC_DEF_TU_ERR_DETECT;
        info->opts.loopBack         = IFX_SSC_DEF_LOOP_BACK;
        info->opts.echoMode         = IFX_SSC_DEF_ECHO_MODE;
        info->opts.idleValue        = IFX_SSC_DEF_IDLE_DATA;
        info->opts.clockPolarity    = IFX_SSC_DEF_CLOCK_POLARITY;
        info->opts.clockPhase       = IFX_SSC_DEF_CLOCK_PHASE;
        info->opts.headingControl   = IFX_SSC_DEF_HEADING_CONTROL;
        info->opts.dataWidth        = IFX_SSC_DEF_DATA_WIDTH;
        info->opts.modeRxTx         = IFX_SSC_MODE_TX;
//        info->opts.modeRxTx         = IFX_SSC_DEF_MODE_RXTX;
        info->opts.gpoCs            = IFX_SSC_DEF_GPO_CS;
        info->opts.gpoInv           = IFX_SSC_DEF_GPO_INV;
        info->opts.masterSelect     = IFX_SSC_DEF_MASTERSLAVE;

        /* default values for framing options */
        info->frm_opts.FrameEnable      = 0;
        info->frm_opts.IntBefore        = 0;
        info->frm_opts.IntAfter         = 0;
        info->frm_opts.DataLength       = 0;
        info->frm_opts.PauseLength      = 0;
        info->frm_opts.IdleData         = 0;
        info->frm_opts.StopAfterPause   = 0;
        info->frm_callback.IntCallback  = NULL;
        
        info->baud  = IFX_SSC_DEF_BAUDRATE;
        info->rxbuf = NULL;
        info->txbuf = NULL;
        info->port_is_available= ((ifx_ssc_port_map & (1 << i)) != 0);

        /* values specific to SSC0 */
        info->mapbase = ifx_ssc_port[i].base_address;
        /* TODO: power management */
        /* setting interrupt vectors */
        info->txirq = ifx_ssc_port[i].tx_int;
        info->rxirq = ifx_ssc_port[i].rx_int;
        info->errirq = ifx_ssc_port[i].err_int;
#ifdef SSC_FRAME_INT_ENABLE
        info->frmirq = ifx_ssc_port[i].frm_int;
#endif
        sprintf(dma_dev_name, "SSC%d", i); 
        info->dma_dev = dma_device_reserve(dma_dev_name);
        
        if (info->dma_dev == NULL) {
            printk("%s: no memory for dma_dev %s\n", __FUNCTION__, dma_dev_name);
            goto errout;
        }

        ifx_ssc_init_dma_device(i,info->dma_dev);
//        printk("SSC DMA register %s\n",info->dma_dev->device_name);
        if ( dma_device_register(info->dma_dev) != 0 )
        {
            printk("%s: Registration of DMA device %s failed!\n", 
               __FUNCTION__, info->dma_dev->device_name);
            goto dma_errout; 
        }
        info->dma_dev->rx_chan[0]->open(info->dma_dev->rx_chan[0]);
        info->dma_dev->tx_chan[0]->open(info->dma_dev->tx_chan[0]);
        
        /* activate SSC */
        /* CLC.DISS = 0 */
        SSC_WRITE_REG(IFX_SSC_DEF_RMC << IFX_CLC_RUN_DIVIDER_OFFSET, 
                      info->mapbase + IFX_SSC_CLC);

        init_waitqueue_head(&info->rwait);
#if 0 /* CHECKME */        
        init_waitqueue_head(&info->pwait);
#endif        

        if (info->port_is_available != 0) {        	
            local_irq_save(flags);

            /* init serial framing register */
            SSC_WRITE_REG(IFX_SSC_DEF_SFCON, info->mapbase + IFX_SSC_SFCON);
            
            /* try to get the interrupts */
            /* TODO: interrupt handling with multiple instances */
            sprintf(info->irqname[0],"ifx_ssc%d_tx",info->port_nr); 
            ret_val = ifx_int_wrapper.request(info->txirq, ifx_ssc_tx_int, 
                                              SA_INTERRUPT, info->irqname[0], info);
            if (ret_val) {
                printk("%s: unable to get irq %d\n", __FUNCTION__,info->txirq);
                local_irq_restore(flags);
                goto dma_errout;
            }
            
            sprintf(info->irqname[1],"ifx_ssc%d_rx",info->port_nr); 
            ret_val = ifx_int_wrapper.request(info->rxirq, ifx_ssc_rx_int, 
                                              SA_INTERRUPT, info->irqname[1], info);
            if (ret_val) {
                printk("%s: unable to get irq %d\n", __FUNCTION__,info->rxirq);
                local_irq_restore(flags);
                goto irqerr;
            }
            sprintf(info->irqname[2],"ifx_ssc%d_err",info->port_nr); 
            ret_val = ifx_int_wrapper.request(info->errirq, ifx_ssc_err_int, 
                                              SA_INTERRUPT, info->irqname[2], info);
            if (ret_val) {
                printk("%s: unable to get irq %d\n", __FUNCTION__,info->errirq);
                local_irq_restore(flags);
                goto irqerr;
            }
#ifdef SSC_FRAME_INT_ENABLE
            sprintf(info->irqname[3],"ifx_ssc%d_frm",info->port_nr); 
            ret_val = ifx_int_wrapper.request(info->frmirq, ifx_ssc_frm_int, 
                              SA_INTERRUPT, info->irqname[3], info);
            if (ret_val) {
                printk("%s: unable to get irq %d\n", __FUNCTION__,info->frmirq);
                local_irq_restore(flags);
                goto irqerr;
            }
            ifx_int_wrapper.disable(info->frmirq);
    
#endif
            SSC_WRITE_REG(IFX_SSC_DEF_IRNEN, info->mapbase + IFX_SSC_IRN_EN);
           
            /* init the SSCs with default values */
            if (ifx_ssc_hwinit(info) < 0) {
                printk("%s: hardware init failed for port %d\n",__FUNCTION__, i);
                goto irqerr;
            }
            
            local_irq_restore(flags);
            ifx_int_wrapper.disable(info->txirq);
            ifx_int_wrapper.disable(info->rxirq);
            ifx_int_wrapper.disable(info->errirq);
            
            /* These will get enabled in the open call */
        }
    }

    /* register /proc read handler */
    create_proc_read_entry("driver/ifx_ssc", 0, NULL, ifx_ssc_read_proc, NULL);
    return 0;

irqerr:
    for (t=0; t<i; t++) {
        ifx_int_wrapper.free(isp[t].txirq,&isp[t]);
        ifx_int_wrapper.free(isp[t].rxirq,&isp[t]);
        ifx_int_wrapper.free(isp[t].errirq,&isp[t]);
#ifdef SSC_FRAME_INT_ENABLE
        ifx_int_wrapper.free(isp[t].frmirq, &isp[t]);
#endif
    }
dma_errout:
    dma_device_unregister(info->dma_dev);
    dma_device_release(info->dma_dev);
errout:
    /* free up any allocated memory in the error case */
    kfree(isp);
    return (ret_val);
}


/**
 * SSC Module Cleanup.
 * Upon removal of the SSC module this function will free all allocated resources
 * and unregister devices. 
 * 
 * \return  none
 * \ingroup Internal
 */
void ifx_ssc_cleanup_module(void)
{
    int i;

    /* free up any allocated memory */
    for (i = 0; i < IFX_SSC_MAX_PORT_NUM; i++)
    {
        /* disable the SSC */
        SSC_WRITE_REG(IFX_SSC_WHBSTATE_CLR_ENABLE,
                      isp[i].mapbase + IFX_SSC_WHBSTATE);
        /* free the interrupts */
        ifx_int_wrapper.free(isp[i].txirq, &isp[i]);
        ifx_int_wrapper.free(isp[i].rxirq, &isp[i]);
        ifx_int_wrapper.free(isp[i].errirq, &isp[i]);
#ifdef SSC_FRAME_INT_ENABLE
        ifx_int_wrapper.free(isp[i].frmirq, &isp[i]);
#endif
        if (isp[i].rxbuf != NULL)
            kfree(isp[i].rxbuf);
        if (isp[i].txbuf != NULL)
            kfree(isp[i].txbuf);
    }
    kfree(isp);
    /* unregister the device */
    if (unregister_chrdev(maj, "ifx_ssc"))
    {
        printk("Unable to unregister major %d for the SSC\n", maj);
    }
    /* delete /proc read handler */
    remove_proc_entry("driver/ifx_ssc", NULL);
}


#ifndef MODULE
/**
 * Set Major Number.
 * Set variable maj in case driver is not compiled as module.
 *
 * \param  str Pointer to major number string
 * \return 1
 * \ingroup Internal
 */
static int __init ifx_ssc_set_maj(char *str)
{
    maj = simple_strtol(str, NULL, 0);
    return 1;
}
__setup("ssc_maj=", ifx_ssc_set_maj);
#endif /* !MODULE */

/**
 * Chip select enable.
 *
 * \param   pin Selected CS pin
 * \ingroup API
 */
inline int ifx_ssc_cs_low(u32 pin)
{
    int ret=0;
    if ((ret=ifx_ssc_ioctl((struct inode *)0, NULL,IFX_SSC_GPO_OUT_CLR, 
                           (unsigned long)&pin))) {
        IFX_SSC_EMSG("clear CS %d fails\n",pin);
    }
    wmb();
    return ret;
}

/**
 * Cip select disable.
 *
 * \param   pin Selected CS pin
 * \ingroup API
 */
inline int ifx_ssc_cs_high(u32 pin)
{
    int ret=0;
    /* FIXME: Make it work with several lines */
    if ((ret=ifx_ssc_ioctl((struct inode *)0, NULL,IFX_SSC_GPO_OUT_SET, 
                           (unsigned long)&pin))) {
        IFX_SSC_EMSG("set CS %d fails\n", pin);
    }
    wmb();
    return ret;
}

/**
 * SSC session.
 * Session_mode: IFX_SSC_MODE_RXTX or IFX_SSC_MODE_TX
 * 0. copy data to internal buffer 
 * 1. Write command
 * 2a. If SSC_SESSION_MODE_TXONLY, read tx_len data
 * 2b. If not Read back (tx_len + rx_len) data
 * 3. copy internal buffer to rx buf if necessary
 *
 * \param  tx_buf Pointer to Tx buffer
 * \param  tx_len Size of Tx buffer
 * \param  rx_buf Pointer to Rx buffer
 * \param  rx_len Size of Rx buffer
 * \return >=0    number of bytes received (if rx_buf != 0) or transmitted
 * \return <0     error code
 * \ingroup Internal
 */
static int ifx_ssc_session(char * tx_buf, u32 tx_len, char * rx_buf, u32 rx_len)
{
    int ret=0;

    char * ssc_tx_buf = NULL;
    char * ssc_rx_buf = NULL;

    int eff_size = 0;
    u8 mode = 0;
    
    if (tx_buf == NULL && tx_len ==0 && rx_buf == NULL && rx_len == 0) {
        IFX_SSC_EMSG("invalid parameters\n");
        ret=-EINVAL;
        goto ssc_session_exit;
    } else if (tx_buf == NULL || tx_len == 0) {
        if (rx_buf != NULL && rx_len != 0) {
            mode = IFX_SSC_MODE_RX;
        } else {
            IFX_SSC_EMSG("invalid parameters\n");
            ret =- EINVAL;
            goto ssc_session_exit;
        }
    } else if (rx_buf == NULL || rx_len ==0) {
        if (tx_buf != NULL && tx_len != 0) {
            mode = IFX_SSC_MODE_TX;
        } else {
            IFX_SSC_EMSG("invalid parameters\n");
            ret =- EINVAL;
            goto ssc_session_exit;
        }
    } else {
        mode = IFX_SSC_MODE_RXTX;
    }
    
    if (mode == IFX_SSC_MODE_RXTX) {
        eff_size = tx_len + rx_len;
    } else if (mode == IFX_SSC_MODE_RX) {
        eff_size = rx_len;
    } else {
        eff_size = tx_len;
    }

    /* 4 bytes alignment,  required by driver */
#if 0 /* CHECKME */
    if (in_irq()){
#endif
    if (1) {
        ssc_tx_buf = (char*) kmalloc(sizeof(char) * ((eff_size + 3) & (~3)), 
                                     GFP_ATOMIC);
        ssc_rx_buf = (char*) kmalloc(sizeof(char) * ((eff_size + 3) & (~3)), 
                                     GFP_ATOMIC);
    } else {
        ssc_tx_buf = (char*) kmalloc(sizeof(char) * ((eff_size + 3) & (~3)), 
                                     GFP_KERNEL);
        ssc_rx_buf = (char*) kmalloc(sizeof(char) * ((eff_size + 3) & (~3)),
                                     GFP_KERNEL);
    }
    if (ssc_tx_buf == NULL || ssc_rx_buf == NULL) {
        IFX_SSC_EMSG("no memory for size of %d\n", eff_size);
        ret = -ENOMEM;
        goto ssc_session_exit;
    }
    memset((void*)ssc_tx_buf, 0, eff_size);
    memset((void*)ssc_rx_buf, 0, eff_size);

    if (tx_len>0) {
        memcpy(ssc_tx_buf, tx_buf, tx_len);
    }
    
    ret = ifx_ssc_kwrite(0, ssc_tx_buf, eff_size);

    if (ret > 0) {
        ssc_tx_buf = NULL; /* should be freed by ifx_ssc_kwrite */
    }
        
    if (ret != eff_size) {
        IFX_SSC_EMSG("ifx_ssc_write return %d\n",ret); 
        goto ssc_session_exit;
    }
    ret=ifx_ssc_kread(0, ssc_rx_buf,eff_size);
    if (ret != eff_size) {
        IFX_SSC_EMSG("ifx_ssc_read return %d\n",ret); 
        goto ssc_session_exit;
    }
    
    memcpy(rx_buf, ssc_rx_buf+tx_len, rx_len);
    
    if (mode == IFX_SSC_MODE_TX) {
        ret = tx_len;
    } else {
        ret = rx_len;
    }

ssc_session_exit:
    if (ssc_tx_buf != NULL) kfree(ssc_tx_buf);
    if (ssc_rx_buf != NULL) kfree(ssc_rx_buf);

    if (ret<0) {
        printk("ssc session fails\n");
    }
    return ret;
}

/**
 * TX-RX session.
 * 1. TX session
 * 2. RX session
 * 
 * \param   tx_buf Pointer to Tx buffer
 * \param   tx_len Lenght of Tx buffer
 * \param   rx_buf Pointer to Rx buffer
 * \param   rx_len Length of Rx Buffer
 * \return  >=0    number of bytes received
 * \return  <0     error code
 * \ingroup API
 */
int ifx_ssc_txrx(char * tx_buf, u32 tx_len, char * rx_buf, u32 rx_len)
{
    return ifx_ssc_session(tx_buf,tx_len,rx_buf,rx_len);
}

/**
 * TX-only session.
 * 1. TX session
 * 
 * \param   tx_buf Pointer to Tx buffer
 * \param   tx_len Length of Tx buffer
 * \return  >=0    number of bytes received
 * \return  <0     error code
 * \ingroup API
 */
int ifx_ssc_tx(char * tx_buf, u32 tx_len)
{
    return ifx_ssc_session(tx_buf,tx_len,NULL,0);
}

/**
 * RX-only session.
 * 1. TX session
 * 
 * \param   rx_buf Pointer to Tx buffer
 * \param   rx_len Pointer to Rx buffer
 * \return  >=0    number of bytes received
 * \return  <0     error code
 * \ingroup API
 */
int ifx_ssc_rx(char * rx_buf, u32 rx_len)
{
    return ifx_ssc_session(NULL,0,rx_buf,rx_len);
}

module_exit(ifx_ssc_cleanup_module);
module_init(ifx_ssc_init);

EXPORT_SYMBOL(ifx_ssc_open);
EXPORT_SYMBOL(ifx_ssc_close);
EXPORT_SYMBOL(ifx_ssc_register_frm_callback);
EXPORT_SYMBOL(ifx_ssc_unregister_frm_callback);
EXPORT_SYMBOL(ifx_ssc_kread);
EXPORT_SYMBOL(ifx_ssc_kwrite);
EXPORT_SYMBOL(ifx_ssc_ioctl);
EXPORT_SYMBOL(ifx_ssc_cs_low);
EXPORT_SYMBOL(ifx_ssc_cs_high);
EXPORT_SYMBOL(ifx_ssc_txrx);
EXPORT_SYMBOL(ifx_ssc_tx);
EXPORT_SYMBOL(ifx_ssc_rx);

EXPORT_SYMBOL(sidecar_ifx_wait_sleep);

