/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

 /******************************************************************************
 * Common Head File
 ******************************************************************************/  

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/unistd.h>
#include <asm/irq.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/fs.h> 
#include <linux/slab.h>
#include <linux/delay.h>

#include <asm/system.h>
#include <asm/io.h>
#include <asm/r4kcache.h>
#include <asm/cacheflush.h>
#include <linux/mm.h>

#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/irq.h>
#include <asm/incaip2/incaip2_pm.h>
#include <asm/mipsregs.h>
#include <asm/stackframe.h>
#include <asm/time.h>

#ifdef CONFIG_PROC_FS
#include <linux/proc_fs.h>
#endif

#define __do_five_nops			\
do {							\
	__asm__ __volatile__(		\
	"ssnop\n\t"					\
	"ssnop\n\t"					\
	"ssnop\n\t"					\
	"ssnop\n\t"					\
	"ssnop\n\t"					\
	);							\
} while (0)

static int ifx_pm_open(struct inode *, struct file *);
static int ifx_pm_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);
static int ifx_pm_release(struct inode *inode, struct file *filp);

volatile struct mips_cpu *ifx_pm_mips_register_cpu0; 
volatile struct mips_jtlb_entry *ifx_pm_jtlb;

unsigned int incaip2_freq_div_pm_current;
unsigned int incaip2_freq_div_pm_new; 	/* indicates new frequency setting */
								/* needed to adjust CPU-timer to 	*/
								/* new frequency (in time.c)		*/

 static struct file_operations pm_fops = {
   open    : ifx_pm_open,
   release : ifx_pm_release,
   ioctl   : ifx_pm_ioctl,
};

static struct miscdevice pm_miscdev = {
    MISC_DYNAMIC_MINOR,
    "pm",
    &pm_fops,
    NULL,
    NULL,
    NULL
}; 
 

volatile u32 ifx_pm_stack_pointer_save=0;
volatile u32 ifx_pm_mps_rvec_save=0;
volatile int ifx_pm_debug_flagger=0;
volatile int ifx_pm_reset_stop_flag = 1;
static u32 ifx_pm_regs_mem_backup[32];
static int ifx_pm_index = 0;
static int ifx_pm_param = 0;
static u32 ifx_pm_reg = 0;


#define Fetch_Lock_Tag_I	0x1c
static inline void fetch_lock_icache32(unsigned long addr)
{
	unsigned long start = addr;
	unsigned long end = start + PAGE_SIZE;

	do {
		cache32_unroll32(start,Fetch_Lock_Tag_I);
		start += 0x400;
	} while (start < end);
}

 /**
 * Store CP0 registers
 * \ingroup Internal  
 */  
void ifx_pm_store_cp0_register(void)
{

	ifx_pm_mips_register_cpu0->CP0_reg.index 	= __read_32bit_c0_register($0, 0);
	ifx_pm_mips_register_cpu0->CP0_reg.entry_lo0	= __read_32bit_c0_register($2, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.entry_lo1	= __read_32bit_c0_register($3, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.context	= __read_32bit_c0_register($4, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.page_mask	= __read_32bit_c0_register($5, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.wired	= __read_32bit_c0_register($6, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.hwrena	= __read_32bit_c0_register($7, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.bad_vaddr	= __read_32bit_c0_register($8, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.count	= __read_32bit_c0_register($9, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.entry_hi	= __read_32bit_c0_register($10, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.compare	= __read_32bit_c0_register($11, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.status	= __read_32bit_c0_register($12, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.intctl	= __read_32bit_c0_register($12, 1);	
	ifx_pm_mips_register_cpu0->CP0_reg.srsctl	= __read_32bit_c0_register($12, 2);	
	ifx_pm_mips_register_cpu0->CP0_reg.srsmap	= __read_32bit_c0_register($12, 3);	
	ifx_pm_mips_register_cpu0->CP0_reg.cause	= __read_32bit_c0_register($13, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.epc		= __read_32bit_c0_register($14, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.pid		= __read_32bit_c0_register($15, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.ebase	= __read_32bit_c0_register($15, 1);		
	ifx_pm_mips_register_cpu0->CP0_reg.config0	= __read_32bit_c0_register($16, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.config7	= __read_32bit_c0_register($16, 7);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchlo0	= __read_32bit_c0_register($18, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchlo1	= __read_32bit_c0_register($18, 1);
	ifx_pm_mips_register_cpu0->CP0_reg.watchlo2	= __read_32bit_c0_register($18, 2);
	ifx_pm_mips_register_cpu0->CP0_reg.watchlo3	= __read_32bit_c0_register($18, 3);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchhi0	= __read_32bit_c0_register($19, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchhi1	= __read_32bit_c0_register($19, 1);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchhi2	= __read_32bit_c0_register($19, 2);	
	ifx_pm_mips_register_cpu0->CP0_reg.watchhi3	= __read_32bit_c0_register($19, 3);	
	ifx_pm_mips_register_cpu0->CP0_reg.debug	= __read_32bit_c0_register($23, 0);
	ifx_pm_mips_register_cpu0->CP0_reg.depc		= __read_32bit_c0_register($24, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.perfcount0= __read_32bit_c0_register($25, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.perfcount1= __read_32bit_c0_register($25, 1);	
	ifx_pm_mips_register_cpu0->CP0_reg.perfcount2= __read_32bit_c0_register($25, 2);	
	ifx_pm_mips_register_cpu0->CP0_reg.perfcount3= __read_32bit_c0_register($25, 3);	
	ifx_pm_mips_register_cpu0->CP0_reg.errctl	= __read_32bit_c0_register($26, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.cacheerr	= __read_32bit_c0_register($27, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.taglo0	= __read_32bit_c0_register($28, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.taglo1	= __read_32bit_c0_register($28, 2);	
	ifx_pm_mips_register_cpu0->CP0_reg.taglo2	= __read_32bit_c0_register($28, 4);	
	ifx_pm_mips_register_cpu0->CP0_reg.datalo0	= __read_32bit_c0_register($28, 1);
	ifx_pm_mips_register_cpu0->CP0_reg.datalo1	= __read_32bit_c0_register($28, 3);	
	ifx_pm_mips_register_cpu0->CP0_reg.datalo2	= __read_32bit_c0_register($28, 5);	
	ifx_pm_mips_register_cpu0->CP0_reg.datahi0	= __read_32bit_c0_register($29, 1);	
	ifx_pm_mips_register_cpu0->CP0_reg.errorepc	= __read_32bit_c0_register($30, 0);	
	ifx_pm_mips_register_cpu0->CP0_reg.desave	= __read_32bit_c0_register($31, 0);	

}
 
 /**
 * Retore CP0 registers
 * \ingroup Internal  
 */  
void ifx_pm_restore_cp0_register(void)
{ 

	__write_32bit_c0_register($0,  0, ifx_pm_mips_register_cpu0->CP0_reg.index);
	__write_32bit_c0_register($2,  0, ifx_pm_mips_register_cpu0->CP0_reg.entry_lo0);	
	__write_32bit_c0_register($3,  0, ifx_pm_mips_register_cpu0->CP0_reg.entry_lo1);	
	__write_32bit_c0_register($4,  0, ifx_pm_mips_register_cpu0->CP0_reg.context);	
	__write_32bit_c0_register($5,  0, ifx_pm_mips_register_cpu0->CP0_reg.page_mask);	
	__write_32bit_c0_register($6,  0, ifx_pm_mips_register_cpu0->CP0_reg.wired);	
	__write_32bit_c0_register($7,  0, ifx_pm_mips_register_cpu0->CP0_reg.hwrena);	
//	__write_32bit_c0_register($8,  0, ifx_pm_mips_register_cpu0->CP0_reg.bad_vaddr);	
//	__write_32bit_c0_register($9,  0, ifx_pm_mips_register_cpu0->CP0_reg.count);	
	__write_32bit_c0_register($10, 0, ifx_pm_mips_register_cpu0->CP0_reg.entry_hi);	
	__write_32bit_c0_register($11, 0, ifx_pm_mips_register_cpu0->CP0_reg.compare);	
	__write_32bit_c0_register($12, 0, ifx_pm_mips_register_cpu0->CP0_reg.status);	
	__write_32bit_c0_register($12, 1, ifx_pm_mips_register_cpu0->CP0_reg.intctl);	
	__write_32bit_c0_register($12, 2, ifx_pm_mips_register_cpu0->CP0_reg.srsctl);	
	__write_32bit_c0_register($12, 3, ifx_pm_mips_register_cpu0->CP0_reg.srsmap);	
	__write_32bit_c0_register($13, 0, ifx_pm_mips_register_cpu0->CP0_reg.cause);	
	__write_32bit_c0_register($14, 0, ifx_pm_mips_register_cpu0->CP0_reg.epc);	
//	__write_32bit_c0_register($15, 0, ifx_pm_mips_register_cpu0->CP0_reg.pid);	
	__write_32bit_c0_register($15, 1, ifx_pm_mips_register_cpu0->CP0_reg.ebase);		
	__write_32bit_c0_register($16, 0, ifx_pm_mips_register_cpu0->CP0_reg.config0);	
	__write_32bit_c0_register($16, 7, ifx_pm_mips_register_cpu0->CP0_reg.config7);	
	__write_32bit_c0_register($18, 0, ifx_pm_mips_register_cpu0->CP0_reg.watchlo0);	
	__write_32bit_c0_register($18, 1, ifx_pm_mips_register_cpu0->CP0_reg.watchlo1);
	__write_32bit_c0_register($18, 2, ifx_pm_mips_register_cpu0->CP0_reg.watchlo2);
	__write_32bit_c0_register($18, 3, ifx_pm_mips_register_cpu0->CP0_reg.watchlo3);	
	__write_32bit_c0_register($19, 0, ifx_pm_mips_register_cpu0->CP0_reg.watchhi0);	
	__write_32bit_c0_register($19, 1, ifx_pm_mips_register_cpu0->CP0_reg.watchhi1);	
	__write_32bit_c0_register($19, 2, ifx_pm_mips_register_cpu0->CP0_reg.watchhi2);	
	__write_32bit_c0_register($19, 3, ifx_pm_mips_register_cpu0->CP0_reg.watchhi3);	
	__write_32bit_c0_register($23, 0, ifx_pm_mips_register_cpu0->CP0_reg.debug);
	__write_32bit_c0_register($24, 0, ifx_pm_mips_register_cpu0->CP0_reg.depc);	
	__write_32bit_c0_register($25, 0, ifx_pm_mips_register_cpu0->CP0_reg.perfcount0);	
	__write_32bit_c0_register($25, 1, ifx_pm_mips_register_cpu0->CP0_reg.perfcount1);	
	__write_32bit_c0_register($25, 2, ifx_pm_mips_register_cpu0->CP0_reg.perfcount2);	
	__write_32bit_c0_register($25, 3, ifx_pm_mips_register_cpu0->CP0_reg.perfcount3);	
	__write_32bit_c0_register($26, 0, ifx_pm_mips_register_cpu0->CP0_reg.errctl);	
	__write_32bit_c0_register($27, 0, ifx_pm_mips_register_cpu0->CP0_reg.cacheerr);	
	__write_32bit_c0_register($28, 0, ifx_pm_mips_register_cpu0->CP0_reg.taglo0);	
	__write_32bit_c0_register($28, 2, ifx_pm_mips_register_cpu0->CP0_reg.taglo1);	
	__write_32bit_c0_register($28, 4, ifx_pm_mips_register_cpu0->CP0_reg.taglo2);	
	__write_32bit_c0_register($28, 1, ifx_pm_mips_register_cpu0->CP0_reg.datalo0);
	__write_32bit_c0_register($28, 3, ifx_pm_mips_register_cpu0->CP0_reg.datalo1);	
	__write_32bit_c0_register($28, 5, ifx_pm_mips_register_cpu0->CP0_reg.datalo2);	
	__write_32bit_c0_register($29, 1, ifx_pm_mips_register_cpu0->CP0_reg.datahi0);	
	__write_32bit_c0_register($30, 0, ifx_pm_mips_register_cpu0->CP0_reg.errorepc);	
	__write_32bit_c0_register($31, 0, ifx_pm_mips_register_cpu0->CP0_reg.desave);	

}

 /**
 * Store MIPS registers
 * This function stores all MIPS general purpose registers in onchip memory
 * \ingroup Internal  
 */  
void inline ifx_pm_store_mips_regs(void)
{
	__asm__ __volatile__(
	".set noreorder \n"		
	"sw  $29,	%0\n"
	: "=m" (ifx_pm_stack_pointer_save));	

     ifx_pm_regs_mem_backup[0] = ifx_pm_stack_pointer_save;
     
	/* store MIPS gp registers*/
     __asm__ __volatile__(
     "lui $29, 0xbf10 \n"
     "ori $29, 0x25e0 \n"
     "lw  $29, 0($29) \n"
	"sw  $1,	0x4($29)\n"		
	"sw  $2,	0x8($29)\n"		
	"sw  $3, 	0xc($29)\n"		
	"sw  $4, 	0x10($29)\n"		
	"sw  $5, 	0x14($29)\n"		
	"sw  $6, 	0x18($29)\n"		
	"sw  $7, 	0x1c($29)\n"		
	"sw  $8, 	0x20($29)\n"		
	"sw  $9, 	0x24($29)\n"		
	"sw  $10, 0x28($29)\n"		
	"sw  $11, 0x2c($29)\n"		
	"sw  $12,	0x30($29)\n"		
	"sw  $13, 0x34($29)\n"		
	"sw  $14, 0x38($29)\n"		
	"sw  $15, 0x3c($29)\n"		
	"sw  $16, 0x40($29)\n"		
	"sw  $17, 0x44($29)\n"		
	"sw  $18, 0x48($29)\n"		
	"sw  $19, 0x4c($29)\n"
	"sw  $20, 0x50($29)\n"
	"sw  $21, 0x54($29)\n"
	"sw  $22,	0x58($29)\n"		
	"sw  $23, 0x5c($29)\n"		
	"sw  $24, 0x60($29)\n"		
	"sw  $25, 0x64($29)\n"		
	"sw  $26, 0x68($29)\n"		
	"sw  $27, 0x6c($29)\n"		
	"sw  $28, 0x70($29)\n"		
	"sw  $30, 0x74($29)\n"
	"sw  $31, 0x78($29)\n"
	);

	__asm__ __volatile__(
	"lw	$29,	%0\n"
	".set reorder\n"
	: "=m" (ifx_pm_stack_pointer_save));	
     
}

 /**
 * Restore MIPS registers
 * This function restores all MIPS general purpose registers from onchip memory
 * \ingroup Internal  
 */  
void inline ifx_pm_restore_mips_regs(void)
{
	/* restore MIPS gp registers*/
     __asm__ __volatile__(
	".set noreorder\n"		
     "lui $29, 0xbf10 \n"
     "ori $29, 0x25e0 \n"
     "lw  $29, 0($29) \n"
//	"lw  $1,	0x4($29)\n"		
//	"lw  $2,	0x8($29)\n"		
	"lw  $3, 	0xc($29)\n"		
	"lw  $4, 	0x10($29)\n"		
	"lw  $5, 	0x14($29)\n"		
	"lw  $6, 	0x18($29)\n"		
	"lw  $7, 	0x1c($29)\n"		
	"lw  $8, 	0x20($29)\n"		
	"lw  $9, 	0x24($29)\n"		
	"lw  $10, 0x28($29)\n"		
	"lw  $11, 0x2c($29)\n"		
	"lw  $12,	0x30($29)\n"		
	"lw  $13, 0x34($29)\n"		
	"lw  $14, 0x38($29)\n"		
	"lw  $15, 0x3c($29)\n"		
	"lw  $16, 0x40($29)\n"		
	"lw  $17, 0x44($29)\n"		
	"lw  $18, 0x48($29)\n"		
	"lw  $19, 0x4c($29)\n"
	"lw  $20, 0x50($29)\n"
	"lw  $21, 0x54($29)\n"
	"lw  $22,	0x58($29)\n"		
	"lw  $23, 0x5c($29)\n"		
	"lw  $24, 0x60($29)\n"		
	"lw  $25, 0x64($29)\n"		
	"lw  $26, 0x68($29)\n"		
	"lw  $27, 0x6c($29)\n"		
	"lw  $28, 0x70($29)\n"		
	"lw  $30, 0x74($29)\n"
	"lw  $31, 0x78($29)\n"

	"lw  $29, 0x0($29)\n"
	".set reorder\n"
	);

}

/**
 * Set a new CPU frequency.
 * To change the frequency the following needs to be done:
 * - adjust the clock divider register 
 * - save MIPS registers, CP0 registers and TLB
 * - trigger a software reset
 * - restore MIPS registers, CP0 registers and TLB
 *  
 * \ingroup Internal  
 */ 
void ifx_pm_change_frequency(int param)
{
     if ((param < 1) || (param > 4)) return;
     if (param == incaip2_freq_div_pm_current) return;
     if ((INCA_IP2_MPS_CHIPID_PARTNUM_GET(*INCA_IP2_MPS_CHIPID) == 0x11a) && (param == 1) ) {
          printk("Chip version does not support selected frequency\n");
          return;
     }
     ifx_pm_param=param;

//printk("TRACE %s %s %d\n",__FILE__,__FUNCTION__,__LINE__);
   	/* stop incomming Eth-packages via CPU-Port */
	ifx_pm_mips_register_cpu0->mips_reg.st_pt_reg= *((volatile u32*) ST_PT_REG);
	*((volatile u32*) ST_PT_REG) = *((volatile u32*) ST_PT_REG) & ~(3 << 4);

	/* disable interrupts */
	save_and_cli(ifx_pm_mips_register_cpu0->mips_reg.flags);

     /* save CPU1 RVEC value */
     ifx_pm_mps_rvec_save=*INCA_IP2_MPS_CPU1_RVEC;
     *INCA_IP2_MPS_CPU1_RVEC=(u32)ifx_pm_regs_mem_backup;

     ifx_pm_mips_register_cpu0->mips_reg.soft_rvec=(u32)&&restart_vector;

     
	/* storage of old bootvector */
	ifx_pm_mips_register_cpu0->mips_reg.orig_rvec = *BOOT_RVEC;

     /* new start-adress */
	*BOOT_RVEC = ifx_pm_mips_register_cpu0->mips_reg.soft_rvec;

     /* adjust CPU-frequency */
     switch ( ifx_pm_param )
     {
          case 1: 		/* divider factor to MIPS0-CLK = 1,  freq. = 393 MHz */
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR	& ~(3 << 27);	
               incaip2_freq_div_pm_new = 0;
               break;
                  
          case 2: 		/* divider factor to MIPS0-CLK = 2, freq. = 196,5 MHz */
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR	& ~(3 << 27);	
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR | (1 << 27);
               incaip2_freq_div_pm_new = 1;
               break;		
             
          case 3: 		/* divider factor to MIPS0-CLK = 3, freq. = 131 MHz */
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR	& ~(3 << 27);	
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR | (2 << 27);
               incaip2_freq_div_pm_new = 2;
               break;	
             
          case 4: 		/* divider factor to MIPS0-CLK = 1,5, freq. = 262 MHz */
               *INCA_IP2_CGU_PLL2CR = *INCA_IP2_CGU_PLL2CR | (3 << 27);
               incaip2_freq_div_pm_new = 3;			
               break;
     }
          
	/* store TLB */
	for (ifx_pm_index = 0; ifx_pm_index <= 15; ifx_pm_index++)
	{	
		__write_32bit_c0_register($0,0,ifx_pm_index);
          __do_five_nops;
          tlb_read();
          (ifx_pm_jtlb[ifx_pm_index]).entry_hi  = __read_32bit_c0_register($10,0);
		(ifx_pm_jtlb[ifx_pm_index]).entry_lo0 = __read_32bit_c0_register($2,0);		
		(ifx_pm_jtlb[ifx_pm_index]).entry_lo1 = __read_32bit_c0_register($3,0);
		(ifx_pm_jtlb[ifx_pm_index]).pagemask  = __read_32bit_c0_register($5,0);
	}

     /* store CPU-coprocessor-registers */
	ifx_pm_store_cp0_register();

     ifx_pm_store_mips_regs();

     __flush_cache_all();

     fetch_lock_icache32((unsigned long) &&reset_label);
#ifdef EBU_FLUSHING
     /* dummy read on EBU to flush any pending transactions
      * this address is not mapped on the WP_LO phones
      */

     *((volatile u32*) 0xb0000000) = 0x00000000;
     ifx_pm_reg = *((volatile u32*) 0xb0000000);

#endif
     __asm__("sync");

               
reset_label:
     *INCA_IP2_RCU_RST_REQ = *INCA_IP2_RCU_RST_REQ | (1 << 31);
     __asm__("sync");
     do {__asm__("wait"); } while (ifx_pm_reset_stop_flag) ;

restart_vector:
     while (ifx_pm_debug_flagger);
          
     ifx_pm_restore_mips_regs();

     /* restore CPU-coprocessor-registers */
     ifx_pm_restore_cp0_register();

     /* restore TLB */
     for (ifx_pm_index = 0; ifx_pm_index <= 15; ifx_pm_index++)
     {
          __write_32bit_c0_register($0,0,ifx_pm_index);
          __do_five_nops;
          __write_32bit_c0_register($10,0,ifx_pm_jtlb[ifx_pm_index].entry_hi);
          __write_32bit_c0_register($2,0,ifx_pm_jtlb[ifx_pm_index].entry_lo0);			
          __write_32bit_c0_register($3,0,ifx_pm_jtlb[ifx_pm_index].entry_lo1);			
          __write_32bit_c0_register($5,0,ifx_pm_jtlb[ifx_pm_index].pagemask);
          tlb_write_indexed();
     }
     
     
     *INCA_IP2_MPS_CPU1_RVEC=ifx_pm_mps_rvec_save;
          
     *BOOT_RVEC = ifx_pm_mips_register_cpu0->mips_reg.orig_rvec;
     incaip2_freq_div_pm_current=ifx_pm_param;
          
     /* enable interrupts */
     restore_flags(ifx_pm_mips_register_cpu0->mips_reg.flags);

     /* allow incoming ethernet-packets */	
     *((volatile u32*) ST_PT_REG) =	ifx_pm_mips_register_cpu0->mips_reg.st_pt_reg;

     return;
}


/**
 * IOCTL Handler
 * This function handles all ioctl calls.
 *
 * \param   inode	struct inode *, file descriptor on drive
 * \param   file 	struct file *, file descriptor of virtual file system
 * \param   cmd  	unsigned int, device specific commands
 *		1. IFX_CPU0_FREQUENCY_SET 	- Set the frequency of CPU0		
 *
 * \param arg --- unsigned long, pointer to structure pm_ioctl_param to pass
 *              arguments or store return value. Members are listed below:		
 *		1. flag			flag is used to adjust the intended frequency.
 *					1:	CPU is running with 393 MHz
 *					2:	CPU is running with 196,5 MHz	
 *					3: 	CPU is running with 131 MHz	
 *					4: 	CPU is running with 262 MHz
 *
 * \return  0       OK, IOCTRL success
 * \return  -EFAULT 		Error on copying data to user space
 * \return  -ENOIOCTLCMD    An invalid command was specified
 * \ingroup API
 */
static int ifx_pm_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{

   int 				ret = 0;	/* return value of function */
   unsigned long 	param;		/* user-parameter for frequency-adjustment */


//printk("TRACE %s %s %d\n",__FILE__,__FUNCTION__,__LINE__);

    switch ( cmd ) {
    case IFX_CPU0_FREQUENCY_SET: 		/* adjustment of a new frequency */
         if ((INCA_IP2_MPS_CHIPID_PARTNUM_GET(*INCA_IP2_MPS_CHIPID) == 0x11a) && (param == 1) ) {
              printk("Chip version does not support selected frequency\n");
              return -EFAULT;
         }
         /* check validity user space pointer */
         if ( !access_ok(VERIFY_READ, arg, sizeof(unsigned long)) )
              return -EFAULT;
         copy_from_user(&param, (void*)arg, sizeof(param));
         /* adjustment of new CPU-frequency */
         ifx_pm_change_frequency((int) param);
         break;
		  
    case IFX_CPU0_FREQUENCY_GET:				/* get frequency setting	*/
         /* check validity user space pointer */
         if ( !access_ok(VERIFY_WRITE, arg, sizeof(unsigned long)) )
              return -EFAULT;
         copy_to_user((void*)arg, &incaip2_freq_div_pm_current, sizeof(incaip2_freq_div_pm_current));
         break;
    default:
         ret = -ENOIOCTLCMD;
    }

    return ret;
}


/**
 * Open PM device.
 * This function is called when the pm device is opend.
 *
 * \param   inode   Inode of device 
 * \param   filp    File structure of device
 * \return  0       OK, device opened
 * \ingroup API  
 */
static int ifx_pm_open(struct inode *inode, struct file *filp)
{ 
	/* memory-allocation for CPU-contents and TLB */ 
	ifx_pm_mips_register_cpu0 = (struct mips_cpu *) kmalloc(sizeof(struct mips_cpu),GFP_KERNEL);
	ifx_pm_jtlb = (struct mips_jtlb_entry *) kmalloc((sizeof(struct mips_jtlb_entry)*16), GFP_KERNEL);

	return 0;

}


/**
 * Release PM device.
 * This function is called when the pm device is closed. All allocated memory 
 * is freed here.
 *
 * \param   inode   Inode of device 
 * \param   filp    File structure of device
 * \return  0       OK, device closed
 * \ingroup API  
 */
static int ifx_pm_release(struct inode *inode, struct file *filp)
{
	/* free memory for CP0 registers and TLB */ 
	kfree((const void *) ifx_pm_mips_register_cpu0);
	kfree((const void *) ifx_pm_jtlb);

    return 0;
}

#ifdef CONFIG_PROC_FS
int ifx_pm_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
     long len = 0;
     u32 mode;
     int freq;
    
     len = sprintf(buf,"\nINCA-IP2 power management driver\n");

     mode = (*INCA_IP2_CGU_PLL2CR & (3 << 27))>>27;	
     switch ( mode )
     {
          case 0: 		/* divider factor to MIPS0-CLK = 1,  freq. = 393 MHz */
               freq = 393;
               break;
                  
          case 1: 		/* divider factor to MIPS0-CLK = 2, freq. = 196,5 MHz */
               freq = 196;
               break;		
             
          case 2: 		/* divider factor to MIPS0-CLK = 3, freq. = 131 MHz */
               freq = 131;
               break;	
             
          case 3: 		/* divider factor to MIPS0-CLK = 1,5, freq. = 262 MHz */
               freq = 262;			
               break;
          default:
               freq = 0;			
     }
     if (freq == 0) 
          len=len+sprintf(buf+len,"CPU0 running at unknown frequency\n");
     else
          len=len+sprintf(buf+len,"CPU0 running at %dMHz\n",freq);
     *eof = 1;   // No more data available

     return len;
}
#endif

/**
 * Initialize PM unit.
 * This function registers the PM device
 *
 * \ingroup Internal  
 */
int __init incaip2_pm_init(void)
{

	int ret = 0;
     u32 mode = 0;
	
	printk(KERN_INFO "Inca-IP2 PM initialization\n");

	/* register misc device */
	ret = misc_register(&pm_miscdev);

     if ( ret )
     {
          printk(KERN_ERR "pm: can't misc_register, error %d\n", -ret);
          return ret;
     }
     else
          printk(KERN_INFO "pm: misc_register on minor %d\n", pm_miscdev.minor);

     mode = (*INCA_IP2_CGU_PLL2CR & (3 << 27))>>27;	
     switch ( mode )
     {
          case 0: 		/* divider factor to MIPS0-CLK = 1,  freq. = 393 MHz */
               incaip2_freq_div_pm_current = 1;
               break;
                  
          case 1: 		/* divider factor to MIPS0-CLK = 2, freq. = 196,5 MHz */
               incaip2_freq_div_pm_current = 2;
               break;		
             
          case 2: 		/* divider factor to MIPS0-CLK = 3, freq. = 131 MHz */
               incaip2_freq_div_pm_current = 3;
               break;	
             
          case 3: 		/* divider factor to MIPS0-CLK = 1,5, freq. = 262 MHz */
               incaip2_freq_div_pm_current = 4;			
               break;
          default:
               incaip2_freq_div_pm_current = 0;			
     }
#ifdef CONFIG_PROC_FS
    create_proc_read_entry("driver/ifx_pm", 0, NULL, ifx_pm_read_procmem, NULL);
#endif

     return ret;
}

module_init(incaip2_pm_init);

