
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <asm/semaphore.h>

#ifdef CONFIG_CRYPTO_DEV_IFX_DMA   //deudma
#ifdef CONFIG_CRYPTO_DEV_INCAIP1_DES
#include <asm/incaip/inca-ip.h>
#include <asm/incaip/incaip-deu-structs.h>
//#define DES_3DES_START INCA_IP_DES_3DES
#endif
#ifdef CONFIG_CRYPTO_DEV_INCAIP2_DES
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/incaip2-deu.h>
#include <asm/incaip2/incaip2-deu-structs.h>
//#define DES_3DES_START DES_CON
#endif
#ifdef CONFIG_CRYPTO_DEV_DANUBE_DES
#include <asm/danube/danube.h>
#include <asm/danube/danube_deu.h>
#include <asm/danube/danube_deu_structs.h>
//#define DES_3DES_START DANUBE_DES_CON
#endif
#include "ifxdeu-dma.h"

//#define DEFAULT_DEU_BUFFER_SIZE 16384
#define DEFAULT_DEU_BUFFER_SIZE 32768

struct dma_device_info *_dma_device;
char deu_dma_read_buffer_data[DEFAULT_DEU_BUFFER_SIZE] __attribute__ ((aligned (32)));
u32* deu_dma_test_ptr = &deu_dma_read_buffer_data;
char* deu_dma_read_buffer=deu_dma_read_buffer_data;
volatile int deu_dma_receive_flag=0;
volatile int deu_dma_receive_sleeping_flag=0;
int deu_dma_nbytes = 0;
wait_queue_head_t deu_dma_rx_wait;
struct semaphore deu_dma_sem;

int deu_dma_intr_handler( struct dma_device_info* dma_dev, int status )
{
    int len;
    unsigned int flags;

//printk("TRACE %s %s %d status=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,status);
    switch(status)
    {
        case RCV_INT:
                len = dma_device_read(dma_dev,&deu_dma_read_buffer,NULL);
                up(&deu_dma_sem);
            break;  	  
        case TX_BUF_FULL_INT:
//printk("TRACE %s %s %d TX_BUF_FULL_INT\n",__FILE__,__FUNCTION__,__LINE__);
            break;
        case TRANSMIT_CPT_INT:
//printk("TRACE %s %s %d TRANSMIT_CPT_INT\n",__FILE__,__FUNCTION__,__LINE__);
            break;
        default:
            printk("TRACE %s %s %d UNKNOWN int\n",__FILE__,__FUNCTION__,__LINE__);

    }
    return 0;
}


u8* deu_dma_buffer_alloc(int len, int* byte_offset,void** opt)
{
//printk("TRACE %s %s %d \n",__FILE__,__FUNCTION__,__LINE__);
    *byte_offset=0;
    return deu_dma_read_buffer;
}


int deu_dma_buffer_free(u8* dataptr,void* opt)
{
//printk("TRACE %s %s %d \n",__FILE__,__FUNCTION__,__LINE__);
    return 0;
} 

static int deu_dma_initialized=0;

void deu_dma_init_device(int num_rx_chan, int tx_endianness_mode,int rx_endianness_mode)
{
    int i;
    int ret;
    static int dma_num_inits=0;
    struct dma_device_info *dma_device;

    if (deu_dma_initialized == 1) {
        deu_dma_release();
    }
    sema_init(&deu_dma_sem,0);
    init_waitqueue_head(&deu_dma_rx_wait);
    

//printk("TRACE %s %s %d\n",__FILE__,__FUNCTION__,__LINE__);

    /* 
     * configure switch DMA device acc. to default values in incaip2_dma.h 
     */
    _dma_device = dma_device_reserve("DEU");
    dma_device = _dma_device;
    dma_device->priv = NULL;
    dma_device->tx_weight=1; 
    dma_device->num_tx_chan=1;
    dma_device->num_rx_chan=num_rx_chan;
    dma_device->tx_burst_len=DEFAULT_DEU_TX_BURST_LEN;
    dma_device->rx_burst_len=DEFAULT_DEU_RX_BURST_LEN;
    dma_device->tx_endianness_mode = tx_endianness_mode;
    dma_device->rx_endianness_mode = rx_endianness_mode;
    for(i=0;i<dma_device->num_tx_chan;i++)  /* set tx channel parameters */
    {
        dma_device->tx_chan[i]->tx_weight=1; 
        dma_device->tx_chan[i]->desc_len=DEFAULT_DEU_TX_CHANNEL_DESCR_NUM;  
        dma_device->tx_chan[i]->packet_size=DEFAULT_DEU_BUFFER_SIZE;
    }
    for(i=0;i<dma_device->num_rx_chan;i++)   /* set rx channel parameters */
    {
        dma_device->rx_chan[i]->desc_len=DEFAULT_DEU_RX_CHANNEL_DESCR_NUM;
        dma_device->rx_chan[i]->packet_size=DEFAULT_DEU_BUFFER_SIZE;
    }
    /* 
     * set DMA handler functions for rx-interrupts, 
     * buffer allocation and release
     */
    dma_device->intr_handler=deu_dma_intr_handler;
    dma_device->buffer_alloc=deu_dma_buffer_alloc;
    dma_device->buffer_free=deu_dma_buffer_free;

    ret=dma_device_register(dma_device);
    dma_num_inits++;
    if (ret != 0) printk("dma_device_register failed. ret=%d dma_num_inits=%d\n",ret,dma_num_inits);

    for(i=0;i<dma_device->num_rx_chan;i++)
        dma_device->rx_chan[i]->open(dma_device->rx_chan[i]);
    for(i=0;i<dma_device->num_tx_chan;i++)
        dma_device->tx_chan[i]->open(dma_device->tx_chan[i]);

    deu_dma_initialized=1;
}

void deu_dma_release(void)
{
    struct dma_device_info *dma_device=_dma_device;
    int i;
    int ret;
    static int dma_num_releases=0;
 
    if (deu_dma_initialized == 0) return;
    
	ret = dma_device_unregister(dma_device);
    dma_num_releases++;
    if (ret != 0) printk("dma_device_unregister failed. ret=%d dma_num_releases=%d\n",ret,dma_num_releases);
    dma_device_release(dma_device);

    deu_dma_initialized=0;
}

EXPORT_SYMBOL(deu_dma_init_device);
EXPORT_SYMBOL(deu_dma_release);

#endif
