/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \defgroup API API-Functions */
/** \defgroup Internal Internally used functions */

#ifndef __KERNEL__
#define __KERNEL__
#endif

#include <linux/config.h>

#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
#define MODVERSIONS
#endif


#if defined(MODVERSIONS) && !defined(__GENKSYMS__)
#include <linux/modversions.h>
#endif


#include <linux/module.h>
#include <linux/string.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <linux/mii.h>
#include <asm/uaccess.h>
#include <linux/in.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/ip.h> 
#include <linux/tcp.h> 

/* overrule definition in skbuff.h to disable IP data alignment for RX */
//#define NET_IP_ALIGN 0
#include <linux/skbuff.h>
#include <linux/in6.h>
#include <linux/mm.h>
#include <linux/ethtool.h>
#include <asm/checksum.h>
#include <linux/init.h>

#include <asm/types.h>
#include <asm/addrspace.h>
#include <asm/incaip2/ifx_types.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/switch.h>
#include <asm/incaip2/ifx_dma.h>

#ifdef CONFIG_DEBUG_MINI_BOOT
#define IKOS_MINI_BOOT
#endif


/* debugging */
#undef INCAIP2_SW_DUMP
//#define INCAIP2_SW_DUMP

#define INCAIP2_SW_EMSG(fmt,args...) printk("%s: " fmt, __FUNCTION__ , ##args)

#ifdef CONFIG_INCAIP2_SWITCH_RESERVED_MC
#undef RESERVED_MC_USE_FLOW_ENGINE

#define VLANID_CPU2LAN 4094
#define VLANID_CPU2PC  4093
#define PORTMAP_LAN (1<<0)
#define PORTMAP_PC  (1<<1)
#define PORTMAP_CPU (1<<2)
#define REDIRECTION         1
#define DELETE_ACTION_ENTRY 6
#define STATUS_HEADER_SIZE 8
#define RESERVED_MC_PKT_NUM 6
#else
#define STATUS_HEADER_SIZE 0     /* 0 = disabled, 8 = enabled */
#endif /* CONFIG_INCAIP2_SWITCH_RESERVED_MC */

#define ETHERNET_PACKET_DMA_BUFFER_SIZE (1536-NET_IP_ALIGN)

#define INCAIP2_SW_CHIP_NO 1
#define INCAIP2_SW_CHIP_ID 0
#define INCAIP2_SW_DEVICE_NO 1


#define INCAIP2_MDIO_WRITE(phy_id, reg, value) \
		do { \
			INCAIP2_SW_REG32( SWITCH_MDIO_ACC ) = 0x08000000 | (phy_id << 21) | (reg << 16) | value; \
			while (!(INCAIP2_SW_REG32( SWITCH_MDIO_ACC ) & 0x80000000)); \
		} while (0)

#define INCAIP2_MDIO_READ(phy_id, reg, value) \
		do { \
			u32 tmp; \
			INCAIP2_SW_REG32( SWITCH_MDIO_ACC ) = 0x04000000 | (phy_id << 21) | (reg << 16); \
			do { \
				tmp = INCAIP2_SW_REG32( SWITCH_MDIO_ACC ); \
			} while (!(tmp & 0x80000000)); \
			value = (u16) tmp; \
		} while (0)

s32 ifx_phy_read(u8 chipId, u8 phy_addr, u8 RegOffset, u16 * pwData);
s32 ifx_get_portId_phyId(u8 chipId, u8 portId, u8* phy_adr);
s32 ifx_get_MDIO_phy( u8 chipId, u32 portId, BOOL* int_ext_phy_ptr );

void ifx_switch_do_autoneg_tasklet(unsigned long arg);

DECLARE_TASKLET(ifx_switch_autoneg_tasklet,ifx_switch_do_autoneg_tasklet, 0);
#define SWITCH_GMAC_LG 0x00000010

struct timer_list ifx_switch_timer_handler;

#undef IFX_DEBUG_MDIO_STATE_MACHINE
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
static char* ifx_switch_phy_speed_names[]={ "10MBit/s", "100MBit/s","1000MBits/s"};
static char* ifx_switch_phy_duplex_names[]={ "half duplex", "full duplex"};
static char* ifx_switch_phy_pause_names[]={ "no pause", "with pause"};
#endif


/***************************************** Module Parameters *************************************/
char* mode="bridge";
MODULE_PARM(mode,"s");

#ifdef HAVE_TX_TIMEOUT
static int timeout = 10*HZ;
MODULE_PARM(timeout, "i");
#endif

int ifx_probed=0; 
int ifx_chip_init=0;

u32 ifx_switch_dma_rx=0;
u32 ifx_switch_dma_alloc=0;
u32 ifx_switch_dma_rx_err=0;
u32 ifx_switch_dma_alloc_drop=0;
u32 ifx_switch_dma_free=0;
u32 ifx_switch_dma_tx=0;
u32 ifx_switch_dma_tx_full=0;
u32 ifx_switch_dma_tx_drop=0;
u32 ifx_switch_dma_free_err=0;

extern s32 ifx_add_MAC_table_entry( u8* macAddrptr, BOOL lan_port, BOOL pc_port, BOOL cpu_port,
                                     BOOL criticalFlag, u32 maState );
extern s32 ifx_set_flow_pattern( u8 chipId, u32 flowId, u32* IKeyData, u32* IKeyMask );
extern s32 ifx_set_flow_action_param(u8 chipId,u32 flowId, u32 feature, u32 flowAction);
extern s32 ifx_delete_flow_action_param( u8 chipId, u32 flowId, u32 feature );

#ifdef IKOS_MINI_BOOT
extern s32 incaip2_sw_to_mbx(struct sk_buff* skb);
#endif

/***************************************** Functions  Declaration *********************************/
int ifx_switch_init(struct net_device *dev);
void ifx_switch_tx_timeout (struct net_device *dev);

/***************************************** Global Data *******************************************/
struct net_device switch_devs_array[INCAIP2_SW_DEVICE_NO] = {
    { init: ifx_switch_init }
};
struct net_device* switch_devs=NULL;

struct switch_priv {
    struct net_device_stats stats;
    int rx_packetlen;
    u8 *rx_packetdata;
    int rx_status;
    int tx_packetlen;
#ifdef CONFIG_NET_HW_FLOWCONTROL    
    int fc_bit;
#endif //CONFIG_NET_HW_FLOWCONTROL    
    u8 *tx_packetdata;
    int tx_status;
    struct dma_device_info *dma_device;
    struct sk_buff *skb;
    spinlock_t lock;
    int mdio_phy_addr;
    int current_speed;
    int current_speed_selection;
    int rx_queue_len;
    int full_duplex;
/*    enum duplex current_duplex;  */
};


/**
 * Print hexdump of data.
 * This function prints a hexdump of the given data. If INCAIP2_SW_DUMP is not
 * defined, this function will be do nothing.
 *
 * \param   len      Length of data buffer
 * \param   *data    Pointer to data
 * \ingroup Internal
 */
static inline void ifx_switch_dump_skb(u32 len, char * data)
{
#ifdef INCAIP2_SW_DUMP
	int i;

	for(i=0;i<len;i++)
    {	
		printk("%2.2x ",(u8)(data[i]));
		if (i % 16 == 15)
			printk("\n");
	}
	printk("\n");
#endif
}

/**
 * Add entry to MAC table.
 * This function stores a new MAC address in the internal MAC table of the
 * switch. The MAC address is stored in one 64bit value, where the upper 16bit
 * should be 0x0000 (e.g. 0x00aabbccddeeff11). 
 *
 * \param   entry_value MAC address stored in 64bit
 * \return  0           OK, entry stored
 * \return  -1          Error during store 
 * \ingroup Internal
 */
int ifx_switch_add_mac_table_entry(u64 entry_value)
{
    BOOL lan_port     = FALSE; 
    BOOL pc_port      = FALSE;
    BOOL cpu_port     = TRUE;    
    BOOL criticalFlag = TRUE;
    u32 maState       = STATIC;
    u8 *adr_byte_ptr = (u8*) &entry_value;

/*    
    printk("%s -> INCAIP2_MACTableAddEntry(%02X:%02X:%02X:%02X:%02X:%02X)\n",
           __FUNCTION__, *adr_byte_ptr, *(adr_byte_ptr+1), *(adr_byte_ptr+2),
           *(adr_byte_ptr+3), *(adr_byte_ptr+4),*(adr_byte_ptr+5) );
*/           
    /* add passed mac address to mac table as static entry */
    if( ifx_add_MAC_table_entry( adr_byte_ptr+2,
                                  lan_port, 
                                  pc_port, 
                                  cpu_port,
                                  criticalFlag,
                                  maState ) != 0)
    {
        return -1;   
    }

    return OK;  
}

/**
 * Update GMAC register settings.
 * This funtion update the values in GMAC according to the values in the phy 
 * registers.
 *
 * \param   phy_id     MDIO id of phy
 * \param   gmac_value Current value of GMAC register
 * \ingroup Internal
 */
u32 ifx_switch_update_gmac_autoneg(int phy_id,u32 gmac_value)
{
     int chip_id = 0;
     int port_id = 0;
     u8 phy_duplex=0;
     u8 phy_pause=0;
     u8 phy_speed=0;
     BOOL mode;
     u16 phy_reg4=0;
     u16 phy_reg5=0;
     u16 phy_reg9=0;
     u16 phy_reg10=0;
     u32 reg=0;
     
     ifx_phy_read(chip_id, phy_id, 4, &phy_reg4);
     ifx_phy_read(chip_id, phy_id, 5, &phy_reg5);
     
     ifx_get_MDIO_phy(chip_id,port_id,&mode);
     if (mode==1) {
          /* external phy used => 1000 MBit */
          
          /* Read register #9 for GBit advertisement settings */
          ifx_phy_read(chip_id, phy_id, 9, &phy_reg9);
          
          /* Read register #10 for GBit partner settings */
          ifx_phy_read(chip_id, phy_id, 10, &phy_reg10);
     }
     
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
     printk("TRACE %s %s %d mode=%d\n",__FILE__,__FUNCTION__,__LINE__,mode);
     printk("TRACE %s %s %d PHY Registers: #4=0x%08x #5=0x%08x #9=0x%08x #10=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,phy_reg4,phy_reg5,phy_reg9,phy_reg10);
#endif

     /* Check if advertise and partner agree on pause */
     phy_pause = ((phy_reg4 & phy_reg5) & 0x0400) >> 10;
     
     /* Find the best mode both partners support */
     /* Priority: 1GB-FD, 1GB-HD, 100MB-FD, 100MB-HD, 10MB-FD, 10MB-HD */
     phy_speed = ((((phy_reg9<<2) & phy_reg10) & 0x0c00) >> 6) |
                 (((phy_reg4 & phy_reg5) & 0x01e0) >> 5);
     
     if (phy_speed >= 0x0020) {
          phy_speed = 2;
          phy_duplex = 1;
     }
     else if (phy_speed >= 0x0010) {
          phy_speed = 2;
          phy_duplex = 0;
     }
     else if (phy_speed >= 0x0008) {
          phy_speed = 1;
          phy_duplex = 1;
     }
     else if (phy_speed >= 0x0004) {
          phy_speed = 1;
          phy_duplex = 0;
     }
     else if (phy_speed >= 0x0002) {
          phy_speed = 0;
          phy_duplex = 1;
     }
     else if (phy_speed >= 0x0001) {
          phy_speed = 0;
          phy_duplex = 0;
     }
     if (phy_duplex == 0) phy_pause = 1;
     reg=(gmac_value & ~0x0000006e) | (phy_pause << 6) |
                      (phy_pause << 5) | (phy_duplex << 3) | (phy_speed << 1);
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
//printk("TRACE %s %s %d speed=%d duplex=%d pause=%d reg=0x%08x\n",__FILE__,__FUNCTION__,__LINE__,phy_speed,phy_duplex,phy_pause,reg);
     if ( ((phy_speed & ~0x3)==0)  && ((phy_duplex & ~0x1)==0) && ((phy_pause & ~0x1)==0) ) { 
          printk("TRACE %s %s %d GMAC set to %s %s %s\n",__FILE__,__FUNCTION__,__LINE__,ifx_switch_phy_speed_names[phy_speed],ifx_switch_phy_duplex_names[phy_duplex],ifx_switch_phy_pause_names[phy_pause]);
     }
#endif

     return reg;
     
}

/**
 * Autonegotiation timer task.
 * This funtion is called as tasklet every system tick and checks for link 
 * changes. If a link change has occured it will update certain switch 
 * registers.
 *
 * \param   arg      Not used
 * \ingroup Internal
 */
void ifx_switch_do_autoneg_tasklet(unsigned long arg)
{
     static int link_loss_detected_port0=1;
     static int link_loss_detected_port1=1;
     int chip_id = 0;
     int port_id = 0;
	u32 reg=0;
     u16 phy_reg=0;
     u16 phy_link=0;
     u8 phy_id=0;
     u8 phy_duplex;
     u8 phy_pause;
     u8 phy_speed;
     
     static int counter=0;
     
     counter++;
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
     if ((counter % 500) == 0) printk("TRACE %s %s called %d times\n",__FILE__,__FUNCTION__,counter);
#endif

     /* we need to check both ports */
     
     /* First we check LAN port */
     port_id=0;
     ifx_get_portId_phyId(chip_id, port_id, &phy_id);
     ifx_phy_read(chip_id, phy_id, 0, &phy_reg);
	
     /* Is autoneg enabled? */
     if (phy_reg & 0x1000) {
          reg=INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG);
          if ((reg & SWITCH_GMAC_LG)==0) {
               link_loss_detected_port0 = 1;
          } 
          else {
               if (link_loss_detected_port0) {
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
printk("TRACE %s %s %d link loss detected port %d\n",__FILE__,__FUNCTION__,__LINE__,port_id);
#endif
                    /* transfer phy registers settings to switch */
                    reg=ifx_switch_update_gmac_autoneg(phy_id,reg);
                    INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG)=reg;
                    link_loss_detected_port0 = 0;
               }
          
          }
     }
     /* We are in fixed state */
     else {
          /* Check link status in phy register #1 */
          ifx_phy_read(chip_id, phy_id, 1, &phy_link);
          if (phy_link & 0x0004) {
               /* duplex and speed is stored in phy register #0 */
               phy_speed = ( ((phy_reg & 0x0040) >> 5) | ((phy_reg & 0x2000) >> 13));
               phy_duplex = ((phy_reg & 0x0100) >> 8);
               if (phy_duplex == 0) {
                    phy_pause = 1;
               }
               else {
                    /* Read phy register #4 for pause values */
                    ifx_phy_read(chip_id, phy_id, 4, &phy_reg);
                    phy_pause = ((phy_reg & 0x0400) >> 10);
               }
               
               /* Set bits in GMAC register (Link status is ok) */
               reg=INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG);
               reg=(reg & ~(0x0000007e)) | (phy_pause<<6) | (phy_pause<<5) |
                    (phy_duplex << 3) | (phy_speed << 1) | 0x00000010;
               INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG)=reg;
          }
          else {
               /* Set link status in GMAC register to not ok */
               reg=INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG);
               reg=(reg & ~(0x00000010));
               INCAIP2_SW_REG32(SWITCH_P0_GMAC_REG)=reg;
          }               
     }     
     
    
     /* Now we check PC port */
     port_id = 1;
     ifx_get_portId_phyId(chip_id, port_id, &phy_id);
     /* Read phy register #0 */
     ifx_phy_read(chip_id, phy_id, 0, &phy_reg);

     /* Is autoneg enabled? */
     if (phy_reg & 0x1000) {
          reg=INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG);
          if ((reg & SWITCH_GMAC_LG)==0) {
               link_loss_detected_port1 = 1;
          }
          else {
              if (link_loss_detected_port1) {
#ifdef IFX_DEBUG_MDIO_STATE_MACHINE
printk("TRACE %s %s %d link loss detected port %d\n",__FILE__,__FUNCTION__,__LINE__,port_id);
#endif
                    /* transfer phy registers settings to switch */
                    reg=ifx_switch_update_gmac_autoneg(phy_id,reg);
                    INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG)=reg;
                    link_loss_detected_port1 = 0;
               }
          
          }
     }
     /* We are in fixed state */
     else {
          /* Check link status in phy register #1 */
          ifx_phy_read(chip_id, phy_id, 1, &phy_link);
          if (phy_link & 0x0004) {
               /* duplex and speed is stored in phy register #0 */
               phy_speed = ( ((phy_reg & 0x0040) >> 5) | ((phy_reg & 0x2000) >> 13));
               phy_duplex = ((phy_reg & 0x0100) >> 8);
               if (phy_duplex == 0) {
                    phy_pause = 1;
               }
               else {
                    /* Read phy register #4 for pause values */
                    ifx_phy_read(chip_id, phy_id, 4, &phy_reg);
                    phy_pause = ((phy_reg & 0x0400) >> 10);
               }
               
               /* Set bits in GMAC register (Link status is ok) */
               reg=INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG);
               reg=(reg & ~(0x0000007e)) | (phy_pause<<6) | (phy_pause<<5) |
                    (phy_duplex << 3) | (phy_speed << 1) | 0x00000010;
               INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG)=reg;
          }
          else {
               /* Set link status in GMAC register to not ok */
               reg=INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG);
               reg=(reg & ~(0x00000010));
               INCAIP2_SW_REG32(SWITCH_P1_GMAC_REG)=reg;
          }               
         
     }
     
}

/**
 * Autonegotiation timer task.
 * This funtion is called every system tick and will schedule a tasklet for
 * phy status checks.
 *
 * \param   arg      Not used
 * \ingroup Internal
 */
void ifx_switch_autoneg_timer_handler(unsigned long arg)
{
     static int counter=0;
     
     counter++;
//     if ((counter % 300) == 0) printk("TRACE %s %s called %d times\n",__FILE__,__FUNCTION__,counter);

     if (counter < 3) {
          ifx_switch_timer_handler.expires=jiffies+1*HZ;
          add_timer(&ifx_switch_timer_handler);
          return;
     }
     
     ifx_switch_timer_handler.expires=jiffies+1;
     tasklet_schedule(&ifx_switch_autoneg_tasklet); 
     add_timer(&ifx_switch_timer_handler);
}

/**
 * Initialize switch hardware.
 * This funtion initializes the basic switch registers to enable reception 
 * of packets and set aging time of MAC table entries. 
 * PHY selection and configuration is determined by the kernel configuration 
 * parameters CONFIG_PHY_SKIP and CONFIG_PHY_MV88E1111. By means of this 
 * parameters either the u-boot PHY configuration can be used or the 
 * kernel configuration determines the PHY to be used.
 *
 * \return  0           OK
 * \ingroup Internal
 */
int ifx_switch_chip_init(void)
{
	
#ifdef CONFIG_PHY_SKIP  /* use PHY settings done by u-boot */
        const u16 PHY_PC = 1;
        u16 pc_temp_reg = 0x800;

        printk(KERN_ERR "No PHY Reconfiguration -> using Bootloader PHY Configuration\n");

        // H62899 - NA03939396:lose connectivity of the PC port after upgrade
        printk(KERN_ERR "Turning off PHY_PC -> this is the default for the OpenStage sw\n");
        INCAIP2_MDIO_READ(PHY_PC, 0, pc_temp_reg);
        // (0800:0000) 0.11    =     0    power-down
        pc_temp_reg |= 0x800;
        INCAIP2_MDIO_WRITE(PHY_PC, 0, pc_temp_reg);
#else
#define PHY_LAN 0
#define PHY_PC  1
	/* configure PHYs acc. to kernel configuration parameters */
#if defined(CONFIG_PHY_MV88E1111) || defined(CONFIG_PHY_MV88E1121)
	u16 temp_reg;
	/*
	 * use external PHYs, disable MDIO hardware state machine for the 
	 * following PHY register configurations.
	 */   
	INCAIP2_SW_REG32( MDIO_MOD_SEL ) = 0x00000020;

	/* Set Switch Gigabit Mode Selection bit for LAN and PC port */
	INCAIP2_SW_REG32( SWITCH_P0_GMAC_REG ) = 0x0000A009;
	INCAIP2_SW_REG32( SWITCH_P1_GMAC_REG ) = 0x0000A009;

#if defined(CONFIG_PHY_MV88E1121)
	/* Port0=LAN gets ID1, Port1=PC gets ID0 (flaw on reference board) */
	INCAIP2_SW_REG32( SWITCH_MDIO_ID ) = 0x00000001;

	/* Add delay to Rx and Tx Clock w.r.t. data */
	INCAIP2_MDIO_WRITE(PHY_LAN, 22, 2 );      /* select page 2 */
	INCAIP2_MDIO_READ(PHY_LAN, 21, temp_reg);
	INCAIP2_MDIO_WRITE(PHY_LAN, 21, temp_reg | 0x30);
	INCAIP2_MDIO_WRITE(PHY_LAN, 22, 0 );      /* select page 0 */
	INCAIP2_MDIO_WRITE(PHY_PC, 22, 2 );       /* select page 2 */
	INCAIP2_MDIO_READ(PHY_PC, 21, temp_reg);
	INCAIP2_MDIO_WRITE(PHY_PC, 21, temp_reg | 0x30);
	INCAIP2_MDIO_WRITE(PHY_PC, 22, 0 );      /* select page 0 */
#elif defined ( CONFIG_PHY_MV88E1111 ) 
	/* Add delay to Rx and Tx Clock w.r.t. data */
	INCAIP2_MDIO_READ(PHY_LAN, 20, temp_reg);
	INCAIP2_MDIO_WRITE(PHY_LAN, 20, temp_reg | 0x82);
	INCAIP2_MDIO_READ(PHY_PC, 20, temp_reg);
	INCAIP2_MDIO_WRITE(PHY_PC, 20, temp_reg | 0x82);
#endif

	/* Configure PHY Control Register for Port 0
	     (B100)                 -- PHY control register --
	(8000:8000) 0.15    =     1    reset
	(4000:0000) 0.14    =     0    loopback
	(2040:2000) 0. 6,13 =   b01    speed selection = 100 Mbps
	(1000:1000) 0.12    =     1    A/N enable
	(0800:0000) 0.11    =     0    power-down
	(0400:0000) 0.10    =     0    isolate
	(0200:0000) 0. 9    =     0    restart A/N
	(0100:0100) 0. 8    =     1    duplex = full
	(0080:0000) 0. 7    =     0    collision test enable
	(003f:0000) 0. 5- 0 =     0    (reserved)
	*/
	INCAIP2_MDIO_WRITE(PHY_LAN, 0, 0x3100);
    /* Soft-reset PHY0 */
	INCAIP2_MDIO_WRITE(PHY_LAN, 0, 0xb100);

    /* ... same configuration for Port 1 */
	INCAIP2_MDIO_WRITE(PHY_PC, 0, 0x3100);
    /* Soft-reset PHY1 */
	INCAIP2_MDIO_WRITE(PHY_PC, 0, 0xb100);
											 
	/* dummy read needed because MV88E1111/MV88E1121 latches the Link Status bit */
	INCAIP2_MDIO_READ( PHY_LAN, 1, temp_reg);
	INCAIP2_MDIO_READ( PHY_PC, 1, temp_reg);

#else /* use INCA-IP2 internal PHYs */

    /*
     * use internal PHYs, disable MDIO hardware state machine for the 
     * following PHY register configurations.
     */   
    INCAIP2_SW_REG32( MDIO_MOD_SEL ) = 0x00000000;

    /* Configure LAN and PC port MACs to operate in 100Mbit MII mode  */
	INCAIP2_SW_REG32( SWITCH_P0_GMAC_REG ) = 0x0000A06A;
	INCAIP2_SW_REG32( SWITCH_P1_GMAC_REG ) = 0x0000A06A;

    /*
     * Configure PHY Control Register for Port 0
     * - RST = 0 -> no reset
     * - LPBK = 0 -> no loopback mode
     * - SPD_L = 1 speed (low bit) = 100 Mbit/s
     * - ANEN = 1 -> autonegotiation enable
     * - PDN = 0 -> no power down, normal operation
     * - ISO = 0 -> no isolation from network
     * - RSTAR = 0 -> no restart of ANEG
     * - DPLX = 1 -> Full Duplex
     * - COLTST = 0 -> Collision Test disable
     */
    INCAIP2_MDIO_WRITE(PHY_LAN, 0, 0x3100);
    /* ... same configuration for Port 1 */
    INCAIP2_MDIO_WRITE(PHY_PC, 0, 0x3100);

#endif
#endif   /* CONFIG_PHY_SKIP */

    /* enable ANEG and MDIO HW state machine */
    INCAIP2_SW_REG32(MDIO_MOD_SEL) |= 0x00000007;
    
    /* forward unknown broadcast/multicast/unicast to all ports */
    INCAIP2_SW_REG32(UNKNOWN_DEST_REG)=0x000001FF;

#if (STATUS_HEADER_SIZE > 0)
    /* Configure PMAC to add Status Header and CRC in packet */
    INCAIP2_SW_REG32(SWITCH_PMAC_HD_CTL) = 0x001C0000;  
#else
    /* Configure PMAC to add CRC in packet */
    INCAIP2_SW_REG32(SWITCH_PMAC_HD_CTL) = 0x00140000;  
#endif

    /* workaround for aging bug 
     * Since the hardware driven mac table aging is ~15 times faster than desired
     * it is configured to 1 day aging period which results in appr. 2:30 hours
     */
    INCAIP2_SW_REG32(PR_CTRL_REG) = (INCAIP2_SW_REG32(PR_CTRL_REG) & 0xFFFFC7FF) | 0x00003000;
    
    /* additionally enable CPU port in forwarding mode */    
    INCAIP2_SW_REG32(ST_PT_REG)=0x0000003F;  	

    /* Set watermark registers to correct values */
    INCAIP2_SW_REG32(PORT0_RX_WM_REG0)=0x00006040;
    INCAIP2_SW_REG32(PORT1_RX_WM_REG0)=0x00006040;
    INCAIP2_SW_REG32(PORT2_RX_WM_REG0)=0x00006028;
    
    	
    /* spawn timer task to handle autoneg specialities */

    printk("Starting timer for MDIO checks\n");    
    init_timer(&ifx_switch_timer_handler);
    ifx_switch_timer_handler.function=ifx_switch_autoneg_timer_handler;
    ifx_switch_timer_handler.data=0;
    ifx_switch_timer_handler.expires=jiffies+1;
    add_timer(&ifx_switch_timer_handler);
    return 0;    
}

static unsigned char my_ethaddr[MAX_ADDR_LEN];
/**
 * Initialize MAC address.
 * This function copies the ethernet address from kernel command line.
 *
 * \param   line     Pointer to parameter
 * \return  0        OK
 * \ingroup Internal
 */
static int __init ifx_switch_ethaddr_setup(char *line)
{
	char *ep;
	int i;

	memset(my_ethaddr, 0, MAX_ADDR_LEN);
	/* there should really be routines to do this stuff */
	for (i = 0; i < 6; i++)
	{
		my_ethaddr[i] = line ? simple_strtoul(line, &ep, 16) : 0;
		if (line)
			line = (*ep) ? ep+1 : ep;
	}
	INCAIP2_SW_DMSG("mac address %2x-%2x-%2x-%2x-%2x-%2x \n"
		,my_ethaddr[0]
		,my_ethaddr[1]
		,my_ethaddr[2]
		,my_ethaddr[3]
		,my_ethaddr[4]
		,my_ethaddr[5]);
	return 0;
}
__setup("ethaddr=", ifx_switch_ethaddr_setup);


/**
 * Open RX DMA channels.
 * This function opens all DMA rx channels by clearing their control bit. 
 *
 * \param   dev      pointer to net device structure that comprises 
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
static void ifx_switch_open_rx_dma(struct net_device *dev)
{

    struct switch_priv* priv=(struct switch_priv*)dev->priv;
    struct dma_device_info* dma_dev=(struct dma_device_info*) (priv->dma_device); 
    int i;
    
    for(i=0;i<dma_dev->num_rx_chan;i++)
        (dma_dev->rx_chan[i])->open(dma_dev->rx_chan[i]);
}

/**
 * Closes all RX DMA channels.
 * This function closes all DMA rx channels by clearing their control bit. 
 *
 * \param   dev      pointer to net device structure that comprises 
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
static void ifx_switch_close_rx_dma(struct net_device *dev)
{
    struct switch_priv* priv=(struct switch_priv*)dev->priv;
    struct dma_device_info* dma_dev=(struct dma_device_info*)(priv->dma_device); 
    int i;
    
    for(i=0;i<dma_dev->num_rx_chan;i++)
        dma_dev->rx_chan[i]->close(dma_dev->rx_chan[i]);   
}

/**
 * Open TX DMA channels.
 * This function opens all DMA tx channels. 
 *
 * \param   dev      pointer to net device structure that comprises 
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
static void ifx_switch_open_tx_dma(struct net_device *dev)
{

    struct switch_priv* priv=(struct switch_priv*)dev->priv;
    struct dma_device_info* dma_dev=(struct dma_device_info*) (priv->dma_device); 
    int i;

    for(i=0;i<dma_dev->num_tx_chan;i++)
        (dma_dev->tx_chan[i])->open(dma_dev->tx_chan[i]);
}

/**
 * Close all TX DMA channels.
 * This function closes all DMA tx channels. 
 *
 * \param   dev      pointer to net device structure that comprises 
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
static void ifx_switch_close_tx_dma(struct net_device *dev)
{
    struct switch_priv* priv=(struct switch_priv*)dev->priv;
    struct dma_device_info* dma_dev=(struct dma_device_info*)(priv->dma_device); 
    int i;
    
    for(i=0;i<dma_dev->num_tx_chan;i++)
        dma_dev->tx_chan[i]->close(dma_dev->tx_chan[i]);   
}

#ifdef CONFIG_NET_HW_FLOWCONTROL
/**
 * Enable receiving DMA.
 * This function enables the receiving DMA channel. 
 *
 * \param   dev      pointer to net device structure that comprises 
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
void ifx_switch_xon(struct net_device *dev)
{   
    unsigned long flag;
    local_irq_save(flag);
    INCAIP2_SW_DMSG("wakeup\n");
    ifx_switch_open_rx_dma(dev);
    local_irq_restore(flag);
}
#endif /* CONFIG_NET_HW_FLOWCONTROL */  

/**
 * Open network device.
 * This functions opens the network device and starts the interface queue.
 *
 * \param   dev  Device structure for Ethernet device
 * \return  0    OK, device opened
 * \return  -1   Error, registering DMA device
 * \ingroup API
 */
int ifx_switch_open(struct net_device *dev)
{
#if defined (CONFIG_NET_HW_FLOWCONTROL) && !defined (IKOS_MINI_BOOT)
    struct switch_priv* priv=(struct switch_priv*)dev->priv;
#endif

    MOD_INC_USE_COUNT;

    if (ifx_chip_init == 0) {
        ifx_chip_init=1;
        ifx_switch_chip_init();
        printk("Enabled switch hardware for %s\n",dev->name);
    }
    
    printk("%s: Register Switch DMA Module at DMA core\n",__FUNCTION__);
    ifx_switch_open_rx_dma(dev);
    ifx_switch_open_tx_dma(dev);
	
#ifndef IKOS_MINI_BOOT        
#ifdef CONFIG_NET_HW_FLOWCONTROL
    if ( (priv->fc_bit = netdev_register_fc(dev, ifx_switch_xon)) == 0){
    	INCAIP2_SW_EMSG("Hardware Flow Control register fails\n");
    }    
#endif /* CONFIG_NET_HW_FLOWCONTROL */  

    netif_start_queue(dev);
#endif /* IKOS_MINI_BOOT */
    return OK;
}

/**
 * Close network device.
 * This functions closes the network device, which will also stop the interface
 * queue.
 *
 * \param   dev  Device structure for Ethernet device
 * \return  0    OK, device closed (cannot fail)
 * \ingroup API
 */
int ifx_switch_release(struct net_device *dev)
{
   struct switch_priv* priv=(struct switch_priv*)dev->priv;
   struct dma_device_info *dma_dev;
   
   dma_dev=(struct dma_device_info*)(priv->dma_device);
   
   ifx_switch_close_rx_dma(dev);
   ifx_switch_close_tx_dma(dev);
   dma_device_unregister(dma_dev);
   
#ifndef IKOS_MINI_BOOT      
#ifdef CONFIG_NET_HW_FLOWCONTROL     
    if (priv->fc_bit){
        netdev_unregister_fc(priv->fc_bit);
    }
#endif //CONFIG_NET_HW_FLOWCONTROL  
    netif_stop_queue(dev); 
#endif //IKOS_MINI_BOOT    
    MOD_DEC_USE_COUNT;
    
    return OK;
}

#ifdef CONFIG_INCAIP2_SWITCH_RESERVED_MC

#define PORTMAP_LAN (1<<0)
#define PORTMAP_PC  (1<<1)
#define PORTMAP_CPU (1<<2)

/* global "MAC Table" for reserved multicasts. To be maintained by Switch MAC table function. */
u8 ifx_switch_reserved_mc_dst_ports[48] = {[0 ... 47] = PORTMAP_CPU };
/**
 * Switch reserved multicast hook.
 * This function ...
 * 
 * \param   dev  pointer to network device structure that holds DMA device information
 * \param   skb  pointer to socket buffer structure that contains the received data 
 *               to be sent
 * \ingroup Internal
 */
static inline struct sk_buff* ifx_switch_reserved_mc_hook(struct net_device *dev, struct sk_buff* skb)
{
    struct sk_buff* skb2 = skb;
    
    /* is this a reserved multicast address? */
    if ( skb->data[0] == 0x01
      && skb->data[1] == 0x80
      && skb->data[2] == 0xC2
      && skb->data[3] == 0x00
      && skb->data[4] == 0x00
      && ((skb->data[5] <= 0x2F) && ((skb->data[5] <= 0x10) || (skb->data[5] >= 0x20))) )
    {
        int rsvd_idx = skb->data[5];
#if (STATUS_HEADER_SIZE > 0)
        /* extract Source Port ID of packet from status header */
        int src_port = *(skb->data-6) >> 6;
#else
#warning "Packet Src Port unknown for Soft Switching"
        int src_port = 0;
#endif
        u8 src_port_map = 1 << src_port;
        u8 dst_port_map = ifx_switch_reserved_mc_dst_ports[rsvd_idx] & ~src_port_map;

        /* store packet's destination ports in skb's private control buffer */
        skb->cb[0] = dst_port_map;

        /* drop the packet? */
        if (dst_port_map == 0)
        {
            dev_kfree_skb_any(skb);
            return NULL;
        }
        
        /* do we have to send the packet to 2 destinations? */
        if ( (dst_port_map & PORTMAP_CPU) && (dst_port_map & (PORTMAP_LAN | PORTMAP_PC)) )
        {
            skb2 = skb_copy(skb, GFP_ATOMIC);
        }

        if ( dst_port_map & (PORTMAP_LAN | PORTMAP_PC) )
        {
            struct switch_priv *priv=dev->priv;
            struct dma_device_info* dma_dev=priv->dma_device;
#ifdef RESERVED_MC_USE_FLOW_ENGINE
            static int curr_flowid = 24-RESERVED_MC_PKT_NUM;
            static u16 ikey_data[8] = { 0xc200, /* Eth DA[31:16] */
                                        0x0000, /* Eth DA[15:0]  */
                                        0x0000, /* Eth SA[15:0]  */
                                        0x0180, /* Eth DA[47:32] */
                                        0x0000, /* Eth SA[47:32] */
                                        0x0000, /* Eth SA[31:16] */
                                        0x0000,    /* don't care */
                                        0x0000 };  /* don't care */
            static u16 ikey_mask[8] = { 0x0000, /* Eth DA[31:16] */
                                        0x0000, /* Eth DA[15:0]  */
                                        0x0000, /* Eth SA[15:0]  */
                                        0x0000, /* Eth DA[47:32] */
                                        0x0000, /* Eth SA[47:32] */
                                        0x0000, /* Eth SA[31:16] */
                                        0xFFFF,    /* don't care */
                                        0xFFFF };  /* don't care */


            /* TODO: setup flow engine */
            ikey_data[1] = (u16) rsvd_idx;
            ikey_data[2] = *(((u16*) skb->data) + 5);
            ikey_data[5] = *(((u16*) skb->data) + 4);
            ikey_data[4] = *(((u16*) skb->data) + 3);
            ifx_set_flow_pattern(0, curr_flowid, (u32*) &ikey_data, (u32*) &ikey_mask );
            ifx_set_flow_action_param(0, curr_flowid, REDIRECTION, 
                (u32) dst_port_map & (PORTMAP_LAN | PORTMAP_PC));
#else
            /* add the VLAN ID that the destination port is member of */
            skb_push(skb, 4);
            memcpy(KSEG1ADDR(skb->data), skb->data+4, 12);
            *((u16*)KSEG1ADDR(skb->data) + 6) = 0x8100;
            *((u16*)KSEG1ADDR(skb->data) + 7) = (dst_port_map & PORTMAP_LAN ? VLANID_CPU2LAN : VLANID_CPU2PC );
#endif /* RESERVED_MC_USE_FLOW_ENGINE */

            dma_dev->current_tx_chan=1;
            if (dma_device_write( dma_dev, KSEG1ADDR(skb->data), skb->len, skb ) > 0)
            {
#ifdef RESERVED_MC_USE_FLOW_ENGINE
                if (++curr_flowid >= 24) curr_flowid = 24 - RESERVED_MC_PKT_NUM;
#endif
            }
            else
            {
                /* drop the packet */
                dev_kfree_skb_any(skb);
            }

        }
        
        if (!(dst_port_map & PORTMAP_CPU))
            skb2 = NULL;
    }

    return skb2;
}
#endif /* CONFIG_INCAIP2_SWITCH_RESERVED_MC */

/**
 * Switch receive.
 * This function reads data from Ethernet switch and passes it to upper network 
 * layer. It is called by ifx_switch_hw_receive after an DMA receive interrupt 
 * has occured.
 * 
 * \param   dev  pointer to network device structure that holds DMA device information
 * \param   len  number of data bytes received from DMA
 * \param   skb  pointer to socket buffer structure that contains the receiveddata 
 *               to be sent
 * \ingroup API
 */
void ifx_switch_rx(struct net_device *dev, int len,struct sk_buff* skb)
{
    struct switch_priv *priv = (struct switch_priv *) dev->priv;
#ifdef CONFIG_NET_HW_FLOWCONTROL
    int mit_sel=0;
#endif
    skb->dev = dev;
    skb->protocol = eth_type_trans(skb, dev);
    

#ifdef CONFIG_NET_HW_FLOWCONTROL
    mit_sel = netif_rx(skb);
    switch(mit_sel){
    	case NET_RX_SUCCESS:
        case NET_RX_CN_LOW:
        case NET_RX_CN_MOD:
        	break;
        case NET_RX_CN_HIGH:
		break;
	case NET_RX_DROP:
		if ( (priv->fc_bit) && ( ! test_and_set_bit(priv->fc_bit, &netdev_fc_xoff))){
			ifx_switch_close_rx_dma(dev);
		}
		break;
    }
#else    
    netif_rx(skb);
#endif

    priv->stats.rx_packets++;
    priv->stats.rx_bytes+=len;
    
    return;
}

/**
 * Write data to DMA device.
 * This function passes data to be transmitted to the DMA dma device.
 * 
 * \param   buf pointer to data to be sent
 * \param   len amount of data bytes to be sent 
 * \param   dev pointer to net device structure that holds the private field
 *              which contains the DMA device information
 * \return  0   in case of transmission errors
 *          >0  in case of successful data transmission, number of bytes
 *              that were passed to DMA
 * \ingroup Internal
 */
int ifx_switch_hw_tx(char *buf, int len, struct net_device *dev)
{

    struct switch_priv *priv=dev->priv;
    struct dma_device_info* dma_dev=priv->dma_device;

/*    printk("%s(buf=0x%08X, len=%d)\n",__FUNCTION__, (u32)buf, len);  */  
    dma_dev->current_tx_chan=0;
    return dma_device_write( dma_dev, (u8*)buf, (s32)len, priv->skb );
}

/**
 * Write data to Ethernet switch.
 * This function writes the data comprised in skb structure via DMA to the 
 * Ethernet Switch. It is installed as the switch driver's hard_start_xmit 
 * method.
 * 
 * \param   skb  Pointer to socket buffer structure that contains the data 
 *               to be sent
 * \param   dev  Pointer to network device structure which is used for 
 *               data transmission 
 * \return  1    Transmission error
 * \return  0    OK, successful data transmission
 * \ingroup API
 */
int ifx_switch_tx(struct sk_buff *skb, struct net_device *dev)
{
 
    int len;
    char *data;
    struct switch_priv *priv = (struct switch_priv *) dev->priv;
    u32 flags;
    
    save_and_cli(flags);
    
    len = skb->len < ETH_ZLEN ? ETH_ZLEN : skb->len;
    data = skb->data;
    priv->skb = skb;
    dev->trans_start = jiffies;

#ifdef INCAIP2_SW_DUMP
    printk("%s %d transmitting packet\n",__FILE__,__LINE__);
    ifx_switch_dump_skb(skb->len, (char *)skb->data);
#endif
/*    printk("%s -> ifx_switch_hw_tx \n", __FUNCTION__);  */  
    if( ifx_switch_hw_tx((u8*)data, len, dev) != len)
    {
#ifdef INCAIP2_SW_DUMP
        printk("%s -> ifx_switch_hw_tx failed\n", __FUNCTION__);
#endif
        ifx_switch_dma_tx_drop++;
        restore_flags(flags);
	    return 1;
    }
    priv->stats.tx_packets++;
    priv->stats.tx_bytes+=len;
    restore_flags(flags);
    return OK;
}

/**
 * Transmission timeout callback.
 * This functions is called when a trasmission timeout occurs. It will wake up 
 * the interface queue again.
 *
 * \param   dev Device structure for Ethernet device
 * \ingroup API
 */
void ifx_switch_tx_timeout (struct net_device *dev)
{
    struct switch_priv* priv=(struct switch_priv*)dev->priv;

    INCAIP2_SW_DMSG("%s tx_status==%d\n",dev->name,priv->tx_status);
#if 0
    //TODO:must restart the TX channels
    printk("ifx_switch_tx_timeout: reset and enable DMA channel\n");
    if( dma_reset_and_enable_channel( priv->dma_device->logic_tx_chan_base +
                                      priv->dma_device->current_tx_chan ) != OK )
    {
        printk("Failed to reset and enable channel %d\n",
               priv->dma_device->current_tx_chan);
    }
#endif
    priv->stats.tx_errors++;
    netif_wake_queue(dev);
    return;
}

/**
 * Get device statistics.
 * This functions returns the device statistics, stored in the device structure.
 *
 * \param   dev   Device structure for Ethernet device
 * \return  stats Pointer to statistics structure
 * \ingroup API
 */
struct net_device_stats *ifx_switch_stats(struct net_device *dev)
{
    struct switch_priv *priv = (struct switch_priv *) dev->priv;
   
    return &priv->stats;
}

/**
 * Read data from DMA device.
 * This function reads data from the DMA device. The function is called by 
 * the switch/DMA pseudo interrupt handler dma_intr_handler on occurence of
 * a DMA receive interrupt.
 * 
 * \param   dev      Pointer to network device structure
 * \param   dma_dev  Pointer to dma device structure
 * \return  OK       In case of successful data reception from dma
 *          -EIO     Incorrect opt pointer provided by device
 * \ingroup Internal
 */
int ifx_switch_hw_receive( struct net_device* dev, struct dma_device_info* dma_dev )
{
    u8* buf=NULL;
    int len=0;
    void* opt=NULL;
    struct sk_buff *skb=NULL;
    u32 flags;
    
    save_and_cli(flags);
    
    /*
     * read data from DMA device, remove CRC, put the received data to 
     * socket buffer and pass received data to upper layer for further processing 
     */
    len = dma_device_read( dma_dev, &buf, &opt );
 
    INCAIP2_SW_DMSG("DMA rx len:%d\n",len);

#ifdef INCAIP2_SW_DUMP  
    printk("%s %d received packet from HW\n",__FILE__,__LINE__);
    ifx_switch_dump_skb(len, (char *)buf);
#endif    

    skb=(struct sk_buff*)opt;   /* pass received data to skb structure and check it */
    if (skb == NULL )
    {
        printk("%s - dma_device_read returned opt == NULL\n", __FUNCTION__);
    	ifx_switch_dma_rx_err++;
    	return -EIO;
    }
    if (len > (skb->end - skb->tail))
    {
    	INCAIP2_SW_EMSG("BUG, len:%d end:%p tail:%p\n", (len+4), skb->end, skb->tail);
        printk("%s - BUG, len:%d end:%p tail:%p\n",  __FUNCTION__, (len+4), skb->end, skb->tail);
        while (1);
    }
    skb_put(skb,len);
#if (STATUS_HEADER_SIZE > 0)
    /* take optional status header into account */
    skb_pull(skb, STATUS_HEADER_SIZE);
#endif
    skb->dev = dev;
    
#ifdef CONFIG_INCAIP2_SWITCH_RESERVED_MC
    /* returned skb can be the original skb, a copy of original skb or NULL */
    skb = ifx_switch_reserved_mc_hook(dev, skb);
    if (skb == NULL)
    {
        restore_flags(flags);
        return OK;
    }
#endif

#ifdef IKOS_MINI_BOOT
	incaip2_sw_to_mbx( skb );
#else /* IKOS_MINI_BOOT */
    ifx_switch_rx(dev,len,skb);
#endif /* IKOS_MINI_BOOT */
    restore_flags(flags);
    return OK;
}

/**
 * Pseudo Interrupt handler for DMA.
 * This function processes DMA interrupts notified to the switch device driver.
 * The function is installed at the DMA core as interrupt handler for the 
 * switch dma device. 
 * It handles the following DMA interrupts:
 * passes received data to the upper layer in case of rx interrupt,  
 * In case of a dma receive interrupt the received data is passed to the upper layer.
 * In case of a transmit buffer full interrupt the transmit queue is stopped. 
 * In case of a transmission complete interrupt the transmit queue is restarted.
 * 
 * \param   dma_dev pointer to dma device structure
 * \param   status  type of interrupt being notified (RCV_INT: dma receive 
 *                  interrupt, TX_BUF_FULL_INT: transmit buffer full interrupt, 
 *                  TRANSMIT_CPT_INT: transmission complete interrupt)
 * \return  OK      In case of successful data reception from dma
 * \ingroup Internal
 */
int ifx_switch_dma_int_handler( struct dma_device_info* dma_dev, int status )
{
    struct net_device* dev;
    static int cleanup_flowid = 24-RESERVED_MC_PKT_NUM;

    /* printk("%s(status %d)\n", __FUNCTION__, status); */
    dev=switch_devs + (u32) dma_dev->priv;
    INCAIP2_SW_DMSG("status:%d \n", status);
    switch(status)
    {
        case RCV_INT:        
/*            printk("%s - switch receive chan=%d\n", __FUNCTION__,dma_dev->current_rx_chan);  */
            INCAIP2_SW_DMSG("switch receive chan=%d\n",dma_dev->current_rx_chan);
            ifx_switch_dma_rx++;
            ifx_switch_hw_receive(dev,dma_dev);
            break;  	  
        case TX_BUF_FULL_INT:
#ifdef CONFIG_INCAIP2_SWITCH_RESERVED_MC
            if (dma_dev->current_tx_chan != 1)
#endif
            {
                ifx_switch_dma_tx_full++;
                INCAIP2_SW_DMSG("tx buffer full\n");
                netif_stop_queue(dev);
            }
            break;
        case TRANSMIT_CPT_INT:
/*            printk("%s - tx complete \n", __FUNCTION__);  */
#ifdef CONFIG_INCAIP2_SWITCH_RESERVED_MC
            if (dma_dev->current_tx_chan == 1)
            {
#ifdef RESERVED_MC_USE_FLOW_ENGINE
                /* Delete flow engine rule when DMA transfer to Switch is complete.
                   Is the packet already classified by this time or is this too early? */
                ifx_delete_flow_action_param(0, cleanup_flowid, DELETE_ACTION_ENTRY);
                if (++cleanup_flowid >= 24) cleanup_flowid = 24 - RESERVED_MC_PKT_NUM;
#endif
            }
            else 
#endif
            {
                ifx_switch_dma_tx++;
                INCAIP2_SW_DMSG("tx buffer released\n");
                netif_wake_queue(dev);
            }
            break;
    }
    return OK;
}


/**
 * Allocates buffer sufficient for Ethernet Frame.
 * This function is installed as DMA callback function to be called on DMA 
 * receive interrupt.
 * 
 * \param   len          Unused
 * \param   *byte_offset Pointer to byte offset
 * \param   **opt        pointer to skb structure  
 * \return  NULL         In case of buffer allocation fails
 *          buffer       Pointer to allocated memory
 * \ingroup Internal
 */
u8* ifx_switch_dma_buffer_alloc(int len, int* byte_offset,void** opt)
{
    u8* buffer=NULL;
    struct sk_buff *skb=NULL;
    
    skb = dev_alloc_skb(ETHERNET_PACKET_DMA_BUFFER_SIZE+NET_IP_ALIGN);

    if (skb == NULL) 
    {
    	printk("%s - memory allocation failed\n", __FUNCTION__);
        ifx_switch_dma_alloc_drop++;
    	return NULL;
    }
    buffer=(u8*)(skb->data);
    INCAIP2_SW_DMSG("%8p\n",skb->data);
    *(int*)opt=(int)skb;

    /* make sure IP frame data is 32-bit aligned */
    skb_reserve(skb, NET_IP_ALIGN);
    *byte_offset=NET_IP_ALIGN;

    ifx_switch_dma_alloc++;
    return buffer;
}

/**
 * Free DMA buffer.
 * This function frees a buffer, which can be either a data buffer or an 
 * skb structure.
 * 
 * \param   *dataptr Pointer to data buffer
 * \param   *opt     Pointer to skb structure  
 * \return  0        OK
 * \ingroup Internal
 */
int ifx_switch_dma_buffer_free(u8* dataptr,void* opt)
{
    struct sk_buff *skb=NULL;
   
    if(opt==NULL)
    {
        ifx_switch_dma_free_err++;
        kfree(dataptr);
    }
    else 
    {
        skb=(struct sk_buff*)opt;

        INCAIP2_SW_DMSG("%8p\n",skb->data);
#ifdef IKOS_MINI_BOOT       
        dev_kfree_skb(skb); 
#else
        dev_kfree_skb_any(skb); 
#endif
        ifx_switch_dma_free++;
    }
    return OK;
} 

/**
 * Initialize dma device as switch upper layer module.
 * This function initializes the passed dma device structure for usage as 
 * switch dma device. It prepares the passed dma device structure for registration 
 * as switch dma device at the dma core. In advance the memory upper layer 
 * module has to be registered at the dma core. 
 * 
 * \param   *dma_dev Pointer to dma device structure to be initialized
 * \return  0        OK
 * \ingroup Internal
 */
int ifx_switch_init_dma_device(struct dma_device_info *dma_dev)
{
    int i;

    /* 
     * configure switch DMA device acc. to default values in incaip2_dma.h 
     */
    dma_dev->tx_weight=DEFAULT_SW_PORT_WEIGHT; 
    dma_dev->num_tx_chan=min(dma_dev->max_tx_chan_num, DEFAULT_SW_TX_CHANNEL_NUM);
    dma_dev->num_rx_chan=min(dma_dev->max_rx_chan_num, DEFAULT_SW_RX_CHANNEL_NUM); 
    dma_dev->tx_burst_len=DEFAULT_SW_TX_BURST_LEN;
    dma_dev->rx_burst_len=DEFAULT_SW_RX_BURST_LEN;

    for(i=0;i<dma_dev->num_tx_chan;i++)  /* set tx channel parameters */
    {
        dma_dev->tx_chan[i]->tx_weight=DEFAULT_SW_CHANNEL_WEIGHT; 
        dma_dev->tx_chan[i]->packet_size=ETHERNET_PACKET_DMA_BUFFER_SIZE;
#ifdef RESERVED_MC_USE_FLOW_ENGINE
        if (i == 1)
            dma_dev->tx_chan[i]->desc_len=RESERVED_MC_PKT_NUM;
        else
#endif
    }
    for(i=0;i<dma_dev->num_rx_chan;i++)   /* set rx channel parameters */
    {
        dma_dev->rx_chan[i]->packet_size=ETHERNET_PACKET_DMA_BUFFER_SIZE;
    }

    /* 
     * set DMA handler functions for rx-interrupts, 
     * buffer allocation and release
     */
    dma_dev->intr_handler=ifx_switch_dma_int_handler;
    dma_dev->buffer_alloc=ifx_switch_dma_buffer_alloc;
    dma_dev->buffer_free=ifx_switch_dma_buffer_free;
    return 0;
}


/**
 * Set MAC address.
 * This functions sets the MAC address of the specified Ethernet device.
 *
 * \param   dev  Device structure for Ethernet device
 * \param   p    Pointer to MAC address (array of 6 octets)
 * \return  0    OK, MAC address set
 * \ingroup API
 */
int ifx_switch_set_mac_address(struct net_device *dev, void *p)
{
   struct sockaddr *addr=p; 
   int i; 

   INCAIP2_SW_DMSG("mac address set!\n");
   memcpy(dev->dev_addr,addr->sa_data,dev->addr_len);
   INCAIP2_SW_EMSG("%s:changed MAC to ",dev->name);

   for(i=0;i<6;i++)
   INCAIP2_SW_EMSG("%02x:",dev->dev_addr[i]);
  
   INCAIP2_SW_EMSG("\n");
   return OK;
}

extern void dma_disable_interrupts( void );

/**
 * Initialize switch driver.
 * This functions initializes the switch driver structures and registers the 
 * Ethernet device.
 *
 * \param   dev    Device structure for Ethernet device
 * \return  0      OK
 * \return  ENOMEM No memory for structures available 
 * \return  -1     Error during DMA init or Ethernet address configuration.
 * \ingroup API
 */
int ifx_switch_init(struct net_device *dev)
{
    struct switch_priv *priv;
  	u64 retval=0;
	int i;
    
    /* Return -ENODEV if we were already probed */
    if (ifx_probed != 0) return -ENODEV;
    
    /* Disable Switch Ports forwarding mode */
    INCAIP2_SW_REG32( ST_PT_REG ) = 0x00000000;

    printk("Initializing %s\n",dev->name);

#ifndef MODULE
    switch_devs=dev;
#endif

    ether_setup(dev); /* assign some of the fields */

    dev->open            = ifx_switch_open;
    dev->stop            = ifx_switch_release;
    dev->hard_start_xmit = ifx_switch_tx;
    dev->get_stats       = ifx_switch_stats;
    dev->set_mac_address = ifx_switch_set_mac_address;
    dev->tx_timeout      = ifx_switch_tx_timeout;
    dev->watchdog_timeo  = timeout;

    SET_MODULE_OWNER(dev);

    dev->priv = kmalloc(sizeof(struct switch_priv), GFP_KERNEL);
    if (dev->priv == NULL)
    {
        printk("%s: Allocating Memory for dev->priv failed !\n",__FUNCTION__);
        return -ENOMEM;
    }
    
    memset(dev->priv, 0, sizeof(struct switch_priv));
    priv=dev->priv;
    
    /* reserve the dma device */
    priv->dma_device=dma_device_reserve("SW"); 
    if( priv->dma_device == NULL )                                                        
    {
        printk("%s: Allocating DMA device failed !\n",__FUNCTION__);
        return -ENOMEM;
    }   

    /* 
     * register switch as upper-layer module to DMA 
     */
     
    ifx_switch_init_dma_device(priv->dma_device); 
    dma_device_register(priv->dma_device);
    
    /* 
     * Initialize switch for operation as INCAIP2 3 port GB Ethernet SWitch 
     */
	/*read the mac address from the mac table and put them into the mac table.*/
   	for (i = 0; i < 6; i++){
		retval +=my_ethaddr[i];
	}
	/* ethaddr not set in u-boot ? */
	if (retval == 0)
    {
        printk("No MAC address provided. Cannot setup networking.");
        kfree(dev->priv);
        kfree(priv->dma_device);
        return -1;          
	}
    else
    {
		for (i = 0; i < 6; i++)
        {
			dev->dev_addr[i] = my_ethaddr[i];
		}
		dev->dev_addr[5] += + (unsigned char)(dev-switch_devs);
	}

    
	retval = 0;
	for (i = 0; i < 6; i++)
    {
		retval = (retval<<8) + dev->dev_addr[i];
	}
    /*
     * Reset MAC table and add own ethernet MAC address as static
     */
    if( ifx_switch_add_mac_table_entry(retval) != OK)
    {
        printk("%s -> ifx_switch_add_mac_table_entry failed\n",__FUNCTION__);
        kfree(dev->priv);
        kfree(priv->dma_device);
        return -1;          
    }

    ifx_probed++;

    return 0;
}


#ifdef MODULE
/**
 * Initialize switch driver as module.
 * This functions initializes the switch driver structures and registers the 
 * Ethernet device for module usage.
 *
 * \return  0      OK
 * \return  ENODEV An error occured during initialization 
 * \ingroup API
 */
int ifx_switch_init_module(void)
{
    int i=0,result,device_present = 0;
    
    INCAIP2_SW_DMSG("switch init, working in %s mode\n",mode); 

    switch_devs=switch_devs_array;
    for(i=0;i<INCAIP2_SW_DEVICE_NO;i++)
    {  
        sprintf(switch_devs[i].name, "eth%d",i);

        if ( (result = register_netdev(switch_devs+i)) )
        {
            printk("error %i registering device \"%s\"\n",
                            result, switch_devs[i].name);
        }
        else 
            device_present++;
    }
    
    return device_present ? 0 : -ENODEV;
}

/**
 * Remove driver module.
 * This functions removes the driver and unregisters all devices. 
 *
 * \ingroup API
 */
void  ifx_switch_cleanup(void)
{
    int i;
    struct switch_priv *priv;
    for(i=0;i<INCAIP2_SW_DEVICE_NO;i++)  
    {
     priv=switch_devs[i].priv;
     if(priv->dma_device) 
      {
       dma_device_unregister(priv->dma_device); 
       dma_device_release(priv->dma_device);
      }
     kfree(switch_devs[i].priv);
     unregister_netdev(switch_devs+i);
    }
}
module_exit(ifx_switch_cleanup);
module_init(ifx_switch_init_module);
#endif /* MODULE */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("IFX");
