/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <asm/types.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/irq.h>
#include <asm/incaip2/ifx_ssc_defines.h>
#include <asm/mipsregs.h>
#include "fpga_defs.h"
#include "fpga.h"
#ifdef CONFIG_PROC_FS
#include <linux/proc_fs.h>
static struct proc_dir_entry *entry;
#endif


#define INCA_IP2_TSF                          (KSEG1+0x1E001000)
/***********************************************************************/

#define INCA_IP2_TSF_TSF_CONF      ((volatile u32*)(INCA_IP2_TSF + 0x0000))
#define INCA_IP2_TSF_KEY_CONF      ((volatile u32*)(INCA_IP2_TSF + 0x0004))
#define INCA_IP2_TSF_SREG01        ((volatile u32*)(INCA_IP2_TSF + 0x0008))
#define INCA_IP2_TSF_SREG23        ((volatile u32*)(INCA_IP2_TSF + 0x000C))
#define INCA_IP2_TSF_SREG456       ((volatile u32*)(INCA_IP2_TSF + 0x0010))
#define INCA_IP2_TSF_SREG7TO13     ((volatile u32*)(INCA_IP2_TSF + 0x0014))
#define INCA_IP2_TSF_LEDMUX_CONF   ((volatile u32*)(INCA_IP2_TSF + 0x0018))
#define INCA_IP2_TSF_LED_REG		((volatile u32*)(INCA_IP2_TSF + 0x001C))
#define INCA_IP2_TSF_PWM12         ((volatile u32*)(INCA_IP2_TSF + 0x0020))
#define INCA_IP2_TSF_SREG7TO10     ((volatile u32*)(INCA_IP2_TSF + 0x0024))
#define INCA_IP2_TSF_SREG11TP14    ((volatile u32*)(INCA_IP2_TSF + 0x0028))

//#define INCA_IP2_TSF_IRNEN			((volatile u32*)(INCA_IP2_TSF + 0x00F4))
//#define INCA_IP2_TSF_IRNICR			((volatile u32*)(INCA_IP2_TSF + 0x00F8))
//#define INCA_IP2_TSF_IRNCR			((volatile u32*)(INCA_IP2_TSF + 0x00FC))


#define INCAIP2_KEY_IRQ_LINE    INT_NUM_IM3_IRL28
#define INCAIP2_KEY_IRQ_NR    		 28


#define SSC_WRITE_REG(data, addr)   (*(volatile u32 *)(addr)) = (data) 
#define SSC_READ_REG(addr)          (*(volatile u32 *)(addr))

typedef struct
{
	volatile u32 * clc;			/* CLC register address */
	volatile u32 * id;			/* ID register address */
	volatile u32 * mcon;		/* MCON register address */
	volatile u32 * state;		/* STATE register address */
	volatile u32 * whbstate;	/* WHBSTATE register address */
	volatile u32 * txb;			/* TB tx buffer register address */
	volatile u32 * rxb;			/* RB rx buffer register address */
	volatile u32 * fstat;		/* FSTAT register address */
	volatile u32 * rxfcon;		/* RXFCON rx fifo control register address */
	volatile u32 * txfcon;		/* TXFCON tx fifo control register address */
	volatile u32 * br;			/* baudrate reload register */
	volatile u32 * brstate;		/* baudrate timer status register */
	volatile u32 * sfcon;		/* serial frame control register */
	volatile u32 * sfstate;		/* serial frame status register */
} SSC_Regs_t;

SSC_Regs_t sscReg[2] = 
{
	{ 
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_CLC),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_ID),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_CON),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_STATE),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_WHBSTATE),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_TB),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_RB),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_FSTAT),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_RXFCON),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_TXFCON),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_BR),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_BRSTAT),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_SFCON),
		(volatile u32 *)(INCA_IP2_SSC0_BASE+IFX_SSC_CON)
	},
	{
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_CLC),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_ID),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_CON),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_STATE),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_WHBSTATE),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_TB),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_RB),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_FSTAT),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_RXFCON),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_TXFCON),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_BR),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_BRSTAT),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_SFCON),
		(volatile u32 *)(INCA_IP2_SSC1_BASE+IFX_SSC_SFSTAT)
	}
}
;

typedef  const char *key_text_t;

key_text_t keyText[8][8] = {
	{ "0:0","0:1 <ok>","0:2 <down>","0:3 <right>","0:4 <9>","0:5 <3>","0:6 <mute>","0:7 <speaker>" },
	{ "1:0","1:1","1:2 <left>","1:3 <up>","1:4 <6>","1:5 <7>","1:6 <0>","1:7 <#>" },
	{ "2:0","2:1","2:2","2:3: <forward>","2:4 <5>","2:5 <headset>","2:6 <4>","2:7 <2>" },
	{ "3:0","3:1","3:2","3:3","3:4 <*>","3:5 <voice>","3:6 <8>","3:7 <1>" },
	{ "4:0","4:1","4:2","4:3","4:4","4:5","4:6 <release>","4:7" },
	{ "5:0","5:1","5:2","5:3","5:4","5:5","5:6","5:7" },
	{ "6:0","6:1","6:2","6:3","6:4","6:5","6:6","6:7" },
	{ "7:0","7:1","7:2","7:3","7:4","7:5","7:6","7:7" },
};

typedef  const char *mux_text_t;

mux_text_t muxAlt1Text[16][4] = {
	{ "KEY 12  ","GP 16   ","SCLK0   " ,"CLK_OUT2" },
	{ "KEY 13  ","GP 17   ","MTSR0   " ,"TXD1    " },
	{ "KEY 14  ","GP 18   ","MRST0   " ,"RXD1    " },
	{ "LED 6   ","GP 19   ","RES     " ,"RES     " },
	{ "LED 7   ","GP 20   ","RES     " ,"RES     " },
	{ "LED 8   ","GP 21   ","EXIN2   " ,"EBU_CS3 " },
	{ "LED 9   ","GP 22   ","EXIN1   " ,"EBU_CS2 " },
	{ "TXD0    ","GP 23   ","RES     " ,"RES     " },
	{ "RXD0    ","GP 24   ","RES     " ,"RES     " },
	{ "EXIN0   ","GP 25   ","RES     " ,"RES     " },
	{ "DP_USB  ","GP 26   ","RES     " ,"RES     " },
	{ "DN_USB  ","GP 27   ","RES     " ,"RES     " },
	{ "(res)   ","(res)   ","(res)   " ,"(res)   " },
	{ "(res)   ","(res)   ","(res)   " ,"(res)   " },
	{ "(res)   ","(res)   ","(res)   " ,"(res)   " },
	{ "(res)   ","(res)   ","(res)   " ,"(res)   " }
};

mux_text_t muxAlt2Text[16][4] = {
	{ "GP 0    ","FSC     ","RES     " ,"RES     " },
	{ "GP 1    ","DCL     ","RES     " ,"RES     " },
	{ "GP 2    ","DI      ","RES     " ,"RXD1    " },
	{ "GP 3    ","DO      ","SSC1_CS1" ,"RES     " },
	{ "GP 4    ","TC      ","SSC1_CS0" ,"RES     " },
	{ "GP 5    ","RESET   ","SCLK1   " ,"RES     " },
	{ "GP 6    ","CLK_OUT1","MTSR1   " ,"RES     " },
	{ "GP 7    ","CLK32   ","MRST1   " ,"RES     " },
	{ "A25     ","GP 8    ","PWM1    " ,"RES     " },
	{ "A24     ","GP 9    ","PWM2    " ,"RES     " },
	{ "RDBY    ","GP 10   ","RES     " ,"RES     " },
	{ "KEY 7   ","GP 11   ","RES     " ,"RES     " },
	{ "KEY 8   ","GP 12   ","RES     " ,"RES     " },
	{ "KEY 9   ","GP 13   ","RES     " ,"RES     " },
	{ "KEY 10  ","GP 14   ","SCC0_CS1" ,"RES     " },
	{ "KEY 11  ","GP 15   ","SSC0_CS0" ,"RES     " }
};


static char * binaryTextCore( u32 reg, char zero, char one )
{
	static char text[40] = { "0000 0000 0000 0000 0000 0000 0000 0000" };
	int index;
	int i,j;
	
	index = 38;
	
	for( i=0, j=0; i < 32; i++ )
	{
		if( j == 4 )
		{
			j = 0; text[index--] = ' ';
		}
		text[index--] = ( (reg & ( 1 << i )) != 0 )? one : zero;
		j++;
	}
	text[39] = 0;
	return( text );
}

static char * binaryText( u32 reg )
{
	return( binaryTextCore( reg, '0' , '1' ) );
}

static u32 muxAltReg( int pin )
{
	if( pin < 16 )
		return( *INCA_IP2_GPIO_ALTSEL2 );
	return( *INCA_IP2_GPIO_ALTSEL1 );
}

static mux_text_t muxText( int pin )
{
	u32 reg = muxAltReg( pin );
	int offset = ( pin < 16 ) ? pin : pin - 16;
	
	reg >>= (offset << 1);
	reg &= 0x3;
	if( pin < 16 )
	{
		return( muxAlt2Text[offset][reg] );
	}
	return( muxAlt1Text[offset][reg] );
}

static int is_gpio( int pin  )
{
	u32 reg = muxAltReg( pin );
	int offset = ( pin < 16 ) ? pin : pin - 16;

	reg >>= (offset << 1);
	reg &= 0x3;
	
	
	if( pin < 8 )
		return( reg == 0 );

	if( pin < 28 )
		return( reg == 1 );
	return( 0 );
}

volatile int keyIntCount;
volatile int keyIntNullCount;

static int scanInfo( u32 line, u32 scan )
{
	int i;
	int keyoffset = line+1;
	int ret = 0;
	
	for( i=0; i < 8; i++ )
	{
		if( (scan & ( 1 << i )) != 0 )
		{
			printk(KERN_NOTICE "key %d:%d %s %d %d\n",line,keyoffset+i,keyText[line][keyoffset+i],keyIntCount,keyIntNullCount);
			ret = 1;
		}
	}
	return( ret );
}

static void keyScanReg(void)
{
	u32 sreg01 = *INCA_IP2_TSF_SREG01;
	u32 sreg23 = *INCA_IP2_TSF_SREG23;
	u32 sreg456 = *INCA_IP2_TSF_SREG456;
	u32 sreg7to13 = *INCA_IP2_TSF_SREG7TO13;
	u32 sreg7to10 = *INCA_IP2_TSF_SREG7TO10;
	u32 sreg11TP14 = *INCA_IP2_TSF_SREG11TP14;
//	u32 irnen = *INCA_IP2_TSF_IRNEN;
//	u32 irnicr = *INCA_IP2_TSF_IRNICR;
//	u32 irncr = *INCA_IP2_TSF_IRNCR;
	
	u32 line0 = ((sreg01) & 0x3fff) ^ 0x3fff;
	u32 line1 = ((sreg01 >> 16) & 0x1fff) ^ 0x1fff;
	u32 line2 = ((sreg23) & 0xfff) ^ 0xfff;
	u32 line3 = ((sreg23 >> 16) & 0x7ff) ^ 0x7ff;
	u32 line4 = ((sreg456) & 0x3ff) ^ 0x3ff;
	u32 line5 = ((sreg456 >> 12 ) & 0x1ff) ^ 0x1ff;
	u32 line6 = ((sreg456 >> 24) & 0xff) ^ 0xff;
	u32 line7 = ((sreg7to13) & 0x7f) ^ 0x7f;

	printk(KERN_NOTICE "sreg01 = %08x\n",sreg01);
	printk(KERN_NOTICE "sreg23 = %08x\n",sreg23);
	printk(KERN_NOTICE "sreg456 = %08x\n",sreg456);
	printk(KERN_NOTICE "sreg7to13 = %08x\n\n",sreg7to13);
	printk(KERN_NOTICE "sreg7to10 = %08x\n",sreg7to10);
	printk(KERN_NOTICE "sreg11TP14 = %08x\n",sreg11TP14);
//	printk(KERN_NOTICE "irnen = %08x\n",irnen);
//	printk(KERN_NOTICE "irnicr = %08x\n",irnicr);
//	printk(KERN_NOTICE "irncr = %08x\n\n",irncr);

	printk(KERN_NOTICE "Line 0 %08x\n",line0);
	printk(KERN_NOTICE "Line 1 %08x\n",line1);
	printk(KERN_NOTICE "Line 2 %08x\n",line2);
	printk(KERN_NOTICE "Line 3 %08x\n",line3);
	printk(KERN_NOTICE "Line 4 %08x\n",line4);
	printk(KERN_NOTICE "Line 5 %08x\n",line5);
	printk(KERN_NOTICE "Line 6 %08x\n",line6);
	printk(KERN_NOTICE "Line 7 %08x\n\n",line7);
	
	scanInfo(0,line0);
	scanInfo(1,line1);
	scanInfo(2,line2);
	scanInfo(3,line3);
	scanInfo(4,line4);
	scanInfo(5,line5);
	scanInfo(6,line6);
	scanInfo(7,line7);
}

static u32 sreg01_prev;
static u32 sreg23_prev;
static u32 sreg456_prev;
static u32 sreg7to13_prev;
static u32 sreg7to10_prev;
static u32 sreg11TP14_prev;

static void keyScanRegQuiet(void)
{
	u32 line0;
	u32 line1;
	u32 line2;
	u32 line3;
	u32 line4;
	u32 line5;
	u32 line6;
	u32 line7;

	u32 sreg01 = *INCA_IP2_TSF_SREG01;
	u32 sreg23 = *INCA_IP2_TSF_SREG23;
	u32 sreg456 = *INCA_IP2_TSF_SREG456;
	u32 sreg7to13 = *INCA_IP2_TSF_SREG7TO13;
	u32 sreg7to10 = *INCA_IP2_TSF_SREG7TO10;
	u32 sreg11TP14 = *INCA_IP2_TSF_SREG11TP14;

//	u32 irnen = *INCA_IP2_TSF_IRNEN;
//	u32 irnicr = *INCA_IP2_TSF_IRNICR;
//	u32 irncr = *INCA_IP2_TSF_IRNCR;

	int keyCount = 0;
	
//	if( sreg01_prev != sreg01 )
//	{
//		printk(KERN_NOTICE "sreg01 = %08x\n",sreg01);
//	}
//	if( sreg23_prev != sreg23 )
//	{
//		printk(KERN_NOTICE "sreg23 = %08x\n",sreg23);
//	}
//	if( sreg456_prev != sreg456 )
//	{
//		printk(KERN_NOTICE "sreg456 = %08x\n",sreg456);
//	}
//	if( sreg7to13_prev != sreg7to13 )
//	{
//		printk(KERN_NOTICE "sreg7to13 = %08x\n\n",sreg7to13);
//	}
//	if( sreg7to10_prev != sreg7to10 )
//	{
//		printk(KERN_NOTICE "sreg7to10 = %08x\n",sreg7to10);
//	}
//	if( sreg11TP14_prev != sreg11TP14 )
//	{
//		printk(KERN_NOTICE "sreg11TP14 = %08x\n\n",sreg11TP14);
//	}

	sreg01_prev = sreg01;
	sreg23_prev = sreg23;
	sreg456_prev = sreg456;
	sreg7to13_prev = sreg7to13;
	sreg7to10_prev = sreg7to10;
	sreg11TP14_prev = sreg11TP14;
	
	line0 = ((sreg01) & 0x3fff) ^ 0x3fff;
	line1 = ((sreg01 >> 16) & 0x1fff) ^ 0x1fff;
	line2 = ((sreg23) & 0xfff) ^ 0xfff;
	line3 = ((sreg23 >> 16) & 0x7ff) ^ 0x7ff;
	line4 = ((sreg456) & 0x3ff) ^ 0x3ff;
	line5 = ((sreg456 >> 12 ) & 0x1ff) ^ 0x1ff;
	line6 = ((sreg456 >> 24) & 0xff) ^ 0xff;
	line7 = ((sreg7to13) & 0x7f) ^ 0x7f;
	
	if( line0 + line1 + line2 + line3 + line4 + line5 + line6 + line7  )
		printk("\n\nPress\n");
	
	keyCount += scanInfo(0,line0);
	keyCount += scanInfo(1,line1);
	keyCount += scanInfo(2,line2);
	keyCount += scanInfo(3,line3);
	keyCount += scanInfo(4,line4);
	keyCount += scanInfo(5,line5);
	keyCount += scanInfo(6,line6);
	keyCount += scanInfo(7,line7);
	
	if( keyCount == 0 )
	{
		keyIntNullCount++;
	}
	else
	{
//		printk(KERN_NOTICE "irnen = %08x\n",irnen);
//		printk(KERN_NOTICE "irnicr = %08x\n",irnicr);
//		printk(KERN_NOTICE "irncr = %08x\n\n",irncr);
	}
	keyIntCount++;
}

#define FREQ_393MHZ 393216000							/* 0x17700000 */
#define MIPSCOUNT_FREQ (393216000/2)					/* 0xbb80000 */
#define MIPSCLKS_TO_MSEC(x) ((x)/(MIPSCOUNT_FREQ/1000)) /* 196608 */
#define MIPSCLKS_TO_USEC(x) ((x)/(MIPSCOUNT_FREQ/1000000)) /* 196.608 rounds to 196 */

static void my_keypad_isr(int irq, void *dev_id, struct pt_regs *regs )
{
	u32 stime;
	u32 etime;
	u32 dtime;
//	u32 ntime;
//	u32 count = 0;
//	u32 delay = 0;
	stime = read_c0_count();


	while( (*INCA_IP2_ICU_IM3_ISR & INCA_IP2_ICU_IM3_IR(INCAIP2_KEY_IRQ_NR)) != 0 )
	*INCA_IP2_ICU_IM3_ISR = INCA_IP2_ICU_IM3_IR(INCAIP2_KEY_IRQ_NR);

	etime = read_c0_count();
	dtime = etime-stime;

	keyScanRegQuiet();

//	if( count != 0 )
//	{
//		printk( "Clear interrupt..after %d attemps clks=%d %dus\n",count,(dtime),MIPSCLKS_TO_USEC(dtime));
//	}
//	else
//	{
//		printk( "Clear interrupt..after %d attemps clks=%d %dus\n",count,(dtime),MIPSCLKS_TO_USEC(dtime));
//	}
	
//	ack_incaip2_irq(INCAIP2_KEY_IRQ_LINE);

}

static void tsf_settings( void )
{
	u32 keyConf = *INCA_IP2_TSF_KEY_CONF;
	u32 keyRange = keyConf & 0xf;
	char * keymode = ((keyConf & 0x10) == 0) ? "full matrix" : "standard matix";

	printk(KERN_NOTICE "key configuration %08X %s key0-%d\n",keyConf,keymode,keyRange);
	printk(KERN_NOTICE "led configuration %08X\n",*INCA_IP2_TSF_LEDMUX_CONF);
	printk(KERN_NOTICE "led register      %08X\n",*INCA_IP2_TSF_LED_REG);
	printk(KERN_NOTICE "pwm register      %08X\n",*INCA_IP2_TSF_PWM12);
//	*INCA_IP2_TSF_PWM12 &= 0xff;
//	*INCA_IP2_TSF_PWM12 |= (50 << 16);
	*INCA_IP2_TSF_LED_REG = 0x333;
//	printk(KERN_NOTICE "led register      %08X\n",*INCA_IP2_TSF_LED_REG);
//	printk(KERN_NOTICE "pwm register      %08X\n",*INCA_IP2_TSF_PWM12);
}

static void init_key_interrupt( void )
{
	tsf_settings();
    if( request_irq(INCAIP2_KEY_IRQ_LINE, my_keypad_isr, SA_INTERRUPT,"keypad-test", NULL) != 0 )
    {
		printk(KERN_NOTICE "key Interrupt busy!!!\n");
		free_irq(INCAIP2_KEY_IRQ_LINE, 0);
		request_irq(INCAIP2_KEY_IRQ_LINE, my_keypad_isr, SA_INTERRUPT,"keypad-test", NULL);
    }
}

static void externalInterrupt( void )
{
	printk(KERN_NOTICE "IM0        Register %08x %08x\n",*INCA_IP2_ICU_IM0_ISR,*INCA_IP2_ICU_IM0_ISR & (1<<22));
	printk(KERN_NOTICE "IM0 Mask   Register %08x %08x\n",*INCA_IP2_ICU_IM0_IER,*INCA_IP2_ICU_IM0_IER & (1<<22));

	printk(KERN_NOTICE "IM1        Register %08x %08x\n",*INCA_IP2_ICU_IM1_ISR,*INCA_IP2_ICU_IM1_ISR & (1<<8));
	printk(KERN_NOTICE "IM1 Mask   Register %08x %08x\n",*INCA_IP2_ICU_IM1_IER,*INCA_IP2_ICU_IM1_IER & (1<<8));

	printk(KERN_NOTICE "IM2        Register %08x %08x\n",*INCA_IP2_ICU_IM2_ISR,*INCA_IP2_ICU_IM2_ISR & (1<<23));
	printk(KERN_NOTICE "IM2 Mask   Register %08x %08x\n",*INCA_IP2_ICU_IM2_IER,*INCA_IP2_ICU_IM2_IER & (1<<23));

	printk(KERN_NOTICE "IM3        Register %08x %08x\n",*INCA_IP2_ICU_IM3_ISR,*INCA_IP2_ICU_IM3_ISR & (1<<29));
	printk(KERN_NOTICE "IM3 Mask   Register %08x %08x\n",*INCA_IP2_ICU_IM3_IER,*INCA_IP2_ICU_IM3_IER & (1<<29));

	printk(KERN_NOTICE "IM4        Register %08x %08x\n",*INCA_IP2_ICU_IM4_ISR,*INCA_IP2_ICU_IM4_ISR & (1<<22));
	printk(KERN_NOTICE "IM4 Mask   Register %08x %08x\n",*INCA_IP2_ICU_IM4_IER,*INCA_IP2_ICU_IM4_IER & (1<<22));

	printk(KERN_NOTICE "IM5        Register %08x %08x\n",*INCA_IP2_ICU_IM5_ISR, *INCA_IP2_ICU_IM5_ISR & (1<<6));
	printk(KERN_NOTICE "IM5 Mask   Register %08x %08x\n\n",*INCA_IP2_ICU_IM5_IER, *INCA_IP2_ICU_IM5_IER & (1<<6) );

	printk(KERN_NOTICE "EIU CR     Register %08x\n",*INCA_IP2_ICU_EIU_CR);
	printk(KERN_NOTICE "EIU IRNEN  Register %08x\n",*INCA_IP2_ICU_EIU_IRNEN);
	printk(KERN_NOTICE "EIU IRNCR  Register %08x\n",*INCA_IP2_ICU_EIU_IRNCR);
	printk(KERN_NOTICE "EIU IRNICR Register %08x\n\n",*INCA_IP2_ICU_EIU_IRNICR);
}


static void operaEbuSetting( void )
{
	u32 ebu_addr[4];
	u32 ebu_con[4];
	int i,j;
	u32 ebuClock = incaip2_get_fbs0clk()/1000;
	
	u32 ebuWriteTime;
	u32 ebuReadTime;
	u32 ebuReadClks;
	u32 ebuWriteClks;
 	u32 addrMask;
	
	
	ebu_addr[0] = *INCA_IP2_EBU_ADDR_SEL_0;
	ebu_addr[1] = *INCA_IP2_EBU_ADDR_SEL_1;
	ebu_addr[2] = *INCA_IP2_EBU_ADDR_SEL_2;
	ebu_addr[3] = *INCA_IP2_EBU_ADDR_SEL_3;
	
	ebu_con[0] = *INCA_IP2_EBU_CON_0;
	ebu_con[1] = *INCA_IP2_EBU_CON_1;
	ebu_con[2] = *INCA_IP2_EBU_CON_2;
	ebu_con[3] = *INCA_IP2_EBU_CON_3;

/*
 * 0x10000000 - 0x13FFFFFF EBU segment 1 64M
 * 0x19000000 - 0x19FFFFFF EBU segment 2 16M
 * 0x1B000000 - 0x1BFFFFFF EBU segment 3 16M
 * 0x1C000000 - 0x1C0FFFFF EBU segment 4  1M
 * 0x1C100000 - 0x1C1FFFFF EBU segmnet 5  1M
 */
 
 
	for(i=0;i<4;i++)
	{
		u32 su = ((u32)ebu_con[i]>>22) & 0x1;
		u32 wide = ((u32)ebu_con[i]>>16) & 0x3;
//		u32 wr = ((u32)ebu_con[i] >> 8) & 0x7;
		u32 rd = ((u32)ebu_con[i] >> 6 ) & 0x3;
		u32 hld = ((u32)ebu_con[i] >> 4 ) & 0x3;
		u32 rcv = ((u32)ebu_con[i] >> 2 ) & 0x3;
		u32 mul = ((u32)ebu_con[i] & 0x3);
		u32 mulcalc = (mul == 0)? 1 : 2 << mul;
//		u32 wait = ((u32)ebu_con[i]>>20) & 0x3;
		u32 mask;
		u32 regionEnabled;
		u32 startAddr;
		u32 ebu_segment;
		u32 segStartAddr;
		u32 segEndAddr;
		u32 segEndAddrLimit;
		
		ebuReadClks = 2+su+((rd+rcv)*mulcalc);
		ebuWriteClks = 2+su+((rd+rcv+hld)*mulcalc);
		
		ebuReadTime = (ebuReadClks*1000000)/(ebuClock);
		ebuWriteTime = (ebuWriteClks*1000000)/(ebuClock);

		 addrMask = 0x08000000;
		 mask = (ebu_addr[i] & 0xf0) >> 4;
		 regionEnabled = (ebu_addr[i] & 0x1);
		 startAddr = ebu_addr[i]& 0xffffff00;
		 
		 for( j=0; j<mask;j++)
		 {
		 	addrMask |= (0x04000000 >> j);
		 }
 
		 addrMask = 0xffffff & ~addrMask;

		 if( startAddr >= 0x10000000 && startAddr < 0x14000000 )
		 {
		 	segStartAddr = startAddr - 0x10000000;
		 	segEndAddr = (startAddr | addrMask) - 0x10000000;
		 	segEndAddrLimit = 0x4000000;
		 	ebu_segment = 1;
		 }
		 else if( startAddr >= 0x19000000 && startAddr < 0x1A000000 )
		 {
		 	segStartAddr = startAddr - 0x19000000;
		 	segEndAddr = (startAddr | addrMask) - 0x19000000;
		 	segEndAddrLimit = 0x1000000;
		 	ebu_segment = 2;
		 }
		 else if( startAddr >= 0x1B000000 && startAddr < 0x1C000000 )
		 {
		 	segStartAddr = startAddr - 0x1B000000;
		 	segEndAddr = (startAddr | addrMask) - 0x1B000000;
		 	segEndAddrLimit = 0x1000000;
		 	ebu_segment = 3;
		 }
		 else if( startAddr >= 0x1C000000 && startAddr < 0x1C100000 )
		 {
		 	segStartAddr = startAddr - 0x1C000000;
		 	segEndAddr = (startAddr | addrMask) - 0x1C000000;
		 	segEndAddrLimit = 0x100000;
		 	ebu_segment = 4;
		 }
		 else if( startAddr >= 0x1C100000 && startAddr < 0x1C200000 )
		 {
		 	segStartAddr = startAddr - 0x1C100000;
		 	segEndAddr = (startAddr | addrMask) - 0x1C100000;
		 	segEndAddrLimit = 0x100000;
		 	ebu_segment = 5;
		 }
		 else
		 {
		 	segStartAddr = startAddr;
		 	segEndAddr = (startAddr | addrMask);
		 	segEndAddrLimit = segEndAddr;
		 	ebu_segment = 6;
		 }

		
		printk(KERN_NOTICE "EBU CS %d at %08x %08x %s\n",i,startAddr,startAddr | addrMask, (regionEnabled)? "enabled":"disabled" ) ;
		printk(KERN_NOTICE "EBU SEG %d address %08x - %08x Limit=%08x\n",ebu_segment,segStartAddr,segEndAddr , segEndAddrLimit) ;
		printk(KERN_NOTICE "EBU_ADDR_SEL reg %08x\n",(u32)ebu_addr[i] );
		printk(KERN_NOTICE "EBU_CON reg      %08x\nData width=%s\n",(u32)ebu_con[i],(wide==0)? "8 bit":"16 bit");
//		printk(KERN_NOTICE "su=%d wr=%d rd=%d hld=%d rcv=%d mul=%d\n",su,wr,rd,hld,rcv,mul);
//		printk(KERN_NOTICE "Multiplier mul=%d gives %d\nVariable Wait state insertion=%d %s\n",mul, mulcalc,wait,(wait!=0)? "async":"disabled");
		printk(KERN_NOTICE "Read  Clks %d access=%d ns\n",ebuReadClks, ebuReadTime);
		printk(KERN_NOTICE "Write Clks %d access=%d ns\n\n",ebuWriteClks, ebuWriteTime);
	}
}

#define read_c0_intCtl()	__read_32bit_c0_register($12, 1)
#define read_c0_SRSCtl()	__read_32bit_c0_register($12, 2)
#define read_c0_SRSMap()	__read_32bit_c0_register($12, 3)
#define read_c0_Ebase()	    __read_32bit_c0_register($15, 1)
#define read_c0_perfCtl_0()	    __read_32bit_c0_register($25, 0)
#define read_c0_perfCount_0()	__read_32bit_c0_register($25, 1)
#define read_c0_perfCtl_1()	    __read_32bit_c0_register($25, 2)
#define read_c0_perfCount_1()	__read_32bit_c0_register($25, 3)


mux_text_t config7Text[6][3] = {
	{ "SL    Scheduled Loads misses non Blocking ","Yes","No" },
	{ "BP    Branch Prediction is speculative    ","Yes","No" },
	{ "RPS   Return Prediction Stack is Enabled  ","Yes","No" },
	{ "BHT   Branch History Table is Enabled     ","Yes","No" },
	{ "ULB   Uncached Loads non blocking         ","Yes","No" },
	{ "NBLSU Non Blocking LSU                    ","Yes","ALU/LSU Tied" }
};


static void config7_decode( u32 reg )
{
	int i;
	
	for(i = 0; i < 6; i++ )
	{
		int index = ( (reg & (1 << i)) == 0 ) ? 1 : 2;
		printk("%s , %s", config7Text[i][0], config7Text[i][index]);
	}
}

static u32 setConfig7( const char *str )
{
	if( strcmp("SL", str) == 0 )
		return( 1 );
	if( strcmp("BP", str) == 0 )
		return( 2 );
	if( strcmp("RPS", str) == 0 )
		return( 4 );
	if( strcmp("BHT", str) == 0 )
		return( 8 );
	if( strcmp("ULB", str) == 0 )
		return( 16 );
	if( strcmp("NBLSU", str) == 0 )
		return( 32 );
	return( 0 );
}

#define SET_REQUEST (0)
#define CLEAR_REQUEST (1)
#define SHOW_REQUEST (2)

static const char *getCommand( const char *str, u32 *cmd )
{
	*cmd = SET_REQUEST;
	if( strlen(str) == 0 )
		return( str );
	if( strncmp("show ", str, 4) == 0 )
	{
		*cmd = SHOW_REQUEST;
		return( str+5 );
	}
	if( strncmp("set ", str, 4) == 0 )
	{
		*cmd = SET_REQUEST;
		return( str+4 );
	}
	if( strncmp("clear ", str, 6) == 0 )
	{
		*cmd = CLEAR_REQUEST;
		return( str+6 );
	}
	return( str );
}

static void SRSCtl_decode( u32 reg )
{
	u32 value;
		
	printk("\nSRSCtl:\n" );
	value = reg & 0xf;
	printk("CSS   = %d\n", value );
	reg >>= 6;
	value = reg & 0xf;
	printk("PSS   = %d\n", value );
	reg >>= 6;
	value = reg & 0xf;
	printk("ESS   = %d\n", value );
	reg >>= 6;
	value = reg & 0xf;
	printk("EICSS = %d\n", value );
	reg >>= 8;
	value = reg & 0xf;
	printk("HSS   = %d\n\n", value );
}

static void ProcessID_decode( u32 reg )
{
	u32 CompanyOption = (reg >> 24);
	u32 CompanyID  = ((reg >> 16) & 0xff);
	u32 ProcessorID = ((reg >> 8) & 0xff);
	u32 MajorRevision = ((reg >> 5) & 0x7);
	u32 MinorRevision =((reg >> 2) & 0x7);
	u32 patch = (reg & 0x3);

		
	printk("CompanyOption   %3d\n", CompanyOption );
	printk("CompanyID       %3d\n", CompanyID );
	printk("ProcessorID     %3xh\n", ProcessorID );
	printk("Revision        %3d.%d P:%d\n\n", MajorRevision,MinorRevision, patch);
}
/*
The first column is as documented
in the schematic. The second column
is as read from the GPIO port. This
is just a byte swop of the first.

  			ttvv    vvtt
OS20 		0000	0000
OS20G 		0010	1000
OS40 		0100	0001
OS40G 		0110	1001
OS60 		1010	1010
OS60G 		1000	0010
OS80 		1100	0011
OS80G 		1110	1011
OS15  		0001	0100
os40 CORE 	0101	0101
*/
static void phoneVersion(void)
{
	u32 Phonetype = ((*INCA_IP2_GPIO_PIN) >> 12) & 0xF;
	char *p = "(null)";
	
	switch( Phonetype )
	{
	case 0x0: p = "OS20"; break;
	case 0x1: p = "OS40"; break;
	case 0x2: p = "OS60"; break;
	case 0x3: p = "OS80"; break;
	case 0x4: p = "OS15"; break;
	case 0x5: p = "OS40 CORE"; break;
	case 0x8: p = "OS20G"; break;
	case 0x9: p = "OS40G"; break;	
	case 0xA: p = "OS60G"; break;	
	case 0xB: p = "OS80G"; break;	
	}
	printk(KERN_NOTICE "Phone Type %s (%02X)\n",p,Phonetype);
}

static void chipVersion(void)
{
	u32 chipId;
	u32 version;
	u32 part;
	u32 manufacturer;
	u32 subversion;
	u32 reg0, reg1, reg2, reg3, reg4, reg5, reg6, reg7, reg8;

	char *p;
	char *m;
	
	chipId = *INCA_IP2_MPS_CHIPID;
	subversion = *INCA_IP2_MPS_SUBVER;
	version = INCA_IP2_MPS_CHIPID_VERSION_GET(chipId);
	part = INCA_IP2_MPS_CHIPID_PARTNUM_GET(chipId);
	manufacturer = INCA_IP2_MPS_CHIPID_MANID_GET(chipId);
	
	if( part == 0x118 )
		p = "INCA-IP2S - (Standard)";
	else if( part == 0x119 )
		p = "INCA-IP2C - (Compact)";
	else if( part == 0x11A )
		p = "INCA-IP2F - (Cost reduced)";
	else
		p = "UnKnown";

	m = (manufacturer == 0xC1)? "Infineon":"Unknown"; 	
	printk(KERN_NOTICE "%s %s Version 1.%d:%d\n\n",m,p,version,subversion);
	
	
	printk(KERN_NOTICE "CPU Clk@ %dHz  FPI 0 Clk@ %dHz FPI 1 Clk@ %dHz\n\n",incaip2_get_cpu_hz(), incaip2_get_fbs0clk(), incaip2_get_fpi_hz());

	
	reg0 = read_c0_config7();
	reg1 = read_c0_intCtl();
	reg2 = read_c0_SRSCtl();
	reg3 = read_c0_SRSMap();
	reg4 = read_c0_Ebase();
	reg5 = read_c0_perfCtl_0();
	reg6 = read_c0_perfCount_0();
	reg7 = read_c0_perfCtl_1();
	reg8 = read_c0_perfCount_1();
	printk(KERN_NOTICE "config 0    %08x\n",  read_c0_config());
	printk(KERN_NOTICE "config 3    %08x\n",  read_c0_config3());
	printk(KERN_NOTICE "config 7    %08x\n", reg0);
	printk(KERN_NOTICE "intCtl      %08x\n", reg1);
	printk(KERN_NOTICE "SRSCtl      %08x\n", reg2);
	printk(KERN_NOTICE "SRSMap      %08x\n", reg3);
	printk(KERN_NOTICE "Ebase       %08x\n", reg4);
	printk(KERN_NOTICE "perfCtl_0   %08x\n", reg5);
	printk(KERN_NOTICE "perfCount_0 %08x\n", reg6);
	printk(KERN_NOTICE "perfCtl_1   %08x\n", reg7);
	printk(KERN_NOTICE "perfCount_1 %08x\n\n", reg8);
	
//	write_c0_config7(0);
	printk(KERN_NOTICE "config 7    %08x %s\n", read_c0_config7(), binaryText(read_c0_config7()) );
	printk(KERN_NOTICE "SRSCtl      %08x %s\n", reg2, binaryText(reg2) );
	printk(KERN_NOTICE "Processor   %08x %s\n", read_c0_prid(), binaryText(read_c0_prid()) );
	
	SRSCtl_decode(reg2);
	ProcessID_decode(read_c0_prid());
	
}

static void keyPullups(void)
{
	u32 pullup = *INCA_IP2_GPIO_PUDEN;
//	u32 direction = *INCA_IP2_GPIO_PUDSEL;
	u32 keypulls = (pullup & (0x7fff0)) >> 4;
	printk(KERN_NOTICE "All Pull up %08x\n",pullup);
//	printk(KERN_NOTICE "Direction %08x\n",direction);
	printk(KERN_NOTICE "Keys Pulled up %02X\n",keypulls);

	*INCA_IP2_GPIO_PUDEN |= 0xff0;
//	*INCA_IP2_GPIO_PUDSEL |= 0xff0;

	pullup = *INCA_IP2_GPIO_PUDEN;
	keypulls = (pullup & (0x7fff0)) >> 4;
	printk(KERN_NOTICE "All Pull up %08x\n",pullup);
	printk(KERN_NOTICE "Keys Pulled up %02X\n",keypulls);
//	printk(KERN_NOTICE "Direction %08x\n",*INCA_IP2_GPIO_PUDSEL);

}

static void sccInfo(void)
{
	u32 ssc0id = SSC_READ_REG(INCA_IP2_SSC0_BASE+IFX_SSC_ID);
	u32 ssc1id = SSC_READ_REG(INCA_IP2_SSC1_BASE+IFX_SSC_ID);
	u32 ssc0FifoTxSize = (SSC_READ_REG(INCA_IP2_SSC0_BASE+IFX_SSC_ID) & 0x3f000000)>>24;
	u32 ssc0FifoRxSize = (SSC_READ_REG(INCA_IP2_SSC0_BASE+IFX_SSC_ID) & 0x003f0000)>>16;
	u32 ssc1FifoTxSize = (SSC_READ_REG(INCA_IP2_SSC1_BASE+IFX_SSC_ID) & 0x3f000000)>>24;
	u32 ssc1FifoRxSize = (SSC_READ_REG(INCA_IP2_SSC1_BASE+IFX_SSC_ID) & 0x003f0000)>>16;

	u32 ssc0txCon = SSC_READ_REG(INCA_IP2_SSC0_BASE+IFX_SSC_TXFCON);
	u32 ssc0rxCon = SSC_READ_REG(INCA_IP2_SSC0_BASE+IFX_SSC_RXFCON);

	u32 ssc1txCon = SSC_READ_REG(INCA_IP2_SSC1_BASE+IFX_SSC_TXFCON);
	u32 ssc1rxCon = SSC_READ_REG(INCA_IP2_SSC1_BASE+IFX_SSC_RXFCON);

	printk(KERN_NOTICE "SSC0 ID %08x\n     txFifo=%d rxFifo=%d DMA %s\n",ssc0id,ssc0FifoTxSize,ssc0FifoRxSize,(ssc0id & 0x20)? "Yes":"No");
	printk(KERN_NOTICE "     TXFCON %08x\n     RXFCON %08x\n\n",ssc0txCon,ssc0rxCon);
	printk(KERN_NOTICE "SSC1 ID %08x\n     txFifo=%d rxFifo=%d DMA %s\n",ssc1id,ssc1FifoTxSize,ssc1FifoRxSize,(ssc1id & 0x20)? "Yes":"No");
	printk(KERN_NOTICE "     TXFCON %08x\n     RXFCON %08x\n\n",ssc1txCon,ssc1rxCon);
}


static void muxShow( void )
{
	int i;
	int direction;
	char * dirText;
	char * stateText;
	u32 stateReg;
//	u32 GpioAltsel1 = *INCA_IP2_GPIO_ALTSEL1;
//	u32 GpioAltsel2 = *INCA_IP2_GPIO_ALTSEL2;
	u32 GpioPin = *INCA_IP2_GPIO_PIN;
	u32 GpioDir = *INCA_IP2_GPIO_PDIR;
	u32 GpioOut = *INCA_IP2_GPIO_POUT;

	for( i=0 ; i < 28; i++ )
	{


		/* INCA-IP2 bug GPIO 8 and 9 direction is back to front on version 1.2 and latter */

		if(  (i != 8 && i != 9) || (incaip2_get_cpu_ver() < 2) )
			direction = (((1 << i ) & GpioDir) == 0 ) ? 0 : 1;
		else
			direction = (((1 << i ) & GpioDir) == 0 ) ? 1 : 0;

		/* INCA-IP2 bug GPIO 26 controls direct of 26 and 27 */
		if( i == 27  )
			direction = (((1 << 26 ) & GpioDir) == 0 ) ? 0 : 1;

		if( direction == 0 )
		{
			dirText = "input ";
			stateReg = GpioPin;
		}
		else
		{
			dirText = "output";
			stateReg = GpioOut;
		}

		stateText = (( (1 << i ) & stateReg) == 0 ) ? "low ": "high";
		
		if( is_gpio( i ) )
		{
			printk(KERN_NOTICE "%s %s - %s port\n",muxText( i ),dirText,stateText);
		}
		else
		{
			printk(KERN_NOTICE "%s %s - %s\n",muxText( i ),dirText,stateText);
		}
	}
}

static void muxSettings(void)
{
	u32 GpioAltsel1 = *INCA_IP2_GPIO_ALTSEL1;
	u32 GpioAltsel2 = *INCA_IP2_GPIO_ALTSEL2;
	u32 GpioPin = *INCA_IP2_GPIO_PIN;
	u32 GpioDir = *INCA_IP2_GPIO_PDIR;
	u32 GpioOut = *INCA_IP2_GPIO_POUT;
	u32 GpioPuden = *INCA_IP2_GPIO_PUDEN;
	u32 GpioPudsel = *INCA_IP2_GPIO_PUDSEL;
//	u32 keypulls = (GpioPuden & (0x7fff0)) >> 4;

	printk(KERN_NOTICE "Gpio Alt SEL 1  %s %08x\n",binaryText(GpioAltsel1), GpioAltsel1);
	printk(KERN_NOTICE "Gpio Alt SEL 2  %s %08x\n",binaryText(GpioAltsel2), GpioAltsel2);
	printk(KERN_NOTICE "Gpio Dir        %s %08x\n",binaryText(GpioDir), GpioDir);
	printk(KERN_NOTICE "Gpio In         %s %08x\n",binaryText(GpioPin), GpioPin);
	printk(KERN_NOTICE "Gpio Out        %s %08x\n",binaryText(GpioOut), GpioOut);
	printk(KERN_NOTICE "Gpio Puden      %s %08x\n",binaryText(GpioPuden), GpioPuden);
	printk(KERN_NOTICE "Gpio PudSel     %s %08x\n",binaryText(GpioPudsel), GpioPudsel);

}

#ifdef CONFIG_PROC_FS


int ip2_hw_proc_read(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
	int i;
	int len=0;
	u32 reg = read_c0_config7();
    len+=sprintf(buf+len,"E106 Incorrect Implementation of Config7.BP and Config7.BHT fields.\nThe fix is to swap the bits\n");
    len+=sprintf(buf+len,"Config 7    %08x\n\n%s\n\n", reg, binaryText( reg ));

	for(i = 0; i < 6; i++ )
	{
		int index = ( (reg & (1 << i)) == 0 ) ? 1 : 2;
		len += sprintf(buf+len,"%s , %s\n", config7Text[i][0], config7Text[i][index]);
	}
        
	len += sprintf(buf+len,"\n");
  	*eof = 1;
   	return len;
}

int ip2_hw_proc_write(struct file *file, const char *buffer,unsigned long count, void *data)
{
	char buff[50];
	const char *p;
	int len;
	u32 reg;
	u32 cmd;
	
	len = ( count < sizeof(buff) )? count : sizeof(buff);

	memcpy( buff, buffer,len);
	buff[len-1] = 0;

	p = getCommand(buff, &cmd );
	
	if( cmd == SHOW_REQUEST)
	{
		if( strcmp( p, "ebu" )== 0)			
			operaEbuSetting();
		if( strcmp( p, "mux" )== 0)
			muxShow();
		if( strcmp( p, "key" )== 0)
			init_key_interrupt();
		if( strcmp( p, "version" )== 0)
			chipVersion();
		return( count );
	}
	reg = setConfig7( p );
	if( cmd == CLEAR_REQUEST )
	{
		printk("clear %s %08x ", p, reg);
		reg = read_c0_config7() & (~reg);
	}
	else
	{
		printk("set %s %08x ", p, reg);		
		reg |= read_c0_config7();
	}
	printk("-> %08x\n", reg);
	write_c0_config7( reg );

	return count;
}
#endif /* CONFIG_PROC_FS */

static int __init operaInfo_init(void)
{
//	printk(KERN_NOTICE "INCA_IP2_MPS_CHIPID %08x\n",*INCA_IP2_MPS_CHIPID);
//	printk(KERN_NOTICE "INCA_IP2_MPS_SUBVER %08x\n",*INCA_IP2_MPS_SUBVER);
	
	

//	printk(KERN_NOTICE "\nINCA_IP2_GPIO_PDIR %08x\n",*INCA_IP2_GPIO_PDIR);
//	printk(KERN_NOTICE "INCA_IP2_GPIO_POUT %08x\n",*INCA_IP2_GPIO_POUT);
	
//	keyPullups();
	
	chipVersion();
	phoneVersion();
	
//	operaEbuSetting();

//	muxSettings();
//	muxShow();
	
//	*INCA_IP2_GPIO_ALTSEL1 &= ~( (0x3<<18) | (0x3 << 12) );
//	*INCA_IP2_GPIO_ALTSEL1 |= ( (0x1<<18) | (0x1 << 12) );
//	*INCA_IP2_GPIO_POUT &= ~1;
//
//	printk(KERN_NOTICE "--------------------------\n");
//	muxShow();

//	sccInfo();

//	externalInterrupt();
	
//	keyScanReg();

//	init_key_interrupt();
#ifdef CONFIG_PROC_FS
	entry = create_proc_entry("driver/ip2HW", 0, 0);
	entry->read_proc  = ip2_hw_proc_read;
	entry->write_proc = ip2_hw_proc_write;
#endif /* CONFIG_PROC_FS */
	
	return 0; 
}
							   
static void __exit operaInfo_exit(void)
{
//	free_irq(INCAIP2_KEY_IRQ_LINE, 0);
	remove_proc_entry("driver/ip2HW", 0);
	printk(KERN_NOTICE "done\n");
}

module_init(operaInfo_init);
module_exit(operaInfo_exit);
