/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#ifndef _DEVDRIVERS_H_
#define _DEVDRIVERS_H_

void writeLatch (unsigned char yMask, unsigned char ySetReset);
/**
*	Synchronous Serial Controller settings 
*/

# define MINI_HSI_SSC_NUM (0)
# define MINI_HSI_SSC_INODE ((struct inode *)MINI_HSI_SSC_NUM)
# define MINI_HSI_SSC_BAUDRATE 32000
# define MINI_HSI_DATAWIDTH 8

#ifdef __KERNEL__
/**
*	Kernel interface to write to device	
*/
int quantum_write_spi(void *buffer, int len );
/**
*	Kernel interface to read from device	
*/
int quantum_read_spi(void *buffer, int len );

/**
*	Callback function for ssc frame end	
*/

void quantum_frame_end_int(int port);

#endif

/**
*	Callback function for Led update	
*/

int QLedOutput( void );

typedef struct {
	char* txdata;
	char* rxdata;
	int length;
	int chip_select;
	
} spi_transaction_data_t;

/**
*	structure for the data from the QTouch device
*/
typedef struct {
	unsigned char poll;
} poll_data_t;

/**
*	structure for the data from the QLed device
*/

typedef struct {
	
	int fadeTime;
	unsigned char fadeDirection;	
	unsigned char fadeLevel;
	unsigned char lampLevel;	
	
} backlight_data_t;

/**
*	structure for the data for the Backlight device
*/

typedef struct {
	unsigned int ledmap[2];
	unsigned char dmledmap[2];	
} led_data_t;
/**
*	Structure for the data from the QWheel device
*/
typedef struct {
	unsigned char btnStatus;
	signed char xMove;
	signed char yMove;
	signed char zMove;
} mPacket_t;

/**
*	Structure for the data from the hwVersion module
*/
typedef struct {
	signed int  hwVersion;
	unsigned long  freeRAM;
} hwStats_t;



#define MPKT_Y_OVFL	(1<<7)
#define MPKT_X_OVFL	(1<<6)
#define MPKT_Y_SIGN	(1<<5)
#define MPKT_X_SIGN	(1<<4)
#define MPKT_IS_ONE	(1<<3)
#define MPKT_M_BTN	(1<<2)
#define MPKT_R_BTN	(1<<1)
#define MPKT_L_BTN	(1<<0)



/** 
*	Macros for the QWheel device
*/
#define TOUCH_BIT (1<<7)
#define QW_TYPE_BIT (1<<6)
#define QW_POLE_BIT (1<<1)
#define QW_PROX_BIT (1<<0)
#define IS_TOUCHED(x) ((x)& TOUCH_BIT)
#define QWHEEL_DATA(x) ((x) & (~TOUCH_BIT))

#define MOUSEDEV_MINOR_BASE 	32
#define MOUSEDEV_MINORS		32
#define MOUSEDEV_MIX		31

#define QKEYERROR_BIT (1<<7)
#define QKEYRESV_BIT (1<<6)
#define QKEYDETECT_BITS ((1<<5) | (1<<4))
#define QKEYSINGLE_BIT (1<<4)
#define QKEY_ERROR(x) ( ((x) & (QKEYDETECT_BITS|QKEYERROR_BIT)) == QKEYERROR_BIT )
#define QKEY_DETECT(x) ( (x) & QKEYDETECT_BITS )
#define QKEY_DATA(x) ((x) & 0xF)

/**
*	SetupBlock (36 bytes) controls the internal operation of the QTouch device, 
*	including critical functions such as sensitivity, filtering, sample rate, 
*   and communication parameters.
*\n
*\n	bytes 0  - 9(bits 7-4)	: NTHR - Netgative Threshold Bits
*\n	bytes 0  - 9(bits 3-2)	:		 Not Used
*\n	byte  0  - 9(bits 1-0)	: NHYS - Negative Hysteresis Bits
*\n	bytes 10 - 19(bits 7-4)	: NDCR - Negative Drift Comp Bits
*\n	bytes 10 - 19(bits 3-0)	: NRRD - Negative Recal Delay Bits
*\n	bytes 20 - 29 (bit 7)	: AKS  - Adjacent Key Suppresstion Bits
*\n	bytes 20 - 29 (bit 6)	:		 Not Used
*\n	bytes 20 - 29 (bit 5)	: EK   - Error Key Control Bits
*\n	bytes 20 - 29 (bit 4)	: K2L  - Key-To-LED Bit
*\n	bytes 20 - 29 (bits 3-0): NDIL - Negative Detect Integrator Bits
*\n	byte  30 (bits 7-4)		: PTHR - Positive Threshold Bits
*\n	byte  30 (bits 3-0)		: PHYS - Positive Hysteresis Bits
*\n	byte  31(bits 7-4)		: PDCR - Positive Drift Comp Bits
*\n	byte  31(bits 3-0)		: PRD  - Positive Recal Delay Bits
*\n byte  32(bits 7-4)		: FDIL - Positive Detect Integrator Bits
*\n byte  32(bit 3)			: LEDP - LED Polarity Function Control Bit
*\n byte  32(bit 2)			: NTM  - Negative Threshold Mode
*\n byte  32(bits 1)		: SE   - Sync Enable Bit
*\n byte  32(bits 0)		: SYNC - Sync Bit
*\n	byte  33				: LBLL - Lower Burst Length Limit
*\n byte  34(bit 7)			: KEYO - Key Output Cobtrol Bit
*\n byte  34(bits 6-5)		:		 Not Used
*\n	byte  34(bits 4-2)		: BS   - Burst Spacing Control Bits
*\n	byte  34(bits 1-0)		: BR   - Baud rate Bits
*\n	byte  35				: HCRC - Host CRC
*/
/*const unsigned char default_SetupBlock[35] = {0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,0x93,
											   0x75,0x75,0x75,0x75,0x75,0x75,0x75,0x75,0x75,0x75,
											   0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,0x02,
											   0x51,0x57,0x50,0x03,0x69};*/

#endif //_DEVDRIVERS_H_
