/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#include <linux/module.h>
#include <asm/types.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <linux/devfs_fs_kernel.h>

#include <linux/tqueue.h> /* task queues */
#include <asm/incaip2/ifx_types.h>

#include <asm/incaip2/irq.h>
#include <asm/incaip2/mux.h>
#include <asm/incaip2/port.h>
#include <asm/incaip2/ifx_ssc.h>
#include "Quantum_drivers.h"
#include "latch.h"
#include "fpga.h"

int ifx_ssc_open(struct inode *, struct file *);
int ifx_ssc_close(struct inode *, struct file *);
int ifx_ssc_ioctl(struct inode *, struct file *, unsigned int, unsigned long);



/**
* standard success return definition
*/
#define OK	0

/**
* inca_ip parallel port module_id to control ownership of the port
*/
#define PORT_MODULE_ID 0xff


/**
* Development debug print switch
*/
#define QW_DEBUG

//static int major = 0;	// Major ID of ledmatrix device (default: 0=dynamic)
/**
* Module parameter declaration of variable major as type integer
*/
MODULE_PARM(major, "i");
/**
* Module parameter description of variable major
*/
MODULE_PARM_DESC(major, "Major ID of ledmatrix device (default: 0=dynamic)");
/**
* Length of command queue buffer
*/
#define CMD_Q_LEN	(1<<4)
/**
* Mask for command queue buffer
*/
#define CMD_Q_MASK	((1<<4)-1)



/**
 *	Initialisation of the INCAIP SSC interface
 *\returns success (0) or error code
*/

//#define QW_DEBUG_SSC 1

void ssc_show_hwopts( struct ifx_ssc_hwopts *x_opts)
{
		printk("SSC Hardware options\n");
		printk("x_opts.modeRxTx=%x\n",(u32)x_opts->modeRxTx);
		printk("x_opts.masterSelect=%x\n",(u32)x_opts->masterSelect);
		printk("x_opts.dataWidth=%x\n", (u32)x_opts->dataWidth);
		printk("x_opts.loopBack=%x\n",(u32)x_opts->loopBack);
		printk("x_opts.clockPolarity=%x\n", (u32)x_opts->clockPolarity);
		printk("x_opts.clockPhase=%x\n", (u32)x_opts->clockPhase);
		printk("x_opts.headingControl=%x\n",(u32)x_opts->headingControl);
		printk("x_opts.rxOvErrDetect=%x\n",(u32)x_opts->rxOvErrDetect);
		printk("x_opts.rxUndErrDetect=%x\n",(u32)x_opts->rxUndErrDetect);
		printk("x_opts.txOvErrDetect=%x\n",(u32)x_opts->txOvErrDetect);
		printk("x_opts.txUndErrDetect=%x\n",(u32)x_opts->txUndErrDetect);

		printk("x_opts.AbortErrDetect=%x\n",(u32)x_opts->AbortErrDetect);
		printk("x_opts.echoMode=%x\n",(u32)x_opts->echoMode);
		printk("x_opts.idleValue=%x\n", (u32)x_opts->idleValue);
		printk("x_opts.gpoCs=%x\n",(u32)x_opts->gpoCs);
		printk("x_opts.gpoInv=%x\n\n",(u32)x_opts->gpoInv);
}


spi_transaction_data_t spi_trans_array[5];

	
void quantum_frame_end_int(int port)
{
	struct ifx_ssc_frm_opts frm_opts;
	static int count=0;
	int i_ret=0;


//	spi_ss_clear(SS_MM_LED);
//	return;
	if ((spi_trans_array[count].chip_select != SS_NONE) && (spi_trans_array[count].length == 0)) {
		spi_ss_clear(spi_trans_array[count].chip_select);
		spi_trans_array[count].chip_select = SS_NONE;
	}
	
	if (spi_trans_array[count].length != 0 ) {
		if((i_ret = ifx_ssc_ioctl(MINI_HSI_SSC_INODE, NULL,IFX_SSC_FRM_CONTROL_GET,(unsigned long)&frm_opts)))
		{
			printk("ifx_ssc_ioctl IFX_SSC_FRM_CONTROL_GET failed Ret = %d \n",i_ret);
			return ;
		}

		frm_opts.DataLength = (spi_trans_array[count].length*8);	
		frm_opts.PauseLength = 1;					
		frm_opts.FrameEnable = 1;
		frm_opts.IntAfter = 1;
		frm_opts.IdleClock = 1;
		
		if((i_ret = ifx_ssc_ioctl(MINI_HSI_SSC_INODE,NULL,IFX_SSC_FRM_CONTROL_SET,(unsigned long)&frm_opts)))
		{
			printk("ifx_ssc_ioctl IFX_SSC_FRM_CONTROL_SET failed Ret = %d \n",i_ret);
			return ;
		}		

		quantum_write_spi(spi_trans_array[count].txdata,spi_trans_array[count].length);
		quantum_read_spi(spi_trans_array[count].rxdata,spi_trans_array[count].length);
		spi_trans_array[count].length = 0;
	}

	return;
}


int mini_hsi_ssc_init(void)
{
	int i_ret=0;

	struct ifx_ssc_hwopts x_opts;
	struct ifx_ssc_frm_opts frm_opts;
	unsigned long ul_baud ;
	unsigned long ul_mode ;

	/* Initialize SSC1 Interface *******************************************/
	if((i_ret = ifx_ssc_open(MINI_HSI_SSC_INODE,NULL)))
	{
		printk("mini hsi ssc_open %d failed Ret = %d \n",(u32)MINI_HSI_SSC_INODE,i_ret);
		return i_ret;
	}


	ul_baud = 32000;

	if((i_ret = ifx_ssc_ioctl(MINI_HSI_SSC_INODE, NULL,IFX_SSC_BAUD_SET,(unsigned long)&ul_baud)))
	{
		printk("ifx_ssc_ioctl SSC_SET_BAUD failed Ret = %d \n",i_ret);
		return i_ret;
	}
#ifdef QW_DEBUG_SSC
	else
	{
		printk("Success ifx_ssc_ioctl SSC_SET_BAUD Ret = %d \n",i_ret);
	}
#endif

	if((i_ret = ifx_ssc_ioctl(MINI_HSI_SSC_INODE, NULL,IFX_SSC_HWOPTS_GET,(unsigned long)&x_opts)))
	{
		printk("ifx_ssc_ioctl IFX_SSC_HWOPTS_GET failed Ret = %d \n",i_ret);
		return i_ret;
	}
#ifdef QW_DEBUG_SSC
	else
	{
		printk("Success ifx_ssc_ioctl IFX_SSC_HWOPTS_GET Ret = %d \n",i_ret);
		ssc_show_hwopts(&x_opts);
	}
#endif

	x_opts.modeRxTx		  = IFX_SSC_MODE_RXTX;			/* full duplex */
	x_opts.masterSelect   = IFX_SSC_MASTER_MODE;		/* we are master */
	x_opts.dataWidth      = MINI_HSI_DATAWIDTH;			/* use 8 bit data */
	x_opts.loopBack       = 0;							/* loop back off */
	x_opts.clockPolarity  = 1;							/* idle clock high */
	x_opts.clockPhase     = 0;	/* tx on leading edge */
	x_opts.headingControl = IFX_SSC_MSB_FIRST;			/*MSB first */

	x_opts.rxOvErrDetect  = 0;
	x_opts.rxUndErrDetect = 0;
	x_opts.txOvErrDetect  = 0;
	x_opts.txUndErrDetect = 0;

	x_opts.AbortErrDetect = 0;
	x_opts.echoMode = 0;
	x_opts.idleValue = 1;
	x_opts.gpoCs = IFX_SSC_DEF_GPO_CS;
	x_opts.gpoInv = IFX_SSC_DEF_GPO_INV;


#ifdef QW_DEBUG_SSC
	ssc_show_hwopts(&x_opts);
#endif

	if((i_ret = ifx_ssc_ioctl(MINI_HSI_SSC_INODE,NULL,IFX_SSC_HWOPTS_SET,(unsigned long)&x_opts)))
	{
		printk("ifx_ssc_ioctl IFX_SSC_HWOPTS_SET failed Ret = %d \n",i_ret);
		return i_ret;
	}
#ifdef QW_DEBUG_SSC
	else
	{
		printk("Success ifx_ssc_ioctl IFX_SSC_HWOPTS_SET Ret = %d \n",i_ret);
	}
#endif

	/* clear the statistics */
	ul_mode = 0;
	return OK;
}


/**
 *	kernel interface to write to SPI
 *\param buffer pointer to outgoing data
 *\param len size of outgoing data buffer
 *\returns length of data written to SPI
*/
int quantum_write_spi(void *buffer, int len )
{
	int i_ret;
	char *p = kmalloc(len+ 3, GFP_KERNEL);
	memcpy(p,buffer,len);
	if((i_ret = ifx_ssc_kwrite(MINI_HSI_SSC_NUM,p,len)) != len)
	{
		printk("Failed incaip_ssc_kernel_write len = %d Ret = %d \n",len,i_ret);
	}
	return( i_ret );
}

/**
 *	kernel interface to read from SPI
 *\param buffer pointer to incoming data buffer
 *\param len of incoming data buffer
 *\returns length of data read from SPI
*/
int quantum_read_spi(void *buffer, int len )
{
	int i_ret;
	char *p = kmalloc(len+ 3, GFP_KERNEL);

	if((i_ret = ifx_ssc_kread(MINI_HSI_SSC_NUM,p,len)) != len)
	{
		printk("Failed incaip_ssc_kernel_read len = %d Ret = %d \n",len,i_ret);
		return( i_ret );
	}
	

	memcpy(buffer,p,len);
	kfree(p);
	return( i_ret );
}


/**
* Kernel initialisation function run when the module is installed via insmod
*/
static int __init quantum_mux_init(void)
{
	mini_hsi_ssc_init();


	return( 0 );
}

/**
* Kernel clean-up function run when the module is removed via rmmod
*/
static void __exit quantum_mux_exit(void)
{
	int i_ret;
	if((i_ret = ifx_ssc_close(MINI_HSI_SSC_INODE,NULL)))
	{
		printk(KERN_ERR "quantum_mux_exit: ssc_close failed Ret = %d \n",i_ret);
	}
}



module_init(quantum_mux_init);
module_exit(quantum_mux_exit);

EXPORT_SYMBOL(quantum_read_spi);
EXPORT_SYMBOL(quantum_write_spi);
EXPORT_SYMBOL(quantum_frame_end_int);
EXPORT_SYMBOL(spi_trans_array);



/**
* Macro to tell the kernel that the module bears a free license (stops the kernel complaining!)
*/
MODULE_LICENSE("GPL");
