/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#ifndef FPGA_H
#define FPGA_H

typedef enum {
	SS_NONE         = 0,
	SS_DM_LED       = 1,
	SS_DM_KEY       = 2,
	SS_DM_TFT       = 3,
	SS_MM_LED       = 4,
	SS_MM_KEY       = 5,
	SS_MM_SLIDER    = 6,
	SS_MM_WHEEL     = 7
} spi_ss_t;

typedef enum {
	FPGA_HOOKSWITCH_INT,
	FPGA_PS2_INT,
	FPGA_DHSG_INT,
	FPGA_SPARE_3_INT,
	FPGA_V24_INT,
	FPGA_DRDY_INT,
	FPGA_MM_KEY_DRDY_INT,
	FPGA_SPARE7_INT,
	FPGA_LAN_INT,
	FPGA_SPARE_9_INT,
	FPGA_DISPLAY_INT,
	FPGA_MM_KEY_INT,
	FPGA_DM_KEY_INT,
	FPGA_MIDI_INT,
	FPGA_USB_INT,
	FPGA_SPARE_15_INT,
} eFpgaInterruptNum;

typedef int (*fpgaIntHander)(eFpgaInterruptNum vector, void *data);

typedef enum { SIDECAR_POWER_OFF, SIDECAR_POWER_ON } eSidecarPower;

int spi_ss_clear( spi_ss_t device );
int spi_ss_set( spi_ss_t device );
int spi_ss_sync( spi_ss_t device );

int spi_drdy( spi_ss_t device );

int request_fpga_irq( eFpgaInterruptNum number, fpgaIntHander hander, void *data );
int free_fpga_irq( eFpgaInterruptNum number );
int enable_fpga_irq( eFpgaInterruptNum number );
int disable_fpga_irq( eFpgaInterruptNum number );

void set_chassis_spk_amp_on(void);
void set_chassis_spk_amp_off(void);
int is_chassis_spk_amp_on(void);

int spi_ss_sidecar_set(void);
int spi_ss_sidecar_clear(void);
void sidecar_power( eSidecarPower state );

#endif
