/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/ioport.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/mux.h>
#include <asm/incaip2/irq.h>
//#include "fpga_defs.h"
#include "fpga.h"
#include "latch.h"
#include <asm/types.h>
#include <linux/version.h>



/* General Port bit mask declarations */
#define WP2_DRDY_KEY   0x200000  /* display data ready key bit */


static void writeLatch (unsigned char yMask, unsigned char ySetReset);
int get_WP_version(void);
static int wp_version = -1;

int fpgaDefaultIntHander( eFpgaInterruptNum vector, void *data )
{
	return( 0 );
}

void initFpgaIntDescriptors( void )
{
}

int request_fpga_irq( eFpgaInterruptNum number, fpgaIntHander hander, void *data )
{

	return( 1 );
}

int free_fpga_irq( eFpgaInterruptNum number )
{

	return( 1 );
}

int enable_fpga_irq( eFpgaInterruptNum vector )
{
	return( 1 );
}

int disable_fpga_irq( eFpgaInterruptNum vector )
{
	return( 1 );
}


void fpga_show_interrupt_pending(void)
{

}

//u8 fpga_reg_read( int reg )
//{
//	return( 1);
//}

//int fpga_reg_write( int reg, u8 value )
//{
//	return( 0 );
//}

void fpgaIsrHandle( void )
{
}

//void fpga_isr(int irq, void *dev_id, struct pt_regs *regs)
//{
//}
int fpga_isr_init(void)
{
	return( 1 );
}

int fpga_isr_release(void)
{
	return( 1   );
}

int spi_ss_set( spi_ss_t device )
{
	if( device == SS_DM_KEY )
	{
		if ( ((*INCA_IP2_GPIO_PIN) & WP2_DRDY_KEY) )
		{
			writeLatch (CSQ_KEY, LATCH_SET);	//Chip select enabled (active low)
			return( 1 );
		}
	}
	if( device == SS_DM_LED )
	{
		writeLatch (CSQ_LED, LATCH_SET);	//Chip select enabled (active low)
		return( 1 );
	}
	return( 0 );
}

int spi_ss_clear( spi_ss_t device )
{
	if( device == SS_DM_KEY )
	{
		writeLatch (CSQ_KEY, LATCH_CLEAR);	//Chip select disabled (active low)
		return( 1 );
	}
	if( device == SS_DM_LED )
	{
		writeLatch (CSQ_LED, LATCH_CLEAR);	//Chip select disabled (active low)
		return( 1 );
	}
	return( 0 );
}

int spi_ss_sync( spi_ss_t device )
{
	return( 0 );
}

int spi_drdy( spi_ss_t device )
{
	return (0);
}

void set_chassis_spk_amp_on(void)
{
	*INCA_IP2_GPIO_POUT |= (1<<2);
	return;
}

void set_chassis_spk_amp_off(void)
{
	*INCA_IP2_GPIO_POUT &= ~(1<<2);
	return;
}

int is_chassis_spk_amp_on(void)
{
	return((*INCA_IP2_GPIO_POUT) & (1<<2));
}

int spi_ss_sidecar_clear()
{
    // this function differs for OS15, see fpga_init for details
    if (wp_version == 4)
    {
        (*(volatile unsigned int*)INCA_IP2_GPIO_POUT) |= (1<<18);
    }
    else
    {
   	writeLatch (SIDECAR_SPI_SEL, LATCH_CLEAR);
    }

    return 0;
}

int spi_ss_sidecar_set()
{
    // this function differs for OS15, see fpga_init for details
    if (wp_version == 4)
    { 
        (*(volatile unsigned int*)INCA_IP2_GPIO_POUT) &= ~(1<<18);
    }
    else
    {
        writeLatch (SIDECAR_SPI_SEL, LATCH_SET);
    }

    return 0;
}


void sidecar_power( eSidecarPower state )
{
    // this function differs for OS15, see fpga_init for details
    if (wp_version == 4)
    {
        switch( state )
        {
            case SIDECAR_POWER_OFF:
                {
                    (*(volatile unsigned int*)INCA_IP2_GPIO_POUT) |= (1<<17); break;
                }
            case SIDECAR_POWER_ON:
                {
                    (*(volatile unsigned int*)INCA_IP2_GPIO_POUT) &= ~(1<<17); break;
                }
            default: break;
        }
    }
    else
    {
	switch( state )
	{
		 case SIDECAR_POWER_OFF: writeLatch (SIDECAR_POWER_EN, LATCH_CLEAR); break;
		 case SIDECAR_POWER_ON: writeLatch (SIDECAR_POWER_EN, LATCH_SET); break;
		 default: break;
	}
    }
}

static unsigned char yLatchValue ;		// holds the content of the LATCH device
/**
*@param 	yMask:      masks the respective bits to be modifies
*@param   	ySetReset:  0:  masked bit(s) will be reset (0)
*           		    1:  masked bit(s) will be set (1)
*/
static void writeLatch (unsigned char yMask, unsigned char ySetReset)
{
	unsigned long flags;
	spinlock_t lock;

	spin_lock_init(&lock);

	spin_lock_irqsave(lock,flags);

    if (ySetReset)
    {
        yLatchValue |= yMask ;
    }
    else
    {
        yLatchValue &= ~yMask ;
    }

    *CSLATCH = yLatchValue ;
	spin_unlock_irqrestore(lock,flags);
}

void ST7529_RESET(void)
{
	if( wp_version == 5 ){
		// The Cost reduced Openstage 40 display reset line is on the Latch not GPIO
		writeLatch (RESET_LCD_OS40core, 0); udelay(1000);
		writeLatch (RESET_LCD_OS40core, 1); udelay(1000);
	}
	else {
		(*(volatile unsigned int*)INCA_IP2_GPIO_POUT) &= ~0x01; udelay(1000);
		(*(volatile unsigned int*)INCA_IP2_GPIO_POUT) |=  0x01; udelay(1000);
	}
}


static int __init fpga_init(void)
{
    wp_version = ( (*INCA_IP2_GPIO_PIN >> 12) & 0x0000000F );
	writeLatch (LATCH_INIT, LATCH_CLEAR);
    spi_ss_sidecar_clear();
    set_chassis_spk_amp_off();
	return (1);
}

static void __exit fpga_exit(void)
{
}

module_init(fpga_init);
module_exit(fpga_exit);

//EXPORT_SYMBOL(fpga_reg_read);
//EXPORT_SYMBOL(fpga_reg_write);
EXPORT_SYMBOL(spi_ss_set);
EXPORT_SYMBOL(spi_ss_clear);
EXPORT_SYMBOL(spi_ss_sync);
EXPORT_SYMBOL(spi_drdy);
EXPORT_SYMBOL(set_chassis_spk_amp_on);
EXPORT_SYMBOL(set_chassis_spk_amp_off);
EXPORT_SYMBOL(is_chassis_spk_amp_on);
EXPORT_SYMBOL(request_fpga_irq);
EXPORT_SYMBOL(free_fpga_irq);
EXPORT_SYMBOL(enable_fpga_irq);
EXPORT_SYMBOL(disable_fpga_irq);
EXPORT_SYMBOL(spi_ss_sidecar_set);
EXPORT_SYMBOL(spi_ss_sidecar_clear);
EXPORT_SYMBOL(sidecar_power);
EXPORT_SYMBOL(ST7529_RESET);
