/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#ifndef KEYINPUT_H
#define KEYINPUT_H

// Key definitions used by the kernel
// These are the actual key codes generated by the hardware (or by the
// infineon software).

#define KERNEL_NULL_KEY			0xff	// Null value only - NOT a valid key

#define KERNEL_KEYPAD_0			11
#define KERNEL_KEYPAD_1			21
#define KERNEL_KEYPAD_2			17
#define KERNEL_KEYPAD_3			4
#define KERNEL_KEYPAD_4			16
#define KERNEL_KEYPAD_5			14
#define KERNEL_KEYPAD_6			9
#define KERNEL_KEYPAD_7			10
#define KERNEL_KEYPAD_8			20
#define KERNEL_KEYPAD_9			3
#define KERNEL_KEYPAD_AST		18
#define KERNEL_KEYPAD_HASH		12
#define KERNEL_NAVI_KEY_NORTH	8
#define KERNEL_NAVI_KEY_SOUTH	1
#define KERNEL_NAVI_KEY_WEST	7
#define KERNEL_NAVI_KEY_EAST	2
#define KERNEL_NAVI_KEY_CENTRAL	0

#define KERNEL_CONTROL_KEY_0	23
#define KERNEL_CONTROL_KEY_1	13
#define KERNEL_CONTROL_KEY_2	19
#define KERNEL_CONTROL_KEY_3	6
#define KERNEL_CONTROL_KEY_4	15
#define KERNEL_CONTROL_KEY_5	5

// Extras needed for WP_LO
#define KERNEL_CONTROL_KEY_6    22 
#define KERNEL_CONTROL_KEY_7    24   
#define KERNEL_CONTROL_KEY_8    30 

// Key definitions for hooded claw key combinations.
// These are made up values for the key combinations
// as only the individual key codes are generated
// by the hardware.
#define KERNEL_HOODED_CLAW_147		25
#define KERNEL_HOODED_CLAW_35HASH	26
#define KERNEL_HOODED_CLAW_289		27
#define KERNEL_HOODED_CLAW_013		28

#define KERNEL_LANSTATUS_KEY		29

//Values for key ranges
#define KERNEL_MODE_KEY			0x30
#define KERNEL_MODE_KEY_RANGE	0x0a

#define KERNEL_FPK_KEY			0x40
#define KERNEL_FPK_KEY_RANGE	0x0d

#define KERNEL_HEADSET_HOOKSW	0x4d
#define KERNEL_HEADSET_KEY		0x4e
#define KERNEL_WHEEL_KEY		0x4f

#define KERNEL_VOLUME_KEY		0x50
#define KERNEL_VOLUME_KEY_RANGE	0x10
#define KERNEL_SLIDER_KEY		0x70


// Selected definitions for messages passed by the kernel
// Note: these are duplicated in GPAL module
// /vobs/Opera_Platform_Linux/GPAL/Devices/src/keypadDevice.cpp.
// Any changes to kernel key definitions made here must be
// duplicated in the above module (and vice versa).

// Key states
#define KEY_RELEASED 	0
#define KEY_PRESSED		1

// Alternative version of key_event_t structure defined in tsf.h,
// with keycode of 32 bits (instead of just 8 bits).
typedef struct {
    unsigned long key_code;   /* Key code of current key */
    int           key_state;  /* Key state [KEY_PRESSED, KEY_RELEASED] */
    unsigned long time_stamp; /* Value of jiffies at time of event */
} key_event_t_ul;

// End of duplicated kernel key definitions


// Key definitions passed by the GPAL keypad device
// Note: these are duplicated in GPAL header file
// /vobs/Opera_Platform_Linux/Interfaces/physicalKeyDefs.h.
// Any changes to physical key definitions made here must be
// duplicated in the above header file (and vice versa).

enum physicalKeyCode {

/* 0-255 reserved for extended ASCII */

/* Key pad keys */
PHYSICAL_KEYPAD_0 = 0x100, /* ZERO KEY */
PHYSICAL_KEYPAD_1,
PHYSICAL_KEYPAD_2,
PHYSICAL_KEYPAD_3,
PHYSICAL_KEYPAD_4,
PHYSICAL_KEYPAD_5,
PHYSICAL_KEYPAD_6,
PHYSICAL_KEYPAD_7,
PHYSICAL_KEYPAD_8,
PHYSICAL_KEYPAD_9,
PHYSICAL_KEYPAD_AST,
PHYSICAL_KEYPAD_HASH,
/* SPARE KEYPAD keys */

/* Mode keys */
PHYSICAL_MODE_KEY_0 = 0x114, /*e.g. CALLVIEW 0x100 + 20d*/
PHYSICAL_MODE_KEY_1,
PHYSICAL_MODE_KEY_2,
PHYSICAL_MODE_KEY_3,
PHYSICAL_MODE_KEY_4,
PHYSICAL_MODE_KEY_5,
/* SPARE MODE KEYS */

/* Control keys */
PHYSICAL_CONTROL_KEY_0 = 0x11E, /*e.g. RELEASE 0x100 + 30d*/
PHYSICAL_CONTROL_KEY_1,
PHYSICAL_CONTROL_KEY_2,
PHYSICAL_CONTROL_KEY_3,
PHYSICAL_CONTROL_KEY_4,
PHYSICAL_CONTROL_KEY_5,
PHYSICAL_CONTROL_KEY_6, /* e.g. "-" key on WP1 */
PHYSICAL_CONTROL_KEY_7, /* e.g. "+" key on WP1 */
PHYSICAL_CONTROL_KEY_8, /* e.g. redial */
/* SPARE CONTROL KEYS */

/* Navigation keys */
PHYSICAL_NAVI_KEY_NORTH = 0x128, /*e.g. Clickwheel UP 0x100 + 40d */
PHYSICAL_NAVI_KEY_SOUTH,
PHYSICAL_NAVI_KEY_WEST,
PHYSICAL_NAVI_KEY_EAST,
PHYSICAL_NAVI_KEY_CENTRAL,
PHYSICAL_HOOKSWITCH_KEY,	/* Hook switch event */
PHYSICAL_NAVI_TOUCH,		/* The event of pressing the wheel, but with no movement, includes release */

/* Special keys (Needs reasseing!) */
PHYSICAL_AUDIO_MENU = 0x13C,  /* Taken from CORNET TS i.e. "+" & "-" keys together -> 60d */
PHYSICAL_TA_A_B,			  /* Taken from CORNET TS i.e. ? */
PHYSICAL_FLASH_EVENT,		  /* Taken from CORNET TS i.e. ? */
PHYSICAL_HEADSET_HOOK,		  /* Taken from CORNET TS i.e. From wireless headset	 */
PHYSICAL_HOODED_CLAW,		  /* e.g. Admin menu support e.g. "1","3","0" on 410 */
PHYSICAL_PHONE_CALL_ALERT,    /* Dummy key used for LED control */
PHYSICAL_HEADSET_KEY,                  /* Press indicates headset is present, Release is not present */
PHYSICAL_SOCKETED_HEADSET_KEY,         /* Press indicates a socketed headset plugged in, Release is pulled out */
PHYSICAL_BLUETOOTH_HEADSET_KEY,         /* Press indicates bluetooth headset is present, is not present */
PHYSICAL_BLUETOOTH_CONFUNIT_KEY,       /* Press indicates bluetooth conference unit is present, is not present */
PHYSICAL_WHEEL_EVENT,         /* Used to indicate a clickwheel scroll event */
PHYSICAL_HOODED_CLAW_147,	  /* Simultaneous 3 key event allowed by hardware */
PHYSICAL_HOODED_CLAW_35HASH,  /* Simultaneous 3 key event allowed by hardware */
PHYSICAL_HOODED_CLAW_289,	  /* Simultaneous 3 key event allowed by hardware */
PHYSICAL_HOODED_CLAW_013,	  /* Simultaneous 3 key event allowed by hardware */
PHYSICAL_LANSTATUS_KEY,		  /* Press indicates LAN link up, Release is LAN link down */
PHYSICAL_CONFUNIT_HOOK,

/* Phone FPK keys */
PHYSICAL_PHONE_FPK_0 = 0x164, /*  0x100 + 100d*/
PHYSICAL_PHONE_FPK_1,
PHYSICAL_PHONE_FPK_2,
PHYSICAL_PHONE_FPK_3,
PHYSICAL_PHONE_FPK_4,
PHYSICAL_PHONE_FPK_5,
PHYSICAL_PHONE_FPK_6,
PHYSICAL_PHONE_FPK_7,
PHYSICAL_PHONE_FPK_8,
/* SPARE PHONE_FPKs */

/* Sidecar FPK keys */
PHYSICAL_SIDECAR_1_FPK_0 = 0x22C, /* 0x100 + 300d */
PHYSICAL_SIDECAR_1_FPK_1,
PHYSICAL_SIDECAR_1_FPK_2,
PHYSICAL_SIDECAR_1_FPK_3,
PHYSICAL_SIDECAR_1_FPK_4,
PHYSICAL_SIDECAR_1_FPK_5,
PHYSICAL_SIDECAR_1_FPK_6,
PHYSICAL_SIDECAR_1_FPK_7,
PHYSICAL_SIDECAR_1_FPK_8,
PHYSICAL_SIDECAR_1_FPK_9,
PHYSICAL_SIDECAR_1_FPK_10,
PHYSICAL_SIDECAR_1_FPK_11,
PHYSICAL_SIDECAR_1_FPK_12,
PHYSICAL_SIDECAR_1_FPK_13,
PHYSICAL_SIDECAR_1_FPK_14,
PHYSICAL_SIDECAR_1_FPK_15,
PHYSICAL_SIDECAR_1_FPK_16,
PHYSICAL_SIDECAR_1_FPK_17,
PHYSICAL_SIDECAR_1_FPK_18,

PHYSICAL_SIDECAR_2_FPK_0 = 0x2F4, /* 0x100 + 500d */
PHYSICAL_SIDECAR_2_FPK_1,
PHYSICAL_SIDECAR_2_FPK_2,
PHYSICAL_SIDECAR_2_FPK_3,
PHYSICAL_SIDECAR_2_FPK_4,
PHYSICAL_SIDECAR_2_FPK_5,
PHYSICAL_SIDECAR_2_FPK_6,
PHYSICAL_SIDECAR_2_FPK_7,
PHYSICAL_SIDECAR_2_FPK_8,
PHYSICAL_SIDECAR_2_FPK_9,
PHYSICAL_SIDECAR_2_FPK_10,
PHYSICAL_SIDECAR_2_FPK_11,
PHYSICAL_SIDECAR_2_FPK_12,
PHYSICAL_SIDECAR_2_FPK_13,
PHYSICAL_SIDECAR_2_FPK_14,
PHYSICAL_SIDECAR_2_FPK_15,
PHYSICAL_SIDECAR_2_FPK_16,
PHYSICAL_SIDECAR_2_FPK_17,
PHYSICAL_SIDECAR_2_FPK_18,

PHYSICAL_BLF_FPK_0 = 0x4BC, /* 0x100 + 700d */
PHYSICAL_BLF_FPK_1,
PHYSICAL_BLF_FPK_2,
PHYSICAL_BLF_FPK_3,
PHYSICAL_BLF_FPK_4,
PHYSICAL_BLF_FPK_5,
PHYSICAL_BLF_FPK_6,
PHYSICAL_BLF_FPK_7,
PHYSICAL_BLF_FPK_8,
PHYSICAL_BLF_FPK_9,
PHYSICAL_BLF_FPK_10,
PHYSICAL_BLF_FPK_11,
PHYSICAL_BLF_FPK_12,
PHYSICAL_BLF_FPK_13,
PHYSICAL_BLF_FPK_14,
PHYSICAL_BLF_FPK_15,
PHYSICAL_BLF_FPK_16,
PHYSICAL_BLF_FPK_17,
PHYSICAL_BLF_FPK_18,
PHYSICAL_BLF_FPK_19,
PHYSICAL_BLF_FPK_20,
PHYSICAL_BLF_FPK_21,
PHYSICAL_BLF_FPK_22,
PHYSICAL_BLF_FPK_23,
PHYSICAL_BLF_FPK_24,
PHYSICAL_BLF_FPK_25,
PHYSICAL_BLF_FPK_26,
PHYSICAL_BLF_FPK_27,
PHYSICAL_BLF_FPK_28,
PHYSICAL_BLF_FPK_29,
PHYSICAL_BLF_FPK_30,
PHYSICAL_BLF_FPK_31,
PHYSICAL_BLF_FPK_32,
PHYSICAL_BLF_FPK_33,
PHYSICAL_BLF_FPK_34,
PHYSICAL_BLF_FPK_35,
PHYSICAL_BLF_FPK_36,
PHYSICAL_BLF_FPK_37,
PHYSICAL_BLF_FPK_38,
PHYSICAL_BLF_FPK_39,
PHYSICAL_BLF_FPK_40,
PHYSICAL_BLF_FPK_41,
PHYSICAL_BLF_FPK_42,
PHYSICAL_BLF_FPK_43,
PHYSICAL_BLF_FPK_44,
PHYSICAL_BLF_FPK_45,
PHYSICAL_BLF_FPK_46,
PHYSICAL_BLF_FPK_47,
PHYSICAL_BLF_FPK_48,
PHYSICAL_BLF_FPK_49,
PHYSICAL_BLF_FPK_50,
PHYSICAL_BLF_FPK_51,
PHYSICAL_BLF_FPK_52,
PHYSICAL_BLF_FPK_53,
PHYSICAL_BLF_FPK_54,
PHYSICAL_BLF_FPK_55,
PHYSICAL_BLF_FPK_56,
PHYSICAL_BLF_FPK_57,
PHYSICAL_BLF_FPK_58,
PHYSICAL_BLF_FPK_59,
PHYSICAL_BLF_FPK_60,
PHYSICAL_BLF_FPK_61,
PHYSICAL_BLF_FPK_62,
PHYSICAL_BLF_FPK_63,
PHYSICAL_BLF_FPK_64,
PHYSICAL_BLF_FPK_65,
PHYSICAL_BLF_FPK_66,
PHYSICAL_BLF_FPK_67,
PHYSICAL_BLF_FPK_68,
PHYSICAL_BLF_FPK_69,
PHYSICAL_BLF_FPK_70,
PHYSICAL_BLF_FPK_71,
PHYSICAL_BLF_FPK_72,
PHYSICAL_BLF_FPK_73,
PHYSICAL_BLF_FPK_74,
PHYSICAL_BLF_FPK_75,
PHYSICAL_BLF_FPK_76,
PHYSICAL_BLF_FPK_77,
PHYSICAL_BLF_FPK_78,
PHYSICAL_BLF_FPK_79,
PHYSICAL_BLF_FPK_80,
PHYSICAL_BLF_FPK_81,
PHYSICAL_BLF_FPK_82,
PHYSICAL_BLF_FPK_83,
PHYSICAL_BLF_FPK_84,
PHYSICAL_BLF_FPK_85,
PHYSICAL_BLF_FPK_86,
PHYSICAL_BLF_FPK_87,
PHYSICAL_BLF_FPK_88,
PHYSICAL_BLF_FPK_89,

PHYSICAL_SLIDER_TOUCH = 0x600,	/* Volume slider discreet values */
PHYSICAL_SLIDER_POSITION_0,
PHYSICAL_SLIDER_POSITION_1,
PHYSICAL_SLIDER_POSITION_2,
PHYSICAL_SLIDER_POSITION_3,
PHYSICAL_SLIDER_POSITION_4,
PHYSICAL_SLIDER_POSITION_5,
PHYSICAL_SLIDER_POSITION_6,
PHYSICAL_SLIDER_POSITION_7,
PHYSICAL_SLIDER_POSITION_8,
PHYSICAL_SLIDER_POSITION_9,
PHYSICAL_SLIDER_POSITION_10,
PHYSICAL_SLIDER_POSITION_11,
PHYSICAL_SLIDER_POSITION_12,
PHYSICAL_SLIDER_POSITION_13,
PHYSICAL_SLIDER_POSITION_14,
PHYSICAL_SLIDER_POSITION_15

};

// End of duplicated physical key definitions


// Alternative version of keypad_callback_t structure defined in tsf.h,
// with key parameter of 32 bits (instead of just 8 bits).
typedef int (*keypad_callback_t_ul) (unsigned long key, int updown,
                                     unsigned long timestamp);

typedef struct {
	keypad_callback_t_ul keyInputEvent;
	char * name;
} keyRegInterface_t;

int keyInputRegister( keyRegInterface_t *keyThing );
int keyInputUnRegister( keyRegInterface_t *keyThing );

typedef int (*QWheel_callback_t) ( unsigned char value, unsigned long timestamp);
typedef int (*QWheelSensitivity_callback_t) ( int sensitivity );

typedef struct {
	QWheel_callback_t QWheelInputEvent;
	char * name;
} QWheelRegInterface_t;

typedef struct {
	QWheel_callback_t QWheelInputEvent;
	QWheelSensitivity_callback_t QWheelSetSensitivity;
	char * name;
} QWheelTestIfRegInterface_t;

int QWheelInputRegister( QWheelRegInterface_t *wheelThing );
int QWheelInputUnRegister( QWheelRegInterface_t *wheelThing );

int QWheelTestIfcRegister( QWheelTestIfRegInterface_t *wheelThing );
int QWheelTestIfcUnRegister( QWheelTestIfRegInterface_t *wheelThing );
#endif
