/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

#ifndef _HAPI_C_
#define _HAPI_C_


/*
macro to print some debug informations
*/
//#define _HAPI_DEBUG_IN_USERSPACE_
//#define _HAPI_DEBUG_IN_KERNEL_
//#define _HAPI_DEBUG_NO_NOTIFY_

#ifdef _HAPI_DEBUG_IN_KERNEL_
#define PDEBUG(fmt, args... ) printk(KERN_INFO fmt, ##args)
#endif

#ifdef _HAPI_DEBUG_IN_USERSPACE_
#define PDEBUG(fmt, args... ) printf(fmt, ##args)
#endif

#ifdef _HAPI_DEBUG_NO_NOTIFY_
#define PDEBUG(fmt, args... ) 
#endif

/*
Definition of bus states
*/
#define BUS_NOT_OPENNED		0
#define BUS_OPENNED		1

/*
Definition of sidecars states
*/
#define SIDECAR_NOT_INITIALIZED	0
#define SIDECAR_INITIALIZED	1
#define SIDECAR_RESET		2
#define SIDECAR_DISABLED	3

//sidecar should be present or missing
#define SIDECAR_PRESENT	0x0ff
#define SIDECAR_MISSING 0x0ee


/*
Masks for recognizing octets
*/
#define CONTROL_RECOGNITION_MASK 0x80
#define IS_CONTROL1(octet) (octet & CONTROL_RECOGNITION_MASK)
#define IS_CONTROL2_OR_DATA(octet) !(octet & CONTROL_RECOGNITION_MASK)

#define DATA_OR_CONTROL2_MASK 0x20
#define DATA_FOLLOWS(octet) (octet & DATA_OR_CONTROL2_MASK)
#define CONTROL2_FOLLOWS(octet) !(octet & DATA_OR_CONTROL2_MASK)

#define READ_PACING_MASK	0x40
#define READ_PACING(octet) (octet & READ_PACING_MASK)

#define POLL_MASK	0x8
#define POLL(octet) (octet & POLL_MASK)

#define DATA_INITIALIZE_MASK		0x10
#define IS_DATA_INITIALIZE_SET(octet) (octet & DATA_INITIALIZE_MASK)

#define REPEATER_TERMINATOR_MASK	0x20
#define IS_REP_TER_SET(octet) (octet & REPEATER_TERMINATOR_MASK)

#define IR_MASK		0x40

#define RESET_MASK	0x08
#define IS_RESET(octet) (octet & RESET_MASK)

/*
variable for distinguishing between data and control2
*/
#define DATA_OCTET	1
#define CONTROL2_OCTET	0

unsigned int next_octet = CONTROL2_OCTET;

/*
we need to distinguish if the sidecar is available
*/
unsigned int sidecar_0_available = SIDECAR_PRESENT;
unsigned int sidecar_1_available = SIDECAR_PRESENT;
unsigned int sidecar_2_available = SIDECAR_MISSING;
unsigned int sidecar_3_available = SIDECAR_MISSING;
/*
State varialbles of two sidecars
*/
unsigned int sidecar_0_status = SIDECAR_NOT_INITIALIZED;
unsigned int sidecar_1_status = SIDECAR_NOT_INITIALIZED;
unsigned int sidecar_2_status = SIDECAR_NOT_INITIALIZED;
unsigned int sidecar_3_status = SIDECAR_NOT_INITIALIZED;
/*
state variables for bus 
*/
unsigned int bus_status = BUS_NOT_OPENNED;

/*
state variables for each option
*/
#define SIDECAR_NOT_SELECTED	0
#define SIDECAR_SELECTED	1

unsigned int sidecar_0_selected = SIDECAR_NOT_SELECTED;
unsigned int sidecar_1_selected = SIDECAR_NOT_SELECTED;
unsigned int sidecar_2_selected = SIDECAR_NOT_SELECTED;
unsigned int sidecar_3_selected = SIDECAR_NOT_SELECTED;

/*
to store options address from control 1
*/
#define OPTION_ADDRESS_MASK	0x07
#define GET_ADDRESS(adr,data) adr = (data & OPTION_ADDRESS_MASK)
int addressed_option = -1;

/*
waiting message
*/
#define NO_MESSAGE_MASK	0x10
#define MESSAGE_WAITING_MASK 0x20
#define NO_MESSAGE	0
#define MESSAGE_WAITING 1
unsigned int sidecar_0_message = NO_MESSAGE;
unsigned int sidecar_1_message = NO_MESSAGE;
unsigned int sidecar_2_message = NO_MESSAGE;
unsigned int sidecar_3_message = NO_MESSAGE;


/*
ten times send nomessage, then send one of three predefined messages
*/
#define MESSAGE_INTERVAL	0
unsigned int message_interval = MESSAGE_INTERVAL;

/*message */
#define DSS_KEY_LEN	3
#define OPTION_INFO_IND_LEN	10
#define OPTION_INIT_REQ_LEN	11
unsigned char	dss_key[DSS_KEY_LEN+1] = {0x03,0x81,0x11,0x42};
unsigned char	option_info_ind[OPTION_INFO_IND_LEN+1] = {0x8a,0x81,0x01,0x04,0x10,0x12,0x14,0x16,0x18,0x20,0x22};
unsigned char	option_init_req[OPTION_INIT_REQ_LEN+1] = {0x8b,0x81,0x00,0x04,0x02,0x04,0x06,0x08,0x0a,0x0c,0x0d,0x01};
unsigned char *p_message;
unsigned int l_message = 0;
unsigned int message_pointer = 0;
unsigned int message_chooser = 0;

/*
remeber initialization bit
*/
unsigned int init_bit_set = 0;

/*
HAPISIM interface
These functions are implemented in this file. 
Only for testing functionality of sidecar kernel module.
*/

unsigned int SidecarHAPI_Open(void);
unsigned int SidecarHAPI_Close(void);
unsigned int SidecarHAPI_Flush(unsigned int mode);
unsigned int SidecarHAPI_Reset(void);
unsigned int SidecarHAPI_Read(unsigned int length, unsigned char * pRxData);
unsigned int SidecarHAPI_Write(unsigned int length, unsigned char * pTxData);
unsigned int SidecarHAPI_Fastmode_Write(unsigned int length, unsigned char * pTxData);

void PrintBuffer(void);
void ClearBuffer(void);

/*
output data buffer
To this buffer are stored bytes and if is called read function, values from this buffer are returned
*/
#define OUTPUT_BUFFER_SIZE 64
unsigned char output_buffer[OUTPUT_BUFFER_SIZE];
unsigned char write_pointer = 0;
unsigned char read_pointer = 0;



unsigned int SidecarHAPI_Open(void){

    PDEBUG("SidecarHAPI_Open function\n");
    bus_status = BUS_OPENNED;
    return 0;
}

unsigned int SidecarHAPI_Close(void){

    PDEBUG("SidecarHAPI_Close function\n");
    if(bus_status==BUS_OPENNED){
	bus_status = BUS_NOT_OPENNED;
    }
    else{
	PDEBUG("Sidecar bus error: closing of not openned bus\n");
	return 1;
    }
    return 0;
}


void ClearBuffer(void){
    int i;
    for(i=0;i<OUTPUT_BUFFER_SIZE;i++){
	output_buffer[i] = 0;
    }
    
}


unsigned int SidecarHAPI_Flush(unsigned int mode){

    PDEBUG("SidecarHAPI_Flush function, mode:%d\n",mode);
    if(bus_status==BUS_OPENNED){
	switch(mode){
	    case 0://read buffer
		ClearBuffer();
		write_pointer = 0;
	    break;
	    case 1://write buffer
	    break;
	    case 2://both buffers
		ClearBuffer();
		write_pointer = 0;
	    break;
	    default:break;
	}
    }
    else{
	PDEBUG("Sidecar bus error: flush of not openned bus\n");
	return 1;
    }
    return 0;
}

unsigned int SidecarHAPI_Reset(void){

    PDEBUG("SidecarHAPI_Reset function\n");
    if(bus_status==BUS_NOT_OPENNED){
	PDEBUG("Sidecar bus error: reset of not openned bus\n");
	return 1;
    }
    SidecarHAPI_Flush(2);
    //set sidecars to reset state
    //first sidecar is reset
    if(sidecar_0_available==SIDECAR_PRESENT){
        sidecar_0_status = SIDECAR_RESET;
    }
    if(sidecar_1_available==SIDECAR_PRESENT){
        sidecar_1_status = SIDECAR_DISABLED;
    }
    if(sidecar_2_available==SIDECAR_PRESENT){
        sidecar_2_status = SIDECAR_DISABLED;
    }
    if(sidecar_3_available==SIDECAR_PRESENT){
        sidecar_3_status = SIDECAR_DISABLED;
    }
    return 0;
}

unsigned int SidecarHAPI_Read(unsigned int length, unsigned char * pRxData){
    
    int i;

    PDEBUG("SidecarHAPI_Read function, length:%d, pRxData:%lu\n",length,(unsigned long)pRxData);
    if(bus_status==BUS_NOT_OPENNED){
	PDEBUG("Sidecar bus error: reset of not openned bus\n");
	return 1;
    }
    
    //two situations can occur
    //length > write_pointer .. this is bad
    //length <= write_pointer .. ok
    
    //read data
    for(i=0;i<length;i++){
	pRxData[i]=output_buffer[read_pointer+i];
    }
    
    //PDEBUG("A\n");
    //garbage collection
    for(i=0;i<((int)write_pointer-(int)length);i++){
//	PDEBUG("i:%d, write_pointer:%d, length:%d\n",i,write_pointer,length);
	output_buffer[i] = output_buffer[i+length];
    }
    //PDEBUG("B\n");
    
    //PDEBUG("C\n");
    if(((int)write_pointer-(int)length)<0){
	write_pointer = 0;
	for(i=0;i<OUTPUT_BUFFER_SIZE;i++){
	    output_buffer[i] = 0;
	}
    }
    else{
	for(i=write_pointer-length;i<write_pointer;i++){
	    output_buffer[i] = 0;
	}
        write_pointer = write_pointer - length;
    }
    return 0;
}

/*
This function writes bytes to the Tx buffer.
*/
unsigned int SidecarHAPI_Write(unsigned int length, unsigned char * pTxData){
    
    int i;
    unsigned int data;
    unsigned int written_data=0;
    
    PDEBUG("SidecarHAPI_Write function, length:%d, pTxData:%lu\n",length,(unsigned long)pTxData);
    

    if(bus_status==BUS_NOT_OPENNED){
	PDEBUG("Sidecar bus error: writing onto not openned bus\n");
	return 1;
    }

//test, print out written data
    for(i=0;i<length;i++){
	PDEBUG("%X ",pTxData[i]);
    }
    PDEBUG("\n");
//


    for(i=0;i<length;i++){
	data = pTxData[i];
/*
Concerning the written value this function has to store the according value to the output buffer.
*/
	    
	    if(IS_CONTROL1(data)){
		//PDEBUG("SidecarHAPI_Write function, control 1 written\n");
    		//what is in the next octet?
		//dw bit
		if(DATA_FOLLOWS(data)){
		    //PDEBUG("SidecarHAPI_Write function, data will follow\n");
		    next_octet = DATA_OCTET;
		}
		if(CONTROL2_FOLLOWS(data)){
		    //PDEBUG("SidecarHAPI_Write function, control2 will follow\n");
		    next_octet = CONTROL2_OCTET;
		}
		
		//PDEBUG("Pred\n");
		//read pacing bit recognition		
		if(READ_PACING(data)){
		
//		    PDEBUG("Sidecar selected 0:%d\n",sidecar_0_selected);
//		    PDEBUG("Sidecar selected 1:%d\n",sidecar_1_selected);
//		    PDEBUG("Sidecar selected 2:%d\n",sidecar_2_selected);
//		    PDEBUG("Sidecar selected 3:%d\n",sidecar_3_selected);
		    //if this is set, previously selected option sends its data
		    //ask all options
		    if(sidecar_0_selected==SIDECAR_SELECTED){
			//PDEBUG("selected 0\n");
			//PDEBUG("p=%d , l=%d\n",message_pointer,l_message);
			if(message_pointer==1){
			    written_data = 0;
			    written_data |= p_message[message_pointer++];
			    written_data |= 0<<4;
			    output_buffer[write_pointer++] = written_data;
			}
			else{
			    output_buffer[write_pointer++] = p_message[message_pointer++];
			}
			
			//PDEBUG("p=%d , l=%d\n",message_pointer,l_message);
			if(message_pointer==l_message){
			    //whole message copied to output buffer
			    //PDEBUG("p==l\n");
			    sidecar_0_selected=SIDECAR_NOT_SELECTED;
			    sidecar_0_message=NO_MESSAGE;
			    message_pointer = 0;
			}
			//sidecar_1_selected=SIDECAR_NOT_SELECTED;
			//sidecar_2_selected=SIDECAR_NOT_SELECTED;
			//sidecar_3_selected=SIDECAR_NOT_SELECTED;
		    }
		    //PDEBUG("mezi\n");
		    if(sidecar_1_selected==SIDECAR_SELECTED){
			//PDEBUG("selected 1\n");
			if(message_pointer==1){
			    written_data = 0;
			    written_data |= p_message[message_pointer++];
			    written_data |= 1<<4;
			    output_buffer[write_pointer++] = written_data;
			}
			else{
			    output_buffer[write_pointer++] = p_message[message_pointer++];
			}
			 
			if(message_pointer==l_message){
			    //whole message copied to output buffer
			    sidecar_1_selected=SIDECAR_NOT_SELECTED;
			    sidecar_1_message=NO_MESSAGE;
			    message_pointer = 0;
			}
		    }
		    if(sidecar_2_selected==SIDECAR_SELECTED){
			//PDEBUG("selected 2\n");
			if(message_pointer==1){
			    written_data = 0;
			    written_data |= p_message[message_pointer++];
			    written_data |= 2<<4;
			    output_buffer[write_pointer++] = written_data;
			}
			else{
			    output_buffer[write_pointer++] = p_message[message_pointer++];
			}
			 
			if(message_pointer==l_message){
			    //whole message copied to output buffer
			    sidecar_2_selected=SIDECAR_NOT_SELECTED;
			    sidecar_2_message=NO_MESSAGE;
			    message_pointer = 0;
			}
		    }
		    if(sidecar_3_selected==SIDECAR_SELECTED){
			//PDEBUG("selected 3\n");
			
			if(message_pointer==1){
			    written_data = 0;
			    written_data |= p_message[message_pointer++];
			    written_data |= 3<<4;
			    output_buffer[write_pointer++] = written_data;
			}
			else{
			    output_buffer[write_pointer++] = p_message[message_pointer++];
			}
			
			output_buffer[write_pointer++] = p_message[message_pointer++]; 
			if(message_pointer==l_message){
			    //whole message copied to output buffer
			    sidecar_3_selected=SIDECAR_NOT_SELECTED;
			    sidecar_3_message=NO_MESSAGE;
			    message_pointer = 0;
			}
		    }
		}
		else{//if no read pacing and some option selected, interupt the transmission of message
		    //deselect the sidecar
		    //PDEBUG("No read pacing\n");
		    sidecar_0_selected=SIDECAR_NOT_SELECTED;
		    sidecar_1_selected=SIDECAR_NOT_SELECTED;
		    sidecar_2_selected=SIDECAR_NOT_SELECTED;
		    sidecar_3_selected=SIDECAR_NOT_SELECTED;
		    message_pointer = 0;
		    sidecar_0_message=NO_MESSAGE;
		    sidecar_1_message=NO_MESSAGE;
		    sidecar_2_message=NO_MESSAGE;
		    sidecar_3_message=NO_MESSAGE;
		}
		
		
		//get address from the lowest three bits
		GET_ADDRESS(addressed_option,data);
		//PDEBUG("SidecarHAPI_Write function, get_address:%u\n",addressed_option);
				
		//initialization
		//if active option receives this control, it is an error
		//have to disable next options and reset itself
		
		//write response to the output buffer
		//create written data
		//PDEBUG("%X\n",data);
		if((IS_DATA_INITIALIZE_SET(data))&&(!READ_PACING(data))&&(CONTROL2_FOLLOWS(data))&&(!POLL(data))){
		switch(addressed_option){
		    case 0:
//			PDEBUG("AA\n");
			if(sidecar_0_available==SIDECAR_PRESENT){
//			    PDEBUG("BB\n");
			    if(sidecar_0_status==SIDECAR_RESET){
//				PDEBUG("CC\n");
				//sidecar is initialized return desired value
				sidecar_0_status = SIDECAR_INITIALIZED;
				written_data = 0;
				written_data |= IR_MASK;
				written_data |= addressed_option;
				output_buffer[write_pointer++] = written_data;
				//PDEBUG("SidecarHAPI_Write function, addressed option 0\n");
				
				//next sidecar to reset state
				if(sidecar_1_available==SIDECAR_PRESENT) sidecar_1_status = SIDECAR_RESET;
			    }
			    else{
				//initialization of sidecar not in reset state
				if(sidecar_0_status==SIDECAR_INITIALIZED){
				    //set sidecar to reset state
				    sidecar_0_status = SIDECAR_RESET;
				    //next sidecars seems to be missing
				    if(sidecar_1_available==SIDECAR_PRESENT) sidecar_1_status = SIDECAR_DISABLED;
				    if(sidecar_2_available==SIDECAR_PRESENT) sidecar_2_status = SIDECAR_DISABLED;
				    if(sidecar_3_available==SIDECAR_PRESENT) sidecar_3_status = SIDECAR_DISABLED;
				}
				//remaining state is disabled
				//return ff
				written_data = 0x0ff;
				output_buffer[write_pointer++] = written_data;
			    }
			}
			else{
			    //sidecar not present, return ff
			    written_data = 0x0ff;
			    output_buffer[write_pointer++] = written_data;
			}
		    break;
		    case 1:
			if(sidecar_1_available==SIDECAR_PRESENT){
			    //we also need a previous option initialized
			    if((sidecar_1_status==SIDECAR_RESET)&&(sidecar_0_status == SIDECAR_INITIALIZED)){
				sidecar_1_status = SIDECAR_INITIALIZED;
				written_data = 0;
				written_data |= IR_MASK;
				written_data |= addressed_option;
				output_buffer[write_pointer++] = written_data;
				//PDEBUG("SidecarHAPI_Write function, addressed option 1\n");
				
				//next sidecar to reset state
				if(sidecar_2_available==SIDECAR_PRESENT) sidecar_2_status = SIDECAR_RESET;
			    }
			    else{
				//initialization of sidecar not in reset state
				if(sidecar_1_status==SIDECAR_INITIALIZED){
				    //set sidecar to reset state
				    sidecar_1_status = SIDECAR_RESET;
				    //next sidecars seems to be missing
				    if(sidecar_2_available==SIDECAR_PRESENT) sidecar_2_status = SIDECAR_DISABLED;
				    if(sidecar_3_available==SIDECAR_PRESENT) sidecar_3_status = SIDECAR_DISABLED;
				}
				//disabled sidecar return ff
				written_data = 0x0ff;
				output_buffer[write_pointer++] = written_data;
			    }
			}
			else{
			    //sidecar not present, return ff
			    written_data = 0x0ff;
			    output_buffer[write_pointer++] = written_data;
			}
		    break;
		    case 2:
			if(sidecar_2_available==SIDECAR_PRESENT){
			    //we also need a previous option initialized
			    if((sidecar_2_status==SIDECAR_RESET)&&(sidecar_1_status == SIDECAR_INITIALIZED)){
				sidecar_2_status = SIDECAR_INITIALIZED;
				written_data = 0;
				written_data |= IR_MASK;
				written_data |= addressed_option;
			        output_buffer[write_pointer++] = written_data;
				//PDEBUG("SidecarHAPI_Write function, addressed option 2\n");
				
				//next sidecar to reset state
				if(sidecar_3_available==SIDECAR_PRESENT) sidecar_3_status = SIDECAR_RESET;
			    }
			    else{
				//initialization of sidecar not in reset state
				if(sidecar_2_status==SIDECAR_INITIALIZED){
				    //set sidecar to reset state
				    sidecar_2_status = SIDECAR_RESET;
				    //next sidecars seems to be missing
				    if(sidecar_3_available==SIDECAR_PRESENT) sidecar_3_status = SIDECAR_DISABLED;
				}
				
				//error, return ff
				written_data = 0x0ff;
				output_buffer[write_pointer++] = written_data;
			    }
			}
			else{
			    written_data = 0x0ff;
			    output_buffer[write_pointer++] = written_data;
			}
		    break;
		    case 3:
			if(sidecar_3_available==SIDECAR_PRESENT){
			    //we also need a previous sidecar initialized
			    if((sidecar_3_status==SIDECAR_RESET)&&(sidecar_2_status == SIDECAR_INITIALIZED)){
				sidecar_3_status = SIDECAR_INITIALIZED;
				written_data = 0;
				written_data |= IR_MASK;
				written_data |= addressed_option;
			        output_buffer[write_pointer++] = written_data;
				//PDEBUG("SidecarHAPI_Write function, addressed option 3\n");
			    }
			    else{
				//initialization of sidecar not in reset state
				if(sidecar_3_status==SIDECAR_INITIALIZED){
				    //set sidecar to reset state
				    sidecar_3_status = SIDECAR_RESET;
				}
				//error, return ff
				written_data = 0x0ff;
				output_buffer[write_pointer++] = written_data;
			    }
			}
			else{
			    written_data = 0x0ff;
			    output_buffer[write_pointer++] = written_data;
			}
		    break;
		    default:
		    
			written_data = 0x0ff;
			output_buffer[write_pointer++] = written_data;
		    break;
		}
		}
		
		//test of polling
		if(POLL(data)&&(!IS_DATA_INITIALIZE_SET(data))&&(!READ_PACING(data))&&(CONTROL2_FOLLOWS(data))){
		    
		    //generating of example message
		    //poll counter, every MESSAGE_INTERVAL there is a message for uplink
		    //PDEBUG("SidecarHAPI_Write function, test of polling\n");
		    if(message_interval==0){
			message_interval=MESSAGE_INTERVAL;
			//generate message for one of initialized and present messages
			switch(message_chooser){
			    case 0:    p_message = dss_key; l_message = DSS_KEY_LEN+1; message_chooser = 1; break;	
			    case 1:    p_message = option_info_ind; l_message = OPTION_INFO_IND_LEN+1; message_chooser = 2; break;
			    case 2:    p_message = option_init_req; l_message = OPTION_INIT_REQ_LEN+1; message_chooser = 0; break;
			    default: break;
			}
			//choose one of four sidecars
			switch(addressed_option){
			    case 0:
				sidecar_0_message = MESSAGE_WAITING;
			    break;
			    case 1:
				sidecar_1_message = MESSAGE_WAITING;
			    break;
			    case 2:
				sidecar_2_message = MESSAGE_WAITING;
			    break;
			    case 3:
				sidecar_3_message = MESSAGE_WAITING;
			    break;
			    default:break;
			}
		    }
		    else{
			message_interval--;
		    }
		    
		    //deselect all options
		    sidecar_0_selected=SIDECAR_NOT_SELECTED;
		    sidecar_1_selected=SIDECAR_NOT_SELECTED;
		    sidecar_2_selected=SIDECAR_NOT_SELECTED;
		    sidecar_3_selected=SIDECAR_NOT_SELECTED;
//		    PDEBUG("xSidecar selected 0:%d\n",sidecar_0_selected);
//		    PDEBUG("xSidecar selected 1:%d\n",sidecar_1_selected);
//		    PDEBUG("xSidecar selected 2:%d\n",sidecar_2_selected);
//		    PDEBUG("xSidecar selected 3:%d\n",sidecar_3_selected);

		    //which sidecar was chosen
		    switch(addressed_option){
			case 0:
			    if((sidecar_0_available==SIDECAR_PRESENT)&&(sidecar_0_status==SIDECAR_INITIALIZED)){
				written_data = 0;
    				if(sidecar_0_message==NO_MESSAGE){
				    written_data |= NO_MESSAGE_MASK;
				}
				else{//message waiting set
				    written_data |= MESSAGE_WAITING_MASK;
				    sidecar_0_selected = SIDECAR_SELECTED;
			//	    PDEBUG("Selecting sidecar 0\n");
				}
				written_data |= addressed_option;
			    }
			    else{//if option is not initialized or even is missing returned value is ffh
				written_data = 0x0ff;
			    }
			    output_buffer[write_pointer++] = written_data;
			break;
			case 1:
			    if((sidecar_1_available==SIDECAR_PRESENT)&&(sidecar_1_status==SIDECAR_INITIALIZED)){
				written_data = 0;
    				if(sidecar_1_message==NO_MESSAGE){
				    written_data |= NO_MESSAGE_MASK;
				}
				else{
				    written_data |= MESSAGE_WAITING_MASK;
				    sidecar_1_selected = SIDECAR_SELECTED;
			//	    PDEBUG("Selecting sidecar 1\n");
				}
				written_data |= addressed_option;
			    }
			    else{
				//if option is not initialized or even is missing returned value is ffh
				written_data = 0x0ff;
			    }
			    output_buffer[write_pointer++] = written_data;
			break;
			case 2:
			    if((sidecar_2_available==SIDECAR_PRESENT)&&(sidecar_2_status==SIDECAR_INITIALIZED)){
				written_data = 0;
    				if(sidecar_2_message==NO_MESSAGE){
				    written_data |= NO_MESSAGE_MASK;
				}
				else{
				    written_data |= MESSAGE_WAITING_MASK;
				    sidecar_2_selected = SIDECAR_SELECTED;
			//	    PDEBUG("Selecting sidecar 2\n");
				}
				written_data |= addressed_option;
			    }
			    else{
				//if option is not initialized or even is missing returned value is ffh
				written_data = 0x0ff;
			    }
			    output_buffer[write_pointer++] = written_data;
			break;
			case 3:
			    if((sidecar_3_available==SIDECAR_PRESENT)&&(sidecar_3_status==SIDECAR_INITIALIZED)){
    				written_data = 0;
				if(sidecar_3_message==NO_MESSAGE){
				    written_data |= NO_MESSAGE_MASK;
				}
				else{
				    written_data |= MESSAGE_WAITING_MASK;
				    sidecar_3_selected = SIDECAR_SELECTED;
			//	    PDEBUG("Selecting sidecar 3\n");
				}
				written_data |= addressed_option;
			    }
			    else{
				//if option is not initialized or even is missing returned value is ffh
				written_data = 0x0ff;
			    }
			    output_buffer[write_pointer++] = written_data;
			break;
			default:
			    written_data = 0x0ff;
			    output_buffer[write_pointer++] = written_data;
			break;
		    }/*switch*/
		}/*if(POLL)*/
		
	    }	    
	    
	    //we distinguish between two types of input bytes
	    if(IS_CONTROL2_OR_DATA(data)){
		//PDEBUG("SidecarHAPI_Write function, control2 or data written\n");
		if(next_octet==DATA_OCTET){
		    PDEBUG(" Option %d, received data octet \n",addressed_option);
		    
		}
		if(next_octet==CONTROL2_OCTET){
		    //PDEBUG("SidecarHAPI_Write function, control2 written\n");
		    
		    if(IS_REP_TER_SET(data)&&init_bit_set){
		//	PDEBUG("SidecarHAPI_Write function, init bit set\n");
			
		    }
		    
		    //reset state to addressed options
		    if(IS_RESET(data)){
		//	PDEBUG("SidecarHAPI_Write function, reset\n");
			switch(addressed_option){
			    case 0:
				if(sidecar_0_available==SIDECAR_PRESENT){
				    sidecar_0_status = SIDECAR_RESET;
		//		    PDEBUG("SidecarHAPI_Write function, reset option 0\n");
				    sidecar_1_status = SIDECAR_DISABLED;
				    sidecar_2_status = SIDECAR_DISABLED;
				    sidecar_3_status = SIDECAR_DISABLED;
				}
			    break;
			    case 1:
				if(sidecar_1_available==SIDECAR_PRESENT){
				    sidecar_1_status = SIDECAR_RESET;
		//		    PDEBUG("SidecarHAPI_Write function, reset option 1\n");
				    sidecar_2_status = SIDECAR_DISABLED;
				    sidecar_3_status = SIDECAR_DISABLED;
				}
			    break;
			    case 2:
				if(sidecar_2_available==SIDECAR_PRESENT){
				    sidecar_2_status = SIDECAR_RESET;
		//		    PDEBUG("SidecarHAPI_Write function, reset option 2\n");
				    sidecar_3_status = SIDECAR_DISABLED;
				}
			    break;
			    case 3:
				if(sidecar_3_available==SIDECAR_PRESENT){
				    sidecar_3_status = SIDECAR_RESET;
		//		    PDEBUG("SidecarHAPI_Write function, reset option 3\n");
				}
			    break;
			    case 6:
				if(sidecar_0_available==SIDECAR_PRESENT) sidecar_0_status = SIDECAR_RESET;
				if(sidecar_1_available==SIDECAR_PRESENT) sidecar_1_status = SIDECAR_DISABLED;
				if(sidecar_2_available==SIDECAR_PRESENT) sidecar_2_status = SIDECAR_DISABLED;
				if(sidecar_3_available==SIDECAR_PRESENT) sidecar_3_status = SIDECAR_DISABLED;
		//		PDEBUG("SidecarHAPI_Write function, reset all options\n");
			    break;
			    default:
			    break;
			}
		    }    
		}
	    }
    }
    return 0;
}

unsigned int SidecarHAPI_Fastmode_Write(unsigned int length, unsigned char * pTxData){

    PDEBUG("SidecarHAPI_Fastmode_Write function, length:%d, pTxData:%lu\n",length,(unsigned long)pTxData);
    return 0;
}

/* helping functions*/

void PrintBuffer(void){
    int i;
    for(i=0;i<OUTPUT_BUFFER_SIZE;i++){
	PDEBUG("%X ",output_buffer[i]);
    }
    
    PDEBUG("write pointer:%d",write_pointer);
    PDEBUG("\n");
}

#endif




