/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/


/*****************************************************************************
 * st7529Thread.c
 *
 * O.Phillips 24/7/2006
 * Worker thread to sync virtual framebuffer data out to real video memory
 * Various magic copied from web tutorial to create a kernel thread.
 */
#include <linux/sched.h>
#include <linux/smp_lock.h>

typedef struct st7529threader
{
    void (*function)    (struct st7529threader *threadp);
    struct tq_struct    tqueue;
    struct task_struct  *thread;
    struct semaphore    sema;
    wait_queue_head_t   queue;
    int                 end;
}ST7529THREAD;

static void loadThread(void *data);
static void st7529ThreadBody( struct st7529threader *threadData );

void st7529_SyncVirtToPhys( void );


ST7529THREAD st7529ThreadData;

int st7529ThreadCreate(void)
{
    init_MUTEX_LOCKED( &st7529ThreadData.sema );
    st7529ThreadData.function = st7529ThreadBody;
    st7529ThreadData.tqueue.sync = 0;
    INIT_LIST_HEAD( &st7529ThreadData.tqueue.list );
    st7529ThreadData.tqueue.routine = loadThread;
    st7529ThreadData.tqueue.data = &st7529ThreadData;
    schedule_task( &st7529ThreadData.tqueue );
    down( &st7529ThreadData.sema );

    return(0);
}


void st7529ThreadDestroy(void)
{
    if( st7529ThreadData.thread == NULL ){
        	return;
    }

    lock_kernel();
    init_MUTEX_LOCKED( &st7529ThreadData.sema );
    mb();
    st7529ThreadData.end = 1;
    mb();
    kill_proc( st7529ThreadData.thread->pid, SIGKILL, 1 );
    down( &st7529ThreadData.sema );
    unlock_kernel();
    kill_proc( 2, SIGCHLD, 1 );
    return;
}

static void st7529ThreadBody( ST7529THREAD *myThreadData )
    {
    lock_kernel();
    myThreadData->thread = current;
    siginitsetinv( &current->blocked, sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGTERM) );
    init_waitqueue_head( &myThreadData->queue );
    myThreadData->end = 0;
    sprintf( current->comm, "st7529 thread" );
    unlock_kernel();
    up( &myThreadData->sema );

    do
    {
        interruptible_sleep_on_timeout( &myThreadData->queue, HZ/10 );
        mb();
        if( myThreadData->end ){
        	break;
	}
        st7529_SyncVirtToPhys();
    }while( 1 );

    lock_kernel();
    myThreadData->thread = NULL;
    mb();
    up( &myThreadData->sema );
}



static void loadThread(void *data)
{
    ST7529THREAD *myThreadData = data;
    kernel_thread((int (*)(void *))myThreadData->function, (void *)myThreadData, 0);
}


