/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

/* ST7529 Memory Map */
#define ST7529_PHYSICAL_REG_ADDR 	(KSEG1+0x11800000)
#define ST7529_PHYSICAL_REG_SIZE	2
#define POUT				(KSEG1+0x1f102600)

/* ST7529 Register Access Macros */

#define WR_CMD( x )	*((volatile unsigned char *)(ST7529_PHYSICAL_REG_ADDR)) = x
#define WR_DAT( x )	*((volatile unsigned char *)(ST7529_PHYSICAL_REG_ADDR+1)) = x

/* ST7529 Commands */

#define	EXTIN	0x30	/* Select Ext 0 Command table */
#define	EXTOUT	0x31	/* Select Ext 1 Command table */

/* Ext0 Commands */

/*======================================================================*/
/*   Mneumonic  Value      Discription         No of Parameter Bytes    */
/*======================================================================*/
#define	DISON	0xaf	/* Display On			0		*/
#define	DISOFF	0xae	/* Display Off			0		*/
#define	DISNOR	0xa6	/* Normal Display		0		*/
#define	DISINV	0xa7	/* Inverse Display		0		*/
#define	COMSCN	0xbb	/* Comm Scan Direction		1		*/
#define	DISCTRL	0xca	/* Display Control		3		*/
#define	SLPIN	0x95	/* Sleep In			0		*/
#define	SLPOUT	0x94	/* Sleep Out			0		*/
#define	LASET	0x75	/* Line Address Set		2		*/
#define	CASET	0x15	/* Column Address Set		2		*/
#define	DATSDR	0xbc	/* Data Scan Direction		3		*/
#define	RAMWR	0x5c	/* Writing to Memory		data		*/
#define	RAMRD	0x5d	/* Reading from memory		data		*/
#define	PTLIN	0xa8	/* Partial Display In		2		*/
#define	PTLOUT	0xa9	/* Partial Display Out		0		*/
#define	RMWIN	0xe0	/* Read and Modify Write	0		*/
#define	RMWOUT	0xee	/* RMW end			0		*/
#define	ASCSET	0xaa	/* Area Scroll Set		4		*/
#define	SCSTART	0xab	/* Scrol1 start set		1		*/
#define	OSCON	0xd1	/* Internal OSC on		0		*/
#define	OSCOFF	0xd2	/* Internal OSC off		0		*/
#define	PWRCTRL	0x20	/* Power Control		1		*/
#define	VOLCTRL	0x81	/* EC control			2		*/
#define	VOLUP	0xd6	/* EC increase 1		0		*/
#define	VOLDOWN	0xd7	/* EC decrease 1		0		*/
#define	EPSRRD1	0x7c	/* Read register 1		0		*/
#define	EPSRRD2	0x7d	/* Read register 2		0		*/
#define	NOP	0x25	/* NOP Instruction		0		*/
#define	STREAD	0x10	/* Status Read (| readData)     -		*/
#define	EPINT	0x07	/* Initial Code (1)		1		*/

/* Ext1 Commands */

/*======================================================================*/
/*   Mneumonic  Value      Discription         No of Parameter Bytes    */
/*======================================================================*/
#define	GRYSET1	0x20	/* Frame 1 Gray PWM set		16		*/
#define	GRYSET2	0x21	/* Frame 2 Gray PWN set		16		*/
#define	ANASET	0x32	/* Analog Circuit Set		3		*/
#define	SWINT	0x34	/* Software Initial		0		*/
#define	EPCTIN	0xcd	/* Control EEPROM		1		*/
#define	EPCOUT	0xcc	/* Cancel EEPROM		0		*/
#define	EPMWR	0xfc	/* Write to EEPROM		0		*/
#define	EPMRD	0xfd	/* Read from EEPROM		0		*/

/*======================================================================*/
/*   Specifics for Wp1 and Wp2 Controller initialisation		*/
/*======================================================================*/
#define	WP1_START_COLUMN	0
#define WP1_END_COLUMN		68
#define WP1_START_LINE		60
#define WP1_END_LINE		101

#define	WP2_START_COLUMN	5
#define WP2_END_COLUMN		84
#define WP2_START_LINE		16
#define WP2_END_LINE		143

