/****************************************************************************************
	Copyright (c) Siemens Enterprise Communications Limited

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
****************************************************************************************/

/*

*/

#include <linux/module.h>
#include <asm/types.h>
#include <linux/version.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/incaip2/tsf.h>
#include <asm/incaip2/port.h>


#include <linux/tqueue.h> /* task queues */
#include <linux/time.h>


typedef struct  {
	spinlock_t lock;
} timer_device_t;





static timer_device_t timer_data;

/** Major ID of timer deivice (default: 0=dynamic)
*/
static int major = 0;
/**
* Module parameter declaration of variable major as type integer
*/
MODULE_PARM(major, "i");
/**
* Module parameter description of variable major
*/

MODULE_PARM_DESC(major, "Major ID timer (default: 0=dynamic)");


/* External requests for timers (user space) */




/* Internal requests for timers (kernel space) */
int request_timer_irq( eHw_TimerId number, timerCallback callback, void *data ){

	switch(cmd)
	{
		case TIMER_FOR_HOOKSWITCH:
		{
			//specific timer for 1 mS
			*INCA_IP2_GPTC_CON_2A  		=   0x000001ca;
			// don't think that this value represents the 1 mS
			*INCA_IP2_GPTC_RELOAD_2A	=   0x00000001;
  			*INCA_IP2_GPTC_COUNT_2A		=   0x00000001;	
   			//enable interrupt      
   			*INCA_IP2_GPTC_IRNEN   	   	|=  0x00000004; 
			/* stop the timer running */
			*INCA_IP2_GPTC_RUN_2A  		=   0x00000002;
			//take care of the interrupt
			
			
			
			
			
			
			
			
			break;
		}


		case TIMER_FOR_HEADSET:
		{
			break;
		}
		default:
		{
			break;
		}
	}

	
	
	
	
	int retVal = 0;
	unsigned long flags;
	spinlock_t lock;

	spin_lock_init(&lock);
	spin_lock_irqsave(lock,flags);


	if( irq_desc[number].handler == fpgaDefaultIntHander )
	{
		irq_desc[number].handler = hander;
		irq_desc[number].data = data;
		irq_desc[number].status = DISABLED;
		retVal = 1;
	}

	spin_unlock_irqrestore(lock,flags);




	return( retVal );


}


int free_timer_irq(eHw_TimerId number ){

}


int enable_timer_irq( eHw_TimerId number ){

}


int disable_timer_irq(eHw_TimerId number ){

}



/**
* Device filesystem handle structure for the creation of device files
*/
static devfs_handle_t input_devfs_handle;


ssize_t timer_write(struct file *fp, const char *buf, size_t size, loff_t *off);
int timer_open(struct inode *inode, struct file *fp);




/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>> IOCTL Section <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

/**
* Declartion of the file operations structure which effectively aliases application
* functions such as read, write etc. to equivalent kernel funtion
*/

struct file_operations timer_fops = {
	open:		timer_open,  /* open */
	write:		timer_write, /* write */
};

/**
 * Aliased open function called from user space by open
 *\param *inode pointer to the inode structure for the device this contains such information such
 *as the major and minor numbers.
 *\param *fp pointer to a kernel file data structure
 *\returns 0 for success (in turn the kernel returns a file descriptor)
 */
int timer_open(struct inode *inode, struct file *fp)
{
	int omajor;
	int ominor;

	ominor = MINOR(inode->i_rdev);
	omajor = MAJOR(inode->i_rdev);
	printk(KERN_NOTICE "timer_open: major = %d minor = %d\n", omajor, ominor);
	
	fp->private_data = &timer_data;



	return 0;
}

/**
 * Aliased write function called from user space
 *\param *fp pointer to a kernel file data structure
 *\param *buf pointer to a user space data buffer
 *\param size of the user space data buffer
 *\param *off pointer to file position
 *\returns Number of data items sent or error state
 */

ssize_t timer_write(struct file *fp, const char *buf, size_t size, loff_t *off)
{

	timer_device_t* dev = (timer_device_t*)fp->private_data;


	if (size != sizeof(timer_device_t))
	{
		printk(KERN_WARNING "timer_write: illegal size %d\n", size);
		return 0;
	}
	
	if (copy_from_user(&dev->timer_period, buf, sizeof(timer_device_t)))
	{
		return( -EFAULT );
	}
	return size;

}


/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>> PROC Section <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/

/**
* A pointer to a proc directory entry structure used to create a proc entry for a device
*/
static struct proc_dir_entry *timer_proc;

/**
* Function to handle a proc entry read i.e. 'cat /proc/drivers/Backlight'
*\param *buf pointer to buffer where data is to be written
*\param **start used by the function to point to a specific page of data
*\param offset offset into file structure
*\param count number of data items returned
*\param *eof pointer to an interger set by the driver to signal no more data is to be sent
*\param *data general purpose data pointer
*\returns number of bytes of data returned
*/

static int timer_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
	int len = 0;
	const char info[] = "timer Device version 1.0\n";
    printk(KERN_INFO "/proc/drivers/timer %d\n",count);

    len = sizeof(info);

    if( count < sizeof(info))
    	len = count;
    memcpy(buf,info,len);

	return len;
}

/**
* Function to handle a proc entry write not used
*/

static int timer_write_procmem(struct file *file, const char *buffer,unsigned long count, void *data)
{
	printk(KERN_INFO "Write procmem\n");
	return count;

}

/**
* Kernel initialisation function run when the module is installed via insmod
*/

static int __init timer_init(void)
{
	int res = 0;

	/* create and register device */

	if( (input_devfs_handle = devfs_mk_dir(NULL, "input", NULL)) == 0 )
		return -EBUSY;

	devfs_register(input_devfs_handle, "timer",DEVFS_FL_AUTO_DEVNUM,0, 0, S_IFCHR | S_IRUGO | S_IWUGO,&timer_fops,0);


	if (major == 0) major = res;

	SET_MODULE_OWNER(&timer_fops);

	printk(KERN_NOTICE "\ntimer device registered.\n");

	/* create PROC entry */
	if( (timer_proc = create_proc_entry( "driver/timer", 0, 0)) == 0 )
	{
		printk(KERN_ERR "could not create /proc/driver/timer!\n");
		devfs_unregister(input_devfs_handle);

		return( -ENOENT );
	}

	timer_proc->read_proc  = timer_read_procmem;
	timer_proc->write_proc = timer_write_procmem;

	printk(KERN_NOTICE "/proc/driver/timer created!\n");
	
	spin_lock_init(&timer_data.lock);

	/* set up the timers */
		int err;
	disable_incaip2_irq(INT_NUM_IM3_IRL24);
	if( (err = request_irq(INT_NUM_IM3_IRL24,hookSwitchPulse_isr, SA_INTERRUPT,"hookSwitchPulse", NULL)) == 0 )
	{ 
		enable_incaip2_irq(INT_NUM_IM3_IRL24);
	}
	else
	{
		printk(KERN_NOTICE "\nHOOK SWITCH PULSE ISR Init Failed\n");
	}
	
	//Power supply for GPTC module
	*INCA_IP2_PMS_PMS_GEN		|=	0x00000400;
	// set up the timer run at the same speed as the 8 kHz signal
	//set up Clock Control the same as mps_device.c, for voice firmware
	*INCA_IP2_GPTC_CLC 			=   0x00010114;



	return( 0 );
}

/**
* Kernel clean-up function run when the module is removed via rmmod
*/

static void __exit timer_exit(void)
{


	remove_proc_entry("driver/timer", 0);

	devfs_unregister(input_devfs_handle);

	printk(KERN_NOTICE "timer_exit\n");
}


#ifdef MODULE
module_init(timer_init);
module_exit(timer_exit);
#endif

EXPORT_SYMBOL(fpga_reg_read);
EXPORT_SYMBOL(request_timer_irq);
EXPORT_SYMBOL(free_timer_irq);
EXPORT_SYMBOL(enable_timer_irq);
EXPORT_SYMBOL(disable_timer_irq);

/**
* Macro to tell the kernel that the module bears a free license (stops the kernel complaining!)
*/
MODULE_LICENSE("GPL");
