//============================================================================
//
//  Copyright (c) 2000,2005 Epson Research and Development, Inc.
//  All rights reserved.
//
//  This file is subject to the terms and conditions of the GNU General Public
//  License. See the file COPYING in the main directory of this archive for
//  more details.
//
//=============================================================================
// This file is for S1D13733. It may need some modifications for other S1D13XXX conrollers.
#include <linux/sched.h>
#include <linux/smp_lock.h>

#define REG0022_II_MEM_ADDR0    0x0022
#define REG0024_II_MEM_ADDR1    0x0024
#define REG0026_IAUTOINC        0x0026
#define REG0028_II_MEM_PORT     0x0028

#define CMD_WRITE        0
#define CMD_READ         1
#define LOOP_LIMIT       1000

struct fb_info *info;

typedef struct
    {
    u16 *pCmd;
    u16 *pData16;
    u16 *pIndex;
    u16 *pStatus;
    u16 *pData;
}IND_STRUCT;

IND_STRUCT IndInfo;     // indirect pointers structure.


//---------------------------------------------------------------------------
// INDIRECT FUNCTION: indirect_init() - Initialize indirect interface
//-------------------------------struct
void indirect_init( u32 RegAddr )
    {
    IndInfo.pIndex	= (u16 *)(RegAddr + 0);
    IndInfo.pStatus	= (u16 *)(RegAddr + 2);
    IndInfo.pData	= (u16 *)(RegAddr + 4);
    }


//---------------------------------------------------------------------------
// INDIRECT FUNCTION: indirect_write_reg()
// WARNING: DO NOT USE ODD ADDRESSES FOR 16-bit or 32bit ACCESSES!
//---------------------------------------------------------------------------
void indirect_write_reg( u16 Index, u16 Value )
    {
    u16 timeout=LOOP_LIMIT;
    u16 wIndex16 = (u16)(Index & ~1);

    lock_kernel();
    *IndInfo.pIndex = (u16)(wIndex16 | CMD_WRITE);
    *IndInfo.pData  = (u16)(Value);
    while( ((*IndInfo.pStatus)&1) && timeout-- );
    unlock_kernel();
    }



//---------------------------------------------------------------------------
// INDIRECT FUNCTION: indirect_read_reg()
// WARNING: DO NOT USE ODD ADDRESSES FOR 16-bit or 32bit ACCESSES!
//---------------------------------------------------------------------------
u16 indirect_read_reg( u16 Index )
    {
    u16 timeout=LOOP_LIMIT;
    u16 wIndex16 = Index & ~1;
    u16 wData;

    lock_kernel();
    *IndInfo.pIndex = (u16)(wIndex16 | CMD_READ);
    wData = *IndInfo.pData;
    while( ((*IndInfo.pStatus)&1) && timeout-- );
    unlock_kernel();

    return wData;
}



void indirect_write_display_address(u32 Offset)
    {

    // Set the upper 4 bits of memory address in REG[0024h].
    *IndInfo.pIndex  = REG0024_II_MEM_ADDR1;
    *IndInfo.pData   = (u16)(Offset >> 16);

    // Set the lower bits of memory address in REG[0022h].
    *IndInfo.pIndex  = REG0022_II_MEM_ADDR0;
    *IndInfo.pData   = (u16)Offset;

    // Point to the indirect memory port
    *IndInfo.pIndex  = REG0028_II_MEM_PORT;

    return;
}



void indirect_write_display(u32 Offset, u32 Value)
    {
    u16 timeout=LOOP_LIMIT;
    u32 dwOffset16 = (Offset & ~1); 	// 16-bit aligned offset

    lock_kernel();
    indirect_write_display_address(dwOffset16 | CMD_WRITE);
    *IndInfo.pData = (u16)Value;
    while( ((*IndInfo.pStatus)&1) && timeout-- );
    unlock_kernel();
    }















