/*==================================================================
**  Generic Header information 
**  Copyright (c) 2003 Epson Research and Development, Inc.
**  All rights reserved.
**
++  This file is subject to the terms and conditions of the GNU General Public
++  License. See the file COPYING in the main directory of this archive for
++  more details.
++
**  DISPLAYS      WxHxBPP     FREQ    ADDITIONAL                                 
**  ------------- ----------- ------- -------------------------------------------
** *LCD1=RGB      320x240     64 Hz   General TFT, ND-TFD (18-bit)
**  LCD2=Serial   2x1x8       0 Hz    4 pins (8-bit)
** 
**  DIMENSIONS    WxHxBPP @ STRIDE   START   SADDR                    ADDITIONAL 
**  ------------- ------------------ ------- ------------------------ -----------
**  Main Window   320x240x32 @ 320   NA      00030000h + bias of 0h   Swivel=0
**  PIP+ Window   88x72x16 @ 176     8,8     00010000h + bias of 0h   Swivel=0
**  View Resizer  88x72              0,0     00010000h                Scaling=OFF
**  Capt Resizer  352x288            0,0     00010000h                Scaling=OFF
** 
**  CLOCKS        FREQ        SOURCE                                             
**  ------------- ----------- ---------------------------------------------------
**  INCLK         54.001 MHz  PLL (via CLKI)
**  SYSCLK        54.001 MHz  INCLK/1
**  PCLK          6.750 MHz   SYSCLK/8
**  SERCLK        4.500 MHz   SYSCLK/12
**  CM1CLK        13.500 MHz  SYSCLK/4
**  CM2CLK        13.500 MHz  SYSCLK/4
**
**  This file defines the configuration environment and registers,
**  which can be used by any software, such as display drivers.
**
**  Note: If you transfer this file to any non-PC system, use ASCII
**  mode (not BINARY) to maintain system-specific line terminators.
++
++  Note: This file has been manually edited and tested with SHARP 
++  LQ038Q5DR01 320x240 18 Bit Color TFT 3.3V panel and EPSON S1D1371XB00B 
++  rev. 1.0 board using S1D13714 in FC_BGA13714 Daughter board. 
++  Dip switches: 1,2,5: OFF, 3,4,6,7,8: ON              
++  Panel connected using the H5 Header connector.
++  EPSON S1D1371XB00B rev. 1.0 board jumpers: 
++  JP1: 2-3
++  JP2: 2-3
++  JP3: 2-3
++  JP4: 2-3
++  JP5: 2-3
++  JP6: 1-2
++  JP7: 2-3
++  JP8: 1-2
++  JP9: 1-2
++  JP11:1-2,
++  JP12:2-3
++  JP13:2-3
++  JP15:2-3
++  JP16:1-2
++  JP17:1-2
++  PCI Bridge card BEM version : 3.02
**==================================================================*/

#define S1D_13714
#define S1D_DISPLAY_WIDTH               320
#define S1D_DISPLAY_HEIGHT              240
#define S1D_DISPLAY_BPP                 32
#define S1D_DISPLAY_SCANLINE_BYTES      1280
#define S1D_DISPLAY_FRAME_RATE          64
#define S1D_DISPLAY_PCLK                6750208L
#define S1D_PHYSICAL_REG_ADDR           0x00000000L
#define S1D_PHYSICAL_BLT_ADDR           0x00010000L
#define S1D_PHYSICAL_VMEM_ADDR          0x00000000L
#define S1D_PHYSICAL_REG_SIZE           36864L
#define S1D_PHYSICAL_VMEM_SIZE          327680L
#define S1D_PHYSICAL_VMEM_REQUIRED      307200L
#define S1D_PALETTE_SIZE                256
#define S1D_POWER_DELAY_OFF             1200
#define S1D_POWER_DELAY_ON              50
#define S1D_HWBLT
#define S1D_SWBLT
#define S1D_REGRESERVED                 0xFFF0
#define S1D_REGDELAYPLL                 0xFFFC
#define S1D_REGDELAYOFF                 0xFFFD
#define S1D_REGDELAYON                  0xFFFE

#define S1D_WRITE_PALETTE(p,i,r,g,b)  \
  {  \
    ((volatile S1D_VALUE*)(p))[(0x400/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))] = (S1D_VALUE)(g)<<8 | (r); \
    ((volatile S1D_VALUE*)(p))[(0x402/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))] = (b);  \
  }

#define S1D_READ_PALETTE(p,i,r,g,b)  \
  {  \
    S1D_VALUE reg1, reg2;  \
    reg1 = ((volatile S1D_VALUE*)(p))[(0x400/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))];  \
    reg2 = ((volatile S1D_VALUE*)(p))[(0x402/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))];  \
    (g) = (reg1>>8)&0xFF;  \
    (r) = reg1&0xFF;  \
    (b) = reg2&0xFF;  \
  }

#define S1D_WRITE_PALETTE_LUT2(p,i,r,g,b)  \
  {  \
    ((volatile S1D_VALUE*)(p))[(0x800/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))] = (S1D_VALUE)(g)<<8 | (r); \
    ((volatile S1D_VALUE*)(p))[(0x802/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))] = (b);  \
  }

#define S1D_READ_PALETTE_LUT2(p,i,r,g,b)  \
  {  \
    S1D_VALUE reg1, reg2;  \
    reg1 = ((volatile S1D_VALUE*)(p))[(0x800/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))];  \
    reg2 = ((volatile S1D_VALUE*)(p))[(0x802/sizeof(S1D_VALUE))+((i)*sizeof(S1D_VALUE))];  \
    (g) = (reg1>>8)&0xFF;  \
    (r) = reg1&0xFF;  \
    (b) = reg2&0xFF;  \
  }

typedef unsigned short S1D_INDEX;
typedef unsigned short S1D_VALUE;
typedef struct
{
    S1D_INDEX Index;
    S1D_VALUE Value;
} S1D_REGS;

#define S1D_INSTANTIATE_REGISTERS(scope_prefix,variable_name)  \
  scope_prefix S1D_REGS variable_name[] =  \
  {  \
    { 0x0014,           0x00D1 },\
    { 0x0012,           0x0001 },\
    { 0x000E,           0xF199 },   /* PLL Setting Register 0                             */  \
    { 0x0010,           0x2000 },   /* PLL Setting Register 1                             */  \
    { 0x0018,           0x0000 },   /* System Clock Setting Register                      */  \
    { 0x0012,           0x0000 },\
    { S1D_REGDELAYON,   0x0064 },   /* DELAY (in ms)                                      */  \
    { 0x0014,           0x00D0 },\
    { 0x0300,           0x0000 },   /* GPIO Status and Control Register 0                 */  \
    { 0x0302,           0x0000 },   /* GPIO Status and Control Register 1                 */  \
    { 0x0304,           0x0000 },   /* GPIO Status and Control Register 2                 */  \
    { 0x0306,           0x0000 },   /* GPIO Status and Control Register 3                 */  \
    { 0x0308,           0xFFFF },   /* GPIO Pull Down Control Register 0                  */  \
    { 0x030A,           0x003F },   /* GPIO Pull Down Control Register 1                  */  \
    { 0x030C,           0x0000 },   /* GPIO Status and Control Register 4                 */  \
    { 0x030E,           0x0000 },   /* GPIO Status and Control Register 5                 */  \
    { 0x0006,           0x0000 },   /* Bus Timeout Setting Register                       */  \
    { 0x0022,           0x0000 },   /* Indirect Interface Memory Address Register 1       */  \
    { 0x0024,           0x0000 },   /* Indirect Interface Memory Address Register 2       */  \
    { 0x0026,           0x0000 },   /* Indirect Interface Auto Increment Register         */  \
    { 0x0030,           0x0503 },   /* LCD Interface Clock Setting Register               */  \
    { 0x0032,           0x0030 },   /* LCD Module Clock Setting Register                  */  \
    { 0x0034,           0x0000 },   /* LCD Interface Command Register                     */  \
    { 0x0036,           0x0000 },   /* LCD Interface Parameter Register                   */  \
    { 0x003A,           0x0000 },   /* LCD Interface Frame Transfer Register              */  \
    { 0x003C,           0x0000 },   /* LCD Interface Transfer Setting Register            */  \
    { 0x0040,           0x0031 },   /* LCD1 Horizontal Total Register                     */  \
    { 0x0042,           0x009F },   /* LCD1 Horizontal Display Period Register            */  \
    { 0x0044,           0x0035 },   /* LCD1 Horizontal Display Period Start Position Register */  \
    { 0x0046,           0x008F },   /* LCD1 FPLINE Register                               */  \
    { 0x0048,           0x000F },   /* LCD1 FPLINE Pulse Position Register                */  \
    { 0x004A,           0x0106 },   /* LCD1 Vertical Total Register                       */  \
    { 0x004C,           0x00EF },   /* LCD1 Vertical Display Period Register              */  \
    { 0x004E,           0x0016 },   /* LCD1 Vertical Display Period Start Position Register */  \
    { 0x0050,           0x0081 },   /* LCD1 FPFRAME Register                              */  \
    { 0x0052,           0x0010 },   /* LCD1 FPFRAME Pulse Position Register               */  \
    { 0x0054,           0x0001 },   /* LCD1 Serial Interface Setting Register             */  \
    { 0x0056,           0x0000 },   /* LCD1 Parallel Interface Setting Register           */  \
    { 0x0058,           0x0000 },   /* LCD2 Horizontal Display Period Register            */  \
    { 0x005A,           0x0000 },   /* LCD2 Vertical Display Period Register              */  \
    { 0x005C,           0x0001 },   /* LCD2 Serial Interface Setting Register             */  \
    { 0x005E,           0x0000 },   /* LCD2 Parallel Interface Setting Register           */  \
    { 0x0070,           0x0000 },   /* TFT Type 5 Horizontal Total Register               */  \
    { 0x0080,           0x0000 },   /* Samsung a-TFT Horizontal Total Register            */  \
    { 0x0082,           0x0000 },   /* Samsung a-TFT LD Rising Edge Register              */  \
    { 0x0084,           0x0000 },   /* Samsung a-TFT CKV Toggle Point Register            */  \
    { 0x0086,           0x0000 },   /* Samsung a-TFT VCOM Toggle Point Register           */  \
    { 0x0088,           0x0000 },   /* Samsung a-TFT Pulse Width Register                 */  \
    { 0x008A,           0x0000 },   /* Toshiba Type-H AP1 Register 0                      */  \
    { 0x008C,           0x0000 },   /* Toshiba Type-H AP1 Register 1                      */  \
    { 0x008E,           0x0000 },   /* Toshiba Type-H AP1 Register 2                      */  \
    { 0x0090,           0x0000 },   /* HR-TFT Configuration Register                      */  \
    { 0x0092,           0x012C },   /* HR-TFT CLS Width Register                          */  \
    { 0x0094,           0x0032 },   /* HR-TFT PS1 Rising Edge Register                    */  \
    { 0x0096,           0x0064 },   /* HR-TFT PS2 Rising Edge Register                    */  \
    { 0x0098,           0x000A },   /* HR-TFT PS2 Toggle Width Register                   */  \
    { 0x009A,           0x0064 },   /* HR-TFT PS3 Signal Width Register                   */  \
    { 0x009E,           0x000A },   /* HR-TFT REV Toggle Point Register                   */  \
    { 0x00A0,           0x0007 },   /* HR-TFT PS1/2 End Register                          */  \
    { 0x00A2,           0x0000 },   /* Type2 TFT Configuration Register                   */  \
    { 0x00A4,           0x0E09 },   /* Casio TFT Timing Register 0                        */  \
    { 0x00A6,           0x0918 },   /* Casio TFT Timing Register 1                        */  \
    { 0x00A8,           0x0000 },   /* Type3 TFT Configuration Register 0                 */  \
    { 0x00C6,           0x0000 },   /* Toshiba Type-H/Type-D YST Register 0               */  \
    { 0x00C8,           0x0000 },   /* Toshiba Type-H YST Register 1                      */  \
    { 0x00CA,           0x0000 },   /* Toshiba Type-H COM Register                        */  \
    { 0x00CC,           0x0000 },   /* Toshiba Type-H/Type-D VNDP Register                */  \
    { 0x00CE,           0x0000 },   /* Toshiba Type-H YCK Register                        */  \
    { 0x00D0,           0x018F },   /* Toshiba Type-H/Type-D Horizontal Total Register    */  \
    { 0x00D2,           0x0000 },   /* Toshiba Type-H OE Register                         */  \
    { 0x00D4,           0x0000 },   /* Toshiba Type-H STH and Type-D XST Register         */  \
    { 0x00D6,           0x0000 },   /* Toshiba Type-H LOAD Register                       */  \
    { 0x00D8,           0x0000 },   /* Toshiba Type-H ASW Register 0                      */  \
    { 0x00DA,           0x0000 },   /* Toshiba Type-H ASW Register 1                      */  \
    { 0x00DC,           0x0000 },   /* Toshiba Type-H ASW Register 2                      */  \
    { 0x00DE,           0x0000 },   /* Toshiba Type-H/Type-D HNDP Register [RESERVED]     */  \
    { 0x00E0,           0x0000 },   /* Toshiba Type-D Miscellaneous Register 0            */  \
    { 0x00E2,           0x0000 },   /* Toshiba Type-D Miscellaneous Register 1            */  \
    { 0x00E4,           0x0000 },   /* Toshiba Type-D Miscellaneous Register 2            */  \
    { 0x00E6,           0x0000 },   /* Toshiba Type-D Miscellaneous Register 3            */  \
    { 0x00E8,           0x0000 },   /* Toshiba Type-D Miscellaneous Register 4            */  \
    { 0x00EA,           0x0023 },   /* Toshiba Miscellaneous Register                     */  \
    { 0x00EC,           0x0000 },   /* Toshiba Type-D SRAM Mode Register [RESERVED]       */  \
    { 0x00EE,           0x0000 },   /* Partial Drive Area0 Start Line Register            */  \
    { 0x00F0,           0x0000 },   /* Partial Drive Area0 End Line Register              */  \
    { 0x00F2,           0x0000 },   /* Partial Drive Area1 Start Line Register            */  \
    { 0x00F4,           0x0000 },   /* Partial Drive Area1 End Line Register              */  \
    { 0x00F6,           0x0000 },   /* Partial Drive Area0 Start Address Register 0       */  \
    { 0x00F8,           0x0000 },   /* Partial Drive Area0 Start Address Register 1       */  \
    { 0x00FA,           0x0000 },   /* Partial Drive Area1 Start Address Register 0       */  \
    { 0x00FC,           0x0000 },   /* Partial Drive Area1 Start Address Register 1       */  \
    { 0x00FE,           0x0001 },   /* LCD Interface ID Register                          */  \
    { 0x0120,           0x0000 },   /* Strobe Line Delay Register                         */  \
    { 0x0122,           0x0000 },   /* Strobe Pulse Width Register                        */  \
    { 0x0124,           0x0000 },   /* Strobe Control Register                            */  \
    { 0x0128,           0x0000 },   /* MPEG Codec Interface Height Register               */  \
    { 0x012A,           0x0000 },   /* MPEG Codec Interface Width Register                */  \
    { 0x0200,           0x0027 },   /* Display Mode Setting Register 0                    */  \
    { 0x0202,           0x0400 },   /* Display Mode Setting Register 1                    */  \
    { 0x0204,           0x0000 },   /* Transparent Overlay Key Color Red Data Register    */  \
    { 0x0206,           0x0000 },   /* Transparent Overlay Key Color Green Data Register  */  \
    { 0x0208,           0x0000 },   /* Transparent Overlay Key Color Blue Data Register   */  \
    { 0x0210,           0x0000 },   /* Main Window Display Start Address Register 0       */  \
    { 0x0212,           0x0000 },   /* Main Window Display Start Address Register 1       */  \
    { 0x0216,           0x0500 },   /* Main Window Line Address Offset Register           */  \
    { 0x0218,           0x0000 },   /* PIP+ Display Start Address Register 0              */  \
    { 0x021A,           0x0001 },   /* PIP+ Display Start Address Register 1              */  \
    { 0x021E,           0x00B0 },   /* PIP+ Window Line Address Offset Register           */  \
    { 0x0220,           0x0008 },   /* PIP+ X Start Positions Register                    */  \
    { 0x0222,           0x0008 },   /* PIP+ Y Start Positions Register                    */  \
    { 0x0224,           0x005F },   /* PIP+ X End Positions Register                      */  \
    { 0x0226,           0x004F },   /* PIP+ Y End Positions Register                      */  \
    { 0x0228,           0x0000 },   /* PIP+ Start Field Register                          */  \
    { 0x022A,           0x0000 },   /* Back Buffer Display Start Address Register 0       */  \
    { 0x022C,           0x0000 },   /* Back Buffer Display Start Address Register 1       */  \
    { 0x0240,           0x0405 },   /* YUV/RGB Translate Mode Register                    */  \
    { 0x0242,           0x0000 },   /* YUV Image Input Write Start Address 0 Register 0   */  \
    { 0x0244,           0x0001 },   /* YUV Image Input Write Start Address 0 Register 1   */  \
    { 0x024A,           0x0000 },   /* UV Data Fix Register                               */  \
    { 0x024C,           0x0030 },   /* YRC Rectangular Pixel Width Register               */  \
    { 0x024E,           0x0000 },   /* YRC Rectangular Line Address Offset Register       */  \
    { 0x0260,           0x0005 },   /* RGB/YUV Converter Configuration Register           */  \
    { 0x0262,           0x0000 },   /* RGB/YUV UV Data Clip Register [RESERVED]           */  \
    { 0x0264,           0x0000 },   /* Memory Image Encode Horizontal Display Period Register */  \
    { 0x0266,           0x0000 },   /* Memory Image Encode Vertical Display Period Register */  \
    { 0x0310,           0x0000 },   /* Average Overlay Key Color Red Data Register        */  \
    { 0x0312,           0x0000 },   /* Average Overlay Key Color Green Data Register      */  \
    { 0x0314,           0x0000 },   /* Average Overlay Key Color Blue Data Register       */  \
    { 0x0316,           0x0000 },   /* AND Overlay Key Color Red Data Register            */  \
    { 0x0318,           0x0000 },   /* AND Overlay Key Color Green Data Register          */  \
    { 0x031A,           0x0000 },   /* AND Overlay Key Color Blue Data Register           */  \
    { 0x031C,           0x0000 },   /* OR Overlay Key Color Red Data Register             */  \
    { 0x031E,           0x0000 },   /* OR Overlay Key Color Green Data Register           */  \
    { 0x0320,           0x0000 },   /* OR Overlay Key Color Blue Data Register            */  \
    { 0x0322,           0x0000 },   /* INV Overlay Key Color Red Data Register            */  \
    { 0x0324,           0x0000 },   /* INV Overlay Key Color Green Data Register          */  \
    { 0x0326,           0x0000 },   /* INV Overlay Key Color Blue Data Register           */  \
    { 0x0328,           0x0000 },   /* Overlay Miscellaneous Register                     */  \
    { 0x0944,           0x0000 },   /* View Resize Start X Position Register              */  \
    { 0x0946,           0x0000 },   /* View Resize Start Y Position Register              */  \
    { 0x0948,           0x0057 },   /* View Resize End X Position Register                */  \
    { 0x094A,           0x0047 },   /* View Resize End Y Position Register                */  \
    { 0x094E,           0x0000 },   /* View Resize Operation Setting Register 1           */  \
    { 0x094C,           0x0004 },   /* View Resize Operation Setting Register 0           */  \
    { 0x0964,           0x0000 },   /* Capture Resize Start X Position Register           */  \
    { 0x0966,           0x0000 },   /* Capture Resize Start Y Position Register           */  \
    { 0x0968,           0x015F },   /* Capture Resize End X Position Register             */  \
    { 0x096A,           0x011F },   /* Capture Resize End Y Position Register             */  \
    { 0x096E,           0x0000 },   /* Capture Resize Operation Setting Register 1        */  \
    { 0x096C,           0x0004 },   /* Capture Resize Operation Setting Register 0        */  \
    { 0x0940,           0x0000 },   /* View Resize Control Register                       */  \
    { 0x0960,           0x0000 },   /* Capture Resize Control Register                    */  \
    { 0x0930,           0x0000 },   /* Global Resize Control Register                     */  \
    { 0x0980,           0x0000 },   /* JPEG Control Register                              */  \
    { 0x09A0,           0x0020 },   /* JPEG FIFO Control Register                         */  \
    { 0x09A4,           0x000F },   /* JPEG FIFO Size Register                            */  \
    { 0x09B0,           0x0000 },   /* Encode Size Limit Register 0                       */  \
    { 0x09B2,           0x0020 },   /* Encode Size Limit Register 1                       */  \
    { 0x09BC,           0x0000 },   /* JPEG FIFO Address Offset Register                  */  \
    { 0x0100,           0x0003 },   /* Camera 1 Clock Setting Register                    */  \
    { 0x0102,           0x0000 },   /* Camera 1 Signal Setting Register                   */  \
    { 0x0104,           0x0003 },   /* Camera 2 Clock Divide Select Register              */  \
    { 0x0106,           0x0000 },   /* Camera 2 Input Signal Format Select Register       */  \
    { 0x0108,           0x0000 },   /* Camera 1 Type2 Interface Vertical Counter Register [RESERVED] */  \
    { 0x010A,           0x0000 },   /* Camera 1 Type2 Interface Horizontal Counter Register [RESERVED] */  \
    { 0x010C,           0x0000 },   /* Camera 1 Type2 Control Register [RESERVED]         */  \
    { 0x010E,           0x0000 },   /* Camera 1 Type2 Status Register [RESERVED]          */  \
    { 0x0112,           0x0000 },   /* Camera Frame Setting Register                      */  \
    { 0x0114,           0x0000 },   /* Camera Control Register [WRITEONLY]                */  \
    { 0x0110,           0x1000 },   /* Camera Mode Setting Register                       */  \
    { 0x0A02,           0x0000 },   /* Interrupt Control Register 0                       */  \
    { 0x0A04,           0x0000 },   /* Interrupt Control Register 1                       */  \
    { 0x0A06,           0x0000 },   /* Debug Status Register                              */  \
    { 0x0A08,           0x0000 },   /* Interrupt Control for Debug Register               */  \
    { 0x0A0A,           0x0000 },   /* Host Cycle Interrupt Status Register               */  \
    { 0x0A0C,           0x0000 },   /* Host Cycle Interrupt Control Register              */  \
    { 0x0A0E,           0x0000 },   /* Cycle Cycle Time Out Control Register              */  \
    { 0x0F00,           0x0001 },   /* JPEG Encode Performance Register                   */  \
    { 0x00EA,           0x8023 },   /* Toshiba Miscellaneous Register                     */  \
    { S1D_REGDELAYON,   0x0032 },   /* LCD PANEL POWER ON DELAY (in ms)                   */  \
    { 0x030C,           0x0000 },   /* GPIO Status and Control Register 4                 */  \
    { 0x030E,           0x0000 },   /* GPIO Status and Control Register 5                 */  \
    { 0x0014,           0x0000 }    /* Miscellaneous Configuration Register               */  \
  }


