//=============================================================================
//
//  Copyright (c) 2000,2005 Epson Research and Development, Inc.
//  All rights reserved.
//
//  This file is subject to the terms and conditions of the GNU General Public
//  License. See the file COPYING in the main directory of this archive for
//  more details.
//
//=============================================================================

#include <linux/sched.h>
#include <linux/smp_lock.h>

typedef struct s1d13xxx_thread_struct
{
    void (*function)    (struct s1d13xxx_thread_struct *s1d13xxx_thread);
    struct tq_struct    tqueue;
    struct task_struct  *thread;
    struct semaphore    critical_section;
    wait_queue_head_t   queue;
    int                 end;
} S1D13XXXTHREAD;

static void load_thread(void *data);
static void working_thread(S1D13XXXTHREAD *s1d13xxx_thread);
#ifdef CONFIG_FBCON_EPSON_INDIRECT
void copy_from_virtual_to_real( void );
#endif
void do_frame_transfer( void );

#define NUMBER_OF_THREADS 1

S1D13XXXTHREAD thread_data[NUMBER_OF_THREADS];

int thread_init(void)
    {
    int i;
    S1D13XXXTHREAD *s1d13xxx_thread=thread_data;

    for (i=0; i <NUMBER_OF_THREADS; i++)
        {
        //start_kthread(working_thread, &thread_data[i]);
        init_MUTEX_LOCKED(&s1d13xxx_thread->critical_section);
        s1d13xxx_thread->function = working_thread;
        s1d13xxx_thread->tqueue.sync = 0;
        INIT_LIST_HEAD(&s1d13xxx_thread->tqueue.list);
        s1d13xxx_thread->tqueue.routine = load_thread;
        s1d13xxx_thread->tqueue.data = s1d13xxx_thread;
        schedule_task(&s1d13xxx_thread->tqueue);
        down(&s1d13xxx_thread->critical_section);
        }

    return(0);
    }


void thread_cleanup(void)
    {
    int i;
    S1D13XXXTHREAD *s1d13xxx_thread=thread_data;

    for (i=0; i<NUMBER_OF_THREADS; i++)
    {
        if (s1d13xxx_thread->thread == NULL)
        return;

        lock_kernel();
        init_MUTEX_LOCKED(&s1d13xxx_thread->critical_section);
        mb();
        s1d13xxx_thread->end = 1;
        mb();
        kill_proc(s1d13xxx_thread->thread->pid, SIGKILL, 1);
        down(&s1d13xxx_thread->critical_section);
        unlock_kernel();
        kill_proc(2, SIGCHLD, 1);
    }
    return;
    }

static void working_thread(S1D13XXXTHREAD *s1d13xxx_thread)
    {
    lock_kernel();
    s1d13xxx_thread->thread = current;
    siginitsetinv(&current->blocked, sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGTERM));
    init_waitqueue_head(&s1d13xxx_thread->queue);
    s1d13xxx_thread->end = 0;
    sprintf(current->comm, "s1d13xxx thread");
    unlock_kernel();
    up(&s1d13xxx_thread->critical_section);

    do
    {
        interruptible_sleep_on_timeout(&s1d13xxx_thread->queue, HZ/40);
        mb();
        if (s1d13xxx_thread->end)
        break;
#ifdef CONFIG_FBCON_EPSON_INDIRECT
        copy_from_virtual_to_real();
#endif
        do_frame_transfer();
    } while(1);

    lock_kernel();
    s1d13xxx_thread->thread = NULL;
    mb();
    up(&s1d13xxx_thread->critical_section);
    }



static void load_thread(void *data)
    {
    S1D13XXXTHREAD *s1d13xxx_thread = data;
    kernel_thread((int (*)(void *))s1d13xxx_thread->function, (void *)s1d13xxx_thread, 0);
    }


