#ifndef __INCAIP2_PM_DEV_H__2007_07_26__10_19__
#define __INCAIP2_PM_DEV_H__2007_07_26__10_19__


/******************************************************************************
**
** FILE NAME    : incaip2_pm.h
** PROJECT      : 
** MODULES     	: PM
**
** DATE         : 
** AUTHOR       : 
** DESCRIPTION  : Power Management Driver Header File
** COPYRIGHT    : 	Copyright (c) 2006
**			Infineon Technologies AG
**			Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date        $Author         $Comment
** 26 JUL 2007  H. Bex.	        Initiate Version
*******************************************************************************/

#define IFX_PM_IOCTL_MAGIC     'K' 



#define MPS_SRAM_BASE_ADDRESS   0xBF102400
#define MPS_SRAM_BOOT_OFFSET    0x01C0
//#define MPS_CODE_OFFSET         0x1D50

#define BOOT_BASE_ADDRESS       (MPS_SRAM_BASE_ADDRESS + MPS_SRAM_BOOT_OFFSET)
//#define BOOT_CPU_OFFSET         0x20
#define BOOT_RVEC                ((volatile u32*)(BOOT_BASE_ADDRESS + 0x00))




/* 	Flags to adjust CPU-speed	*/
#define PM_CPU_393_MHz		0x1			/* to set CPU to 393 MHz */
#define PM_CPU_196_5_MHz	0x2			/* to set CPU to 196.5 MHz  */
#define PM_CPU_131_MHz		0x3			/* to set CPU to 131 MHz  */
#define PM_CPU_262_MHz		0x4			/* to set CPU to 262 MHz  */

/* parameter for IOCTL */

#ifdef __KERNEL__
struct mips_register {
	long unsigned orig_rvec;			/* */
	long unsigned soft_rvec;
	long unsigned st_pt_reg;
	long unsigned clk_reg;
	long unsigned flags;
};

struct mips_cp0_register {
	long unsigned index; 		/* MMU-reg: to address TLB entry */
	long unsigned entry_lo0;	/* MMU-reg: TLB-entry */
	long unsigned entry_lo1;	/* MMU-reg: TLB-entry */
	long unsigned context;		/* MMU-reg: TLB-entry */
	long unsigned page_mask;	/* MMU-reg: TLB-entry */
	long unsigned wired;		/* MMU-reg: defines stable TLB-entries */
	long unsigned hwrena;		/* regulates accesst to HW-register */
	long unsigned bad_vaddr;	/* set by every address errors */
	long unsigned count;		/* Processor cycle count */
	long unsigned entry_hi;		/* MMU-reg: TLB-entry */
	long unsigned compare;		/* timer interrupt control */
	long unsigned status;		/* status register */
	long unsigned intctl;		/* controls expanded interrupt capability */
	long unsigned srsctl; 		/* controls opertion of GPR shadows */
	long unsigned srsmap;		/* provides voectros to the shadows */
	long unsigned cause;		/* reason for exception or interrupt */
	long unsigned epc;			/* exception program counter */
	long unsigned pid;			/* MIPS CPU identifyer */
	long unsigned ebase;		/* base address of exception vectors */
	long unsigned config0;		/* configuration register */
	long unsigned config7;		/* configuration register7 */
	long unsigned watchlo0;		/* provide the interface for debug watchpoints */
	long unsigned watchlo1;		
	long unsigned watchlo2;	
	long unsigned watchlo3;	
	long unsigned watchhi0;
	long unsigned watchhi1;	
	long unsigned watchhi2;	
	long unsigned watchhi3;	
	long unsigned debug;		/* debug control/exception status */
	long unsigned depc;			/* debug counter at last debug exception */
	long unsigned perfcount0;	/* performance counter register */
	long unsigned perfcount1;	
	long unsigned perfcount2;	
	long unsigned perfcount3; 	
	long unsigned errctl;		/* parity-protection of caches and more */	
	long unsigned cacheerr;		/* records information about cache parity err.*/	
	long unsigned taglo0;		/* interface to the cache tag array */
	long unsigned taglo1;
	long unsigned taglo2;	
	long unsigned datalo0;
	long unsigned datalo1;
	long unsigned datalo2;	
	long unsigned datahi0;		/* upper bits for I-cache interface */
	long unsigned errorepc;		/* program counter at last error */
	long unsigned desave; 		/* debug handler scratchpad register */
};

struct mips_cpu {
	struct mips_cp0_register 	CP0_reg;
	struct mips_register 		mips_reg;
};

struct mips_jtlb_entry {			/* register to read/write TLB entries */
	u32 entry_hi; 
	u32 entry_lo0;		
	u32 entry_lo1;		
	u32 pagemask;			
};
#endif



/**
 * selection of clk-source for PM 
 * \param   flag
 * \ingroup IOCTL
 */
#define IFX_CPU0_FREQUENCY_SET		_IOW(IFX_PM_IOCTL_MAGIC, 1, unsigned long)


/**
 *  Get current clock mode
 * \param   none
 * \ingroup IOCTL
 */
#define IFX_CPU0_FREQUENCY_GET		_IOW(IFX_PM_IOCTL_MAGIC, 2, unsigned long)			


/******************************************************************************
 * Exported functions
 ******************************************************************************/
extern void ifx_cpu0_frequency(int flag);
extern int ifx_cpu1_enable(void);
extern int ifx_cpu1_disable(void);
#endif






