/************************************************************************
 *
 * Copyright (c) 2004
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/
 
/* Group definitions for Doxygen */
/** \addtogroup IOCTL Defined IOCTLs */

#ifndef PORT_H
#define PORT_H

/******************************************************************************
 * Exported types
 ******************************************************************************/
/**
 * Port IOCTL Parameters
 */
struct ifx_port_ioctl_parm {
	int port;   /**< Port index [0-1] */
	int pin;    /**< Pin index [0-27] */
	int value;  /**< Configuration value */
};

/******************************************************************************
 * Exported IOCTLs
 ******************************************************************************/
#define IFX_PORT_IOC_MAGIC	0xbf
/**
 * Sets/clears the open drain enable bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCOD		_IOW(IFX_PORT_IOC_MAGIC,0,struct ifx_port_ioctl_parm)
#if 0
/**
 * Sets/clears the pull up/down selection bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCPUDSEL	_IOW(IFX_PORT_IOC_MAGIC,1,struct ifx_port_ioctl_parm)
#endif
/**
 * Sets/clears the pull up/down enable bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCPUDEN	_IOW(IFX_PORT_IOC_MAGIC,2,struct ifx_port_ioctl_parm)
/**
 * Sets/clears the schmitt trigger enable bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCSTOFF	_IOW(IFX_PORT_IOC_MAGIC,3,struct ifx_port_ioctl_parm)
/**
 * Sets/clears the direction selection bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCDIR		_IOW(IFX_PORT_IOC_MAGIC,4,struct ifx_port_ioctl_parm)
/**
 * Sets/clears the output value bit.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCOUTPUT	_IOW(IFX_PORT_IOC_MAGIC,5,struct ifx_port_ioctl_parm)
/**
 * Reads the input value bit and stores it in arg->value.
 * \param   arg Pointer to structure #ifx_port_ioctl_parm
 * \ingroup IOCTL
 */
#define IFX_PORT_IOCINPUT	_IOWR(IFX_PORT_IOC_MAGIC,6,struct ifx_port_ioctl_parm)


/******************************************************************************
 * Exported Functions
 ******************************************************************************/
int ifx_port_ssc_master_set(int ssc_port, int master);
int ifx_port_ssc_cs_set(int ssc_port, int cs, int master);
int ifx_port_reserve_pin(int port, int pin, int module_id);
int ifx_port_free_pin(int port, int pin, int module_id);
int ifx_port_set_open_drain(int port, int pin, int module_id);
int ifx_port_clear_open_drain(int port, int pin, int module_id);
int ifx_port_set_pudsel(int port, int pin, int module_id);
int ifx_port_clear_pudsel(int port, int pin, int module_id);
int ifx_port_set_puden(int port, int pin, int module_id);
int ifx_port_clear_puden(int port, int pin, int module_id);
int ifx_port_set_stoff(int port, int pin, int module_id);
int ifx_port_clear_stoff(int port, int pin, int module_id);
int ifx_port_set_dir_out(int port, int pin, int module_id);
int ifx_port_set_dir_in(int port, int pin, int module_id);
int ifx_port_set_output(int port, int pin, int module_id);
int ifx_port_clear_output(int port, int pin, int module_id);
int ifx_port_get_input(int port, int pin, int module_id);


#endif /* PORT_H */

