/******************************************************************************
 *
 *
 *
 * Copyright 2000 Infineon Technologies Bangalore,India, ., All Rights Reserved
 *	
 *	
 *	
 * Revision History
 *	
 * Name					Date								Description
 * ============	======	================================================
 * Manamohan shetty		5/8/2002						Phase I Release
 *
 *                      4/12/2002                       Phase II Release 
 *                      3/6/2003                        Removed constant declerations (see pr_const.h)  
 * DESCRIPTION : This file contains the constants and forward declarations for 
 *               MAC address(MA) table configuration API's
 *
 ******************************************************************************/

#ifndef _SWITCH_H_
#define _SWITCH_H_

#define INCAIP2_SW_REG32(reg_num) *((volatile u32*)(reg_num))

#define		MAX_MACTABLEINDEX	63


typedef enum
{
    /*
     * the following periods are needed for hardware controlled
     * MAC table aging . Due to the fast MAC table aging bug which causes 
     * the entries to age ~15 times faster the documented 
     * periods lead to the values listed in this enumeration.
     */    
    MAC_AGING_IMMEDIATE = 0,  /* as documented */
    MAC_AGING_2_SEC     = 1,  /* documented as 20 seconds */
    MAC_AGING_35_SEC    = 2,  /* documented as 5 minutes */
    MAC_AGING_95_SEC    = 3,  /* documented as 15 minutes */
    MAC_AGING_388_SEC   = 4,  /* documented as 1 hour */
    MAC_AGING_24_MIN    = 5,  /* documented as 4 hours */    
    MAC_AGING_150_MIN   = 6,  /* documented as 1 day */
    MAC_AGING_NO_AGING  = 7,  /* as documented */
    /*
     * the following periods are processed by software 
     * MAC table aging based on timer control. Hence the enumeration 
     * stores the desired period in milliseconds
     */
    MAC_AGING_1_HRS     =  3600000,  
    MAC_AGING_6_HRS     =  21600000,                  
    MAC_AGING_10_HRS    =  36000000, 
    MAC_AGING_12_HRS    =  43200000, 
    MAC_AGING_1_DAY     =  86400000,            
    MAC_AGING_3_DAYS    = 259200000,

}aging_period_e;



typedef struct 
{
    u32  txPauseCount;
    u32  txSingleCollCount;
    u32  txMultCollCount;
    u32  txExcessCollCount;
    u32  txLateCollCount;
    u32  txDeferredCount;  
    
 } INCAIP2_SwitchMac_MIB_Table_t;


typedef struct 
{
    u32  rxBytes;
    u32  rxUnicastPkts;
    u32  rxBroadcastPkts;
    u32  rxMulticastPkts;
    u32  rxCRCErrorPkts;
    u32  rxUnderSizeGoodPkts;  /* Packet length < 64 */
    u32  rxUnderSizeErrorPkts;  /* Packet length < 64 */    
    u32  rx64BytePkts;         /* Packet length == 64 bytes */
    u32  rx127BytePkts;        /* 64 < Packet length <= 127 */
    u32  rx255BytePkts;        /* 127 < Packet length <= 255 */
    u32  rx511BytePkts;        /* 255 < Packet length <= 511 */
    u32  rx1023BytePkts;       /* 511 < Packet length <= 1023 */
    u32  rxMaxBytePkts;        /* 1023 < Packet length <= MAX_PKT_LEN */
    u32  rxOversizeGoodPkts;   /* Packet length > MAX_PKT_LEN bytes */
    u32  rxOversizeBadPkts;    /* Packet length > MAX_PKT_LEN bytes */
    u32  rxGoodPausePkts;
    u32  rxDroppedPkts;    
    u32  txBytes;
    u32  txUnicastPkts;
    u32  txBroadcastPkts;
    u32  txMulticastPkts;
 } INCAIP2_SwitchPort_MIB_Table_t;


s32 INCAIP2_SwitchPortGetMIBCounters( u32 portId, INCAIP2_SwitchPort_MIB_Table_t *MIB_TablePtr);
s32 INCAIP2_SwitchMacGetMIBCounters( u32 portId, INCAIP2_SwitchMac_MIB_Table_t *MIB_TablePtr);


s32 ifx_get_MAC_table_idx( u8* macAddrPtr, /* u32 fId, */ u32* mactableIndexPtr );

#define		STATIC				0
/*
s32 ifx_get_MAC_table_entry_idx(u8 chipId,u32 macTableIndex, u8* macAddrPtr, u32* mpidPtr, 
						          BOOL* criticalFlagPtr,u32* maStatePtr, u32* fIdPtr);
*/
s32 INCAIP2_MACTableAddEntry( u8* macAddrptr, BOOL lan_port, BOOL pc_port, BOOL cpu_port,
                              BOOL criticalFlag, u32 maState );
 
/*
s32 ifx_add_MAC_table_entry_idx( u8 chipId, u32 macTableIndex, u8* macAddrPtr, 
                                 BOOL lan_port,  BOOL pc_port, BOOL cpu_port,
                                 BOOL criticalFlag, u32 maState );
*/
s32 ifx_set_chip_number(u8 No_of_Chips);
s32 ifx_set_base_address(u8 chipId,u32 Base_Address);    


                              
/*
s32 ifx_API_init(u8 chipId);                                  
*/

/*
int s_api_ioctl(struct inode *inode, struct file *file,
		                                    unsigned int cmd, unsigned long arg);

*/

/* Forward declaration 		*/
#if 0
s32 ifx_get_MAC_hash_idx(u8* macAddrPtr,u32 fId,u32* hashIndexPtr);

s32 ifx_get_MAC_table_entry_idx(u8 chipId,u32 macTableIndex, u8* macAddrPtr, u32* mpidPtr, 
						          BOOL* criticalFlagPtr,u32* maStatePtr, u32* fIdPtr);

s32 ifx_add_MAC_table_entry_hash(u8 chipId,u8* macAddrPtr,u32 mpid,BOOL criticalFlag,
							       u32 maState, u32 fId);

s32 ifx_delete_MAC_table_entry_hash(u8 chipId,u8* macAddrPtr, u32 fId);


s32 ifx_get_MAC_table_stat(u8 chipId,u32* multiCastPtr,u32* staticPtr,u32* validPtr);



#endif
#endif    /* __PR_MEMACC_H */
