/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup IOCTL Defined IOCTLs */

/******************************************************************************
 * Exported IOCTLs
 ******************************************************************************/
/**
 * Switch LED on.
 * Sets the LED bits specified in arg.
 * \param   arg Bit vector as in register LED_REG 
 * \ingroup IOCTL
 */
#define LED_ON      _IOW('T', 1, unsigned int)

/**
 * Switch LED off.
 * Clears the LED bits specified in arg.
 * \param   arg Bit vector as in register LED_REG 
 * \ingroup IOCTL
 */
#define LED_OFF     _IOW('T', 2, unsigned int)

/**
 * Set PC Port LEDs.
 * Sets the Ethernet LED bits specified in arg for PC port.
 * \param   arg Bits to be set
 * \ingroup IOCTL
 */
#define LED_ETH_PC  _IOW('T', 3, unsigned char)

/**
 * Set LAN Port LEDs.
 * Sets the Ethernet LED bits specified in arg for LAN port.
 * \param   arg Bits to be set
 * \ingroup IOCTL
 */
#define LED_ETH_LAN _IOW('T', 4, unsigned char)
/**
 * Get KEY Event.
 * Get next key event an put it into arg. The value format is
 * <timestamp[30:8]> + <key_state[7:7]> + <keycode[6:0]>.
 * \param   arg Pointer to #key_event_t
 */
#define KEYPAD_GET  _IOR('G', 1, key_event_t)


typedef struct {
    char          key_code;   /**< Key code of current key */
    int           key_state;  /**< Key state [KEY_PRESSED, KEY_RELEASED] */
    unsigned long time_stamp; /**< Value of jiffies at time of event */
} key_event_t;



/******************************************************************************
 * Exported types
 ******************************************************************************/

/**
 * Key Event Descriptor
 */

#ifdef __KERNEL__




typedef int (*keypad_callback_t) (char key, int updown, 
                                  unsigned long timestamp);



typedef struct {
    wait_queue_head_t   waitq;
} keypad_dev_t;


/******************************************************************************
 * Exported functions
 ******************************************************************************/
extern int  incaip2_key_register_callback   (keypad_callback_t fp);
extern int  incaip2_key_unregister_callback (void);
#endif


